package gov.va.med.bhie.dao.patient;

import gov.va.med.bhie.beans.QueryBean;
import gov.va.med.bhie.beans.v4.NarrativeV4;
import gov.va.med.bhie.beans.v4.NoteV4;
import gov.va.med.bhie.beans.v4.SPDCBeanV4;
import gov.va.med.bhie.beans.v4.SiteStatusV4;
import gov.va.med.bhie.common.ClinicalDomainLoincCode;
import gov.va.med.bhie.common.QueryMapFactory;
import gov.va.med.bhie.dao.DaoBase;
import gov.va.med.bhie.dao.DaoException;
import gov.va.med.bhie.datarecord.V4DataRecord;
import gov.va.med.bhie.dataservice.BHIERestServiceHandler;
import gov.va.med.bhie.resultcollection.ResultCollection;
import gov.va.med.bhie.resultcollection.V4ResultCollection;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.List;

public class V4Dao extends DaoBase {

    /**
     * Gets patient data from DES interface.
     *
     * @param queryBean Query parameters.
     * @return BHIE REST interface response bound to a VLERResultCollection object.
     * @throws DaoException If there is a failure.
     */
    public V4ResultCollection getV4Data(ClinicalDomainLoincCode loinc, QueryBean queryBean) throws DaoException {
        V4ResultCollection rcoll = new V4ResultCollection();

        rcoll.setDataRecordList(new V4DataRecord());
        rcoll.setErrorListv4(new ArrayList<String>());
        rcoll.setStatusListv4(new ArrayList<SiteStatusV4>());
        rcoll.setNarrativeList(new ArrayList<NarrativeV4>());


        try {
            checkQueryParameters(queryBean);

            List<ResultCollection> resultCollectionList =
                    BHIERestServiceHandler.getInstance().queryAndPollRESTInterface(
                            QueryMapFactory.create("4", queryBean, loinc),
                            V4ResultCollection.class);
            if (resultCollectionList != null && !resultCollectionList.isEmpty()) {

                for (int index = 0; index < resultCollectionList.size(); index++) {
                    V4ResultCollection tempColl = (V4ResultCollection) resultCollectionList.get(index);
                    if (tempColl != null) {
                        if (tempColl.getStatusListv4() != null && !tempColl.getStatusListv4().isEmpty()) {
                            rcoll.getStatusListv4().addAll(tempColl.getStatusListv4());
                        }
                        if (tempColl.getErrorListv4() != null && !tempColl.getErrorListv4().isEmpty()) {
                            rcoll.getErrorListv4().addAll(tempColl.getErrorListv4());
                        }

                        if (tempColl.getNarrativeList() != null && !tempColl.getNarrativeList().isEmpty()) {
                            rcoll.getNarrativeList().addAll(tempColl.getNarrativeList());
                        }

                        if (tempColl.getDataRecordList() != null) {
                            if (tempColl.getDataRecordList().getAdmissions() != null && !tempColl.getDataRecordList().getAdmissions().isEmpty()) {
                                if (rcoll.getDataRecordList().getAdmissions() != null) {
                                    rcoll.getDataRecordList().getAdmissions().addAll(tempColl.getDataRecordList().getAdmissions());
                                } else {
                                    rcoll.getDataRecordList().setAdmissions(tempColl.getDataRecordList().getAdmissions());
                                }
                            }

                            if (tempColl.getDataRecordList().getAllergy() != null && !tempColl.getDataRecordList().getAllergy().isEmpty()) {
                                if (rcoll.getDataRecordList().getAllergy() != null) {
                                    rcoll.getDataRecordList().getAllergy().addAll(tempColl.getDataRecordList().getAllergy());
                                } else {
                                    rcoll.getDataRecordList().setAllergy(tempColl.getDataRecordList().getAllergy());
                                }
                            }
                            if (tempColl.getDataRecordList().getAppointments() != null && !tempColl.getDataRecordList().getAppointments().isEmpty()) {
                                if (rcoll.getDataRecordList().getAppointments() != null) {
                                    rcoll.getDataRecordList().getAppointments().addAll(tempColl.getDataRecordList().getAppointments());
                                } else {
                                    rcoll.getDataRecordList().setAppointments(tempColl.getDataRecordList().getAppointments());
                                }
                            }
                            if (tempColl.getDataRecordList().getConsultOrders() != null && !tempColl.getDataRecordList().getConsultOrders().isEmpty()) {
                                if (rcoll.getDataRecordList().getConsultOrders() != null) {
                                    rcoll.getDataRecordList().getConsultOrders().addAll(tempColl.getDataRecordList().getConsultOrders());
                                } else {
                                    rcoll.getDataRecordList().setConsultOrders(tempColl.getDataRecordList().getConsultOrders());
                                }
                            }
                            if (tempColl.getDataRecordList().getDemographics() != null && !tempColl.getDataRecordList().getDemographics().isEmpty()) {
                                if (rcoll.getDataRecordList().getDemographics() != null) {
                                    rcoll.getDataRecordList().getDemographics().addAll(tempColl.getDataRecordList().getDemographics());
                                } else {
                                    rcoll.getDataRecordList().setDemographics(tempColl.getDataRecordList().getDemographics());
                                }
                            }
                            if (tempColl.getDataRecordList().getDeploymentForms() != null && !tempColl.getDataRecordList().getDeploymentForms().isEmpty()) {
                                if(rcoll.getDataRecordList().getDeploymentForms() != null) {
                                    rcoll.getDataRecordList().getDeploymentForms().addAll(tempColl.getDataRecordList().getDeploymentForms());
                                } else {
                                    rcoll.getDataRecordList().setDeploymentForms(tempColl.getDataRecordList().getDeploymentForms());
                                }
                            }
                            if (tempColl.getDataRecordList().getDocumentInfo() != null && !tempColl.getDataRecordList().getDocumentInfo().isEmpty()) {
                                if (rcoll.getDataRecordList().getDocumentInfo() != null) {
                                    rcoll.getDataRecordList().getDocumentInfo().addAll(tempColl.getDataRecordList().getDocumentInfo());
                                } else {
                                    rcoll.getDataRecordList().setDocumentInfo(tempColl.getDataRecordList().getDocumentInfo());
                                }
                            }
                            if (tempColl.getDataRecordList().getDocuments() != null && !tempColl.getDataRecordList().getDocuments().isEmpty()) {
                                if (rcoll.getDataRecordList().getDocuments() != null) {
                                    rcoll.getDataRecordList().getDocuments().addAll(tempColl.getDataRecordList().getDocuments());
                                } else {
                                    rcoll.getDataRecordList().setDocuments(tempColl.getDataRecordList().getDocuments());
                                }
                            }
                            if (tempColl.getDataRecordList().getEncounterNotes() != null && !tempColl.getDataRecordList().getEncounterNotes().isEmpty()) {
                                if (rcoll.getDataRecordList().getEncounterNotes() != null) {
                                    rcoll.getDataRecordList().getEncounterNotes().addAll(tempColl.getDataRecordList().getEncounterNotes());
                                } else {
                                    rcoll.getDataRecordList().setEncounterNotes(tempColl.getDataRecordList().getEncounterNotes());
                                }
                            }
                            if (tempColl.getDataRecordList().getEncounters() != null && !tempColl.getDataRecordList().getEncounters().isEmpty()) {
                                if (rcoll.getDataRecordList().getEncounters() != null) {
                                    rcoll.getDataRecordList().getEncounters().addAll(tempColl.getDataRecordList().getEncounters());
                                } else {
                                    rcoll.getDataRecordList().setEncounters(tempColl.getDataRecordList().getEncounters());
                                }
                            }
                            if (tempColl.getDataRecordList().getExternalPartnerResults() != null && !tempColl.getDataRecordList().getExternalPartnerResults().isEmpty()) {
                                if (rcoll.getDataRecordList().getExternalPartnerResults() != null) {
                                    rcoll.getDataRecordList().getExternalPartnerResults().addAll(tempColl.getDataRecordList().getExternalPartnerResults());
                                } else {
                                    rcoll.getDataRecordList().setExternalPartnerResults(tempColl.getDataRecordList().getExternalPartnerResults());
                                }
                            }
                            if (tempColl.getDataRecordList().getImmunizations() != null && !tempColl.getDataRecordList().getImmunizations().isEmpty()) {
                                if (rcoll.getDataRecordList().getImmunizations() != null) {
                                    rcoll.getDataRecordList().getImmunizations().addAll(tempColl.getDataRecordList().getImmunizations());
                                } else {
                                    rcoll.getDataRecordList().setImmunizations(tempColl.getDataRecordList().getImmunizations());
                                }
                            }
                            if (tempColl.getDataRecordList().getLabApResults() != null && !tempColl.getDataRecordList().getLabApResults().isEmpty()) {
                                if (rcoll.getDataRecordList().getLabApResults() != null) {
                                    rcoll.getDataRecordList().getLabApResults().addAll(tempColl.getDataRecordList().getLabApResults());
                                } else {
                                    rcoll.getDataRecordList().setLabApResults(tempColl.getDataRecordList().getLabApResults());
                                }
                            }
                            if (tempColl.getDataRecordList().getLabChemResults() != null && !tempColl.getDataRecordList().getLabChemResults().isEmpty()) {
                                if(rcoll.getDataRecordList().getLabChemResults() != null) {
                                    rcoll.getDataRecordList().getLabChemResults().addAll(tempColl.getDataRecordList().getLabChemResults());
                                } else {
                                    rcoll.getDataRecordList().setLabChemResults(tempColl.getDataRecordList().getLabChemResults());
                                }
                            }
                            if (tempColl.getDataRecordList().getLabMicroResults() != null && !tempColl.getDataRecordList().getLabMicroResults().isEmpty()) {
                                if(rcoll.getDataRecordList().getLabMicroResults() != null) {
                                    rcoll.getDataRecordList().getLabMicroResults().addAll(tempColl.getDataRecordList().getLabMicroResults());
                                } else {
                                    rcoll.getDataRecordList().setLabMicroResults(tempColl.getDataRecordList().getLabMicroResults());
                                }
                            }
                            if (tempColl.getDataRecordList().getLabOrders() != null && !tempColl.getDataRecordList().getLabOrders().isEmpty()) {
                                if(rcoll.getDataRecordList().getLabOrders() != null) {
                                    rcoll.getDataRecordList().getLabOrders().addAll(tempColl.getDataRecordList().getLabOrders());
                                } else {
                                    rcoll.getDataRecordList().setLabOrders(tempColl.getDataRecordList().getLabOrders());
                                }
                            }
                            if (tempColl.getDataRecordList().getMedicationOrders() != null && !tempColl.getDataRecordList().getMedicationOrders().isEmpty()) {
                                if(rcoll.getDataRecordList().getMedicationOrders() != null) {
                                    rcoll.getDataRecordList().getMedicationOrders().addAll(tempColl.getDataRecordList().getMedicationOrders());
                                } else {
                                    rcoll.getDataRecordList().setMedicationOrders(tempColl.getDataRecordList().getMedicationOrders());
                                }
                            }
                            if (tempColl.getDataRecordList().getMedications() != null && !tempColl.getDataRecordList().getMedications().isEmpty()) {
                                if (rcoll.getDataRecordList().getMedications() != null) {
                                    rcoll.getDataRecordList().getMedications().addAll(tempColl.getDataRecordList().getMedications());
                                } else {
                                    rcoll.getDataRecordList().setMedications(tempColl.getDataRecordList().getMedications());
                                }
                            }
                            if (tempColl.getDataRecordList().getNotes() != null && !tempColl.getDataRecordList().getNotes().isEmpty()) {
                                if(rcoll.getDataRecordList().getNotes() != null) {
                                    rcoll.getDataRecordList().getNotes().addAll(tempColl.getDataRecordList().getNotes());
                                } else {
                                    rcoll.getDataRecordList().setNotes(tempColl.getDataRecordList().getNotes());
                                }
                            }
                            if (tempColl.getDataRecordList().getInpatientNotes() != null && !tempColl.getDataRecordList().getInpatientNotes().isEmpty()) {
                                if(rcoll.getDataRecordList().getInpatientNotes() != null) {
                                    rcoll.getDataRecordList().getInpatientNotes().addAll(tempColl.getDataRecordList().getInpatientNotes());
                                } else {
                                    rcoll.getDataRecordList().setInpatientNotes(tempColl.getDataRecordList().getInpatientNotes());
                                }
                            }
                            if (tempColl.getDataRecordList().getDischargeNotes() != null && !tempColl.getDataRecordList().getDischargeNotes().isEmpty()) {
                                for(NoteV4 note : tempColl.getDataRecordList().getDischargeNotes()){
                                    if(note.getContentType() == null && StringUtils.isEmpty(note.getComplexDataUrl())){
                                        SPDCBeanV4 contentType = new SPDCBeanV4();
                                        contentType.setCode("text/xml");
                                        contentType.setSystem("2.16.840.1.113883.5.79");
                                        ArrayList<SPDCBeanV4> contentTypes = new ArrayList<SPDCBeanV4>();
                                        contentTypes.add(contentType);
                                        note.setContentType(contentTypes);
                                    }
                                }
                                if(rcoll.getDataRecordList().getInpatientNotes() != null) {
                                    rcoll.getDataRecordList().getInpatientNotes().addAll(tempColl.getDataRecordList().getDischargeNotes());
                                } else {
                                    rcoll.getDataRecordList().setInpatientNotes(tempColl.getDataRecordList().getDischargeNotes());
                                }
                            }
                            if (tempColl.getDataRecordList().getPatientManagement() != null && !tempColl.getDataRecordList().getPatientManagement().isEmpty()) {
                                if(rcoll.getDataRecordList().getPatientManagement() != null) {
                                    rcoll.getDataRecordList().getPatientManagement().addAll(tempColl.getDataRecordList().getPatientManagement());
                                } else {
                                    rcoll.getDataRecordList().setPatientManagement(tempColl.getDataRecordList().getPatientManagement());
                                }
                            }
                            if (tempColl.getDataRecordList().getPayers() != null && !tempColl.getDataRecordList().getPayers().isEmpty()) {
                                if(rcoll.getDataRecordList().getPayers() != null) {
                                    rcoll.getDataRecordList().getPayers().addAll(tempColl.getDataRecordList().getPayers());
                                } else {
                                    rcoll.getDataRecordList().setPayers(tempColl.getDataRecordList().getPayers());
                                }
                            }
                            if (tempColl.getDataRecordList().getProblems() != null && !tempColl.getDataRecordList().getProblems().isEmpty()) {
                                if(rcoll.getDataRecordList().getProblems() != null) {
                                    rcoll.getDataRecordList().getProblems().addAll(tempColl.getDataRecordList().getProblems());
                                } else {
                                    rcoll.getDataRecordList().setProblems(tempColl.getDataRecordList().getProblems());
                                }
                            }
                            if (tempColl.getDataRecordList().getProcedures() != null && !tempColl.getDataRecordList().getProcedures().isEmpty()) {
                                if(rcoll.getDataRecordList().getProcedures() != null) {
                                    rcoll.getDataRecordList().getProcedures().addAll(tempColl.getDataRecordList().getProcedures());
                                } else {
                                    rcoll.getDataRecordList().setProcedures(tempColl.getDataRecordList().getProcedures());
                                }
                            }
                            if (tempColl.getDataRecordList().getQuestionnaires() != null && !tempColl.getDataRecordList().getQuestionnaires().isEmpty()) {
                                if(rcoll.getDataRecordList().getQuestionnaires() != null) {
                                    rcoll.getDataRecordList().getQuestionnaires().addAll(tempColl.getDataRecordList().getQuestionnaires());
                                } else {
                                    rcoll.getDataRecordList().setQuestionnaires(tempColl.getDataRecordList().getQuestionnaires());
                                }
                            }
                            if (tempColl.getDataRecordList().getRadiologyOrders() != null && !tempColl.getDataRecordList().getRadiologyOrders().isEmpty()) {
                                if(rcoll.getDataRecordList().getRadiologyOrders() != null) {
                                    rcoll.getDataRecordList().getRadiologyOrders().addAll(tempColl.getDataRecordList().getRadiologyOrders());
                                } else {
                                    rcoll.getDataRecordList().setRadiologyOrders(tempColl.getDataRecordList().getRadiologyOrders());
                                }
                            }
                            if (tempColl.getDataRecordList().getRadiologyReports() != null && !tempColl.getDataRecordList().getRadiologyReports().isEmpty()) {
                                if(rcoll.getDataRecordList().getRadiologyReports() != null) {
                                    rcoll.getDataRecordList().getRadiologyReports().addAll(tempColl.getDataRecordList().getRadiologyReports());
                                } else {
                                    rcoll.getDataRecordList().setRadiologyReports(tempColl.getDataRecordList().getRadiologyReports());
                                }
                            }
                            if (tempColl.getDataRecordList().getVitals() != null && !tempColl.getDataRecordList().getVitals().isEmpty()) {
                                if(rcoll.getDataRecordList().getVitals() != null) {
                                    rcoll.getDataRecordList().getVitals().addAll(tempColl.getDataRecordList().getVitals());
                                } else {
                                    rcoll.getDataRecordList().setVitals(tempColl.getDataRecordList().getVitals());
                                }
                            }
                            if (tempColl.getDataRecordList().getFamilyHistory() != null && !tempColl.getDataRecordList().getFamilyHistory().isEmpty()) {
                                if(rcoll.getDataRecordList().getFamilyHistory() != null) {
                                    rcoll.getDataRecordList().getFamilyHistory().addAll(tempColl.getDataRecordList().getFamilyHistory());
                                } else {
                                    rcoll.getDataRecordList().setFamilyHistory(tempColl.getDataRecordList().getFamilyHistory());
                                }
                            }
                            if (tempColl.getDataRecordList().getSocialHistory() != null && !tempColl.getDataRecordList().getSocialHistory().isEmpty()) {
                                if(rcoll.getDataRecordList().getSocialHistory() != null) {
                                    rcoll.getDataRecordList().getSocialHistory().addAll(tempColl.getDataRecordList().getSocialHistory());
                                } else {
                                    rcoll.getDataRecordList().setSocialHistory(tempColl.getDataRecordList().getSocialHistory());
                                }
                            }
                            if (tempColl.getDataRecordList().getOpmHistory() != null && !tempColl.getDataRecordList().getOpmHistory().isEmpty()) {
                                if(rcoll.getDataRecordList().getOpmHistory() != null) {
                                    rcoll.getDataRecordList().getOpmHistory().addAll(tempColl.getDataRecordList().getOpmHistory());
                                } else {
                                    rcoll.getDataRecordList().setOpmHistory(tempColl.getDataRecordList().getOpmHistory());
                                }
                            }
                        }
                    }
                }
            }
            return rcoll;
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }
}
