/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.bhie.dao.patient;

import gov.va.med.bhie.beans.QueryBean;
import gov.va.med.bhie.beans.v4.SiteStatusV4;
import gov.va.med.bhie.common.AppConfig;
import gov.va.med.bhie.common.QueryMapFactory;
import gov.va.med.bhie.dao.DaoBase;
import gov.va.med.bhie.dao.DaoException;
import gov.va.med.bhie.dataservice.BHIERestServiceHandler;
import gov.va.med.bhie.resultcollection.ResultCollection;
import gov.va.med.bhie.resultcollection.V4ResultCollection;

import java.util.ArrayList;
import java.util.List;

import static gov.va.med.bhie.common.ClinicalDomainLoincCode.VLERCCDA;

/**
 * VLER data access object.
 */
public class VLERDao extends DaoBase {

    /**
     * Gets patient data from VLER interface.
     * @param queryBean Query parameters.
     * @return BHIE REST interface response bound to a VLERResultCollection object.
     * @throws DaoException If there is a failure.
     */
    public V4ResultCollection getVLERData(QueryBean queryBean) throws DaoException {
        V4ResultCollection rcoll = null;
        try {
            checkQueryParameters(queryBean);

            if (AppConfig.getInstance().getVlerTestPat() != null && !"".equalsIgnoreCase(AppConfig.getInstance().getVlerTestPat())) {
                queryBean.setPatientId("EDIPI:" + AppConfig.getInstance().getVlerTestPat());
            }

            List<ResultCollection> resultCollectionList =
                    BHIERestServiceHandler.getVlerInstance().queryAndPollRESTInterface(
                            QueryMapFactory.create("VLER", queryBean, VLERCCDA),
                            V4ResultCollection.class);
            if (resultCollectionList != null && !resultCollectionList.isEmpty()) {
                int index = resultCollectionList.size() - 1;
                rcoll = (V4ResultCollection) resultCollectionList.get(index);
                for (int i = 0; i < index; ++i) {
                    V4ResultCollection rc1 = (V4ResultCollection) resultCollectionList.get(i);
                    if (rc1 != null && "true".equalsIgnoreCase(rc1.getQueryComplete())) {
                        index = i;
                        rcoll = rc1;
                        break;
                    }
                }
                if (rcoll != null) {
                    if (rcoll.getErrorListv4() == null) {
                        rcoll.setErrorListv4(new ArrayList<String>());
                    }
                    if (rcoll.getStatusListv4() == null) {
                        rcoll.setStatusListv4(new ArrayList<SiteStatusV4>());
                    }
                    for (int j = 0; j < resultCollectionList.size(); ++j) {
                        if (j == index) {
                            continue;
                        }
                        V4ResultCollection rc2 = (V4ResultCollection) resultCollectionList.get(j);
                        if (rc2 != null && rc2.getStatusListv4() != null && !rc2.getStatusListv4().isEmpty()) {
                            rcoll.getStatusListv4().addAll(rc2.getStatusListv4());
                        }
                        if (rc2 != null && rc2.getErrorListv4() != null && !rc2.getErrorListv4().isEmpty()) {
                            rcoll.getErrorListv4().addAll(rc2.getErrorListv4());
                        }
                    }
                }
            }
            return rcoll;
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }
}
