package gov.va.med.bhie.dataservice;

import org.apache.log4j.Logger;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.util.Locale;

public class BHIECacheClient extends Thread {
    /**
     * Logger.
     */
    private static final Logger LOGGER =
            Logger.getLogger(BHIECacheClient.class.getName());

    private static final String REQUESTBEGIN = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:web=\"http://webservice.bhie.URL       /\">\n"
        + "   <soapenv:Header/>\n"
        + "   <soapenv:Body>\n"
        + "      <web:addCache>\n";
    private static final String REQUESTEND = "      </web:addCache>\n"
        + "   </soapenv:Body>\n"
        + "</soapenv:Envelope>";

    private String url = "";
    private String auth = "";
    private String type = "";
    private String key = "";
    private String value = "";
    private String timestamp = "";
    
    public BHIECacheClient() {
        super();
    }

    public BHIECacheClient(String url, String auth, String type, String key, String value, String timestamp) {
        super();
        this.url = url;
        this.auth = auth;
        this.type = type;
        this.key = key;
        this.value = value;
        this.timestamp = timestamp;
    }

    /**
     * Run the service.
     */
    public void run() {
        try {
            if (url != null && url.toLowerCase(Locale.getDefault()).startsWith("http")) {
                StringBuilder request = new StringBuilder(1024);
                request.append(REQUESTBEGIN);
                request.append("         <type><![CDATA[" + type + "]]></type>\n"); 
                request.append("         <key><![CDATA[" + key + "]]></key>\n");
                request.append("         <value><![CDATA[" + value + "]]></value>\n");
                request.append("         <timestamp><![CDATA[" + timestamp + "]]></timestamp>\n");
                request.append(REQUESTEND);
                String soapreq = request.toString();
                URL cacheurl = new URL(url);
                URLConnection uconn = cacheurl.openConnection();
                HttpURLConnection connection = (HttpURLConnection) uconn;
                connection.setReadTimeout(60000);
                connection.setDoOutput(true);
                connection.setDoInput(true);
                connection.setRequestProperty("Content-Type", "text/xml");
                connection.setRequestProperty("Accept", "text/xml");
                connection.setRequestProperty("Content-length", Integer.toString(soapreq.length()));
                if (auth != null && auth.length() > 0) {
                    // auth should be an encoded string like: Base64Encoder.encode ("test1:test1")
                    connection.setRequestProperty("Authorization", "Basic " + auth);
                }
                connection.setRequestMethod("POST");
                
                OutputStream out = connection.getOutputStream();
                Writer wout = null;
                try {
                    wout = new OutputStreamWriter(out, Charset.forName("UTF-8").newEncoder());
                    wout.write(soapreq);
                    wout.flush();
                } finally {
                    if (wout != null) {
                        try {
                            wout.close();
                        } catch (Exception ex) {
                            LOGGER.error(ex.getLocalizedMessage(), ex);
                        }
                    }
                }
                InputStream input = null;
//                StringBuilder resp = new StringBuilder(1024);
                try {
                    input = connection.getInputStream();
                    int chr = input.read();
                    while (chr != -1) {
//                        resp.append((char) chr);
                        chr = input.read();
                    }
                } finally {
                    if (input != null) {
                        input.close();
                    }
                }
            }
//            LOGGER.debug("Received: \n" + resp.toString().replaceAll("><", ">\n<"));
//            LOGGER.debug("Response code = " + connection.getResponseCode());
        } catch (Exception ex) {
            LOGGER.error(ex.getLocalizedMessage(), ex);
        }
    }

    /**
     * @return the url
     */
    public String getUrl() {
        return url;
    }

    /**
     * @param url the url to set
     */
    public void setUrl(String url) {
        this.url = url;
    }

    /**
     * @return the auth
     */
    public String getAuth() {
        return auth;
    }

    /**
     * @param auth the auth to set
     */
    public void setAuth(String auth) {
        this.auth = auth;
    }

    /**
     * @return the type
     */
    public String getType() {
        return type;
    }

    /**
     * @param type the type to set
     */
    public void setType(String type) {
        this.type = type;
    }

    /**
     * @return the key
     */
    public String getKey() {
        return key;
    }

    /**
     * @param key the key to set
     */
    public void setKey(String key) {
        this.key = key;
    }

    /**
     * @return the value
     */
    public String getValue() {
        return value;
    }

    /**
     * @param value the value to set
     */
    public void setValue(String value) {
        this.value = value;
    }

    /**
     * @return the timestamp
     */
    public String getTimestamp() {
        return timestamp;
    }

    /**
     * @param timestamp the timestamp to set
     */
    public void setTimestamp(String timestamp) {
        this.timestamp = timestamp;
    }
}
