package gov.va.med.bhie.webservice;

import gov.va.med.vhahon.common.Utils;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.Text;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.util.Set;

public class InvalidXMLCharSOAPHandler implements SOAPHandler<SOAPMessageContext> {

    /**
     * Logger.
     */
    private static final Logger LOGGER =
            Logger.getLogger(InvalidXMLCharSOAPHandler.class.getName());

    @Override
    public Set<QName> getHeaders() {
        return null;
    }

    @Override
    public boolean handleMessage(SOAPMessageContext smc) {

        Boolean outboundProperty = (Boolean) smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        //remove any invalid xml characters from outbound SOAP messages
        if (outboundProperty) {
            try {
                SOAPMessage message = smc.getMessage();
                SOAPBody body = message.getSOAPBody();
                removeInvalidXMLCharacters(body.getChildNodes());
            } catch (SOAPException ex) {
                LOGGER.error(ex.getMessage(), ex);
            }
        }

        return true;
    }

    private void removeInvalidXMLCharacters(NodeList nodeList) {
        int listLength = nodeList.getLength();
        for (int i = 0; i < listLength; i++) {
            Node node = nodeList.item(i);
            if (node instanceof Text) {
                Text textNode = (Text) node;
                textNode.setData(Utils.stripInvalidXMLCharacters(textNode.getData()));
            }
            removeInvalidXMLCharacters(node.getChildNodes());
        }
    }

    /**
     * Handle fault.
     */
    @Override
    public boolean handleFault(SOAPMessageContext smc) {
        LOGGER.debug("handle fault");
        return true;
    }

    /**
     * Close context.
     */
    @Override
    public void close(MessageContext context) {
        LOGGER.debug("close context");
    }
}