/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.bhie.patient;

import gov.va.med.bhie.BaseTest;
import gov.va.med.bhie.common.ClinicalDomainLoincCode;
import gov.va.med.bhie.dao.DaoException;
import gov.va.med.bhie.dao.patient.V4Dao;
import gov.va.med.bhie.dao.patient.VLERDao;
import gov.va.med.bhie.resultcollection.V4ResultCollection;
import org.junit.Test;

import static org.junit.Assert.assertTrue;

public class BDAv4DaoTest extends BaseTest
{

    @Test
    public void testGetPatientAllergiesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        queryBean.setEndDate(null);
        queryBean.setStartDate(null);
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ALLERGIESV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getAllergy().size() > 0);
    }

    @Test
    public void testGetPatientNotesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.NOTES, queryBean);
        assertTrue(v4Results.getDataRecordList().getNotes().size() > 0);
    }

    @Test
    public void testGetPatientDischargeSummaries() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.INPATIENT_NOTES, queryBean);
        assertTrue(v4Results.getDataRecordList().getInpatientNotes().size() > 0);
    }

    @Test
    public void testGetHAIMSNotesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.DOCUMENTS, queryBean);
        assertTrue(v4Results.getDataRecordList().getDocuments().size() > 0);
    }

    @Test
    public void testGetInpatientNotesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.INPATIENT_NOTES, queryBean);
        assertTrue(v4Results.getDataRecordList().getInpatientNotes().size() > 0);
    }

    @Test
    public void testGetPatientEncounterNotesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ENCOUNTER_NOTES, queryBean);
        assertTrue(v4Results.getDataRecordList().getEncounterNotes().size() > 0);
    }

    @Test
    public void testGetPatientEncountersV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ENCOUNTERS, queryBean);
        assertTrue(v4Results.getDataRecordList().getEncounters().size() > 0);
    }

    @Test
    public void testGetPatientProceduresV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.PROCEDURES, queryBean);
        assertTrue(v4Results.getDataRecordList().getProcedures().size() > 0);
    }

    @Test
    public void testGetPatientAppointmentsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.APPOINTMENTS, queryBean);
        assertTrue(v4Results.getDataRecordList().getAppointments().size() > 0);
    }

    @Test
    public void testGetPatientAdmissionsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ADMISSIONSV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getAdmissions().size() > 0);
    }

    @Test
    public void testGetPatientImmunizationsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.IMMUNIZATIONS, queryBean);
        assertTrue(v4Results.getDataRecordList().getImmunizations().size() > 0);
    }

    @Test
    public void testGetPatientMedicationsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.MEDICATIONS, queryBean);
        assertTrue(v4Results.getDataRecordList().getMedications().size() > 0);
    }

    @Test
    public void testGetPatientOrdersV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ALL_ORDERS, queryBean);
        assertTrue(v4Results.getDataRecordList().getMedicationOrders().size() > 0
            && v4Results.getDataRecordList().getLabOrders().size() > 0
            && v4Results.getDataRecordList().getRadiologyOrders().size() > 0);
    }

    @Test
    public void testGetPatientDemographicsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.PATIENT_DEMOGRAPHICSV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getDemographics().size() > 0);
    }

    @Test
    public void testGetPatientProblemsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.PROBLEMS, queryBean);
        assertTrue(v4Results.getDataRecordList().getProblems().size() > 0);
    }

    @Test
    public void testGetPatientQuestionnairesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.QUESTIONNAIRES, queryBean);
        assertTrue(v4Results.getDataRecordList().getQuestionnaires().size() > 0);
    }

    @Test
    public void testGetPatientDeploymentFormsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.DEPLOYMENT_FORMS, queryBean);
        assertTrue(v4Results.getDataRecordList().getNotes().size() > 0);
    }

    @Test
    public void testGetPatientInsuranceV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.INSURANCES, queryBean);
        assertTrue(v4Results.getDataRecordList().getPayers().size() > 0);
    }

    @Test
    public void testGetPatientLabChemistriesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.LAB_CHEMISTRIES, queryBean);
        assertTrue(v4Results.getDataRecordList().getLabChemResults().size() > 0);
    }

    @Test
    public void testGetPatientLabMicroV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.LAB_MICROBIOLOGIES, queryBean);
        assertTrue(v4Results.getDataRecordList().getLabMicroResults().size() > 0);
    }

    @Test
    public void testGetPatientLabAPV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.LAB_ANATOMIC_PATHOLOGIES, queryBean);
        assertTrue(v4Results.getDataRecordList().getLabMicroResults().size() > 0);
    }

    @Test
    public void testGetPatientRadiologyReportsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.RADIOLOGY_REPORTSV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getRadiologyReports().size() > 0);
    }

    @Test
    public void testGetPatientFamilyHistoriesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.FAMILY_HISTORIES, queryBean);
        assertTrue(v4Results.getDataRecordList().getFamilyHistory().size() > 0);
    }

    @Test
    public void testGetPatientSocialHistoriesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.SOCIAL_HISTORIES, queryBean);
        assertTrue(v4Results.getDataRecordList().getSocialHistory().size() > 0);
    }

    @Test
    public void testGetPatientOPMHistoriesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.OTHER_PAST_MEDICAL_HISTORIES, queryBean);
        assertTrue(v4Results.getDataRecordList().getOpmHistory().size() > 0);
    }

    @Test
    public void testGetPatientVitalsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.VITALS, queryBean);
        assertTrue(v4Results.getDataRecordList().getVitals().size() > 0);
    }

}
