/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for Pacific Telehealth & Technology Hui/JITC
 */

package gov.va.med.foundations.utilities;

import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Exposes utility methods for handling exceptions
 * @author Application Modernization - Foundations Team
 * @version 1.4.3.007
 */
public class ExceptionUtils {
	
	/**
	 * Private constructor to prevent instantiation of this class objects.
	 */
	private ExceptionUtils() {
	}

	/**
	 * Method getFullStackTrace.
	 * <br>Gets the full stack trace as a string. 
	 * 
	 * @param e
	 * @return String
	 */
	public static String getFullStackTrace(Throwable e) {
//		ByteArrayOutputStream byteOutStr = new ByteArrayOutputStream();
//		PrintWriter writer = new PrintWriter(byteOutStr);
//		e.printStackTrace(writer);
//		writer.flush();
//		writer.close();
//		return new String(byteOutStr.toByteArray());
        //---
//        ByteArrayOutputStream byteOutStr = new ByteArrayOutputStream();
//        e.printStackTrace(new PrintStream(byteOutStr));
//        return new String(byteOutStr.toByteArray());
        //--
        String stackTrace = "";
        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        try {

            PrintStream ps = new PrintStream( baos, true, "UTF8" );
            e.printStackTrace(ps);
            stackTrace = baos.toString( "UTF8" ); // charsetName e.g. ISO-8859-1
            ps.close();
        }
        catch( UnsupportedEncodingException ex )
        {
            Logger.getLogger(ExceptionUtils.class.getName()).log(Level.SEVERE, "UnsupportedEncodingException: ", ex);
        }
        try {
            baos.close();
        }
        catch( IOException ex )
        {
            Logger.getLogger(ExceptionUtils.class.getName()).log(Level.SEVERE, "IOException: ", ex);
        }
        return stackTrace;
	}
	
	/**
	 * Method getNestedExceptionByClass.
	 * <br>Gets the nested exception if exception is an instance of the exceptionClass or
	 * if exception implements FoundationsExceptionInterface and one of the nested 
	 * exceptions is an instance of the type exceptionClass.
	 * <br>If desired instance of exceptionClass is not found in the nested exception stack
	 * then null is returned.
	 * <br>Can be used to unwind nested exception stack that implement FoundationsExceptionInterface.  
	 * @param e
	 * @param exceptionClass
	 * @return Throwable
	 */
	public static Throwable getNestedExceptionByClass(Throwable e, Class exceptionClass) {
		// If this is the one we are looking for, just return it
		if (exceptionClass.isInstance(e)) {
			return e;
		}
		
		// If e is FoundationsExceptionInterface check nested exception
		if (FoundationsExceptionInterface.class.isInstance(e)) {
			Throwable nestedException = ((FoundationsExceptionInterface)e).getNestedException();
			return ExceptionUtils.getNestedExceptionByClass(nestedException, exceptionClass);
		}
		  
		return null;	
	}

}
