/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for Pacific Telehealth & Technology Hui/JITC
 */

package gov.va.med.vhahon.common;
/*
 * Utils.java
 */

import hcrypto.Crypto;
import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import org.json.*;

public final class Utils
{
    private static final Logger logger = Logger.getLogger(Utils.class);

    public static final int DATE_FORMAT_TO_MODEL = 0;
    public static final int DATE_FORMAT_FROM_MODEL = 1;
    public static final int DATE_FORMAT_FROM_MODEL_DASHES = 2;

    public static final int DATE_FORMAT_TYPE1 = 0;
    public static final int DATE_FORMAT_TYPE2 = 1;
    public static final int DATE_FORMAT_TYPE3 = 2;
    public static final int DATE_FORMAT_TYPE4 = 3;

    public static final int MONTH_NAME_SHORT = 0;
    public static final int MONTH_NAME_FULL = 1;

    private static final String[] months = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
    private static final String[] monthsFull = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};

    private Utils() {
    }

    /*
       // For retrieving application-scoped data... this data in initialized
       // when the ControlServlet "init()" method executes. A jsp:Usebean tag
       // can be used to retrieve this data (AppSettingsBean), from Application scope.
       public static Object getApplicationSettings(HttpServlet servlet)
       {
            return servlet.getServletContext().getAttribute(HttpKeys.KEY_APPSETTINGSBEAN);
       }
    */
    // For Testing...
    public static long getSeconds()
    {
        Calendar cal = GregorianCalendar.getInstance();
        Date d = cal.getTime();
        return d.getTime() / 1000;
    }

    public static Date getDateYYYYMMDD(String dateX)
    {
        Date retDate = null;

        if (dateX != null)
        {
            String format = "yyyy-MM-dd";

            if (dateX.length() != 0)
            {
                try
                {
                    java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                    retDate = df.parse(dateX);
                }
                catch (ParseException e)
                {
                    retDate = null;
                }
            }
        }

        return retDate;
    }

    /**
     * Returns todays date in dd-MMM-yy format
     */
    public static String getTodaysDate()
    {
        return getFormattedDate(null, DATE_FORMAT_TYPE2);
    }

    public static String getFormattedDate(Calendar cal)
    {
        if (cal == null) cal = GregorianCalendar.getInstance();
        String y = Integer.toString(cal.get(Calendar.YEAR)).substring(2);
        return cal.get(Calendar.DAY_OF_MONTH) + "-" +
                months[cal.get(Calendar.MONTH)] + "-" + y;
    }

    /**
     * Returns a formatted string
     * Format type:
     * DATE_FORMAT_TYPE1 (0) -  dd-MMM-yy (default)
     * DATE_FORMAT_TYPE2 (1) -  dd-MMM-yyyy   THIS FORMAT NEEDED TO GET PREVIOUS AND NEXT DATES!!!
     * DATE_FORMAT_TYPE3 (2) -  mm/dd/yyyy
     */
    public static String getFormattedDate(Calendar cal, int type)
    {
        String s = "";
        String y;
        int m;
        if (cal == null) cal = GregorianCalendar.getInstance();
        switch (type)
        {
            case DATE_FORMAT_TYPE1:
                y = Integer.toString(cal.get(Calendar.YEAR)).substring(2);
                s = cal.get(Calendar.DAY_OF_MONTH) + "-" + months[cal.get(Calendar.MONTH)] + "-" + y;
                break;
            case DATE_FORMAT_TYPE2:
                s = cal.get(Calendar.DAY_OF_MONTH) + "-" + months[cal.get(Calendar.MONTH)] + "-" + cal.get(Calendar.YEAR);
                break;
            case DATE_FORMAT_TYPE3:
                m = cal.get(Calendar.MONTH) + 1;
                s = m + "/" + cal.get(Calendar.DAY_OF_MONTH) + "/" + cal.get(Calendar.YEAR);
                break;
            case DATE_FORMAT_TYPE4:
                m = cal.get(Calendar.MONTH) + 1;
                s = cal.get(Calendar.YEAR) + "-" + m + "-" + cal.get(Calendar.DAY_OF_MONTH);
                break;
            default:
                y = Integer.toString(cal.get(Calendar.YEAR)).substring(2);
                s = cal.get(Calendar.DAY_OF_MONTH) + "-" + months[cal.get(Calendar.MONTH)] + "-" + y;
                break;
        }
        return s;
    }

    /**
     * Returns a date relative to todays date using the params year, month and day
     * in mm/dd/yyyy format (model side format)
     */
    public static String getRelativeDate(int yrs, int mths, int days)
    {
        Calendar cal = GregorianCalendar.getInstance();
        cal.add(Calendar.YEAR, yrs);
        cal.add(Calendar.MONTH, mths);
        cal.add(Calendar.DAY_OF_YEAR, days);
        return getFormattedDate(cal, DATE_FORMAT_TYPE2);
    }

    public static String getRelativeYMD(int years, int months, int days)
    {
        Calendar c = GregorianCalendar.getInstance();
        c.add(Calendar.YEAR, years);
        c.add(Calendar.MONTH, months);
        c.add(Calendar.DAY_OF_YEAR, days);
        return getFormattedDate(c, DATE_FORMAT_TYPE4);
    }

    /**
     * Returns a date relative to the one passed in. If it is null, this retrieves
     * a date relative to todays date.
     * Inputs:  date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  date in 'dd-MMM-yyyy'
     */
    public static String getRelativeDate(String selectedDate, int years, int months, int days)
    {
        Calendar cal = null;

        if (years == 0 && months == 0 && days == 0) return selectedDate;

        if (selectedDate == null || selectedDate.equals(""))
        {
            cal = GregorianCalendar.getInstance();
        }
        else
        {
            cal = getCalendar(selectedDate, null);
        }

        if (years != 0) cal.add(Calendar.YEAR, years);
        if (months != 0) cal.add(Calendar.MONTH, months);
        if (days != 0) cal.add(Calendar.DAY_OF_YEAR, days);

        return getFormattedDate(cal, DATE_FORMAT_TYPE2);
    }

    /**
     * Taken from String.asp
     * *****************************************************************
     * Name:		formattedSSN
     * Purpose:	formats a SSN (20111223333 to 20/111-22-3333)
     * Inputs:	ssn
     * Returns:	formatted ssn (Success)
     * unformatted string (input is invalid, null or not 11 chars).
     */
    public static String formattedSSN(String t)
    {
        StringBuffer f = new StringBuffer("");
        if (t != null && t.length() == 11)
        {
            f.append(t.substring(0, 2));
            f.append("/");
            f.append(t.substring(2, 5));
            f.append("-");
            f.append(t.substring(5, 7));
            f.append("-");
            f.append(t.substring(7, 11));
            return f.toString();
        }
        else
        {
            return t;
        }
    }

    /**
     * Taken from String.asp
     * *****************************************************************
     * Name:		formString
     * Purpose:	ensures that a string exists (returns "" for NULL)
     * Inputs:	string
     * Returns:	string (same text string, or "" if NULL)
     */
    public static String formString(String s)
    {
        if (s == null) return "";
        else return s;
    }

    public static Date getDate(String dateX)
    {
        Date retDate = null;
        String format = "dd-MMM-yyyy";
        if (dateX != null && dateX.length() != 0)
        {
            try
            {
//                if (dateX.indexOf("/")>0)
//                  format = "MM/dd/yy";
//                else if (dateX.indexOf("-")<=0)
//                	format = "dd MMM yyyy";
                if (dateX.indexOf("/") > 0)
                    format = "MM/dd/yy";
                else if (dateX.indexOf(" ") == 2)
                    format = "dd MMM yyyy";
                else if (dateX.indexOf("-") > 0 && dateX.indexOf(":") > 0)
                    format = "yyyy-MM-dd HH:mm:ss";
                else if (dateX.indexOf("-") > 0 && dateX.indexOf("@") > 0)
                    format = "dd-MMM-yyyy@HHmm";
                else if (dateX.indexOf(",") > 0 && dateX.indexOf("@") > 0)
                    format = "MMM dd, yyyy@HH:mm";
                else if (dateX.indexOf("@") < 0 && dateX.indexOf("-") == 4)
                    format = "yyyy-MM-dd";
                else if (dateX.indexOf("@") > 0 && dateX.indexOf("-") < 0)
                    format = "ddMMMyy@HHmm";
                else if (dateX.length() == 8 && dateX.indexOf("-") < 0)
                    format = "yyyyMMdd";
                else if (dateX.length() == 12)
                    format = "yyyyMMddHHmm";
                else if (dateX.length() == 14)
                    format = "yyyyMMddHHmmss";
                else if (dateX.indexOf(".") > 12 && dateX.length() > 14 && dateX.length() < 19)
                    format = "yyyyMMddHHmmss.SSS";
                else if (dateX.indexOf(".") > 12 && dateX.length() > 18)
                    format = "yyyyMMddHHmmss.SSSZ";
                else if (dateX.length() == 7)
                    format = "ddMMMyy";
                else if (dateX.length() == 19 && (dateX.contains("AM") || dateX.contains("PM")))
                    format = "MMM dd yyyy hh:mma";
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                retDate = df.parse(dateX);
            }
            catch (ParseException e)
            {
                retDate = null;
            }
        }
        return retDate;

    }

    public static XMLGregorianCalendar getXMLCal(String dateX)
    {
        Date dt = getDate(dateX);
        return toXMLGregorianCalendar(dt);
    }

    public static Calendar getCal(String dateX)
    {
        Date d = null;
        Calendar retCal = null;
        String format = "dd-MMM-yyyy";
        if (dateX != null && dateX.length() != 0)
        {
            try
            {

                if (dateX.indexOf("/") > 0)
                    format = "MM/dd/yy";
                else if (dateX.indexOf(" ") == 2)
                    format = "dd MMM yyyy";
                else if (dateX.indexOf("-") > 0 && dateX.indexOf(":") > 0)
                    format = "yyyy-MM-dd HH:mm:ss";
                else if (dateX.indexOf("-") > 0 && dateX.indexOf("@") > 0)
                    format = "dd-MMM-yyyy@HHmm";
                else if (dateX.indexOf(",") > 0 && dateX.indexOf("@") > 0)
                    format = "MMM dd, yyyy@HH:mm";
                else if (dateX.indexOf("@") < 0 && dateX.indexOf("-") == 4)
                    format = "yyyy-MM-dd";
                else if (dateX.indexOf("@") > 0 && dateX.indexOf("-") < 0)
                    format = "ddMMMyy@HHmm";
                else if (dateX.length() == 12)
                    format = "yyyyMMddHHmm";
                else if (dateX.length() == 14)
                    format = "yyyyMMddHHmmss";
                else if (dateX.length() == 7)
                    format = "ddMMMyy";
                else if (dateX.length() == 8)
                    format = "yyyyMMdd";
                else if (dateX.indexOf(".") > 12 && dateX.length() > 14 && dateX.length() < 19)
                    format = "yyyyMMddHHmmss.SSS";
                else if (dateX.indexOf(".") > 12 && dateX.length() > 18)
                    format = "yyyyMMddHHmmss.SSSZ";
                else if (dateX.indexOf(".") > 7)
                    format = "yyyyMMdd.HHmmss";
                else if (dateX.length() == 19 && (dateX.contains("AM") || dateX.contains("PM")))
                    format = "MMM dd yyyy hh:mma";
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                d = df.parse(dateX);
                retCal = new java.util.GregorianCalendar();
                retCal.setTime(d);
            }
            catch (ParseException e)
            {
                retCal = null;
            }
        }
        return retCal;

    }

    public static Date getDateFromVistA(String dateX)
    {
        Date retDate = null;
        String format = "MMM dd, yyyy@HH:mm";
        TimeZone tz = TimeZone.getDefault();

        if (dateX != null && dateX.length() != 0)
        {
            if (Utils.getPiece(dateX, "@", 2) == null)
                format = "MMM dd, yyyy";
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                df.setTimeZone(tz);
                retDate = df.parse(dateX);
            }
            catch (ParseException e)
            {
                retDate = null;
            }
        }
        return retDate;

    }

    public static Calendar getCalFromVistA(String dateX)
    {
        Calendar cal = null;
        Date vistADate = getDateFromVistA(dateX);
        if (vistADate != null)
        {
            cal = new java.util.GregorianCalendar();
            cal.setTime(vistADate);
        }

        return cal;
    }

    public static Calendar getCalFromSQLFormat(String dateX)
    {
        Calendar cal = null;

        Date dt = getDateFromSQLFormat(dateX);
        if (dt != null)
        {
            cal = new java.util.GregorianCalendar();
            cal.setTime(dt);
        }
        return cal;
    }

    public static Calendar getCalFromFormat(String dateTime, String format)
            throws IllegalArgumentException, ParseException
    {
        if (null == format || null == dateTime)
        {
            throw new IllegalArgumentException("Supplied arguments cannot be null");
        }

        SimpleDateFormat df = new java.text.SimpleDateFormat(format);
        Date date = df.parse(dateTime);
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTime(date);
        return gregorianCalendar;
    }

    public static Date getDateFromSQLFormat(String dateX)
    {
        Date retDate = null;

        if (dateX != null)
        {
            String format = "yyyy-MM-dd HH:mm:ss";

            if (dateX.indexOf("-") > 0 && dateX.indexOf(":") > 0)
                format = "yyyy-MM-dd HH:mm:ss";
            else if (dateX.indexOf("-") > 0 && dateX.indexOf("@") > 0)
                format = "dd-MMM-yyyy@HHmm";
            else if (dateX.indexOf(",") > 0 && dateX.indexOf("@") > 0)
                format = "MMM dd, yyyy@HH:mm";
            else if (dateX.indexOf("@") < 0 && dateX.indexOf("-") > 0)
                format = "yyyy-MM-dd";
            else if (dateX.indexOf("@") > 0 && dateX.indexOf("-") < 0)
                format = "ddMMMyy@HHmm";
            else if (dateX.length() == 14)
                format = "yyyyMMddHHmmss";
            else if (dateX.indexOf(".") > 12 && dateX.length() > 14 && dateX.length() < 19)
                format = "yyyyMMddHHmmss.SSS";
            else if (dateX.indexOf(".") > 12 && dateX.length() > 18)
                format = "yyyyMMddHHmmss.SSSZ";
            else format = "ddMMMyy";

            if (dateX.length() != 0)
            {
                try
                {
                    java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                    retDate = df.parse(dateX);
                }
                catch (ParseException e)
                {
                    retDate = null;
                }
            }
        }

        return retDate;

    }

    /**
     * Taken from DateUtils.asp
     * *****************************************************************
     * Name:		formattedDate
     * Purpose:	formats a Date dd Mon yyyy
     * Inputs:	date in 'dd MMM yyyy' format - will remove a trailing '@time'
     * Returns:	formatted date
     */
    public static String formatDate(String dateX)
    {
        if (dateX != null && dateX.length() != 0)
        {
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd MMM yyyy");
                java.util.Date d = df.parse(dateX);
                df.applyPattern("dd-MMM-yyyy");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return "";
            }
        }
        return " ";
    }

    /**
     * Taken from DateUtils.asp
     * *****************************************************************
     * Name:		formattedDate
     * Purpose:	formats a Date dd Mon yyyy
     * Inputs:	date in 'dd MMM yyyy' format - will remove a trailing '@time'
     * Returns:	formatted date
     */
    public static String formatDate(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd MMM yyyy");

            return df.format(dateX);
        }
        return "";
    }

    public static String formatDateYMD(Date date)
    {
        java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd");
        return df.format(date);
    }

    /**
     * Taken from DateUtils.asp
     * *****************************************************************
     * Name:		formattedDate2
     * Purpose:	formats a Date dd Mon yyyy
     * Inputs:	date in 'dd MMM yyyy' format - will remove a trailing '@time'
     * Returns:	formatted date dd-MM-yyyy
     */
    public static String formatDate2(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd-MMM-yyyy");

            return df.format(dateX);
        }
        return "";
    }

    /**
     * Taken from DateUtils.asp
     * *****************************************************************
     * Name:		formattedDate2
     * Purpose:	formats a Date dd Mon yyyy
     * Inputs:	date in 'dd MMM yyyy' format - will remove a trailing '@time'
     * Returns:	formatted date dd-MM-yyyy
     */
    public static String formatSQLDateTime(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            return df.format(dateX);
        }
        return "";
    }

    /**
     * *****************************************************************
     * Name:		shortDate
     * Purpose:	formats a Calendar to yyyyMMdd
     * Inputs:	Calendar object
     * Returns:	formatted date yyyyMMdd
     */
    public static String shortDate(Calendar cal)
    {
        if (cal != null)
        {
            Date dt = cal.getTime();
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyyMMdd");

            return df.format(dt);
        }
        return " ";
    }

    /**
     * *****************************************************************
     * Name:		getFormattedDate
     * Purpose:	formats a Calendar to yyyyMMdd
     * Inputs:	Calendar object
     * Returns:	formatted date yyyyMMdd
     */
    public static String getFormattedDate(Calendar cal, String dateFormat)
    {
        if (cal != null)
        {
            Date dt = cal.getTime();
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(dateFormat);

            return df.format(dt);
        }
        return " ";
    }

    public static String formatSQLDate(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd");

            return df.format(dateX);
        }
        return " ";
    }


    /**
     * ***************************************************************
     * Name:		formatDisplayDateTime
     * Purpose:	formats a Date MMM dd, yyyy HH:MM
     * Inputs:	date
     * Returns:	formatted date
     */
    public static String formatDisplayDateTime(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("MMM dd, yyyy HH:mm");
            df.setTimeZone(df.getTimeZone());
            return df.format(dateX);
        }
        return "";
    }

    /**
     * ***************************************************************
     * Name:		formatDisplayDate
     * Purpose:	formats a Date MMM dd, yyyy
     * Inputs:	date
     * Returns:	formatted date
     */
    public static String formatDisplayDate(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("MMM dd, yyyy");

            return df.format(dateX);
        }
        return "";
    }

    /**
     * ***************************************************************
     * Name:		formatDisplayDate
     * Purpose:	formats a Date MMM dd, yyyy
     * Inputs:	Calendar
     * Returns:	formatted date
     */
    public static String formatDisplayDate(Calendar dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("MMM dd, yyyy");

            return df.format(dateX.getTime());
        }
        return "";
    }


    /**
     * Formats a date coming from ESI model or going to ESI Model.
     * FROM Model (input);
     * input expected; "dd MMM yyyy"
     * output; "d-MMM-yyyy"
     * <p/>
     * TO Model (output);
     * input expected; "dd-MMM-yyyy"
     * output; "M/d/yyyy"
     * <p/>
     * NOTE; there are NO leading 0's in the output month and date fields.
     */
    public static String formatESIModelDate(String dateX, int outputFormat)
    {
        if (dateX != null && dateX.length() != 0)
        {
            try
            {
                java.text.SimpleDateFormat df = null;
                java.util.Date d = null;

                switch (outputFormat)
                {
                    case DATE_FORMAT_FROM_MODEL_DASHES:
                        df = new java.text.SimpleDateFormat("dd-MMM-yyyy");
                        d = df.parse(dateX);
                        df.applyPattern("d-MMM-yyyy");
                        break;
                    case DATE_FORMAT_FROM_MODEL:
                        df = new java.text.SimpleDateFormat("dd MMM yyyy");
                        d = df.parse(dateX);
                        df.applyPattern("d-MMM-yyyy");
                        break;

                    case DATE_FORMAT_TO_MODEL:
                    default:
                        df = new java.text.SimpleDateFormat("dd-MMM-yyyy");
                        d = df.parse(dateX);
                        df.applyPattern("M/d/yyyy");
                        break;
                }

                return df.format(d);
            }
            catch (ParseException e)
            {
                return null;
            }
        }
        return null;
    }

    /**
     * ***************************************************************
     * Name:		formatDisplayDateTime
     * Purpose:	formats a Date MMM dd, yyyy HH:MM
     * Inputs:	date
     * Returns:	formatted date
     */
    public static String formatDisplayDateTime(XMLGregorianCalendar calX)
    {
        if (calX != null)
        {
            SimpleDateFormat sdf = new SimpleDateFormat("MMM dd, yyyy hh:mm aaa");
            sdf.setTimeZone(calX.getTimeZone(DatatypeConstants.FIELD_UNDEFINED));
            String date = sdf.format(calX.toGregorianCalendar().getTime());
            return date;
        }
        return "";

    }

    /**
     * ***************************************************************
     * Name:		formatDisplayDate
     * Purpose:	formats a Date MMM dd, yyyy
     * Inputs:	XMLGregorianCalendar
     * Returns:	formatted date
     */
    public static String formatDisplayDate(XMLGregorianCalendar calX)
    {
        if (calX != null)
        {
            SimpleDateFormat sdf = new SimpleDateFormat("MMM dd, yyyy hh:mm aaa");
            sdf.setTimeZone(calX.getTimeZone(DatatypeConstants.FIELD_UNDEFINED));
            String outputDate = sdf.format(calX.toGregorianCalendar().getTime());
            return outputDate;
        }
        return "";
    }

    /**
     * ***************************************************************
     * Name:		formatDateMilTime
     * Purpose:	formats a Date to dd-MMM-yyyy HHmm
     * Inputs:	date in 'dd MMM yyyy@HHmm' format
     * Returns:	String formatted date and military time
     */
    public static String formatDateMilTime(String dateX)
    {

        if (dateX != null && dateX.length() != 0)
        {
            if (dateX.indexOf("@0000") > 0)
            {
                return formatDate(dateX);
            }
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd MMM yyyy@HHmm");
                java.util.Date d = df.parse(dateX);
                df.applyPattern("dd-MMM-yyyy@HHmm");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return " ";
            }
        }
        return " ";
    }

    /**
     * ***************************************************************
     * Name:		formatDateMilTime
     * Purpose:	formats a Date to yyyy-dd-MM HH:mm:ss
     * Inputs:	date in 'MMM dd, yyyy@HH:mm:ss' format
     * Returns:	String SQL formatted date time
     */
    public static String formatSQLDateTime(String dateX)
    {
        if (dateX == null)
            return " ";

        if (dateX.length() != 0)
        {

            String format = "";

            if (dateX.indexOf("-") > 0)
                format = "dd-MMM-yyyy@HHmm";
            else if (dateX.indexOf(",") > 0)
                format = "MMM dd, yyyy@HH:mm";
            else format = "ddMMMyy@HHmm";

            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                java.util.Date d = df.parse(dateX);
                df.applyPattern("yyyy-MM-dd HH:mm");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return " ";
            }
        }
        return " ";
    }

    /**
     * ***************************************************************
     * Name:		formatDateMilTime
     * Purpose:	formats a Date to yyyy-dd-MM HH:mm:ss
     * Inputs:	date in 'yyyy-mm-dd' format
     * Returns:	String SQL formatted date time
     */

    public static String getDateYYYYMMDDHHmmss(String dateX)
    {
        if (dateX == null)
            return " ";

        if (dateX.length() != 0)
        {

            String format = "";

            if (dateX.indexOf("-") > 0 && dateX.indexOf("@") > 0)
                format = "yyyy-MM-dd@HHmmss";
            else if (dateX.indexOf("-") > 0)
                format = "yyyy-MM-dd";
            else if (dateX.indexOf(",") > 0)
                format = "MMM dd, yyyy@HH:mm:ss";
            else format = "ddMMMyy@HHmmss";

            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                java.util.Date d = df.parse(dateX);
                df.applyPattern("yyyyMMddHHmmss");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return " ";
            }
        }
        return " ";
    }

    /**
     * ***************************************************************
     * Name:		formatDateMilTime
     * Purpose:	formats a Date to yyyy-dd-MM HH:mm
     * Inputs:	date in 'yyyy-mm-dd' format
     * Returns:	String SQL formatted date time
     */

    public static String getDateYYYYMMDDHHmm(String dateX)
    {
        if (dateX == null)
            return " ";

        if (dateX.length() != 0)
        {

            String format = "";

            if (dateX.indexOf("-") > 0 && dateX.indexOf("@") > 0)
                format = "yyyy-MM-dd@HHmm";
            else if (dateX.indexOf("-") > 0)
                format = "yyyy-MM-dd";
            else if (dateX.indexOf(",") > 0)
                format = "MMM dd, yyyy@HH:mm";
            else format = "ddMMMyy@HHmm";

            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                java.util.Date d = df.parse(dateX);
                df.applyPattern("yyyyMMddHHmmss");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return " ";
            }
        }
        return " ";
    }

    /**
     * ***************************************************************
     * Name:		formatDateMilTime
     * Inputs:	formats a Date to yyyy-dd-MM HH:mm:ss
     * Outputs:	date in 'MMM dd, yyyy@HH:mm' format
     * Returns:	String SQL formatted date time
     */
    public static String formatDateTime(String dateX)
    {
        if (dateX != null && dateX.length() != 0)
        {
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                java.util.Date d = df.parse(dateX);
                df.applyPattern("MMM dd, yyyy@HH:mm");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return " ";
            }
        }
        return " ";
    }

    /**
     * ***************************************************************
     * Name:		getCalendar
     * Purpose:	Get a calendar object for the date string
     * Inputs:	date in 'dd-MMM-yyyy' format
     * optional time in 'HHmm' format
     * Returns:	GregorianCalendar object
     */
    public static java.util.Calendar getCalendar(String d, String t)
    {
        Calendar c = null;
        if (d == null || d.length() == 0)
        {
            d = getFormattedDate(null, 1);
        }
        if (t == null || t.length() == 0)
        {
            t = "0000";
        }
        try
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd-MMM-yyyy@HHmm");
            java.util.Date date = df.parse(d + "@" + t);
            c = new java.util.GregorianCalendar();
            c.setTime(date);
        }
        catch (ParseException e)
        {
            c = null;
        }

        return c;
    }

    /**
     * ***************************************************************
     * Name:		getCalendar
     * Purpose:	Get a calendar object for the supplied date
     * Inputs:	date
     * Returns:	Calendar object
     */
    public static java.util.GregorianCalendar getCalendar(Date t)
    {
        GregorianCalendar c = null;
        c = new java.util.GregorianCalendar();
        c.setTime(t);
        return c;
    }

    /**
     * Return the date for start of calendar week, SUNDAY.
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  date in 'dd-MMM-yyyy'
     */
    public static String getWeekStartDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return null;

        Calendar calDate = getCalendar(selectedDate, null);

        int offset = -1;

        while (calDate.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY)
        {
            calDate.add(Calendar.DATE, offset);
        }

        String date = getFormattedDate(calDate, DATE_FORMAT_TYPE2);
        return date;
    }

    /**
     * Return the date for end of calendar week, SATURDAY.
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  date in 'dd-MMM-yyyy'
     */
    public static String getWeekEndDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return null;

        Calendar calDate = getCalendar(selectedDate, null);

        int offset = 1;

        while (calDate.get(Calendar.DAY_OF_WEEK) != Calendar.SATURDAY)
        {
            calDate.add(Calendar.DATE, offset);
        }

        String date = getFormattedDate(calDate, DATE_FORMAT_TYPE2);
        return date;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  date in 'dd-MMM-yyyy'
     */
    public static String getPreviousDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return null;

        Calendar calDate = getCalendar(selectedDate, null);

        int offset = -1;

        // This line skip weekends
        // if(calDate.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY) offset = -3;

        calDate.add(Calendar.DATE, offset);
        String date = getFormattedDate(calDate, DATE_FORMAT_TYPE2);
        return date;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  date in 'dd-MMM-yyyy' format.
     */
    public static String getNextDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return null;

        Calendar calDate = getCalendar(selectedDate, null);
        int offset = 1;

        // This line skip weekends
        // if(calDate.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) offset = 3;

        calDate.add(Calendar.DATE, offset);
        String date = getFormattedDate(calDate, DATE_FORMAT_TYPE2);
        return date;
    }

    /**
     * Checks if selectedDate is a SUNDAY, the assumed week start day used by the
     * date methods in this class.
     */
    public static boolean isWeekStartDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return false;

        Calendar calDate = getCalendar(selectedDate, null);
        if (calDate.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) return true;
        else return false;
    }

    /**
     * Checks if selectedDate is a SATURDAY, the assumed week start day used by the
     * date methods in this class.
     */
    public static boolean isWeekEndDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return false;

        Calendar calDate = getCalendar(selectedDate, null);
        if (calDate.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) return true;
        else return false;
    }

    /**
     * Checks if selectedDate is a WEEKDAY, MON-FRI
     */
    public static boolean isWeekDayDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return false;

        Calendar calDate = getCalendar(selectedDate, null);
        if (calDate.get(Calendar.DAY_OF_WEEK) != Calendar.SATURDAY &&
                calDate.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY) return true;
        else return false;
    }

    /**
     * Compare dates to see which is sooner or later...
     * Returns;
     * -1 if selectedDate < otherDate
     * 0 if selectedDate = otherDate
     * 1 if selectedDate > otherDate
     */
    public static int compareDates(String selectedDate, String otherDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return 0;
        if (otherDate == null || otherDate.equals("")) return 0;

        Calendar selectedCalDate = getCalendar(selectedDate, null);
        int selectedCalYear = selectedCalDate.get(selectedCalDate.YEAR);
        //int selectedCalMonth = selectedCalDate.get(selectedCalDate.MONTH);
        //int selectedCalDay = selectedCalDate.get(selectedCalDate.DAY_OF_MONTH);

        Calendar otherCalDate = getCalendar(otherDate, null);
        int otherCalYear = otherCalDate.get(otherCalDate.YEAR);
        //int otherCalMonth = otherCalDate.get(otherCalDate.MONTH);
        //int otherCalDay = otherCalDate.get(otherCalDate.DAY_OF_MONTH);

/**
 if(selectedCalYear == otherCalYear &&
 selectedCalMonth == otherCalMonth &&
 selectedCalDay == otherCalDay) return 0;
 **/

        if (selectedCalYear == otherCalYear)
        {
            int selectedCalDay = selectedCalDate.get(selectedCalDate.DAY_OF_YEAR);
            int otherCalDay = otherCalDate.get(otherCalDate.DAY_OF_YEAR);
            if (selectedCalDay == otherCalDay) return 0;
            else if (selectedCalDay < otherCalDay) return -1;
            else return 1;
        }
        else if (selectedCalYear < otherCalYear) return -1;
        else return 1;
    }

    /**
     *
     */
    public static boolean isDateWithinMonth(String selectedDate, int month, int year)
    {
        if (selectedDate == null || selectedDate.equals("")) return false;

        String monthName = getMonthName(month, Utils.MONTH_NAME_SHORT);
        String startMonthDate = "01-" + monthName + "-" + year;
        String endMonthDate = getMonthEndDate(month, year);

        Calendar startCalDate = getCalendar(startMonthDate, null);
        int startCalYear = startCalDate.get(startCalDate.YEAR);
        int startCalMonth = startCalDate.get(startCalDate.MONTH);
        int startCalDay = startCalDate.get(startCalDate.DAY_OF_MONTH);

        Calendar endCalDate = getCalendar(endMonthDate, null);
        int endCalDay = endCalDate.get(endCalDate.DAY_OF_MONTH);

        Calendar selectedCalDate = getCalendar(selectedDate, null);
        int selectedCalYear = selectedCalDate.get(selectedCalDate.YEAR);
        int selectedCalMonth = selectedCalDate.get(selectedCalDate.MONTH);
        int selectedCalDay = selectedCalDate.get(selectedCalDate.DAY_OF_MONTH);

        if (selectedCalYear == startCalYear &&
                selectedCalMonth == startCalMonth)
        {
            if (selectedCalDay >= startCalDay)
            {
                if (selectedCalDay <= endCalDay)
                {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  date in 'dd-MMM-yyyy' format.
     */
    public static String getMonthEndDate(int month, int year)
    {
        // There are at least 28 days in a month, so start incrementing
        // to find the end starting on the 28th...
        if (month <= 0 || month > 12) month = 1;

        String mo = months[month - 1];
        String date = "28-" + mo + "-" + year;
        return getMonthEndDate(date);
    }

    public static String getMonthEndDate(String selectedDate)
    {
        if (selectedDate == null || selectedDate.equals("")) return null;

        Calendar calDate = getCalendar(selectedDate, null);

        int offset = 1;
        int currMonth = calDate.get(Calendar.MONTH);
        int nextMonth = currMonth;

        while (currMonth == nextMonth)
        {
            calDate.add(Calendar.DATE, offset);
            nextMonth = calDate.get(Calendar.MONTH);
        }

        calDate.add(Calendar.DATE, -offset);

        String date = getFormattedDate(calDate, DATE_FORMAT_TYPE2);
        return date;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  year number
     */
    public static int getYear(String selectedDate) {
        String format = "dd-MMM-yyyy";

        if (selectedDate == null || selectedDate.equals("")) return -1;

        int i = selectedDate.indexOf('-');
        if (i <= 0) format = "dd MMM yyyy";


        try {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
            java.util.Date d = df.parse(selectedDate);
            df.applyPattern("yyyy");
            String year = df.format(d);
            return Integer.parseInt(year);
        }catch (ParseException e) {
            logger.error("Invalid date: " + selectedDate);
        }catch (NumberFormatException nfe) {
            logger.error("Invalid year: " + selectedDate);
        }
        return -1;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  month number (1 = JAN).
     */
    public static int getMonthNumber(String selectedDate) {
        String format = "dd-MMM-yyyy";

        if (selectedDate == null || selectedDate.equals("")) return -1;

        int i = selectedDate.indexOf('-');
        if (i <= 0) format = "dd MMM yyyy";


        try {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
            java.util.Date d = df.parse(selectedDate);
            df.applyPattern("M");
            String month = df.format(d);
            return Integer.parseInt(month);
        }catch (ParseException e) {
            logger.error("Invalid date: " + selectedDate);
        }catch (NumberFormatException nfe) {
            logger.error("Invalid month: " + selectedDate);
        }
        return -1;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  month name abbreviated or full name
     */
    public static String getMonthName(String selectedDate, int type)
    {
        String monthName = null;
        int month = getMonthNumber(selectedDate);
        return getMonthName(month, type);
    }

    /**
     * Inputs:	month number (1 = Jan)
     * Output:  month name abbreviated or full name
     */
    public static String getMonthName(int month, int type)
    {
        String monthName = null;
        if (month > 0 && month < 13)
        {
            if (type == MONTH_NAME_SHORT) monthName = months[month - 1];
            else if (type == MONTH_NAME_FULL) monthName = monthsFull[month - 1];
            else monthName = months[month - 1];
        }
        return monthName;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * Output:  day (1 = JAN).
     */
    public static int getDay(String selectedDate) {
        String format = "dd-MMM-yyyy";

        if (selectedDate == null || selectedDate.equals("")) return -1;

        int i = selectedDate.indexOf('-');
        if (i <= 0) format = "dd MMM yyyy";

        try {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
            java.util.Date d = df.parse(selectedDate);
            df.applyPattern("d");
            String day = df.format(d);
            return Integer.parseInt(day);
        }catch (ParseException e) {
            logger.error("Invalid date: " + selectedDate);
        }catch (NumberFormatException nfe) {
            logger.error("Invalid day: " + selectedDate);
        }
        return -1;
    }

    /**
     * Inputs:	date in 'dd-MMM-yyyy' format since getCalendar() requires this.
     * day is abbreviated if abbreviation is true
     * Output:  day (Mon, Tue, etc).
     */
    public static String getDayName(String selectedDate, boolean abbreviation)
    {

        String format = "dd-MMM-yyyy";

        if (selectedDate == null || selectedDate.equals("")) return null;

        int i = selectedDate.indexOf('-');
        if (i <= 0) format = "dd MMM yyyy";

        try
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
            java.util.Date d = df.parse(selectedDate);
            df.applyPattern("E");
            String day = df.format(d);
            if (abbreviation) return day.substring(0, 3);
            else return day;
        }
        catch (ParseException e)
        {
            return null;
        }
    }

    /**
     * <p>Checks if two dates are on the same day ignoring time.</p>
     * @param date1  the first date, not altered, not null
     * @param date2  the second date, not altered, not null
     * @return true if they represent the same day
     * @throws IllegalArgumentException if either date is <code>null</code>
     */
    public static boolean isSameDay(Date date1, Date date2) {
        if (date1 == null || date2 == null) {
            throw new IllegalArgumentException("The dates must not be null");
        }
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        return isSameDay(cal1, cal2);
    }

    /**
     * <p>Checks if two calendars represent the same day ignoring time.</p>
     * @param cal1  the first calendar, not altered, not null
     * @param cal2  the second calendar, not altered, not null
     * @return true if they represent the same day
     * @throws IllegalArgumentException if either calendar is <code>null</code>
     */
    public static boolean isSameDay(Calendar cal1, Calendar cal2) {
        if (cal1 == null || cal2 == null) {
            throw new IllegalArgumentException("The dates must not be null");
        }
        return (cal1.get(Calendar.ERA) == cal2.get(Calendar.ERA) &&
                cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR) &&
                cal1.get(Calendar.DAY_OF_YEAR) == cal2.get(Calendar.DAY_OF_YEAR));
    }

    /**
     * <p>Checks if a date is today.</p>
     * @param date the date, not altered, not null.
     * @return true if the date is today.
     * @throws IllegalArgumentException if the date is <code>null</code>
     */
    public static boolean isToday(Date date) {
        return isSameDay(date, Calendar.getInstance().getTime());
    }

    /**
     * <p>Checks if a calendar date is today.</p>
     * @param cal  the calendar, not altered, not null
     * @return true if cal date is today
     * @throws IllegalArgumentException if the calendar is <code>null</code>
     */
    public static boolean isToday(Calendar cal) {
        return isSameDay(cal, Calendar.getInstance());
    }

    /**
     * Return a copy of String "s" where all occurances of 'oldchar' in String "s"
     * are replaced with 'newchar'. If 'oldchar' does not exist in String "s", "s"
     * is returned.
     */
    public static String stringReplaceChars(String s, char oldchar, char newchar)
    {
        if (s == null) return null;

        int idx = s.indexOf(oldchar);
        if (idx == -1) return s;

        StringBuffer outString = new StringBuffer("");
        char[] inString = s.toCharArray();
        int len = inString.length;
        for (int i = 0; i < len; i++)
        {
            char c = inString[i];
            if (c == oldchar)
            {
                outString.append(newchar);
            }
            else
            {
                outString.append(c);
            }
        }
        return outString.toString();
    }

    /**
     * Return a copy of String "s" where all occurances of 'oldchar' in String "s"
     * are replaced with "newString". If 'oldchar' does not exist in String "s", "s"
     * is returned.
     */
    public static String stringReplaceChars(String s, char oldchar, String newString)
    {
        if (s == null) return null;

        int idx = s.indexOf(oldchar);
        if (idx == -1) return s;

        StringBuffer outString = new StringBuffer("");
        char[] inString = s.toCharArray();
        int len = inString.length;
        for (int i = 0; i < len; i++)
        {
            char c = inString[i];
            if (c == oldchar)
            {
                outString.append(newString);
            }
            else
            {
                outString.append(c);
            }
        }
        return outString.toString();
    }

    public static boolean isEmptyString(String str)
    {
        if (str.equals(""))
        {
            return true;
        }
        return false;
    }

    /**
     * Parse a delimited string to get a set of Name/Value pairs.
     *
     * @param    str        a delimited string
     * @param    delimiter1    a delimiter used in the given string
     * @param    delimiter2    a second delimiter
     * @return Properties    a set of NV pairs
     */
    public static Properties splitToProperties(String str, String delimiter1, String delimiter2)
    {
        StringTokenizer strTokenizer = new StringTokenizer(str, delimiter1);
        Object obj = null;
        Object obj1 = null;
        Object obj2 = null;
        Properties properties = new Properties();

        while (strTokenizer.hasMoreElements())
        {
            String s = strTokenizer.nextToken();
            int i = s.indexOf(delimiter2);
            if (i > 0)
            {
                String name = s.substring(0, i);
                String value = s.substring(i + 1);
                properties.setProperty(name, value);
            }
        }

        return properties;
    }

    /**
     * Parse a delimited string to get a piece using delimiter.
     * like the $P(data,"^",2) function in mumps
     *
     * @param    str        a delimited string
     * @param    delimiter    a delimiter can be '^' -> data^data^data^
     * @return data
     */
    public static String getPiece(String str, String delimiter, int piece)
    {

        if (str == null)
            return str;

        StringTokenizer strTokenizer = new StringTokenizer(str, delimiter);
        int count = 0;
        String sret = null;
        String s = "";
        while (strTokenizer.hasMoreElements())
        {
            count++;
            s = strTokenizer.nextToken();
            if (count == piece)
            {
                return s;
            }
        }
        return sret;
    }

    /**
     * Parse a delimited string to get a set of Name/Value pairs.
     *
     * @param    str        a delimited string
     * @param    delimiter    a delimiter can be '@' -> 11 NOV 2002@08:00
     * @return Properties    a set of NV pairs
     */
    public static String getTime(String str, String delimiter)
    {
        StringTokenizer strTokenizer = new StringTokenizer(str, delimiter);
        String time = strTokenizer.nextToken();

        return time;
    }

    public static String fixPath(String pFilename)
    {
        String id = pFilename;
        if (id.indexOf("\\") >= 0)
        {
            id = id.replace('\\', '/');
        }
        return id;
    }

    // May need to implement this in the future to determine if we are
    // running WS and AS on different machines.
    public static boolean isSingleTier(String propertiesFile)
    {
        return false;
    }

    /**
     * Returns a new String "s" where all occurences of "quote" in String "s"
     * are replaced with HTML representation of "quote". If "quote" does not
     * exist in String "s", original String is returned.
     *
     * @param    s    the original string of characters.
     * @return String    the new string.
     */
    public static String replaceWithHTMLQuote(String s)
    {
        StringBuffer ret = new StringBuffer();

        String newStr = null;

        if (s != null)
        {
            newStr = s;
            char c[] = new char[newStr.length()];
            newStr.getChars(0, newStr.length(), c, 0);

            for (int i = 0; i < c.length; i++)
            {
                if (c[i] == '\"')
                    ret.append("&quot;");
                else
                    ret.append(c[i]);
            }

            return ret.toString();
        }

        return null;
    }

    /**
     * Returns a new String "s" with new line character at 80
     *
     * @param    s    the original string of characters.
     * @return String    the new string.
     */
    public static String formatText(String s)
    {
        if (s == null)
            return s;

        StringTokenizer strTokenizer = new StringTokenizer(s, " ");

        StringBuilder newStr = new StringBuilder("");
        int length = 0;
        while (strTokenizer.hasMoreElements())
        {
            s = strTokenizer.nextToken();
            length = length + s.length() + 1;
            if (length > 80)
            {
                newStr.append("\n");
                length = s.length();
                newStr.append(s);
            }
            else
            {
                newStr.append(" ");
                newStr.append(s);
            }

        }

        return newStr.toString();

    }

    /**
     * Retrieves and formats last name (e.g. TEST,PATIENT D. -> PATIENT D)
     */
    public static String getFirstName(String aString)
    {
        if (aString == null || aString.length() == 0)
        {
            return "";
        }

        int idx = aString.indexOf(',') + 1;
        if (idx <= 1) return "";

        String firstName = aString.substring(idx);

        return firstName.toString().trim();
    }

    /**
     * Retrieves and formats last name (e.g. TEST,PATIENT D. -> Test)
     */
    public static String getLastName(String aString)
    {
        if (aString == null || aString.length() == 0)
        {
            return "";
        }

        int idx = aString.indexOf(',');
        if (idx <= 0) return "";

        StringBuffer finalString = new StringBuffer("");

        String lastName = aString.substring(0, idx);
        if (lastName.length() > 1)
        {
            String firstLetter = lastName.substring(0, 1).toUpperCase();
            finalString.append(firstLetter + lastName.substring(1).toLowerCase());
        }
        else
        {
            finalString.append(lastName);
        }

        return finalString.toString().trim();
    }

    /**
     * For NAME FORMATTING...
     * Capitalizing the first letter of each word in the string passed in,
     * delimited with the "delimiter" passed in. Remaining letters are lower-cased.
     * Whitespace is replaced with "delimiter".
     * return the formatted string.
     * <p/>
     * Ex; KEES,DAVE A  ---> becomes ---> Kees, Dave A
     */
    public static String formatName(String aString, char delimiter)
    {
        if (aString == null || aString.length() == 0)
        {
            return "";
        }

        // First replace whitespaces with the delimiter char...
        char oldchar = ' ';
        // char newchar = delimiter.charAt(0);
        aString = stringReplaceChars(aString, oldchar, delimiter);

        // get string tokens
        StringTokenizer tokens = new StringTokenizer(aString, "" + delimiter);
        StringBuffer finalString = new StringBuffer("");
        // make the first letter to be uppercase for each token
        int i = 0;
        while (tokens.hasMoreTokens())
        {
            String temp = tokens.nextToken();
            String firstLetter = temp.substring(0, 1);
            firstLetter = firstLetter.toUpperCase();
            if (i == 0)
            {
                finalString.append(firstLetter + temp.substring(1).toLowerCase());
            }
            else if (i == 1)
            {
                finalString.append(", " + firstLetter + temp.substring(1).toLowerCase());
            }
            else
            {
                finalString.append(" " + firstLetter + temp.substring(1).toLowerCase());
            }
            i++;
        }
        return finalString.toString().trim();
    }

    /**
     * Retrieves HOUR from time string (e.g. 12:30 -> 12)
     */
    public static int getHour(String aString) {
        if (aString != null && !aString.isEmpty()) {
            String time = aString.substring(0, 2);
            try {
                return Integer.parseInt(time);
            }catch (NumberFormatException nfe) {
                logger.error("Invalid time: " + time);
            }
        }
        return -1;
    }

    public static String getNowTime()
    {
        Calendar cal = Calendar.getInstance();

        int Hour = cal.get(Calendar.HOUR_OF_DAY);
        int Mins = cal.get(Calendar.MINUTE);

        String sHour = Integer.toString(Hour);
        String sMins = Integer.toString(Mins);
        if (Hour < 10)
        {
            sHour = "0" + sHour;
        }
        if (Mins < 10)
        {
            sMins = "0" + sMins;
        }

        String now = sHour + sMins;
        return now;
    }

    public static String getFilemanDate(String dt)
    {
        if (dt == null || " ".equals(dt))
        {
            return "";
        }
        else
        {
            int day = Utils.getDay(dt);
            int month = Utils.getMonthNumber(dt);
            int year = Utils.getYear(dt);
            year = year - 1700;

            String sMonth = "";
            if (month < 10)
                sMonth = "0";

            sMonth = sMonth.concat(Integer.toString(month));
            String sDay = "";
            if (day < 10)
                sDay = "0";
            sDay = sDay.concat(Integer.toString(day));

            String sFdt = Integer.toString(year);
            sFdt = sFdt.concat(sMonth);
            sFdt = sFdt.concat(sDay);

            return sFdt;
        }
    }

    public static Date getDateFromHl7Format(String dateX)
    {
        Date retDate = null;
        String format = "yyyyMMdd";
        TimeZone tz = TimeZone.getDefault();

        if (dateX != null && dateX.length() != 0)
        {
            if (dateX.length() > 8)
                format = "yyyyMMddhhmmss";
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat(format);
                df.setTimeZone(tz);
                retDate = df.parse(dateX);
            }
            catch (ParseException e)
            {
                return retDate;
            }
        }
        return retDate;

    }

    public static String getDateTimeInHL7Format(Date date)
    {
        return new SimpleDateFormat("yyyyMMddHHmmssZ").format(date);
    }

    public static String getDateTimeInHL7Format(String date)
    {
        return new SimpleDateFormat("yyyyMMddHHmmssZ").format(getDate(date));
    }

    public static String getUpImage(String sort, String oldsort)
    {

        if (sort.equals(oldsort))
            return "../images/up1.gif";

        return "../images/up.gif";

    }

    public static String getDownImage(String sort, String oldsort)
    {
        if (sort.equals(oldsort))
            return "../images/down1.gif";

        return "../images/down.gif";

    }

    public static String formatSQLDateSlash(Date dateX)
    {
        if (dateX != null)
        {
            java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("yyyy/MM/dd");

            return df.format(dateX);
        }
        return " ";
    }

    public static javax.xml.datatype.XMLGregorianCalendar toXMLGregorianCalendar(Date _date)
    {
        if (_date == null)
            return null;

        GregorianCalendar c = new GregorianCalendar();
        c.setTime(_date);
        XMLGregorianCalendar date2 = null;
        try {
            date2 = javax.xml.datatype.DatatypeFactory.newInstance().newXMLGregorianCalendar(c);
        } catch (DatatypeConfigurationException e) {
            logger.error(e.getMessage());
        }
        return date2;
    }

    public static javax.xml.datatype.XMLGregorianCalendar toXMLGregorianCalendar(String _date)
    {
        if (_date == null) return null;

        GregorianCalendar c = new GregorianCalendar();
        XMLGregorianCalendar date2 = null;

        c.setTime(getDate(_date));
        try {
            date2 = javax.xml.datatype.DatatypeFactory.newInstance().newXMLGregorianCalendar(c);
        } catch (DatatypeConfigurationException e) {
            logger.error(e.getMessage());
        }
        return date2;
    }

    /**
     * Pushes the Date's time to 12:00AM (00:00:00) of the given date MM/DD/YYYY.
     * Please note a new Date instance is not returned, this method will write over the Date parameter's time.
     *
     * @param date
     * @return Date pushed to start of day
     */
    public static Date pushDateToStartOfDay(Date date)
    {
        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(date.getTime());
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);

        date = cal.getTime();

        return date;
    }

    /**
     * Pushes the Calendar's time to 12:00AM (00:00:00) of the given date MM/DD/YYYY.
     * Please note a new Calendar instance is not returned, this method will write over the Calendar parameter's time
     *
     * @param cal
     * @return Calendar pushed to start of day
     */
    public static Calendar pushCalendarToStartOfDay(Calendar cal)
    {
        Calendar cal2 = Calendar.getInstance();
        cal2.setTimeInMillis(cal.getTimeInMillis());
        cal2.set(Calendar.HOUR_OF_DAY, 00);
        cal2.set(Calendar.MINUTE, 00);
        cal2.set(Calendar.SECOND, 00);

        cal = cal2;

        return cal;
    }

    /**
     * Pushes the Date's time to 11:59PM (23:59:59) of the given date MM/DD/YYYY.
     * Please note a new Date instance is not returned, this method will write over the Date parameter's time.
     *
     * @param date
     * @return Date pushed to end of day
     */
    public static Date pushDateToEndOfDay(Date date)
    {
        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(date.getTime());
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);

        date = cal.getTime();

        return date;
    }

    /**
     * Pushes the Calendar's time to 11:59PM (23:59:59) of the given date MM/DD/YYYY.
     * Please note a new Calendar instance is not returned, this method will write over the Calendar parameter's time
     *
     * @param cal
     * @return Calendar pushed to end of day
     */
    public static Calendar pushCalendarToEndOfDay(Calendar cal)
    {
        Calendar cal2 = Calendar.getInstance();
        cal2.setTimeInMillis(cal.getTimeInMillis());
        cal2.set(Calendar.HOUR_OF_DAY, 23);
        cal2.set(Calendar.MINUTE, 59);
        cal2.set(Calendar.SECOND, 59);

        cal = cal2;

        return cal;
    }

    public static Date XMLGregorianCalendarToDate(javax.xml.datatype.XMLGregorianCalendar xmlCal)
    {
        GregorianCalendar cal = xmlCal.toGregorianCalendar();
        return new Date(cal.getTimeInMillis());
    }

    /**
     * Creates an SHA-256 hash of the input text.
     * Returns a hexadecimal string representation of the hash value.
     *
     * @param text Input text for one-way encryption.
     * @return Hexadecimal representation of the SHA1 hash value.
     * @throws NoSuchAlgorithmException
     * @throws UnsupportedEncodingException
     */
    public static String SHA256(String text)
            throws NoSuchAlgorithmException, UnsupportedEncodingException
    {
        MessageDigest md;
        md = MessageDigest.getInstance("SHA-256");

        md.update(text.getBytes("iso-8859-1"), 0, text.length());

        byte[] sha1hash = md.digest();
        return convertToHex(sha1hash);
    }

    /**
     * Converts array of binary data into a hexadecimal string.
     *
     * @param data Binary data.
     * @return Hexadecimal string.
     */
    public static String convertToHex(byte[] data)
    {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < data.length; i++)
        {
            int halfbyte = (data[i] >>> 4) & 0x0F;
            int two_halfs = 0;

            do
            {
                if ((0 <= halfbyte) && (halfbyte <= 9))
                {
                    buf.append((char) ('0' + halfbyte));
                }
                else
                {
                    buf.append((char) ('a' + (halfbyte - 10)));
                }

                halfbyte = data[i] & 0x0F;
            }
            while (two_halfs++ < 1);
        }

        return buf.toString();
    }

    public static String chkValue(Object x, Object y) {
        return (x == y) ? "selected" : "";
    }

    public static javax.xml.datatype.XMLGregorianCalendar backDate()
    {
        XMLGregorianCalendar date2 = null;
        GregorianCalendar c = Utils.getCalendar(Utils.getDate(Utils.getRelativeDate(-200, 0, 0)));
        try {
            date2 = javax.xml.datatype.DatatypeFactory.newInstance().newXMLGregorianCalendar(c);
        } catch (DatatypeConfigurationException e) {
            logger.error(e.getMessage());
        }
        return date2;
    }

    public static boolean isAvailable(String s)
    {
        return s != null && s.length() > 0;
    }

    public static boolean isAvailable(Integer i)
    {
        return i != null;
    }

    public static boolean isAvailable(Double d)
    {
        return d != null && d > 0.;
    }

    public static String stripInvalidXMLCharacters(String in)
    {
        StringBuilder out = new StringBuilder(); // Used to hold the output.
        char current; // Used to reference the current character.

        if (in == null || ("".equals(in))) return ""; // vacancy test.
        int strLength = in.length();
        for (int i = 0; i < strLength; i++)
        {
            current = in.charAt(i); // NOTE: No IndexOutOfBoundsException caught here; it should not happen.
            if ((current == 0x9 ||
                    current == 0xA ||
                    current == 0xD) ||
                    ((current >= 0x20) && (current <= 0xD7FF)) ||
                    ((current >= 0xE000) && (current <= 0xFFFD)) ||
                    ((current >= 0x10000) && (current <= 0x10FFFF)))
            {
                out.append(current);
            }
        }
        return out.toString();
    }

    public static String generateEncryptedQuery(String params)
    {

        try {
            String encryptedParams = Base64.encodeBase64URLSafeString(params.getBytes("UTF8"));
            Crypto cy = new Crypto();
            encryptedParams = cy.encode(encryptedParams);
            return Base64.encodeBase64URLSafeString(encryptedParams.getBytes("UTF8"));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static Map decodeEncryptedQuery(String encryptedParams)
    {

        try {
            encryptedParams = new String(Base64.decodeBase64(encryptedParams), "UTF8");
            Crypto cy = new Crypto();
            String params = null;
            params = cy.decode(encryptedParams);
            params = new String(Base64.decodeBase64(params), "UTF8");

            return generateURLParameterMap(params);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static Map generateURLParameterMap(String params)
    {
        Map<String, String> paramMap = new LinkedHashMap<String, String>();

        String[] paramVals = params.split("&");
        for (String param : paramVals)
        {
            String[] vals = param.split("=");
            if (vals.length > 1)
            {
                String key = vals[0];
                String val = vals[1];
                paramMap.put(key, val);
            }
        }

        return paramMap;
    }

    public static String formatDate3(String dateX)
    {
        if (dateX != null && dateX.length() != 0)
        {
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd-MMM-yyyy");
                java.util.Date d = df.parse(dateX);
                df.applyPattern("MM/dd/yyyy");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return " ";
            }
        }
        return " ";
    }

    public static String formatDateYYYYMMDD(String dateX)
    {
        if (dateX != null && dateX.length() != 0)
        {
            try
            {
                java.text.SimpleDateFormat df = new java.text.SimpleDateFormat("dd-MMM-yyyy");
                java.util.Date d = df.parse(dateX);
                df.applyPattern("yyyy-MM-dd");
                return df.format(d);
            }
            catch (ParseException e)
            {
                return "";
            }
        }
        return "";
    }


    /**
     * Parses a foreign date using the given parameter [patternIn] and returns
     * the given date [dateIn] in a properly formatted string as given in
     * the parameter [patternOut]. The pattern API can be found at
     * http://download.oracle.com/javase/6/docs/api/java/text/SimpleDateFormat.html
     *
     * @param patternIn  the pattern used to parse the date string
     * @param patternOut the pattern to apply when returning the date string
     * @param dateIn     the date to parse
     * @return the properly formatted date string
     * @throws ParseException when the the date fails to parse correctly
     */
    public static String getDateStringFromPattern(String patternIn,
                                                  String patternOut, String dateIn) throws ParseException
    {
        SimpleDateFormat incomingFormatter = new SimpleDateFormat(patternIn);
        Date dateOut = incomingFormatter.parse(dateIn);
        SimpleDateFormat outgoingFormatter = new SimpleDateFormat(patternOut);
        return outgoingFormatter.format(dateOut);
    }

    /**
     * Parses a foreign date using the given pattern and string representation
     * of date and returns a Date object. The pattern API can be found at
     * http://download.oracle.com/javase/6/docs/api/java/text/SimpleDateFormat.html
     *
     * @param pattern the pattern used to parse the date string
     * @param date    the date to parse
     * @return a date object which represents the date string
     * @throws ParseException when the date fails to parse correctly
     */
    public static Date getDateFromPattern(String pattern, String date)
            throws ParseException
    {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
        return simpleDateFormat.parse(date);
    }

    /**
     * Fetches a String from a resource bundle.
     *
     * @param resourceBundle The name of the resource bundle (ex: 'VistaDataService')
     * @param resourceKey    The resource key value (Ex. 'SITE_URL')
     * @return A String resource
     */
    public static String getStringFromResourceBundle(String resourceBundle, String resourceKey)
    {
        ResourceBundle prop = ResourceBundle.getBundle(resourceBundle);
        return prop.getString(resourceKey);
    }

    /**
     * Parses a string in the form "{1, 2, 3, ..., 16}" which is a representation of 16 bytes in 
     * a byte array. 
     *
     * Will return a length 16 byte array that may be padded with zeros if there is an error during parsing
     * 
     * @param byteArrayString
     * @return a length 16 byte array
     */
    public static byte[] parseByteArray16(String byteArrayString) {
        // Assume format "{1, 2, 3, ..., 16}"
        byte[] rtn = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

        byteArrayString = byteArrayString.trim();
        if (byteArrayString != null && !isEmptyString(byteArrayString)) {

            if (byteArrayString.indexOf("{") != 0) {
                return rtn;
            }

            int originalLength = byteArrayString.length();

            if (byteArrayString.indexOf("}") != (originalLength - 1)) {
                return rtn;
            }

            byteArrayString = byteArrayString.substring(1, originalLength - 1);
            String[] bytesStringArray = byteArrayString.split(",");
            for (int i = 0; i < bytesStringArray.length && i < 16; i++) {
                try {
                    byte b = Byte.parseByte(bytesStringArray[i].trim());
                    rtn[i] = b;
                } catch (NumberFormatException nfe) {
                    // error so just return
                    return rtn;
                }
            }
        }
        
        return rtn;
    }

    public static boolean isJSONValid(String test) {
        try {
            new JSONObject(test);
        } catch (JSONException ex) {
            // e.g. in case JSONArray is valid as well...
            try {
                new JSONArray(test);
            } catch (JSONException ex1) {
                return false;
            }
        }
        return true;
    }
}