$.widget("ui.pager", {

    options: {
        pageCount: 1
    },
    _create: function () {

        this.element.addClass('ui-pager');
        this.refresh();

    },
    _createNavButtons: function () {

        var self = this;

        $('<div tabindex="0">')
            .addClass('ui-pager-btn left')
            .attr({
                'title': 'Previous Page',
                'aria-label': 'Previous Page'
            })
            .append('<div class="icon-chevron-left-blue">')
            .button()
            .on('click', function () {
                self.setSelectedPage(self._selectedPage - 1, true);
            })
            .prependTo(this.element);

        $('<div tabindex="0">')
            .addClass('ui-pager-btn right')
            .attr({
                'title': 'Next Page',
                'aria-label': 'Next Page'
            })
            .append('<div class="icon-chevron-right-blue">')
            .button()
            .on('click', function () {
                self.setSelectedPage(self._selectedPage + 1, true);
            })
            .appendTo(this.element);

    },
    _createPageButtons: function (start) {

        var count = Math.min(6, this.options.pageCount - start + 1),
            i = 0;

        for (; i < count; i++) {
            $('<button tabindex="0">').addClass('ui-pager-btn page-number')
                .attr({
                    'title': 'Page ' + (start + i)
                })
                .text(start + i)
                .button()
                .appendTo(this.element);
        }

        if (this._selectedPage) {
            this.element.find('.page-number').eq(this._selectedPage - this._start).addClass('selected');
        }

        var self = this;
        this.element.find(".page-number").on('click', function () {
            self.setSelectedPage($(this).index() + start - 1, true);
        });
    },
    setSelectedPage: function (pageIndex, fireEvent) {

        if (this._selectedPage !== pageIndex && pageIndex > 0 && pageIndex <= this.options.pageCount) {

            this.element.find('.page-number').removeClass('selected');

            this._selectedPage = pageIndex;

            if (this._selectedPage < this._start) {
                this._start = Math.max(1, this._selectedPage - 3);
                this.render();

            }
            else if (this._selectedPage >= this._start + 3) {

                this._start = this._selectedPage;
                var extra = 6 - (this.options.pageCount - this._selectedPage + 1);
                if (extra > 0) {
                    this._start = Math.max(1, this._selectedPage - extra);
                }

                this.render();
            }

            this.element.find('.page-number').eq(this._selectedPage - this._start).addClass('selected');

            if (fireEvent == true)
                this.element.trigger('selection', [pageIndex]);
        }

        this.element.find('.left').button('option', 'disabled', pageIndex === 1);
        this.element.find('.right').button('option', 'disabled', pageIndex === this.options.pageCount);

    },
    getSelectedPage: function () {
        return this._selectedPage;
    },
    render: function () {

        this.element.find('.ui-pager-btn').remove();
        this._createPageButtons(this._start);
        this._createNavButtons();
    },
    refresh: function () {

        this._start = 1;

        if (this.options.pageCount < 2) {
            this.element.hide();
        }
        else {
            this.element.css('display', 'inline-block');
            this.render();

        }
    },
    _setOption: function (key, value) {

        if (key === 'pageCount') {

            this._super(key, value);
            this.refresh();
            return;
        }

        this._super(key, value);
    }


});