//= require ./View.js
//= require ../jquery/datepicker.js
//= require ../jquery/maskedinput.js

AppKit.DatePicker = AppKit.Control.extend({
    classID: 'appkit-datepicker',
    options: {
        tabindex: -1,
        minDate: null,
        maxDate: null,
        date: null,
        format: 'mm/dd/yy'
    },
    initialize: function (options) {

        var self = this;
        this._input = $('<input type="text"></input>')
            .addClass('ui-datepicker-input')
            .appendTo(this.$el);


        this._input.datepicker(_.extend({
            showOn: 'button',
            yearRange: 'c-25:c+25',
            changeYear: true,
            changeMonth: true,
            onClose: function () {
                self._dpvis = false;
                self.$el.trigger('close');
            },
            onSelect: function () {
                self.$el.trigger('select');
                self.$el.trigger('dateSet');
            },
            beforeShow: function () {
                self._dpvis = true;
                self.$el.trigger('beforeShow');

            }
        }, options));

        this._trigger = $('<div>')
            .addClass('ui-datepicker-trigger')
            .html('<div class="icon-date-calendar"></div>')
            .attr({
                alt: "Show calendar button",
                title: "Show Calendar"
            })
            .button({text: false})
            .on('click', function () {

                if (self.get('enabled')) {
                    if (self._dpvis) {
                        self._input.datepicker('hide');
                        self._dpvis = false;
                    }
                    else {
                        self._input.datepicker('show');
                        self._dpvis = true;
                    }
                }
            })
            .appendTo(this.$el);

        this.input().bind('focus change keydown paste', function () {
            AppKit.Form.clearValidationWarnings();
        }).on('change keydown paste', function () {
            setTimeout(function () {
                self.$el.trigger("valueChanged", [self.get('value')]);
            }, 0);
        });

        this._super(options);
    },
    input: function () {
        return this.$el.find('input');
    },
    placeholder: function (aString) {
        if (aString !== undefined) {
            this._input.watermark(aString);
        }

        return this._input.watermark();
    },
    format: function (aString) {
        if (aString !== undefined) {

            var mask = aString.replace('yy', 'yyyy');
            var self = this;
            this._input.mask(mask.replace(/[A-Za-z]/g, '9'), {
                completed: function () {
                    self.set('date', self._input.val());
                }
            });

            this._input.datepicker('option', 'dateFormat', aString);

        }

        return this._input.datepicker('option', 'dateFormat');
    },
    date: function (aDate) {
        if (aDate !== undefined) {
            this._input.datepicker('setDate', aDate);
            this.$el.trigger('dateSet');
        }

        //validate the text
        if (AppKit.DatePicker.dateFromString(this._input.val().replace(/_/g, ''), this.get('format'))) {
            return this._input.datepicker('getDate');
        }

        return undefined;

    },
    enabled: function (aBool) {
        if (aBool == true) {
            this._input.datepicker('option', 'disabled', false);
            this.$el.attr('tabindex', 0);
        }
        else if (aBool == false) {
            this._input.datepicker('option', 'disabled', true);
            this.$el.attr('tabindex', -1);
        }


        return this._super(aBool);
    },
    maxDate: function (aDate) {
        if (aDate !== undefined) {
            this._input.datepicker("option", "maxDate", aDate);
        }

        return this._input.datepicker("option", "maxDate");
    },
    minDate: function (aDate) {
        if (aDate !== undefined) {
            this._input.datepicker("option", "minDate", aDate);
        }

        return this._input.datepicker("option", "minDate");
    },
    value: function (aString) {
        if (aString !== undefined) {
            this.input().val(aString);
        }

        return this.input().val();

    },
    clearValidationWarning: function () {
        if (this.input().hasClass('validation-error')) {
            this.input().removeClass('validation-error');
            this.input().tooltip('destroy');
            this.input().removeAttr('title');
        }
    },
    validate: function (aValidator) {
        if (!_.isFunction(aValidator))
            return true;

        var message = aValidator(this.get('value').replace(/_/g, ''));

        if (message) {
            if (this.input().hasClass('validation-error'))
                this.input().tooltip('destroy');

            this.input().tooltip({
                content: message,
                items: 'input',
                show: {
                    duration: 0,
                    easing: false,
                    effect: null
                },
                position: {
                    my: "left center",
                    at: "right+10 center",
                    collision: "flipfit",
                    using: function (position, feedback) {
                        $(this).css(position);
                        $("<div>")
                            .addClass("arrow")
                            .addClass(feedback.vertical)
                            .addClass(feedback.horizontal)
                            .appendTo(this);
                    }
                },
                tooltipClass: "right"
            });

            this.input().addClass('validation-error');
            this.input().tooltip('open');

            return false;
        }

        return true;
    }
});


AppKit.DatePicker.dateFromString = function (aString, format) {
    format = format.replace('yy', 'yyyy').toUpperCase();

    if (aString.length == format.length) {
        var m = moment(aString, format);
        if (m.isValid()) {
            return m.toDate();
        }

    }

    return undefined;
};