//= require ./Control.js
//= require ../jquery/tabs.js


AppKit.TabView = AppKit.View.extend({
    classID: 'appkit-tabview',
    initialize: function (options) {
        this._tabTemplate = _.template('<li id="{{tabID}}" class="ui-tab"><div data-panel-id="{{panelID}}"' +
            'class="ui-tab-label"><span class="label">{{label}}</span><div class="instructions" style="display:none;">Use the arrow keys to navigate tab view.</div></div><span title="Close" class="ui-tab-close" role="button">' +
        '<div class="icon-close-small"></div></span></li>');


        this.$ul = $('<ul>')
            .attr('id', _.uniqueId('t'));

        this.$el.prepend(this.$ul);

        this.$el.tabs(options);

        var self = this;
        $.each(this.$el.children('.appkit-view'), function (idx, panel) {
            var tabName = $(panel).data('title') ? $(panel).data('title') : 'Tab';
            var panelID = $(panel).attr('id') ? $(panel).attr('id') : _.uniqueId('panel');
            $(this).attr('id', panelID);
            self.addTab(tabName, panelID, $(panel).data('closable'));
        });

        if (this.get('tabCount') > 0) {
            this.selectTabAtIndex(0);
        }

        this._super(options);
    },
    addTab: function (tabName, panelID, closable) {

        var tabID = _.uniqueId("tab");

        var tab = $(this._tabTemplate({
            label: $.trim(tabName).length ? tabName : "&nbsp;",
            panelID: panelID,
            tabID: tabID
        }));

        tab.appendTo(this.$ul);
        
        if (closable) {
            var self = this;
            this.$el.find('#' + tabID).children('.ui-tab-close').on('click', function (evt) {
                evt.preventDefault();
                self.onCloseTab(tabID);
            });

            //use CTRL-X to close tab with keyboard
            this.$el.find('#' + tabID).on('keydown', function (evt) {
                if (evt.keyCode == 27) {
                    evt.preventDefault();
                    self.onCloseTab(tabID);
                }
            });
        }
        else {
            this.$el.find('#' + tabID).children('.ui-tab-close').hide();
        }

        if (this.$el.tabs('instance'))
            this.$el.tabs('refresh');

        this.$el.find('.instructions').text('This tab list has ' + this.get('tabCount') + 'tabs. Use the arrow keys to navigate tabs.')
    },
    activeTab: function (anIndex) {
        if (this.$el.tabs('instance')) {
            if (anIndex !== undefined) {
                this.selectTabAtIndex(anIndex);
            }

            return this.$el.tabs('option', 'active');
        }

        return -1;

    },
    selectTabAtIndex: function (anIndex) {
        if (this.$el.tabs('instance')) {
            this.$el.tabs('option', 'active', anIndex);
        }
    },
    onCloseTab: function (tabID) {
        this.removeTab(tabID);
    },
    removeTab: function (tabID) {
        var tab = this.$el.find('#' + tabID);

        if (tab) {
            var panelID = tab.children('.ui-tab-label').data('panel-id');
            $('#' + panelID).remove();
            tab.remove();
            this.$el.tabs('refresh');
            this.$el.trigger('tabremoved');
        }
    },
    removeTabAtIndex: function (anIndex, silent) {
        var tab = this.$el.children('.ui-tabs-nav').children('.ui-tab').eq(anIndex);
        if (tab) {
            var panelID = tab.children('.ui-tab-label').data('panel-id');
            $('#' + panelID).remove();
            tab.remove();
            this.$el.tabs('refresh');

            if (silent !== true)
                this.$el.trigger('tabremoved');
        }
    },
    removeAllTabs: function (silent) {
        var count = this.get('tabCount'),
            i = 0;

        for (; i < count; i++) {
            this.removeTabAtIndex(0, true);

        }

        if (silent !== true)
            this.$el.trigger('tabremoved');

    },
    tabLabelAtIndex: function (anIndex, aString) {
        if (anIndex > -1 && anIndex < this.get('tabCount')) {
            var tab = this.$el.children('.ui-tabs-nav').children('.ui-tab').eq(anIndex);
            if (tab) {
                var tabLabel = tab.find('.label');
                if (aString !== undefined) {
                    tabLabel.html(aString);
                }
                return tabLabel.text();
            }
        }
    },
    refresh: function () {
        if (this.$el.tabs('instance'))
            this.$el.tabs("refresh");
    },
    tabCount: function () {
        return this.$el.children('.ui-tabs-nav').children('.ui-tab').length;
    },
    hidden: function (aBOOL) {
        this._super(aBOOL);
        if (this.$el.tabs('instance'))
            this.$el.tabs('refresh');
    }
});