/**
 *
 * @param patientModel - patient backbone model
 * @param callback - a function execute when select is complete
 * @constructor
 */

JLV.SelectPatientFromPatientSearch = function (patientModel) {

    var sensitive = patientModel.get('sensitive');

    if (sensitive) {
        AppKit.AlertDialog.confirm('Sensitive Record',
            'This is a VA SENSITIVE Patient record. <br/> Do you wish to continue?',
            function (answer) {
                if (answer) {
                    JLV.SelectPatient(patientModel, callback);
                }
            }
        );
    }
    else {
        JLV.SelectPatient(patientModel);
    }


};

JLV.SelectPatient = function (patientModel) {

    var patientID;
    if ("ICN" === patientModel.get('lookupType')) {
        patientID = patientModel.get('icn');
    } else {
        patientID = patientModel.get('edipi');
    }

    JLV.Session.viewedFlags = false;

    //check to see if this patient already has a portal open -
    //if it does, select its screen
    if (AppKit.get('PatientPortal') !== undefined && JLV.ActivePatientModel) {
        if (JLV.ActivePatientModel.get('edipi') === patientID || JLV.ActivePatientModel.get('icn') === patientID) {
            JLV.Desktop.ScreenLaunchablesController.selectScreen('PatientPortal');
            return;
        }
    }
    $('#PatientSearchDialog').widget().close();

    // Close all browser tabs/windows that were opened during this patient for patient switch
    JLV.common.closeAuxBrowserWindows();

    //remove current patient screen
    JLV.Desktop.ScreenLaunchablesController.removeScreen('PatientPortal');

    var theme = 'blue';
    if (JLV.Session.ui_conf.options[0].theme !== undefined) {
        theme = JLV.Session.ui_conf.options[0].theme;
    }

    if (theme === 'default') {
        theme = "blue";
    }

    $('<div>')
        .data('id', 'PatientPortal')
        .addClass('appkit-view patient-select-screen')
        .html('<div class="jlv-loading" aria-label="JLV is loading patient data."><div class="jlv-loadingMessage">' +
            String.format('<img src="assets/{0}/patientSelectLoading.gif" alt="selecting patient loading indicator"/>', theme) +
        '<br/><br/>Selecting patient, please wait...</div></div>')
        .insertBefore($('#WindowDock'))
        .awaken();

    var isCloseable = true;
    if (JLV.common.AHLTA_LOGIN_FLAG) {
        isCloseable = false;
    }

    JLV.Desktop.ScreenLaunchablesController.addScreen({
        closable: isCloseable,
        setActive: true,
        screenID: 'PatientPortal',
        title: 'Patient Portal',
        ariaLabel: 'Patient Portal to activate press spacebar, to close press escape',
        content: '<div class="icon-user3" style="float:left;font-size:20px;margin:10px 6px 0 0;"></div><label style="text-align:left;">Patient Portal&nbsp;</label>'
    });

    // Coming from context management, edipi will be the patient token value. We need to blank it out so that
    // It doesn't get added to the recently viewed patients table because the edipi is not a valid edipi.
    if (isNaN(patientModel.get('edipi'))) {
        patientModel.set('edipi', '');
    }

    JLV.ActivePatientModel = patientModel;

    JLV.ReportBuilderController.clearRecords();

    //select patient
    $.post('patientSelect/select.json', {
        patToken: patientModel.get("patientToken"),
        cmNotify: patientModel.get("cmNotify"),
        patName: patientModel.get("name"),
        patEdipi: patientModel.get("edipi"),
        patSsn: patientModel.get("fmpssn"),
        patSponsorSsn: patientModel.get("sponsorSsn"),
        patGender: patientModel.get("gender"),
        patDob: patientModel.get("dob"),
        patSensitive: patientModel.get("sensitive"),
        patIcn: patientModel.get("icn"),
        csrfToken: $('meta[name="csrf-token"]').attr('content')
    }, function (response) {

        JLV.Session.dodVLERReady = false;
        JLV.Session.dasVLERReady = false;
        JLV.Session.vlerData = [];
        JLV.Session.dasvlerData = [];

        patientModel.set("hieSharingOptIn", response.hieSharingOptIn);
        patientModel.set("hieSharingOptInLastUpdateDate", response.hieSharingOptInLastUpdateDate);

        if (AppKit.get('PatientPortal')) {
            JLV.ActivePatientModel = patientModel;
            var resultScript = document.createElement('script');
            JLV.CheckMVIServiceError(response, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'),
                JLV.ActivePatientModel.get('fmpssn'), JLV.ActivePatientModel.get('edipi'), resultScript);
        }
    }).fail(function (response) {
        AppKit.AlertDialog.alert('Patient Select Error (' + response.status + ')', "An error occurred while trying to select this patient. Please try again.", function () {
            JLV.PatientSelectCancelCleanup(patientID);
        });
    });

};


JLV.CreateActivePatientPortal = function () {

    if (JLV.ActivePatientModel) {

        JLV.PatientDemographics.Fetch(JLV.ActivePatientModel, function (demographicsData) {

            if (AppKit.get('PatientPortal')) {

                JLV.ActivePatientModel.set("demographics", demographicsData);
                AppKit.get('PatientPortal').$el.remove();

                // create a patient portal from a div fragment
                $('<div>').data("id", 'PatientPortal')
                    .addClass('jlv-patient-portal jlv-tabview')
                    .insertBefore($('#WindowDock')).awaken('jlv-patient-portal');

                //get the patient portal and set the first tab to have patient name

                var patientPortal = AppKit.get('PatientPortal');
                if (patientPortal) {
                    patientPortal.tabLabelAtIndex(0, JLV.ActivePatientModel.get("name") + ' <div style="display: inline-block; vertical-align:middle;" class="alert-btn"></div>');
                    JLV.Session.loadPatientUIConfig(patientPortal, JLV.ActivePatientModel);
                    patientPortal.displayActivePatient();
                }

                JLV.Desktop.ScreenLaunchablesController.selectScreen('PatientPortal');

                JLV.LoadVlerData();

            }
            else {
                JLV.ActivePatientModel = null;
            }
        });

    }
};


JLV.CheckMVIServiceError = function (jsonData, patientToken, patName, patNum, patEdipi, resultScript) {
    //skip MVI error and BtG checks if sensitive record 4.
    if (jsonData.sensitivePatientAlertLevel &&
        jsonData.sensitivePatientAlertLevel == "4") //user has no SSN on file, 4
    {
        AppKit.AlertDialog.alert('Access Denied',
            ('You do not have authorization to view this record.<br/><br/>' + jsonData.sensitivePatientAlertText).replace(/\n/g, "<br/>"), function () {
                JLV.PatientSelectCancelCleanup(patientToken);
            }, {height: 320});
    }
    // If no error or if error but message is empty string (when it's a VA user,
    // they don't need to see message), skip error message.
    else if (jsonData.isSuccess || ((jsonData.errorMsg != null) && (jsonData.errorMsg.length == 0))) {
        // Check if VA user is accessing DoD-only patient
        JLV.CheckVAUserDoDOnlyPatientRestriction(jsonData, patientToken, patName, patNum, patEdipi, resultScript);
    }
    else {
        //MVI error
        if (jsonData.sensitivePatientAlertLevel &&
            jsonData.sensitivePatientAlertLevel == "3") //viewing own medical record, level 3 (note: determined in JLV layer, not from VISTA host DG Sensitive alert)
        {
            AppKit.AlertDialog.alert('Access Denied',
                ('You do not have authorization to view this record.<br/><br/>' + jsonData.sensitivePatientAlertText).replace(/\n/g, "<br/>"), function () {
                    JLV.PatientSelectCancelCleanup(patientToken);
                }, {height: 320});

        }
        else {
            JLV.ActivePatientModel.set("mviError", jsonData.errorMsg);
        // Alert user of MVI error
        // AppKit.AlertDialog.alert('Warning', "MVI ERROR MESSAGE: " + jsonData.errorMsg, function () {
            JLV.CheckVAUserDoDOnlyPatientRestriction(jsonData, patientToken, patName, patNum, patEdipi, resultScript);
        // });
	    }
    }
};


JLV.CheckVAUserDoDOnlyPatientRestriction = function (jsonData, patientToken, patName, patNum, patEdipi, resultScript) {

    if (jsonData.isVARestricted) {
        // VA user is trying to access patient with only DoD records. Determine VA user type and display
        // proper BtG dialog message
        if (jsonData.providerSiteMoniker.toUpperCase() == "VBA") {
            // VBA user is trying to access patient with only DoD records

            JLV.BreakTheGlass.VBADeniedController.open(function () {
                JLV.PatientSelectCancelCleanup(patientToken);
            });

        }
        else {
            // VHA user is trying to access patient with only DoD records
            JLV.ActivePatientModel.set("mviError", "VHA User trying to access a patient with only DoD records");
            JLV.BreakTheGlass.VHAController.open(jsonData, function (agree) {

                if (agree) {
                    var auditTxt = $("#btgWindowVHA .btgOrgTxt").text() + "^" +
                        $("#btgWindowVHA .btgProvNameTxt").text() + "^" +
                        patNum + "^" + patEdipi + "^" +
                        $("#btgWindowVHA .btgDateTxt").text() + "^" +
                        $("#btgWindowVHA .btgIPTxt").text() + "^" +
                        $('input[name="btgAuthPurposeRadioGrp"]:checked').data('label');

                    // Audit VA user's confirmation to continue to view DOD-only patient record
                    // Check if patient is a VA sensitive patient - callback CheckVASensitivePatient
                    $.post('patientSelect/auditVARestrictedAccess', {patToken: patientToken, auditTxt: auditTxt},
                        function () {
                            JLV.CheckVASensitivePatient(jsonData, patientToken, patName, resultScript);
                        }
                    );

                }
                else {
                    // Clean up and cancel patient selection
                    JLV.PatientSelectCancelCleanup(patientToken);

                }

            });


        }
    }
    else {
        // BtG feature is either turned off, user is DoD, or user is VA and patient is registered in MVI.
        if (jsonData.providerSite && (jsonData.providerSiteMoniker.toUpperCase() == "VBA")) {
            // VBA users will still need to agree to BtG regardless if patient is registered in MVI

            JLV.BreakTheGlass.VBAAccessController.open(jsonData, function (agree) {
                if (agree) {
                    var auditTxt = $("#btgWindowVBAAccess .btgOrgTxt").text() + "^" +
                        $("#btgWindowVBAAccess .btgProvNameTxt").text() + "^" +
                        patNum + "^" + patEdipi + "^" +
                        $("#btgWindowVBAAccess .btgDateTxt").text() + "^" +
                        $("#btgWindowVBAAccess .btgIPTxt").text();

                    // Audit VA user's confirmation to continue to view DOD-only patient record
                    $.post('patientSelect/auditVARestrictedAccess',
                        {patToken: patientToken, auditTxt: auditTxt},
                        function () {
                            JLV.CheckVASensitivePatient(jsonData, patientToken, patName, resultScript);
                        }
                    );
                }
                else {
                    JLV.PatientSelectCancelCleanup(patientToken);
                }
            });
        }
        else {
            // DoD or VA VHA users can continue and check if patient is VA sensitive patient
            JLV.CheckVASensitivePatient(jsonData, patientToken, patName, resultScript);
        }
    }
};

JLV.CheckVASensitivePatient = function (jsonData, patientToken, patName, resultScript) {
    var continueSelect;

    if (jsonData.isSensitive) {

        if (jsonData.sensitivePatientAlertLevel == "1" || jsonData.sensitivePatientAlertLevel == "2")
            AppKit.AlertDialog.confirm('Sensitive Record',
                ('This is a VA SENSITIVE Patient record. <br/> Do you wish to continue?<br><br>' + jsonData.sensitivePatientAlertText).replace(/\n/g, "<br>"),
                function (answer) {
                    if (answer) {
                        continueSelect = JLV.Context.setContext(jsonData.patientEdipi, jsonData.patientIcn,
                            jsonData.patientVaIen, jsonData.patientVaIenSiteId, patName);

                        if (continueSelect) {
                            // Clean up and select patient
                            JLV.PatientSelectCleanup(jsonData, patientToken, patName,
                                resultScript)
                        }
                        else {
                            // Clean up and cancel patient selection
                            JLV.PatientSelectCancelCleanup(patientToken);
                        }
                    }
                    else {
                        // Clean up and cancel patient selection
                        JLV.PatientSelectCancelCleanup(patientToken);
                    }
                }, {height: 320});
        else //3 accessing own record, 4 no SSN
            AppKit.AlertDialog.alert('Access Denied',
                ('You do not have authorization to view this record.<br/><br/>' + jsonData.sensitivePatientAlertText).replace(/\n/g, "<br/>"), function () {
                    JLV.PatientSelectCancelCleanup(patientToken);
                }, {height: 320});
    }
    else {
        //the case where getVASensitivePatientAlerts fails, or VDS is unavailable.
        if (jsonData.sensitivePatientAlertLevel == "-1") {

            AppKit.AlertDialog.alert('Patient Select Error Detected',
                (jsonData.sensitivePatientAlertText).replace(/\n/g, "<br/>"));
        }
        continueSelect = JLV.Context.setContext(jsonData.patientEdipi, jsonData.patientIcn,
            jsonData.patientVaIen, jsonData.patientVaIenSiteId, patName);

        if (continueSelect) {
            // Clean up and select patient
            JLV.PatientSelectCleanup(jsonData, patientToken, patName,
                resultScript)
        }
        else {
            // Clean up and cancel patient selection
            JLV.PatientSelectCancelCleanup(patientToken);
        }
    }
};


JLV.PatientSelectCleanup = function (jsonData, patientToken, patName, resultScript) {
    if (patientToken) {
        // Perform asynchronous call to selectPatient (VDS) for VA patients
        $.ajax({
            url: 'patientSelect/selectPatientForVASensitive',
            type: 'POST',
            data: {
                patToken: patientToken
            }
        });
    }

    if (jsonData.resultScript) {
        try {
            // doesn't work on ie...
            resultScript.appendChild(document.createTextNode(jsonData.resultScript));
        } catch (e) {
            // IE has funky script nodes
            resultScript.text = jsonData.resultScript;
        }
    }

    // Adding a short delay because the selectPatientForVASensitive AJAX call above does a selectedPatientMap.put()
    // while CreateActivePatientPortal below triggers SecurityFilters to do a selectPatientMap.get() on the selected
    // patient that should've just been put into the map. Because the AJAX call is asynchronous, there is a possiblity
    // that selectPatientMap.get() is called before selectPatientMap.put(), resulting in a null selectedPatient object
    // being returned from the get call. -Jimi (JLVENT-624)
    setTimeout(function() {
        JLV.CreateActivePatientPortal();
    }, 500);

    document.body.appendChild(resultScript);
};

JLV.PatientSelectCancelCleanup = function (patientToken) {
    // Clean up and cancel patient selection
    JLV.Desktop.ScreenLaunchablesController.removeScreen('PatientPortal');

    JLV.Context.cancelVaultChange();
};

AppKit.launch(function () {
    if (JLV.common.AHLTA_LOGIN_FLAG) {
        // Prevent logout when coming from AHLTA link
        $('#LogoutButton').hide();
        $('#PatientSearchToolbarButton').hide();

        JLV.SelectPatient(new Backbone.Model({
                patientToken: JLV.common.AHLTA_LOGIN_PATIENT.patientToken,
                cmNotify: true,
                name: JLV.common.AHLTA_LOGIN_PATIENT.name,
                edipi: JLV.common.AHLTA_LOGIN_PATIENT.edipi,
                fmpssn: JLV.common.AHLTA_LOGIN_PATIENT.fmpssn,
                sponsorSsn: '',
                gender: JLV.common.AHLTA_LOGIN_PATIENT.gender,
                dob: JLV.common.AHLTA_LOGIN_PATIENT.dob,
                sensitive: JLV.common.AHLTA_LOGIN_PATIENT.sensitive
            })
        );
    }
});
