/**
 * controller for restricted user dialog
 **/

JLV.RestrictedUser = {};

JLV.RestrictedUser.Controller = {

    init: function () {
        this.dialog.set('height', 520);
        $('#RestrictedUserTabView').height(475);
    },

    onDialogOpen: function (aDialog) {
        aDialog.center();

        if (this.tabView) {
            this.tabView.set('activeTab', 1);
        }

        /* update recently viewed */
        $('#RURecentlyViewedPatientTable').widget().showMessage('<div style="width:115px;margin:auto;"><div class="jlv-table-data-loading" style="float:left;width:22px;"></div><label style="display:inline-block;">Loading...</label></div>')
        $('#RestrictedUserTable').widget().showMessage('<div style="width:115px;margin:auto;"><div class="jlv-table-data-loading" style="float:left;width:22px;"></div><label style="display:inline-block;">Loading...</label></div>')

        //select patient search tab
        if (this.tabView) {
            this.tabView.set('activeTab', 1);
        }

        var self = this;
        
        JLV.RestrictedUser.RUPatients.fetch({
            //data: data,
            //type: 'POST',
            reset: true,
            success: function (collection) {
                var i = 0;

                if (collection.length === 0) {
                    return self.RUPatientsTable.showMessage('No Results');
                }
                else {
                    self.RUPatientsTable.render();
                }
            }
            ,
            error: function (model, response) {
                console.error(response.responseText);
                AppKit.AlertDialog.alert('Patient Search Error',
                    'An error occurred while trying to retrieve your list of patients. Please try again.',
                    function () {
                        callback(model);
                    }
                );
            }
        });

        JLV.RestrictedUser.RecentlyViewed.fetch({
            reset: true,
            success: function (collection) {
                if (collection.length === 0) {
                    return self.RURecentlyViewedTable.showMessage('No Recently Viewed Patients');
                }
                else {
                    self.RURecentlyViewedTable.render();
                }

            },
            error: function () {
                self.RURecentlyViewedTable.showMessage('No Recently Viewed Patients');
            }
        });

    },

    onDialogClose: function () {
        //clear the collection of patients
        JLV.RestrictedUser.RUPatients.reset();
    },

    // patient name column formatter in patient list table
    patientNameFormatter: function (value, model, index) {
        if (model.get('ICN'))
            return _.template('<a href="#" class="patient-name-search" data-index="{{index}}" data-patient-id="{{ICN}}" data-patient-name="{{name}}" data-patient-lookuptype="{{lookupType}}">{{name}}</a>')(
                _.extend({index: index}, model.toJSON())
            );
        else
            return _.template('<span data-index="{{index}}" >{{name}}</span>')(
                _.extend({index: index}, model.toJSON())
            );
    },

    // patient name column formatter in recently viewed table
    patientNameFormatterRV: function (value, model, index) {
        if (model.get('ICN'))
            return _.template('<a href="#" class="patient-name-search-rv" data-index="{{index}}" data-patient-id="{{ICN}}" data-patient-name="{{name}}" data-patient-lookuptype="{{lookupType}}">{{name}}</a>')(
                _.extend({index: index}, model.toJSON())
            );
        else
            return _.template('<span data-index="{{index}}" >{{name}}</span>')(
                _.extend({index: index}, model.toJSON())
            );
    },
};

/**
 *  Collection that stores patient list
 */
JLV.RestrictedUser.PatientCollection = Backbone.Collection.extend({
        model: Backbone.Model.extend({
            parse: function (data) {
                if (data.name) {
                    data.name = data.name.replace(/,/g, ', ');
                }

                return data;
            }

        })
    }
);

JLV.RestrictedUser.RUPatientCollection = JLV.RestrictedUser.PatientCollection.extend({
    url: 'patientSearch/getRestrictedUserPatients.json'
});
JLV.RestrictedUser.RUPatients = new JLV.RestrictedUser.RUPatientCollection();

/**
 *  Collection that stores recently viewed patients
 */

JLV.RestrictedUser.RecentlyViewedPatientCollection = JLV.RestrictedUser.PatientCollection.extend({
    url: 'patientSearch/getRecentlyViewedPatients.json'
});

JLV.RestrictedUser.RecentlyViewed = new JLV.RestrictedUser.RecentlyViewedPatientCollection();


JLV.ActivePatientModel = null;

AppKit.launch(function () {
    JLV.RestrictedUser.Controller.init();
});

AppKit.launch(function () {
    var restrictedUsersTable = $('#RestrictedUserTable').widget();
    var recentPatientsTable = $('#RURecentlyViewedPatientTable').widget();


    /**
     * after rendering the restricted users table, add patient select handlers
     **/
    restrictedUsersTable.on('afterRender', function () {
        $('.patient-name-search').on('click', function () {

                var patientID = "" + $(this).data('patient-id');
                var patientName = "" + $(this).data('patient-name');
            var lookupType = "" + $(this).data('patient-lookuptype');

                if (patientID) {
                    var data = {
                        "edipi": "",
                        "icn": "",
                        "vaIen": "",
                        "vaIenSiteId": "",
                        "csrfToken": $('meta[name="csrf-token"]').attr('content')
                    };
                    if ("ICN" === lookupType) {
                        data.icn = patientID;
                    } else {
                        data.vaIen = patientID;
                    }

                    $.ajax({
                        url: "context/getPatientToken",
                        type: "POST",
                        data: data,
                        success: function (data) {
                            JLV.SelectPatient(new Backbone.Model({
                                patientToken: data.patientToken,
                                cmNotify: false,
                                name: patientName,
                                edipi: data.patientToken,
                                fmpssn: '',
                                sponsorSsn: '',
                                gender: '',
                                dob: '',
                                sensitive: false
                            }));
                            $('#RestrictedUserDialog').widget().close();
                        },
                        error: function () {
                            AppKit.AlertDialog.alert('Patient Load Error',
                                'An error occurred while trying to retrieve your patient token via ICN. ',
                                function () {
                                }
                            );
                        }
                    });

                }

            }
        );
    });

    /**
     * after rendering the recently view patients table, add patient select handlers
     **/
    recentPatientsTable.on('afterRender', function () {
        $('.patient-name-search-rv').on('click', function () {

            var patientID = "" + $(this).data('patient-id');
            var patientName = "" + $(this).data('patient-name');
            var lookupType = "" + $(this).data('patient-lookuptype');

            if (patientID) {
                var data = {
                    "edipi": "",
                    "icn": "",
                    "vaIen": "",
                    "vaIenSiteId": "",
                    "csrfToken": $('meta[name="csrf-token"]').attr('content')
                };
                if ("ICN" === lookupType) {
                    data.icn = patientID;
                } else {
                    data.vaIen = patientID;
                }

                $.ajax({
                    url: "context/getPatientToken",
                    type: "POST",
                    data: data,
                    success: function (data) {
                        JLV.SelectPatient(new Backbone.Model({
                            patientToken: data.patientToken,
                            cmNotify: false,
                            name: patientName,
                            edipi: data.patientToken,
                            fmpssn: '',
                            sponsorSsn: '',
                            gender: '',
                            dob: '',
                            sensitive: false
                        }));
                        $('#RestrictedUserDialog').widget().close();
                    },
                    error: function () {
                        AppKit.AlertDialog.alert('Patient Load Error',
                            'An error occurred while trying to retrieve your patient token via ICN. ',
                            function () {
                            }
                        );
                    }
                });

            }

        });
    });
});

