//= require ./JLV.js

JLV.Session = {};

JLV.Session.COOKIE_EXPIRATION = new Date(new Date().getTime() + (1000 * 60 * 60 * 24 * 365));
JLV.Session.suspendSaveConfig = false;

JLV.Session.saveUIConfigForPortal = function (/* type : JLV.Desktop.TabView */aPortal) {

    if (JLV.Session.suspendSaveConfig)
        return;

    if (!aPortal)
        return;

    var cfg = null;

    if (aPortal instanceof JLV.Desktop.ProviderPortal || aPortal instanceof JLV.Desktop.PatientPortal) {
        cfg = {}
        cfg.tabs = [];
    } else {
        return;
    }

    var count = aPortal.get('tabCount'),
        i = 0;

    for (; i < count; i++) {
        var theTab = /*type: JLV.Desktop.WidgetPortal */ aPortal.portalAtIndex(i);
        var cols = [];

        //get the widgets
        var j = 0;
        for (; j < 3; j++) {
            var widgetObjects = theTab.widgetsInColumn(j),
                wcount = widgetObjects.length,
                k = 0;
            var widgets = [];
            for (; k < wcount; k++) {
                var widget = widgetObjects[k];

                if (widget) {
                    widgets.push({
                        widgetType: widget.get('widgetId'),
                        height: widget.get('height'),
                        defaultSort: widget.get('defaultSort'),
                        startDay: widget.get('startDay'),
                        endDay: widget.get('endDay'),
                        hiddenColumnsMinimized: widget.get('hiddenColumnsMinimized'),
                        hiddenColumnsExpanded: widget.get('hiddenColumnsExpanded')
                    });
                }
            }

            cols.push({
                widgets: widgets
            });

        }

        cfg.tabs.push({
            label: aPortal.tabLabelAtIndex(i),
            cols: cols
        });
    }

    var options = [];

    if(JLV.Session.ui_conf.options[0].theme){
        options.push({theme: JLV.Session.ui_conf.options[0].theme,
        menu: JLV.Session.ui_conf.options[0].menu});
    } else {
        options.push({theme: "default",
        menu: JLV.Session.ui_conf.options[0].menu});
    }

    options.push({pat_insurance_sort_info: JLV.PatientDemographics.InsuraceSortInfo});
    options.push({pat_insurance_hidden_columns: JLV.PatientDemographics.HiddenColumns});
    options.push({prov_ref: JLV.Session.prov_ref});
    options.push({loginAgency: JLV.Session.loginAgency});
    options.push({loginSite: JLV.Session.loginSite});
    options.push({dodVLER: JLV.Session.dodVLER});
    options.push({dasVLER: JLV.Session.dasVLER});

    //delete any instance of UI_CONF in the JSON object
    for (var i = cfg.length - 1; i >= 0; i--) {
        if (cfg[i].cfgItem == "UI_CONF") {
            cfg.splice(i, 1);
        }
    }

    if (cfg.tabs != null && cfg.tabs.length > 0) {

        if (aPortal instanceof JLV.Desktop.ProviderPortal) {
            cfg.cfgItem = "PORTAL_PROVIDER";
            JLV.Session.prov_view_cfg = cfg;
        } else if (aPortal instanceof JLV.Desktop.PatientPortal) {
            cfg.cfgItem = "PORTAL_PATIENT"
            JLV.Session.pat_view_cfg = cfg;
        }

        //add new config to the JSON object
        JLV.Session.cfg = [];
        JLV.Session.cfg.push({"cfgItem": "UI_CONF", "options": options});
        JLV.Session.cfg.push(JLV.Session.prov_view_cfg);
        JLV.Session.cfg.push(JLV.Session.pat_view_cfg);

        $.post('userCfg/save.json', {cfg: JSON.stringify(JLV.Session.cfg)}, function () {
            console.info("config was successfully saved");
        }).fail(function (response) {
            AppKit.AlertDialog.alert('Warning', "An error occurred trying to save your configuration - " + response.responseText);
        });
    } else {
        AppKit.AlertDialog.alert('Warning', "We've detected an error in your patient portal configuration. If you continue to see this message repeatedly, please close your browser window and try again.");
    }
};


JLV.Session.loadProviderUIConfig = function () {
    //load the provider view widgets

    if (JLV.Session.prov_view_cfg) {
        var providerPortal = AppKit.get('ProviderPortal');
        if (!providerPortal)
            return;

        var tabs = JLV.Session.prov_view_cfg.tabs,
            count = tabs.length,
            i = 0;

        for (; i < count; i++) {
            var tab = tabs[i];

            if (i === 0) {
                var theTab = providerPortal.portalAtIndex(i);
                var columns = tab.cols,
                    colCount = columns.length,
                    j = 0;

                for (; j < colCount; j++) {
                    var column = columns[j],
                        widgets = column.widgets;

                    var widgetCount = widgets.length,
                        k = 0;

                    for (; k < widgetCount; k++) {
                        var widgetInfo = widgets[k],
                            name = widgetInfo.widgetType,
                            defaultSort = widgetInfo.defaultSort,
                            height = widgetInfo.height,
                            startDay = widgetInfo.startDay,
                            endDay = widgetInfo.endDay;


                        if (JLV.isRegisteredWidget(name)) {
                            var theWidget = JLV.createJLVWidget(name, {
                                height: height,
                                startDay: startDay,
                                endDay: endDay,
                                defaultSort: defaultSort,
                                portal: theTab
                            });
                            theTab.addWidget(theWidget, j);
                        }
                    }
                }
                theTab.load();
            }
            else {
                var theTab = providerPortal.addTabbedPortal(tab.label, true);
                var columns = tab.cols,
                    colCount = columns.length,
                    j = 0;

                for (; j < colCount; j++) {
                    var column = columns[j],
                        widgets = column.widgets;

                    var widgetCount = widgets.length,
                        k = 0;

                    for (; k < widgetCount; k++) {

                        var widgetInfo = widgets[k],
                            name = widgetInfo.widgetType,
                            height = widgetInfo.height,
                            defaultSort = widgetInfo.defaultSort,
                            startDay = widgetInfo.startDay,
                            endDay = widgetInfo.endDay;


                        if (JLV.isRegisteredWidget(name)) {
                            var theWidget = JLV.createJLVWidget(name,
                                {
                                    height: height,
                                    defaultSort: defaultSort,
                                    startDay: startDay,
                                    endDay: endDay,
                                    portal: theTab
                                });
                            theTab.addWidget(theWidget, j);
                        }
                    }
                }
            }
        }

        providerPortal.selectTabAtIndex(0);
    }
};


JLV.Session.loadPatientUIConfig = function (patientPortal, patient) {
    //load the patient view widgets
    if (JLV.Session.pat_view_cfg) {
        var tabs = JLV.Session.pat_view_cfg.tabs,
            count = tabs.length,
            i = 0;

        for (; i < count; i++) {
            var tab = tabs[i];

            if (i === 0) {
                var theTab = patientPortal.portalAtIndex(i);

                var columns = tab.cols,
                    colCount = columns.length,
                    j = 0;

                for (; j < colCount; j++) {
                    var column = columns[j],
                        widgets = column.widgets;

                    var widgetCount = widgets.length,
                        k = 0;

                    for (; k < widgetCount; k++) {
                        var widgetInfo = widgets[k],
                            name = widgetInfo.widgetType,
                            defaultSort = widgetInfo.defaultSort,
                            height = widgetInfo.height,
                            startDay = widgetInfo.startDay,
                            endDay = widgetInfo.endDay,
                            hiddenColumnsMinimized = widgetInfo.hiddenColumnsMinimized || [],
                            hiddenColumnsExpanded = widgetInfo.hiddenColumnsExpanded || [];

                        if (JLV.isRegisteredWidget(name)) {
                            var theWidget = JLV.createJLVWidget(name, {
                                height: height,
                                startDay: startDay,
                                endDay: endDay,
                                defaultSort: defaultSort,
                                portal: theTab,
                                patient: patient,
                                hiddenColumnsMinimized: hiddenColumnsMinimized,
                                hiddenColumnsExpanded: hiddenColumnsExpanded

                            });
                            theTab.addWidget(theWidget, j);
                        }
                    }
                }
            }
            else {
                var theTab = patientPortal.addTabbedPortal(tab.label, true);
                var columns = tab.cols,
                    colCount = columns.length,
                    j = 0;

                for (; j < colCount; j++) {
                    var column = columns[j],
                        widgets = column.widgets;

                    var widgetCount = widgets.length,
                        k = 0;

                    for (; k < widgetCount; k++) {

                        var widgetInfo = widgets[k],
                            name = widgetInfo.widgetType,
                            height = widgetInfo.height,
                            defaultSort = widgetInfo.defaultSort,
                            startDay = widgetInfo.startDay,
                            endDay = widgetInfo.endDay,
                            hiddenColumnsMinimized = widgetInfo.hiddenColumnsMinimized || [],
                            hiddenColumnsExpanded = widgetInfo.hiddenColumnsExpanded || [];


                        if (JLV.isRegisteredWidget(name)) {

                            var theWidget = JLV.createJLVWidget(name, {
                                height: height,
                                startDay: startDay,
                                endDay: endDay,
                                portal: theTab,
                                patient: patient,
                                defaultSort: defaultSort,
                                hiddenColumnsMinimized: hiddenColumnsMinimized,
                                hiddenColumnsExpanded: hiddenColumnsExpanded
                            });

                            theTab.addWidget(theWidget, j);
                        }
                    }
                }
            }

            patientPortal.selectTabAtIndex(0);
        }
    }
};

JLV.VLERDataAvailable = function () {
    //place this where data is loaded
    //JLV.Session.vlerData is setup through PatientDODVLERController
    if (JLV.Session != null && ((JLV.Session.vlerData != null) || (JLV.Session.dasvlerData != null))) {
        $('.jlv-widget').each(function (index, element) {
            if (JLV.Session.vlerData.indexOf("MEDICATIONS") >= 0
                && $(element).widget()._widgetId.indexOf("medications") >= 0) {
                $(element).widget().vlerLoadMessage(JLV.common.DOD_VLER_DATA_IN_WIDGET_MSG);
            }
            if (JLV.Session.vlerData.indexOf("ENCOUNTERS") >= 0
                && (($(element).widget()._widgetId.indexOf("encounters") >= 0)
                || ($(element).widget()._widgetId.indexOf("admissions") >= 0) )) {
                $(element).widget().vlerLoadMessage(JLV.common.DOD_VLER_DATA_IN_WIDGET_MSG);
            }
            if (JLV.Session.vlerData.indexOf("RESULTS") >= 0
                && (($(element).widget()._widgetId.indexOf("radiology") >= 0)
                || ($(element).widget()._widgetId.indexOf("labs_results") >= 0) )) {
                $(element).widget().vlerLoadMessage(JLV.common.DOD_VLER_DATA_IN_WIDGET_MSG);
            }

        });
    }
}

jQuery(function () {

    //JLV.Session.cfg = [];
    if (!JLV.Session.cfg)
        JLV.Session.cfg = [];

    JLV.Session.prov_view_cfg = "";
    JLV.Session.pat_view_cfg = "";
    JLV.Session.ui_conf = "";
    JLV.Session.dodVLER = false;
    JLV.Session.dasVLER = false;


    $.each(JLV.Session.cfg, function (index, conf) {
        if (conf.cfgItem === "PORTAL_PROVIDER") {
            JLV.Session.prov_view_cfg = conf;
        } else if (conf.cfgItem === "PORTAL_PATIENT") {
            JLV.Session.pat_view_cfg = conf;
        } else if (conf.cfgItem === "UI_CONF") {
            JLV.Session.ui_conf = conf;
        }
    });

    if (JLV.Session.prov_view_cfg === null || JLV.Session.prov_view_cfg === "") {
        JLV.Session.prov_view_cfg = {
            tabs: [
                {
                    "label": "", "cols": [
                    {
                        "widgets": [
                            {"widgetType": "prov_admissions_widget", "height": 300},
                            {"widgetType": "prov_orderspending_widget", "height": 300}
                        ]
                    }
                    ,
                    {
                        "widgets": [
                            {"widgetType": "prov_appointments_widget", "height": 300}
                        ]
                    }
                    ,
                    {"widgets": []}
                ]
                },
                {
                    "label": "My Custom Tab", "cols": [
                    {
                        "widgets": [
                            {"widgetType": "prov_consultsrequested_widget", "height": 300}
                        ]
                    }
                    ,
                    {"widgets": []}
                    ,
                    {"widgets": []}
                ]
                }
            ]
        };
    }

    if (JLV.Session.pat_view_cfg === null || JLV.Session.pat_view_cfg === "") {
	    JLV.Session.pat_view_cfg = JLV.common.defaultPatWidgetLayout;
    }


    //check to see if there is a UI configuration loaded.
    if (JLV.Session.ui_conf === null || JLV.Session.ui_conf === "") {
        //if there are no configuration loaded, set the theme to "blue"
        JLV.Session.ui_conf = {
            options: [
                {"theme": "blue", "menu": "bottom"}
            ]
        };
    }

    //TODO: melcolm handle dodVLER item - default
    if(JLV.Session.dodVLER === null || JLV.Session.dodVLER === ""){
        JLV.Session.dodVLER = false;
    }
    if (JLV.Session.dasVLER === null || JLV.Session.dasVLER === "") {
        JLV.Session.dasVLER = false;
    }

    //load ui config (theme, prov_ref)
    if (JLV.Session.ui_conf.options !== null && JLV.Session.ui_conf.options.length > 0) {

        $.each(JLV.Session.ui_conf.options, function (index, opt) {

            if (opt.theme) {
                JLV.Session.theme = opt.theme;
            } else if (opt.prov_ref) {
                JLV.Session.prov_ref = opt.prov_ref;
            } else if (opt.pat_insurance_sort_info) {
                JLV.PatientDemographics.InsuraceSortInfo = opt.pat_insurance_sort_info;
            }
            else if (opt.pat_insurance_hidden_columns) {
                JLV.PatientDemographics.HiddenColumns = opt.pat_insurance_hidden_columns;
            }
            else if (opt.loginAgency) {
                JLV.Session.loginAgency = opt.loginAgency;
            } else if (opt.loginSite) {
                JLV.Session.loginSite = opt.loginSite;
            } else if (opt.dodVLER == true) {
                JLV.Session.dodVLER = opt.dodVLER;
            } else if (opt.dasVLER == true) {
                JLV.Session.dasVLER = opt.dasVLER;
            }
        });

        //for ahlta login, no login agency is set...manually set:
        if(typeof JLV.Session.loginAgency == "undefined") {
            JLV.Session.loginAgency = "DOD";
        }

    }

});

//session timeout warning

JLV.Session.Timer = null;

JLV.Session.RunTimeoutNotification = function () {

    clearTimeout(JLV.Session.Timer);

    var timeOut = setTimeout(function () {
        JLV.common.closeAuxBrowserWindows();
        window.location = 'authenticate/logout';
    }, 5 * 60 * 1000);

    AppKit.AlertDialog.confirm('Session Timeout Notification',
        'Your session will expire in 5 minutes and the server will log you out. Do you wish to continue using JLV?',
        function (answer) {

            if (answer) {

                $.post('view/refresh', function () {
                    JLV.Session.StartSessionTimer();
                });

                clearTimeout(timeOut);
            }
            else {
                //do logout
                JLV.common.closeAuxBrowserWindows();
                window.location = 'authenticate/logout';
            }


        });


};

JLV.Session.StartSessionTimer = function () {

    if (!JLV.common.AHLTA_LOGIN_FLAG) {
        clearTimeout(JLV.Session.Timer);

        JLV.Session.Timer = null;

        JLV.Session.Timer = setTimeout(JLV.Session.RunTimeoutNotification,
            (JLV.Session.Timeout - 5) * 1000 * 60);
    }
};

AppKit.launch(function () {

    JLV.Session.StartSessionTimer();

    //handle server timeout
    $.ajaxSetup({
        statusCode: {
            403: function () {
                window.location = 'authenticate/login';
            }
        }
    });


    //send CSRF token in all ajax request headers
    var CSRF_HEADER = 'X-CSRF-Token';

    var setCSRFToken = function (securityToken) {
        jQuery.ajaxPrefilter(function (options, _, xhr) {
            if (!xhr.crossDomain)
                xhr.setRequestHeader(CSRF_HEADER, securityToken);
        });
    };

    setCSRFToken($('meta[name="csrf-token"]').attr('content'));


    $(document).ajaxSuccess(function (event, xhr, settings) {
        if (settings.url !== 'app/heartbeatStatus'
        ) {
            JLV.Session.StartSessionTimer();
        }
    });
});
