//= require ./PatientWidget.js
//= require ./windows/PatientAdmissionsWidgetWindow.js


JLV.PatientAdmissionsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'ward',
    url: 'patientAdmissions/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['admissionDate', 'dischargeDate'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system === '2.16.840.1.113883.6.96' ||
                            data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientAdmissions = JLV.PatientWidget.extend({
    options: {
        widgetId: 'admissions_widget',
        name: 'Admissions',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientAdmissionsWidgetWindow',
        columns: [
            {
                title: 'Adm Date',
                id: 'admissionDate',
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken') &&
                        model.get('site').agency === JLV.common.AGENCY_VA) {
                        return JLV.common.linkDateFormatter(value, model);
                    }
                    else {
                        return JLV.common.dateFormatter(value);
                    }
                },
                sortable: true,
                width: 110
            },
            {
                title: 'Discharge Date',
                id: 'dischargeDate',
                width: 150,
                sortable: true,
                formatter: JLV.common.dateFormatter

            },
            {
                title: 'Diagnosis',
                width: 120,
                sortable: true,
                id: 'diagnosisText',
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Ward',
                width: 125,
                sortable: true,
                id: 'ward'
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'admissionDate',
            asc: false
        },
        data: JLV.PatientAdmissionsCollection
    },
    initialize: function (options) {
        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.checkConnectionUnavailable();

            }, this)
        });
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {

            var detailStartDate = new Date(),
                detailEndDate = new Date(),
                ONE_DAY = 24 * 60 * 60 * 1000;

            var eventId = model.attributes.admissionId;

            detailStartDate.setTime(model.get('admissionDate').getTime() - ONE_DAY);
            detailEndDate.setTime(model.get('admissionDate').getTime() + ONE_DAY);

            var dialog = new JLV.DetailsWindow({
                title: String.format("Admission Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('ward')),
                url: 'patientAdmissions/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken'),
                    recordSiteMoniker: model.get('site').moniker,
                    recordSiteAgency: model.get('site').agency,
                    startDate: moment(detailStartDate).format("M/D/YY"),
                    endDate: moment(detailEndDate).format("M/D/YY")
                },
                model: model,
                eventId: eventId,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {

                    return '<pre>' + response.detailText + '</pre>';
                }
            });

            dialog.center();
            dialog.open();
        }

    }

});


JLV.registerJLVWidget('admissions_widget', JLV.PatientAdmissions);