//= require ./PatientWidget.js
//= require ./windows/PatientAllergiesWidgetWindow.js

JLV.AllergiesCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'allergyName',
    url: 'patientAllergies/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['enteredDate'],
        parse: function (data) {
            return this._super(data);
        }
    })
});

JLV.PatientAllergies = JLV.PatientWidget.extend({
    options: {
        widgetId: 'allergies_widget',
        name: 'Allergies',
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientAllergiesWidgetWindow',
        columns: [
            {
                id: 'enteredDate',
                title: 'Date Recorded',
                width: '20%',
                sortable: true,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Allergen',
                width: '55%',
                id: 'allergyName',
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (value == undefined || value == null || value == "") {
                        value = "Allergen name was not supplied"
                    }
                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('codeNumber'),
                            normDisplay = model.get('codeName'),
                            systemDisplay = model.get('codeSystem');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                systemDisplay + ' Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Site',
                width: 50,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'allergyName',
            asc: true
        },
        data: JLV.AllergiesCollection
    },
    itemClicked: function (model) {

        var eventId = model.get('allergyId');

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                width: 800,
                height: 500,
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                model: model,
                eventId: eventId,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.open();

        } else if (model.get('recordDetailsToken')) {

            var dialog = new JLV.DetailsWindow({
                title: String.format("Allergy Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('allergyName')),
                url: 'patientAllergies/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                model: model,
                eventId: eventId,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {

                    if (response) {
                        response.onsetDate = model.get('onsetDate');
                        response.severity = model.get('severity');

                        try {
                            return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                            '<td>Onset Date:</td><td>{{onsetDate}}</td></tr><tr>' +
                            '<td>Severity:</td><td>{{severity}}</td></tr><tr>' +
                                '<td style="width:120px;">Report Text:</td><td><pre>{{reportText}}</pre></td></tr><tr>' +
                            '</tr></table>')(response);
                        } catch (e) {
                            console.info('template error');
                        }
                    }

                    return '';
                }
            });

            dialog.center();
            dialog.open();
        }
        else {
            var dialog = new JLV.DetailsWindow({
                height: 300,
                width: 500,
                title: String.format("Allergy Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('allergyName')),
                model: model,
                eventId: eventId,
                widget: this,
                reportBuilder: true,
                renderer: function () {
                    try {
                        return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                        '<td>Onset Date:</td><td>{{onsetDate}}</td></tr><tr>' +
                        '<td>Severity:</td><td>{{severity}}</td></tr><tr>' +
                        '<td>Comments:</td><td>{{comment}}</td></tr><tr>' +
                        '</table>')(model.toJSON());
                    } catch (e) {
                        console.info('template error');
                    }

                }
            });

            dialog.center();
            dialog.open();
        }
        $.post('auditUserAction/auditDetails', {
            recordType: "AllergyDetail",
            patToken: JLV.ActivePatientModel.get('patientToken'),
            recordSiteMoniker: model.get('site').moniker,
            recordSiteAgency: model.get('site').agency,
            info: model.get('allergyName'),
            recordItemId: eventId
        });
    },
    initialize: function (options) {
        this._super(options);

        //add banner that TMDS allergies is not available
        //if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {
        this.addBanner({
            fgcolor: '#fff',
            bgcolor: '#ff0000',
            height: 25,
            text: 'Allergies from Theater Data Sources are not displayed at this time.'
        });

        //}
    }


});

JLV.registerJLVWidget('allergies_widget', JLV.PatientAllergies);

