//= require ./PatientWidget.js
//= require ./windows/PatientClinicalRemindersWidgetWindow.js

JLV.PatientClinicalRemindersCollection = JLV.PatientDateFilteredDataCollection.extend({
    url: 'patientClinicalReminders/list.json',
    model: JLV.PatientDataModel.extend({
        parse: function (data) {
            if (data.name) {
                data.reminder = data.name;
                delete data['name'];
            }
            if (data.lastDate) {
                data.lastDate = moment(data.lastDate, "MMM DD, YYYY hh:mm A").toDate();
                if (!data.lastDate.isValid()) {
                    data.lastDate = null;
                }
            }
            else {
                data.lastDate = null;
            }

            if (data.lastDone) {
                data.lastDone = moment(data.lastDone, "MMM DD, YYYY hh:mm A").toDate();
                if (!data.lastDone.isValid()) {
                    data.lastDone = null;
                }
            }
            else {
                data.lastDone = null;
            }

            if (data.dueDateString === 'DUE NOW') {
                data.dueDate = new Date();
            }
            else if (data.dueDate) {
                data.dueDate = moment(data.dueDate, "MMM DD, YYYY hh:mm A").toDate();
                if (!data.dueDate.isValid()) {
                    data.dueDate = null;
                }
            }
            return this._super(data);
        }

    })
});


JLV.PatientClinicalReminders = JLV.PatientWidget.extend({
    options: {
        widgetId: 'clinicalreminders_widget',
        name: 'Clinical Reminders - VA Only',
        dateFilter: false,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientClinicalRemindersWidgetWindow',
        columns: [
            {
                title: 'Due',
                id: 'dueDate',
                sortable: true,
                width: 100,
                formatter: function (value, model) {

                    if (model.get('dueDateString') === 'DUE NOW') {
                        return model.get('dueDateString');
                    }

                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY');
                }
            },
            {
                title: 'Last Done',
                id: 'lastDate',
                width: 100,
                sortable: true,
                formatter: JLV.common.dateFormatter

            },
            {
                title: 'Reminder',
                id: 'reminder',
                width: 200,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 80,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'dueDate',
            asc: false
        },
        data: JLV.PatientClinicalRemindersCollection
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Clinical Reminder Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('name')),
                url: 'patientClinicalReminders/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {
                    return '<pre>' + response.clinicalReminderText + '</pre>';
                }
            });

            dialog.center();
            dialog.open();
        }

    }

});


JLV.registerJLVWidget('clinicalreminders_widget', JLV.PatientClinicalReminders);
