//= require ./lodash.min.js
//= require ./PatientWidget.js
//= require ./PatientAppointments.js
//= require ./windows/PatientEncounterDetailsWindow.js
//= require ./windows/PatientEncountersWidgetWindow.js

JLV.PatientEncountersCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'clinic',
    url: 'patientEncounters/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['apptDate'],
        parse: function (data) {

            if (data.codes) {
                if (data.codes[0]) {
                    if ((data.codes[0].system === '2.16.840.1.113883.6.96' ||
                        data.codes[0].system.toUpperCase().indexOf("SNOMED") >= 0)) {
                        data.normCode = data.codes[0].code;
                        data.normDisplay = data.codes[0].display;
                    }
                }
            }
            if (data.diagnosis) {
                if (data.diagnosis[0]) {
                    data.diagnosisList = data.diagnosis[0].description;
                    // firstDisplayRun = false;
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientEncounters = JLV.PatientWidget.extend({
    options: {
        widgetId: 'encounters_widget',
        name: 'Outpatient Encounters',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientEncountersWidgetWindow',
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Encounter<br/>Date',
                id: 'apptDate',
                sortable: true,
                width: 100,
                formatter: function (value, model) {

                    if (model && model.get('clinic') === 'Connection unavailable.') {
                        return JLV.common.dateFormatter(value);
                    }

                    if (model) {
                        if (model.get('complexDataUrl') || model.get('noteText') || model.get('vaNote')) {
                            var detailLink = "";
                            if("2.16.840.1.113883.3.42.10024.100001.13" != model.get("site").siteCode){
                                detailLink =  JLV.common.linkFormatter("Details", model);
                            }
                            var links = JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + detailLink;
                            var documents = model.get('documents');
                            if (documents && documents.length > 1) {
                                for (var i = 0; i < documents.length; i++) {
                                    links += ' ' + JLV.common.linkFormatter("Note" + (i + 1), model);
                                }
                            } else {
                                links += ' ' + JLV.common.linkFormatter("Note", model);
                            }
                            return links;
                        } else {
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model) + ' ' + "Note"
                        }
                    }
                    return value;
                }
            },
            {
                title: 'Clinic',
                id: 'clinic',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter

            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Diagnosis',
                width: 100,
                id: 'diagnosisList',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var documents = model.get('documents');
                        if (documents && documents.length > 0) {
                            var study = documents[0].vixStudy;
                            if (study && study.viewerUrl) {
                                var viewerUrl = study.viewerUrl;

                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var viewerCmd = String.format("JLV.common.openEncounterVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));

                                return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                            } else if (model.get('hasImage')) {  //encounter has image
                                var imageDocIndex = -1;
                                for (var i = 0; i < documents.length; i++) {
                                    if (documents[i].hasImage) {
                                        imageDocIndex = i;
                                        break;
                                    }
                                }

                                if (imageDocIndex >= 0) {
                                    var document = documents[imageDocIndex];
                                    var documentId = document.id;
                                    var documentSiteId = model.get('site').siteCode;

                                    if (documentId && documentSiteId) {
                                        if (forPrint) {
                                            return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                        }

                                        var uniqueId = documentId + documentSiteId;
                                        var imageId = 'encounterWidgetImage' + uniqueId;
                                        var spinnerId = 'encounterWidgetSpinner' + uniqueId;

                                        var viewerCmd = String.format("JLV.common.getUrlAndOpenEncounterVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\");return false;",
                                            documentId, documentSiteId, model.get('recordDetailsToken'), imageId, spinnerId);

                                        return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                            + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id=' + spinnerId + ' style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                                    }
                                }
                            }

                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'apptDate',
            asc: false
        },
        data: JLV.PatientEncountersCollection
    },
    initialize: function (options) {

        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });
        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.checkConnectionUnavailable();

            }, this)
        });


    },
    itemClicked: function (model, item) {

        //if (model.get('sensitive')) {
        //    AppKit.AlertDialog.alert('Sensitive Encounter', JLV.common.SENSITIVE_ENCOUNTER_MSG);
        //    return;
        //}

        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            }

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function(agree, auditText) {
                if (agree) {
                    self._handleDetails(model, item, auditText);
                }
            });
        } else {
            this._handleDetails(model, item);
        }

    },
    _handleDetails: function (model, item, auditText) {


        if (_.startsWith(item.text(), 'Note')) {
            var auditUrl;
            if (model.get('noteAsBinaryStream')) {
                //detect if GENESIS document
                if(model.get('site') && model.get('site').siteCode == "2.16.840.1.113883.3.42.10024.100001.13"){
                    var docUrl = String.format("mhsGenesis?patToken={0}&recToken={1}&sourceDomain={2}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), "ENCOUNTER");

                    JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                        if (download) {
                            JLV.common.openAuxBrowserWindow(docUrl, "JLV - " + model.get('apptType'));
                        }
                        else if (rb) {
                            this._addToReportBuilder(model, true, 'GetNoteDataStream');
                        }

                    }, this));
                }
                else {
                    if (auditText) {
                        auditUrl = String.format("patientEncounters/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}",
                            JLV.ActivePatientModel.get('patientToken'),
                            model.get('recordDetailsToken'),
                            auditText
                        );
                    } else {
                        auditUrl = String.format("patientEncounters/getNoteDataStream?patToken={0}&recToken={1}",
                            JLV.ActivePatientModel.get('patientToken'),
                            model.get('recordDetailsToken'));
                    }

                    var noteURL = auditUrl;

                    if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {

                        JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                            if (download) {
                                $.post('patientEncounters/auditDetails', {
                                    patToken: this.patient.get('patientToken'),
                                    recordDate: model.get('apptDate'),
                                    recordSiteMoniker: model.get('site') ? model.get('site').moniker : '',
                                    recordSiteAgency: model.get('site') ? model.get('site').agency : '',
                                    info: model.get('diagnosisList'),
                                    eventId: model.get('eventId')
                                });
                                JLV.common.openAuxBrowserWindow(noteURL, 'JLV_DEPLOYMENT_FORMS');
                            }
                            else if (rb) {
                                this._addToReportBuilder(model, true, 'GetEncounterDataStream');
                            }

                        }, this));
                    }
                    else {
                        JLV.common.openAuxBrowserWindow(noteURL, 'JLV_DEPLOYMENT_FORMS');
                    }
                }


            }
            else {
                var itemLength = item.text().length;
                var noteIndex = (itemLength > 4) ? parseInt(item.text().substring(4, itemLength)) -1 : 0;
                var noteId = model.get('documents')[noteIndex].id;

                auditUrl = 'patientEncounters/details.json';
                if (auditText) {
                    auditUrl = String.format(auditUrl + "?auditText={0}", auditText);
                }

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Outpatient Encounters Note Detail ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        JLV.common.dateFormatter(model.get('apptDate'))),
                    url: auditUrl,
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken'),
                        widgetName: this.name,
                        noteId: noteId
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response, forPrint) {
                        var thumbnail = "";
                        if (JLV.common.isVixDisplayEnabled && model) {
                            var documents = model.get('documents');
                            if (documents && documents.length > 0) {
                                var study = documents[noteIndex].vixStudy;
                                if (study && study.viewerUrl) {
                                    var series;
                                    var viewerCmd =
                                        String.format('JLV.common.openEncounterVixViewer(\'{0}\',\'{1}\');',
                                            study.viewerUrl, model.get('recordDetailsToken'));

                                    if (study.series.length > 0) {
                                        series = study.series[0];

                                        if (forPrint) {
                                            thumbnail = '<div style="float:right;display: inline-block;">' +
                                                '<div>Associated image(s) not available for printing.</div>' +
                                                '</div>';

                                            response.images = '';
                                        }
                                        else {
                                            thumbnail = String.format(
                                                '<div style="float:right;display: inline-block;">' +
                                                '<a href="#" onClick="{0}">' +
                                                '<span style="position: relative;">' +
                                                '<img src="{1}" alt="{2}" title="{3}"/>' +
                                                '<div class="camera-icon">' +
                                                '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                                '</div>' +
                                                '</a></span>' +
                                                '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                                '</div>',
                                                viewerCmd, series.thumbnailUrl, series.caption, series.caption, viewerCmd);

                                        }
                                    }
                                } else if (model.get('hasImage')) {
                                    var documentSiteId = model.get('site').siteCode;

                                    if (noteId && documentSiteId) {

                                        var uniqueId = noteId + documentSiteId;
                                        var imageId = 'encounterDetailImage' + uniqueId;
                                        var spinnerId = 'encounterDetailSpinner' + uniqueId;

                                        var viewerCmd = String.format('JLV.common.getUrlAndOpenEncounterVixViewer(\'{0}\',\'{1}\',\'{2}\',\'{3}\',\'{4}\');return false;',
                                            noteId, documentSiteId, model.get('recordDetailsToken'), imageId, spinnerId);

                                        if (forPrint) {
                                            thumbnail = '<div style="float:right;display: inline-block;">' +
                                                '<div>Associated image(s) not available for printing.</div>' +
                                                '</div>';

                                            response.images = '';
                                        } else {
                                            thumbnail = String.format(
                                                '<div id="' + imageId + '" style="float:right;display: inline-block;">' +
                                                '<a href="#" onClick="{0}">' +
                                                '<span style="position: relative;">' +
                                                '<img src="{1}" alt="{2}" title="{3}"/>' +
                                                '<div class="camera-icon">' +
                                                '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                                '</div>' +
                                                '</a></span>' +
                                                '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                                '</div><div id=' + spinnerId + ' style="display: none;float:right;"><img src="assets/loading.gif"/></div>',
                                                viewerCmd, "assets/thumbnail.png", documents[noteIndex].localTitle, documents[noteIndex].localTitle, viewerCmd);
                                        }
                                    }
                                }
                            }
                        }
                        return thumbnail + '<pre class="report-details">' +
                            response.noteText +
                            response.images + '</pre>'
                    }
                });

                dialog.center();
                dialog.open();
            }
        }
        else {

            if (model.get('sensitive')) {
                var auditUrl = String.format('patientEncounters/sensitiveEncounter.json?auditText={0}', auditText);

                var widget = this;

                // need to retrieve the encounter from the cache
                $.post(auditUrl, {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                    },
                    function (response) {

                        model = new JLV.WidgetDataModel(response, {parse: true});

                        if (model.attributes.codes) {
                            for (var i in model.attributes.codes) {
                                if (model.attributes.codes[i].system) {
                                    if (model.attributes.codes[i].system === '2.16.840.1.113883.6.96') {
                                        model.attributes.normCode = model.attributes.codes[i].code;
                                        model.attributes.normDisplay = model.attributes.codes[i].display;
                                        break;
                                    }
                                }
                            }
                        }

                        var documents = model.get('documents');
                        var noteIDList = "";
                        if(documents) {
                            for (var i in documents) {
                                noteIDList += documents[i].id + ";";
                            }
                        }


                        $.post('patientEncounters/getEncounterPCE', {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recordSiteCode: model.get('site') ? model.get('site').siteCode : '',
                            noteIDList : noteIDList
                        }, function (response) {
                            model.set('pces', response);
                            var dialog = new JLV.PatientEncounterDetailsWindow({
                                title: String.format("Outpatient Encounters Detail ~ {0} {1}",
                                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                    model.get('clinic')),
                                encounter: model,
                                model: model,
                                widget: widget,
                                reportBuilder: true,
                                patientToken: JLV.ActivePatientModel.get('patientToken')
                            });

                            dialog.center();
                            dialog.open();
                        });

                })
            } else {
                var documents = model.get('documents');
                var noteIDList = "";
                if(documents) {
                    for (var i in documents) {
                       noteIDList += documents[i].id + ";";
                    }
                }


                $.post('patientEncounters/getEncounterPCE', {
                    patToken: this.patient.get('patientToken'),
                    recordSiteCode: model.get('site') ? model.get('site').siteCode : '',
                    noteIDList : noteIDList
                }, function (response){
                    model.set('pces', response);
                    var dialog = new JLV.PatientEncounterDetailsWindow({
                        title: String.format("Outpatient Encounters Detail ~ {0} {1}",
                            JLV.common.formatSiteMonikerDisplay(model.get('site')),
                            model.get('clinic')),
                        encounter: model,
                        model: model,
                        widget: this,
                        reportBuilder: true,
                        patientToken: JLV.ActivePatientModel.get('patientToken')
                    });

                    dialog.center();
                    dialog.open();
                });



            }

        }
    }

});




JLV.registerJLVWidget('encounters_widget', JLV.PatientEncounters);