//= require ./PatientWidget.js
//= require ./windows/PatientHealthSummariesWidgetWindow.js

JLV.HealthSummariesCollection = JLV.PatientDataCollection.extend({
    url: 'patientHealthSummaries/list.json'
});


JLV.PatientHealthSummaries = JLV.PatientWidget.extend({
    options: {
        widgetId: 'health_summaries_widget',
        name: 'Health Summaries',
        clipboard: true,
        print: true,
        refresh: true,
        status: true,
        moreWindowClass: 'JLV.PatientHealthSummariesWidgetWindow',
        columns: [
            {
                title: 'Description',
                width: '55%',
                id: 'name',
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (value == undefined || value == null || value == "") {
                        value = "Description was not supplied"
                    }
                    if (model.get('recordDetailsToken'))
                        return JLV.common.linkFormatter(value, model);

                    return value;
                }
            },
            {
                title: 'Site',
                width: 50,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'name',
            asc: true
        },
        data: JLV.HealthSummariesCollection
    },
    itemClicked: function (model) {
        if (model.get('recordDetailsToken')) {
            var dialog = new JLV.DetailsWindow({
                width: 800,
                height: 500,
                title: String.format("{0} ~ {1}", model.get('name'),
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientHealthSummaries/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {
                    if (response.reportText.trim().length > 0) {
                        return '<pre>' + response.reportText + '</pre>';
                    }
                    else {
                        return 'No summary found';
                    }

                }
            });

            dialog.center();
            dialog.open();
        }

    }

});

JLV.registerJLVWidget('health_summaries_widget', JLV.PatientHealthSummaries);