//= require ./PatientWidget.js
//= require ./windows/PatientLabResultsWidgetWindow.js

JLV.PatientLabResultsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'testName',
    url: 'patientLabs/results.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['collectionDate'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("LABS") >= 0 ||
                            data.codes[i].system === '2.16.840.1.113883.6.1' ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {

                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }

                data.testLabel = JLV.common.labLabels[data.accessionType];
            }

            return this._super(data);
        }
    })
});




JLV.PatientLabResults = JLV.PatientWidget.extend({
    options: {
        widgetId: 'labs_results_widget',
        name: 'Lab Results',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientLabResultsWidgetWindow',
        columns: [
            {
                title: 'Collection Date',
                id: 'collectionDate',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                },
                sortable: true,
                width: 80
            },
            {
                title: 'Lab Test',
                id: 'testName',
                width: 150,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && model.get('interpretation')) {
                        if(model.get('interpretation').toLowerCase().indexOf("critical") > -1
                            || model.get('interpretation').toLowerCase().indexOf("*") > -1){
                            return '<div class="jlv-high-alert-row">' + JLV.common.linkFormatter(value, model) + '</div>';
                        }
                        return '<div class="jlv-alert-row">' + JLV.common.linkFormatter(value, model) + '</div>';
                    }

                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes<br/>' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Result',
                id: 'result',
                width: 60,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if( model.get('accessionType') == "MICRO"
                        || model.get('accessionType') == "MI"
                        || model.get('accessionType') == "CY"
                        || model.get('accessionType') == "SP"
                        || model.get('accessionType') == "ST"
                        || model.get('accessionType') == "BB"
                        || model.get('accessionType') == "AP"
                        || model.get('accessionType') == "AU"){
                        return JLV.common.linkFormatter("result", model);
                    } else if (!isNaN(parseFloat(value)) && model.get('units')) {
                        return value + " " + model.get('units');
                    }

                    return value;
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 80,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'resultDate',
            asc: false
        },
        data: JLV.PatientLabResultsCollection
    },
    initialize: function (options) {

        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.checkConnectionUnavailable();

            }, this)
        });

        if (this._datagrid) {
            //make rows with abnormal results pink
            this._datagrid.on('afterRender', function () {

                var $row = this.$el.find('.jlv-alert-row').parents('tr');
                $row.css(
                    {
                        'backgroundColor': '#FFC0CB',
                        'color': '#000000'
                    }
                );
                $row.find('a').css('color', '#000');

                var $cRow = this.$el.find('.jlv-high-alert-row').parents('tr');
                $cRow.css(
                    {
                        'backgroundColor': 'red',
                        'color': '#FFFFFF'
                    }
                );
                $cRow.find('a').css('color', '#FFF');
            });
        }
    },
    itemClicked: function (model) {

        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            };

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function (agree, auditText) {
                if (agree) {
                    self._handleDetails(model, auditText);
                }
            });
        } else {
            this._handleDetails(model);
        }
    },
    _handleDetails: function(model, auditText) {


        if (model.get('sensitive')) {
            // if sensitive, need to retrieve the model from the cache first.
            var labResultUrl = String.format('patientLabs/sensitiveResult.json?auditText={0}', auditText)

            var widget = this;

            $.post(labResultUrl, {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                }, function (response) {
                    model = new JLV.WidgetDataModel(response, {parse: true});

                    if (model.attributes.codes) {
                        for (var i in model.attributes.codes) {
                            if (model.attributes.codes[i].system) {
                                if (model.attributes.codes[i].system.toUpperCase().indexOf("LABS") >= 0 ||
                                    model.attributes.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {

                                    model.attributes.normCode = model.attributes.codes[i].code;
                                    model.attributes.normDisplay = model.attributes.codes[i].display;
                                    break;
                                }
                            }
                        }
                    }

                    if (model.get('freeTextReport')) {
                        var dialog = new JLV.PatientLabReportWindow({
                            title: String.format("Lab Results ~ {0} {1}",
                                JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                model.get('testName')),
                            model: model,
                            widget: widget,
                            reportBuilder: true,
                            patientToken: JLV.ActivePatientModel.get('patientToken'),
                            labResult: model.toJSON()
                        });
                    } else {
                        var dialog = new JLV.PatientLabResultsWindow({
                            title: String.format("Lab Results ~ {0} {1}",
                                JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                model.get('testName')),
                            model: model,
                            widget: widget,
                            reportBuilder: true,
                            patientToken: JLV.ActivePatientModel.get('patientToken'),
                            labResults: [model.toJSON()]
                        });
                    }
                    dialog.center();
                    dialog.open();

                }
            )
        } else {
            if (model.get('freeTextReport')) {
                var dialog = new JLV.PatientLabReportWindow({
                    title: String.format("Lab Results ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('testName')),
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    patientToken: JLV.ActivePatientModel.get('patientToken'),
                    labResult: model.toJSON()
                });
            } else {
                var dialog = new JLV.PatientLabResultsWindow({
                    title: String.format("Lab Results ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('testName')),
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    patientToken: JLV.ActivePatientModel.get('patientToken'),
                    labResults: [model.toJSON()]
                });
            }
            dialog.center();
            dialog.open();
        }
    }

});


JLV.registerJLVWidget('labs_results_widget', JLV.PatientLabResults);
