//= require PatientWidget.js
//= require ./windows/PatientProblemListWidgetWindow.js

JLV.ProblemListCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'description',
    url: 'patientProblemList/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['onsetDate', 'lastUpdated'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system === '2.16.840.1.113883.6.96' ||
                            data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            if(data.codes[i].display){
                                data.normDisplay = data.codes[i].display;
                            }
                            else {
                                data.normDisplay = '';
                            }
                            break;
                        }
                    }
                }
            }

            if (_.isString(data.status)) {
                data.status = data.status.toUpperCase();
            }

            var type = data.type;

            if (!type || type.trim() === '') {

                if (data.site) {
                    if (data.site.agency === 'DOD') {
                        data.type = 'Major';
                    }
                }

            }
            else {
                if (type.toLowerCase().indexOf('limited') > -1) {
                    data.type = 'Minor';
                }
                else if (type.toLowerCase().indexOf('admin') > -1) {
                    data.type = 'Admin';
                }
            }

            return this._super(data);
        }
    }),
    doFetch: function (patient, callback) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                includeVLER: JLV.Session.dodVLERReady == true,
                includeDASVLER: JLV.Session.dasVLERReady == true,
                status: 'A'
            },
            reset: true,
            type: 'POST',
            success: callback
        });
    }
});


JLV.PatientProblemList = JLV.PatientWidget.extend({
    options: {
        widgetId: 'problemlist_widget',
        name: 'Problem List',
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        gridHeaderHeight: 30,
        moreWindowClass: 'JLV.PatientProblemListWidgetWindow',
        columns: [
            {
                title: 'Updated<br/>Date',
                width: 80,
                stretchable: false,
                id: 'lastUpdated',
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Problem Description',
                minWidth: 80,
                id: 'description',
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'ICD',
                width: 52,
                stretchable: false,
                id: 'icdCode'
            },
            {
                title: 'Status',
                width: 60,
                stretchable: false,
                id: 'status'
            },
            {
                title: 'Site',
                width: 70,
                stretchable: false,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastUpdated',
            asc: false
        },
        data: JLV.ProblemListCollection
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.set('visible', true);
        }
        else if (model.get('recordDetailsToken')) {
            var _isValidStr = function (str) {
                return str != null && str.length > 0;
            };

            var dialog = new JLV.DetailsWindow({
                title: String.format("Problem Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('description')),
                url: 'patientProblemList/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken'),
                    info: model.get('description'),
                    eventId: model.get('eventId'),
                    problemId: model.get('problemId')
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {

                    if (!response.sessionCachedRecord) {
                        return '<pre>' + response.detailText + '</pre>';
                    }
                    else {
                        var html = _.template([
                            '<div class="problemdetailstitle">{{description}}</div>',
                            '<table align="center" class="problemdetails">',
                            '<tr>',
                            '<td align="right">Onset Date:</td>',
                            '<td align="left">{{onsetDate}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Status:</td>',
                            '<td align="left">{{status}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Provider:</td>',
                            '<td align="left">{{responsibleProvider}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Clinic:</td>',
                            '<td align="left">{{hospitalLocation}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Recorded Date:</td>',
                            '<td align="left">{{recordedDate}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Recorded By:</td>',
                            '<td align="left">{{recordedBy}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Entered Date:</td>',
                            '<td align="left">{{enteredDate}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Entered By:</td>',
                            '<td align="left">{{enteredBy}}</td>',
                            '</tr>',
                            '<tr>',
                            '<td align="right">Modified Date:</td>',
                            '<td align="left">{{lastUpdated}}</td>',
                            '</tr>',
                            '</table>',
                            '<div class="problemdetailstitle">Comments</div>'
                        ].join(''))(response);

                        for (var note in response.notes) {
                            if (_isValidStr(response.notes[note].noteEnteredBy)) {
                                html += _.template([
                                    '<table align="center" class="problemdetails">',
                                    '<tr>',
                                    '<td align="left">{{noteText}}</td>',
                                    '</tr>',
                                    '<tr>',
                                    '<td align="left">Entered By: {{noteEnteredBy}} on {{noteDate}}</td>',
                                    '</tr>',
                                    '</table>',
                                ].join(''))(response.notes[note]);
                            }
                            else {
                                html += _.template([
                                    '<table align="center" class="problemdetails">',
                                    '<tr>',
                                    '<td align="left">{{noteText}}</td>',
                                    '</tr>',
                                    '</table>',
                                ].join(''))(response.notes[note]);
                            }
                        }

                        return html;
                    }
                }
            });

            dialog.center();
            dialog.open();
        }
    },
    initialize: function (options) {
        this._super(options);

        if (this._datagrid) {
            this._datagrid.set('filter', function (model) {

                if (model.get('site').agency === 'DOD') {
                    var type = model.get('type'),
                        status = model.get('status');
                    return type.toUpperCase() === 'MAJOR' && status.toUpperCase() === "ACTIVE"
                }
                else {
                    if (_.isString(model.get('status')))
                        return (model.get('status').toUpperCase() === "ACTIVE");

                }


                return false;
            });
        }
    },

    loadData: function () {
        this._super(_.bind(function () {

            if (this._datagrid.get('tableData').length == 0) {
                if (this.get('data').length > 0) {
                    this._datagrid.showMessage("No Active Problems. <br/> Click More to see all Problems.");
                    if (this.get('moreWindowClass')) {
                        this._moreBtn.show();
                    }
                }
            }
        }, this));
    }


});

JLV.registerJLVWidget('problemlist_widget', JLV.PatientProblemList);
