//= require ./lodash.min.js
// = require ./PatientWidget.js
//= require ./windows/PatientProceduresWidgetWindow.js

JLV.PatientProceduresCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'location',
    url: 'patientProcedures/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['date'],
        parse: function (data) {

            if(data.code){
                data.normCode = data.code;
            }
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system === '2.16.840.1.113883.6.12' ||
                            data.codes[i].system.toUpperCase().indexOf("CPT") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            if (!data.normDisplay){
                                data.normDisplay = '';
                            }
                            break;
                        }
                    }
                }
            }

            var type = data.type;

            if (!type || type.trim() === '') {

                if (data.site) {
                    if (data.site.agency === 'DOD') {
                        data.type = 'Medical';
                    }
                }
            }
            else {
                if (type.toLowerCase() === 'metrics') {
                    data.type = 'Medical';
                }
                else {
                    data.type = 'Administrative';
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientProcedures = JLV.PatientWidget.extend({
    options: {
        widgetId: 'procedures_widget',
        name: 'Procedures',
        gridHeaderHeight: 30,
        dateFilter: true,
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientProceduresWidgetWindow',
        columns: [
            {
                title: 'Procedure Date',
                id: 'date',
                formatter: function (value, model) {
                    if (model) {
                        // Note link for VA Outpatient Encounter notes
                        if (model.get('complexDataUrl') || model.get('noteText') || model.get('vaNote')) {
                            var links = JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>';
                            var documents = model.get('documents');
                            if (documents && documents.length > 1) {
                                for (var i = 0; i < documents.length; i++) {
                                    if (i > 0) {
                                        links += ' ';
                                    }
                                    links += JLV.common.linkFormatter("Note" + (i + 1), model);
                                }
                            } else {
                                links += JLV.common.linkFormatter("Note", model);
                            }
                            return links;
                        } else {
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                        }
                    }
                    return value;
                },
                sortable: true,
                width: 110
            },
            {
                width: 100,
                title: 'Location',
                sortable: true,
                id: 'location'
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Procedure <br/> Description',
                sortable: true,
                id: 'description',
                formatter: function (value, model) {
                    // Only DoD VLER narratives have a link
                    if (model && model.get('narrativeId')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    return value;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Current Procedural Terminology<br/>' +
                                '(CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Current Procedural Terminology<br/>' +
                        '(CPT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var documents = model.get('documents');
                        if (documents && documents.length > 0) {
                            var study = documents[0].vixStudy;
                            if (study && study.viewerUrl) {
                                var viewerUrl = study.viewerUrl;
                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var viewerCmd = String.format("JLV.common.openProcedureVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));

                                return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                            } else if (model.get('hasImage')) {  //procedure has image
                                var imageDocIndex = -1;
                                for (var i = 0; i < documents.length; i++) {
                                    if (documents[i].hasImage) {
                                        imageDocIndex = i;
                                        break;
                                    }
                                }

                                if (imageDocIndex >= 0) {
                                    var document = documents[imageDocIndex];
                                    var documentId = document.id;
                                    var documentSiteId = model.get('site').siteCode;

                                    if (documentId && documentSiteId) {
                                        if (forPrint) {
                                            return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                        }

                                        var uniqueId = documentId + documentSiteId;
                                        var imageId = 'procedureWidgetImage' + uniqueId;
                                        var spinnerId = 'procedureWidgetSpinner' + uniqueId;

                                        var viewerCmd = String.format("JLV.common.getUrlAndOpenProcedureVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\");return false;",
                                            documentId, documentSiteId, model.get('recordDetailsToken'), imageId, spinnerId);

                                        return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                            + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id=' + spinnerId + ' style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                                    }
                                }
                            }
                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'date',
            asc: false
        },
        data: JLV.PatientProceduresCollection
    },
    itemClicked: function (model, item) {
        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            }
            var self = this;
            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function (agree, auditText) {
                if (agree) {
                    self._handleDetails(model, item, auditText);
                }
            });
        } else {
            this._handleDetails(model, item);
        }
    },
    _handleDetails: function (model, item, auditText) {

        if(model.get('site') && model.get('site').siteCode == "2.16.840.1.113883.3.42.10024.100001.13"){
            var docUrl = String.format("mhsGenesis?patToken={0}&recToken={1}&sourceDomain={2}",
                JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), "PROCEDURE");

            JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                if (download) {
                    JLV.common.openAuxBrowserWindow(docUrl, "JLV - " + model.get('document'));
                }
                else if (rb) {
                    this._addToReportBuilder(model, true, 'GetNoteDataStream');
                }

            }, this));
        } else if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.set('visible', true);
        }
        else if (model.get('vaNote')) {
            var itemLength = item.text().length;
            var noteIndex = (itemLength > 4) ? parseInt(item.text().substring(4, itemLength)) -1 : 0;
            var noteId = model.get('documents')[noteIndex].id;
            var dialog = new JLV.DetailsWindow({
                title: String.format("Outpatient Encounters Note Detail ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    JLV.common.dateFormatter(model.get('date'))),
                url: 'patientProcedures/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken'),
                    noteId: noteId
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response, forPrint) {
                    var thumbnail = "";
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var documents = model.get('documents');
                        if (documents && documents.length > 0) {
                            var study = documents[noteIndex].vixStudy;
                            if (study && study.viewerUrl) {
                                var series;
                                var viewerCmd =
                                    String.format('JLV.common.openProcedureVixViewer(\'{0}\',\'{1}\');',
                                        study.viewerUrl, model.get('recordDetailsToken'));

                                if (study.series.length > 0) {
                                    series = study.series[0];

                                    if (forPrint) {
                                        thumbnail = '<div style="float:right;display: inline-block;">' +
                                            '<div>Associated image(s) not available for printing.</div>' +
                                            '</div>';

                                        response.images = '';
                                    }
                                    else {
                                        thumbnail = String.format(
                                            '<div style="float:right;display: inline-block;">' +
                                            '<a href="#" onClick="{0}">' +
                                            '<span style="position: relative;">' +
                                            '<img src="{1}" alt="{2}" title="{3}"/>' +
                                            '<div class="camera-icon">' +
                                            '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                            '</div>' +
                                            '</a></span>' +
                                            '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                            '</div>',
                                            viewerCmd, series.thumbnailUrl, series.caption, series.caption, viewerCmd);

                                    }

                                }
                            } else if (model.get('hasImage')) {
                                var documentSiteId = model.get('site').siteCode;

                                if (noteId && documentSiteId) {

                                    var uniqueId = noteId + documentSiteId;
                                    var imageId = 'procedureNoteImage' + uniqueId;
                                    var spinnerId = 'procedureNoteSpinner' + uniqueId;

                                    var viewerCmd = String.format('JLV.common.getUrlAndOpenProcedureVixViewer(\'{0}\',\'{1}\',\'{2}\',\'{3}\',\'{4}\');return false;',
                                        noteId, documentSiteId, model.get('recordDetailsToken'), imageId, spinnerId);

                                    if (forPrint) {
                                        thumbnail = '<div style="float:right;display: inline-block;">' +
                                            '<div>Associated image(s) not available for printing.</div>' +
                                            '</div>';

                                        response.images = '';
                                    } else {
                                        thumbnail = String.format(
                                            '<div id="' + imageId + '" style="float:right;display: inline-block;">' +
                                            '<a href="#" onClick="{0}">' +
                                            '<span style="position: relative;">' +
                                            '<img src="{1}" alt="{2}" title="{3}"/>' +
                                            '<div class="camera-icon">' +
                                            '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                            '</div>' +
                                            '</a></span>' +
                                            '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                            '</div><div id=' + spinnerId + ' style="display: none;float:right;"><img src="assets/loading.gif"/></div>',
                                            viewerCmd, "assets/thumbnail.png", documents[noteIndex].localTitle, documents[noteIndex].localTitle, viewerCmd);
                                    }
                                }
                            }
                        }
                    }

                    return thumbnail + '<pre class="report-details">' +
                        response.noteText +
                        response.images + '</pre>'
                }
            });

            dialog.center();
            dialog.open();
        } else {
            if (_.startsWith(item.text(), 'Note')) {

                var auditUrl;
                if (model.get('complexDataUrl') && (model.get('complexDataUrl').toLowerCase().indexOf("http") > -1) || (model.get('complexDataUrl').indexOf("Sensitive") > -1)) {

                    if (auditText) {
                        auditUrl = String.format("patientProcedures/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}&examDate={3}&info={4}",
                            JLV.ActivePatientModel.get('patientToken'),
                            model.get('recordDetailsToken'),
                            auditText,
                            model.get('date'),
                            model.get('description')
                        );
                    } else {
                        auditUrl = String.format("patientProcedures/getNoteDataStream?patToken={0}&recToken={1}&examDate={2}&info={3}",
                            JLV.ActivePatientModel.get('patientToken'),
                            model.get('recordDetailsToken'),
                            model.get('date'),
                            model.get('description'));
                    }

                    var noteURL = auditUrl;

                    if (true) {//JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {

                        JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                            if (download) {
                                JLV.common.openAuxBrowserWindow(noteURL, 'JLV_ENCOUNTER_NOTE');
                            }
                            else if (rb) {
                                this._addToReportBuilder(model, true, 'GetEncounterDataStream');
                            }

                        }, this));
                    }
                    else {
                        JLV.common.openAuxBrowserWindow(noteURL, 'JLV_ENCOUNTER_NOTE');

                    }
                }
            }
        }


    }
});


JLV.registerJLVWidget('procedures_widget', JLV.PatientProcedures);