//= require ./PatientWidget.js
//= require ./windows/PatientProgressNotesWidgetWindow.js

JLV.PatientProgressNotesCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'noteTitle',
    url: 'patientProgressNotes/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['noteDate'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("NOTES") === 0 ||
                            data.codes[i].system === '2.16.840.1.113883.6.1' ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientProgressNotes = JLV.PatientWidget.extend({
    options: {
        widgetId: 'progressnotes_widget',
        name: 'Progress Notes',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientProgressNotesWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 90
            },
            {
                title: 'Document Type/Title',
                id: 'noteTitle',
                width: 180,
                sortable: true,
                formatter: function (value, model) {
                    if (!model || value === 'Connection unavailable.' || !model.get('recordDetailsToken')) {
                        return value;
                    }

                    var noteTitle = value;
                    if (!noteTitle || noteTitle.length < 1) {
                        noteTitle = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if (model.get('sensitive')) {
                        noteTitle = noteTitle.replace(/\n/g, '<br />');
                    }

                    return JLV.common.linkFormatter(noteTitle, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                        if(model.get('recordDetailsToken') && model.get('recordDetailsToken').indexOf('NOT_SUPPORTED') >= 0){
                            return 'This document can only be opened in HAIMS.';
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes<br/>' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var study = model.get('vixStudy');

                        if (study && study.viewerUrl) {
                            var viewerUrl = study.viewerUrl;
                            if (forPrint) {
                                return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                            }

                            var viewerCmd = String.format("JLV.common.openDocVixViewer(\"{0}\",\"{1}\");return false;",
                                viewerUrl, model.get('recordDetailsToken'));

                            return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);

                        } else if (model.get('hasImage')) {
                            var siteCode = model.get('site').siteCode;

                            if (forPrint) {
                                return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                            }

                            var uniqueId = '';
                            var imageId = '';
                            var spinnerId = '';

                            var noteId = model.get('noteId');
                            var images = model.get('images');

                            uniqueId = noteId + siteCode;
                            imageId = 'progressNoteWidgetImage' + uniqueId;
                            spinnerId = 'progressNoteWidgetSpinner' + uniqueId;

                            var viewerCmd = String.format("JLV.common.getUrlAndOpenProgressNoteVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\");return false;",
                                siteCode, noteId, images, model.get('recordDetailsToken'), imageId, spinnerId);

                            return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id="' + spinnerId + '" style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);

                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 80,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        },
        data: JLV.PatientProgressNotesCollection
    },
    itemClicked: function (model, item) {

        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            }

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function (agree, auditText) {
                if (agree) {
                    self._handleDetails(model, item, auditText);
                }
            });

        } else {

            this._handleDetails(model, item);
        }
    },
    _handleDetails: function (model, item, auditText) {

        var auditUrl;

            if (model.get('noteAsBinaryStream')) {
                if (auditText) {
                    auditUrl = String.format("patientProgressNotes/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), auditText);
                } else {
                    auditUrl = String.format("patientProgressNotes/getNoteDataStream?patToken={0}&recToken={1}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));
                }

                var noteURL = auditUrl;
                if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {
                    JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                        if (download) {
                            JLV.common.openAuxBrowserWindow(noteURL, 'JLV_PROGRESS_NOTE');
                        }
                        else if (rb) {
                            this._addToReportBuilder(model, true, 'GetNoteDataStream');
                        }

                    }, this));
                }
                else {
                    JLV.common.openAuxBrowserWindow(noteURL, 'JLV_PROGRESS_NOTE');
                }
        }

            if (model.get('noteAsPlainText')) {
                auditUrl = 'patientProgressNotes/details.json';

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Notes ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('noteTitle')),
                    url: auditUrl,
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken'),
                        title: model.get('noteTitle'),
                        widgetName: this.name
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response, forPrint) {
                        var thumbnail = "";
                        if (JLV.common.isVixDisplayEnabled && model) {
                            var study = model.get('vixStudy');
                            if (study && study.viewerUrl) {
                                var series;
                                var viewerCmd =
                                    String.format('JLV.common.openAuxBrowserWindow(\'{0}\',\'JLV_VistAImagingViewer\');',
                                        study.viewerUrl);

                                if (study.series.length > 0) {
                                    series = study.series[0];

                                    if (forPrint) {
                                        thumbnail = '<div style="float:right;display: inline-block;">' +
                                            '<div>Associated image(s) not available for printing.</div>' +
                                            '</div>';

                                        response.images = '';
                                    } else {
                                        thumbnail = String.format(
                                            '<div style="float:right;display: inline-block;">' +
                                            '<a href="#" onClick="{0}">' +
                                            '<span style="position: relative;">' +
                                            '<img src="{1}" alt="{2}" title="{3}"/>' +
                                            '<div class="camera-icon">' +
                                            '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                            '</div>' +
                                            '</a></span>' +
                                            '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                            '</div>',
                                            viewerCmd, series.thumbnailUrl, series.caption, series.caption, viewerCmd);
                                    }
                                }
                            } else if (model.get('hasImage')) {
                                var siteCode = model.get('site').siteCode;
                                var noteId = model.get('noteId');
                                var images = model.get('images');

                                var uniqueId = noteId + siteCode;
                                var imageId = 'progressNoteDetailImage' + uniqueId;
                                var spinnerId = 'progressNoteDetailSpinner' + uniqueId;

                                var viewerCmd = String.format('JLV.common.getUrlAndOpenProgressNoteVixViewer(\'{0}\',\'{1}\',\'{2}\',\'{3}\',\'{4}\',\'{5}\');return false;',
                                    siteCode, noteId, images, model.get('recordDetailsToken'), imageId, spinnerId);

                                if (forPrint) {
                                    thumbnail = '<div style="float:right;display: inline-block;">' +
                                        '<div>Associated image(s) not available for printing.</div>' +
                                        '</div>';

                                    response.images = '';
                                } else {
                                    thumbnail = String.format(
                                        '<div id="' + imageId + '" style="float:right;display: inline-block;">' +
                                        '<a href="#" onClick="{0}">' +
                                        '<span style="position: relative;">' +
                                        '<img src="{1}" alt="{2}" title="{3}"/>' +
                                        '<div class="camera-icon">' +
                                        '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                        '</div>' +
                                        '</a></span>' +
                                        '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                        '</div><div id=' + spinnerId + ' style="display: none;float:right;"><img src="assets/loading.gif"/></div>',
                                        viewerCmd, "assets/thumbnail.png", model.get('noteTitle'), model.get('noteTitle'), viewerCmd);
                                }
                            }
                        }

                        return thumbnail + '<pre>' + response.noteText + response.images + '</pre>';
                    }
                });

            dialog.center();
            dialog.set('visible', true);

        }
    }

});


JLV.registerJLVWidget('progressnotes_widget', JLV.PatientProgressNotes);
