//= require ./PatientWidgetWindow.js

JLV.PatientAdmissionsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        width: 1000,
        name: 'Admissions',
        dateFilter: true,
        filter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['taxonomy', 'site'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Adm Date',
                id: 'admissionDate',
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken') &&
                        model.get('site').agency === JLV.common.AGENCY_VA) {
                        return JLV.common.linkDateFormatter(value, model);
                    }
                    else {
                        return JLV.common.dateFormatter(value);
                    }
                },
                sortable: true,
                width: 160
            },
            {
                title: 'Discharge Date',
                id: 'dischargeDate',
                width: 180,
                sortable: true,
                formatter: JLV.common.dateFormatter

            },
            {
                width: 120,
                title: 'Ward',
                sortable: true,
                id: 'ward'
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 140,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            // {
            //     title: 'Provider Specialty',
            //     id: 'taxonomy',
            //     width: 200,
            //     sorter: JLV.common.linkTextSorter,
            //     sortable: true,
            //     tooltip: function (row, model) {
            //         if (model) {
            //             var taxonomyCode = model.get('taxonomyCode'),
            //                 taxonomy = model.get('taxonomy');
            //
            //             if (taxonomyCode && taxonomy) {
            //                 return '<b>National Uniform Claim Committee -<br/>' +
            //                     'Taxonomy<br/>' +
            //                     '(NUCC)</b><br/>' +
            //                     'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
            //                     'NUCC Taxonomy Code: ' + taxonomyCode;
            //             }
            //
            //         }
            //
            //         return '<b>National Uniform Claim Committee -<br/>' +
            //             'Taxonomy<br/>' +
            //             '(NUCC)</b><br/>' +
            //             'NUCC Taxonomy Description: none<br/>' +
            //             'NUCC Taxonomy Code: none';
            //     }
            // },
            {
                title: 'Diagnosis',
                width: 200,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'diagnosisText',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized <br/>Diagnosis',
                width: 180,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            /*{
                width: 180,
                title: 'Discharge <br/> ICD Code',
                sortable: true,
                id: 'diagnosisCode'
             },*/
            {
                width: 160,
                title: 'Registration<br/>Number',
                sortable: true,
                id: 'registrationNum'
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 50,
                stretchable: false,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'admissionDate',
            asc: false
        }
    }
});

