JLV.PatientDemographicsWidgetWindow = AppKit.Dialog.extend({
    demographicsWidget: null,
    data: null,
    options: {
        height: 500,
        width: 700,
        title: 'Patient Demographics' 
    },
    initialize: function (options) {

        this._super(options);

        this.tabView = new AppKit.TabView({
            className: 'jlv-patient-demographics-details'
        });

        this.$el.append(this.tabView.$el);

        var tabsCount = options.data.length,
            i = 0;

        for (; i < tabsCount; i++) {

            var demographicData = options.data[i];
            var id = _.uniqueId('jlv-demographics-site-tab');


            $('<div class="jlv-demographics-details-view">')
                .data('index', "" + i)
                .attr('id', id)
                .appendTo(this.tabView.$el);

            this.tabView.addTab(JLV.common.formatSiteMonikerDisplay(demographicData.site), id, false);
        }
        // this.demographicsDetailsWindow.tabView.selectTabAtIndex(this._tabview.activeTab());
        this.tabView.selectTabAtIndex(options.demographicsWidget._tabview.activeTab());

        JLV.NotificationCenter.addObserverForNotification({
            name: 'closeNotification',
            observer: this,
            action: 'close'
        });

        var tools = $('<div>').addClass('jlv-window-tools');
        var self = this;

        this._reportBuilderBtn = $('<div title="Add to Report Builder" tabindex="0" class="jlv-window-toolbutton" style="padding-top:5px;">')
            .append('<div class="icon-plus">')
            .button()
            .on('click', function () {
                self.addToReportBuilder();
            }).appendTo(tools);
        tools.appendTo(this.$el.parent().find('.ui-dialog-titlebar-tools'));

        this.loadData();
    },
    addToReportBuilder: function () {
        var tabIndex = this.tabView.get('activeTab');
        var siteName = this.tabView.tabLabelAtIndex(tabIndex);
        var model = new Backbone.Model(this.data[tabIndex]);
        JLV.ReportBuilderController.addDemographic(model, siteName, true);
    },
    loadData: function () {

        this.tabView.$el.children('.jlv-demographics-details-view').html(
            '<div class="jlv-loader">' +
            '<div class="jlv-loader-image">' +
            '<div class="jlv-table-data-loading" alt="loading indicator"></div></div></div>');

        //disable add to report builder before loading
        this._reportBuilderBtn.button('option', 'disabled', true);
        this._reportBuilderBtn.attr('title', 'Adding to Report Builder is disabled until details load');


        //load the details data
        var self = this;
        $.post('patientDemographics/details.json', {
            patToken: JLV.ActivePatientModel.get('patientToken')
        }, function (records) {
            var count = records.length,
                i = 0,
                k = 0;
            for (; i < count; i++) {

                var data = records[i];
                var moniker = data.site.moniker;

                if (moniker !== JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {

                    var $detail = self.tabView.$el.children('.jlv-demographics-details-view').eq(k);

                    var demographics = self.data[i];
                    demographics.report = data.report;

                    $detail.empty();

                    $detail.append(
                        $('<div>').html('<div style="margin:15px;"><div style="text-decoration: underline;">Details:</div><br/><pre>' + _.text(data.report) + '</pre></div>')
                    );

                    if (self.demographicsWidget) {
                        var $s =
                            $(self.demographicsWidget._generatePatientDemographicsContent(self.data[k])).css({
                                position: 'relative',
                                'height': 'auto',
                                'overflow': 'hidden'
                            });

                        $s.find('.jlv-sponsor-link').remove();
                        $s.prependTo($detail);

                        $detail.find('button').remove();
                    }

                    k++;
                }
            }

            //data loaded: re-enable report builder button
            self._reportBuilderBtn.button('option', 'disabled', false);
            self._reportBuilderBtn.attr('title', 'Add to Report Builder');
        });
    }
});