//= require ./PatientWidgetWindow.js
//= require ./PatientAppointmentsWidgetWindow.js

JLV.PatientEncountersWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Outpatient Encounters',
        width: 950,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['clinic', 'taxonomy', 'site'],
        columns: [
            {
                title: 'Encounter<br/>Date',
                id: 'apptDate',
                sortable: true,
                width: 100,
                formatter: function (value, model) {

                    if (model && model.get('clinic') === 'Connection unavailable.') {
                        return JLV.common.dateFormatter(value);
                    }
                    if (model) {

                        if (model.get('complexDataUrl') || model.get('noteText') || model.get('vaNote')) {
                            var detailLink = "";
                            if("2.16.840.1.113883.3.42.10024.100001.13" != model.get("site").siteCode){
                                detailLink =  JLV.common.linkFormatter("Details", model);
                            }
                            var links = JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + detailLink;
                            var documents = model.get('documents');
                            if (documents && documents.length > 1) {
                                for (var i = 0; i < documents.length; i++) {
                                    links += ' ' + JLV.common.linkFormatter("Note" + (i + 1), model);
                                }
                            } else {
                                links += ' ' + JLV.common.linkFormatter("Note", model);
                            }
                            return links;
                        } else {
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model) + ' ' + "Note"
                        }


                    }
                    return value;
                }
            },
            {
                title: 'Clinic',
                id: 'clinic',
                width: 90,
                sortable: true,
                sorter: JLV.common.linkTextSorter

            },
            {
                width: 75,
                title: 'Status',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'status'
            },
            {
                width: 90,
                title: 'Type',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'apptType'

            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 90,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider Specialty',
                id: 'taxonomy',
                width: 160,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Reason',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'reason'
            },
            {
                title: 'Diagnosis',
                width: 100,
                id: 'diagnosisList',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized Diagnosis',
                width: 180,
                id: 'normDisplay',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var documents = model.get('documents');
                        if (documents && documents.length > 0) {
                            var study = documents[0].vixStudy;
                            if (study && study.viewerUrl) {
                                var viewerUrl = study.viewerUrl;
                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var viewerCmd = String.format("JLV.common.openEncounterVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));

                                return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                            } else if (model.get('hasImage')) {  //encounter has image
                                var imageDocIndex = -1;
                                for (var i = 0; i < documents.length; i++) {
                                    if (documents[i].hasImage) {
                                        imageDocIndex = i;
                                        break;
                                    }
                                }

                                if (imageDocIndex >= 0) {
                                    var document = documents[imageDocIndex];
                                    var documentId = document.id;
                                    var documentSiteId = model.get('site').siteCode;

                                    if (documentId && documentSiteId) {
                                        if (forPrint) {
                                            return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                        }

                                        var uniqueId = documentId + documentSiteId;
                                        var imageId = 'encounterWindowImage' + uniqueId;
                                        var spinnerId = 'encounterWindowSpinner' + uniqueId;

                                        var viewerCmd = String.format("JLV.common.getUrlAndOpenEncounterVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\");return false;",
                                            documentId, documentSiteId, model.get('recordDetailsToken'), imageId, spinnerId);

                                        return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                            + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id="' + spinnerId + '" style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                                    }
                                }
                            }

                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'appointmentDate',
            asc: false
        }

    }
});