//= require ./PatientWidgetWindow.js

JLV.PatientFlagsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: '<span><div title="Patient Flags and Postings icon" id="flags_icon"> </div>Patient Flags and Postings</span>',
        width: 600,
        filter: true,
        dateFilter: false,
        status: false,
        clipboard: true,
        print: true,
        refresh: false,
        fitColumns: false,
        settings: false,
        filters: ['site'],
        columns: [
            {
                title: 'Message',
                id: 'name',
                sortable: true,
                width: 400,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (value == undefined || value == null || value == "") {
                        value = "Flag name was not supplied"
                    }
                    if ((model.get("content") && model.get("content").length > 0) || (model.get("comment") && model.get("comment").length > 0)) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    return value;

                },
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'name',
            asc: true
        }
    },
    itemClicked: function (model) {

        var dialog = new JLV.DetailsWindow({
            height: 300,
            width: 500,
            title: String.format("Flag Details ~ {0} {1}",
                JLV.common.formatSiteMonikerDisplay(model.get('site')),
                model.get('name')),
            model: model,
            widget: this,
            reportBuilder: true,
            renderer: function () {
                try {
                    var detailName = 'Assignment Narrative';
                    var detailField = 'content';

                    if (model.get('site') != null && model.get('site').agency == JLV.common.AGENCY_DOD) {
                        detailName = 'Comment';
                        detailField = 'comment';
                    }

                    return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                        '<td>Flag Name:</td><td>{{name}}</td></tr><tr>' +
                        '<td>' + detailName + ':</td><td>{{' + detailField + '}}</td></tr><tr>' +
                        '</table>')(model.toJSON());
                } catch (e) {
                    console.info('template error');
                }

            }
        });

        dialog.center();
        dialog.open();

    },
    doClipboard: function () {

        if (this._datagrid) {
            var clone = this._datagrid.$el.clone();
            clone.find('.appkit-table-column, .ui-datatable-loading-rows, .dummy-row-1, .dummy-column, .hidden').remove();
            JLV.common.copyGridPanelHTML(clone.html());
        }

    },
    initialize: function (options) {

        var self = this;
        JLV.NotificationCenter.addObserverForNotification({
            name: 'JLV.PatientFlagsWidgetWindow.Update',
            action: function () {
                self.set('_data', new Backbone.Collection(options.data));
                self.loadData();
                self.savePortalUI();
            }
        });

        this._super(options);
    },

    _setup: function () {

        this._super();

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            this.options.defaultSort = {
                asc: sortInfo.asc,
                id: sortInfo.column.id
            };

            this.$el.trigger('sortChange', [this.options.defaultSort]);
        }, this));

        var dg = this._datagrid;
        var self = this;
        this._datagrid.on('afterRender.click', function () {
            this.$el.find('.jlv-clickable-link').on('click', function () {
                var modelId = $(this).data('model-id'),
                    model = dg.get('source').get(modelId);

                self.itemClicked(model, $(this));
            });
        });
        this.$el.find('.jlv-show-all-btn').hide();

        this.toggleFilterPanel();
    },

    /* loadData: function () {

        //load the data
        var data = this.get('data'),
            patient = this.get('patient');

        if (!data) {
            console.warn(this.get('name') + ' widget data not set.');
        }

        if (!patient) {
            console.warn(this.get('name') + ' widget patient not set.');
        }

        if (data &&
            patient) {

            //if (!_.isFunction(data.doFetch)) {
            //    console.warn('Widget data sources must be a Backbone collection implementing a doFetch method.');
            //    return;
            //}

            //data.reset();

            this.set('title', this.get('name'));
            this.dodAdapterSourceReport = [];
            this.dodAdapterStatusRec = [];
            this._updatePageInfoDisplay();

            this._datagrid.showMessage('<div class="jlv-table-data-loading"></div>');

            var self = this;
            //data.doFetch(patient, function () {

            if (self._datagrid.get('rowCount') === 0) {
                self._datagrid.showMessage("No Data");
            }
            else if (self._defaultSort !== null) {

                var column = _.findWhere(self.options.columns, {id: self._defaultSort.id});
                if (column) {
                    column.index = _.indexOf(self.options.columns, column);
                    var sortInfo = {
                        asc: self._defaultSort.asc,
                        column: column
                    };

                    self._datagrid.sortByColumn(sortInfo);
                    self._datagrid.render();


                }
                else {
                    self._datagrid.render();
                }
            }
            else {
                self._datagrid.render();

            }
            //update the widget title to reflect the number of records
            self.set('title', self.get('name') + ' (' + self._datagrid.get('rowCount') + ')');

            self._updatePageInfoDisplay();
        }
        //},
        //// handle error
        //function (collection, err) {
        //
        //    console.info(err)
        //
        //
        //    self._datagrid.showMessage(_.template(
        //            '<div class="server-error-msg" style="color:red;cursor:pointer;text-decoration:underline;">{{statusText}} ({{status}})</div>')(err)
        //    );
        //
        //    self._datagrid.$el.find('.server-error-msg').click(function () {
        //        var alertDialog = new AppKit.AlertDialog({
        //            title: err.statusText,
        //            message: err.responseText,
        //            icon: 'icon-frown-o',
        //            cancelButton: false,
        //            height: 500
        //        });
        //
        //        alertDialog.center();
        //        alertDialog.set('visible', true);
        //    });
        //});
        //}
     }, */
    _checkConnectionUnavailable: function () {

        //do nothing

    }
});

