//= require ./PatientWidgetWindow.js
//= require ./LabResultsGraphWindow.js



JLV.PatientLabResultsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Lab Results',
        width: 1000,
        minWidth: 800,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        gridHeaderHeight: 30,
        fitColumns: false,
        filters: ['testName', 'normDisplay', 'accessionType', 'site'],
        columns: [
            {
                title: 'Collection Date',
                id: 'collectionDate',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                },
                sortable: true,
                width: 120
            },
            {
                title: 'Order Number',
                id: 'orderId',
                sortable: true,
                width: 80
            },
            {
                title: 'Status',
                id: 'resultStatus',
                sortable: true,
                width: 80
            },
            {
                width: 120,
                title: 'Ordering HCP',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'providerName',
                tooltip: function (row, model) {

                    if (model) {
                        try {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description:' +
                                '{[ if (obj.taxonomy) { ]}' +
                                '{{-taxonomy}}' +
                                '{[ } ]}' +
                                '<br/>NUCC Taxonomy Code: ' +
                                '{[ if (obj.taxonomyCode) { ]}' +
                                '{{-taxonomyCode}}' +
                                '{[ } ]}' +
                                '<br/>Requesting Location: {{-requestingLocation}}' //+
                                //'<b>Provider Address: </b>{{-provider.street1}} {{-provider.street2}} {{-provider.street3}}' +
                                //'{{-provider.city}}, {{-provider.stateProvince}} {{-provider.postalCode}} <br/>' +
                                // '<b>Provider Phone: </b> {{-provider.phone}}'
                            )(model.toJSON());
                        } catch (e) {
                            console.info('template error');
                        }
                    }
                }
            },
            {
                width: 120,
                title: 'Ordering HCP <br/> Specialty',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'taxonomy',
                tooltip: function (row, model) {

                    if (model) {
                        try {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description:' +
                                '{[ if (obj.taxonomy) { ]}' +
                                '{{-taxonomy}}' +
                                '{[ } ]}' +
                                '<br/>NUCC Taxonomy Code: ' +
                                '{[ if (obj.taxonomyCode) { ]}' +
                                '{{-taxonomyCode}}' +
                                '{[ } ]}' +
                                '<br/>Requesting Location: {{-requestingLocation}}' //+
                                //'<b>Provider Address: </b>{{-provider.street1}} {{-provider.street2}} {{-provider.street3}}' +
                                //'{{-provider.city}}, {{-provider.stateProvince}} {{-provider.postalCode}} <br/>' +
                                // '<b>Provider Phone: </b> {{-provider.phone}}'
                            )(model.toJSON());
                        } catch (e) {
                            console.info('template error');
                        }
                    }
                }
            },
            {
                title: 'Lab Test',
                id: 'testName',
                width: 110,
                sortable: true,
                formatter: function (value, model) {

                    if (model && model.get('interpretation')) {
                        if(model.get('interpretation').toLowerCase().indexOf("critical") > -1
                            || model.get('interpretation').toLowerCase().indexOf("*") > -1){
                            return '<div class="jlv-high-alert-row">' + JLV.common.linkFormatter(value, model) + '</div>';
                        }
                        return '<div class="jlv-alert-row">' + JLV.common.linkFormatter(value, model) + '</div>';
                    }

                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes<br/>' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized Lab<br/> Test',
                width: 150,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes<br/>' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 50,
                title: 'Type',
                sortable: true,
                id: 'accessionType',
                formatter: function (value, model) {
                    //if model doesn't exist, it is filter setup that is calling this formatter
                    if (!model) {
                        var testLabel = JLV.common.labLabels[value];

                        if (testLabel) {
                            return value + " - " + testLabel;
                        }
                    }
                    return value;


                },
                tooltip: function (row, model) {
                    if (model) {
                        var testLabel = model.get('testLabel');

                        if (testLabel) {
                            return '<b>' + testLabel + '<br/>';
                        }
                    }
                }
            },
            {
                width: 70,
                title: 'Specimen<br/>Source',
                sortable: true,
                id: 'specimen',
            },
            {
                title: 'Result',
                id: 'result',
                width: 60,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if( model.get('accessionType') == "MICRO"
                        || model.get('accessionType') == "MI"
                        || model.get('accessionType') == "CY"
                        || model.get('accessionType') == "SP"
                        || model.get('accessionType') == "ST"
                        || model.get('accessionType') == "BB"
                        || model.get('accessionType') == "AP"
                        || model.get('accessionType') == "AU"){
                        return JLV.common.linkFormatter("result", model);
                    } else if (!isNaN(parseFloat(value)) && model.get('units')) {
                        return value + " " + model.get('units')
                    }

                    return value;
                }
            },
            /*{
                title: 'Abnormal <br/> Indicators',
                id: 'abnormalIndicators',
                width: 120,
                sortable: true
             },*/
            {
                title: 'Interpretation',
                id: 'interpretation',
                width: 120,
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('interpretationDetail') && model.get('interpretationDetail').length > 0) {
                        if(value){
                            return value + " - " + JLV.common.linkFormatter("Interpretation Available", model);
                        } else {
                            return JLV.common.linkFormatter("Interpretation Available", model);
                        }

                    } else {
                        return value;
                    }

                }
            },
            {
                width: 60,
                title: 'Units',
                sortable: true,
                id: 'units'
            },
            {
                width: 70,
                title: 'Ref Range',
                sortable: true,
                id: 'referenceRange'
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 50,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'resultDate',
            asc: false
        }
    },
    toggleFilterPanel: function () {
        if (!this._filterPanel) {

            var filterIds = this.get('filters');
            var filters = _.toArray(_.pick(_.indexBy(this.get('columns'), 'id'), filterIds));

            var data = this.get('data');

            if (data) {


                if (data.startDate !== undefined && data.endDate !== undefined) {
                    this._dateLabel.show();
                    this._filterPanel = new JLV.PatientLabResultsWidgetWindow.FilterPanel({
                        window: this,
                        startDate: data.startDate.toDate(),
                        endDate: data.endDate.toDate(),
                        filters: filters,
                        data: data
                    });

                    this._filterPanel.$el.prependTo(this._body);
                    this._filterPanel.$el.hide();
                }
            }
        }

        this._super();
    },

    _setup: function () {
        this._super();

        if (this._datagrid) {
            //make rows with abnormal results pink
            this._datagrid.on('afterRender', function () {

                var $row = this.$el.find('.jlv-alert-row').parents('tr');

                $row.css(
                    {
                        'backgroundColor': '#FFC0CB',
                        'color': '#000000'
                    }
                );

                $row.find('a').css('color', '#000');

                var $cRow = this.$el.find('.jlv-high-alert-row').parents('tr');
                $cRow.css(
                    {
                        'backgroundColor': 'red',
                        'color': '#FFFFFF'
                    }
                );
                $cRow.find('a').css('color', '#FFF');
            });
        }


    }
});


jQuery(function () {

    JLV.PatientLabResultsWidgetWindow.FilterPanel = JLV.WidgetWindow.DateFilterPanel.extend({
        initialize: function (options) {

            this._super(options);

            //add table button
            this._tableButton = new JLV.PatientWidgetWindow.LinkButton({
                label: "Show Table",
                enabled: false,
                linkDisabledMessage: 'Enable “Show Table” by setting one item from either of the "Filter by Lab Test" or "Filter by Standardized Lab Test" filters.',
                action: _.bind(function () {
                    //generate table html
                    if (this.window) {

                        var labs = this.subview('.first-combo').get('selected'),
                            lab = labs[0];

                        var stdLabs = this.subview('.second-combo').get('selected'),
                            stdLab = stdLabs[0];

                        var data = this.window.get('datagrid').tableData();
                        var i = 0,
                            count = data.length;

                        var html = '<div></div><table class="jlv-filtered-table-view"><tr><th>Date</th>' +
                            '<th>Test</th>' +
                            '<th>Specimen Source</th>' +
                            '<th>Result</th>' +
                            '<th>Units</th>' +
                            '<th>Interpretation</th>' +
                            //'<th>Quantity</th>' +
                            '<th>Reference Range</th>' +
                            '<th>Site</th>' +
                            '</tr>';

                        var itemCount = 0;
                        var units = '';
                        for (; i < count; i++) {
                            var model = data[i];
                            var interpretation = model.get('interpretation');

                            var bgcolor = "#ffffff",
                                fgcolor = '#000000';
                            if (interpretation && interpretation.length) {

                                bgcolor = '#FFC0CB';

                                if (interpretation.toLowerCase().indexOf("critical") > -1
                                    || interpretation.toLowerCase().indexOf("*") > -1) {
                                    bgcolor = '#901100';
                                    fgcolor = '#ffffff';
                                }
                            }

                            html += '<tr style="background:' + bgcolor + ';color:' + fgcolor + '">';
                            html += '<td>' + JLV.common.dateFormatter(model.get('resultDate')) + '</td>';
                            html += '<td>' + model.get('testName') + '</td>';
                            html += '<td>' + model.get('specimen') + '</td>';
                            html += '<td>' + model.get('result') + '</td>';
                            html += '<td>' + model.get('units') + '</td>';
                            html += '<td>' + model.get('interpretation') + '</td>';
                            //html += '<td>' + model.get('quantity') + '</td>';
                            html += '<td>' + model.get('referenceRange') + '</td>';
                            html += '<td>' + JLV.common.formatMonikerDisplay(model.get('site')) + '</td>';
                            html += '</tr>';

                            itemCount++;
                        }

                        html += '</table>';

                        //key
                        html += "<table style='font-size:11px;'><tr><td style='padding:10px;'><span>L=Low | H=High | *=Critical | R=Resist | S=Susc | MS=Mod Susc | I=Intermed | []=Uncert <br/>/A=Amended | Comments=  (O)rder, (I)nterpretations, (R)esult</span></td></tr></table></div>";


                        var tableDialog = new JLV.DetailsWindow({
                            title: String.format("Lab Results - {0} ({1})", lab ? lab : stdLab, itemCount),
                            width: 850,
                            height: 500,
                            renderer: function () {
                                return html;
                            }
                        });

                        tableDialog.center();
                        tableDialog.open();
                    }

                }, this)
            });

            //add graph button
            this._graphButton = new JLV.PatientWidgetWindow.LinkButton({
                label: "Show Graph",
                linkDisabledMessage: 'Enable “Show Graph” by setting one item from the "Filter by Lab Test" filter.',
                enabled: false,
                action: _.bind(function () {

                    var labs = this.subview('.first-combo').get('selected'),
                        lab = labs[0];
                    var dataArray = this.window.get('datagrid').tableData();

                    if (dataArray.length) {
                        var graphDialog = new JLV.LabResultsGraphWindow({
                            title: String.format("Lab Results Graph - {0}", lab),
                            data: dataArray,
                            units: dataArray[0].get('units'),
                            type: lab
                        });

                        graphDialog.center();
                        graphDialog.open();
                    }


                }, this)
            });

            $("<div>").addClass('field-filter-buttons')
                .append(this._graphButton.$el)
                .append(this._tableButton.$el)
                .appendTo(this.$el);


        },
        clearFilters: function (skip) {

            if (this._graphButton) {
                this._graphButton.set('enabled', false);
            }

            if (this._tableButton) {
                this._tableButton.set('enabled', false);
            }

            this._super(skip);
        },
        applyFilters: function () {
            this._super();

            var firstCombo = this.subview('.first-combo'),
                firstSelected = firstCombo.get('selected');

            if (firstSelected.length === 1) {
                this._tableButton.set('enabled', true);

                if (this.window) {
                    var dataArray = this.window.get('datagrid').tableData();
                    if (dataArray.length) {
                        var model = dataArray[0];
                        var result = model.get('result');

                        this._graphButton.set('enabled', !_.isNaN(parseInt(result, 10)));
                    }
                }
            }
            else {
                this._tableButton.set('enabled', false);
                this._graphButton.set('enabled', false);
            }

            var secondCombo = this.subview('.second-combo'),
                secondSelected = secondCombo.get('selected');

            if (secondSelected.length === 1) {
                this._tableButton.set('enabled', true);
            }
            else if (firstSelected.length !== 1) {
                this._tableButton.set('enabled', false);
            }
        }
    });
});