//= require ./PatientWidgetWindow.js

JLV.PatientProblemListWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Problem List',
        filter: true,
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        width: 900,
        filters: ['description', 'normDisplay', 'status', 'type', 'site'],
        columns: [
            {
                title: 'Updated Date',
                width: 100,
                stretchable: false,
                id: 'lastUpdated',
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Onset',
                width: 70,
                stretchable: false,
                id: 'onsetDate',
                formatter: function (aDate, model) {
                    return JLV.common.dateFormatter(model.get('dateTime'));
                }
            },
            {
                title: 'Problem Description',
                width: 140,
                id: 'description',
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'Standardized Description',
                width: 160,
                id: 'normDisplay',
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br/>' +
                                'of Medicine - Clinical Terms<br/>' +
                                '(SNOMED CT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br/>' +
                        'of Medicine - Clinical Terms<br/>' +
                        '(SNOMED CT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'ICD',
                width: 52,
                stretchable: false,
                id: 'icdCode'
            },
            {
                title: 'Severity',
                width: 100,
                sortable: true,
                id: 'priority'
            },
            {
                title: 'Type',
                id: 'type',
                width: 130,
                filter: function (model) {
                    var site = model.get('site');
                    if (site.agency === 'VA')
                        return true;
                }
            },
            {
                title: 'Status',
                width: 60,
                stretchable: false,
                id: 'status'
            },
            {
                title: 'Site',
                width: 80,
                stretchable: false,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastUpdated',
            asc: false
        },
        defaultFilter: {
            id: 'status',
            value: 'ACTIVE'
        }
    },

    _setup: function () {

        this._super();

        /* add blue text to type filter dropdown */
        $('<div>').css({
            fontSize: 11,
            color: '#6998c3'
        }).html('Filter applies to DoD data only')
            .appendTo(this.$el.find('.fourth-filter').find('.ui-button'));


        /*add hover-over to type column header */

        setTimeout(_.bind(function () {
            var $h = this.$el.find('.ui-table-header').find('[data-col="7"]');
            $h.attr("title", "");
            $h.tooltip({
                content: '<div>Major Problems (Diagnoses / Chronic Problems) <br/> Minor Problems (Symptoms / Acute Problems) <br/> Administrative</div>',
                show: {
                    delay: 1100,
                    duration: 0,
                    easing: false,
                    effect: null
                },
                hide: false,
                position: {my: "center top", at: "center bottom+5"}

            });
        }, this), 500);


    },


    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.open();
        }
    }
});
