//= require ../../WidgetWindow.js

JLV.PatientWidgetWindow = JLV.WidgetWindow.extend({
    _setup: function () {

        this._super();


        var self = this;

        if (this._datagrid) {
            //add report builder functionality
            this._datagrid.on('afterRender', function () {

                self.checkConnectionUnavailable();

                //508 keyboard access to add record to RB
                this.$el.find('tr').on('keydown', function (evt) {
                    
                    if (evt.which === 82 && evt.ctrlKey) { //ctrl-R
                        evt.preventDefault();

                        var $row = $(this);
                        if ($row.text().indexOf('Connection unavailable') == -1) {
                            var modelId = $row.find('td').data('model-id'),
                                model = self._datagrid.get('source').get(modelId).clone();

                            model.set('_name', self.get('name'));
                            var columns = self.options.columns;

                            if (self.widget) {
                                model.set('_name', self.widget.get('name'));
                                columns = self.widget.options.columns;
                            }

                            JLV.ReportBuilderController.addRecord(model, self.widget.options.columns, columns);
                        }

                    }

                });
                this.$el.find('tr').on('contextmenu', function (evt) {

                    var $row = $(this);

                    if ($row.text().indexOf('Connection unavailable') == -1) {
                        var modelId = $row.find('td').data('model-id');
                        var model;
                        if (modelId) {
                            model = self._datagrid.get('source').get(modelId).clone();


                            evt.preventDefault();

                            var menu = new AppKit.Menu({
                                menuItems: [
                                    new AppKit.MenuItem({
                                        label: 'Add Record to Report',
                                        action: function () {
                                            model.set('_name', self.get('name'));
                                            var columns = self.options.columns;
                                            if (self.widget) {
                                                model.set('_name', self.widget.get('name'));
                                                columns = self.widget.options.columns;
                                            }
                                            JLV.ReportBuilderController.addRecord(model, columns, self.options.columns);
                                        }
                                    })
                                ]
                            });

                            menu.open({
                                left: evt.clientX - 30,
                                top: evt.clientY
                            });

                            menu.on('close', function () {
                                setTimeout(_.bind(function () {
                                    this.$el.remove();
                                }, this), 0);
                            });
                        }
                    }

                });

            });
        }

    },


    checkConnectionUnavailable: function () {

        var collection = this.get('data'),
            errors = collection.connectionErrors;


        this.addConnectionUnavailableBanner((errors && Object.keys(errors).length > 0)
            || (JLV.Session.dodVLERFail && this.options.vlerDomain)
            || (JLV.Session.dasVLERFail && this.options.dasvlerDomain));
    },


    addConnectionUnavailableBanner: function (flag) {

        if (flag === undefined) {
            flag = true;
        }

        this.set('statusAlert', flag);

        if (flag) {
            this.addBanner({
                id: 'connUnavailableStatus',
                bgcolor: 'yellow',
                fgcolor: '#111',
                closable: true,
                text: 'One or more data sources could not be connected  <br/> and some entries will not appear',
                height: 34
            });
        }
        else {
            this.clearBanner('connUnavailableStatus');
        }

    },


    doPrint: function () {

        if (this._displayMode !== 'all') {
            if (this._datagrid) {
                var data = this._datagrid.get('tableData'),
                    patient = JLV.ActivePatientModel;


                if (data.length && patient) {

                    var columns = this.get('columns');
                    var hiddenColumns = this._datagrid.get('hiddenColumns');
                    var visibleColumns = [],
                        formatters = {};

                    var l = columns.length, i = 0;

                    for (; i < l; i++) {

                        var column = columns[i]
                        if (hiddenColumns.indexOf(column.id) < 0) {
                            visibleColumns.push({
                                id: column.id,
                                title: column.title
                            });

                            formatters[column.id] = column.formatter;
                        }
                    }

                    var data = this._datagrid.get('tableData');
                    //flatted grouped data
                    if (!_.isArray(data)) {
                        var groups = data.data;
                        l = groups.length;
                        i = 0;

                        data = [];
                        for (; i < l; i++) {
                            var group = groups[i];
                            for (var key in group) {
                                data = _.union(data, group[key])
                            }
                        }
                    }

                    l = data.length;
                    i = 0;

                    var toPrint = [];
                    for (; i < l; i++) {
                        var model = data[i].clone();

                        var vcl = visibleColumns.length,
                            j = 0;

                        for (; j < vcl; j++) {
                            var vc = visibleColumns[j];
                            var fvalue = model.get(vc.id);

                            if (_.isString(fvalue)) {
                                fvalue = _.escape(fvalue);
                            }

                            if (_.isFunction(formatters[vc.id])) {

                                fvalue = formatters[vc.id](
                                    fvalue, model, true
                                );
                            }

                            model.set(vc.id, fvalue);
                        }

                        toPrint.push(model.toJSON());
                    }

                    var title = this.get('name');

                    $.post('print/setup', {
                        patToken: patient.get('patientToken'),
                        data: JSON.stringify({
                            title: title,
                            columns: visibleColumns,
                            data: toPrint,
                            patient: patient.toJSON(),
                            datetime: moment().format('MMMM DD, YYYY h:mm a')
                        })
                    }, function (response) {

                        var jobID = response;

                        JLV.Widget.PrintConfirm(function (ok) {

                            if (ok) {
                                var printUrl = 'print/table';
                                printUrl += ('?jobId=' + jobID);
                                printUrl += ('&patToken=' + patient.get('patientToken'));
                                window.open(printUrl, '_blank');
                            }
                        });

                    });


                }
            }
        }

    }
});

JLV.PatientWidgetWindow.LinkButton = AppKit.Button.extend({
    className: 'jlv-filter-field-button',
    enabled: function (isEnabled) {

        this._super(isEnabled);

        if (isEnabled) {
            this.$el.attr('title', '');
        }
        else {
            this.$el.attr('title', this.get('linkDisabledMessage'));
        }
    }
});
