/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.AppConfig
import gov.va.med.common.SSOiUtil
import gov.va.med.foundations.utilities.EncryptUtil
import gov.va.med.groovy.user.AppUser
import gov.va.med.guibeans.GUIAuditEntry
import gov.va.med.guibeans.GUISite
import gov.va.med.guibeans.GUIUser
import gov.va.med.jmeadows.webservice.IehrUserProfile
import grails.converters.JSON
import grails.util.Metadata
import groovy.json.JsonBuilder
import groovy.json.StringEscapeUtils
import org.apache.commons.collections.CollectionUtils
import org.apache.commons.lang3.StringUtils
import org.apache.commons.validator.routines.UrlValidator
import org.codehaus.groovy.grails.web.json.JSONObject

import javax.xml.soap.SOAPException

class AuthenticateController extends BaseController {
    def appConfig
    def userDao
    def heartbeatDao
    def tokenUtils
    def samlTokenService
    def ssoiStsService

    def login = { LoginCommand loginCmd ->

        //populate stuff
        def IehrUserProfile userProfile
        def List<GUISite> sites
        def List<GUIAuditEntry> loginEntries
        def String validLoginDate = ""
        def String validLoginIP = ""
        def String badLoginDate = ""
        def String badLoginIP = ""
        def String cfg = ""
        def String theme = "default"
        def String dodVLER = "false"
        def String dasVLER = "false"
        def String loginAgency = ""
        def String loginSite = ""
        def jsonCfg = null
        def String agreement = (StringUtils.equalsIgnoreCase(session.acceptedAgreement, 'true')) ? "true" : "false"
        def String clientIP = request.remoteAddr;
        def subjectDN = ""
        def certificate = ""

        //generate and set csrf token
        def csrfToken = UUID.randomUUID().toString()
        session.setAttribute("X-CSRF-Token", csrfToken)

        // Service Statuses
        def serviceStatus = heartbeatDao.getServicesStatus();

        if (session.subjectDN != null) {
            subjectDN = session.subjectDN
        }
        if (session.certificate != null) {
            certificate = session.certificate
        }

        if (session.userProfile != null) {
            userProfile = session.userProfile
            //prep the profile config items for profile window
            cfg = userProfile.cfg
            if (cfg != null && cfg.length() > 0) {
                jsonCfg = JSON.parse(cfg)
                for (item in jsonCfg) {
                    if (item.cfgItem == "UI_CONF") {
                        for (option in item.options) {
                            if (option.theme) {
                                theme = option.theme
                            } else if (option.loginAgency) {
                                loginAgency = option.loginAgency
                            } else if (option.loginSite) {
                                loginSite = option.loginSite
                            } else if (option.dodVLER == true && ("DOD".equals(loginAgency) || (StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT")))) {
                                dodVLER = option.dodVLER
                            } else if (option.dasVLER == true && ("VA".equals(loginAgency) || (StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT")))) {
                                dasVLER = option.dasVLER
                            }

                        }
                    }
                }
            }
            if (loginAgency == "" && userProfile.getSmartCardAgency() != "")
                loginAgency = userProfile.getSmartCardAgency()
            if (loginSite == "" && userProfile.getLocID() != null)
                loginSite = userProfile.getLocID()
        }

        if (session.sites == null) {
            try {
                sites = userDao.getSites()
                if (sites != null) {
                    Iterator<GUISite> i = sites.iterator()
                    while (i.hasNext()) {
                        GUISite site = i.next()
                        if (Metadata.current.getApplicationName() == "JLV_Claims") {
                            // for the JLV_Claims configuration, only keep VBA site in the list
                            if (site.moniker != "VBA") {
                                i.remove()
                            } else {
                                continue;
                            }
                        } else {
                            // for all other configurations, only remove sites that have no endpoint, or are inactive
                            if (site.endpoints == null || site.endpoints.size() == 0 || site.status != "active") {
                                i.remove()
                            }
                            if (site.moniker == "VBA" && appConfig.getRemoveVbaSite()) {
                                i.remove()
                            }
                        }
                    }
                    session.sites = sites
                }
            }
            catch (Exception e) {
                log.error("AuthenticateController::login - Unable to retrieve site list", e)
                String errorText = "There's an error retrieving the sites list.<br/>Please notify the system administrator";
                render(view: "login", model: [errorText: errorText, grailsConfig: grailsApplication.config.grails, "userProfile": userProfile, "theme": theme, "loginAgency": loginAgency, "loginSite": loginSite, "serviceStatus": serviceStatus, "csrfToken": csrfToken])
                return
            }
        } else {
            sites = session.sites
        }

        //DEBUG data sent to login page
        log.debug(new JsonBuilder(userProfile).toPrettyString())
        log.debug(new JsonBuilder(sites).toPrettyString())

        if (request.method == "POST") {

            if (!loginCmd.hasErrors()) {
                String errorText = "Failed to Login - Error contacting the server.<br/>Please notify the system administrator";

                loginAgency = StringEscapeUtils.escapeJavaScript(loginCmd.loginAgency)
                loginSite = StringEscapeUtils.escapeJavaScript(loginCmd.siteCode)
                agreement = "true".equalsIgnoreCase(loginCmd.acceptedAgreement)

                def loginCfg = loginCmd.userCfg
                def loginAC = loginCmd.accessCode
                def loginVC = loginCmd.verifyCode

                if (session.userProfile != null) {
                    userProfile = session.userProfile;
                    //userProfile.setSmartCardAgency(loginAgency);
                    userProfile.setLocID(loginSite);

                    //update userCfg with loginAgency and loginSiteCode
                    if (loginCfg != null && loginCfg.length() > 0) {
                        boolean foundUIConf = false
                        boolean foundLoginAgency = false
                        boolean foundLoginSite = false

                        jsonCfg = JSON.parse(loginCfg)

                        for (item in jsonCfg) {
                            if (item.cfgItem == "UI_CONF") {
                                foundUIConf = true
                                for (option in item.options) {
                                    if (option.theme) {
                                        theme = option.theme
                                    }
                                    if (option.loginAgency) {
                                        option.loginAgency = loginAgency
                                        foundLoginAgency = true
                                    } else if (option.loginSite) {
                                        option.loginSite = loginSite
                                        foundLoginSite = true
                                    }
                                }
                                if (!foundLoginAgency)
                                    item.options.add(new JSONObject("{loginAgency:" + loginAgency + "}"))
                                if (!foundLoginSite)
                                    item.options.add(new JSONObject("{loginSite:" + loginSite + "}"))
                            }
                        }
                        if (!foundUIConf) {
                            jsonCfg.add(new JSONObject("{cfgItem:UI_CONF,options:[{dodVLER : " + dodVLER + "},{dasVLER : " + dasVLER + "},{theme:" + theme + "},{loginSite:" + loginSite + "},{loginAgency:" + loginAgency + "}]}"))
                        }
                        userProfile.setCfg(jsonCfg.toString())
                    }

                    if (userProfile.getFlags() == null) {
                        userProfile.setFlags("");
                    }
                }

                try {
                    def user = userDao.login(userProfile.smartCardID, loginAgency, loginSite, loginAC, loginVC, clientIP, subjectDN, userProfile.emailAddress)

                    //check if there was a problem logging in user
                    if (user == null ||
                            user.getUserIen() == null ||
                            user.getUserIen().isEmpty() ||
                            user.getUserIen().contains("^")) {
                        if (user != null && user.getUserIen() != null) {
                            String userIen = user.getUserIen();

                            if (userIen.contains("^")) {
                                errorText = userIen.substring(userIen.indexOf("^") + 1)
                            }
                        }

                        try {
                            loginEntries = userDao.getLoginInfo(userProfile.smartCardID, userProfile.emailAddress);
                            if (loginEntries != null) {
                                Iterator<GUIAuditEntry> i = loginEntries.iterator()
                                while (i.hasNext()) {
                                    GUIAuditEntry loginEntry = i.next()
                                    if (loginEntry.userID.startsWith("^")) {
                                        badLoginDate = loginEntry.entryDate;
                                        badLoginIP = loginEntry.ipAddress;
                                    } else {
                                        validLoginDate = loginEntry.entryDate;
                                        validLoginIP = loginEntry.ipAddress;
                                    }
                                }
                            }
                        }
                        catch (Exception e) {
                            log.error("AuthenticateController::login - Unable to retrieve previous login information", e)
                        }

                        render(view: "login", model: [errorText     : errorText, grailsConfig: grailsApplication.config.grails, "dodVLER": dodVLER, "dasVLER": dasVLER, "sites": sites, "userProfile": userProfile, "theme": theme, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
                                                      "badLoginDate": badLoginDate, "badLoginIP": badLoginIP, "validLoginDate": validLoginDate, "validLoginIP": validLoginIP, "serviceStatus": serviceStatus, "csrfToken": csrfToken])
                        return
                    } else {

                        if (appConfig.getDodSAMLTokenEnabled()) {
                            user.setSamlToken(samlTokenService.createSAMLToken(user))
                        }

                        AppUser appUser = new AppUser(user)

                        //save profile
                        try {
                            if (userDao.setIehrUserProfile(userProfile)) {
                                if (userProfile.dbUserID == "") {
                                    if (StringUtils.isNotEmpty(userProfile.emailAddress)) {
                                        userProfile = userDao.getIehrUserProfile(userProfile.smartCardID, userProfile.smartCardAgency, userProfile.emailAddress)
                                    } else {
                                        userProfile = userDao.getIehrUserProfile(userProfile.smartCardID, userProfile.smartCardAgency)
                                    }
                                }
                            }
                        }
                        catch (Exception e) {
                            errorText = "Could not save the User Profile."
                            log.error("AuthenticateController::setIehrUserProfile", e)
                            render(view: "login", model: [errorText     : errorText, grailsConfig: grailsApplication.config.grails, "sites": sites, "userProfile": userProfile, "theme": theme, "dodVLER": dodVLER, "dasVLER": dasVLER, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
                                                          "badLoginDate": badLoginDate, "badLoginIP": badLoginIP, "validLoginDate": validLoginDate, "validLoginIP": validLoginIP, "serviceStatus": serviceStatus, "csrfToken": csrfToken])
                            return
                        }

                        appUser.setSecurityKey(tokenUtils.generateSecurityKey())
                        appUser.setUserId(userProfile.dbUserID);
                        appUser.setCfg(userProfile.cfg);

                        //**melcolm remove this if/when jimi fixes login mock service
                        if (appUser.getHostSite() == null && appConfig.getSmartCardBypass()) {
                            GUISite hostSite = new GUISite()
                            hostSite.setAgency(loginAgency);
                            hostSite.setPermissions(null);
                            appUser.setHostSite(hostSite);
                            appUser.setName("McLuvin,Dood");
                            appUser.setAgency(userProfile.getSmartCardAgency());
                            appUser.setCardId(userProfile.getSmartCardID());
                            appUser.setEmail(userProfile.getEmailAddress());
                        }

                        session.appUser = appUser
                        session.userProfile = userProfile
                        session.setAttribute("acceptedAgreement", agreement)
                        redirect(controller: 'app')
                    }
                }
                catch (Exception e) {
                    log.error("AuthenticateController::login", e)

                    render(view: "login", model: [errorText     : errorText, grailsConfig: grailsApplication.config.grails, "sites": sites, "userProfile": userProfile, "theme": theme, "dodVLER": dodVLER, "dasVLER": dasVLER, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
                                                  "badLoginDate": badLoginDate, "badLoginIP": badLoginIP, "validLoginDate": validLoginDate, "validLoginIP": validLoginIP, "serviceStatus": serviceStatus, "csrfToken": csrfToken])
                    return
                }
            } else {
                render(view: "login", model: [loginCmd      : loginCmd, grailsConfig: grailsApplication.config.grails, "sites": sites, "theme": theme, "dodVLER": dodVLER, "dasVLER": dasVLER, "userProfile": userProfile, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
                                              "badLoginDate": badLoginDate, "badLoginIP": badLoginIP, "validLoginDate": validLoginDate, "validLoginIP": validLoginIP, "serviceStatus": serviceStatus, "csrfToken": csrfToken])
                return
            }
        }

        if (userProfile.smartCardID != null) {
            try {
                loginEntries = userDao.getLoginInfo(userProfile.smartCardID, userProfile.emailAddress);
                if (loginEntries != null) {
                    Iterator<GUIAuditEntry> i = loginEntries.iterator()
                    while (i.hasNext()) {
                        GUIAuditEntry loginEntry = i.next()
                        if (loginEntry.userID.startsWith("^")) {
                            badLoginDate = loginEntry.entryDate;
                            badLoginIP = loginEntry.ipAddress;
                        } else {
                            validLoginDate = loginEntry.entryDate;
                            validLoginIP = loginEntry.ipAddress;
                        }
                    }
                    session.loginEntries = loginEntries
                }
            }
            catch (Exception e) {
                log.error("AuthenticateController::login - Unable to retrieve previous login information", e)
//                String errorText = "There's an error retrieving your previous login information<br/>Please notify the system administrator";
//                render(view:"login", model:[errorText:errorText, grailsConfig:grailsApplication.config.grails,"userProfile":userProfile,"theme":theme, "loginAgency":loginAgency, "loginSite":loginSite, "csrfToken" : csrfToken])
//                return
            }
        }

        [grailsConfig  : grailsApplication.config.grails, "sites": sites, "userProfile": userProfile, "theme": theme, "dodVLER": dodVLER, "dasVLER": dasVLER, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
         "badLoginDate": badLoginDate, "badLoginIP": badLoginIP, "validLoginDate": validLoginDate, "validLoginIP": validLoginIP, "serviceStatus": serviceStatus, "csrfToken": csrfToken]
    }

    def logout = {
        if (appConfig.getSsoiEnabled()) {
            String logoutUrl = request.getHeader("SSOI-LANDING-URL")
            if (StringUtils.isNotEmpty(logoutUrl) && UrlValidator.getInstance().isValid(logoutUrl)) {
                session.invalidate()
                redirect(url: logoutUrl)
                return
            }
        }
        session.setAttribute("acceptedAgreement", false)
        session.invalidate()
    }

    def disclaimer = {
        if (request.method == 'GET') {
            //generate and set csrf token
            def csrfToken = UUID.randomUUID().toString()
            session.setAttribute("X-CSRF-Token", csrfToken)
            render(view: 'disclaimer', model: [csrfToken: csrfToken])
        } else if (request.method == 'POST') {
            session.setAttribute("acceptedAgreement", params['acceptAgreement']);
            redirect(controller: 'app')
        }
    }

    def selectSite = { SelectSiteCommand selectSiteCmd ->
        if (request.method == 'GET') {
            //generate and set csrf token
            def csrfToken = UUID.randomUUID().toString()
            session.setAttribute("X-CSRF-Token", csrfToken)
            def String agreement = (StringUtils.equalsIgnoreCase(params.agreement, 'true')) ? "true" : "false"
            render(view: "selectSite", model: ["sites": session.sites, "loginAgency": "", "loginSite": "", "agreement": agreement, "csrfToken": csrfToken])
        } else if (request.method == 'POST') {
            def IehrUserProfile userProfile
            def String loginAgency = ""
            def String loginSite = ""
            def jsonCfg = null
            def String agreement = "false"

            if (!selectSiteCmd.hasErrors()) {
                String errorText = "Failed to Login - Error contacting the server.<br/>Please notify the system administrator"

                loginAgency = StringEscapeUtils.escapeJavaScript(selectSiteCmd.loginAgency)
                loginSite = StringEscapeUtils.escapeJavaScript(selectSiteCmd.siteCode)
                agreement = "true".equalsIgnoreCase(selectSiteCmd.acceptedAgreement)

                if (session.userProfile != null) {
                    userProfile = session.userProfile
                    //userProfile.setSmartCardAgency(loginAgency)
                    userProfile.setLocID(loginSite)

                    if (userProfile.getFlags() == null) {
                        userProfile.setFlags("")
                    }

                    //save profile
                    try {
                        if (userDao.setIehrUserProfile(userProfile)) {
                            if (userProfile.dbUserID == "") {
                                if (StringUtils.isNotEmpty(userProfile.emailAddress)) {
                                    userProfile = userDao.getIehrUserProfile(userProfile.smartCardID, userProfile.smartCardAgency, userProfile.emailAddress)
                                } else {
                                    userProfile = userDao.getIehrUserProfile(userProfile.smartCardID, userProfile.smartCardAgency)
                                }
                            }
                        }
                    } catch (Exception e) {
                        errorText = "Could not save the User Profile."
                        log.error("AuthenticateController::setIehrUserProfile", e)
                        render(view: "login", model: [errorText     : errorText, grailsConfig: grailsApplication.config.grails, "sites": sites, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
                                                      "csrfToken": csrfToken])
                        return
                    }

                    session.userProfile = userProfile
                    session.setAttribute("acceptedAgreement", agreement)
                }

                redirect(controller: 'authenticate', action: 'login')

            } else {
                render(view: "login", model: [selectSiteCmd: selectSiteCmd, "sites": session.sites, "loginAgency": loginAgency, "loginSite": loginSite, "agreement": agreement,
                                              "csrfToken": session.csrfToken])
                return
            }

        }
    }

    def linkMySite = {
        if (request.method == 'GET') {
            //generate and set csrf token
            def csrfToken = UUID.randomUUID().toString()
            session.setAttribute("X-CSRF-Token", csrfToken)
            def String agreement = (StringUtils.equalsIgnoreCase(params.agreement, 'true')) ? "true" : "false"
            render(view: "linkMySite", model: ["linkMyAccountURL": appConfig.getSsoiLinkMyAccountURL(), "loginAgency": session.userProfile.smartCardAgency, "csrfToken": csrfToken, "agreement": agreement])
        }
    }

    def selectOrLinkSite = {
        if (request.method == 'GET') {
            //generate and set csrf token
            def csrfToken = UUID.randomUUID().toString()
            session.setAttribute("X-CSRF-Token", csrfToken)
            def String agreement = (StringUtils.equalsIgnoreCase(params.agreement, 'true')) ? "true" : "false"
            render(view: "selectOrLinkSite", model: ["loginAgency": session.userProfile.smartCardAgency, "csrfToken": csrfToken, "agreement": agreement])
        }
    }

}

class LoginCommand {
    String siteCode
    String loginAgency
    String accessCode
    String verifyCode
    String userCfg
    String acceptedAgreement

    static constraints = {
        siteCode blank: false
        accessCode blank: false
        verifyCode blank: false
        loginAgency blank: false
        userCfg blank: false
        acceptedAgreement blank: false
    }
}

class SelectSiteCommand {
    String siteCode
    String loginAgency
    String acceptedAgreement

    static constraints = {
        siteCode blank: false
        loginAgency blank: false
        acceptedAgreement blank: false
    }
}

