/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import org.apache.commons.lang3.StringUtils

class ContextController extends BaseController {
    def appConfig
    def tokenUtils

    def getPatientToken = {
        String edipi = params["edipi"];
        String icn = params["icn"];
        String vaIen = params["vaIen"];
        String vaIenSiteId = params["vaIenSiteId"];

        String patientToken = "";

        if (StringUtils.isNotBlank(edipi)) {
            patientToken = tokenUtils.generatePatientToken(session.appUser.securityKey,
                    appConfig.getIdTypeEDIPI(), edipi, "");
        } else if (StringUtils.isNotBlank(vaIen)) {
            if (StringUtils.isNotBlank(vaIenSiteId)) {
                patientToken = tokenUtils.generatePatientToken(session.appUser.securityKey,
                        appConfig.getIdTypeVAIen(), vaIen, vaIenSiteId);
            }
            else {
                // No site code... use site code of user (for VA user restricted patient list that only provides IENs)
                vaIenSiteId = session.appUser.hostSite.siteCode;
                patientToken = tokenUtils.generatePatientToken(session.appUser.securityKey,
                        appConfig.getIdTypeVAIen(), vaIen, vaIenSiteId);
            }
        } else if (StringUtils.isNotBlank(icn)) {
            patientToken = tokenUtils.generatePatientToken(session.appUser.securityKey,
                    appConfig.getIdTypeICN(), icn, "");
        }

        sendResponse(["patientToken": patientToken])
    }

    def index = {

    }

    def getPatientSiteCode = {
        def patientToken = params['patientToken']
        if (StringUtils.isBlank(patientToken)) {
            return ''
        }

        def tokenMap = tokenUtils.decryptTokenToMap(session.appUser.securityKey, patientToken)
        if (tokenMap == null) {
            return ''
        }

        def siteCode = tokenMap.get('patSiteCode')
        if (siteCode == null) {
            return ''
        }

        sendResponse(['siteCode': siteCode])
    }
}
