package jlv

import gov.va.med.common.HttpKeys
import org.apache.commons.lang3.StringUtils

class LoginController extends ContextController {

    def tokenUtils
    def auditDao
    def appConfig
    def loginService

    final static byte[] encryptionKey = [-82, -98, -98, 126, -31, 70, -93, 76, 7, -41, 57, -59, -52, 74, 34, -110];

    private String[] getEncryptionKey() {
        return Arrays.copyOf(encryptionKey, encryptionKey.length);
    }

    def requestAccess = {
        if (request.method != "POST") {
            return false
        }

        //request access token (User, system)
        String userEDIPI = params["userEDIPI"]
        String userName = params["userName"]
        String userNCID = params["userNCID"]
        String patientEDIPI = params["patientEDIPI"]
        String resourceUserID = params["resourceUserID"]
        String resourcePassword = params["resourcePassword"]
        String facilityNCID = params["facilityNCID"]

        //generate access token (w expiration datetime)
        String token = "";

        if (
        StringUtils.isBlank(userEDIPI) ||
                StringUtils.isBlank(userName) ||
                StringUtils.isBlank(userNCID) ||
                StringUtils.isBlank(resourceUserID) ||
                StringUtils.isBlank(resourcePassword) ||
                StringUtils.isBlank(facilityNCID) ||
                !params.containsKey("patientEDIPI")
        ) {
            response.sendError(403, "UserAccessDenied");
            return false;
        }

        if (StringUtils.isNotBlank(patientEDIPI)) {
            token = tokenUtils.generateTMDSPatientToken(
                    encryptionKey,
                    userEDIPI,
                    userName,
                    userNCID,
                    patientEDIPI,
                    resourceUserID,
                    resourcePassword,
                    facilityNCID);
        } else {
            token = tokenUtils.generateTMDSPatientToken(
                    encryptionKey,
                    userEDIPI,
                    userName,
                    userNCID,
                    null,
                    resourceUserID,
                    resourcePassword,
                    facilityNCID);
        }

        sendResponse(["token": token])
    }

    def accessToken = {
        def token = params["token"]

        if (StringUtils.isBlank(token)) {
            response.sendError(403, "UserAccessDenied");
            return false;
        }

        def tokenParams;
        try {
            tokenParams = tokenUtils.decryptTokenToMap(
                    //session.appUser.securityKey,
                    encryptionKey,
                    token)
        }
        catch (Exception exc) {
            response.sendError(403, "UserAccessDenied");
            log.debug("Login error decrypting token: " + exc.getMessage());
            return false;
        }

        log.debug("Login token, patient params from token: ${tokenParams}")

        if (tokenParams != null && (
                StringUtils.isBlank(tokenParams["userEDIPI"]) ||
                        StringUtils.isBlank(tokenParams["userName"]) ||
                        StringUtils.isBlank(tokenParams["userNCID"]) ||
                        StringUtils.isBlank(tokenParams["resourceUserID"]) ||
                        StringUtils.isBlank(tokenParams["resourcePassword"]) ||
                        StringUtils.isBlank(tokenParams["facilityNCID"]))
        ) {
            response.sendError(403, "UserAccessDenied");
            log.debug("Login token error: null or blank parameters: ${tokenParams}")
            return false;
        }

        //validate access token valid and not expired
        String timestamp = tokenParams[HttpKeys.TIMESTAMP]
        if (timestamp != null && StringUtils.isNotBlank(timestamp)) {
            long ts = Long.parseLong(timestamp);
            int tokenTimeout = appConfig.getTokenLoginTimeout();
            long currentTime = System.currentTimeMillis();

            if (currentTime > (ts + tokenTimeout)) {
                response.sendError(403, "UserAccessDenied");
                log.debug("Login token expired: ${ts} vs ${currentTime}")
                return false;
            }
        } else {
            response.sendError(403, "UserAccessDenied");
            log.debug("Login token error: null or blank timestamp: ${timestamp}")
            return false;
        }

        tokenParams["loginSource"] = "TMDS"

        def loginResponse = loginService.loginAHLTA(tokenParams, session);

        if (loginResponse.success) {
            redirect(controller: 'app')
            return true
        } else {
            response.sendError(loginResponse.code, loginResponse.msg)
            return false
        }
    }

}
