package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.GUIPatientEncounter
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.jmeadows.webservice.NoteImage
import gov.va.med.vhahon.common.Utils
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang.StringUtils
import gov.va.med.jmeadows.webservice.Pce

class PatientEncountersController extends BaseController {

    def appointmentDao
    def auditDao
    def noteDao
    def sensitiveRecordDao

    static allowedMethods = [list: 'POST', auditDetails: 'POST', sensitiveEncounter: 'POST', getEncounterPCE: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        dateRangeCheck();
    }

    def list = {

        def guiEncounters = appointmentDao.getPatientEncounters(session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        String ipAddress = request.getHeader("X-FORWARDED-FOR");
        if (ipAddress == null) {
            ipAddress = request.getRemoteAddr();
        }

        if (guiEncounters != null) {
            for (def guiEncounter : guiEncounters) {
                if (StringUtils.isNotEmpty(guiEncounter.getEncounterId())) {
                    String noteId = "";
                    if (guiEncounter.getDocuments() != null
                            && guiEncounter.getDocuments().size() > 0
                            && guiEncounter.getDocuments().get(0).getId() != null) {
                        noteId = guiEncounter.getDocuments().get(0).getId();
                        guiEncounter.setVaNote(true);
                    }

                    def queryMap = [(HttpKeys.ENCOUNTER_ID)       : guiEncounter.getEncounterId(),
                                    (HttpKeys.NOTE_ID)            : noteId,
                                    (HttpKeys.RECORD_DATE)        : guiEncounter.getApptDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiEncounter.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiEncounter.getSite().getMoniker()]

                    String controllerNameStr = controllerName;
                    if(guiEncounter.getSite() != null && "2.16.840.1.113883.3.42.10024.100001.13".equalsIgnoreCase(guiEncounter.getSite().getSiteCode())){
                        controllerNameStr = "mhsGenesis";
                    }

                    guiEncounter.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerNameStr,
                                    queryMap,
                                    guiEncounter.getSite(),
                                    guiEncounter.getSourceProtocol(),
                                    guiEncounter.isSessionCachedRecord()))

                    guiEncounter.setGuiIpAddress(ipAddress)

                    if (guiEncounter.isSessionCachedRecord()) {
                        //cache encounter report in session
                        def encounterDetail = BeanUtils.cloneBean(guiEncounter)

                        session.appUser.encounterMap.put(
                                getCachedRecordKey(guiEncounter.getRecordDetailsToken()), encounterDetail)

                        //don't send report text with note listings
                        //guiEncounter.setReport("")
                    }

                    // Don't pass out any sensitive data as part of the progress notes list
                    if (guiEncounter.isSensitive()) {
                        guiEncounter.setClinic("** Sensitive **");
                        guiEncounter.setReason("** Sensitive **");
                        guiEncounter.setDiagnosis(null);
                        guiEncounter.setProcedures(null);
                        guiEncounter.setEncounterNumber("");
                        guiEncounter.setEncounterId("");
                        if (guiEncounter.getComplexDataUrl() != null && !guiEncounter.getComplexDataUrl().isEmpty()) {
                            guiEncounter.setComplexDataUrl("** Sensitive **");
                        } else {
                            guiEncounter.setComplexDataUrl("");
                        }
                        guiEncounter.setReasonNarrative("");
                        guiEncounter.setDischargeDisposition("");
                        guiEncounter.setDocuments(null);
                        guiEncounter.setReasonCode("");
                        guiEncounter.setCodes(null);
                    }
                }
            }
        }

        if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
            GUIPatientEncounter statusRecord = new GUIPatientEncounter();
            statusRecord.setClinic(session.appUser.vlerSiteStatus);
            statusRecord.setSite(getStatusReportSite());
            guiEncounters.add(statusRecord);
        }

        sendResponse(guiEncounters)
    }

    def auditDetails = {
        def recordSiteAgency = params[HttpKeys.RECORD_SITE_AGENCY]
        def recordSiteMoniker = params[HttpKeys.RECORD_SITE_MONIKER]
        def info = params[HttpKeys.RECORD_DATE]
        def recordId = params[HttpKeys.EVENT_ID]
        if (!recordId) {
            recordId = params[HttpKeys.ENCOUNTER_ID]
        }

        if (params[HttpKeys.INFO]) {
            info = info + '^' + params[HttpKeys.INFO]
        } else {
            info = info + '^' + params[HttpKeys.AUDIT_TEXT]
        }

        def auditCategory = "encounterDetail";
        if (params[HttpKeys.WIDGET_NAME]) {
            auditCategory = params[HttpKeys.WIDGET_NAME] + '^' + auditCategory;
        }

        auditDao.auditDetailsQuery(auditCategory, info, session.appUser, request.patient, null, recordId, recordSiteMoniker, recordSiteAgency)

        sendResponse([success: "true"])
    }

    def getNoteDataStream = {

        //retrieve cached note
        GUIPatientEncounter guiEncounter = session.appUser.encounterMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        def auditQueryName = "GetEncounterDataStream"

        if ((guiEncounter == null) || !guiEncounter.noteAsBinaryStream) {
            response.sendError 500
            return false;
        }

        if (guiEncounter.isSensitive()) {
            String auditText = params[HttpKeys.AUDIT_TEXT]
            // audit with break the glass
            auditText = "btgPatientEncounters^" + auditText;

            sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText,
                    request.recordQuery[HttpKeys.NOTE_ID], session.scId, session.scEmail, "",
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])
        }

        //stream share/essentris CDA notes
        //if (guiEncounter.getSite().siteCode.startsWith("2.16.840.1.113883.3.42.10002.100001.8")) {
        if (guiEncounter.contentType != null && ("CDA".equalsIgnoreCase(guiEncounter.contentType) || guiEncounter.contentType.indexOf("xml") > 0) && guiEncounter.sourceProtocol == appConfig.getProtocolDODAdapter()) {

            auditDao.auditDetailsQuery(auditQueryName, request.recordQuery[HttpKeys.RECORD_DATE], session.appUser,
                    request.patient, null,
                    request.recordQuery[HttpKeys.ENCOUNTER_ID],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            //old code//
            //makeCIS_CDAText(guiEncounter.getReport())
            return true;
        }
        //fetch dod note image
        else {
            String complexDataUrl = guiEncounter.getComplexDataUrl();

            NoteImage noteImage = null;

            if (complexDataUrl != null) {

                auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                        complexDataUrl,
                        request.recordQuery[HttpKeys.NOTE_ID],
                        request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                        request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

                try {
                    noteImage = noteDao.getBHIENoteImage(session.appUser, complexDataUrl);
                }
                catch (DaoException e) {
                    log.error();
                }

                response.setHeader("Pragma", "public");
                response.setHeader("Cache-Control", "maxage=1");
                //response.setHeader("Expires", "0");

                if (noteImage != null && noteImage.getNoteBytes() != null) {
                    response.setContentType(noteImage.getContentType());
                    String contentDisposition = noteImage.getContentDisposition()
                    response.setHeader("Content-Disposition", contentDisposition.replace("attachment", "inline"))
                    response.getOutputStream().write(noteImage.getNoteBytes());
                    response.getOutputStream().flush();
                    response.getOutputStream().close();
                    return true;
                }
            }

            //if note attachment is null, send an error message

            String errorMsg = "An error occurred while attempting to retrieve note.";

            response.setContentType("text");
            response.setContentLength(errorMsg.length());
            response.getWriter() << errorMsg;
            response.getWriter().flush();

            response.sendError 500
            return false;
        }
    }

    def details = {

        GUIProgressNote guiNote = null

        //multipe note support using request param
        String noteId = StringUtils.isEmpty(params[HttpKeys.NOTE_ID]) ? request.recordQuery[HttpKeys.NOTE_ID] : params[HttpKeys.NOTE_ID];

        if (!request.isSessionCachedRecord) {
            guiNote = noteDao.getProgressNote(
                    session.appUser,
                    request.patient,
                    noteId,
                    request.recordQuery[HttpKeys.RECORD_SITE_CODE],
                    request.recordQuery[HttpKeys.RECORD_DATE] + '^' + params[HttpKeys.TITLE]);

            def auditCategory = "ProgressNoteDetail";
            if (params[HttpKeys.WIDGET_NAME]) {
                auditCategory = params[HttpKeys.WIDGET_NAME] + '^' + auditCategory;
            }

            auditDao.auditDetailsQuery(auditCategory, request.recordQuery[HttpKeys.RECORD_DATE] + '^' +
                    params[HttpKeys.TITLE],
                    session.appUser, request.patient,
                    null,
                    noteId,
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            sendResponse(guiNote)
            return true;
        }

        //retrieve cached note
        guiNote = session.appUser.outpatientNotesMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        if (guiNote.sensitive) {
            String auditText = params[HttpKeys.AUDIT_TEXT]
            // audit with break the glass
            auditText = "btgPatientEncounters^" + auditText;

            sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText,
                    request.recordQuery[HttpKeys.ENCOUNTER_ID], session.scId, session.scEmail, "",
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])
        }

        def auditCategory = "ProgressNoteDetail";
        if (params[HttpKeys.WIDGET_NAME]) {
            auditCategory = params[HttpKeys.WIDGET_NAME] + '^' + auditCategory;
        }

        auditDao.auditDetailsQuery(auditCategory, request.recordQuery[HttpKeys.RECORD_DATE] + '^' +
                params[HttpKeys.TITLE],
                session.appUser, request.patient,
                null,
                request.recordQuery[HttpKeys.ENCOUNTER_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        sendResponse(guiNote)
    }

    def sensitiveEncounter = {
        GUIPatientEncounter guiEncounter = null

        //retrieve cached encounter
        guiEncounter = session.appUser.encounterMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))


        String auditText = params[HttpKeys.AUDIT_TEXT]
        // audit with break the glass
        auditText = "btgPatientEncounters^" + auditText;

        sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText,
                request.recordQuery[HttpKeys.ENCOUNTER_ID], session.scId, session.scEmail, "",
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        sendResponse(guiEncounter)
    }

    def auditVixViewer = {
        auditDao.auditDetailsQuery(
                'EncounterVixViewer',
                session.appUser,
                request.patient,
                null,
                request.recordQuery[HttpKeys.NOTE_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY]);
    }

    def getEncounterVixStudyViewerUrl = {
        def vixStudyViewerUrl = appointmentDao.getEncounterVixStudyViewerUrl(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]),
                params.documentId,
                params.documentSiteId);
        sendResponse([vixStudyViewerUrl: vixStudyViewerUrl])
    }

    def getEncounterPCE = {
        String incomingList = params["noteIDList"]
        def List<Pce> pceHolder;
        if (incomingList.length() > 0) {
            pceHolder = appointmentDao.getPcesForNote(session.appUser,
                    request.patient,
                    params["recordSiteCode"],
                    incomingList
            );
        }

        sendResponse(pceHolder)
    }

}
