/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.guibeans.GUIImmunization
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.NarrativeV4
import org.apache.commons.lang3.StringUtils

class PatientImmunizationsController extends BaseController {
    def immunizationDao

    static allowedMethods = [list: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }
    }

    def list = {


        def includeVLER = false;
        def includeDASVLER = false;

        if (params["includeVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "DOD") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            includeVLER = true;
        }
        if (params["includeDASVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "VA") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            includeDASVLER = true;
        }

        HashSet narrativeIds = new HashSet();

        def guiImmunizations = immunizationDao.getPatientImmunizations(session.appUser, request.patient, includeVLER, includeDASVLER)

        //generate record details token
        if (guiImmunizations != null) {
            for (def guiImmunization : guiImmunizations) {

                if (guiImmunization.getNarrativeId() != null && !"".equalsIgnoreCase(guiImmunization.getNarrativeId())) {
                    narrativeIds.add(guiImmunization.getNarrativeId());

                    if(StringUtils.isNotEmpty(guiImmunization.getNarrative())){
                        NarrativeV4 narrativeV4 = new NarrativeV4();
                        narrativeV4.setNarrativeIdentifier(guiImmunization.getNarrativeId());
                        narrativeV4.setNarrativeText(guiImmunization.getNarrative());
                        session.appUser.narrativesMap.put(guiImmunization.getNarrativeId(), narrativeV4);
                        guiImmunization.setNarrative("");
                    }

                    if(guiImmunization.getSite() != null && guiImmunization.getSite().getName() != null
                            && "VLER".equalsIgnoreCase(guiImmunization.getSite().getName())
                            && guiImmunization.getSite().getSiteCode() != null){
                        String siteName = session.appUser.partnersMap.get(guiImmunization.getSite().getSiteCode());
                        if(siteName != null){
                            guiImmunization.getSite().setName(siteName);
                        }
                        guiImmunization.getSite().setAgency("VLER");
                    }
                }

            }
        }
        if ((includeVLER || includeDASVLER) && session.appUser.narrativesMap != null
                && !session.appUser.narrativesMap.isEmpty()) {
            Iterator narrativeIter = session.appUser.narrativesMap.entrySet().iterator();

            //loop through all narratives, check the immunization ones to see if they are referenced in this call
            //if they are not referenced, then create a record for them so the user is able to view this narrative
            while (narrativeIter.hasNext()) {
                Map.Entry narrative = (Map.Entry) narrativeIter.next();
                if (narrative.getKey().toString().contains("39235-7")) {
                    //check to see if record using this narrative exists
                    if (!narrativeIds.contains(narrative.getKey().toString())) {

                        NarrativeV4 narrativeItem = session.appUser.narrativesMap.get(narrative.getKey().toString());
                        String siteName = "VLER";
                        String siteMoniker = "VLER Partner";
                        if (narrativeItem != null)
                            if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                    && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                                siteName = session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier());
                            }
                        if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                            siteMoniker = session.appUser.partnersMap.get(narrativeItem.getNarrativeAssigningAuthorityIdentifier());
                        }
                        GUISite narrativeSite = new GUISite();
                        narrativeSite.setAgency("VLER");
                        narrativeSite.setName(siteName);
                        narrativeSite.setMoniker(siteMoniker);
                        narrativeSite.setSiteCode("VLER");

                        GUIImmunization narrativeImmunization = new GUIImmunization();
                        narrativeImmunization.setNarrativeId(narrative.getKey().toString());
                        narrativeImmunization.setSite(narrativeSite);
                        narrativeImmunization.setVaccine("Narrative");
                        guiImmunizations.add(narrativeImmunization);

                        narrativeIds.add(narrative.getKey().toString());
                    }

                }
            }

        }

        sendResponse(guiImmunizations)

    }
}
