package jlv

import gov.va.med.common.PrintAttachment
import gov.va.med.common.PrintUtils
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.GUIConsult
import gov.va.med.guibeans.GUIPatientEncounter
import gov.va.med.guibeans.GUIProcedure
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.guibeans.GUIMhsGenesis
import gov.va.med.jmeadows.webservice.NoteImage
import groovy.json.JsonSlurper
import org.apache.commons.lang.StringUtils
import org.codehaus.groovy.grails.web.servlet.mvc.GrailsWebRequest
import org.springframework.stereotype.Controller
import org.springframework.web.context.request.RequestContextHolder

import javax.xml.transform.Transformer
import javax.xml.transform.TransformerFactory
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.stream.StreamSource

@Controller
class PrintController extends BaseController {


    static allowedMethods = [setup: 'POST', table: 'GET', details: 'GET', report: 'GET', xml: 'GET']

    //@Autowired
    def printUtils
    def auditDao
    def noteDao
    def VLERService
    def inpatientNotesService

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }
    }

    def setup = {
        def data = new JsonSlurper().parseText(params.data)
        data.jobId = UUID.randomUUID().toString(); //for security
        getSession().setAttribute("printSetup", data)
        def recordType = "printAction"

        auditDao.auditPrintQuery(recordType, data.title, session.appUser, request.patient, "", "", "", "")
        render(text: data.jobId)
    }


    private boolean isInpatientNoteDocument(record) {
        return (record.codes != null && record.codes.size > 0 &&
                (record.codes[0].code == "34117-2"
                        || record.codes[0].code == "18842-5"));
    }


    private boolean isVLERDocument(record) {
        return record.class.toString().toUpperCase().contains("GUIVLERDOCUMENT");
    }

    private boolean isGenesisDocument(record) {
        return record.isGenesisDoc || record.class.toString().toUpperCase().contains("GENESIS");
    }

    private PrintAttachment processHtmlAttachment(html) {

        def data = printUtils.renderHtmlToPdf(html)

        PrintAttachment attachment = new PrintAttachment()
        attachment.setBytes(data)

        return attachment
    }

    /* finds the note, if it exists */

    private Map<String, String> getNoteData(record) {

        //retrieve cached note
        GUIProgressNote guiNote = null;
        GUIPatientEncounter guiEncounter = null;
        GUIProcedure guiProcedure = null;
        GUIConsult guiConsult = null;
        GUIMhsGenesis guiGenesis = null;

        /* check inpatient & outpatient notes  */
        guiNote = session.appUser.outpatientNotesMap.get(
                getCachedRecordKey(record.recordDetailsToken))

        if (guiNote == null) {
            guiNote = session.appUser.inpatientNotesMap.get(
                    getCachedRecordKey(record.recordDetailsToken))
        }

        if (guiNote != null) {
            return [complexDataUrl: guiNote.getComplexDataUrl(),
                    html          : guiNote.getNoteText()
            ]
        }

        guiEncounter = session.appUser.encounterMap.get(
                getCachedRecordKey(record.recordDetailsToken))


        if (guiEncounter != null) {
            return [complexDataUrl: guiEncounter.getComplexDataUrl(),
                    html          : ""
            ]
        }

        guiProcedure = session.appUser.proceduresMap.get(
                getCachedRecordKey(record.recordDetailsToken))

        if (guiProcedure != null) {
            return [complexDataUrl: guiProcedure.getComplexDataUrl(),
                    html          : ""
            ]
        }

        guiConsult = session.appUser.consultMap.get(
                getCachedRecordKey(record.recordDetailsToken))


        if (guiConsult != null) {
            return [complexDataUrl: guiConsult.getComplexDataUrl(),
                    html          : ""
            ]
        }

        guiGenesis = session.appUser.genesisMap.get(
                getCachedRecordKey(record.recordDetailsToken))

        if (guiGenesis != null) {
            return [complexDataUrl: guiGenesis.getComplexDataUrl(),
                    html          : ""
            ]
        }

        return [complexDataUrl: "",
                html          : ""
        ]

    }

    private PrintAttachment processInpatientNote(record) {

        //retrieve cached note
        GUIProgressNote guiNote = session.appUser.inpatientNotesMap.get(
                getCachedRecordKey(record.recordDetailsToken))

        if (guiNote != null) {
            String html = transformXML(guiNote.getNoteText(), "CDA.xsl")
            return processHtmlAttachment(html)
        }


        return null

    }

    private String transformXML(xml, xslTemplate) {

        def xslResource = grailsApplication.mainContext.getResource('/WEB-INF/xslt/' + xslTemplate)
        StreamSource xslSource = new StreamSource(xslResource.getInputStream(),
                xslResource.getURL().toString())
        def factory = TransformerFactory.newInstance()
        Transformer transformer = factory.newTransformer(xslSource)

        def out = new StringWriter()
        transformer.transform(new StreamSource(new StringReader(xml)),
                new StreamResult(out))

        return out.toString()

    }

    /* processes Document attachment */

    private PrintAttachment processDocumentsAttachment(record) {

        def noteData = getNoteData(record)

        String complexDataUrl = noteData["complexDataUrl"],
               html = noteData["html"];

        log.debug("The complex data url is: " + complexDataUrl)
        log.debug("the html is " + html)

        if (StringUtils.isEmpty(complexDataUrl) && StringUtils.isNotEmpty(html)) {
            String tHtml = transformXML(html, "CDA.xsl")
            return processHtmlAttachment(tHtml)

        } else {

            NoteImage noteImage = null;
            String resultType = null;

            if (StringUtils.isNotEmpty(complexDataUrl)) {

                try {
                    noteImage = noteDao.getBHIENoteImage(session.appUser, complexDataUrl);
                }
                catch (DaoException e) {
                    log.error(e);
                }

                if (noteImage != null)
                    resultType = noteImage.getContentType();
            }

            log.debug("The returned resultType from BHIE is " + resultType)

            if ((resultType != null) &&
                    (resultType.contains("pdf") || resultType.contains("octet-stream"))) {
                PrintAttachment attachment = new PrintAttachment();
                attachment.setBytes(noteImage.getNoteBytes())
                return attachment
            }

        }

        return null;
    }

    /* processes Genesis documents */

    private PrintAttachment processGenesisDocument(record) {

        def noteData = getNoteData(record)

        String complexDataUrl = noteData["complexDataUrl"],
               html = noteData["html"];

        log.debug("The complex data url is: " + complexDataUrl)
        log.debug("the html is " + html)


        NoteImage noteImage = null;
        String resultType = null;

        if (StringUtils.isNotEmpty(complexDataUrl)) {

            try {
                noteImage = noteDao.getBHIENoteAttachment(session.appUser, complexDataUrl);
            }
            catch (DaoException e) {
                log.error(e);
            }

            if (noteImage != null)
                resultType = noteImage.getContentType();
        }

        log.debug("The returned resultType from BHIE is " + resultType)

        if ((resultType != null) &&
                (resultType.contains("pdf") || resultType.contains("octet-stream"))) {
            PrintAttachment attachment = new PrintAttachment();
            attachment.setBytes(noteImage.getNoteBytes())
            return attachment
        } else {
            String tHtml = transformXML(new String(noteImage.getNoteBytes()), "c32styles/nhin/cdaHFG.xsl")
            return processHtmlAttachment(tHtml)
        }

        return null;
    }

    /* renders a simple row of a record in Report Builder */

    private String renderSimpleRecordHTML(record, isAlt) {

        StringBuilder recordTable = new StringBuilder()

        recordTable.append("<div style=\"padding:15px 0px 0px 0px;page-break-inside:avoid;\">")

        if (isAlt) {
            recordTable.append("<table style=\"background-color:#f8f8f8;width:100%; page-break-inside:avoid ;border-bottom:1px solid #bbb;\">")
        } else {
            recordTable.append("<table style=\"width:100%; page-break-inside:avoid;border-bottom:1px solid #bbb;\">")
        }

        recordTable.append("<tr><td><div style=\"padding:8px 0 8px 0; font-size:16px; page-break-inside: avoid;\">" + record._name + "</div></td></tr>")
        recordTable.append("<tr><td><table style=\"width:100%; margin-bottom:6px; page-break-inside:avoid;\" cellspacing=\"0\">")
        recordTable.append("<tr style=\"page-break-inside:avoid; page-break-after:auto;\">")

        def columns = record.columns

        columns.each { column ->
            String title = column.title
            recordTable.append("<th style=\"border-width:0;padding:12px; font-size:13px;font-weight:bold; page-break-inside:avoid;\">" + title + "</th>")
        }

        recordTable.append("</tr><tr style=\"page-break-inside:avoid; page-break-after:auto;\">")
        columns.each { column ->

            String v = record[column.id]

            recordTable.append("<td style=\"border-width:0;font-size:12px; padding:14px;\">" + v + "</td>")
        }

        recordTable.append("</tr></table></td></tr></table></div>")

        return recordTable.toString()
    }


    private static String reportHdr = '''<!DOCTYPE html><html><head>
                    <style type='text/css'>
                        html{font-family:Verdana, Arial, sans-serif;font-size:13px;}
                        table{
                            thead {
                                display: table-header-group;
                            }
                        }
                        tr, td, th, tbody, thead, tfoot, .records, .details {
                            page-break-inside: avoid !important;
                        }
                        th {
                        text-align:left;
                        }
                    </style>'''

    def setupReportBuilder = {

        def printSetup = new JsonSlurper().parseText(params.data)
//        def recordType = "printAction"

//        auditDao.auditPrintQuery(recordType, printSetup.title, session.appUser, printSetup.patient, "", "", "", "")

        def jobId = UUID.randomUUID().toString();
        printSetup.jobId = jobId;

        def reportStatus = [];

        StringBuilder sb = new StringBuilder()

        sb.append(reportHdr)
        sb.append("<title>Document</title></head><body>")
        sb.append(PrintUtils.headerReportBuilder(PrintUtils.createPatient(printSetup.patient), session.appUser, printSetup.datetime))

        def data = printSetup.data

        if (data) {

            def i = 0
            sb.append("<div class=\"records\">");
            def attachments = []
            data.each { row ->
                i++
                if (!row.demographics) {
                    if (row.hasAttachment) {

                        if (isVLERDocument(row)) {

                            log.debug("Attempting to generate a vler Document")
                            log.debug(row)
                            jlv.VLERService.VLERHtmlDocument vlerHtmlDocument = VLERService.xmlToHtml(session, request, row['documentUniqueId'], row['homeCommunityId'], row['repositoryUniqueId'], params.patToken);

                            String vlerHTML = vlerHtmlDocument.html;

                            log.debug("This is the vlerHTML =>" + vlerHTML)

                            if (!StringUtils.isBlank(vlerHTML)) {

                                sb.append(renderSimpleRecordHTML(row, i % 2 == 0))
                                PrintAttachment attachment = processHtmlAttachment(vlerHTML)
                                if (attachment != null) {
                                    attachments.push(attachment)
                                    sb.append("<p style=\"font-style:italic;\">See attachment " + attachments.size() + "</p>");
                                }
                                reportStatus.push([status: 'ready'])


                            } else {
                                reportStatus.push([status: 'error', errorCode: 'BD']);
                            }
                        } else if (isInpatientNoteDocument(row)) {
                            log.debug("inpatient note")

                            PrintAttachment attachment = processInpatientNote(row)

                            if (attachment != null) {
                                sb.append(renderSimpleRecordHTML(row, i % 2 == 0))
                                attachments.push(attachment)
                                sb.append("<p style=\"font-style:italic;\">See attachment " + attachments.size() + "</p>");
                                reportStatus.push([status: 'ready']);
                            } else {
                                reportStatus.push([status: 'error']);
                            }

                        } else if (isGenesisDocument(row)) {

                            PrintAttachment attachment = processGenesisDocument(row)
                            if (attachment != null) {
                                attachments.push(attachment)
                                sb.append(renderSimpleRecordHTML(row, i % 2 == 0))
                                sb.append("<p style=\"font-style:italic;\">See attachment " + attachments.size() + "</p>");
                                reportStatus.push([status: 'ready'])
                            } else {
                                reportStatus.push([status: 'error', errorCode: 'BD'])
                            }
                        } else {

                            PrintAttachment attachment = processDocumentsAttachment(row)
                            if (attachment != null) {
                                attachments.push(attachment)
                                sb.append(renderSimpleRecordHTML(row, i % 2 == 0))
                                sb.append("<p style=\"font-style:italic;\">See attachment " + attachments.size() + "</p>");
                                reportStatus.push([status: 'ready'])
                            } else {
                                reportStatus.push([status: 'error', errorCode: 'BD'])
                            }
                        }
                    } else {

                        if (row.details) {
                            sb.append(renderSimpleRecordHTML(row, i % 2 == 0))
                            sb.append("<div class=\"details\">" + row.detailsTitle + "<div style=\"background-color:#eaeaea;\">" + row.details + "</div></div>");
                            reportStatus.push([status: 'ready'])
                        } else {
                            sb.append(renderSimpleRecordHTML(row, i % 2 == 0))
                            reportStatus.push([status: 'ready'])
                        }
                    }
                } else {
                    if (i % 2 == 0)
                        sb.append('<div style="background-color:#f8f8f8;padding-top:15px;"><div><i>Patient Demographics</i></div>' + row.rbDetails + '</div>');
                    else
                        sb.append('<div style="padding-top:15px;"><div><i>Patient Demographics</i></div>' + row.rbDetails + '</div>');

                    reportStatus.push([status: 'ready'])
                }
            }

            sb.append("</div></body></html>")

            printSetup.html = sb.toString()
            //println printSetup.html

            getSession().setAttribute("printSetup", printSetup)
            getSession().setAttribute("attachments", attachments)
        }

        render(contentType: 'text/json') {
            [
                    'jobId'       : jobId,
                    'reportStatus': reportStatus
            ]
        }
        return
    }

    def report = {
        def printSetup = getSession().getAttribute("printSetup")
        if (printSetup.jobId.equals(params['jobId'])) {

            def recordType = "printAction"
            def title = "ReportBuilder"

            auditDao.auditPrintQuery(recordType, title, session.appUser, request.patient, "", "", "", "")

            byte[] output;

            try {
                output = printUtils.printReport(printSetup.html, getSession().getAttribute("attachments"))

                getSession().setAttribute("printSetup", null)
                getSession().setAttribute("attachments", null)
                sendPDF(output, title)
                return
            }
            catch (Exception e) {

                log.debug("error with generating PDF report builder")
                log.debug("html:")
                log.debug(printSetup.html)
                log.debug("exception: ", e)

                render(text: "<html><body><p>There was an error generating your report.  Please record the following error message and send it to your Administrator:</p><p>" + e.getMessage() + "</p></body></html>")
            }


        }
    }


    private static String detailsHdr = '''<!DOCTYPE html><html><head>
                    <style type='text/css'>
                    html{font-size:13px;font-family:Verdana, Arial, sans-serif;}
                    td{font-size:12px;}
                    td { padding:12px;}
                    th {font-size:12px;font-weight:bold; text-align:center;padding:5px;}
                    table{
                        thead {
                            display: table-row-group;
                        }
                    }
                    tr, td, th, tbody, thead, tfoot {
                        page-break-inside: avoid !important;
                    }
                    a { text-decoration:none; color:#000;}
                    tr.jlv-alert-row{color:#000 !important;background-color:#FFC0CB;}
                    tr.jlv-high-alert-row{color:#fff; background-color:#901100;}
                    tr.jlv-high-alert-row a{color:#fff;}
                    .questionnaire-highlighted {background-color: #ffff00;}
                    </style>'''

    def details = {

        def printSetup = getSession().getAttribute("printSetup")

        if (printSetup.jobId.equals(params['jobId'])) {
            String data = printSetup.data

            //get rid of invalid xml
            data = data.replaceAll("<br>", "<br/>")

            StringBuilder sb = new StringBuilder()

            sb.append(detailsHdr)
            sb.append("<title>Document</title></head><body>")
            sb.append(PrintUtils.header(PrintUtils.createPatient(printSetup.patient), session.appUser, printSetup.datetime))

            sb.append("<h4>" + printSetup.title + "</h4>")
            sb.append(data)
            sb.append("</body></html>")

            byte[] output = printUtils.printToPdf(sb.toString(), false)

            getSession().setAttribute("printData", null)

            sendPDF(output, printSetup.title)
            return;
        }

        response.sendError 500
        return false;

    }


    private static String tableHdr = '''<!DOCTYPE html><html><head>
                    <style type='text/css'>
                    html{font-family:Verdana, Arial, sans-serif;}
                    .title{width:100%;padding:15px;text-align:center;font-size:13px;}
                    table{width:100%;margin-top:15px;font-size:12px;
                        thead {
                            display: table-header-group;
                        }
                    }
                    table, tr, td, th, tbody, thead, tfoot {
                        page-break-inside: avoid !important;
                    }
                    td{font-size:12px;border:1px solid #999;padding:12px;}
                    th {font-size:13px;font-weight:bold; text-align:center;}
                    a { text-decoration:none; color:#000;}
                    tr.jlv-alert-row{color:#000 !important;background-color:#FFC0CB;}
                    tr.jlv-high-alert-row{color:#fff; background-color:#901100;}
                    tr.jlv-high-alert-row a{color:#fff;}
                    </style>'''

    def table = {

        def printSetup = getSession().getAttribute("printSetup")

        if (printSetup.jobId.equals(params['jobId'])) {

            def columnsInfo = printSetup.columns
            def widgetTitle = printSetup.title

            StringBuffer sb = new StringBuffer()

            sb.append(tableHdr)
            sb.append("<title>Document</title></head><body>")
            sb.append(PrintUtils.header(PrintUtils.createPatient(printSetup.patient), session.appUser, printSetup.datetime))

            sb.append("<div class=\"title\">" + widgetTitle + "</div>")

            sb.append("<table cellspacing=\"0\"><tr>")

            columnsInfo.each { columnInfo ->
                sb.append("<th>" + columnInfo.title + "</th>")
            }

            sb.append("</tr>")

            def data = printSetup.data

            if (data) {
                data.each { row ->

                    StringBuilder rowStr = new StringBuilder();

                    boolean alertRow = false
                    boolean highAlertRow = false;

                    columnsInfo.each { columnInfo ->

                        Object value = row[columnInfo.id]
                        String str;
                        if (value != null) {
                            str = value.toString();
                        } else {
                            str = "";
                        }

                        alertRow = str.contains("jlv-alert-row") || alertRow
                        highAlertRow = str.contains("jlv-high-alert-row") || highAlertRow

                        rowStr.append("<td>" + str + "</td>")

                    }

                    if (alertRow) {
                        sb.append("<tr class=\"jlv-alert-row\">")
                    } else if (highAlertRow) {
                        sb.append("<tr class=\"jlv-high-alert-row\">")
                    } else {
                        sb.append("<tr>")
                    }

                    sb.append(rowStr.toString())
                    sb.append("</tr>")
                }

            }

            sb.append("</table></body></html>")

            byte[] output = printUtils.printToPdf(sb.toString(), columnsInfo.size() > 6)
            getSession().setAttribute("printData", null)

            sendPDF(output, widgetTitle)
            return
        }

        response.sendError 500
        return false;

    }


    protected sendPDF(byte[] output, String fileName) {

        fileName = fileName.replaceAll("[^a-zA-Z0-9 _-]", "");
        response.setHeader("Content-Disposition", "inline; filename=\"" + fileName + "\"")
        render(file: output, contentType: "application/pdf")

        GrailsWebRequest webRequest =
                (GrailsWebRequest) RequestContextHolder.currentRequestAttributes()

        webRequest.setRenderView(false)

    }

}
