/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */

package jlv

import gov.va.med.common.HttpKeys

class VLERController extends BaseController {
    def VLERService;
    def auditDao;

    static allowedMethods = [list: 'POST', details: 'POST']

    private static
    final String XPATH_CCDA_TEMPLATEID = "/ClinicalDocument/templateId[@root='2.16.840.1.113883.10.20.22.1.1']";
    private static final String CCDA_NODE_ID = "templateId root=\"2.16.840.1.113883.10.20.22.1.1\"";
    private static final String XSL_PATH_C32 = "/c32styles/nhin/cda2detail.xsl";
    private static final String XSL_PATH_CCDA = "/c32styles/nhin/cdaHFG.xsl";

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        }
    }

    def index = {

        PrintWriter out = response.getWriter();
        // setting some response headers
        response.setHeader("Expires", "0");
        response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
        response.setHeader("Pragma", "public");
        // setting the content type
        response.setContentType("text/html");

        def auditQueryName = "GetVLERDocument"
        auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                null,
                "retrievingC32:" + request.recordQuery[HttpKeys.VLER_DOCUMENT_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        jlv.VLERService.VLERHtmlDocument htmlDocument = VLERService.xmlToHtml(session, request,
                request.recordQuery[HttpKeys.VLER_DOCUMENT_ID],
                request.recordQuery[HttpKeys.VLER_HOMECOMMUNITY_ID],
                request.recordQuery[HttpKeys.VLER_REPOSITORY_ID],
                params.patToken);


        if (htmlDocument.html != null) {

            out.println(htmlDocument.html);

            return true;
        }

        String errorMsg = "An error occurred while attempting to retrieve the document.";

        out.println(errorMsg);

        return true;

    }

    def getDocument = {

        //vler document is cached
        //def vlerDoc = session.appUser.vlerDocumentsMap.get(getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        PrintWriter out = response.getWriter();
        // setting some response headers
        response.setHeader("Expires", "0");
        response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
        response.setHeader("Pragma", "public");
        // setting the content type
        response.setContentType("text/html");

        def auditQueryName = "GetVLERDocument"
        auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                null,
                "retrievingC32:" + request.recordQuery[HttpKeys.VLER_DOCUMENT_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        jlv.VLERService.VLERHtmlDocument htmlDocument = null;

        try {
            htmlDocument = VLERService.convertHtml(session, request,getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]),params.patToken);
        } catch (Exception e){
            String errorMsg = "An error occurred while attempting to display the document.";
            out.println(errorMsg);
            return true;
        }


        if (htmlDocument.html != null) {
            out.println(htmlDocument.html);
            return true;
        }

        String errorMsg = "An error occurred while attempting to retrieve the document.";

        out.println(errorMsg);

        return true;
    }

}
