package jlv

import gov.va.med.enterprise.bean.ResponseGUIPatientQuery
import gov.va.med.foundations.utilities.EncryptUtil
import gov.va.med.foundations.utilities.VistaKernelHash
import gov.va.med.groovy.user.AppUser
import gov.va.med.guibeans.GUISite
import gov.va.med.guibeans.GUIUser
import gov.va.med.jmeadows.webservice.IehrUserProfile
import gov.va.med.jmeadows.webservice.PdwsQueryBean
import org.apache.commons.lang3.StringUtils
import org.codehaus.groovy.grails.web.servlet.mvc.GrailsHttpSession

public class LoginService {
    def authUserDao
    def userDao
    def tokenUtils
    def patientDao
    def auditDao
    def appConfig
    def samlTokenService

    public class LoginResponse {
        int code
        String msg
        boolean success = false;

        public void setStatus(int c, String m) {
            if (c == 200)
                success = true
            code = c
            msg = m
        }


    }

    public LoginResponse loginAHLTA(params, GrailsHttpSession session) {

        LoginResponse loginResponse = new LoginResponse();

        String userEdipi = params["userEDIPI"];
        String userName = params["userName"]
        String userNcid = params["userNCID"]
        String patientEdipi = params["patientEDIPI"]
        String resourceUserId = params["resourceUserID"]
        String resourcePassword = params["resourcePassword"]
        String facilityNcid = params["facilityNCID"]
        String loginAgency = "DOD"
        String loginSource = params["loginSource"];

        IehrUserProfile profile

        session.profileName = userName
        session.ahltaLoginFlag = 'false'

        if (StringUtils.isEmpty(userEdipi)) {
            String prefix = "N";

            if (loginSource == "TMDS") prefix = "T";

            userEdipi = prefix + userNcid;
        }

        if (appConfig.getAuditAhltaLoginAttempt()) {
            StringBuilder auditAttempt = new StringBuilder()
            auditAttempt.append("^userEDIPI=" + userEdipi)
            auditAttempt.append("^userName=" + userName)
            auditAttempt.append("^userNCID=" + userNcid)
            auditAttempt.append("^patientEDIPI=" + patientEdipi)
            auditAttempt.append("^resourceUserID=" + resourceUserId)
            auditAttempt.append("^facilityNCID=" + facilityNcid)
            auditDao.auditDetailsQuery(loginSource + "LoginAttempt", auditAttempt.toString(), null, null, null, null, null, null)
        }

        // Load user profile into session
        try {
            profile = userDao.getIehrUserProfile(userEdipi, loginAgency);

            // Profile exists in database
            if (StringUtils.isNotEmpty(profile.getDbUserID())) {
                if (profile.getFlags() == null) {
                    profile.setFlags("")
                }
                if (profile.getLocID() == null) {
                    profile.setLocID("")
                }
                if (profile.getCfg() == null) {
                    profile.setCfg("")
                }
            }
            // Profile was not in database
            else {
                profile.setDbUserID("")
                profile.setSmartCardID(userEdipi)
                profile.setSmartCardAgency(loginAgency)
                profile.setFlags("")
                profile.setLocID(facilityNcid);
                profile.setCfg("")
            }

            session.userProfile = profile;
        }
        catch (Exception e) {
            log.error("Failed to retrieve user profile: " + e.getMessage())
            loginResponse.setStatus(403, "AHLTALoginError-Profile") // Failed to load user profile
            return loginResponse; //false
        }

        // Get list of active sites that have endpoints from database
        if (!session.sites) {
            try {
                def sites = userDao.getSites()

                if (sites) {
                    Iterator<GUISite> i = sites.iterator()
                    GUISite site

                    while (i.hasNext()) {
                        site = i.next()

                        if (!site.getEndpoints() || site.getEndpoints().size() == 0 || site.getStatus() != "active") {
                            i.remove()
                        }
                    }

                    session.sites = sites
                }
            }
            catch (Exception e) {
                log.error("Failed to retrieve sites list: " + e.getMessage())
                loginResponse.setStatus(403, "AHLTALoginError-Sites") // Failed to load active sites
                return loginResponse; //false
            }
        }

        // Create GUISite
        GUISite hostSite = new GUISite()
        hostSite.setAgency(loginAgency)
        hostSite.setDmisId(facilityNcid)
        hostSite.setMoniker("AHLTA")
        hostSite.setName("AHLTA")
        hostSite.setSiteCode(facilityNcid)
        hostSite.setStatus("active")

        // Create GUIUser
        GUIUser user = new GUIUser()
        if (userName.indexOf(",") < 0) {
            userName += ",JLV"
        }
        user.setHostSite(hostSite)
        user.setAgency(loginAgency)
        user.setName(userName.toUpperCase())
        user.setUserIen(userNcid)
        user.setEncryptedAvCode(VistaKernelHash.encrypt(resourceUserId + ";" + resourcePassword, false));
        user.setEncryptedFederatedUid(EncryptUtil.encrypt("123456789"))
        user.setCardId(userEdipi)

        if (appConfig.getDodSAMLTokenEnabled()) {
            user.setSamlToken(samlTokenService.createSAMLToken(user));
        }

        AppUser appUser = new AppUser(user)

        try {
            if (userDao.setIehrUserProfile(profile)) {
                if (StringUtils.isBlank(profile.getDbUserID())) {
                    profile = userDao.getIehrUserProfile(profile.getSmartCardID(), profile.getSmartCardAgency());
                }
            }
        }
        catch (Exception e) {
            log.error("Failed to save user profile: " + e.getMessage());
            loginResponse.setStatus(403, "AHLTALoginError-ProfileSave")
            return loginResponse; //false;
        }

        appUser.setSecurityKey(tokenUtils.generateSecurityKey())
        appUser.setUserId(profile.getDbUserID())
        appUser.setCfg(profile.getCfg())


        session.appUser = appUser
        session.userProfile = profile

        if (StringUtils.isBlank(patientEdipi) && loginSource == "TMDS") {
            session.ahltaLoginFlag = 'false'

            // Successful login audit
            StringBuilder auditSuccess = new StringBuilder()
            auditSuccess.append(loginSource + "TokenLoginSuccess")
            auditSuccess.append("^userEDIPI=" + userEdipi)
            auditSuccess.append("^userName=" + userName)
            auditSuccess.append("^userNCID=" + userNcid)
            auditSuccess.append("^patientEDIPI=" + patientEdipi)
            auditSuccess.append("^resourceUserID=" + resourceUserId)
            auditSuccess.append("^facilityNCID=" + facilityNcid)

            auditDao.auditDetailsQuery(loginSource + "TokenLoginAttempt", auditSuccess.toString(), appUser, null, null, null, null, null)

            // redirect(controller: 'app')
            loginResponse.setStatus(200, "")
            return loginResponse
        }

        try {
            PdwsQueryBean qb = new PdwsQueryBean();
            qb.setPatientId(patientEdipi);
            qb.setPatientIdType("EDIPI");
            qb.setInquiryType("PR");

            ResponseGUIPatientQuery response = patientDao.lookupPatientPDWS(session.appUser, qb, session.appUser.securityKey)

            // PDWS search by EDIPI returns either 0 (no match) or 1 (match) result
            if (response && response.getGuiPatients() && response.getGuiPatients().size > 0) {
                session.ahltaLoginPatient = response.getGuiPatients().get(0)
                session.ahltaLoginFlag = 'true'

                // Successful login audit
                StringBuilder auditSuccess = new StringBuilder()
                auditSuccess.append(loginSource + "LoginSuccess")
                auditSuccess.append("^userEDIPI=" + userEdipi)
                auditSuccess.append("^userName=" + userName)
                auditSuccess.append("^userNCID=" + userNcid)
                auditSuccess.append("^patientEDIPI=" + patientEdipi)
                auditSuccess.append("^resourceUserID=" + resourceUserId)
                auditSuccess.append("^facilityNCID=" + facilityNcid)

                auditDao.auditDetailsQuery(loginSource + "login", auditSuccess.toString(), appUser, null, null, null, null, null)

                // redirect(controller: 'app')
                loginResponse.setStatus(200, "")
                return loginResponse
            } else {
                log.error("Could not retrieve patient from PDWS")

                if (loginSource == "TMDS") {
                    session.ahltaLoginFlag = 'false'
                    // redirect(controller: 'app')
                    loginResponse.setStatus(200, "")
                    return loginResponse
                } else {
                    loginResponse.setStatus(403, "AHLTALoginError-PDWSRetrieve")
                    return loginResponse; //false
                }

            }
        }
        catch (Exception e) {

            if (loginSource == "TMDS") {
                log.error("Failed retrieve patient from PDWS: " + e.getMessage())
                session.ahltaLoginFlag = 'false'
                // redirect(controller: 'app')
                loginResponse.setStatus(200, "")
                return loginResponse
            } else {
                loginResponse.setStatus(403, "AHLTALoginError-PDWSRetrieve")
                return loginResponse; //false
            }
        }
    }

}

