package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.VlerDocument
import org.apache.commons.codec.binary.Base64
import org.apache.commons.io.IOUtils
import org.apache.commons.lang3.StringUtils
import org.springframework.context.ApplicationContextException
import org.w3c.dom.Document
import org.xml.sax.InputSource

import javax.servlet.ServletContext
import javax.xml.parsers.DocumentBuilder
import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.transform.*
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.stream.StreamSource

class VLERService {

    private
    static String XPATH_CCDA_TEMPLATEID = "/ClinicalDocument/templateId[@root='2.16.840.1.113883.10.20.22.1.1']";
    private static String CCDA_NODE_ID = "templateId root=\"2.16.840.1.113883.10.20.22.1.1\"";
    static String XSL_PATH_C32 = "/c32styles/nhin/cda2detail.xsl";
    static String XSL_PATH_CCDA = "/c32styles/nhin/cdaHFG.xsl";


    def VLERDao;
    def auditDao;
    def tokenUtils;

    static class VLERHtmlDocument {
        String html;
        List<GUIProgressNote> pdfAttachments = [];
    }

    def VLERHtmlDocument xmlToHtml(session, request, String vlerDocumentId, String homeCommunityId, String repositoryUniqueId, String patToken) {

        VLERHtmlDocument vlerHtmlDocument = new VLERHtmlDocument();

        def auditQueryName = "GetVLERDocument"
        auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient, null,
                "retrievingC32:" + vlerDocumentId,
                request['patient'].vistaSites[0].agency,
                request['patient'].vistaSites[0].moniker)

        VlerDocument vDoc = new VlerDocument();
        vDoc.setDocumentUniqueId(vlerDocumentId);
        vDoc.setHomeCommunityId(homeCommunityId);
        vDoc.setRepositoryUniqueId(repositoryUniqueId);
        String titleString = null;

        Document vlerDoc = null;

        try {
            vDoc = VLERDao.getVLERDocument(session.appUser, request.patient, vDoc);
        }
        catch (DaoException e) {
            log.error(e);
        }

        if (vDoc != null && vDoc.getDocument() != null) {
            try {

                String dataXML = new String(vDoc.getDocument(), "UTF-8");

                String xslTemplate = XSL_PATH_C32;
                if (isCcdaDoc(dataXML)) {
                    xslTemplate = XSL_PATH_CCDA
                }

                DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
                InputSource is = new InputSource(new StringReader(dataXML));
                Document doc = dBuilder.parse(is);

                def queryMap = [(HttpKeys.NOTE_ID)    : vDoc.getDocumentUniqueId(),
                                (HttpKeys.RECORD_DATE): vDoc.getCreationTime()]

                GUISite guiSite = new GUISite();
                guiSite.setAgency("VLER");

                String recordDetailsToken = this.generateRecordDetailsToken(
                        patToken,
                        (String) "VLER",// controllerName that will process this attachment request, ie VLERController,
                        (Map) queryMap,
                        (GUISite) guiSite,
                        (String) "",//vDoc.getSourceProtocol(),
                        true,
                        session);

                AttachmentAndTag attachmentAndTag = generateAttachmentTag(doc, patToken, recordDetailsToken, session);
                String attachmentOutput = attachmentAndTag.tag;

                if (attachmentOutput.contains("pdf")) {
                    vlerHtmlDocument.pdfAttachments.add(attachmentAndTag.attachment);
                }
                String outputHTML = transform(session, dataXML, xslTemplate);

                if (request && request.recordQuery)
                    titleString = "JLV - " + request.recordQuery[HttpKeys.VLER_DOCUMENT_INSTITUTION] + " : " + request.recordQuery[HttpKeys.VLER_DOCUMENT_NAME];
                else
                    titleString = "JLV - " + StringUtils.substringBetween(outputHTML, "<title>", "</title>");

                outputHTML = outputHTML.replace("</head>", "<title>" + titleString + "</title>" +
                        "<script type='text/javascript' src='resources/scripts/js/nhin.js' ></script>" +
                        "</head>");

                //inline the CSS style code for itext PDF conversion
                ServletContext context = session.getServletContext();
                InputStream cssResourceStream;

                if (outputHTML.contains("cda_plain.css"))
                    cssResourceStream = context.getResourceAsStream("/c32styles/nhin/cda_plain.css");
                else
                    cssResourceStream = context.getResourceAsStream("/resources/css/nhin.css");

                String cssText = IOUtils.toString(cssResourceStream, "UTF-8");
                outputHTML = "<style>" + cssText + "</style>" + outputHTML;

                if (attachmentOutput != null && attachmentOutput.length() > 1) {
                    String sectionHeader = "<section id=\"attached-file\"><header><h2>Attached File</h2></header>";

                    if (outputHTML.contains("<section id=\"healthcare-providers\">")) {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("<section id=\"healthcare-providers\">"), sectionHeader + attachmentOutput + "</section>\n<section id=\"healthcare-providers\">").toString();
                    } else if (outputHTML.contains("<section id=\"contact-info\">")) {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("<section id=\"contact-info\">"), sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                    } else if (outputHTML.contains("</ul>")) {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("</ul>") + 5, sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                    } else {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("</html>"), sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                    }
                }
                vlerHtmlDocument.html = outputHTML;

                return vlerHtmlDocument;
            }
            catch (TransformerConfigurationException e) {
                log.error(e);
            }
            catch (TransformerException e) {
                log.error(e);
            }
        }

    }

    def VLERHtmlDocument convertHtml(session, request, String recordKey, String patToken) {

        def vDoc = session.appUser.vlerDocumentsMap.get(recordKey)

        VLERHtmlDocument vlerHtmlDocument = new VLERHtmlDocument();

//        def auditQueryName = "GetVLERDocument"
//        auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient, null,
//                "retrievingC32:" + vlerDocumentId,
//                request['patient'].vistaSites[0].agency,
//                request['patient'].vistaSites[0].moniker)

        String titleString = null;
//
//        Document vlerDoc = null;
//
//        try {
//            vDoc = VLERDao.getVLERDocument(session.appUser, request.patient, vDoc);
//        }
//        catch (DaoException e) {
//            log.error(e);
//        }

        if (vDoc != null && vDoc.getDocument() != null) {
            try {

                String dataXML = new String(Base64.decodeBase64(vDoc.getDocument()), "UTF-8");

                String xslTemplate = XSL_PATH_C32;
                if (isCcdaDoc(dataXML)) {
                    xslTemplate = XSL_PATH_CCDA
                }

                DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
                DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
                InputSource is = new InputSource(new StringReader(dataXML));
                Document doc = dBuilder.parse(is);

                def queryMap = [(HttpKeys.NOTE_ID)    : vDoc.getDocumentUniqueId(),
                                (HttpKeys.RECORD_DATE): vDoc.getCreationTime()]

                GUISite guiSite = new GUISite();
                guiSite.setAgency("VLER");

                String recordDetailsToken = this.generateRecordDetailsToken(
                        patToken,
                        (String) "VLER",// controllerName that will process this attachment request, ie VLERController,
                        (Map) queryMap,
                        (GUISite) guiSite,
                        (String) "",//vDoc.getSourceProtocol(),
                        true,
                        session);

                AttachmentAndTag attachmentAndTag = generateAttachmentTag(doc, patToken, recordDetailsToken, session);
                String attachmentOutput = attachmentAndTag.tag;

                if (attachmentOutput.contains("pdf")) {
                    vlerHtmlDocument.pdfAttachments.add(attachmentAndTag.attachment);
                }
                String outputHTML = transform(session, dataXML, xslTemplate);

                if (request && request.recordQuery)
                    titleString = "JLV - " + request.recordQuery[HttpKeys.VLER_DOCUMENT_INSTITUTION] + " : " + request.recordQuery[HttpKeys.VLER_DOCUMENT_NAME];
                else
                    titleString = "JLV - " + StringUtils.substringBetween(outputHTML, "<title>", "</title>");

                outputHTML = outputHTML.replace("</head>", "<title>" + titleString + "</title>" +
                        "<script type='text/javascript' src='resources/scripts/js/nhin.js' ></script>" +
                        "</head>");

                //inline the CSS style code for itext PDF conversion
                ServletContext context = session.getServletContext();
                InputStream cssResourceStream;

                if (outputHTML.contains("cda_plain.css"))
                    cssResourceStream = context.getResourceAsStream("/c32styles/nhin/cda_plain.css");
                else
                    cssResourceStream = context.getResourceAsStream("/resources/css/nhin.css");

                String cssText = IOUtils.toString(cssResourceStream, "UTF-8");
                outputHTML = "<style>" + cssText + "</style>" + outputHTML;

                if (attachmentOutput != null && attachmentOutput.length() > 1) {
                    String sectionHeader = "<section id=\"attached-file\"><header><h2>Attached File</h2></header>";

                    if (outputHTML.contains("<section id=\"healthcare-providers\">")) {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("<section id=\"healthcare-providers\">"), sectionHeader + attachmentOutput + "</section>\n<section id=\"healthcare-providers\">").toString();
                    } else if (outputHTML.contains("<section id=\"contact-info\">")) {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("<section id=\"contact-info\">"), sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                    } else if (outputHTML.contains("</ul>")) {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("</ul>") + 5, sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                    } else {
                        outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("</html>"), sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                    }
                }
                vlerHtmlDocument.html = outputHTML;

                return vlerHtmlDocument;
            }
            catch (TransformerConfigurationException e) {
                log.error(e);
            }
            catch (TransformerException e) {
                log.error(e);
            }
        }

    }



    protected generateRecordDetailsToken(String patToken, String recordControllerName,
                                         Map recordQuery, GUISite recordSite,
                                         String recordSourceProtocol, boolean isSessionCached, session) {
        return tokenUtils.generateRecordDetailsToken(
                session.appUser.securityKey, patToken,
                recordControllerName, recordQuery,
                recordSite, recordSourceProtocol,
                isSessionCached, session.appUser.getCacheCounter());
    }


    def getNoteDataStream = {

        //retrieve cached note
        GUIProgressNote note = session.appUser.attachmentsMap.get(params[HttpKeys.RECORD_DETAILS_TOKEN]);

        def auditQueryName = "GetNoteDataStream"

        auditDao.auditDetailsQuery(auditQueryName, session.appUser, request.patient,
                "VA_VLER_ATTACHMENT: " + note.getContentType(),
                "VLER",
                "VA")

        if ((note == null)) {
            response.sendError 500
            return false;
        }

        response.setHeader("Pragma", "public");
        response.setHeader("Cache-Control", "maxage=1");

        if (note != null) {
            response.setContentType(note.getContentType());

            String contentDisposition = note.getContentType();
            response.setHeader("Content-Disposition", contentDisposition.replace("attachment", "inline"))


            response.getOutputStream().write(Base64.decodeBase64(note.getNoteText()));
            response.getOutputStream().flush();
            response.getOutputStream().close();

            return true;
        }

        //if note attachment is null, send an error message

        String errorMsg = "An error occurred while attempting to retrieve note. - JLV";

        response.setContentType("text");
        response.setContentLength(errorMsg.length());
        response.getWriter() << errorMsg;
        response.getWriter().flush();

        response.sendError 500
        return false;
    }

    public String transform(session, String xmlString, String xslTemplate) throws TransformerConfigurationException, TransformerException {
        try {

            ServletContext context = session.getServletContext();
            InputStream xslInput = context.getResourceAsStream(xslTemplate);

            String xslInputPath = context.getResource(xslTemplate).toExternalForm();
            String xslSystemId = xslInputPath.substring(0, xslInputPath.lastIndexOf('/') + 1);

            StreamSource xmlInput = new StreamSource(new StringReader(xmlString));

            Source xslStreamSource = null;
            if (XSL_PATH_C32.equalsIgnoreCase(xslTemplate)) {
                xslStreamSource = new StreamSource(xslInput, xslSystemId);
            } else {
                xslStreamSource = new StreamSource(xslInput, xslInputPath);
            }

            TransformerFactory factory = TransformerFactory.newInstance();
            try {
                //override Websphere's TransformerFactory
                Class<?> transformClass = Class.forName("org.apache.xalan.processor.TransformerFactoryImpl")
                factory = TransformerFactory.newInstance("org.apache.xalan.processor.TransformerFactoryImpl", null);
            } catch (ClassNotFoundException e) {
                //org.apache.xalan.processor.TransformerFactoryImpl doesn't exist (Websphere specific)
                log.debug(e);
            }
            Transformer transformer = factory.newTransformer(xslStreamSource);

            StringWriter outputWriter = new StringWriter();
            Result outputResult = new StreamResult(outputWriter);
            transformer.transform(xmlInput, outputResult);
            return outputWriter.toString();

        } catch (Exception e) {
            e.printStackTrace(System.out);
        }

        return "An error occurred while attempting to retrieve the document: Error during transform";
    }

    protected Source getStylesheetSource(String xslTemplate) throws ApplicationContextException {

        try {

            URL requestURL = new URL(request.getRequestURL().toString())
            URL xslURL = new URL(requestURL, xslTemplate);
            URL url = xslURL;

            String urlPath = url.toString();
            if (XSL_PATH_CCDA.equals(xslTemplate)) {
                return new StreamSource(url.openStream(), urlPath);
            } else {
                String systemId = urlPath.substring(0, urlPath.lastIndexOf('/') + 1);
                return new StreamSource(url.openStream(), systemId);
            }
        } catch (IOException ex) {
            throw new ApplicationContextException("Can't load XSLT stylesheet from " + xslTemplate, ex);
        }
    }

    private class AttachmentAndTag {
        String tag;
        GUIProgressNote attachment;
    }

    private AttachmentAndTag generateAttachmentTag(Document doc, String patToken, String recordToken, session) {
        StringBuffer attachmentOutput = new StringBuffer("");
        AttachmentAndTag attachmentAndTag = new AttachmentAndTag();
        GUIProgressNote guiNote = new GUIProgressNote();

        try {
            String encodedText = "";
            String decodedText = "";
            String mediaType = "";

            if (doc != null && doc.getElementsByTagName("text") != null
                    && doc.getElementsByTagName("text").item(0) != null
                    && !doc.getElementsByTagName("text").item(0).getTextContent().isEmpty()) {
                encodedText = doc.getElementsByTagName("text").item(0).getTextContent();
                decodedText = new String(Base64.decodeBase64(encodedText), "UTF-8");

                if (doc.getElementsByTagName("text").item(0).getAttribute("mediaType") != null) {
                    mediaType = doc.getElementsByTagName("text").item(0).getAttribute("mediaType")
                }

            }

            if (mediaType.contains("plain") || mediaType.contains("html")) {
                if (StringUtils.containsIgnoreCase(decodedText, "<html>")) {
                    attachmentOutput.append("<iframe id='vlerAttachedText' src='data:");
                    attachmentOutput.append(mediaType);
                    attachmentOutput.append(";base64,");
                    attachmentOutput.append(encodedText);
                    attachmentOutput.append("'");
                } else {
                    attachmentOutput.append("<pre>");
                    attachmentOutput.append(decodedText);
                    attachmentOutput.append("</pre>");
                }

            } else if (mediaType.contains("image")) {
                attachmentOutput.append("<img id='vlerAttachedImg' src='data:");
                attachmentOutput.append(mediaType);
                attachmentOutput.append(";base64,");
                attachmentOutput.append(encodedText);
                attachmentOutput.append("' />");
            } else if (mediaType.contains("application") || mediaType.contains("rtf")) {
                guiNote.setNoteAsBinaryStream(true);
                guiNote.setContentType(mediaType);
                guiNote.setNoteText(encodedText);
                session.appUser.attachmentsMap.put(recordToken, guiNote);
                attachmentOutput.append("<a download='VLER_attachment' target='_blank' id='vlerAttachedFile' title='Attached Document' href='VLER/getNoteDataStream?patToken=");
                attachmentOutput.append(patToken);
                attachmentOutput.append("&recToken=");
                attachmentOutput.append(recordToken);
                attachmentOutput.append("'>" + mediaType + "</a>");
            }
        } catch (Exception e) {
            log.error("Could not parse the VLER attachment: " + e.getMessage());
            attachmentOutput.append("<span class='vlerError'>There was an error while attempting to parse the attached file.</span>");
        }
        attachmentAndTag.tag = attachmentOutput.toString();
        attachmentAndTag.attachment = guiNote;

        return attachmentAndTag;
    }

/*
* Return true if XML document is CCDA document.
*
* @param ccdDoc VLER CCD XML document.
* @return true if root CCDA template ID is present.  Otherwise, false.
* @throws VlerDocumentUtilException If an error occurs while parsing the CCD.
*/

    public static boolean isCcdaDoc(String ccdXML) throws ApplicationContextException {
        if (ccdXML.indexOf(VLERService.CCDA_NODE_ID) > 0) {
            return true;
        }
        return false;
    }

}