package gov.va.med.common

import gov.va.med.guibeans.GUIPatient
import gov.va.med.guibeans.GUIUser
import groovyx.net.http.ContentType
import groovyx.net.http.HTTPBuilder
import groovyx.net.http.Method
import org.apache.log4j.Logger
import org.apache.pdfbox.multipdf.PDFMergerUtility
import org.apache.pdfbox.pdmodel.PDDocument
import org.apache.pdfbox.pdmodel.PDPageContentStream
import org.apache.pdfbox.pdmodel.common.PDRectangle
import org.apache.pdfbox.pdmodel.font.PDFont
import org.apache.pdfbox.pdmodel.font.PDType1Font
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Component

//import jlv.PrinterService;
@Component
class PrintUtils {
    private static final Logger logger = Logger.getLogger(PrintUtils.class.getName());

    @Autowired
    protected AppConfig appConfig


    static String header(GUIPatient patient, GUIUser user, String dateString) {

        StringBuilder sb = new StringBuilder();
        sb.append("<h3 style=\"width:100%;text-align:center;\">Medical Record</h3>");

        sb.append("<div style=\"font-size:14px;padding:0 0 15px 0;\">");
        if (patient.getName() != null) {
            sb.append("" + patient.getName() + " &nbsp;&nbsp;&nbsp; ");
        }
        sb.append("<b>DOB: </b>");
        if (patient.getDob() != null) {
            sb.append("" + patient.getDob() + " &nbsp;&nbsp;&nbsp; ");
        }


        if (user.getAgency().contains("VA")) {
            sb.append("<b>SSN: </b>");
            if (patient.getSsn() != null) {
                if (patient.getSsn().length() == 9) {
                    sb.append("***-**-" + patient.getSsn().substring(patient.getSsn().length() - 4));
                } else {
                    sb.append("SSN Unavailable");
                }
            } else {
                sb.append("SSN Unavailable");

            }
        } else {
            sb.append("<b>DoD ID: </b>");
            if (patient.getEDIPI() != null) {
                if (patient.getEDIPI().length() == 10) {
                    sb.append("******" + patient.getEDIPI().substring(patient.getEDIPI().length() - 4));
                } else {
                    sb.append("DoD ID Unavailable");
                }
            } else {
                sb.append("DoD ID Unavailable");
            }

        }

        sb.append(" &nbsp;&nbsp;&nbsp; ");

        sb.append("<b>Created: </b>");
        sb.append(dateString);

        sb.append("</div>");

        return sb.toString();

    }

    static String headerReportBuilder(GUIPatient patient, GUIUser user, String dateString) {

        StringBuilder sb = new StringBuilder();

        sb.append("<h3 style=\"width:100%;text-align:center;\">Medical Record</h3>");

        sb.append("<div style=\"font-size:14px;margin:0 0 15px 0;\">");
        if (patient.getName() != null) {
            sb.append("" + patient.getName() + " &nbsp;&nbsp;&nbsp; ");
        }
        sb.append("<b>DOB: </b>");
        if (patient.getDob() != null) {
            sb.append("" + patient.getDob() + " &nbsp;&nbsp;&nbsp; ");
        }

        sb.append("<b>SSN: </b>");
        if (patient.getSsn() != null) {
            sb.append("***-**-" + patient.getSsn().substring(patient.getSsn().length() - 4));
        }

        sb.append(" &nbsp;&nbsp;&nbsp; ");


        sb.append("<b>Created: </b>");
        sb.append(dateString);


        sb.append("</div>");

        return sb.toString();
    }

    static GUIPatient createPatient(Map<String, String> map) {

        GUIPatient patient = new GUIPatient();
        patient.setName(map.get("name"));
        patient.setDob(map.get("dob"));
        patient.setGender(map.get("gender"));
        patient.setSsn(map.get("fmpssn"));
        patient.setEDIPI(map.get("edipi"));

        return patient;
    }

    byte[] renderHtmlToPdf(String html) {

        def data
        def http = new HTTPBuilder(appConfig.getJlvPdfServiceURL())
        http.request(Method.POST, ContentType.BINARY) { req ->
            uri.path = "html2pdf"
            requestContentType = ContentType.JSON
            body = [html: html]
            response.'200' = { resp, binary ->
                data = binary.bytes //binary in this case is an instance of InputStream
            }

            response.failure = { resp ->
                throw new PrintUtilsException("There was an error retrieving the file")
            }
        }

        http.shutdown()

        return data

    }

    byte[] printToPdf(String html, boolean landscape) {

        def data
        def http = new HTTPBuilder(appConfig.getJlvPdfServiceURL())
        http.request(Method.POST, ContentType.BINARY) { req ->
            uri.path = "print"
            requestContentType = ContentType.JSON
            body = [html: html, landscape: landscape]
            response.'200' = { resp, binary ->
                data = binary.bytes //binary in this case is an instance of InputStream
            }

            response.failure = { resp ->
                throw new PrintUtilsException("There was an error retrieving the file")
            }
        }

        http.shutdown()

        return data

    }

    byte[] printReport(String reportHtml, List<PrintAttachment> attachments) {

        byte[] reportPdf = printToPdf(reportHtml, false)

        PDFMergerUtility merger = new PDFMergerUtility()

        merger.addSource(new ByteArrayInputStream(reportPdf))

        int idx = 1

        attachments.each { att ->

            def bytes = att.getBytes()

            PDDocument pdDocument = PDDocument.load(bytes)

            def allPages = pdDocument.getDocumentCatalog().getPages()

            String attachmentStamp = "Attachment " + idx

            allPages.each { page ->

                PDRectangle pageSize = page.getMediaBox();
                PDPageContentStream contentStream = new PDPageContentStream(pdDocument, page, true, true, true);

                //page.getResources().getFonts();
                PDFont font = PDType1Font.TIMES_ITALIC
                float fontSize = 10.0f

                contentStream.beginText()
                contentStream.newLineAtOffset((float) 25.0, (float) pageSize.getHeight() - 25.0)
                // set font and font size
                contentStream.setFont(font, fontSize)

                contentStream.showText(attachmentStamp)
                contentStream.endText()

                //contentStream.
                contentStream.close()



            }

            ByteArrayOutputStream out = new ByteArrayOutputStream()
            pdDocument.save(out)
            merger.addSource(new ByteArrayInputStream(out.toByteArray()))
            pdDocument.close()

            idx++
        }

        ByteArrayOutputStream out = new ByteArrayOutputStream()

        merger.setDestinationStream(out)
        merger.mergeDocuments(null)

        return out.toByteArray()

    }

}
