/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.common;

import org.apache.commons.codec.EncoderException;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.net.URLCodec;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
//import javax.crypto.spec.GCMParameterSpec;
//import javax.xml.bind.DatatypeConverter;
//import java.nio.charset.StandardCharsets;
//import java.util.Arrays;

/**
 *
 *
 */
@Component
public class AESCryptoUtil {

    private static final String CIPHER_TRANSFORMATION = "AES/CBC/PKCS5Padding";
    private static final String ALGORITHM = "AES";
    private static final Logger logger = Logger.getLogger(AESCryptoUtil.class.getName());
    @Autowired
    private AppConfig appConfig;

    public byte[] encrypt(byte[] securityKey, String message) {
        try {
            SecretKeySpec skeySpec = new SecretKeySpec(securityKey, ALGORITHM);
            Cipher cipher = Cipher.getInstance(CIPHER_TRANSFORMATION);
            cipher.init(Cipher.ENCRYPT_MODE, skeySpec, new IvParameterSpec(appConfig.getIv()));
            return cipher.doFinal(message.getBytes("UTF-8"));

        } catch (Exception e) {
            throw new RuntimeException("AES encryption failed", e);
        }
    }

    public byte[] decrypt(byte[] securityKey, byte[] encrypted) {
        try {
            SecretKeySpec skeySpec = new SecretKeySpec(securityKey, ALGORITHM);
            Cipher cipher = Cipher.getInstance(CIPHER_TRANSFORMATION);
            cipher.init(Cipher.DECRYPT_MODE, skeySpec, new IvParameterSpec(appConfig.getIv()));
            return cipher.doFinal(encrypted);
        } catch (Exception e) {
            throw new RuntimeException("AES decryption failed", e);
        }
    }

    //TODO: when we move to JDK 1.8,  enable the following code (more secure than AES/CBC/PKCS5Padding)
//    public byte[] encrypt(byte[] securityKey, String message) {
//        try {
//            SecureRandom r = SecureRandom.getInstance("SHA1PRNG");
//            // Generate 128 bit IV for Encryption
//            byte[] iv = new byte[12]; r.nextBytes(iv);
//
//            SecretKeySpec eks = new SecretKeySpec(securityKey, "AES");
//            Cipher c = Cipher.getInstance("AES/GCM/NoPadding");
//
//            // Generated Authentication Tag should be 128 bits
//            c.init(Cipher.ENCRYPT_MODE, eks, new GCMParameterSpec(128, iv));
//            byte[] es = c.doFinal(message.getBytes(StandardCharsets.UTF_8));
//
//            // Construct Output as "IV + CIPHERTEXT"
//            byte[] os = new byte[12 + es.length];
//            System.arraycopy(iv, 0, os, 0, 12);
//            System.arraycopy(es, 0, os, 12, es.length);
//
//            // Return a Base64 Encoded String
//            return DatatypeConverter.printBase64Binary(os).getBytes("UTF-8");
//
//        } catch (Exception e) {
//            throw new RuntimeException("AES encryption failed", e);
//        }
//    }
//
//    public byte[] decrypt(byte[] securityKey, byte[] encrypted) throws Exception {
//        try {
//            byte[] os = DatatypeConverter.parseBase64Binary(encrypted.toString());
//
//            // Check Minimum Length (IV (12) + TAG (16))
//            if (os.length > 28) {
//                byte[] iv = Arrays.copyOfRange(os, 0, 12);
//                byte[] es = Arrays.copyOfRange(os, 12, os.length);
//
//                // Perform Decryption
//                SecretKeySpec dks = new SecretKeySpec(securityKey, "AES");
//                Cipher c = Cipher.getInstance("AES/GCM/NoPadding");
//                c.init(Cipher.DECRYPT_MODE, dks, new GCMParameterSpec(128, iv));
//
//                // Return our Decrypted String
//                return new String(c.doFinal(es), StandardCharsets.UTF_8).getBytes("UTF-8");
//            }
//        } catch (Exception e) {
//            throw new RuntimeException("AES decryption failed", e);
//        }
//
//        return "".getBytes("UTF-8");
//    }


    public byte[] generateSecurityKey() {
        KeyGenerator generator;
        try {
            generator = KeyGenerator.getInstance(ALGORITHM);
            generator.init(128);
            return generator.generateKey().getEncoded();

        } catch (Exception e) {
            throw new RuntimeException("AES generateSecurityKey failure", e);
        }
    }

    public boolean isEncryptedTokenDebug() {
        return appConfig.isEncryptedTokenDebug();
    }

    public String generateEncryptedQuery(byte[] securityKey, String queryParams) {
        if (isEncryptedTokenDebug()) {
            try {
                URLCodec urlCodec = new URLCodec();
                return urlCodec.encode(queryParams);
            } catch (EncoderException ex) {
                logger.error(ex.getMessage(), ex);
                return null;
            }
        } else {
            byte[] encryptedBytes = encrypt(securityKey, queryParams);
            return Base64.encodeBase64URLSafeString(encryptedBytes);
        }

    }
}
