package gov.va.med.common;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.*;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public final class SSOiUtil {

    private static final Logger logger = Logger.getLogger(SSOiUtil.class.getName());

    private SSOiUtil() {
    }

    public static String getSSOiCookie(String header) {
        if (StringUtils.isNotEmpty(header)) {
            int beginIndex = header.lastIndexOf("IAMSESSION=");
            if (beginIndex > -1) {
                int endIndex = header.indexOf(";", beginIndex);
                if (endIndex < 0) {
                    endIndex = header.length();
                }
                if (endIndex > 0) {
                    return header.substring(beginIndex + 11, endIndex);
                }
            }
        }
        return null;
    }

    /**
     * Return very first site code as home site
     *
     * @param vistaIdHeader VistA ID HTTP header
     */
    public static String getHomeSiteFromHeader(String vistaIdHeader) {
        if (StringUtils.isNotEmpty(vistaIdHeader)) {
            int firstSiteCodeIndex = vistaIdHeader.indexOf("|");
            if (firstSiteCodeIndex > 0) {
                return vistaIdHeader.substring(0, firstSiteCodeIndex);
            }
        }
        return null;
    }

    public static List<String> getLinkedSitesFromHeader(String vistaIdHeader) {
        List<String> linkedSites = null;
        if (StringUtils.isNotEmpty(vistaIdHeader)) {
            linkedSites = new ArrayList<>();
            StringTokenizer tokenizer = new StringTokenizer(vistaIdHeader, ",");
            while (tokenizer.hasMoreTokens()) {
                String token = tokenizer.nextToken();
                if (token.indexOf("|") > -1) {
                    linkedSites.add(token.substring(0, token.indexOf("|")));
                }
            }
        }
        return linkedSites;
    }

    public static List<String> getLinkedSitesFromToken(String samlToken) {
        List<String> linkedSites = null;
        if (StringUtils.isNotEmpty(samlToken)) {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder;
            try {
                builder = factory.newDocumentBuilder();
            } catch (ParserConfigurationException e) {
                logger.error(e);
                return null;
            }
            Document doc;
            try (StringReader reader = new StringReader(samlToken)) {
                doc = builder.parse(new InputSource(reader));
            } catch (SAXException e) {
                logger.error(e);
                return null;
            } catch (IOException e) {
                logger.error(e);
                return null;
            }

            XPathFactory xPathFactory = XPathFactory.newInstance();
            XPath xPath = xPathFactory.newXPath();
            NodeList nodes;
            try {
                XPathExpression expr = xPath.compile("//*[local-name()='Attribute'][@Name='urn:va:vrm:iam:vistaid']/*[local-name()='AttributeValue']/text()");
                nodes = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                logger.error(e);
                return null;
            }

            if (nodes.getLength() > 0) {
                linkedSites = new ArrayList<>(nodes.getLength());
                for (int i = 0; i < nodes.getLength(); i++) {
                    String nodeValue = nodes.item(i).getNodeValue();
                    if (nodeValue.contains("-")) {
                        linkedSites.add(nodeValue.substring(0, nodeValue.indexOf("-")));
                    } else {
                        logger.error("Invalid node value: " + nodeValue);
                        linkedSites.add(nodeValue);
                    }
                }
            }
        }
        return linkedSites;
    }

}
