/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.common.AppConfig;
import gov.va.med.common.JMeadowsQueryBuilderFactory;
import gov.va.med.common.TokenUtils;
import gov.va.med.guibeans.*;
import gov.va.med.jlvqos.JLVQoSDataServiceHandler;
import gov.va.med.jmeadows.JMeadowsServiceHandler;
import gov.va.med.jmeadows.webservice.*;
import org.apache.log4j.Logger;
import org.dozer.DozerBeanMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.Comparator;


/**
 *
 *
 */
@Component
public class DaoBase {
    private static final Logger logger = Logger.getLogger(DaoBase.class.getName());



    @Autowired
    protected JMeadowsServiceHandler serviceHandler;

    @Autowired
    protected DozerBeanMapper dozerBeanMapper;

    @Autowired
    protected AppConfig appConfig;

    @Autowired
    protected TokenUtils tokenUtils;

    @Autowired
    protected JMeadowsQueryBuilderFactory jMeadowsQueryBuilderFactory;


    public DaoBase() {
//        if (appConfig.getTrustAllSSLHostnamesAndCerts())
//        {
//            SSLDebugUtilities.trustAllHostnames();
//            SSLDebugUtilities.trustAllHttpsCertificates();
//        }
    }

    /**
     * Get GUISite without endpoints or region
     *
     * @param site JMeadows Site
     * @return GUISite without endpoints or region
     */
    protected GUISite getBasicSiteData(Site site) {
        if (site == null) return null;

        GUISite guiSite = new GUISite();
        guiSite.setSiteCode(site.getSiteCode());
        guiSite.setAgency(site.getAgency());
        guiSite.setId(site.getId());
        guiSite.setMoniker(site.getMoniker());
        guiSite.setName(site.getName());
        guiSite.setDmisId(site.getDmisId());

        return guiSite;
    }

    protected void addSiteDataToGUIBean(GUIBean guiBean, DataBean vistaBean) {
        if (vistaBean.getSite() != null) {
            guiBean.setSite(getBasicSiteData(vistaBean.getSite()));
        } else {
            guiBean.setSite(new GUISite());
        }

        guiBean.setSourceProtocol(vistaBean.getSourceProtocol());
    }

    protected void addSiteDataToGUIBean(GUIBean guiBean, PatientDemographics vistaBean) {
        if (vistaBean.getSite() != null) {
            guiBean.setSite(getBasicSiteData(vistaBean.getSite()));
        }

        guiBean.setSourceProtocol(vistaBean.getSourceProtocol());
    }

    protected void addSiteDataToGUIBean(GUIVitalsGraphEntry guiBean, DataBean vistaBean) {
        if (vistaBean.getSite() != null) {
            guiBean.setSite(getBasicSiteData(vistaBean.getSite()));
        }

        guiBean.setSourceProtocol(vistaBean.getSourceProtocol());
    }

    /**
     * Generates a patient token that is derived from input data. \
     * <p/>
     * A patient token is essential an AES encrypted string of patient parameters.
     *
     * @param encryptSecurityKey encryption key (required)
     * @param idType             patient identifier type - EDIPI, IEN, etc. (required)
     * @param patientId          patient identifier (required)
     * @return An patient token.
     */
    protected String generatePatientToken(byte[] encryptSecurityKey, String idType, String patientId) {
        return tokenUtils.generatePatientToken(encryptSecurityKey, idType, patientId, "");
    }

    protected JMeadowsServiceHandler getServiceHandler() {
        return serviceHandler;
    }

    public static class SiteAlphaComparator implements Comparator<GUIBean>, Serializable {
        public int compare(GUIBean o1, GUIBean o2) {
            return o1.getSite().getMoniker().compareTo(o2.getSite().getMoniker());
        }
    }

    protected String getProvQueryPatientToken(byte[] encryptSecurityKey, DataBean bean) {
        String patToken = null;

        if (appConfig.getAgencyDOD().equalsIgnoreCase(bean.getSite().getAgency())) {
            patToken = generatePatientToken(encryptSecurityKey, appConfig.getIdTypeEDIPI(), bean.getPatientId());
        } else if (appConfig.getAgencyVA().equalsIgnoreCase(bean.getSite().getAgency())) {
            patToken = tokenUtils.generatePatientToken(encryptSecurityKey, appConfig.getIdTypeVAIen(),
                    bean.getPatientId(), bean.getSite().getSiteCode());
        }

        return patToken;
    }
}
