package gov.va.med.dao;

import gov.va.med.guibeans.GUIHealthSummaryDetail;
import gov.va.med.guibeans.GUIHealthSummaryType;
import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.FreeTextReport;
import gov.va.med.jmeadows.webservice.HealthSummaryType;
import gov.va.med.jmeadows.webservice.HealthSummaryTypeList;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class HealthSummariesDao extends DaoBase {

    public List<GUIHealthSummaryType> getPatientHealthSummaryTypes(GUIUser user, GUIPatient patient) {

        try {

            List<GUIHealthSummaryType> guiHealthSummaryTypeList = new ArrayList<GUIHealthSummaryType>();

            List<HealthSummaryTypeList> typeLists = getServiceHandler().getPatientHealthSummaryTypes(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .build()
            );

            for (HealthSummaryTypeList typeList : typeLists) {

                List<HealthSummaryType> types = typeList.getHealthSummaryTypes();

                for (HealthSummaryType type : types) {

                    GUIHealthSummaryType guiHealthSummaryType =
                            dozerBeanMapper.map(type, GUIHealthSummaryType.class);

                    guiHealthSummaryTypeList.add(guiHealthSummaryType);
                }

            }

            return guiHealthSummaryTypeList;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIHealthSummaryDetail getPatientHealthSummary(
            GUIUser user, GUIPatient patient, String itemId, String recordSiteCode) {

        try {

            FreeTextReport report = getServiceHandler().getPatientHealthSummary(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .itemId(itemId)
                            .recordSiteCode(recordSiteCode)
                            .build()
            );

            GUIHealthSummaryDetail detail = new GUIHealthSummaryDetail();
            detail.setReportText(report.getReportText());

            return detail;


        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

}
