package gov.va.med.dao;

import gov.va.med.guibeans.GUIMhsGenesis;
import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.MhsGenesis;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class MhsGenesisDao extends DaoBase
{
    private static final Logger LOGGER = Logger.getLogger(MhsGenesisDao.class.getName());

    /**
     * Retrieves a list of patient MHS Genesis documents from all available patient locations.
     *
     * @param user    {@link gov.va.med.guibeans.GUIUser} making the requet.
     * @param patient {@link gov.va.med.guibeans.GUIPatient} patient retrieving MHS Genesis documents from this patient record.
     * @return A list of patient MHS Genesis documents.
     */
    public List<GUIMhsGenesis> getMhsGenesisDocuments(GUIUser user, GUIPatient patient)
    {
        try {
            MhsGenesis[] genesisList = getServiceHandler().getMhsGenesisDocuments(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .build());

            List<GUIMhsGenesis> guiGenesisList = new ArrayList<GUIMhsGenesis>();
            GUIMhsGenesis guiGenesis;
            StringBuilder docId = new StringBuilder();
            int id = 0;

            for (MhsGenesis genesis : genesisList) {
                guiGenesis = dozerBeanMapper.map(genesis, GUIMhsGenesis.class);
                guiGenesis.setTitle(genesis.getTitle());

                if (genesis.getEnteredDate() != null) {
                    guiGenesis.setDate(Utils.formatDisplayDate(genesis.getEnteredDate()));
                }

                docId.setLength(0);

                if (StringUtils.equalsIgnoreCase(appConfig.getProtocolDODAdapter(), genesis.getSourceProtocol())) {
                    guiGenesis.setSessionCachedRecord(true);
                    docId.append(appConfig.getCachedTag());
                    docId.append(id);
                    id++;
                }

                guiGenesis.setEventId(docId.toString());

                // There is a complex data url, therefore it has a note image binary stream
                if (StringUtils.isNotBlank(guiGenesis.getComplexDataUrl())) {
                    guiGenesis.setNoteAsBinaryStream(true);
                }

                guiGenesisList.add(guiGenesis);
            }

            return guiGenesisList;
        }
        catch (Exception ex) {
            throw new DaoException(ex);
        }
    }
}
