/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.JMeadowsQuery;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;


/**
 *
 *
 */
@Component
public class SensitiveRecordDao extends DaoBase {
    private static final Logger logger = Logger.getLogger(SensitiveRecordDao.class.getName());

    public void auditSensitiveRecordAccess(GUIUser user, GUIPatient patient, String info, String recordId, String scId,
                                           String scEmail, String complexTransaction, String siteMoniker, String siteAgency) {
        try {
            JMeadowsQuery queryBean =
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .itemId(recordId)
                            .complexTransaction(complexTransaction)
                            .siteMoniker(siteMoniker)
                            .siteAgency(siteAgency)
                            .build();
            StringBuilder infoSB = new StringBuilder(info);

            if ("VA".equals(user.getAgency())) {
                if ("VBA".equals(user.getHostSite().getMoniker())) {
                    infoSB.append("^Org:VBA");
                } else {
                    infoSB.append("^Org:VHA");
                }
//                infoSB.append("^SCID:");
//                infoSB.append(scId);
//                infoSB.append("^SCEmail:");
//                infoSB.append(scEmail);
            } else {
                infoSB.append("^Org:DoD");
            }
//            infoSB.append("^U_IEN:");
//            infoSB.append(user.getUserIen());

            queryBean.setInfo(infoSB.toString());

            getServiceHandler().auditSensitiveRecordAccess(queryBean);
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

}
