package gov.va.med.saml;

import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;
import gov.va.med.guibeans.GUIUser;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.codehaus.groovy.grails.web.context.ServletContextHolder;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

@Component
public class SAMLTokenServiceHandlerImpl implements SAMLTokenServiceHandler{

    private static final Logger logger = Logger.getLogger(SAMLTokenServiceHandlerImpl.class.getName());

    private static Configuration cfg;

    private void initTemplateConfiguration() {
        cfg = new Configuration(Configuration.VERSION_2_3_25);
        cfg.setServletContextForTemplateLoading(ServletContextHolder.getServletContext(), "WEB-INF/templates");
        cfg.setDefaultEncoding("UTF-8");
        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
        cfg.setLogTemplateExceptions(false);
    }

    @Override
    public String createSAMLToken(GUIUser user) {
        if (cfg == null) {
            initTemplateConfiguration();
        }

        if (user != null && StringUtils.equalsIgnoreCase("DOD", user.getAgency())) {
            Map<String, Object> root = new HashMap<>();
            if (StringUtils.isNotEmpty(user.getNpi())) {
                root.put("NPI", user.getNpi());
            }
            if (StringUtils.isNotEmpty(user.getCardId())) {
                root.put("EDIPI", user.getCardId());
            }
            if (StringUtils.isNotEmpty(user.getEmail())) {
                root.put("EMAIL", user.getEmail());
            }
            if (StringUtils.isNotEmpty(user.getName())) {
                root.put("NAME", user.getName());
            }

            Template template;
            try {
                template = cfg.getTemplate("SAMLTokenDoD.ftlx");
            } catch (IOException e) {
                logger.error("get SAML token DoD template ERROR: " + e.getMessage(), e);
                return null;
            }

            try (Writer out = new StringWriter()) {
                template.process(root, out);
                return out.toString();
            } catch (TemplateException | IOException e) {
                logger.error("proess SAML token template ERROR: " + e.getMessage(), e);
                return null;
            }
        } else {
            logger.error("Invalid create SAML token request parameter");
            return null;
        }
    }

}
