package gov.va.med.jlvqos.webservice;

import gov.va.med.jlvqos.heartbeat.ServiceErrors;
import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.monitor.ServiceMonitor;
import gov.va.med.monitor.ServiceTask;
import gov.va.med.monitor.VistaStatusServiceMonitor;
import gov.va.med.monitor.VistaStatusTask;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServlet;
import javax.xml.soap.SOAPException;
import javax.xml.ws.WebServiceContext;
import java.util.Timer;

public class JLVQoSMonitor extends HttpServlet {

        private static final Logger logger = LoggerFactory.getLogger(JLVQoSMonitor.class);
        private boolean isFirstTime = true;
        private Timer serviceTimer = null;
        private Timer vistaStatusServiceTimer = null;
        private ServiceErrors qosServiceErrors = null;

        /**
         * Web context.
         */
        private WebServiceContext wsContext;

        public JLVQoSMonitor()
        {
            logger.debug("JLVQoSMonitor class created (" + this + ")");
        }

    public void init() {
        cancelTimers();

        initiateServiceTimer();

        logger.debug("Initialize completed");
    }

    private void initiateServiceTimer() {
        logger.debug("Initiating service timer start...");

        logger.debug("isFirstTime = " + isFirstTime);

        if (isFirstTime) {
            isFirstTime = false;

            AppConfig appConfig = AppConfig.getInstance();

            serviceTimer = new Timer();
            serviceTimer.schedule(new ServiceTask(),
                    appConfig.getServiceMonitorTaskTimerDelay(),
                    appConfig.getServiceMonitorTaskTimerPeriod());

            if (appConfig.isServiceMonitorTestVistaSites()) {
                vistaStatusServiceTimer = new Timer();
                vistaStatusServiceTimer.schedule(new VistaStatusTask(),
                        appConfig.getServiceMonitorTaskTimerDelay(),
                        appConfig.getVistaStatusServiceMonitorTaskTimerPeriod());
            }

            logger.debug("isFirstTime = " + isFirstTime);
        }

        logger.debug("Initiating service timer finished");
    }

    private void cancelServiceTimer() {
        if (serviceTimer != null) {
            serviceTimer.cancel();
            serviceTimer.purge();
        }

        logger.debug("Canceling service timers");
    }

    private void cancelVistaStatusServiceTimer() {
        if (vistaStatusServiceTimer != null) {
            vistaStatusServiceTimer.cancel();
            vistaStatusServiceTimer.purge();
        }
    }

    public String verify() {
        return ("JLVQoS WebService is running!");
    }


    public String getServiceErrors() throws SOAPException {
        String retVal = null;
        try {
            retVal = ServiceErrors.getInstance().getServiceErrors();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new SOAPException(e);
        }
        return retVal;
    }

    private void cancelTimers() {
        ServiceMonitor.getInstance().cancelEmailTimer();
        cancelServiceTimer();

        VistaStatusServiceMonitor.getInstance().cancelEmailTimer();
        cancelVistaStatusServiceTimer();
    }

    public void destroy() {
        cancelTimers();
    }

}