/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.webservice.JMeadowsData;
import gov.va.med.jmeadows.webservice.JMeadowsDataService;
import org.apache.log4j.Logger;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import java.util.Map;

public class JMeadowsServiceHandler
{

    private static final Logger logger = Logger.getLogger(JMeadowsServiceHandler.class.getName());
    private String serviceURL;
    private String m_ServiceName;
    private String m_namespace;
    private QName m_QserviceName;
    private int connectionTimeoutMS;
    private int requestTimeoutMS;
    private JMeadowsDataService m_jmeadowsDataService;

    public JMeadowsServiceHandler(String url)
    {
        serviceURL = url;
        m_ServiceName = "JMeadowsDataService";
        m_namespace = "http://webservice.jmeadows.URL       /";
        m_QserviceName = new QName(m_namespace, m_ServiceName);

        connectionTimeoutMS = 45000;
        requestTimeoutMS = 100000;

        m_jmeadowsDataService = new JMeadowsDataService();
    }

    private JMeadowsData getJMeadowsDataPort()
    {

        JMeadowsData port = m_jmeadowsDataService.getJMeadowsDataPort();

        //set connection timeout
        Map<String, Object> ctxt = ((BindingProvider) port).getRequestContext();
        ctxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);

        ctxt.put("com.sun.xml.internal.ws.connect.timeout", connectionTimeoutMS);
        ctxt.put("com.sun.xml.internal.ws.request.timeout", requestTimeoutMS);
        ctxt.put("com.sun.xml.ws.connect.timeout", connectionTimeoutMS);
        ctxt.put("com.sun.xml.ws.request.timeout", requestTimeoutMS);

        return port;
    }

    public String getVersion()
    {
        try
        {
            JMeadowsData port = getJMeadowsDataPort();
            return port.getVersion();
        }
        catch (Exception e)
        {
            throw new RuntimeException(e);
        }
    }
}
