/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.dataservicehandler;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Set;

/*
 * This simple SOAPHandler will output the contents of incoming
 * and outgoing messages.
 */
public class SOAPLoggingHandler implements SOAPHandler<SOAPMessageContext>
{

    private static final Logger logger = LoggerFactory.getLogger(SOAPLoggingHandler.class);

    public Set<QName> getHeaders()
    {
        return null;
    }

    public boolean handleMessage(SOAPMessageContext smc)
    {
        logToSystemOut(smc);
        return true;
    }

    public boolean handleFault(SOAPMessageContext smc)
    {
        logToSystemOut(smc);
        return true;
    }

    // nothing to clean up
    public void close(MessageContext messageContext)
    {
    }

    /*
     * Check the MESSAGE_OUTBOUND_PROPERTY in the context
     * to see if this is an outgoing or incoming message.
     * Write a brief message to the print stream and
     * output the message. The writeTo() method can throw
     * SOAPException or IOException
     */
    private void logToSystemOut(SOAPMessageContext smc)
    {
        Boolean outboundProperty = (Boolean)
                smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

        if (outboundProperty.booleanValue())
        {
            logger.debug("Outbound message:");
        }
        else
        {
            logger.debug("Inbound message:");
        }

        try
        {
            SOAPMessage message = smc.getMessage();

            ByteArrayOutputStream stream = new ByteArrayOutputStream();
            message.writeTo(stream);

            logger.debug(stream.toString("UTF-8"));
        }
        catch (SOAPException e)
        {
            logger.error("SOAP Messaging Exception");
        }
        catch (IOException e)
        {
            logger.error("Error writing byte array to String");
        }
    }
}
