/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.common.JMeadowsException;
import org.apache.axis.client.Stub;
import web.webservice.jvi.citpo.ha.osd.mil.ImagingDataService;
import web.webservice.jvi.citpo.ha.osd.mil.ImagingDataServiceLocator;
import web.webservice.jvi.citpo.ha.osd.mil.ImagingDataServicePortType;
import web.webservice.jvi.citpo.ha.osd.mil.Mil_osd_ha_citpo_jvi_webservice_beans_ProviderBean;

import javax.xml.rpc.ServiceException;


public class ShareImagingServiceHandler extends DataServiceHandler
{
    private ImagingDataService m_locator;


    public ShareImagingServiceHandler(String url)
    {
        super();

        serviceURL = url;
    }

    private ImagingDataServicePortType getPort() throws ServiceException
    {
        if (m_locator == null)
        {
            createService();
        }

        ImagingDataServicePortType port = m_locator.getImagingDataServicePort();

        //set timeout
        Stub s = (Stub) port;
        s.setTimeout(serviceRequestTimeoutMS);

        s._setProperty(Stub.USERNAME_PROPERTY, serviceUsername);
        s._setProperty(Stub.PASSWORD_PROPERTY, servicePassword);

        //set port endpoint address
        s._setProperty(Stub.ENDPOINT_ADDRESS_PROPERTY, serviceURL);

        return port;
    }

    private void createService()
    {
        if (m_locator == null)
        {
            synchronized (this)
            {

                try
                {
                    m_locator = new ImagingDataServiceLocator();
                }
                catch (Exception e)
                {
                    throw new RuntimeException(e);
                }
            }
        }
    }

    public Mil_osd_ha_citpo_jvi_webservice_beans_ProviderBean authenticateProvider(String accessCode, String verifyCode) throws JMeadowsException
    {
        Mil_osd_ha_citpo_jvi_webservice_beans_ProviderBean providerBean = null;

        try
        {
            ImagingDataServicePortType port = getPort();
            providerBean = port.authenticateProvider(accessCode, verifyCode);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e.getMessage());
        }

        return providerBean;
    }
}
