/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vds.webservice.*;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import java.util.List;


/**
 * @author vhahonbrunos
 */
public class VistaDataServiceHandler extends DataServiceHandler
{
    private VistaDataService m_VistaDataService;

    public VistaDataServiceHandler(String url)
    {
        super();

        serviceURL = url;
        serviceName = "VistaDataService";
        serviceNamespace = "http://webservice.vds.URL       /";
        qServiceName = new QName(serviceNamespace, serviceName);

        m_VistaDataService = new VistaDataService();
    }

    private VistaData getVistaDataPort()
    {

        VistaData port = m_VistaDataService.getVistaDataPort();

        //set port endpoint address
        BindingProvider bp = (BindingProvider) port;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        bp.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        return port;
    }

    public User login(Site site, String accessCode, String verifyCode) throws JMeadowsException
    {
        User user = null;

        try
        {
            VistaData port = getVistaDataPort();
            user = port.login(site, accessCode, verifyCode);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return user;
    }

    public AdmissionDetail getAdmissionDetails(QueryBean queryBean) throws JMeadowsException
    {
        AdmissionDetail returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getAdmissionDetails(queryBean);

        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }


        return returnVal;
    }

    public Diagnosis[] getAdmissionDiagnosis(QueryBean queryBean) throws JMeadowsException
    {
        Diagnosis[] diagnosis = null;

        try
        {

            VistaData port = getVistaDataPort();
            diagnosis = port.getAdmissionDiagnosis(queryBean).toArray(new Diagnosis[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
        return diagnosis;

    }

    public Procedure[] getAdmissionProcedures(QueryBean queryBean) throws JMeadowsException
    {
        Procedure[] admissionProcedures = null;

        try
        {
            VistaData port = getVistaDataPort();
            admissionProcedures = port.getAdmissionProcedures(queryBean).toArray(new Procedure[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return admissionProcedures;
    }

    public FreeTextReport getClinicalReminderDetail(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getClinicalReminderDetail(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }


        return returnVal;

    }

    public List<FreeTextReport> getConsultReport(QueryBean queryBean) throws JMeadowsException
    {
        try
        {
            VistaData port = getVistaDataPort();
            return port.getConsultReport(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public FreeTextReport getMedicationDetail(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getMedicationDetail(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public LabResult[] getLabOrderResult(QueryBean queryBean) throws JMeadowsException
    {
        LabResult[] orderResult = null;

        try
        {
            VistaData port = getVistaDataPort();
            orderResult = port.getLabOrderResult(queryBean).toArray(new LabResult[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return orderResult;
    }

    public FreeTextReport getOrderDetail(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getOrderDetail(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public PatientAdmission[] getPatientAdmissions(QueryBean queryBean) throws JMeadowsException
    {
        PatientAdmission[] patAdmission = null;

        try
        {
            VistaData port = getVistaDataPort();
            patAdmission = port.getPatientAdmissions(queryBean).toArray(new PatientAdmission[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return patAdmission;
    }

    /**
     * Retrieves list of patient allergies.
     *
     * @param queryBean Query parameters.
     *                  Required params: User, Patient
     * @return A list of patient allergies.
     * @throws JMeadowsException if there was an error..
     */
    public Allergy[] getPatientAllergies(QueryBean queryBean) throws JMeadowsException
    {
        Allergy[] allergy = null;

        try
        {
            VistaData port = getVistaDataPort();
            allergy = port.getPatientAllergies(queryBean).toArray(new Allergy[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return allergy;
    }

    /**
     * Retrieves details of an allergy.
     *
     * @param queryBean Query parameters.
     *                  Required params: User, Patient, itemId, recordSite
     * @return AllergyDetail containing allergy report text.
     * @throws JMeadowsException if there was an error..
     */
    public AllergyDetail getAllergyDetail(QueryBean queryBean) throws JMeadowsException
    {
        AllergyDetail allergyDetail = null;

        try
        {
            VistaData port = getVistaDataPort();
            allergyDetail = port.getAllergyDetail(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return allergyDetail;
    }

    public PatientAppointments[] getPatientAppointments(QueryBean queryBean) throws JMeadowsException
    {
        PatientAppointments[] appointments = null;

        try
        {
            VistaData port = getVistaDataPort();
            appointments = port.getPatientAppointments(queryBean).toArray(new PatientAppointments[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return appointments;
    }

    public FreeTextReport getEncountersReport(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getEncountersReport(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public ClinicalReminder[] getPatientClinicalReminders(QueryBean queryBean) throws JMeadowsException
    {
        ClinicalReminder[] reminders = null;

        try
        {
            VistaData port = getVistaDataPort();
            reminders = port.getPatientClinicalReminders(queryBean).toArray(new ClinicalReminder[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return reminders;
    }

//    public ServiceType getPatientCodeGreen(QueryBean queryBean) throws JMeadowsException {
//        ServiceType returnVal = null;
//
//        try {
//            VistaData port = getVistaDataPort();
//            returnVal = port.getPatientCodeGreen(queryBean);
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return returnVal;
//    }

    public Consult[] getPatientConsultRequests(QueryBean queryBean) throws JMeadowsException
    {
        Consult[] consults = null;

        try
        {
            VistaData port = getVistaDataPort();
            consults = port.getPatientConsultRequests(queryBean).toArray(new Consult[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return consults;
    }

    public Vitals[] getPatientCurrentVitals(QueryBean queryBean) throws JMeadowsException
    {
        Vitals[] vitals = null;

        try
        {
            VistaData port = getVistaDataPort();
            vitals = port.getPatientCurrentVitals(queryBean).toArray(new Vitals[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return vitals;

    }

    public PatientDemographics[] getPatientDemographics(QueryBean queryBean) throws JMeadowsException
    {
        PatientDemographics[] returnVal = null;
        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getPatientDemographics(queryBean).toArray(new PatientDemographics[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public ProgressNote[] getPatientDischargeSummaries(QueryBean queryBean) throws JMeadowsException
    {
        ProgressNote[] notes = null;

        try
        {
            VistaData port = getVistaDataPort();
            notes = port.getPatientDischargeSummaries(queryBean).toArray(new ProgressNote[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return notes;
    }

//    public Boolean isSensitivePatient(QueryBean queryBean) throws JMeadowsException
//    {
//        try
//        {
//            VistaData port = getVistaDataPort();
//            return port.isSensitivePatient(queryBean);
//        }
//        catch (Exception e)
//        {
//            throw new JMeadowsException(e);
//        }
//    }

    public PatientDemographicsDetail[] getPatientDemographicsDetail(QueryBean queryBean) throws JMeadowsException
    {
        PatientDemographicsDetail[] returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getPatientDemographicsDetail(queryBean).toArray(new PatientDemographicsDetail[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public Immunization[] getPatientImmunizations(QueryBean queryBean) throws JMeadowsException
    {
        Immunization[] immunization = null;
        try
        {
            VistaData port = getVistaDataPort();
            immunization = port.getPatientImmunizations(queryBean).toArray(new Immunization[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return immunization;
    }

    public LabOrder[] getPatientLabs(QueryBean queryBean) throws JMeadowsException
    {
        LabOrder[] orders = null;
        try
        {
            VistaData port = getVistaDataPort();
            orders = port.getPatientLabs(queryBean).toArray(new LabOrder[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return orders;
    }

    public LabResult[] getPatientLabTestResults(QueryBean queryBean) throws JMeadowsException
    {
        LabResult[] results = null;
        try
        {
            VistaData port = getVistaDataPort();
            results = port.getPatientLabTestResults(queryBean).toArray(new LabResult[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return results;
    }

    public Medication[] getPatientMedications(QueryBean queryBean) throws JMeadowsException
    {
        Medication[] medication = null;
        try
        {
            VistaData port = getVistaDataPort();
            medication = port.getPatientMedications(queryBean).toArray(new Medication[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return medication;
    }

//    public Medication[] getPatientMedicationsNVA(QueryBean queryBean) throws JMeadowsException {
//        Medication[] medication = null;
//        try {
//            VistaData port = getVistaDataPort();
//            medication = port.getPatientMedicationsNVA(queryBean).toArray(new Medication[0]);
//
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return medication;
//
//    }

    public Order[] getPatientOrders(QueryBean queryBean) throws JMeadowsException
    {
        Order[] orders = null;

        try
        {
            VistaData port = getVistaDataPort();
            orders = port.getPatientOrders(queryBean).toArray(new Order[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return orders;
    }

    public Problem[] getPatientProblemList(QueryBean queryBean) throws JMeadowsException
    {
        Problem[] problems = null;

        try
        {
            VistaData port = getVistaDataPort();
            problems = port.getPatientProblemList(queryBean).toArray(new Problem[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return problems;
    }

    public Procedure[] getPatientProcedures(QueryBean queryBean) throws JMeadowsException
    {
        Procedure[] procedure = null;

        try
        {
            VistaData port = getVistaDataPort();
            procedure = port.getPatientProcedures(queryBean).toArray(new Procedure[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return procedure;

    }

    public ProgressNote[] getPatientProgressNotes(QueryBean queryBean) throws JMeadowsException
    {
        ProgressNote[] notes = null;

        try
        {
            VistaData port = getVistaDataPort();
            notes = port.getPatientProgressNotes(queryBean).toArray(new ProgressNote[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return notes;
    }

    public RadiologyReport[] getPatientRads(QueryBean queryBean) throws JMeadowsException
    {
        RadiologyReport[] exams = null;

        try
        {
            VistaData port = getVistaDataPort();
            exams = port.getPatientRads(queryBean).toArray(new RadiologyReport[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return exams;
    }

    public Vitals[] getPatientVitals(QueryBean queryBean) throws JMeadowsException
    {
        Vitals[] vitals = null;

        try
        {
            VistaData port = getVistaDataPort();
            vitals = port.getPatientVitals(queryBean).toArray(new Vitals[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return vitals;
    }

//    public Prescription getPrescriptionFills(QueryBean queryBean) throws JMeadowsException {
//        Prescription returnVal = null;
//
//        try {
//            VistaData port = getVistaDataPort();
//            returnVal = port.getPrescriptionFills(queryBean);
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return returnVal;
//
//    }

    public ProblemDetail getProblemDetail(QueryBean queryBean) throws JMeadowsException
    {
        ProblemDetail returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getProblemDetail(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public FreeTextReport getProgressNote(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getProgressNote(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public User getProviderProfile(Site site, String userIen, String providerNPI) throws JMeadowsException
    {
        User returnVal = null;
        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getProviderProfile(site, userIen, providerNPI);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public RadiologyReport getRadiologyReport(QueryBean queryBean) throws JMeadowsException
    {
        RadiologyReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getRadiologyReport(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;

    }

    public List<Alert> getUserAlerts(QueryBean queryBean) throws JMeadowsException
    {

        try
        {
            VistaData port = getVistaDataPort();
            return port.getUserAlerts(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public Diagnosis[] getVisitDiagnosis(QueryBean queryBean) throws JMeadowsException
    {
        Diagnosis[] returnVal = null;
        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getVisitDiagnosis(queryBean).toArray(new Diagnosis[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public FreeTextReport getVisitNotes(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getVisitNotes(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public Procedure[] getVisitProcedures(QueryBean queryBean) throws JMeadowsException
    {
        Procedure[] returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getVisitProcedures(queryBean).toArray(new Procedure[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }


        return returnVal;
    }

    public List<Patient> lookupPatient(QueryBean queryBean) throws JMeadowsException
    {

        try
        {
            VistaData port = getVistaDataPort();
            //return a max of 100 patients
            queryBean.setMax(100);
            return port.lookupPatient(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public boolean selectPatient(QueryBean queryBean) throws JMeadowsException
    {
        boolean returnVal = false;

        try
        {
            VistaData port = getVistaDataPort();
            //return a max of 100 patients
            queryBean.setMax(100);
            returnVal = port.selectPatient(queryBean);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;

    }

    public ReferralBean[] getDRReferrals(QueryBean queryBean) throws JMeadowsException
    {
        ReferralBean[] returnVal = null;

        try
        {
            VistaData port = getVistaDataPort();
            returnVal = port.getDRReferrals(queryBean).toArray(new ReferralBean[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public PatientAdmission[] getProviderAdmissions(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<PatientAdmission> admissions = null;
            VistaData port = getVistaDataPort();
            admissions = port.getProviderAdmissions(queryBean);
            return admissions.toArray(new PatientAdmission[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public PatientAdmission[] getWardAdmissions(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<PatientAdmission> admissions = null;
            VistaData port = getVistaDataPort();
            admissions = port.getWardAdmissions(queryBean);
            return admissions.toArray(new PatientAdmission[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public PatientAppointments[] getProviderAppointments(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<PatientAppointments> appointments = null;
            VistaData port = getVistaDataPort();
            appointments = port.getProviderAppointments(queryBean);
            return appointments.toArray(new PatientAppointments[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public ProgressNote[] getProviderUnsignedNotes(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<ProgressNote> unsignedNotes = null;
            VistaData port = getVistaDataPort();
            unsignedNotes = port.getProviderUnsignedNotes(queryBean);
            return unsignedNotes.toArray(new ProgressNote[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public Consult[] getProviderConsultsRequested(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<Consult> consultsRequested = null;
            VistaData port = getVistaDataPort();
            consultsRequested = port.getProviderConsultsRequested(queryBean);
            return consultsRequested.toArray(new Consult[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public Consult[] getProviderConsultsReceived(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<Consult> consultsReceived = null;
            VistaData port = getVistaDataPort();
            consultsReceived = port.getProviderConsultsReceived(queryBean);
            return consultsReceived.toArray(new Consult[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public Order[] getProviderOrdersPending(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<Order> orders = null;
            VistaData port = getVistaDataPort();
            orders = port.getProviderOrdersPending(queryBean);
            return orders.toArray(new Order[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public Order[] getProviderOrdersResulted(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<Order> orders = null;
            VistaData port = getVistaDataPort();
            orders = port.getProviderOrdersResulted(queryBean);
            return orders.toArray(new Order[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public LabResult[] getProviderLabAbnormalResults(QueryBean queryBean)
            throws JMeadowsException
    {
        try
        {
            List<LabResult> labResults = null;
            VistaData port = getVistaDataPort();
            labResults = port.getProviderLabAbnormalResults(queryBean);
            return labResults.toArray(new LabResult[0]);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public String getVersion() throws JMeadowsException
    {
        try
        {
            VistaData port = getVistaDataPort();
            return port.getVersion();
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    public List<VistaStatus> getVistaStatusList(List<Site> siteList) throws JMeadowsException {
        try
        {
            VistaData port = getVistaDataPort();
            return port.getVistaStatusList(siteList);
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }
}
