package gov.va.med.monitor;

import gov.va.med.bhie.QueryBean;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dataservicehandler.BHIERelayServiceHandler;

public class BHIERelayService extends ServiceBase
{
    private static final BHIERelayService INSTANCE = new BHIERelayService();
    private BHIERelayServiceHandler serviceHandler;

    private BHIERelayService()
    {
        super(ServiceType.BHIERelayService);

        serviceHandler = new BHIERelayServiceHandler(APP_CONFIG.getBHIERelayServiceURL());
    }

    public static BHIERelayService getInstance()
    {
        return INSTANCE;
    }

    @Override
    public void run()
    {
        try
        {
            // Set up query bean
            QueryBean qb = new QueryBean();

            qb.setUserId("56789"); // This is a made up user ID. Okay for querying BHIERelayService.
            qb.setPatientId("EDIPI:" + APP_CONFIG.getPatientEDIPIForBHIE());
            qb.getIncludedSources().add(APP_CONFIG.getBHIECustodianOidAHLTA());
            //qb.getExcludedSources().add(APP_CONFIG.getBHIECustodianOidVA());

            V4ResultCollection result =  serviceHandler.getPatientProblemList(qb);

            // Check result records
            if ((result != null)
                    && (result.getDataRecordList() != null)
                    && (result.getDataRecordList().getProblems() != null)
                    && (result.getDataRecordList().getProblems().size() > 0))
            {
                // Received expected result
                status = ServiceStatus.OK;
                message.append(SERVICE_OK_MSG);
            }
            else
            {
                // Did not receive expected result
                status = ServiceStatus.ERROR;
                message.append(NO_RESULTS_MSG);
            }

            // Extract errors from the error list returned in the result
            if ((result != null) && (result.getErrorList() != null) && result.getErrorList().size() > 0)
            {
                StringBuilder errorList = new StringBuilder();
                int index = 1;

                errorList.append(" The following errors were returned with the results:");

                for (gov.va.med.bhie.Error error : result.getErrorList())
                {
                    errorList.append(LINE_SEPARATOR + index + ") " + error.getErrorCode());
                    index++;
                }

                message.append(errorList.toString());
            }
        }
        catch (JMeadowsException e)
        {
            status = ServiceStatus.ERROR;
            message.append(e.getMessage());
        }

        // Report status
        report(type.toString(), status.toString(), message.toString());
    }
}
