package gov.va.med.monitor;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.common.PDWSQueryBean;
import gov.va.med.jmeadows.dataservicehandler.PDWSServiceHandler;
import gov.va.med.jmeadows.enterprise.beans.ResponsePatientQuery;
import gov.va.med.vds.webservice.User;

import java.net.MalformedURLException;

public class PDWSService extends ServiceBase
{
    private static final PDWSService INSTANCE = new PDWSService();
    private PDWSServiceHandler serviceHandler;

    private PDWSService()
    {
        super(ServiceType.PDWSService);

        serviceHandler = new PDWSServiceHandler(APP_CONFIG.getPDWSURL());
    }

    public static PDWSService getInstance()
    {
        return INSTANCE;
    }

    @Override
    public void run()
    {
        User user = ServiceMonitor.getInstance().getUser();

        if (user == null)
        {
            status = ServiceStatus.UNKNOWN;
            message.append("User was not configured properly therefore the service could not be tested.");
        }
        else
        {
            try
            {
                // Set up query bean
                PDWSQueryBean qb = new PDWSQueryBean();
                qb.setUser(user);
                qb.setPatientIdType(APP_CONFIG.getPDWSPatientIdType());
                qb.setPatientId(APP_CONFIG.getPDWSPatientId());
                qb.setLastName(APP_CONFIG.getPDWSPatientLastName());

                // Retrieve a patient list of a patient with a known SSN and last name
                ResponsePatientQuery response = serviceHandler.lookupPatient(qb);

                // Successful response message (connection is good)
                if (response.isSuccess())
                {
                    if (response.getPatients().size() > 0)
                    {
                        // Received expected result
                        status = ServiceStatus.OK;
                        message.append(SERVICE_OK_MSG);
                    }
                    else
                    {
                        // Did not receive expected result
                        status = ServiceStatus.ERROR;
                        message.append(NO_RESULTS_MSG);
                    }
                }
                else
                {
                    // Error was returned in response message
                    status = ServiceStatus.ERROR;
                    message.append(response.getErrorMsg());
                }
            }
            catch (JMeadowsException e)
            {
                // An exception was caught
                status = ServiceStatus.ERROR;
                message.append(e.getMessage());
            }
            catch (MalformedURLException e)
            {
                // An exception was caught
                status = ServiceStatus.ERROR;
                message.append(e.getMessage());
            }
        }

        // Report status
        report(type.toString(), status.toString(), message.toString());
    }
}
