package gov.va.med.monitor;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dataservicehandler.VistaDataServiceHandler;
import org.apache.commons.lang3.StringUtils;

public class VistaDataService extends ServiceBase
{
    private static final VistaDataService INSTANCE = new VistaDataService();
    private VistaDataServiceHandler serviceHandler;

    private VistaDataService()
    {
        super(ServiceType.VistaDataService);

        serviceHandler = new VistaDataServiceHandler(APP_CONFIG.getVistaWsURL());
    }

    public static VistaDataService getInstance()
    {
        return INSTANCE;
    }

    @Override
    public void run()
    {
        try
        {
            String version = serviceHandler.getVersion();
            if (StringUtils.isBlank(version))
            {
                // Did not receive expected result
                status = ServiceStatus.ERROR;
                message.append(NO_RESULTS_MSG);
            }
            else
            {
                // Received expected result
                status = ServiceStatus.OK;
                message.append(SERVICE_OK_MSG);
            }
        }
        catch (JMeadowsException e)
        {
            // An exception was caught
            status = ServiceStatus.ERROR;
            message.append(e.getMessage());
        }

        // Report status
        report(type.toString(), status.toString(), message.toString());
    }
}
