package gov.va.med.rpc;


import java.io.UnsupportedEncodingException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.List;

public abstract class AbstractVistaQuery implements VistaQuery {
    protected String rpcName;
    protected List<VistaQueryParameter> vistaQueryParameters;

    public AbstractVistaQuery() {
        super();
        this.vistaQueryParameters = new ArrayList<VistaQueryParameter>();
    }

    public AbstractVistaQuery(String rpcName) {
        super();
        if (rpcName != null) {
            this.rpcName = rpcName;
        }

        this.vistaQueryParameters = new ArrayList<VistaQueryParameter>();
    }

    public String toString() {
        return buildMessage();
    }

    public List<VistaQueryParameter> getVistaQueryParameters() {
        return vistaQueryParameters;
    }

    public void setVistaQueryParameters(List<VistaQueryParameter> vistaQueryParameters) {
        this.vistaQueryParameters = vistaQueryParameters;
    }

    public String getRpcName() {
        return rpcName;
    }

    public void setRpcName(String rpcName) {
        this.rpcName = rpcName;
    }

    public void addParameter(int type, String value) {
        vistaQueryParameters.add(new VistaQueryParameter(type, value));
    }

    public void addParameter(int type, String value, String text) {
        vistaQueryParameters.add(new VistaQueryParameter(type, value, text));
    }

    public void addParameter(int type, DictionaryHashList lst) {
        vistaQueryParameters.add(new VistaQueryParameter(type, lst));
    }

    public void addEncryptedParameter(int type, String value) {
        vistaQueryParameters.add(new VistaQueryParameter(type, encrypt(value)));
    }

    protected String buildApi(String rpcName, String parameters, String fText) {
        String sParams = StringUtils.strPack(parameters, 5);
        StringBuilder packStr = new StringBuilder();
        packStr.append(fText);
        packStr.append(rpcName);
        packStr.append('^');
        packStr.append(sParams);
        return StringUtils.strPack(packStr.toString(), 5);
    }

    public abstract String buildMessage();

    protected String getVersion() {
        byte[] num = {1};
        String numChar = null;
        try {
            numChar = new String(num, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
        StringBuilder version = new StringBuilder();
        version.append("|");
        version.append(numChar);
        version.append("1");
        return version.toString();
    }

    public String encrypt(String inString) {
        final int MAXKEY = 19;
        String[] cipherPad = {
            "wkEo-ZJt!dG)49K{nX1BS$vH<&:Myf*>Ae0jQW=;|#PsO`\'%+rmb[gpqN,l6/hFC@DcUa ]z~R}\"V\\iIxu?872.(TYL5_3",
            "rKv`R;M/9BqAF%&tSs#Vh)dO1DZP> *fX\'u[.4lY=-mg_ci802N7LTG<]!CWo:3?{+,5Q}(@jaExn$~p\\IyHwzU\"|k6Jeb",
            "\\pV(ZJk\"WQmCn!Y,y@1d+~8s?[lNMxgHEt=uw|X:qSLjAI*}6zoF{T3#;ca)/h5%`P4$r]G\'9e2if_>UDKb7<v0&- RBO.",
            "depjt3g4W)qD0V~NJar\\B \"?OYhcu[<Ms%Z`RIL_6:]AX-zG.#}$@vk7/5x&*m;(yb2Fn+l\'PwUof1K{9,|EQi>H=CT8S!",
            "NZW:1}K$byP;jk)7\'`x90B|cq@iSsEnu,(l-hf.&Y_?J#R]+voQXU8mrV[!p4tg~OMez CAaGFD6H53%L/dT2<*>\"{\\wI=",
            "vCiJ<oZ9|phXVNn)m K`t/SI%]A5qOWe\\&?;jT~M!fz1l>[D_0xR32c*4.P\"G{r7}E8wUgyudF+6-:B=$(sY,LkbHa#\'@Q",
            "hvMX,\'4Ty;[a8/{6l~F_V\"}qLI\\!@x(D7bRmUH]W15J%N0BYPkrs&9:$)Zj>u|zwQ=ieC-oGA.#?tfdcO3gp`S+En K2*<",
            "jd!W5[];4\'<C$/&x|rZ(k{>?ghBzIFN}fAK\"#`p_TqtD*1E37XGVs@0nmSe+Y6Qyo-aUu%i8c=H2vJ\\) R:MLb.9,wlO~P",
            "2ThtjEM+!=xXb)7,ZV{*ci3\"8@_l-HS69L>]\\AUF/Q%:qD?1~m(yvO0e\'<#o$p4dnIzKP|`NrkaGg.ufCRB[; sJYwW}5&",
            "vB\\5/zl-9y:Pj|=(R\'7QJI *&CTX\"p0]_3.idcuOefVU#omwNZ`$Fs?L+1Sk<,b)hM4A6[Y%aDrg@~KqEW8t>H};n!2xG{",
            "sFz0Bo@_HfnK>LR}qWXV+D6`Y28=4Cm~G/7-5A\\b9!a#rP.l&M$hc3ijQk;),TvUd<[:I\"u1\'NZSOw]*gxtE{eJp|y (?%",
            "M@,D}|LJyGO8`$*ZqH .j>c~h<d=fimszv[#-53F!+a;NC\'6T91IV?(0x&/{B)w\"]Q\\YUWprk4:ol%g2nE7teRKbAPuS_X",
            ".mjY#_0*H<B=Q+FML6]s;r2:e8R}[ic&KA 1w{)vV5d,$u\"~xD/Pg?IyfthO@CzWp%!`N4Z\'3-(o|J9XUE7k\\TlqSb>anG",
            "xVa1\']_GU<X`|\\NgM?LS9{\"jT%s$}y[nvtlefB2RKJW~(/cIDCPow4,>#zm+:5b@06O3Ap8=*7ZFY!H-uEQk; .q)i&rhd",
            "I]Jz7AG@QX.\"%3Lq>METUo{Pp_ |a6<0dYVSv8:b)~W9NK`(r\'4fs&wim\\kReC2hg=HOj$1B*/nxt,;c#y+![?lFuZ-5D}",
            "Rr(Ge6F Hx>q$m&C%M~Tn,:\"o\'tX/*yP.{lZ!YkiVhuw_<KE5a[;}W0gjsz3]@7cI2\\QN?f#4p|vb1OUBD9)=-LJA+d`S8",
            "I~k>y|m};d)-7DZ\"Fe/Y<B:xwojR,Vh]O0Sc[`$sg8GXE!1&Qrzp._W%TNK(=J 3i*2abuHA4C\'?Mv\\Pq{n#56LftUl@9+",
            "~A*>9 WidFN,1KsmwQ)GJM{I4:C%}#Ep(?HB/r;t.&U8o|l[\'Lg\"2hRDyZ5`nbf]qjc0!zS-TkYO<_=76a\\X@$Pe3+xVvu",
            "yYgjf\"5VdHc#uA,W1i+v\'6|@pr{n;DJ!8(btPGaQM.LT3oe?NB/&9>Z`-}02*%x<7lsqz4OS ~E$\\R]KI[:UwC_=h)kXmF",
            "5:iar.{YU7mBZR@-K|2 \"+~`M%8sq4JhPo<_X\\Sg3WC;Tuxz,fvEQ1p9=w}FAI&j/keD0c?)LN6OHV]lGy\'$*>nd[(tb!#"
        };
        SecureRandom rand = new SecureRandom();
        rand.setSeed(System.currentTimeMillis());
        int associatorIndex = rand.nextInt(MAXKEY);
        int identifierIndex = rand.nextInt(MAXKEY);
        while (associatorIndex == identifierIndex) {
            identifierIndex = rand.nextInt(MAXKEY);
        }
        StringBuilder xlatedStringSB = new StringBuilder("");
        int inStringLength = inString.length();
        for (int i = 0; i < inStringLength; i++) {
            char inChar = inString.charAt(i);
            int pos = cipherPad[associatorIndex].indexOf(inChar);
            if (pos == -1) {
                xlatedStringSB.append(inChar);
            } else {
                xlatedStringSB.append(cipherPad[identifierIndex].charAt(pos));
            }
        }
        return (char) (associatorIndex + 32) + xlatedStringSB.toString()
            + (char) (identifierIndex + 32);
    }

    protected String list2string(DictionaryHashList lst) {
        String rtc;
        if (lst == null || lst.size() == 0) {
            StringBuilder returnSB = new StringBuilder();
            returnSB.append(StringUtils.LPack("", 3));
            returnSB.append('f');
            rtc = returnSB.toString();
        } else {
            StringBuilder resultSB = new StringBuilder("");
            int lstSize = lst.size();
            for (int i = 0; i < lstSize; i++) {
                KeyValuePair kvp = lst.get(i);
                String key = (String) kvp.getKey();
                String value = (String) kvp.getValue();
                if (StringUtils.isEmpty(value)) {
                    value = "\u0001";
                }
    
                resultSB.append(StringUtils.LPack(key, 3))
                        .append(StringUtils.LPack(value, 3))
                        .append('t');
            }
            resultSB.setCharAt(resultSB.length() - 1, 'f');
            rtc = resultSB.toString();
        }
        return rtc;
    }

}
