/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;

import java.util.Calendar;

/**
 *
 *
 */
public class DateUtils {
    public static boolean isLeapYear(int year) {
        return (year > 0) && (year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0));
    }

    public static boolean isValidMonth(int month) {
        return month > 0 && month < 13;
    }

    public static boolean isValidDay(int year, int month, int day) {
        boolean rtc;
        if (day < 1 || day > 31) {
            rtc = false;
        } else if (is30DayMonth(month) && day > 30) {
            rtc = false;
        } else if (month != 2) {
            rtc = true;
        } else if (day > 29) {
            rtc = false;
        } else if (!isLeapYear(year) && day > 28) {
            rtc = false;
        } else {
            rtc = true;
        }
        return rtc;
    }

    public static boolean is30DayMonth(int month) {
        return month == 4 || month == 6 || month == 9 || month == 11;
    }

    /// <summary>
    /// Changes yyyyMMdd.HHmmss formatted string to date.
    /// </summary>
    /// <remarks>pads missing least significant digits (e.g. seconds or milliseconds)
    /// with 0s.</remarks>
    /// <param name="dateString"></param>
    /// <returns></returns>
    public static Calendar isoDateStringEndDateTime(String indateString) {
        String dateString = indateString;
        Calendar retTime = Calendar.getInstance();
        retTime.set(Integer.parseInt(StringUtils.CSubstring(dateString, 0, 4))
                    , Integer.parseInt(StringUtils.CSubstring(dateString, 4, 2))
                    , Integer.parseInt(StringUtils.CSubstring(dateString, 6, 2)));

        Integer result = Integer.valueOf(0);
        StringBuilder dateSB = new StringBuilder(dateString);
        if (dateSB.length() < 18) {
            //pad right dateString with 18 zeros
            for (int i = 0; i < 18; i++) {
                dateSB.append("0");
            }
        }

        dateString = dateSB.toString();

        if (intTryParse(StringUtils.CSubstring(dateString, 9, 2))) {
            retTime.add(Calendar.HOUR, result.intValue());
        }
        if (intTryParse(StringUtils.CSubstring(dateString, 11, 2))) {
            retTime.add(Calendar.MINUTE, result);
        }
        if (intTryParse(StringUtils.CSubstring(dateString, 13, 2))) {
            retTime.add(Calendar.SECOND, result);
        }
        if (intTryParse(StringUtils.CSubstring(dateString, 15, 3))) {
            retTime.add(Calendar.MILLISECOND, result);
        }

        return retTime;
    }

    private static boolean intTryParse(String intStr) {
        boolean rtc = true;
        try {
            Integer.parseInt(intStr);
        } catch (NumberFormatException ex) {
            rtc = false;
        }

        return rtc;
    }

    public static String trimSeconds(String timestamp) {
        String rtc;
        int pos = timestamp.indexOf('.');
        if (pos == -1) {
            rtc = timestamp;
        } else {
            String[] parts = StringUtils.split(timestamp, StringUtils.PERIOD);
            if (parts[1].length() < 5) {
                rtc = timestamp;
            } else {
                parts[1] = StringUtils.CSubstring(parts[1], 0, 4);
                rtc = parts[0] + '.' + parts[1];
            }
        }
        return rtc;
    }

    public static String zeroSeconds(String timestamp) {
        String rtc;
        int pos = timestamp.indexOf('.');
        if (pos == -1) {
            rtc = timestamp + ".000000";
        } else {
            String[] parts = StringUtils.split(timestamp, StringUtils.PERIOD);
            if (parts[1].length() < 5) {
                rtc = StringUtils.CSubstring(timestamp + "000000", 0, 6 - parts[1].length());
            } else {
                parts[1] = StringUtils.CSubstring(parts[1], 0, 4)  + "00";
                rtc = parts[0] + '.' + parts[1];
            }
        }
        return rtc;
    }

    public static String trimTime(String timestamp) {
        String rtc;
        int pos = timestamp.indexOf('.');
        if (pos == -1) {
            rtc = timestamp;
        } else {
            String[] parts = StringUtils.split(timestamp, StringUtils.PERIOD);
            rtc = parts[0];
        }
        return rtc;
    }

    public static boolean isValidDateRange(String startDate, String endDate) {
        boolean rtc;
        if (startDate.equals(endDate)) {
            rtc = false;
        } else {
            Calendar fromDT = isoDateStringEndDateTime(startDate);
            Calendar toDT = isoDateStringEndDateTime(endDate);
    
            if (fromDT.compareTo(toDT) > 0 || fromDT.compareTo(Calendar.getInstance()) > 0) {
                rtc = false;
            } else {
                rtc = true;
            }
        }
        return rtc;
    }

//    String subtractSecond(String timestamp) {
//        String rtc = "";
//        String[] parts = StringUtils.split(timestamp, StringUtils.PERIOD);
//
//        // Only do this if there is a time part that ends with 0.
//        if (parts.length < 2 || !parts[1].endsWith("0")) {
//            rtc = timestamp;
//        }
//
//        return rtc;
//    }
}
