package gov.va.med.rpc;


import org.w3c.dom.Node;

import java.util.ArrayList;
import java.util.List;

/**
 * Maintains a list of XML node batches used for parsing.
 *
 * Each batch will contain no more than the specified batch size passed to the constructor.
 */
public class ParserBatchManager {
    private List<List<Node>> parserBatchList;

    private int batchSize;

    /**
     * Constructs a Parser Batch Manager.
     * @param batchSize Size of each batch (i.e. 10).
     */
    public ParserBatchManager(int batchSize) {

        this.batchSize = batchSize;

        this.parserBatchList = new ArrayList<List<Node>>();
    }

    /**
     * Adds a node to a batch.
     * @param node XML dom node.
     */
    public void addNode(Node node) {
        int batchIdx = parserBatchList.size() - 1;

        if (batchIdx == -1 || parserBatchList.get(batchIdx).size() >= batchSize) {
            parserBatchList.add(++batchIdx, new ArrayList<Node>());
        }

        Node parent = node.getParentNode();
        if (parent != null) {
            parent.removeChild(node);
        }

        parserBatchList.get(batchIdx).add(node);
    }

    /**
     * Gets a list of XML node batches.
     * @return List of XML node batches.
     */
    public List<List<Node>> getParserBatchList() {
        return this.parserBatchList;
    }
}
