/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;

import java.io.UnsupportedEncodingException;
import java.util.LinkedHashMap;
import java.util.Locale;

/**
 *
 *
 */
public class RPCUtils {
    public static String adjustForNameSearch(String target) {
        String rtn;
        int lth = target.length();
        if (lth == 0) {
            rtn = "";
        } else {
            //target = target.ToUpper();
            rtn = StringUtils.CSubstring(target, 0, lth - 1);
            char chr = target.charAt(lth - 1);
            int asciiCode = (byte) chr - 1;
            chr = (char) asciiCode;
            StringBuffer sbuff = new StringBuffer();
            sbuff.append(rtn);
            sbuff.append(chr);
            sbuff.append('~');            
            rtn = sbuff.toString();
        }
        return rtn;
    }

    /// <summary>
    /// </summary>
    /// <param name="target"></param>
    /// <returns></returns>
    public static String adjustForNumericSearch(String target) {
        Long iTarget = Long.parseLong(target);
        return Long.toString(iTarget - 1);
    }

    public static String setDirectionParam(String direction) {
        String rtc = direction;
        if (direction == null || "".equals(direction)) {
            rtc = "1";
        }
        if (!"1".equals(direction) && !"-1".equals(direction)) {
            throw new IllegalArgumentException("Invalid direction.  Must be 1 or -1.");
        }
        return rtc;
    }

//    // DoD User
//    public synchronized static Provider getDodUser()
//    {
//        Provider p = new Provider();
//        p.setLogonSite(new KeyValuePair("200", "DOD"));
//        p.setLogonSiteUserId("31066");
//        p.setName("DEPARTMENT OF DEFENSE,USER");
//        p.setPermissionString(VistaConstants.CAPRI_CONTEXT);
//          EncryptUtil encryptUtil = new EncryptUtil();
//        p.setEncryptedFederatedUid(encryptUtil.encrypt("123456789"));
//        return p;
//    }

    public static String buildReportTextRequest(VistaQueryFactory vistaQueryFactory,
        String patientIen, String instartDate, String inendDate, int nrpts, String arg) {
        String startDate = instartDate;
        String endDate = inendDate;
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
//        if (nrpts != 0)
//        {
//            arg += ("" + nrpts);
//        }
        vquery.addParameter(VistaQuery.LITERAL, arg);
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        if ("".equals(startDate)) {
            startDate = "0";
        } else {
            startDate = Timestamp.fromUtcString(startDate);
        }
        vquery.addParameter(VistaQuery.LITERAL, startDate);
        if ("".equals(endDate)) {
            endDate = "0";
        } else {
            endDate = Timestamp.fromUtcString(endDate);
        }
        vquery.addParameter(VistaQuery.LITERAL, endDate);
        return vquery.buildMessage();
    }

    public static String buildReportTextRequest_AllResults(VistaQueryFactory vistaQueryFactory, String patientIen, String arg) {
        VistaQuery vquuery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquuery.addParameter(VistaQuery.LITERAL, patientIen);
        vquuery.addParameter(VistaQuery.LITERAL, arg + '0');
        vquuery.addParameter(VistaQuery.LITERAL, "");
        vquuery.addParameter(VistaQuery.LITERAL, "50000");
        vquuery.addParameter(VistaQuery.LITERAL, "");
        vquuery.addParameter(VistaQuery.LITERAL, "0");
        vquuery.addParameter(VistaQuery.LITERAL, "0");
        return vquuery.buildMessage();
    }

    public static String responseOrOk(String response) {
        String rtc = "OK";
        if (!"".equals(response)) {
            rtc = response;
        }
        return rtc;
    }

    public static String errMsgOrOK(String response) {
        String rtc = "OK";
        String[] flds = StringUtils.split(response, StringUtils.CARET);
        if (!"1".equals(flds[0])) {
            rtc = flds[1];
        }
        return rtc;
    }

    public static String errMsgOrZero(String response) {
        String rtc = "OK";
        String[] flds = StringUtils.split(response, StringUtils.CARET);
        if (!"0".equals(flds[0])) {
            rtc = flds[1];
        }
        return rtc;
    }

    public static String errMsgOrIen(String response) throws RPCException {
        if (!"".equals(response) && !StringUtils.isNumeric(response)) {
            throw new RPCException("Non-numeric IEN");
        }
        return response;
    }

    public static String removeCtlChars(String str) {
        String rtc;
        if (str == null || "".equals(str)) {
            rtc = "";
        } else {
            StringBuilder resultSB = new StringBuilder("");
            int count = 0;
            int strLength = str.length();
            while (count < strLength) {
                byte chr = 0;
                try {
                    chr = Character.toString(str.charAt(count)).getBytes(VistaConstants.CHARSET_UTF8.toString())[0];
                } catch (UnsupportedEncodingException e) {
                    throw new RuntimeException(e);
                }
                if (chr == 9 || chr == 10 || chr == 13 || (chr > 31 && chr < 127)) {
                    resultSB.append(str.charAt(count));
                }
                count++;
            }
            rtc = resultSB.toString();
        }
        return rtc;
    }

    public static String getVistaName(String csv) {
        String rtc;
        /*if (!PersonName.isValid(s))
        {
            return "";
        }*/
        String[] flds = StringUtils.split(csv, StringUtils.COMMA);
        if (flds.length == 2) {
            String result = flds[0] + ',' + flds[1];
            rtc = result.toUpperCase(Locale.getDefault());
        } else {
            rtc = "";
        }
        return rtc;
    }

    public static String getVistaGender(String str) {
        String rtc;
        if (StringUtils.isEmpty(str)) {
            rtc = "";
        } else {
            String result = StringUtils.CSubstring(str, 0, 1).toUpperCase(Locale.getDefault());
            if ("M".equals(result) || "F".equals(result)) {
                rtc = result;
            } else {
                rtc = "";
            }
        }
        return rtc;
    }

    public static LinkedHashMap<String, String> toStringDictionary(String[] response) {
        LinkedHashMap<String, String> result;
        if (response == null || response.length == 0) {
            result = null;
        } else {
            result = new LinkedHashMap<String, String>();
            for (int i = 0; i < response.length; i++) {
                String[] flds = StringUtils.split(response[i], StringUtils.CARET);
                result.put(flds[0], flds[1]);
            }
        }
        return result;
    }

    public static String getVariableValue(VistaQueryFactory vistaQueryFactory, RPCConnection cxn, String arg) throws RPCException
    {
        try {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("XWB GET VARIABLE VALUE");
            vquery.addParameter(VistaQuery.REFERENCE, arg);
            return (String) cxn.query(vquery.buildMessage());
        } catch (Exception ex) {
            throw new RPCException(ex);
        }
    }

    public static String buildFromendDateScreenParam(String instartDate, String inendDate, int node, int pieceNum) {
        String rtc;
        String startDate = instartDate;
        String endDate = inendDate;
        if ("".equals(startDate)) {
            rtc = "";
        } else {
    
            // Need test for valid dates.
            startDate = Timestamp.fromUtcFromDate(startDate);
            endDate = Timestamp.fromUtcString(endDate);
            rtc = "S FD=" + startDate + ",TD=" + endDate + ",CDT=$P(^(" + node + "),U," + pieceNum + ") I CDT>=FD,CDT<TD";
        }
        return rtc;
    }
}
