/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 *
 * @author vhahonuyehac
 */
public class VistaResponse implements Iterator<String> {
    private String rawResponse;
    
    /** Helper variable used for the Iterator **/
    private int nextItem = -1;


    public VistaResponse(String rawResponse) {
        this.rawResponse = rawResponse;
    }

    /**
     * Gets the raw string response of the executed vista query.
     * @return string response
     */
    public String getRawResponse() {
        return rawResponse;
    }

    /**
     * Returns an array of strings using the ^ as the default delimiter
     * @return a zero based array of pieces
     */
    public String[] getArrayOfPieces() {
        return getArrayOfPieces(StringUtils.CARET);
    }

    /**
     * Returns an array of strings using the given delimiter
     * @param delimiter the delimiter to be used to piece (split) the string
     * @return a zero based array of pieces
     */
    public String[] getArrayOfPieces(String delimiter) {
        return StringUtils.split(rawResponse, delimiter);
    }

    /**
     * Gets a specific string of text located in the given piece #; note to
     * retrieve the first piece, the pieceNumber should be 1 and not zero.
     * @param pieceNumber the piece # to return
     * @return string located at the given piece #
     */
    public String getPiece(int pieceNumber) {
        return StringUtils.piece(rawResponse, StringUtils.CARET, pieceNumber);
    }

    public String getPiece(String delimiter, int pieceNumber) {
        return StringUtils.piece(rawResponse, delimiter, pieceNumber);
    }

    @Override
    public String toString() {
        return rawResponse;
    }

    public boolean hasNext() {
        boolean rtc = false;
        if (rawResponse.length() > 0) {
            if (nextItem < 0) {
                rtc = true;
            } else {
                if (rawResponse.indexOf("\r\n", nextItem) > nextItem) {
                    rtc = true;
                } else if (rawResponse.length() > nextItem) {
                    rtc = true;
                }
            }
        }
        return rtc;
    }

    public String next() {
        String rtc;
        String result = null;
        int temp = 0;
        if (nextItem < 0) {
            temp = rawResponse.indexOf("\r\n");
            nextItem = temp + 2;
            rtc = rawResponse.substring(0, temp);
        } else if (nextItem < rawResponse.length()) {
            temp = rawResponse.indexOf("\r\n", nextItem);
            if (temp == -1) {
                result = rawResponse.substring(nextItem);
                nextItem = rawResponse.length();
                rtc = result;
            } else {
                result = rawResponse.substring(nextItem, temp);
                nextItem = temp + 2;
                rtc = result;
            }
        } else {
            throw new NoSuchElementException("Iteration has no more elements");
        }
        return rtc;
    }

    public void remove() {
        throw new UnsupportedOperationException("Not supported.");
    }
}
