/*
 * Janus 4.0 (c)
 * Copyright (c) 2016 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.common;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * User: DNS   UYEHAC
 * Date: 8/29/11
 * Time: 2:36 PM
 */
public class AppConfig {
    private static final Logger LOGGER = LoggerFactory.getLogger(AppConfig.class);

    private static final String AGENCY_VA = "appconfig.agency.va";
    private static final String AGENCY_DOD = "appconfig.agency.dod";
    private static final String LOG4J_PROPS = "appconfig.log4j.props";
    private static final String APP_ENVIRONMENT = "app.environment";
    private static final String APP_VERSION = "app.version";
    private static final String NON_BSE_SECURITY_PHRASE = "appconfig.authentication.nonBseSecurityPhrase";

    private static final String REQUEST_TIMEOUT_MS = "appconfig.request.timeout.ms";
    private static final String CONNECTION_TIMEOUT_MS = "appconfig.connection.timeout.ms";

    private static final String VLER_DQ_URL = "appconfig.vler.dq.url";
    private static final String VLER_DR_URL = "appconfig.vler.dr.url";
//    private static final String VLER_PD_URL = "appconfig.vler.pd.url";
//    private static final String VLER_PA_URL = "appconfig.vler.pa.url";
    private static final String VLER_DAS_URL = "appconfig.vler.das.url";
    private static final String VLER_DAS_CALLBACK = "appconfig.vler.das.callback";

    private static final String EVIX_STUDYQUERY_URL = "appconfig.evix.studyquery.url";
    private static final String EVIX_REQUEST_TIMEOUT = "appconfig.evix.request.timeout.ms";
    private static final String EVIX_CONNECTION_TIMEOUT = "appconfig.evix.connection.timeout.ms";

    //constants
    private static final String PROTOCOL_RPC = "RPC";
    private static final String PROTOCOL_CACHE = "CACHE";
    private static final String DATASOURCE = "appconfig.datasource";

    private static final String RPC_CONNECTION_THREAD_MAX_POOL_SIZE = "appconfig.rpc.thread.maxPoolSize";
    private static final String RPC_CONNECTION_THREAD_TIMEOUT_SECONDS = "appconfig.rpc.thread.timeout.seconds";

    private static final String PARSER_THREAD_MAX_POOL_SIZE = "appconfig.parse.thread.maxPoolSize";
    private static final String PARSER_THREAD_TIMEOUT_SECONDS = "appconfig.parse.thread.timeout.seconds";

    private static final String PARSER_THREAD_MAX_POOL_SIZE_ENCOUNTERS = "appconfig.parse.thread.maxPoolSize.encounters";
    private static final String PARSER_THREAD_MAX_POOL_SIZE_LABS = "appconfig.parse.thread.maxPoolSize.labs";
    private static final String PARSER_THREAD_MAX_POOL_SIZE_NOTES = "appconfig.parse.thread.maxPoolSize.notes";
    private static final String PARSER_THREAD_MAX_POOL_SIZE_ORDERS = "appconfig.parse.thread.maxPoolSize.orders";
    private static final String PARSER_THREAD_MAX_POOL_SIZE_MEDS = "appconfig.parse.thread.maxPoolSize.meds";

    private static final int PARSER_BATCH_SIZE_DEFAULT = 25;
    private static final String PARSER_BATCH_SIZE_ORDERS = "appconfig.parse.batchSize.orders";
    private static final String PARSER_BATCH_SIZE_NOTES = "appconfig.parse.batchSize.notes";
    private static final String PARSER_BATCH_SIZE_APPOINTMENTS = "appconfig.parse.batchSize.appointments";
    private static final String PARSER_BATCH_SIZE_ENCOUNTERS = "appconfig.parse.batchSize.encounters";
    private static final String PARSER_BATCH_SIZE_ALLERGIES = "appconfig.parse.batchSize.allergies";
    private static final String PARSER_BATCH_SIZE_REMINDERS = "appconfig.parse.batchSize.reminders";
    private static final String PARSER_BATCH_SIZE_IMMUNIZATIONS = "appconfig.parse.batchSize.immunizations";
    private static final String PARSER_BATCH_SIZE_INSURANCE = "appconfig.parse.batchSize.insurance";
    private static final String PARSER_BATCH_SIZE_LAB_PANELS = "appconfig.parse.batchSize.lab.panels";
    private static final String PARSER_BATCH_SIZE_LAB_ACCESSIONS = "appconfig.parse.batchSize.lab.accessions";
    private static final String PARSER_BATCH_SIZE_MEDICATIONS = "appconfig.parse.batchSize.medications";
    private static final String PARSER_BATCH_SIZE_PATIENT_FLAGS = "appconfig.parse.batchSize.patient.flags";
    private static final String PARSER_BATCH_SIZE_PCE = "appconfig.parse.batchSize.pce";
    private static final String PARSER_BATCH_SIZE_RADIOLOGY = "appconfig.parse.batchSize.radiology";
    private static final String PARSER_BATCH_SIZE_VITALS = "appconfig.parse.batchSize.vitals";

    //rest configs
    private static final String REST_CONNECTION_TIMEOUT_MS = "appconfig.vler.das.REST_CONNECTION_TIMEOUT_MS";
    private static final String REST_READ_TIMEOUT_MS = "appconfig.vler.das.REST_READ_TIMEOUT_MS";
    private static final String IS_DAS_CACHE_ENABLED = "appconfig.vler.das.CACHE_ENABLED";
    private static final String DAS_TEST_PAT = "appconfig.vler.das.testpat";
    private static final String DAS_ENABLE_COMMUNITY_SUMMARIES = "appconfig.vler.das.enableHealthSummaries";

    private static final String USE_ENHANCED_DEBUG = "appconfig.debug.enhanced";

    private static final String ENCRYPTION_KEY_ARRAY = "appconfig.encryption.key.array";

    private static final String BSE_TOKEN_ENABLED = "appconfig.bse.token.enabled";

    private static final AppConfig INSTANCE = new AppConfig();

    private Properties appProperties;

    private AppConfig() {

        appProperties = new Properties();

        loadAppMetaProperties();
        loadAppConfigProperties();
    }

    private void loadAppMetaProperties() {
        String propertiesFile = "application.properties";

        InputStream inputStream = null;

        ClassLoader classLoader = AppConfig.class.getClassLoader();
        if (classLoader != null) {
            inputStream = classLoader.getResourceAsStream(propertiesFile);
        }

        try {
            appProperties.load(inputStream);
        } catch (IOException e) {
            LOGGER.error("Failed load application.properties: " + e.getMessage());
            throw new RuntimeException(e);
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException e) {
                LOGGER.error(e.getMessage());
            }
        }
    }

    private void loadAppConfigProperties() {
        String propertiesFile = "vistadataservice.properties";

        InputStream inputStream = null;

        ClassLoader classLoader = AppConfig.class.getClassLoader();
        if (classLoader != null) {
            inputStream = classLoader.getResourceAsStream(propertiesFile);
        }

        if (inputStream != null) {
            try {
                appProperties.load(inputStream);
            } catch (IOException e) {
                LOGGER.error("Failed load appconfig properties: " + e.getMessage());
                throw new RuntimeException(e);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        try {

            inputStream = new FileInputStream(propertiesFile);

            try {
                appProperties.load(inputStream);
            } catch (IOException e) {
                LOGGER.error("Failed load appconfig properties: " + e.getLocalizedMessage());
                throw new RuntimeException(e);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }

        } catch (FileNotFoundException e) {
            LOGGER.info("Failed to load appconfig from file (vistadataservice.properties) -"
                    + " utilizing default appconfig properties.");

        }
    }

    public static final AppConfig getInstance() {
        return INSTANCE;
    }

    public final String getProtocolRPC() {
        return PROTOCOL_RPC;
    }

    public final String getProtocolCache() {
        return PROTOCOL_CACHE;
    }

    public final String getAgencyVA() {
        return appProperties.getProperty(AGENCY_VA);
    }

    public final String getAgencyDOD() {
        return appProperties.getProperty(AGENCY_DOD);
    }

    public final String getLog4jProperties() {
        return appProperties.getProperty(LOG4J_PROPS);
    }

    public final String getAppEnvironment() {
        return appProperties.getProperty(APP_ENVIRONMENT);
    }

    public final String getAppVersion() {
        return appProperties.getProperty(APP_VERSION);
    }

    public final String getNonBseSecurityPhrase() {
        return appProperties.getProperty(NON_BSE_SECURITY_PHRASE);
    }

    public final int getConnectionTimeoutMs() {
        return Integer.parseInt(appProperties.getProperty(CONNECTION_TIMEOUT_MS));
    }

    public final int getRequestTimeoutMs() {
        return Integer.parseInt(appProperties.getProperty(REQUEST_TIMEOUT_MS));
    }

    public final String getVLER_DQ_URL() {
        return appProperties.getProperty(VLER_DQ_URL);
    }

    public final String getVLER_DR_URL() {
        return appProperties.getProperty(VLER_DR_URL);
    }

    public final String getVLER_DAS_URL() {
        return appProperties.getProperty(VLER_DAS_URL);
    }

    public final String getVLER_DAS_CALLBACK() {
        return appProperties.getProperty(VLER_DAS_CALLBACK);
    }

    public final String getEvixStudyqueryUrl() { return appProperties.getProperty(EVIX_STUDYQUERY_URL);}

    public final int getEvixRequestTimeout() {
        return Integer.parseInt(appProperties.getProperty(EVIX_REQUEST_TIMEOUT));
    }

    public final int getEvixConnectionTimeout() {
        return Integer.parseInt(appProperties.getProperty(EVIX_CONNECTION_TIMEOUT));
    }

    public final String getDataSource() {
        return appProperties.getProperty(DATASOURCE);
    }

    public int getRestConnectionTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(REST_CONNECTION_TIMEOUT_MS));
    }

    public int getRestReadTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(REST_READ_TIMEOUT_MS));
    }

    public Properties getProperties() {
        return appProperties;
    }

    public final boolean isDASCacheEnabled() {
        return Boolean.valueOf(appProperties.getProperty(IS_DAS_CACHE_ENABLED));
    }

    public final boolean enableDASCommunitySummaries(){
        return Boolean.valueOf(appProperties.getProperty(DAS_ENABLE_COMMUNITY_SUMMARIES));
    }

    public final String getDasTestPat() {return appProperties.getProperty(DAS_TEST_PAT, ""); }

//    public final String getVLER_PD_URL()
//    {
//        return appProperties.getProperty(VLER_PD_URL);
//    }
//
//    public final String getVLER_PA_URL()
//    {
//        return appProperties.getProperty(VLER_PA_URL);
//    }


    public final int getRPCConnectionMaxThreadPoolSize() {
        return Integer.parseInt(appProperties.getProperty(RPC_CONNECTION_THREAD_MAX_POOL_SIZE));
    }

    public final int getRPCConnectionThreadTimeoutSeconds() {
        return Integer.parseInt(appProperties.getProperty(RPC_CONNECTION_THREAD_TIMEOUT_SECONDS));
    }

    public final int getParserMaxThreadPoolSize() {
        return Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE));
    }

    public final int getParserThreadTimeoutSeconds() {
        return Integer.parseInt(appProperties.getProperty(PARSER_THREAD_TIMEOUT_SECONDS));
    }

    private final static int PARSER_THREAD_MAX_POOL_SIZE_LARGE = 30;
    
    public final int getParserMaxThreadPoolSizeEncounters() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE_ENCOUNTERS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_THREAD_MAX_POOL_SIZE_LARGE;
            }
        }
        
        return rtn;
    }

    public final int getParserMaxThreadPoolSizeLabs() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE_LABS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_THREAD_MAX_POOL_SIZE_LARGE;
            }
        }
        return rtn;
    }

    public final int getParserMaxThreadPoolSizeNotes() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE_NOTES));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_THREAD_MAX_POOL_SIZE_LARGE;
            }
        }
        return rtn;
    }

    public final int getParserMaxThreadPoolSizeOrders() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE_ORDERS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_THREAD_MAX_POOL_SIZE_LARGE;
            }
        }
        return rtn;
    }

    public final int getParserMaxThreadPoolSizeMeds() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE_MEDS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_THREAD_MAX_POOL_SIZE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_THREAD_MAX_POOL_SIZE_LARGE;
            }
        }
        return rtn;
    }

    // true for running enhanced debug code
    public final boolean useEnhancedDebug() {
        return Boolean.valueOf(appProperties.getProperty(USE_ENHANCED_DEBUG));
    }

    // true for running enhanced debug code
    public final boolean bseTokenEnabled() {
        return Boolean.valueOf(appProperties.getProperty(BSE_TOKEN_ENABLED));
    }
    
    public static final byte[] EMPTY_BYTES = new byte[16];
    public final byte[] getEncryptionKey() {
        if (!StringUtils.isEmpty(appProperties.getProperty(ENCRYPTION_KEY_ARRAY))) {
            return Utils.parseByteArray16(appProperties.getProperty(ENCRYPTION_KEY_ARRAY));
        } else {
            return EMPTY_BYTES;
        }
    }

    public final int getParserBatchSizeOrders() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_ORDERS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_ORDERS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeNotes() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_NOTES));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_NOTES));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeAppointments() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_APPOINTMENTS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_APPOINTMENTS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeEncounters() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_ENCOUNTERS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_ENCOUNTERS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeAllergies() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_ALLERGIES));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_ALLERGIES));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeReminders() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_REMINDERS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_REMINDERS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeImmunizations() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_IMMUNIZATIONS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_IMMUNIZATIONS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeInsurance() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_INSURANCE));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_INSURANCE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeLabPanels() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_LAB_PANELS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_LAB_PANELS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeLabAccessions() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_LAB_ACCESSIONS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_LAB_ACCESSIONS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeMedications() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_MEDICATIONS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_MEDICATIONS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizePatientFlags() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_PATIENT_FLAGS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_PATIENT_FLAGS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizePCE() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_PCE));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_PCE));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeRadiology() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_RADIOLOGY));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_RADIOLOGY));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }

    public final int getParserBatchSizeVitals() {
        int rtn;
        try {
            rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_VITALS));
        } catch (NumberFormatException nfe) {
            try {
                rtn = Integer.parseInt(appProperties.getProperty(PARSER_BATCH_SIZE_VITALS));
            } catch (NumberFormatException nfe2) {
                rtn = PARSER_BATCH_SIZE_DEFAULT;
            }
        }
        return rtn;
    }
}
