package gov.va.med.vds.das.beans;

/**
 * Created by vhahongotom on 1/19/2017.
 */

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Locale;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DateDeserializer extends StdDeserializer<Date> {
    private static final Logger LOGGER = LoggerFactory.getLogger(DateDeserializer.class);
    
    private static final String[] DATE_FORMATS = new String[]{
            "yyyyMMddHHmmss.SSSZ",
            "yyyyMMddHHmmssZ",
            "yyyyMMddHHmmss.SSS",
            "yyyyMMdd.HHmmss",
            "yyyyMMdd.hhmmss",
            "yyyy-MM-dd'T'HH:mm:ssZ",
            "yyyy-MM-dd'T'HH:mm:ssZZ",
            "yyyy-MM-dd'T'hh:mm:ssXXX",
            "yyyy-MM-dd'T'hh:mm:ss.SSSZ",
            "yyyy-MM-dd HH:mm:ss",
            "dd-MMM-yyyy@HHmm",
            "yyyy-MM-dd",
            "MM/dd/yy",
            "MMM dd yyyy hh:mma",
            "MMM dd, yyyy HH:mm:ss",
            "MMM dd, yyyy@HH:mm",
            "MMM dd, yyyy",
            "dd MMM yyyy",
            "ddMMMyy@HHmm",
            "yyyyMMddhhmmssZ",
            "yyyyMMddHHmmss",
            "yyyyMMddHHmm",
            "yyyyMMddhhmm",
            "ddMMMyy",
            "yyyyMMdd"
    };

    public DateDeserializer() {
        this(null);
    }

    public DateDeserializer(Class<?> vc) {
        super(vc);
    }

    @Override
    public Date deserialize(JsonParser jsonparser, DeserializationContext context)
            throws IOException, JsonProcessingException {
        for (String format : DATE_FORMATS) {
            try {
                return new SimpleDateFormat(format, Locale.US).parse(jsonparser.getText());
            } catch (ParseException e) {
                //Do nothing
                LOGGER.debug("Cannot Parse date for deserialize");
            }
        }
        return null;
    }

    public static Date getDate(String dateStr) {
        Date retValue = null;
        for (String format : DATE_FORMATS) {
            if(StringUtils.isNotEmpty(dateStr)) {
                try {
                    retValue = new SimpleDateFormat(format, Locale.US).parse(dateStr);
                } catch (ParseException e) {
                    //do nothing
                    LOGGER.debug("Cannot Parse date for getDate() : " + dateStr);
                }
            }
            if(retValue != null)
                return retValue;
        }
        return retValue;
    }


}