/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.patient.dao.rpc.AdmissionsRPC;
import gov.va.med.vds.vistabeans.AdmissionDetail;
import gov.va.med.vds.vistabeans.Diagnosis;
import gov.va.med.vds.vistabeans.Encounter;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.Procedure;
import gov.va.med.vds.vistabeans.QueryBean;

/**
 *
 *
 */
public class AdmissionsDao extends DaoBase {
    public AdmissionsDao(RPCConnection cxn) {
        super(cxn);
    }

    public PatientAdmission[] getPatientAdmissions(QueryBean queryBean) throws RPCException {
        AdmissionsRPC rpc = new AdmissionsRPC(cxn);
        return rpc.getAdmissions(queryBean.getStartDate(), queryBean.getEndDate());
    }

    public PatientAdmission[] getPatientAdmissionsSetContext(QueryBean queryBean) throws RPCException {
        AdmissionsRPC rpc = new AdmissionsRPC(cxn);
        setContext(rpc, VistaConstants.CPRS_CONTEXT);
        return rpc.getAdmissions(queryBean.getStartDate(), queryBean.getEndDate());
    }

    public Encounter[] getPatientAdmissionsEncounters(QueryBean queryBean) throws RPCException {
        AdmissionsRPC rpc = new AdmissionsRPC(cxn);
        return rpc.getAdmissionsEncounters(queryBean.getStartDate(), queryBean.getEndDate());
    }

    public AdmissionDetail getAdmissionDetails(QueryBean queryBean) throws RPCException, UnsupportedRPCMethodException {

        AdmissionsRPC rpc = new AdmissionsRPC(cxn);
//        return rpc.getAdmissionDetailsReport(queryBean.getStartDate(), queryBean.getEndDate());
        return rpc.getExpandedAdmissionDetailsReport(queryBean.getStartDate(), queryBean.getEndDate());

    }

    public Diagnosis[] getAdmissionDiagnosis(QueryBean queryBean) throws RPCException, UnsupportedRPCMethodException {

        //TODO: Could we just return the Expanded Admission report?

        throw new UnsupportedRPCMethodException("No supported RPC method");

    }

    public Procedure[] getAdmissionProcedures(QueryBean queryBean) throws RPCException, UnsupportedRPCMethodException {


        //TODO: Could we just return the Expanded Admission report?
        throw new UnsupportedRPCMethodException("No supported RPC method");

    }
}
