/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.patient.dao.rpc.MedicationsRPC;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Medication;
import gov.va.med.vds.vistabeans.QueryBean;

/**
 *
 *
 */
public class MedicationsDao extends DaoBase {
    public MedicationsDao(RPCConnection cxn) {
        super(cxn);
    }

    public Medication[] getPatientMedications(QueryBean queryBean) throws RPCException {
        Medication[] rtc;
        MedicationsRPC rpc = new MedicationsRPC(cxn);

        if ("O".equals(queryBean.getStatus())) {
            rtc = rpc.getOutpatientMedications(queryBean.getStartDate(), queryBean.getEndDate());
        } else {
            rtc = rpc.getInpatientMedications(queryBean.getStartDate(), queryBean.getEndDate());
        }
        return rtc;
    }

    /**
     * Returns the medication detail. The querybean parameter needs to contain the patient IEN, the medication IEN, and
     * the order IEN. The medication and order IEN need to be in the itemId field of the queryBean. The patient IEN is
     * contained in the Patient bean of the queryBean. The IENs are separated by ^
     *
     * @param queryBean In queryBean.itemId the medication IEN and order IEN separated by ^, e.g., 123456;O^8901234
     *                  where the medication IEN is 123456;O and the order IEN is 8901234.
     * @return
     * @throws RPCException
     * @throws UnsupportedRPCMethodException
     */
    public FreeTextReport getMedicationDetail(QueryBean queryBean) throws RPCException, UnsupportedRPCMethodException {

        MedicationsRPC rpc = new MedicationsRPC(cxn);
        return rpc.getMedicationDetail(queryBean.getItemId());
    }
}
