/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.patient.dao.rpc.NotesRPC;
import gov.va.med.vds.vistabeans.*;

import java.util.List;

/**
 *
 *
 */
public class NotesDao extends DaoBase {
    public NotesDao(RPCConnection cxn) {
        super(cxn);
    }

    public ProgressNote[] getPatientProgressNotes(QueryBean queryBean) throws RPCException {

        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getProgressNotesVPR(queryBean.getStartDate(), queryBean.getEndDate());
    }


    public ProgressNote[] getPatientDischargeSummaries(QueryBean queryBean) throws RPCException {

        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getDischargeSummariesVPR(queryBean.getStartDate(), queryBean.getEndDate());
    }

    public Consult[] getPatientConsultRequests(QueryBean queryBean) throws RPCException {

        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getConsultRequestsVPR(queryBean.getStartDate(), queryBean.getEndDate());
    }

    public FreeTextReport[] getConsultReport(QueryBean queryBean) throws RPCException {
        FreeTextReport[] reports = null;


        NotesRPC rpc = new NotesRPC(cxn);
        reports = new FreeTextReport[1];
        reports[0] = rpc.getOutpatientConsultReport(queryBean.getItemId());
        return reports;
    }

    public FreeTextReport getProgressNote(QueryBean queryBean) throws RPCException {
        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getNotesById(queryBean.getItemId());
    }

    public FreeTextReport getVisitNotes(QueryBean queryBean) throws RPCException {

        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getVisitNotes(queryBean);
    }

    public Consult getPatientConsultRequestById(QueryBean queryBean) throws RPCException {
        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getOutpatientConsultRequestByIdVPR(queryBean.getItemId());
    }

    public Consult[] getPatientActiveConsultRequests(QueryBean queryBean) throws RPCException {
        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getActiveConsultRequestsVPR();
    }

    public PatientPostings getPatientPostingsList() throws RPCException {
        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getPatientPostings();
    }

    public FreeTextReport getPatientPostingsAllergiesSummary() throws RPCException {
        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getPostingsAllergiesSummary();
    }

    public List<PCE> getPcesForNote(QueryBean queryBean) throws RPCException {
        NotesRPC rpc = new NotesRPC(cxn);
        return rpc.getPcesForNote(queryBean.getItemId());
    }
}
