/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.patient.dao.rpc.PCERPC;
import gov.va.med.vds.vistabeans.*;

/**
 *
 *
 */
public class PCEDao extends DaoBase {
    public PCEDao(RPCConnection cxn) {
        super(cxn);
    }

    /**
     * Retrieves list of patient skin tests.
     *
     * @param queryBean QueryBean.
     * @return List of patient skin tests.
     * @throws RPCException if there was an error..
     */
    public PCE[] getPatientSkinTests(QueryBean queryBean) throws RPCException {
        PCERPC rpc = new PCERPC(cxn);

        return rpc.getSkinTests(queryBean.getStartDate(), queryBean.getEndDate());
    }

    /**
     * Retrieves list of patient PCE Exams.
     *
     * @param queryBean QueryBean.
     * @return List of patient PCE Exams.
     * @throws RPCException if there was an error..
     */
    public PCE[] getPatientExams(QueryBean queryBean) throws RPCException {
        PCERPC rpc = new PCERPC(cxn);

        return rpc.getExams(queryBean.getStartDate(), queryBean.getEndDate());
    }

    /**
     * Retrieves list of patient PCE education topics.
     *
     * @param queryBean QueryBean.
     * @return List of patient PCE education topics.
     * @throws RPCException if there was an error..
     */
    public PCE[] getPatientEducationTopics(QueryBean queryBean) throws RPCException {
        PCERPC rpc = new PCERPC(cxn);

        return rpc.getEducationTopics(queryBean.getStartDate(), queryBean.getEndDate());
    }

}
