/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 *
 */
public class LabsRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(LabsRPC.class);

    private static final String XPATH_PANEL = "//panel";
    private static final String XPATH_ACCESSION = "//accession";
    private static final String XPATH_ORDER_NAME = "./order/@name";
    private static final String XPATH_ORDER_CODE = "./order/@code";
    private static final String XPATH_COLLECTED = "./collected/@value";
    private static final String XPATH_RESULTED = "./resulted/@value";
    private static final String XPATH_COMMENT = "./comment";
    private static final String XPATH_GROUPNAME = "./groupName/@value";
    private static final String XPATH_SPECIMEN = "./specimen/@name";
    private static final String XPATH_STATUS = "./status/@value";
    private static final String XPATH_TYPE = "./type/@value";
    private static final String XPATH_NAME = "./name/@value";
    private static final String XPATH_FACILITY_NAME = "./facility/@name";
    private static final String XPATH_FACILITY_CODE = "./facility/@code";
    private static final String XPATH_VALUES = "./values";
    private static final String XPATH_TEST_VALUE = "./value";
    private static final String XPATH_TEST_ID = "@id";
    private static final String XPATH_TEST_TEST = "@test";
    private static final String XPATH_TEST_RESULT = "@result";
    private static final String XPATH_TEST_UNITS = "@units";
    private static final String XPATH_TEST_INTERPRETATION = "@interpretation";
    private static final String XPATH_TEST_HIGH = "@high";
    private static final String XPATH_TEST_LOW = "@low";
    private static final String XPATH_TEST_LOINC = "@loinc";
    private static final String XPATH_TEST_VUID = "@vuid";
    private static final String XPATH_TEST_PERFORMING_LAB = "@performingLab";
    private static final String XPATH_DOCUMENTS = "./documents";
    private static final String XPATH_DOCUMENT_LOCAL_TITLE = "./@localTitle";
    private static final String XPATH_DOCUMENT = "./document";
    private static final String XPATH_CONTENT = "./content";
    private static final String XPATH_PROVIDER = "./provider";
    private static final String XPATH_PROVIDER_CODE = "./@code";
    private static final String XPATH_PROVIDER_NAME = "./@name";
    private static final String XPATH_PROVIDER_TAXONOMY_CODE = "./@taxonomyCode";
    private static final String XPATH_PROVIDER_PROVIDER_TYPE = "./@providerType";
    private static final String XPATH_PROVIDER_CLASSIFICATION = "./@classification";
    private static final String XPATH_PROVIDER_SPECIALIZATION = "./@specialization";
    private static final String XPATH_SERVICE_ATTR = "./@service";
    private static final String XPATH_OFFICE_PHONE = "./@officePhone";
    private static final String XPATH_FAX = "./@fax";
    private static final String XPATH_ANALOG_PAGER = "./@analogPager";
    private static final String XPATH_EMAIL = "./@email";
    
    private static final XPathExpression exprPanel = XPathUtil.xpathCompile(XPATH_PANEL);
    private static final XPathExpression exprAccession = XPathUtil.xpathCompile(XPATH_ACCESSION);
    private static final XPathExpression exprOrderName = XPathUtil.xpathCompile(XPATH_ORDER_NAME);
    private static final XPathExpression exprCollected = XPathUtil.xpathCompile(XPATH_COLLECTED);
    private static final XPathExpression exprComment = XPathUtil.xpathCompile(XPATH_COMMENT);
    private static final XPathExpression exprGroupName = XPathUtil.xpathCompile(XPATH_GROUPNAME);
    private static final XPathExpression exprSpecimen = XPathUtil.xpathCompile(XPATH_SPECIMEN);
    private static final XPathExpression exprStatus = XPathUtil.xpathCompile(XPATH_STATUS);
    private static final XPathExpression exprType = XPathUtil.xpathCompile(XPATH_TYPE);
    private static final XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ORDER_CODE);
    private static final XPathExpression exprFacilityName = XPathUtil.xpathCompile(XPATH_FACILITY_NAME);
    private static final XPathExpression exprFacilityCode = XPathUtil.xpathCompile(XPATH_FACILITY_CODE);
    private static final XPathExpression exprValues = XPathUtil.xpathCompile(XPATH_VALUES);
    private static final XPathExpression exprTestValue = XPathUtil.xpathCompile(XPATH_TEST_VALUE);
    private static final XPathExpression exprTestId = XPathUtil.xpathCompile(XPATH_TEST_ID);
    private static final XPathExpression exprTestTest = XPathUtil.xpathCompile(XPATH_TEST_TEST);
    private static final XPathExpression exprTestResult = XPathUtil.xpathCompile(XPATH_TEST_RESULT);
    private static final XPathExpression  exprTestUnits = XPathUtil.xpathCompile(XPATH_TEST_UNITS);
    private static final XPathExpression exprTestInterpretation = XPathUtil.xpathCompile(XPATH_TEST_INTERPRETATION);
    private static final XPathExpression exprTestHigh = XPathUtil.xpathCompile(XPATH_TEST_HIGH);
    private static final XPathExpression exprTestLow = XPathUtil.xpathCompile(XPATH_TEST_LOW);
    private static final XPathExpression exprTestLoinc = XPathUtil.xpathCompile(XPATH_TEST_LOINC);
    private static final XPathExpression exprTestVuid = XPathUtil.xpathCompile(XPATH_TEST_VUID);
    private static final XPathExpression exprTestPerformingLab = XPathUtil.xpathCompile(XPATH_TEST_PERFORMING_LAB);
    private static final XPathExpression exprResulted = XPathUtil.xpathCompile(XPATH_RESULTED);
    private static final XPathExpression exprProvider = XPathUtil.xpathCompile(XPATH_PROVIDER);
    private static final XPathExpression exprProviderCode = XPathUtil.xpathCompile(XPATH_PROVIDER_CODE);
    private static final XPathExpression exprProviderName = XPathUtil.xpathCompile(XPATH_PROVIDER_NAME);
    private static final XPathExpression exprProviderTaxonomyCode = XPathUtil.xpathCompile(XPATH_PROVIDER_TAXONOMY_CODE);
    private static final XPathExpression exprProviderProviderType = XPathUtil.xpathCompile(XPATH_PROVIDER_PROVIDER_TYPE);
    private static final XPathExpression exprProviderClassification = XPathUtil.xpathCompile(XPATH_PROVIDER_CLASSIFICATION);
    private static final XPathExpression exprProviderSpecialization = XPathUtil.xpathCompile(XPATH_PROVIDER_SPECIALIZATION);
    private static final XPathExpression exprProviderService = XPathUtil.xpathCompile(XPATH_SERVICE_ATTR);
    private static final XPathExpression exprProviderOfficePhone = XPathUtil.xpathCompile(XPATH_OFFICE_PHONE);
    private static final XPathExpression exprProviderAnalogPager = XPathUtil.xpathCompile(XPATH_ANALOG_PAGER);
    private static final XPathExpression exprProviderFax = XPathUtil.xpathCompile(XPATH_FAX);
    private static final XPathExpression exprProviderEmail = XPathUtil.xpathCompile(XPATH_EMAIL);
    private static final XPathExpression exprDocuments = XPathUtil.xpathCompile(XPATH_DOCUMENTS);
    private static final XPathExpression exprDocument = XPathUtil.xpathCompile(XPATH_DOCUMENT);
    private static final XPathExpression exprDocumentLocalTitle = XPathUtil.xpathCompile(XPATH_DOCUMENT_LOCAL_TITLE);
    private static final XPathExpression exprDocumentContent = XPathUtil.xpathCompile(XPATH_CONTENT);
    private static final XPathExpression exprName = XPathUtil.xpathCompile(XPATH_NAME);


    private static final int PARSER_LAB_PANELS_BATCH_SIZE = APP_CONFIG.getParserBatchSizeLabPanels();
    private static final int PARSER_LAB_ACCESSIONS_BATCH_SIZE = APP_CONFIG.getParserBatchSizeLabAccessions();

    private static final String DOUBLE_NEWLINE = "\n\n";
    private static final String DATETIMEFORMAT = "MM/dd/yyy HH:mm";
    private static final String DATEFORMAT = "MM/dd/yyy";
    private static final String FOOTER = "===============================================================================";

    public LabsRPC(RPCConnection cxn) {
        super(cxn);
    }

    public LabOrder[] getPatientLabs(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientLabs(cxn.getPatientIen(), startDate, endDate);
    }

    public LabOrder[] getPatientLabs(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetPatientLabsRequest(patientIen/*, startDate, endDate*/);
        String response = (String) cxn.query(request);

        LabOrder[] labs = toPatientLabs(response);

        ArrayList<LabOrder> labsFiltered = new ArrayList<LabOrder>();
        //filter by date range
        for (LabOrder labOrder : labs) {
            if (labOrder.getCollectionDate() == null) {
                labsFiltered.add(labOrder);
            }  else {
                Calendar labOrderDate = labOrder.getCollectionDate();
                if (labOrderDate.compareTo(startDate) >= 0 && labOrderDate.compareTo(endDate) <= 0) {
                    labsFiltered.add(labOrder);
                }
            }
        }


        return labsFiltered.toArray(new LabOrder[0]);
    }

    public LabOrder[] getPatientMicroLabOrders(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientMicroLabOrders(cxn.getPatientIen(), startDate, endDate);
    }

    public LabOrder[] getPatientMicroLabOrders(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetPatientMicroLabsRequest(patientIen/*, startDate, endDate*/);
        String microResponse = (String) cxn.query(request);

        LabOrder[] microLabs = toPatientMicroLabs(microResponse);

        ArrayList<LabOrder> microLabsFiltered = new ArrayList<LabOrder>();
        //filter by date range
        for (LabOrder labOrder : microLabs) {
            Calendar labOrderDate = labOrder.getCollectionDate();
            if (labOrderDate.compareTo(startDate) >= 0 && labOrderDate.compareTo(endDate) <= 0) {
                microLabsFiltered.add(labOrder);
            }
        }

        return microLabsFiltered.toArray(new LabOrder[0]);
    }

    public LabResult[] getPatientMicroLabResults(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientMicroLabResults(cxn.getPatientIen(), startDate, endDate);
    }

    public LabResult[] getPatientMicroLabResults(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        LabOrder[] microLabs = getPatientMicroLabOrders(patientIen, startDate, endDate);

        return buildLabResultsList(microLabs);
    }


    /**
     * Get the list of Anatomic Pathology Labs. This is done in the same way as CPRS using the ORWRP REPORT TEXT RPC
     * with L:OR_APL:ANATOMIC PATHOLOGY - ALL REPORTS~SP;ORDV02A;0;999 parameter.
     *
     * @return
     * @throws RPCException
     */
    public LabOrder[] getPatientAnaPathLabOrders(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientAnaPathLabOrders(cxn.getPatientIen(), startDate, endDate);
    }

    public LabOrder[] getPatientAnaPathLabOrders(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetPatientAnaPathLabsRequest(patientIen, startDate, endDate);
        String anaPathResponse = (String) cxn.query(request);

        LabOrder[] anaPathLabs = toPatientAnaPathLabs(anaPathResponse);

        ArrayList<LabOrder> apLabsFiltered = new ArrayList<LabOrder>();
        //filter by date range
        for (LabOrder labOrder : anaPathLabs) {
            Calendar labOrderDate = labOrder.getCollectionDate();
            if (labOrderDate.compareTo(startDate) >= 0 && labOrderDate.compareTo(endDate) <= 0) {
                apLabsFiltered.add(labOrder);
            }
        }

        return apLabsFiltered.toArray(new LabOrder[0]);
    }

    public LabResult[] getPatientAnaPathLabResults(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientAnaPathLabResults(cxn.getPatientIen(), startDate, endDate);
    }

    public LabResult[] getPatientAnaPathLabResults(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        LabOrder[] anaPathLabs = getPatientAnaPathLabOrders(patientIen, startDate, endDate);
        
        return buildLabResultsList(anaPathLabs);
    }

    private LabResult[] buildLabResultsList(LabOrder[] microLabs) {
        // Get the first lab result from each order and add them to a list
        ArrayList<LabResult> labResultsList = new ArrayList<LabResult>();
        for (LabOrder labOrder : microLabs) {
            LabResult[] labOrderResultsArray = labOrder.getLabResults();
            if (labOrderResultsArray.length > 0) {
                LabResult labResult = labOrderResultsArray[0];
                labResultsList.add(labResult);
            }
        }

        return labResultsList.toArray(new LabResult[0]);
    }
    

    private String buildGetPatientLabsRequest(String patientIen/*, Calendar startDate, Calendar endDate*/) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        /* ORWRP REPORT TEXT (Lab Order Status)
         *
         * This RPC is used to retrieve information on lab orders and their status from the Lab Order Entry file (#69).
         */
        vquery.addParameter(VistaQuery.LITERAL, "OR_LO:LAB ORDERS~LO;ORDV02;32;999");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "550000");
        vquery.addParameter(VistaQuery.LITERAL, "");

//        String startDateString = Timestamp.fromDateTime(startDate);
//        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
//        startDateString = flds[0];
//        String endDateString = Timestamp.fromDateTime(endDate);
//        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
//        endDateString = flds[0];
//        vq.addParameter(vq.LITERAL, startDateString);
//        vq.addParameter(vq.LITERAL, endDateString);
        vquery.addParameter(VistaQuery.LITERAL, "0");
        vquery.addParameter(VistaQuery.LITERAL, "0");


        return vquery.buildMessage();
    }

    private String buildGetPatientMicroLabsRequest(String patientIen/*, Calendar startDate, Calendar endDate*/) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);

        vquery.addParameter(VistaQuery.LITERAL, "L:OR_MICL:MICROBIOLOGY~MI;ORDV05;38;999");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "550000");
        vquery.addParameter(VistaQuery.LITERAL, "");

//        String startDateString = Timestamp.fromDateTime(startDate);
//        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
//        startDateString = flds[0];
//        String endDateString = Timestamp.fromDateTime(endDate);
//        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
//        endDateString = flds[0];
//        vq.addParameter(vq.LITERAL, startDateString);
//        vq.addParameter(vq.LITERAL, endDateString);
        vquery.addParameter(VistaQuery.LITERAL, "0");
        vquery.addParameter(VistaQuery.LITERAL, "0");

        return vquery.buildMessage();
    }

    private String buildGetPatientAnaPathLabsRequest(String patientIen, Calendar startDate, Calendar endDate) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);

        vquery.addParameter(VistaQuery.LITERAL, "L:OR_APL:ANATOMIC PATHOLOGY - ALL REPORTS~SP;ORDV02A;0;999");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "550000");
        vquery.addParameter(VistaQuery.LITERAL, "");

//        String startDateString = Timestamp.fromDateTime(startDate);
//        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
//        startDateString = flds[0];
//        String endDateString = Timestamp.fromDateTime(endDate);
//        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
//        endDateString = flds[0];
//        vq.addParameter(vq.LITERAL, startDateString);
//        vq.addParameter(vq.LITERAL, endDateString);
        vquery.addParameter(VistaQuery.LITERAL, "0");
        vquery.addParameter(VistaQuery.LITERAL, "0");

        return vquery.buildMessage();
    }

    private LabOrder[] toPatientLabs(String response) {
        /*
         * ORWRP REPORT TEXT (Lab Order Status); OR_LO:LAB ORDERS~LO;ORDV02;32;999
         * The results of this RPC are returned as a delimited 13-node global array.  Each node has different delimited
         * data elements and contains the collection date/time, the lab test name, the lab test IEN, the topography
         * field name (this is also known as the specimen), the topography field IEN, the urgency (i.e., "ROUTINE",
         * "STAT", etc.), the provider name (which is truncated to 10 characters), the provider IEN, the ordered
         * date/time, the accession number, the available date/time and the status (i.e., COMPLETED, CANCELLED, etc.).
         */

        final int ORDER_FACILITY = 1;
        final int ORDER_COLLECTION_DATETIME = 2;
        final int ORDER_LAB_TEST_NAME = 3;
        final int ORDER_LAB_TEST_IEN = 4;
        final int ORDER_TOPO_FIELD_NAME = 5;
        final int ORDER_TOPO_FIELD_IEN = 6;
        final int ORDER_URGENCY = 7;
        final int ORDER_PROVIDER_NAME = 8;
        final int ORDER_PROVIDER_IEN = 9;
        final int ORDER_ORDERED_DATETIME = 10;
        final int ORDER_ACCESSION = 11;
        final int ORDER_AVAILABLE_DATETIME = 12;
        final int ORDER_STATUS = 13;

        LabOrder[] rtc;
        if ("".equals(response)) {
            rtc = new LabOrder[0];
        } else {
    
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<LabOrder> labsList = new ArrayList<LabOrder>();
    
            LabOrder labOrder = null;
    
            for (String line : lines) {
                if ("".equals(line)) {
                    continue;
                }
    
                // Each line is the line number and the data delimited by a CARET
                String[] fields = StringUtils.split(line, StringUtils.CARET);
                int lineNumber = 0;
                String fieldData = "";
                for (int i = 0; i < fields.length; i++) {
                    if (i == 0) {
                        lineNumber = Integer.parseInt(fields[i]);
                    } else if (i == 1) {
                        fieldData = fields[i];
                    }
                }
    
                switch (lineNumber) {
                    case ORDER_FACILITY:
                        // Each Lab Test starts with the facility name (line 1) so add the old one to the list
                        if (labOrder != null) {
                            addLabResultToOrder(labOrder);
                            labsList.add(labOrder);
                        }
                        labOrder = new LabOrder();
                        fillCommonDataBeanValues(labOrder);
    
                        String[] subfields = StringUtils.split(fieldData, StringUtils.SEMICOLON);
                        for (int j = 0; j < subfields.length; j++) {
                            switch (j) {
                                case 0:
                                    labOrder.setFacilityName(subfields[0]);
                                    break;
                                case 1:
                                    labOrder.setStationNumber(subfields[1]);
                                    break;
                                default:
                                    break;
                            }
                        }
                        break;
                    case ORDER_COLLECTION_DATETIME:
                        Calendar collectionDateTime = parseDateTime(fieldData);
                        if (collectionDateTime != null && labOrder != null) {
                            labOrder.setCollectionDate(collectionDateTime);
                        }
                        break;
                    case ORDER_LAB_TEST_NAME:
                        if (labOrder != null) {
                            labOrder.setTestName(fieldData);
                        }
                        break;
                    case ORDER_LAB_TEST_IEN:
                        if (labOrder != null) {
                            labOrder.setLabTestIEN(fieldData);
                        }
                        break;
                    case ORDER_TOPO_FIELD_NAME:
                        if (labOrder != null) {
                            labOrder.setCollectionSample(fieldData);
                        }
                        break;
                    case ORDER_TOPO_FIELD_IEN:
                        if (labOrder != null) {
                            labOrder.setCollectionSampleIEN(fieldData);
                        }
                        break;
                    case ORDER_URGENCY:
                        if (labOrder != null) {
                            labOrder.setUrgency(fieldData);
                        }
                        break;
                    case ORDER_PROVIDER_NAME:
                        if (labOrder != null) {
                            Provider provider = new Provider();
                            provider.setName(fieldData);
                            labOrder.setRequestingProvider(provider);
                        }
                        break;
                    case ORDER_PROVIDER_IEN:
                        if (labOrder != null) {
                            labOrder.setRequestingProviderIEN(fieldData);
                        }
                        break;
                    case ORDER_ORDERED_DATETIME:
                        Calendar orderDateTime = parseDateTime(fieldData);
                        if (orderDateTime != null && labOrder != null) {
                            labOrder.setOrderDate(orderDateTime);
                        }
                        break;
                    case ORDER_ACCESSION:
                        if (labOrder != null) {
                            labOrder.setAccessionNumber(fieldData);
                        }
                        break;
                    case ORDER_AVAILABLE_DATETIME:
                        Calendar availableDateTime = parseDateTime(fieldData);
                        if (availableDateTime != null && labOrder != null) {
                            labOrder.setAvailableDate(availableDateTime);
                        }
                        break;
                    case ORDER_STATUS:
                        if (labOrder != null) {
                            labOrder.setStatus(fieldData);
                        }
                        break;
                    default:
                        break;
                }
            }
    
            if (labOrder != null) {
                addLabResultToOrder(labOrder);
                labsList.add(labOrder);
            }
    
            rtc = labsList.toArray(new LabOrder[0]);
        }
        return rtc;
    }

    private void addLabResultToOrder(LabOrder labOrder) {
        if (labOrder != null) {
            LabResult labResult = new LabResult();
            fillCommonDataBeanValues(labResult);
            // Micro labs have only a single result so copy the result with things from the order
            labResult.setAccession(labOrder.getAccessionNumber());
            labResult.setTestName(labOrder.getTestName());
            labResult.setResultDate(labOrder.getCollectionDate());
            labResult.setSpecimen(labOrder.getCollectionSample());
            labResult.setFacilityName(labOrder.getFacilityName());
            labResult.setStationNumber(labOrder.getStationNumber());
            labResult.setOrderComment(labOrder.getComment());
            // Add the result to the order
            labOrder.setLabResults(new LabResult[]{labResult});
        }
    }

    private LabOrder[] toPatientMicroLabs(String response) {
        /*
         * ORWRP REPORT TEXT (Lab Order Status); OR_LO:LAB ORDERS~LO;ORDV02;32;999
         * The results of this RPC are returned as a delimited 13-node global array.  Each node has different delimited
         * data elements and contains the collection date/time, the lab test name, the lab test IEN, the topography
         * field name (this is also known as the specimen), the topography field IEN, the urgency (i.e., "ROUTINE",
         * "STAT", etc.), the provider name (which is truncated to 10 characters), the provider IEN, the ordered
         * date/time, the accession number, the available date/time and the status (i.e., COMPLETED, CANCELLED, etc.).
         */

        final int ORDER_FACILITY = 1;
        final int ORDER_COLLECTION_DATETIME = 2;
        final int ORDER_LAB_TEST_NAME = 3;
        final int ORDER_TOPO_FIELD_NAME = 4;
        final int ORDER_ACCESSION = 6;
        final int ORDER_REPORT = 7;

        LabOrder[] rtc;
        if ("".equals(response)) {
            rtc = new LabOrder[0];
        } else {
    
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<LabOrder> labsList = new ArrayList<LabOrder>();
    
            LabOrder labOrder = null;
            StringBuilder reportTextSB = new StringBuilder();
    
            for (String line : lines) {
                if ("".equals(line)) {
                    continue;
                }
    
                // Each line is the line number and the data delimited by a CARET
                String[] fields = StringUtils.split(line, StringUtils.CARET);
                int lineNumber = 0;
                String fieldData = "";
                for (int i = 0; i < fields.length; i++) {
                    if (i == 0) {
                        lineNumber = Integer.parseInt(fields[i]);
                    } else if (i == 1) {
                        fieldData = fields[i];
                    }
                }
    
                switch (lineNumber) {
                    case ORDER_FACILITY:
                        // Each Lab Test starts with the facility name (line 1) so add the old one to the list
                        if (labOrder != null && reportTextSB.length() > 0) {
                            addMicroResultToOrder(labOrder, reportTextSB.toString());
                            labsList.add(labOrder);
                        }
                        labOrder = new LabOrder();
                        fillCommonDataBeanValues(labOrder);
                        labOrder.setSubscript("MI");
                        reportTextSB.setLength(0);
    
                        String[] subfields = StringUtils.split(fieldData, StringUtils.SEMICOLON);
                        for (int j = 0; j < subfields.length; j++) {
                            switch (j) {
                                case 0:
                                    labOrder.setFacilityName(subfields[0]);
                                    break;
                                case 1:
                                    labOrder.setStationNumber(subfields[1]);
                                    break;
                                default:
                                    break;
                            }
                        }
                        break;
                    case ORDER_COLLECTION_DATETIME:
                        Calendar collectionDateTime = parseDateTime(fieldData);
                        if (collectionDateTime != null && labOrder != null) {
                            labOrder.setCollectionDate(collectionDateTime);
                        }
                        break;
                    case ORDER_LAB_TEST_NAME:
                        if (labOrder != null) {
                            labOrder.setTestName(fieldData);
                        }
                        break;
                    case ORDER_TOPO_FIELD_NAME:
                        if (labOrder != null) {
                            labOrder.setCollectionSample(fieldData);
                        }
                        break;
                    case ORDER_ACCESSION:
                        if (labOrder != null) {
                            labOrder.setAccessionNumber(fieldData);
                        }
                        break;
                    case ORDER_REPORT:
                        reportTextSB.append(fieldData);
                        reportTextSB.append("\n");
                        break;
                    default:
                        break;
                }
            }
    
            if (labOrder != null && reportTextSB != null) {
                addMicroResultToOrder(labOrder, reportTextSB.toString());
                labsList.add(labOrder);
            }
    
            rtc = labsList.toArray(new LabOrder[0]);
        }
        return rtc;
    }

    private LabOrder[] toPatientAnaPathLabs(String response) {
        /*
         * ORWRP REPORT TEXT (Lab Order Status); OR_LO:LAB ORDERS~LO;ORDV02;32;999
         * The results of this RPC are returned as a delimited 13-node global array.  Each node has different delimited
         * data elements and contains the collection date/time, the lab test name, the lab test IEN, the topography
         * field name (this is also known as the specimen), the topography field IEN, the urgency (i.e., "ROUTINE",
         * "STAT", etc.), the provider name (which is truncated to 10 characters), the provider IEN, the ordered
         * date/time, the accession number, the available date/time and the status (i.e., COMPLETED, CANCELLED, etc.).
         */

        final int ORDER_FACILITY = 1;
        final int ORDER_COLLECTION_DATETIME = 2;
        final int ORDER_TOPO_FIELD_NAME = 3;
        final int ORDER_ACCESSION = 4;
        final int ORDER_REPORT = 5;

        LabOrder[] rtc;
        if ("".equals(response)) {
            rtc = new LabOrder[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<LabOrder> labsList = new ArrayList<LabOrder>();
    
            LabOrder labOrder = null;
            StringBuilder reportTextSB = new StringBuilder();
    
            for (String line : lines) {
                if ("".equals(line)) {
                    continue;
                }
    
                // Each line is the line number and the data delimited by a CARET
                String[] fields = StringUtils.split(line, StringUtils.CARET);
                int lineNumber = 0;
                String fieldData = "";
                for (int i = 0; i < fields.length; i++) {
                    if (i == 0) {
                        lineNumber = Integer.parseInt(fields[i]);
                    } else if (i == 1) {
                        fieldData = fields[i];
                    }
                }
    
                switch (lineNumber) {
                    case ORDER_FACILITY:
                        // Each Lab Test starts with the facility name (line 1) so add the old one to the list
                        if (labOrder != null && reportTextSB.length() > 0) {
                            addAnaPathResultToOrder(labOrder, reportTextSB.toString());
                            labsList.add(labOrder);
                        }
                        labOrder = new LabOrder();
                        fillCommonDataBeanValues(labOrder);
                        labOrder.setSubscript("AP");
    
                        reportTextSB.setLength(0);
    
                        String[] subfields = StringUtils.split(fieldData, StringUtils.SEMICOLON);
                        for (int j = 0; j < subfields.length; j++) {
                            switch (j) {
                                case 0:
                                    labOrder.setFacilityName(subfields[0]);
                                    break;
                                case 1:
                                    labOrder.setStationNumber(subfields[1]);
                                    break;
                                default:
                                    break;
                            }
                        }
                        break;
                    case ORDER_COLLECTION_DATETIME:
                        Calendar collectionDateTime = parseDateTime(fieldData);
                        if (collectionDateTime != null && labOrder != null) {
                            labOrder.setCollectionDate(collectionDateTime);
                        }
                        break;
                    case ORDER_TOPO_FIELD_NAME:
                        if (labOrder != null) {
                            labOrder.setCollectionSample(fieldData);
                        }
                        break;
                    case ORDER_ACCESSION:
                        if (labOrder != null) {
                            labOrder.setAccessionNumber(fieldData);
                        }
                        break;
                    case ORDER_REPORT:
                        reportTextSB.append(fieldData);
                        reportTextSB.append("\n");
                        break;
                    default:
                        break;
                }
            }
    
            if (labOrder != null && reportTextSB != null) {
                addAnaPathResultToOrder(labOrder, reportTextSB.toString());
                labsList.add(labOrder);
            }
    
            rtc = labsList.toArray(new LabOrder[0]);
        }
        return rtc;
    }

    private void addMicroResultToOrder(LabOrder labOrder, String labReport) {
        if (labOrder != null) {
            LabResult labResult = new LabResult();
            fillCommonDataBeanValues(labResult);
            labResult.setResult(labReport);
            // Micro labs have only a single result so copy the result with things from the order
            labResult.setAccession(labOrder.getAccessionNumber());
            labResult.setTestName(labOrder.getTestName());
            labResult.setResultDate(labOrder.getCollectionDate());
            labResult.setSpecimen(labOrder.getCollectionSample());
            labResult.setFacilityName(labOrder.getFacilityName());
            labResult.setStationNumber(labOrder.getStationNumber());
            // Add the result to the order
            labOrder.setLabResults(new LabResult[]{labResult});
        }
    }

//    private static final String titleMarker = "----";

    private void addAnaPathResultToOrder(LabOrder labOrder, String labReport) {
        if (labOrder != null) {
            // extract test name from report text
            // it is in the first non-white space line surrounded by "----" on each side
            StringBuilder titleSB = new StringBuilder();
            if (labOrder.getAccessionNumber() != null) {
                String accessionNumber = labOrder.getAccessionNumber().trim();
                int firstSpaceIndex = labOrder.getAccessionNumber().indexOf(" ");
                if (firstSpaceIndex > 0) {
                    titleSB.append(accessionNumber.substring(0, firstSpaceIndex));
                } else {
                    titleSB.append(accessionNumber);
                }
            }
//            int titleStartIndex = labReport.indexOf(titleMarker) + titleMarker.length();
//            int titleEndIndex = labReport.indexOf(titleMarker, titleStartIndex);
//            titleSB.append(labReport.substring(titleStartIndex, titleEndIndex).trim());
            if (labOrder.getCollectionSample() != null) {
                titleSB.append(" - ");
                titleSB.append(labOrder.getCollectionSample());
            }
            labOrder.setTestName(titleSB.toString());
            // add the fields to thee first result in the order
            addMicroResultToOrder(labOrder, labReport);
        }
    }

    public LabResult[] getLabOrderResult(Calendar startDate, Calendar endDate) throws RPCException {
        return getLabOrderResult(cxn.getPatientIen(), startDate, endDate);
    }

    public LabResult[] getLabOrderResult(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetLabOrderResultRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        LabResult[] labs = toLabOrderResult(/*patientIen, */response);

        return labs;
    }

    public LabResult[] getLabOrderResult(Calendar collectionDate, String accessionNumber) throws RPCException {
        return getLabOrderResult(cxn.getPatientIen(), collectionDate, accessionNumber);
    }

    public LabResult[] getLabOrderResult(String patientIen, Calendar collectionDate, String accessionNumber) throws RPCException {

        // Need to search around the collection date in case the date is off
        Calendar endDate = (Calendar) collectionDate.clone();
        Calendar startDate = (Calendar) collectionDate.clone();

        // Pad by two days before and after 48 hr * 60 min * 60 sec * 1000 ms
        endDate.setTimeInMillis(collectionDate.getTimeInMillis() + 172800000);
        startDate.setTimeInMillis(collectionDate.getTimeInMillis() - 172800000);

        // Get the Lab Orders using the VPR call
        LabOrder[] labOrders = getVPRLabOrders(patientIen, startDate, endDate);

        // find the order(s) that have the accession number
        ArrayList<LabResult> resultsList = new ArrayList<LabResult>();

        for (LabOrder labOrder : labOrders) {
            if (labOrder.getAccessionNumber() != null
                    && accessionNumber.equals(labOrder.getAccessionNumber())
                    && labOrder.getLabResults() != null) {
                resultsList.addAll(Arrays.asList(labOrder.getLabResults()));
            }
        }

        return resultsList.toArray(new LabResult[0]);
    }

    private String buildGetLabOrderResultRequest(String patientIen, Calendar startDate, Calendar endDate) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        /* ORWRP REPORT TEXT (Lab Order Status)
         *
         * This RPC is used to retrieve information on lab results from the Lab Data file (#63).
         */
        vquery.addParameter(VistaQuery.LITERAL, "OR_CH:CHEM & HEMATOLOGY~CH;ORDV02;3;999");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");

        String startDateString = Timestamp.fromDateTime(startDate);
        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
        startDateString = flds[0];
        String endDateString = Timestamp.fromDateTime(endDate);
        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
        endDateString = flds[0];
        vquery.addParameter(VistaQuery.LITERAL, startDateString);
        vquery.addParameter(VistaQuery.LITERAL, endDateString);

        return vquery.buildMessage();
    }

    private LabResult[] toLabOrderResult(/*String patientIen, */String response) {
        /*
         * ORWRP REPORT TEXT (Lab Results); OR_CH:CHEM & HEMATOLOGY~CH;ORDV02;3;999
         * The results of this RPC are returned as a delimited 9 or 11-node global array.  Nodes 10 and 11 only exist
         * if there are comments with the results.  Each node has different delimited data elements and contains the
         * facility name and station number (separated by a ";"), the collection date/time, the lab test name, the
         * specimen (also known as topography field) name, the lab test result, the test result flag (which can be "H"
         * for "High", "L" for "Low" or NULL for "Normal"), the units, the numeric reference low, the numeric reference
         * high, the comments and a "[+]" symbol.  The "[+]" symbol is used in the CPRS GUI to signify that there is more
         * text or lines for comments that may not be seen in the comments field in the CPRS GUI.  Thus, the symbol that
         * is given in Node 11 is not necessarily needed.
         */

        final int RESULT_FACILITY_NAME = 1;
        final int RESULT_COLLECTION_DATETIME = 2;
        final int RESULT_LAB_TEST_NAME = 3;
        final int RESULT_SPECIMEN_NAME = 4;
        final int RESULT_RESULT = 5;
        final int RESULT_FLAG = 6;
        final int RESULT_UNITS = 7;
        final int RESULT_REFERENCE_LOW = 8;
        final int RESULT_REFERENCE_HIGH = 9;
        final int RESULT_COMMENTS = 10;

        LabResult[] rtc;
        if ("".equals(response)) {
            rtc = new LabResult[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<LabResult> resultsList = new ArrayList<LabResult>();
    
            LabResult labResult = new LabResult();
            StringBuilder referenceRangeSB = new StringBuilder();
            StringBuilder commentStringBuilder = new StringBuilder();
    
            for (String line : lines) {
                if ("".equals(line)) {
                    continue;
                }
    
                // Each line is the line number and the data delimited by a CARET
                String[] fields = StringUtils.split(line, StringUtils.CARET);
                int lineNumber = 0;
                String fieldData = "";
                for (int i = 0; i < fields.length; i++) {
                    if (i == 0) {
                        lineNumber = Integer.parseInt(fields[i]);
                    } else if (i == 1) {
                        fieldData = fields[i];
                    }
                }
    
                switch (lineNumber) {
                    case RESULT_FACILITY_NAME:
                        // Each Lab Test starts with the facility name (line 1) so add the old one to the list
                        resultsList.add(labResult);
                        fillCommonDataBeanValues(labResult);
                        referenceRangeSB.setLength(0);
                        commentStringBuilder.setLength(0);
    
                        String[] subfields = StringUtils.split(fieldData, StringUtils.SEMICOLON);
                        for (int j = 0; j < subfields.length; j++) {
                            switch (j) {
                                case 0:
                                    labResult.setFacilityName(subfields[0]);
                                    break;
                                case 1:
                                    labResult.setStationNumber(subfields[1]);
                                    break;
                                default:
                                    break;
                            }
                        }
                        break;
                    case RESULT_COLLECTION_DATETIME:
                        Calendar collectionDateTime = parseDateTime(fieldData);
                        if (collectionDateTime != null) {
                            labResult.setResultDate(collectionDateTime);
                        }
                        break;
                    case RESULT_LAB_TEST_NAME:
                        labResult.setTestName(fieldData);
                        break;
                    case RESULT_SPECIMEN_NAME:
                        labResult.setSpecimen(fieldData);
                        break;
                    case RESULT_RESULT:
                        labResult.setResult(fieldData);
                        break;
                    case RESULT_FLAG:
                        labResult.setInterpretation(fieldData);
                        break;
                    case RESULT_UNITS:
                        labResult.setUnits(fieldData);
                        break;
                    case RESULT_REFERENCE_LOW:
                        referenceRangeSB.append(fieldData);
                        referenceRangeSB.append(" - ");
                        break;
                    case RESULT_REFERENCE_HIGH:
                        referenceRangeSB.append(fieldData);
                        labResult.setReferenceRange(referenceRangeSB.toString());
                        break;
                    case RESULT_COMMENTS:
                        if (labResult.getComment() != null) {
                            commentStringBuilder.delete(0, commentStringBuilder.length());
                            commentStringBuilder.append(labResult.getComment());
                        }
                        commentStringBuilder.append("\r\n");
                        commentStringBuilder.append(fieldData);
                        labResult.setComment(commentStringBuilder.toString());
                        break;
                    default:
                        break;
                }
            }
    
            fillCommonDataBeanValues(labResult);
            resultsList.add(labResult);
    
            rtc = resultsList.toArray(new LabResult[0]);
        }
        return rtc;
    }


    private static Calendar parseDateTime(String date) {
        SimpleDateFormat sdf = new SimpleDateFormat(DATETIMEFORMAT, Locale.getDefault());

        Calendar returnCalendar = new GregorianCalendar();

        try {
            returnCalendar.setTime(sdf.parse(date));
        } catch (ParseException pe) {
            returnCalendar = parseDate(date);
        }

        return returnCalendar;
    }

    private static Calendar parseDate(String date) {
        SimpleDateFormat sdf = new SimpleDateFormat(DATEFORMAT, Locale.getDefault());

        Calendar returnCalendar = new GregorianCalendar();

        try {
            returnCalendar.setTime(sdf.parse(date));
        } catch (ParseException pe) {
            returnCalendar = null;
        }

        return returnCalendar;
    }

    /**
     * For the date range, makes a call to ORWRP REPORT TEXT "OR_CH:CHEM & HEMATOLOGY~CH;ORDV02;3;999" and scans all
     * returned results for those that match the test name.
     *
     * @param startDate Start timestamp for range to search
     * @param endDate   End timestamp for range to search
     * @param testName  Name of the Lab Test to search
     * @return Array of LabResults that match testName in the given date range
     * @throws RPCException
     */
    public LabResult[] getPatientLabTestResults(Calendar startDate, Calendar endDate, String testName) throws RPCException {

        return getPatientLabTestResults(cxn.getPatientIen(), startDate, endDate, testName);
    }

    /**
     * For the date range, makes a call to ORWRP REPORT TEXT "OR_CH:CHEM & HEMATOLOGY~CH;ORDV02;3;999" and scans all
     * returned results for those that match the test name.
     *
     * @param patientIen the patient IEN
     * @param startDate  Start timestamp for range to search
     * @param endDate    End timestamp for range to search
     * @param testName   Name of the Lab Test to search
     * @return Array of LabResults that match testName in the given date range
     * @throws RPCException
     */
    public LabResult[] getPatientLabTestResults(String patientIen, Calendar startDate, Calendar endDate, String testName) throws RPCException {
        LabResult[] results = getVPRLabResults(patientIen, startDate, endDate);

        ArrayList<LabResult> resultArrayList = new ArrayList<LabResult>();

        for (LabResult result : results) {
            if (testName.equals(result.getTestName())) {
                resultArrayList.add(result);
            }
        }

        return resultArrayList.toArray(new LabResult[0]);
    }

    /**
     * Gets a list of Lab Orders using the VPR RPC with "panels" as the type. This will return the results for each
     * order embedded in the LabOrder.
     *
     * @param startDate The start of the date range for the list of orders.
     * @param endDate   The end of the date range for the list of orders.
     * @return The list of lab orders for the specified date range.
     * @throws RPCException
     */
    public LabOrder[] getVPRLabOrders(Calendar startDate, Calendar endDate) throws RPCException {
        return getVPRLabOrders(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Gets a list of Lab Orders using the VPR RPC with "panels" as the type. This will return the results for each
     * order embedded in the LabOrder.
     *
     * @param patientIen The patient data file number
     * @param startDate  The start of the date range for the list of orders.
     * @param endDate    The end of the date range for the list of orders.
     * @return The list of lab orders for the specified date range.
     * @throws RPCException
     */
    public LabOrder[] getVPRLabOrders(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
//        String request = buildGetPatientLabsVPRRequest(patientIen, startDate, endDate);
        String request = VPRUtil.buildVPRRequestWithTypeFilter(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.PANELS, "CH");
        String response = (String) cxn.query(request);

        LabOrder[] labs = parseLabsFromVPR(/*patientIen, */response);
        ArrayList<LabOrder> labsArrayList = new ArrayList<LabOrder>(Arrays.asList(labs));

        request = VPRUtil.buildVPRRequestWithTypeFilter(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.ACCESSIONS, "MI;CY;EM;SP;AU", true);
        response = (String) cxn.query(request);

        LabOrder[] labsAP = parseAccessionsFromVPR(/*patientIen, */response);
        labsArrayList.addAll(Arrays.asList(labsAP));

        return labsArrayList.toArray(new LabOrder[0]);
    }

    /**
     * Builds the VPR RPC request string using "accessions" as the type. The accessions contain the test results. The
     * RPC will fail if the "VPR APPLICATION PROXY" context is not set. That is set when setting up the connection.
     *
     * @param patientIen Patient IEN
     * @param startDate  Start date of range
     * @param endDate    End date of range
     * @return The RPC message string.
     */
//    private String buildGetPatientLabsVPRRequest(String patientIen, Calendar startDate, Calendar endDate) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("VPR GET PATIENT DATA");
//        vq.addParameter(vq.LITERAL, patientIen);
//        /* ORWRP REPORT TEXT (Lab Order Status)
//         *
//         * This RPC is used to retrieve information on lab orders and their status from the Lab Order Entry file (#69).
//         */
////        vq.addParameter(vq.LITERAL, "accessions;orders");
//        vq.addParameter(vq.LITERAL, "panels");
//
//        String startDateString = Timestamp.fromDateTime(startDate);
//        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
//        startDateString = flds[0];
//        String endDateString = Timestamp.fromDateTime(endDate);
//        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
//        endDateString = flds[0];
//        vq.addParameter(vq.LITERAL, startDateString);
//        vq.addParameter(vq.LITERAL, endDateString);
//
//        return vq.buildMessage();
//    }

    /**
     * Parses the response from the "panels" VPR. This is the list of lab orders and the test results associated
     * with those orders.
     *
     * @param response   The response string from the RPC.
     * @return Array of LabOrders, the results are embedded in each LabOrder as an array of LabResults.
     */
//    private LabOrder[] parseLabsFromVPR(String patientIen, String response) {
    private LabOrder[] parseLabsFromVPR(String response) throws RPCException {
        LabOrder[] rtc;
        if ("".equals(response)) {
            rtc = new LabOrder[0];
        } else {
    
            StringBuilder sb = new StringBuilder(Utils.stripInvalidXMLCharacters(response));
            sb.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            ArrayList<LabOrder> labsList = new ArrayList<LabOrder>();
    
            Document doc = null;
            try {
                doc = parseXMLDocument(sb.toString());
            } catch (Exception e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }
    
            NodeList panelNodeList = null;
            try {
                panelNodeList = (NodeList) exprPanel.evaluate(doc, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_LAB_PANELS_BATCH_SIZE);
            int panelNodeListLength = panelNodeList.getLength();
            for (int i = 0; i < panelNodeListLength; i++) {
                parserBatchManager.addNode(panelNodeList.item(i));
            }

            List<VPRLabOrderParser> nodeParserList = new ArrayList<VPRLabOrderParser>();
            for(List<Node> panelNode : parserBatchManager.getParserBatchList()) {
                nodeParserList.add(new VPRLabOrderParser(panelNode));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<LabOrder>>> futures = null;
            try {
                futures = executorService.invokeAll(nodeParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<LabOrder>>> futuresItr = futures.iterator();
    
            while (futuresItr.hasNext()) {
                try {
                    Future<List<LabOrder>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Lab Order Data");
                    } else {
                        labsList.addAll(future.get());
                    }
    
                } catch (InterruptedException e) {
                    logErrorMessage(e);
                } catch (ExecutionException e) {
                    logErrorMessage(e);
                }
            }
    
            rtc = labsList.toArray(new LabOrder[0]);
        }
        return rtc;
    }

    /**
     * Parses the response from the "accessions" VPR. This is the list of lab orders and the test results associated
     * with those orders.
     *
     * @param response   The response string from the RPC.
     * @return Array of LabOrders, the results are embedded in each LabOrder as an array of LabResults.
     */
    private LabOrder[] parseAccessionsFromVPR(String response) throws RPCException {
        LabOrder[] rtc;
        if ("".equals(response)) {
            rtc = new LabOrder[0];
        } else {
    
            StringBuilder sbld = new StringBuilder(Utils.stripInvalidXMLCharacters(response));
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            ArrayList<LabOrder> labsList = new ArrayList<LabOrder>();
    
            Document doc = null;
            try {
                doc = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }
    
            NodeList accessionNodeList = null;
            try {
                accessionNodeList = (NodeList) exprAccession.evaluate(doc, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_LAB_ACCESSIONS_BATCH_SIZE);
            int accessionNodeListLength = accessionNodeList.getLength();
            for (int i = 0; i < accessionNodeListLength; i++) {
                parserBatchManager.addNode(accessionNodeList.item(i));
            }

            List<VPRAccessionParser> nodeParserList = new ArrayList<VPRAccessionParser>();
            for(List<Node> accessionNode : parserBatchManager.getParserBatchList()) {
                nodeParserList.add(new VPRAccessionParser(accessionNode));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<LabOrder>>> futures = null;
            try {
                futures = executorService.invokeAll(nodeParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
    
            } catch (InterruptedException e) {
                logErrorMessage(e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<LabOrder>>> futuresItr = futures.iterator();
    
            while (futuresItr.hasNext()) {
                try {
                    Future<List<LabOrder>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Lab Accession Data");
                    } else {
                        labsList.addAll(future.get());
                    }
    
                } catch (InterruptedException e) {
                    logErrorMessage(e);
                } catch (ExecutionException e) {
                    logErrorMessage(e);
                }
            }
    
            rtc = labsList.toArray(new LabOrder[0]);
        }
        return rtc;
    }


    public FreeTextReport getLabReport(Calendar collectedDate, String accessionNumber) throws RPCException {
        return getLabReport(cxn.getPatientIen(), collectedDate, accessionNumber);
    }

    /**
     * Query vista for lab report.
     */
    public FreeTextReport getLabReport(String patientIen, Calendar collectedDate, String accessionNumber) throws RPCException {

        // There are documents so we'll make an RPC call to get the Report text
        Calendar endDate = (Calendar) collectedDate.clone();
        Calendar startDate = (Calendar) collectedDate.clone();

        // Pad by two days before and after 48 hr * 60 min * 60 sec * 1000 ms
        endDate.setTimeInMillis(collectedDate.getTimeInMillis() + 172800000);
        startDate.setTimeInMillis(collectedDate.getTimeInMillis() - 172800000);

        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWLRR INTERIM");

        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, Timestamp.fromDateTime(endDate));
        vquery.addParameter(VistaQuery.LITERAL, Timestamp.fromDateTime(startDate));
        String reportText = (String) cxn.query(vquery.buildMessage());

        String extractedReport = extractReport(reportText, accessionNumber, collectedDate);

        FreeTextReport report = new FreeTextReport();
        report.setReportText(extractedReport);
        fillCommonDataBeanValues(report);

        return report;
    }

//    private static final String footer = "===============================================================================\r\n \r\n\r\n";

    private String extractReport(String fullReport, String accessionNumber, Calendar collectedDate) {
        int startIndex = 0;
        int endIndex = 0;

        StringBuilder compositeReportsSB = new StringBuilder();
        String report = null;
        String accession = null;

        if (accessionNumber.contains("^")) {
            accession = accessionNumber.substring(0, accessionNumber.indexOf('^'));
        } else {
            accession = accessionNumber;
        }

        int fullReportLength = fullReport.length();
        while (startIndex < fullReportLength) {

            startIndex = endIndex;
            endIndex = fullReport.indexOf(FOOTER, startIndex);
            if (endIndex < 0) {
                // this may be the last report
                endIndex = fullReportLength;
            } else {
                // add in the footer for the report
                endIndex = endIndex + FOOTER.length();
            }

            report = fullReport.substring(startIndex, endIndex);

            if (report.contains(accession)) {
                compositeReportsSB.append(report);
                compositeReportsSB.append("\n\n\n");
            }
        }

        // couldn't find the microbiology report
        if (compositeReportsSB.length() == 0) {
            final String PARSE_DATE_FORMAT = "MMM dd, yyyy HH:mm";
            SimpleDateFormat sdf = new SimpleDateFormat(PARSE_DATE_FORMAT, Locale.getDefault());

            compositeReportsSB.append("No Lab Report on ");
            compositeReportsSB.append(sdf.format(collectedDate.getTime()));
            compositeReportsSB.append(" for ");
            compositeReportsSB.append(accessionNumber);

        }

        return compositeReportsSB.toString();
    }

    private void logErrorMessage(Exception except) {
        LOGGER.error(except.getMessage(), except);
    }

    private class VPRLabOrderParser implements Callable<List<LabOrder>> {
        private List<Node> panelNodeList;

        public VPRLabOrderParser(List<Node> panelNodeList) {
            this.panelNodeList = panelNodeList;
        }

        public List<LabOrder> call() throws RPCException {
            List<LabOrder> labOrders = new ArrayList<LabOrder>();
            for(Node panelNode : panelNodeList) {
                LabOrder labOrder = new LabOrder();
                fillCommonDataBeanValues(labOrder);

                String collected = null;
                String comment = null;
                String groupName = null;
                String specimen = null;
                String status = null;
                String name = null;
                String facilityName = null;
                String facilityCode = null;
                String type = null;

                try {
                    collected = (String) exprCollected.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                String resulted = XPathUtil.evaluateNodeToString(panelNode, exprResulted, LOGGER);
                String labOrderId = XPathUtil.evaluateNodeToString(panelNode, exprId, LOGGER);
                labOrder.setOrderId(labOrderId);

                try {
                    comment = (String) exprComment.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    groupName = (String) exprGroupName.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    specimen = (String) exprSpecimen.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    status = (String) exprStatus.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    type = (String) exprType.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    name = (String) exprOrderName.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    facilityName = (String) exprFacilityName.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    facilityCode = (String) exprFacilityCode.evaluate(panelNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                Calendar collectedDate = null;

                if (!StringUtils.isEmpty(collected)) {
                    collectedDate = Timestamp.toDateTime(collected);
                }

                Calendar resultedDate = null;
                if (!StringUtils.isEmpty(resulted)) {
                    resultedDate = Timestamp.toDateTime(resulted);
                }

                if (StringUtils.isEmpty(name)) {
                    //if name is still empty, use group name
                    name = groupName;
                }
                labOrder.setTestName(name);

                Node providerNode = XPathUtil.evaluateNodeToNode(panelNode, exprProvider, LOGGER);
                if (providerNode != null) {
                    String providerCode = XPathUtil.evaluateNodeToString(providerNode, exprProviderCode, LOGGER);
                    String providerName = XPathUtil.evaluateNodeToString(providerNode, exprProviderName, LOGGER);
                    String providerTaxonomyCode = XPathUtil.evaluateNodeToString(providerNode, exprProviderTaxonomyCode, LOGGER);
                    String providerType = XPathUtil.evaluateNodeToString(providerNode, exprProviderProviderType, LOGGER);
                    String providerClassification = XPathUtil.evaluateNodeToString(providerNode, exprProviderClassification, LOGGER);
                    String providerSpecialization = XPathUtil.evaluateNodeToString(providerNode, exprProviderSpecialization, LOGGER);
                    String providerService = XPathUtil.evaluateNodeToString(providerNode, exprProviderService, LOGGER);
                    String providerOfficePhone = XPathUtil.evaluateNodeToString(providerNode, exprProviderOfficePhone, LOGGER);
                    String providerAnalogPager = XPathUtil.evaluateNodeToString(providerNode, exprProviderAnalogPager, LOGGER);
                    String providerFax = XPathUtil.evaluateNodeToString(providerNode, exprProviderFax, LOGGER);
                    String providerEmail = XPathUtil.evaluateNodeToString(providerNode, exprProviderEmail, LOGGER);

                    Provider provider = new Provider();
                    provider.setCode(providerCode);
                    provider.setName(providerName);
                    provider.setTaxonomyCode(providerTaxonomyCode);
                    provider.setProviderType(providerType);
                    provider.setClassification(providerClassification);
                    provider.setSpecialization(providerSpecialization);
                    provider.setService(providerService);
                    provider.setPhone(providerOfficePhone);
                    provider.setAnalogPager(providerAnalogPager);
                    provider.setFax(providerFax);
                    provider.setEmail(providerEmail);

                    labOrder.setRequestingProvider(provider);
                }

                Node resultNode = null;
                NodeList resultsNodeList = null;

                try {
                    resultNode = (Node) exprValues.evaluate(panelNode, XPathConstants.NODE);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                if (resultNode != null) {
                    try {
                        resultsNodeList = (NodeList) exprTestValue.evaluate(resultNode, XPathConstants.NODESET);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                }

                ArrayList<LabResult> resultsList = new ArrayList<LabResult>();


                if (resultsNodeList != null && "CH".equalsIgnoreCase(type)) {
                    int listLength = resultsNodeList.getLength();
                    for (int j = 0; j < listLength; j++) {
                        Node testNode = resultsNodeList.item(j);
                        testNode.getParentNode().removeChild(testNode);

                        LabResult labResult = new LabResult();
                        fillCommonDataBeanValues(labResult);

                        labResult.setProvider(labOrder.getRequestingProvider());

                        String testId = null;
                        String testName = null;
                        String testResult = null;
                        String testUnits = null;
                        String testInterpretation = null;
                        String testHigh = null;
                        String testLow = null;
                        String testLoinc = null;
                        String testVuid = null;

                        try {
                            testId = (String) exprTestId.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testName = (String) exprTestTest.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testResult = (String) exprTestResult.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testUnits = (String) exprTestUnits.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testInterpretation = (String) exprTestInterpretation.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testHigh = (String) exprTestHigh.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testLow = (String) exprTestLow.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testLoinc = (String) exprTestLoinc.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testVuid = (String) exprTestVuid.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }

                        String testPerformingLab = XPathUtil.evaluateNodeToString(testNode, exprTestPerformingLab, LOGGER);

                        labResult.setTestId(testId);
                        labResult.setTestName(testName);
                        labResult.setResult(testResult);
                        labResult.setUnits(testUnits);
                        labResult.setInterpretation(testInterpretation);
                        labResult.setReferenceRange(testLow + " - " + testHigh);
                        List<Code> codeList = new ArrayList<Code>();
                        if (!StringUtils.isEmpty(testLoinc)) {
                            Code loincCode = new Code();
                            loincCode.setSystem("LOINC");
                            loincCode.setCode(testLoinc);
                            codeList.add(loincCode);
                        }
                        if (!StringUtils.isEmpty(testVuid)) {
                            Code vuidCode = new Code();
                            vuidCode.setSystem("VUID");
                            vuidCode.setCode(testVuid);
                            codeList.add(vuidCode);
                        }
                        labResult.setCodes(codeList.toArray(new Code[0]));
                        labResult.setPerformingLabName(testPerformingLab);

                        // Set the fields that match the order
                        labResult.setCollectedDate(collectedDate);
                        labResult.setResultDate(resultedDate);
                        labResult.setComment(comment);
                        labResult.setAccession(groupName);
                        labResult.setSpecimen(specimen);
                        labResult.setResultStatus(status);
                        labResult.setFacilityName(facilityName);
                        labResult.setStationNumber(facilityCode);
                        labResult.setOrderId(labOrder.getOrderId());
                        labResult.setSubscript(type);

                        resultsList.add(labResult);
                    }
                } else {
                    // These are not CH labs so we just need one result
                    LabResult labResult = new LabResult();
                    fillCommonDataBeanValues(labResult);

                    labResult.setProvider(labOrder.getRequestingProvider());

                    // Set the fields that match the order
                    labResult.setResultDate(resultedDate);
                    labResult.setCollectedDate(collectedDate);
                    labResult.setComment(comment);
                    labResult.setAccession(groupName);
                    labResult.setSpecimen(specimen);
                    labResult.setResultStatus(status);
                    labResult.setFacilityName(facilityName);
                    labResult.setStationNumber(facilityCode);
                    labResult.setTestName(labOrder.getTestName());
                    labResult.setSubscript(type);

                    resultsList.add(labResult);
                }

                if ("ACCESSION".equalsIgnoreCase(name)) {
                    // if the order name is "ACCESSION", change it to the first test name from the resultsList
                    StringBuilder labOrderNameSB = new StringBuilder();
                    for (LabResult labResult : resultsList) {
                        labOrderNameSB.append("; ");
                        labOrderNameSB.append(labResult.getTestName());
                    }
                    labOrder.setTestName(labOrderNameSB.substring(2));
                }

                labOrder.setCollectionDate(collectedDate);
                labOrder.setAvailableDate(resultedDate);
                labOrder.setComment(comment);
                labOrder.setAccessionNumber(groupName);
                labOrder.setCollectionSample(specimen);
                labOrder.setStatus(status);
                labOrder.setSubscript(type);
                labOrder.setFacilityName(facilityName);
                labOrder.setStationNumber(facilityCode);

                labOrder.setLabResults(resultsList.toArray(new LabResult[0]));

                labOrders.add(labOrder);
            }

            return labOrders;
        }
    }

    private class VPRAccessionParser implements Callable<List<LabOrder>> {
        private List<Node> accessionNodeList;

        public VPRAccessionParser(List<Node> accessionNodeList) {
            this.accessionNodeList = accessionNodeList;
        }

        public List<LabOrder> call() throws RPCException {
            List<LabOrder> accessions = new ArrayList<LabOrder>();
            for(Node accessionNode : accessionNodeList) {
                LabOrder labOrder = new LabOrder();
                fillCommonDataBeanValues(labOrder);

                String collected = null;
                String comment = null;
                String groupName = null;
                String specimen = null;
                String status = null;
                String facilityName = null;
                String facilityCode = null;
                String type = null;

                try {
                    collected = (String) exprCollected.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                String resulted = XPathUtil.evaluateNodeToString(accessionNode, exprResulted, LOGGER);
                String labOrderId = XPathUtil.evaluateNodeToString(accessionNode, exprId, LOGGER);

                try {
                    comment = (String) exprComment.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    groupName = (String) exprGroupName.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    specimen = (String) exprSpecimen.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    status = (String) exprStatus.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    type = (String) exprType.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                String name = XPathUtil.evaluateNodeToString(accessionNode, exprName, LOGGER);
                Node documentsNode = XPathUtil.evaluateNodeToNode(accessionNode, exprDocuments, LOGGER);
                String documentContent = null;
                if (documentsNode != null) {
                    NodeList documentNodeList = XPathUtil.evaluateNodeToNodeList(documentsNode, exprDocument, LOGGER);
                    if (documentNodeList != null) {
                        int documentNodeListLength = documentNodeList.getLength();
                        StringBuilder documentContentSB = new StringBuilder();
                        for (int documentNum = 0; documentNum < documentNodeListLength; documentNum++) {
                            Node documentNode = documentNodeList.item(documentNum);
                            if (StringUtils.isEmpty(name)) {
                                //if name not present, fall back on document local title
                                name = XPathUtil.evaluateNodeToString(documentNode, exprDocumentLocalTitle, LOGGER);
                            }
                            // concat all document content
                            String content = XPathUtil.evaluateNodeToString(documentNode, exprDocumentContent, LOGGER);
                            documentContentSB.append(content);
                            documentContentSB.append("\n\n");
                        }
                        if (documentContentSB.length() > 0) {
                            documentContent = documentContentSB.toString();
                        }
                    }
                }

                try {
                    facilityName = (String) exprFacilityName.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                try {
                    facilityCode = (String) exprFacilityCode.evaluate(accessionNode, XPathConstants.STRING);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                Calendar collectedDate = null;

                if (!StringUtils.isEmpty(collected)) {
                    collectedDate = Timestamp.toDateTime(collected);
                }

                Calendar resultedDate = null;
                if (!StringUtils.isEmpty(resulted)) {
                    resultedDate = Timestamp.toDateTime(resulted);
                }

                labOrder.setTestName(name);

                Node providerNode = XPathUtil.evaluateNodeToNode(accessionNode, exprProvider, LOGGER);
                if (providerNode != null) {
                    String providerCode = XPathUtil.evaluateNodeToString(providerNode, exprProviderCode, LOGGER);
                    String providerName = XPathUtil.evaluateNodeToString(providerNode, exprProviderName, LOGGER);
                    String providerTaxonomyCode = XPathUtil.evaluateNodeToString(providerNode, exprProviderTaxonomyCode, LOGGER);
                    String providerType = XPathUtil.evaluateNodeToString(providerNode, exprProviderProviderType, LOGGER);
                    String providerClassification = XPathUtil.evaluateNodeToString(providerNode, exprProviderClassification, LOGGER);
                    String providerSpecialization = XPathUtil.evaluateNodeToString(providerNode, exprProviderSpecialization, LOGGER);
                    String providerService = XPathUtil.evaluateNodeToString(providerNode, exprProviderService, LOGGER);
                    String providerOfficePhone = XPathUtil.evaluateNodeToString(providerNode, exprProviderOfficePhone, LOGGER);
                    String providerAnalogPager = XPathUtil.evaluateNodeToString(providerNode, exprProviderAnalogPager, LOGGER);
                    String providerFax = XPathUtil.evaluateNodeToString(providerNode, exprProviderFax, LOGGER);
                    String providerEmail = XPathUtil.evaluateNodeToString(providerNode, exprProviderEmail, LOGGER);

                    Provider provider = new Provider();
                    provider.setCode(providerCode);
                    provider.setName(providerName);
                    provider.setTaxonomyCode(providerTaxonomyCode);
                    provider.setProviderType(providerType);
                    provider.setClassification(providerClassification);
                    provider.setSpecialization(providerSpecialization);
                    provider.setService(providerService);
                    provider.setPhone(providerOfficePhone);
                    provider.setAnalogPager(providerAnalogPager);
                    provider.setFax(providerFax);
                    provider.setEmail(providerEmail);

                    labOrder.setRequestingProvider(provider);
                }

                Node resultNode = null;
                NodeList resultsNodeList = null;

                try {
                    resultNode = (Node) exprValues.evaluate(accessionNode, XPathConstants.NODE);
                } catch (XPathExpressionException e) {
                    logErrorMessage(e);
                }

                if (resultNode != null) {
                    try {
                        resultsNodeList = (NodeList) exprTestValue.evaluate(resultNode, XPathConstants.NODESET);
                    } catch (XPathExpressionException e) {
                        logErrorMessage(e);
                    }
                }

                ArrayList<LabResult> resultsList = new ArrayList<LabResult>();


                if (resultsNodeList != null && "CH".equalsIgnoreCase(type)) {
                    int listLength = resultsNodeList.getLength();
                    for (int j = 0; j < listLength; j++) {
                        Node testNode = resultsNodeList.item(j);
                        testNode.getParentNode().removeChild(testNode);

                        LabResult labResult = new LabResult();
                        fillCommonDataBeanValues(labResult);

                        labResult.setProvider(labOrder.getRequestingProvider());

                        String testId = null;
                        String testName = null;
                        String testResult = null;
                        String testUnits = null;
                        String testInterpretation = null;
                        String testHigh = null;
                        String testLow = null;
                        String testLoinc = null;
                        String testVuid = null;

                        try {
                            testId = (String) exprTestId.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testName = (String) exprTestTest.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testResult = (String) exprTestResult.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testUnits = (String) exprTestUnits.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testInterpretation = (String) exprTestInterpretation.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testHigh = (String) exprTestHigh.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testLow = (String) exprTestLow.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testLoinc = (String) exprTestLoinc.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        try {
                            testVuid = (String) exprTestVuid.evaluate(testNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            logErrorMessage(e);
                        }
                        String testPerformingLab = XPathUtil.evaluateNodeToString(testNode, exprTestPerformingLab, LOGGER);

                        labResult.setTestId(testId);
                        labResult.setTestName(testName);
                        labResult.setResult(testResult);
                        labResult.setUnits(testUnits);
                        labResult.setInterpretation(testInterpretation);
                        labResult.setReferenceRange(testLow + " - " + testHigh);
                        List<Code> codeList = new ArrayList<Code>();
                        if (!StringUtils.isEmpty(testLoinc)) {
                            Code loincCode = new Code();
                            loincCode.setSystem("LOINC");
                            loincCode.setCode(testLoinc);
                            codeList.add(loincCode);
                        }
                        if (!StringUtils.isEmpty(testVuid)) {
                            Code vuidCode = new Code();
                            vuidCode.setSystem("VUID");
                            vuidCode.setCode(testVuid);
                            codeList.add(vuidCode);
                        }
                        labResult.setCodes(codeList.toArray(new Code[0]));
                        labResult.setPerformingLabName(testPerformingLab);

                        // Set the fields that match the order
                        labResult.setCollectedDate(collectedDate);
                        labResult.setResultDate(resultedDate);
                        labResult.setComment(comment);
                        labResult.setAccession(groupName);
                        labResult.setSpecimen(specimen);
                        labResult.setResultStatus(status);
                        labResult.setFacilityName(facilityName);
                        labResult.setStationNumber(facilityCode);
                        labResult.setSubscript(type);

                        resultsList.add(labResult);
                    }
                } else {
                    // These are not CH labs so we just need one result
                    LabResult labResult = new LabResult();
                    fillCommonDataBeanValues(labResult);

                    labResult.setResult(documentContent);
                    labResult.setProvider(labOrder.getRequestingProvider());

                    // Set the fields that match the order
                    labResult.setResultDate(resultedDate);
                    labResult.setCollectedDate(collectedDate);
                    labResult.setComment(comment);
                    labResult.setAccession(groupName);
                    labResult.setSpecimen(specimen);
                    labResult.setResultStatus(status);
                    labResult.setFacilityName(facilityName);
                    labResult.setStationNumber(facilityCode);
                    labResult.setTestName(labOrder.getTestName());
                    labResult.setSubscript(type);

                    resultsList.add(labResult);
                }

                labOrder.setCollectionDate(collectedDate);
                labOrder.setAvailableDate(resultedDate);
                labOrder.setComment(comment);
                labOrder.setAccessionNumber(groupName);
                labOrder.setCollectionSample(specimen);
                labOrder.setStatus(status);
                labOrder.setSubscript(type);
                labOrder.setFacilityName(facilityName);
                labOrder.setStationNumber(facilityCode);
                labOrder.setOrderId(labOrderId);

                labOrder.setLabResults(resultsList.toArray(new LabResult[0]));

                accessions.add(labOrder);
            }

            return accessions;
        }
    }

    /**
     * Gets a list of Lab Orders using the VPR RPC. This will return the results for each
     * order embedded in the LabOrder.
     *
     * @param startDate The start of the date range for the list of orders.
     * @param endDate   The end of the date range for the list of orders.
     * @return The list of lab orders for the specified date range.
     * @throws RPCException
     */
    public LabResult[] getVPRLabResults(Calendar startDate, Calendar endDate) throws RPCException {
        return getVPRLabResults(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Gets a list of Lab Orders using the VPR RPC. This will return the results for each
     * order embedded in the LabOrder.
     *
     * @param patientIen The patient data file number
     * @param startDate  The start of the date range for the list of orders.
     * @param endDate    The end of the date range for the list of orders.
     * @return The list of lab orders for the specified date range.
     * @throws RPCException
     */
    public LabResult[] getVPRLabResults(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        LabOrder[] labOrders = getVPRLabOrders(patientIen, startDate, endDate);

        ArrayList<LabResult> labResultArrayList = new ArrayList<LabResult>();

        for (LabOrder labOrder: labOrders) {
            labResultArrayList.addAll(Arrays.asList(labOrder.getLabResults()));
        }

        return labResultArrayList.toArray(new LabResult[0]);
    }

    public LabResult[] getPatientLabResultsAutopsy(Calendar startDate, Calendar endDate) throws  RPCException {
        return getPatientLabResultsAutopsy(cxn.getPatientIen(), startDate, endDate);
    }

    private final String AUTOPSY_STRING = "---- AUTOPSY ----";
    private final String AUTOPSY_TABLE_HEADER_STRING = "Acc #   Date/time Died    Age   AUTOPSY DATA        Date/time of Autopsy";
    private static final String MMM_DD_YYYY_REGEX_GROUP = "([A-Z][a-z][a-z] (0[1-9]|[12][0-9]|3[01]), (19|20)\\d\\d [012]\\d:\\d\\d)";

    public LabResult[] getPatientLabResultsAutopsy(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildAutopsyReportRequest(patientIen);
        String response = (String) cxn.query(request);
        ArrayList<LabResult> rtc = new ArrayList<LabResult>();

        if (StringUtils.isEmpty(response) || !response.contains(AUTOPSY_STRING)) {
            return rtc.toArray(new LabResult[0]);
        }

        LabResult autopsy = parseAutopsyReportToLabResult(response);


        // check that the autopsy is in the date range
        if (autopsy.getCollectedDate().compareTo(startDate) >= 0 && autopsy.getCollectedDate().compareTo(endDate) <=0) {
            rtc.add(autopsy);
        }

        return rtc.toArray(new LabResult[0]);
    }

    private LabResult parseAutopsyReportToLabResult(String autopsyReport) {
        // parse out the autopsy date and the accession number
        Pattern dateGroupPattern = Pattern.compile(MMM_DD_YYYY_REGEX_GROUP);
        Matcher dateGroupMatcher = dateGroupPattern.matcher(autopsyReport);

        // assume always has a died date and an autopsy date
        dateGroupMatcher.find();
        int startIndexOfDiedDate = dateGroupMatcher.start();
        int endIndexOfDiedDate = dateGroupMatcher.end();
//        String diedDateString = dateGroupMatcher.group();

        // reset matcher to find next date group after the died date
        dateGroupMatcher.find(endIndexOfDiedDate);
        String autopsyDateString = dateGroupMatcher.group();

        // the accession number is from the end of the header to the start of the died date
        int startOfTableIndex = autopsyReport.indexOf(AUTOPSY_TABLE_HEADER_STRING) + AUTOPSY_TABLE_HEADER_STRING.length();
        String accessionNumber = autopsyReport.substring(startOfTableIndex, startIndexOfDiedDate).trim();

        LabResult autopsy = new LabResult();
        fillCommonDataBeanValues(autopsy);
        autopsy.setTestName("AUTOPSY");
        autopsy.setResult(autopsyReport);
        autopsy.setAccession(accessionNumber);
        autopsy.setSubscript("AU");

        SimpleDateFormat sdtf = new SimpleDateFormat("MMM dd, yyyy HH:mm", Locale.getDefault());

//        Date diedDate = null;
//        try {
//            diedDate = sdtf.parse(diedDateString);
//        } catch (ParseException pe) {
//            LOGGER.error("Error parsing autopsy died date: " + pe.getMessage());
//        }
//        if (diedDate != null) {
//            Calendar diedCalendar = Calendar.getInstance();
//            diedCalendar.setTime(diedDate);
//            autopsy.setCollectedDate(diedCalendar);
//        }

        Date autopsyDate = null;
        try {
            autopsyDate = sdtf.parse(autopsyDateString);
        } catch (ParseException pe) {
            LOGGER.error("Error parsing autopsy date: " + pe.getMessage());
        }
        if (autopsyDate != null) {
            Calendar autopsyCalendar = Calendar.getInstance();
            autopsyCalendar.setTime(autopsyDate);
            autopsy.setCollectedDate(autopsyCalendar);
            autopsy.setOrderDate(autopsyCalendar);
            autopsy.setResultDate(autopsyCalendar);
        }

        return autopsy;
    }


    private String buildAutopsyReportRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, "OR_AU:AUTOPSY~;;0");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "0");
        vquery.addParameter(VistaQuery.LITERAL, "0");

        return vquery.buildMessage();
    }

    public LabOrder[] getPatientLabsAutopsy(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientLabsAutopsy(cxn.getPatientIen(), startDate, endDate);
    }

    public LabOrder[] getPatientLabsAutopsy(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        LabResult[] autopsyList = getPatientLabResultsAutopsy(patientIen, startDate, endDate);
        ArrayList<LabOrder> rtc = new ArrayList<LabOrder>();

        if (autopsyList == null || autopsyList.length < 1) {
            return rtc.toArray(new LabOrder[0]);
        }

        // assume only one autopsy report
        LabOrder autopsyLab = new LabOrder();
        fillCommonDataBeanValues(autopsyLab);
        autopsyLab.setCollectionDate(autopsyList[0].getCollectedDate());
        autopsyLab.setOrderDate(autopsyList[0].getOrderDate());
        autopsyLab.setAvailableDate(autopsyList[0].getResultDate());
        autopsyLab.setSubscript(autopsyList[0].getSubscript());
        autopsyLab.setAccessionNumber(autopsyList[0].getAccession());
        autopsyLab.setLabResults(autopsyList);
        autopsyLab.setTestName(autopsyList[0].getTestName());

        rtc.add(autopsyLab);

        return rtc.toArray(new LabOrder[0]);
    }

    public LabResult[] getPatientLabResultsSkinTests(Calendar startDate, Calendar endDate) throws  RPCException {
        return getPatientLabResultsSkinTests(cxn.getPatientIen(), startDate, endDate);
    }

    public LabResult[] getPatientLabResultsSkinTests(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        ArrayList<LabResult> rtc = new ArrayList<LabResult>();

        PCERPC pcerpc = new PCERPC(cxn);
        PCE[] skinTests = pcerpc.getSkinTests(patientIen, startDate, endDate);

        // map each skinTest into a LabResult
        for (PCE skinTest : skinTests) {
            LabResult labResult = new LabResult();
            fillCommonDataBeanValues(labResult);

            labResult.setCollectedDate(skinTest.getEnteredDate());
            labResult.setTestName(skinTest.getName());
//            labResult.setResultStatus(skinTest.getReading());
            StringBuilder resultSB = new StringBuilder();
            resultSB.append(skinTest.getResult());
            resultSB.append("\n");
            resultSB.append(skinTest.getComment()); // comment should be the report text, see PCERPC.java
            labResult.setResult(resultSB.toString()); // set the LabResult result to the StringBuilder contents
            labResult.setAccession("SKIN TEST");
            labResult.setTestType("SKIN TEST");
            labResult.setSubscript("ST");

            rtc.add(labResult);
        }

        return rtc.toArray(new LabResult[0]);
    }

    public LabResult[] getPatientLabResultsBloodTypeTests(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientLabResultsBloodTypeTests(cxn.getPatientIen(), cxn.getPatientIcn(), startDate, endDate);
    }

    public LabResult[] getPatientLabResultsBloodTypeTests(String patientIen, String patientIcn, Calendar startDate, Calendar endDate) throws RPCException {
        ArrayList<LabResult> rtc = getBloodTypeTests(patientIen, patientIcn, startDate, endDate);
        return rtc.toArray(new LabResult[0]);
    }

    private ArrayList<LabResult> getBloodTypeTests(String patientIen, String patientIcn, Calendar startDate, Calendar endDate) throws RPCException {
        VistaQuery vistaQuery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vistaQuery.addParameter(VistaQuery.LITERAL, patientIen);
        vistaQuery.addParameter(VistaQuery.LITERAL, "2:BLOOD BANK REPORT~;;0");
        vistaQuery.addParameter(VistaQuery.LITERAL, "");
        vistaQuery.addParameter(VistaQuery.LITERAL, "50000");
        vistaQuery.addParameter(VistaQuery.LITERAL, "");
        vistaQuery.addParameter(VistaQuery.LITERAL, "0");
        vistaQuery.addParameter(VistaQuery.LITERAL, "0");
        String request = vistaQuery.buildMessage();
        String response = (String) cxn.query(request);

        // if the report is No blood bank report available, then try with the icn
        if (response.indexOf("No Blood Bank report available...") > 0) {
            vistaQuery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
            vistaQuery.addParameter(VistaQuery.LITERAL, patientIcn);
            vistaQuery.addParameter(VistaQuery.LITERAL, "2:BLOOD BANK REPORT~;;0");
            vistaQuery.addParameter(VistaQuery.LITERAL, "");
            vistaQuery.addParameter(VistaQuery.LITERAL, "50000");
            vistaQuery.addParameter(VistaQuery.LITERAL, "");
            vistaQuery.addParameter(VistaQuery.LITERAL, "0");
            vistaQuery.addParameter(VistaQuery.LITERAL, "0");
            request = vistaQuery.buildMessage();
            response = (String) cxn.query(request);
        }

        // get the list of transfusions, pass in the BB report to be added to each transfusion
        ArrayList<LabResult> bloodTypeTestsList = parseBloodbankReport(response);
        ArrayList<LabResult> rtc = new ArrayList<LabResult>();

        // filter out for date range
        for (LabResult labResult : bloodTypeTestsList) {
            if (labResult.getCollectedDate() != null) {
                Calendar collectedDate = labResult.getCollectedDate();
                if (collectedDate.compareTo(startDate) >= 0 && collectedDate.compareTo(endDate) <=0) {
                    rtc.add(labResult);
                }
            } else {
                // the parsing failed so there is a generic bloodbank lab report, so add it then break
                rtc.add(labResult);
                break;
            }
        }

        return rtc;
    }

    private LabResult fillLabResult(String tranfusionName, Date resultDate) {
        LabResult rtc = new LabResult();
        fillCommonDataBeanValues(rtc);
        Calendar resultCalendar = Calendar.getInstance();
        resultCalendar.setTime(resultDate);
//        rtc.setResultDate(resultCalendar);
        rtc.setCollectedDate(resultCalendar);
        rtc.setTestName(tranfusionName);
        rtc.setSubscript("BB");
        rtc.setAccession("BLOOD BANK");

        return rtc;
    }

    private static final String LEGACY_REPORT_HEADER = "*** [LEGACY VISTA BLOOD BANK REPORT] ***";
    private static final String START_OF_BB_TEST_TABLE_LEGACY = "---------       --- --    ---- ---  ---  --------------   -----------------";
    private static final String DIAGNOSTIC_TESTS_MARKER = "DIAGNOSTIC TESTS:";
    private static final String START_OF_TRANSFUSED_TABLE = "Transfused Units";
    private static final String MMM_DD_YYYY_REGEX = ".*[A-Z][a-z][a-z] (0[1-9]|[12][0-9]|3[01]), (19|20)\\d\\d.*";
    private static final String BLOOD_TYPE_TESTING_NAME = "Blood Type Testing";
    private static final String BLOOD_PRODUCTS_GIVEN_NAME = "Blood Products Given";
    private static final String COMMENT_START_LABEL = "Comment";
    private static final String DATE_FORMAT = "MMM dd, yyyy";
    private static final String DATE_TIME_FORMAT_NO_SECONDS = "MM/dd/yyyy@HH:mm";
    private static final String DATE_TIME_FORMAT_WITH_SECONDS = "MM/dd/yyyy@HH:mm:ss";

    private ArrayList<LabResult> parseBloodbankReport(String bloodbankReport) {
        ArrayList<LabResult> rtc = new ArrayList<LabResult>();

        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT, Locale.getDefault());
        SimpleDateFormat sdtf = new SimpleDateFormat(DATE_TIME_FORMAT_NO_SECONDS, Locale.getDefault());
        SimpleDateFormat sdtfWithSeconds = new SimpleDateFormat(DATE_TIME_FORMAT_WITH_SECONDS, Locale.getDefault());

        String nextLine;
        boolean lookingForTransfusions = false;

        int legacyReportIndex = bloodbankReport.indexOf(LEGACY_REPORT_HEADER);

        // find the start of the diagnostic tests table
        int tableMarkerIndex = bloodbankReport.indexOf(DIAGNOSTIC_TESTS_MARKER);
        // if no blood bank report return empty list
        if (legacyReportIndex == -1 && tableMarkerIndex == -1) {
            return rtc;
        }

        try {
            // check to use production table or legacy table
            if (tableMarkerIndex == -1) {
                // there is no production table so find the legacy table
                tableMarkerIndex = bloodbankReport.indexOf(START_OF_BB_TEST_TABLE_LEGACY);
                if (tableMarkerIndex == -1) {
                    // there is no LEGACY blood bank test table so we don't need to parse anything
                    return  rtc;
                }
            } else {
                // this is the production table, move tableMarkerIndex to dashed line (advance 4 lines)
                for (int i=0; i < 4; i++) {
                    tableMarkerIndex = bloodbankReport.indexOf("\n", tableMarkerIndex) + 1;
                }
            }

            // go to start of blood test table
            int beginningOfLineIndex = bloodbankReport.indexOf("\n", tableMarkerIndex) + 1;
            // get the transfusion table if it exists
            String transfusionTable = "";
            int transfusedTableIndex = bloodbankReport.indexOf(START_OF_TRANSFUSED_TABLE);
            if (transfusedTableIndex > tableMarkerIndex) {
                StringBuilder transfusionTableSB = new StringBuilder(bloodbankReport.substring(transfusedTableIndex));
                transfusionTableSB.append("\n");
                transfusionTable = transfusionTableSB.toString();
            }

            LabResult previousLab = null;
            // each line should start with a date in the form of MM/DD/YYYY@HH:MM (length 16 ch) or MM/DD/YYYY@HH:MM:SS (19 ch), colon at index 16 means there are seconds)
            // get the next line
            int endOfLineIndex = bloodbankReport.indexOf("\n", beginningOfLineIndex);
            while (endOfLineIndex > -1) { // check if not at end of table (empty line)
                nextLine = bloodbankReport.substring(beginningOfLineIndex, endOfLineIndex + 1).trim();
                if (nextLine.startsWith("(")) {
                    int indexOfEndParens = nextLine.indexOf(")");
                    nextLine = nextLine.substring(indexOfEndParens + 1).trim();
                }

                if (StringUtils.isEmpty(nextLine)) {
                    break;
                }
                if (nextLine.contains(COMMENT_START_LABEL)) {
                    beginningOfLineIndex = endOfLineIndex + 1;
                    endOfLineIndex = bloodbankReport.indexOf("\n", beginningOfLineIndex);
                    if (previousLab != null) {
                        StringBuilder previousLabCommentSB = new StringBuilder();
                        if (!StringUtils.isEmpty(previousLab.getComment())) {
                            previousLabCommentSB.append(previousLab.getComment());
                        }
                        nextLine = nextLine.substring(COMMENT_START_LABEL.length()).trim();
                        previousLabCommentSB.append(nextLine);
                        previousLabCommentSB.append("\n");
                        previousLab.setComment(previousLabCommentSB.toString());
                    }
                    continue;
                }
                SimpleDateFormat currentSdf = sdtf;
                Date testDate = null;
                int lengthOfDateTime = DATE_TIME_FORMAT_NO_SECONDS.length(); // length of dateTimeFormat, no seconds
                // check if there are seconds
                if (':' == nextLine.charAt(DATE_TIME_FORMAT_NO_SECONDS.length())) {
                    lengthOfDateTime = DATE_TIME_FORMAT_WITH_SECONDS.length(); // length of dateTimeFormat with seconds
                    currentSdf = sdtfWithSeconds;
                }
                String dateTimeString = nextLine.substring(0, lengthOfDateTime);
                try {
                    testDate = currentSdf.parse(dateTimeString);
                } catch (ParseException pe) {
                    LOGGER.error("Error parsing bloodbank test date: " + pe.getMessage());
                    throw pe;
                }
                if (testDate != null) {
                    LabResult labResult = fillLabResult(BLOOD_TYPE_TESTING_NAME, testDate);
                    previousLab = labResult;
                    labResult.setResult(bloodbankReport);
                    rtc.add(labResult);
                }

                beginningOfLineIndex = endOfLineIndex + 1;
                endOfLineIndex = bloodbankReport.indexOf("\n", beginningOfLineIndex);
            }

            // only do this if there were type tests
            if (rtc.size() > 0) {
                // look for transfusions
                lookingForTransfusions = true;
                // first check if there are dates in the transfusion table
                Pattern monthDayYearRegexPattern = Pattern.compile(MMM_DD_YYYY_REGEX, Pattern.DOTALL);
                Matcher monthDayYearMatcher = monthDayYearRegexPattern.matcher(transfusionTable);
                if (monthDayYearMatcher.matches()) {
                    // in the case that there is only one test type, we can assume that all transfusions relate to that test
                    if (rtc.size() == 1) {
                        rtc.get(0).setTestName(BLOOD_PRODUCTS_GIVEN_NAME);
                    } else {
                        // Sort the rtc by date
                        Collections.sort(rtc, new LabResultDateComparator());

                        // there should be two or more type tests since rtc.size > 1 and rtc.size != 1 from this point on
                        ArrayList<Calendar> transfusionDatesList = new ArrayList<Calendar>();
                        beginningOfLineIndex = transfusionTable.indexOf("\n") + 1;
                        endOfLineIndex = transfusionTable.indexOf("\n", beginningOfLineIndex);
                        nextLine = transfusionTable.substring(beginningOfLineIndex, endOfLineIndex + 1).trim();
                        monthDayYearMatcher = monthDayYearRegexPattern.matcher(nextLine);
                        // collect the transfusion dates line by line until the next line in the transfusion table does not contain a date.
                        while (monthDayYearMatcher.matches()) {
                            String transfusionDateString = nextLine.substring(0, DATE_FORMAT.length());
                            Date transfusionDate = null;
                            try {
                                transfusionDate = sdf.parse(transfusionDateString);
                            } catch (ParseException pe) {
                                LOGGER.error("Error parsing bloodbank transfusion date: " + pe.getMessage());
                                throw pe;
                            }
                            if (transfusionDate != null) {
                                Calendar transfusionCalendar = Calendar.getInstance();
                                transfusionCalendar.setTime(transfusionDate);
                                transfusionDatesList.add(transfusionCalendar);
                            }

                            beginningOfLineIndex = endOfLineIndex + 1;
                            endOfLineIndex = transfusionTable.indexOf("\n", beginningOfLineIndex);
                            nextLine = transfusionTable.substring(beginningOfLineIndex, endOfLineIndex + 1).trim();
                            monthDayYearMatcher = monthDayYearRegexPattern.matcher(nextLine);
                        }

                        // now we can see which transfusions belong to which type test to change the name
                        // first build a "histogram" using the testDates. For the last bin the end date will be today.
                        ArrayList<TestDateBin> testDateHistogram = new ArrayList<TestDateBin>();
                        for (int typeTestIndex = 0; typeTestIndex < rtc.size(); typeTestIndex++) {
                            Calendar startDate = Calendar.getInstance();
                            Calendar endDate = Calendar.getInstance();

                            // first we need calendar dates without the time for when we do the comparison
                            String startDateString = sdf.format(rtc.get(typeTestIndex).getCollectedDate().getTime());
                            try {
                                startDate.setTime(sdf.parse(startDateString));
                            } catch (ParseException pe) {
                                LOGGER.error("Error parsing blood type start test date for finding transfusion: " + pe.getMessage());
                                throw pe;
                            }

                            if (typeTestIndex + 1 < rtc.size()) {
                                String endDateString = sdf.format(rtc.get(typeTestIndex + 1).getCollectedDate().getTime());
                                try {
                                    endDate.setTime(sdf.parse(endDateString));
                                } catch (ParseException pe) {
                                    LOGGER.error("Error parsing blood type end test date for finding transfusion: " + pe.getMessage());
                                    throw pe;
                                }
                            }

                            testDateHistogram.add(new TestDateBin(startDate, endDate));
                        }

                        // now go through each type test and bin it. If it is in the bin, change the name of the type test
                        for (int histogramIndex = 0; histogramIndex < testDateHistogram.size(); histogramIndex++) {
                            for (Calendar transfusionDate : transfusionDatesList) {
                                if (testDateHistogram.get(histogramIndex).isInBin(transfusionDate)) {
                                    rtc.get(histogramIndex).setTestName(BLOOD_PRODUCTS_GIVEN_NAME);
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("Bloodbank parsing error: " + e.getMessage());
            if (!lookingForTransfusions){
                LabResult labReportResult = new LabResult();
                fillCommonDataBeanValues(labReportResult);
                labReportResult.setTestName("BLOOD BANK REPORT");
                labReportResult.setResult(bloodbankReport);
                labReportResult.setSubscript("BB");
                labReportResult.setAccession("BLOOD BANK");
                rtc.add(labReportResult);
            }
        }
        return rtc;
    }

    private class TestDateBin {

        private Calendar startDate;
        private Calendar endDate;

        public TestDateBin(Calendar startDate, Calendar endDate) {
            this.startDate = startDate;
            this.endDate = endDate;
        }

        public Calendar getStartDate() {
            return startDate;
        }

        public void setStartDate(Calendar startDate) {
            this.startDate = startDate;
        }

        public Calendar getEndDate() {
            return endDate;
        }

        public void setEndDate(Calendar endDate) {
            this.endDate = endDate;
        }

        public boolean isInBin(Calendar date) {
            return date.compareTo(this.startDate) >= 0 && date.compareTo(this.endDate) < 0;
        }
    }

    private class LabResultDateComparator implements Comparator<LabResult> {
        public int compare (LabResult labResult1, LabResult labResult2) {
            return labResult1.getCollectedDate().compareTo(labResult2.getCollectedDate());
        }
    }

}

