/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.util.*;
import java.util.concurrent.*;

/**
 *
 *
 */
public class NotesRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(NotesRPC.class);

    private static final String XPATH_DOCUMENT_NODE = "//document";
    private static final String XPATH_ENCOUNTER = "./encounter/@value";
    private static final String XPATH_FACILITYNAME = "./facility/@name";
    private static final String XPATH_ID = "./id/@value";
    private static final String XPATH_IMAGES = "./images/@value";
    private static final String XPATH_LOCALTITLE = "./localTitle/@value";
    private static final String XPATH_REFERENCEDATETIME = "./referenceDateTime/@value";
    private static final String XPATH_STATUS = "./status/@value";
    private static final String XPATH_TYPE = "./type/@value";
    private static final String XPATH_CONTENT = "./content";
    private static final String XPATH_NATLTITLE_CODE = "./nationalTitle/@code";
    private static final String XPATH_NATLTITLE_NAME = "./nationalTitle/@name";
    private static final String XPATH_NATLTITLEROLE_CODE = "./nationalTitleRole/@code";
    private static final String XPATH_NATLTITLEROLE_NAME = "./nationalTitleRole/@name";
    private static final String XPATH_NATLTITLESERVICE_CODE = "./nationalTitleService/@code";
    private static final String XPATH_NATLTITLESERVICE_NAME = "./nationalTitleService/@name";
    private static final String XPATH_NATLTITLESETTING_CODE = "./nationalTitleSetting/@code";
    private static final String XPATH_NATLTITLESETTING_NAME = "./nationalTitleSetting/@name";
    private static final String XPATH_NATLTITLESUBJECT_CODE = "./nationalTitleSubject/@code";
    private static final String XPATH_NATLTITLESUBJECT_NAME = "./nationalTitleSubject/@name";
    private static final String XPATH_NATLTITLETYPE_CODE = "./nationalTitleType/@code";
    private static final String XPATH_NATLTITLETYPE_NAME = "./nationalTitleType/@name";
    private static final String XPATH_CLINICIANS = "./clinicians";
    private static final String XPATH_CLINICIAN = "./clinician";
    private static final String XPATH_CLINICIAN_CODE = "./@code";
    private static final String XPATH_CLINICIAN_NAME = "./@name";
    private static final String XPATH_CLINICIAN_ROLE = "./@role";
    private static final String XPATH_CLINICIAN_DATETIME = "./@dateTime";
    private static final String XPATH_CLINICIAN_SIGNATURE = "./@signature";
    private static final String XPATH_CLINICIAN_TAXONOMY_CODE = "./@taxonomyCode";
    private static final String XPATH_CLINICIAN_PROVIDER_TYPE = "./@providerType";
    private static final String XPATH_CLINICIAN_CLASSIFICATION = "./@classification";
    private static final String XPATH_CLINICIAN_SPECIALIZATION = "./@specialization";

    private static final XPathExpression exprDocumentNode = XPathUtil.xpathCompile(XPATH_DOCUMENT_NODE);
    private static final XPathExpression exprEncounter = XPathUtil.xpathCompile(XPATH_ENCOUNTER);
    private static final XPathExpression exprFacilityName = XPathUtil.xpathCompile(XPATH_FACILITYNAME);
    private static final XPathExpression exprImages = XPathUtil.xpathCompile(XPATH_IMAGES);
    private static final XPathExpression exprLocalTitle = XPathUtil.xpathCompile(XPATH_LOCALTITLE);
    private static final XPathExpression exprReferenceDate = XPathUtil.xpathCompile(XPATH_REFERENCEDATETIME);
    private static final XPathExpression exprContent = XPathUtil.xpathCompile(XPATH_CONTENT);
    private static final XPathExpression exprNationalTitleCode = XPathUtil.xpathCompile(XPATH_NATLTITLE_CODE);
    private static final XPathExpression exprNationalTitleName = XPathUtil.xpathCompile(XPATH_NATLTITLE_NAME);
    private static final XPathExpression exprNationalTitleRoleCode = XPathUtil.xpathCompile(XPATH_NATLTITLEROLE_CODE);
    private static final XPathExpression exprNationalTitleRoleName = XPathUtil.xpathCompile(XPATH_NATLTITLEROLE_NAME);
    private static final XPathExpression exprNationalTitleServiceCode = XPathUtil.xpathCompile(XPATH_NATLTITLESERVICE_CODE);
    private static final XPathExpression exprNationalTitleServiceName = XPathUtil.xpathCompile(XPATH_NATLTITLESERVICE_NAME);
    private static final XPathExpression exprNationalTitleSettingCode = XPathUtil.xpathCompile(XPATH_NATLTITLESETTING_CODE);
    private static final XPathExpression exprNationalTitleSettingName = XPathUtil.xpathCompile(XPATH_NATLTITLESETTING_NAME);
    private static final XPathExpression exprNationalTitleSubjectCode = XPathUtil.xpathCompile(XPATH_NATLTITLESUBJECT_CODE);
    private static final XPathExpression exprNationalTitleSubjectName = XPathUtil.xpathCompile(XPATH_NATLTITLESUBJECT_NAME);
    private static final XPathExpression exprNationalTitleTypeCode = XPathUtil.xpathCompile(XPATH_NATLTITLETYPE_CODE);
    private static final XPathExpression exprNationalTitleTypeName = XPathUtil.xpathCompile(XPATH_NATLTITLETYPE_NAME);
    private static final XPathExpression exprClinicians = XPathUtil.xpathCompile(XPATH_CLINICIANS);
    private static final XPathExpression exprClinician = XPathUtil.xpathCompile(XPATH_CLINICIAN);
    private static final XPathExpression exprClinicianCode = XPathUtil.xpathCompile(XPATH_CLINICIAN_CODE);
    private static final XPathExpression exprClinicianName = XPathUtil.xpathCompile(XPATH_CLINICIAN_NAME);
    private static final XPathExpression exprClinicianRole = XPathUtil.xpathCompile(XPATH_CLINICIAN_ROLE);
    private static final XPathExpression exprClinicianDateTime = XPathUtil.xpathCompile(XPATH_CLINICIAN_DATETIME);
    private static final XPathExpression exprClinicianSignature = XPathUtil.xpathCompile(XPATH_CLINICIAN_SIGNATURE);
    private static final XPathExpression exprClinicianTaxonomyCode = XPathUtil.xpathCompile(XPATH_CLINICIAN_TAXONOMY_CODE);
    private static final XPathExpression exprClinicianProviderType = XPathUtil.xpathCompile(XPATH_CLINICIAN_PROVIDER_TYPE);
    private static final XPathExpression exprClinicianClassification = XPathUtil.xpathCompile(XPATH_CLINICIAN_CLASSIFICATION);
    private static final XPathExpression exprClinicianSpecialization = XPathUtil.xpathCompile(XPATH_CLINICIAN_SPECIALIZATION);

    private static final String XPATH_CONSULT = "//consult";
    private static final String XPATH_ORDER_ID = "./orderID/@value";
    private static final String XPATH_PROCEDURE = "./procedure/@value";
    private static final String XPATH_REQUESTED = "./requested/@value";
    private static final String XPATH_SERVICE = "./service/@value";
    private static final String XPATH_DOCUMENTS = "./documents";
    private static final String XPATH_DOCUMENT = "./document";
    private static final String XPATH_DOCUMENT_ID = "./@id";
    private static final String XPATH_DOCUMENT_LOCAL_TITLE = "./@localTitle";
    private static final String XPATH_DOCUMENT_NATIONAL_TITLE = "./@nationalTitle";
    private static final String XPATH_PROVIDER_CODE = "./provider/@code";
    private static final String XPATH_PROVIDER_NAME = "./provider/@name";
    private static final String XPATH_PROVIDER_ROLE = "./provider/@role";
    private static final String XPATH_PROVIDER_TAXOMOMY_CODE = "./provider/@taxonomyCode";
    private static final String XPATH_PROVIDER_PROVIDER_TYPE = "./provider/@providerType";
    private static final String XPATH_PROVIDER_CLASSIFICATION = "./provider/@classification";
    private static final String XPATH_PROVIDER_SPECIALIZATION = "./provider/@specialization";
    private static final String XPATH_PROVIDER_SERVICE = "./provider/@service";
    private static final String XPATH_PROVIDER_OFFICE_PHONE = "./provider/@officePhone";
    private static final String XPATH_PROVIDER_ANALOG_PAGER = "./provider/@analogPager";
    private static final String XPATH_PROVIDER_FAX = "./provider/@fax";
    private static final String XPATH_PROVIDER_EMAIL = "./provider/@email";

    private static final XPathExpression exprConsult = XPathUtil.xpathCompile(XPATH_CONSULT);
    private static final XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ID);
    private static final XPathExpression exprOrderId = XPathUtil.xpathCompile(XPATH_ORDER_ID);
    private static final XPathExpression exprProcedure = XPathUtil.xpathCompile(XPATH_PROCEDURE);
    private static final XPathExpression exprRequested = XPathUtil.xpathCompile(XPATH_REQUESTED);
    private static final XPathExpression exprService = XPathUtil.xpathCompile(XPATH_SERVICE);
    private static final XPathExpression exprStatus = XPathUtil.xpathCompile(XPATH_STATUS);
    private static final XPathExpression exprType = XPathUtil.xpathCompile(XPATH_TYPE);
    private static final XPathExpression exprProviderCode = XPathUtil.xpathCompile(XPATH_PROVIDER_CODE);
    private static final XPathExpression exprProviderName = XPathUtil.xpathCompile(XPATH_PROVIDER_NAME);
    private static final XPathExpression exprProviderRole = XPathUtil.xpathCompile(XPATH_PROVIDER_ROLE);
    private static final XPathExpression exprProviderTaxonomyCode = XPathUtil.xpathCompile(XPATH_PROVIDER_TAXOMOMY_CODE);
    private static final XPathExpression exprProviderProviderType = XPathUtil.xpathCompile(XPATH_PROVIDER_PROVIDER_TYPE);
    private static final XPathExpression exprProviderClassification = XPathUtil.xpathCompile(XPATH_PROVIDER_CLASSIFICATION);
    private static final XPathExpression exprProviderSpecialization = XPathUtil.xpathCompile(XPATH_PROVIDER_SPECIALIZATION);
    private static final XPathExpression exprProviderService = XPathUtil.xpathCompile(XPATH_PROVIDER_SERVICE);
    private static final XPathExpression exprProviderOfficePhone = XPathUtil.xpathCompile(XPATH_PROVIDER_OFFICE_PHONE);
    private static final XPathExpression exprProviderAnalogPager = XPathUtil.xpathCompile(XPATH_PROVIDER_ANALOG_PAGER);
    private static final XPathExpression exprProviderFax = XPathUtil.xpathCompile(XPATH_PROVIDER_FAX);
    private static final XPathExpression exprProviderEmail = XPathUtil.xpathCompile(XPATH_PROVIDER_EMAIL);
    private static final XPathExpression exprDocuments = XPathUtil.xpathCompile(XPATH_DOCUMENTS);
    private static final XPathExpression exprDocument = XPathUtil.xpathCompile(XPATH_DOCUMENT);
    private static final XPathExpression exprDocumentId = XPathUtil.xpathCompile(XPATH_DOCUMENT_ID);
    private static final XPathExpression exprDocumentLocalTitle = XPathUtil.xpathCompile(XPATH_DOCUMENT_LOCAL_TITLE);
    private static final XPathExpression exprDocumentNationalTitle = XPathUtil.xpathCompile(XPATH_DOCUMENT_NATIONAL_TITLE);

    private static final int PARSER_BATCH_SIZE = APP_CONFIG.getParserBatchSizeNotes();
    
//    private static final String PROGRESS_NOTES_CATEGORY_FILTER = "PN";
    private static final String DISCHARGE_SUMMARIES_CATEGORY_FILTER = "DS";
    private static final String CONSULT_RESULTS_CATEGORY_FILTER = "CR";
//    private static final String CWAD_CATEGORY_FILTER = "CWAD";
    private static final String SURGERIES_CATEGORY_FILTER = "SR";
    private static final String CLINICAL_PROCEDURES_CATEGORY_FILTER = "CP";
    private static final String ACTIVE_STATUS = "ACTIVE";

    private static final Map<String, String> NOTETYPEMAP;
    static {
        Map<String, String> typeMap = new HashMap<String, String>();
        typeMap.put("A", "CWAD Note/Allergy");
        typeMap.put("D", "CWAD Note/Advance Directive");
        typeMap.put("C", "CWAD Note/Crisis Note");
        typeMap.put("W", "CWAD Note/Clinical Warning");
        typeMap.put("PN", "Progress Note");
        typeMap.put("CP", "Clinical Procedure");
        typeMap.put("SR", "Surgery");
        typeMap.put("LR", "Laboratory");
        typeMap.put("DS", "Discharge Summary");
        typeMap.put("CR", "Consult Result");
        typeMap.put("CWAD", "CWAD Notes/All");

        NOTETYPEMAP = Collections.unmodifiableMap(typeMap);
    }

    //VA consult status abbreviation to name mapping
    private static final Map<String, String> VA_STATUSMAP = new LinkedHashMap<String, String>() {
        {
            put("dc", "DISCONTINUED");
            put("c", "COMPLETE");
            put("h", "HOLD");
            put("\"?\"", "FLAGGED");
            put("p", "PENDING");
            put("a", "ACTIVE");
            put("e", "EXPIRED");
            put("s", "SCHEDULED");
            put("pr", "PARTIAL RESULTS");
            put("dly", "DELAYED");
            put("u", "UNRELEASED");
            put("dce", "DISCONTINUED/EDIT");
            put("x", "CANCELLED");
            put("l", "LAPSED");
            put("rn", "RENEWED");
            put("'", "NO STATUS");
        }
    };

    public NotesRPC(RPCConnection cxn) {
        super(cxn);
    }

    //SB: missing location, vistDate, status
    public ProgressNote[] getOutpatientNotes(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        //passing zero returns all notes
        String request = buildGetNotesRequest(patientIen, Timestamp.fromDateTimeToTextTS(startDate), Timestamp.fromDateTimeToTextTS(endDate), 0);
        String response = (String) cxn.query(request);
        return toNotesFromRdv(response/*, patientIen*/);
    }

    public ProgressNote[] getOutpatientNotes(Calendar startDate, Calendar endDate) throws RPCException {
        return getOutpatientNotes(cxn.getPatientIen(), startDate, endDate);
    }

    private String buildGetNotesRequest(String patientIen, String fromDate, String toDate, int nNotes) {
        return RPCUtils.buildReportTextRequest(vistaQueryFactory, patientIen, fromDate, toDate, nNotes, "OR_PN:PROGRESS NOTES~TIUPRG;ORDV04;15;");
    }


    private ProgressNote[] toNotesFromRdv(String response/*, String patientIen*/) {
        ProgressNote[] rtc;
        if ("".equals(response)) {
            rtc = new ProgressNote[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<ProgressNote> lst = new ArrayList<ProgressNote>();
            ProgressNote note = null;
            StringBuilder noteText = new StringBuilder();

            for (int i = 0; i < lines.length; i++) {
                if ("".equals(lines[i])) {
                    continue;
                }

                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);

                if (flds.length < 2) {
                    continue;
                }

                if ("[+]".equals(flds[1])) {
                    //note.Text = VistaDaoUtils.removeCtlChars(note.Text);
                    String text = "";
                    if (note != null) {
                        text = note.getNoteText();
                    }

                    text = RPCUtils.removeCtlChars(text);
                    if (note != null) {
                        note.setNoteText(text);
                    }

                    lst.add(note);
                } else if ("1".equals(flds[0])) {
                    note = new ProgressNote();
                    //                String[] subflds = StringUtils.split(flds[1], StringUtils.SEMICOLON);
                    //                KeyValuePair<String, String> site;
                    //                if (subflds.length == 2)
                    //                {
                    //                    site = new KeyValuePair<String, String>(subflds[1], subflds[0]);
                    //                }
                    //                else if (!flds[1].equals(""))
                    //                {
                    //                    site = new KeyValuePair<String, String>((String) cxn.getSiteCode().getKey(), flds[1]);
                    //                }
                    //                else
                    //                {
                    //                    site = cxn.getSiteCode();
                    //                }

                    fillCommonDataBeanValues(note);
                } else if ("2".equals(flds[0])) {
                    if (note != null) {
                        note.setNoteId(flds[1]);
                    }
                } else if ("3".equals(flds[0])) {
                    if (note != null) {
                        note.setNoteDate(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])));
                    }
                } else if ("4".equals(flds[0])) {
                    //note.LocalTitle = flds[1];
                    if (note != null) {
                        note.setNoteType(flds[1]);
                    }
                } else if ("5".equals(flds[0])) {
                    if (note != null) {
                        Provider provider = new Provider();
                        provider.setName(flds[1]);
                        note.setProvider(provider);
                    }
                    //note.Author = new Author("", flds[1], "");
                } else if ("6".equals(flds[0]) && note != null) {
                    noteText.setLength(0);
                    noteText.append(note.getNoteText());
                    noteText.append(flds[1]);
                    noteText.append("\r\n");
                    note.setNoteText(noteText.toString());
                    /*
                    if (flds[1].startsWith("STANDARD TITLE"))
                    {
                        String[] parts = StringUtils.split(flds[1], StringUtils.COLON);
                        note.StandardTitle = parts[1].Trim();
                    }
                    */
                }
            }

            rtc = lst.toArray(new ProgressNote[0]);
        }
        return rtc;
    }

    //SB: Missing location
    public ProgressNote[] getInpatientNotes(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetDischargeSummariesRequest(patientIen, Timestamp.fromDateTimeToTextTS(startDate), Timestamp.fromDateTimeToTextTS(endDate), 0);
        String response = (String) cxn.query(request);
        return toNotesFromDischargeSummaries(response/*, patientIen*/);
    }

    private String buildGetDischargeSummariesRequest(String patientIen, String fromDate, String toDate, int nNotes) {
        //return VistaUtils.buildReportTextRequest(patientIen, fromDate, toDate, nNotes, "OR_DS:DISCHARGE SUMMARY REPORTS~TIUDCS;ORDV04;57;");
        return RPCUtils.buildReportTextRequest(vistaQueryFactory, patientIen, fromDate, toDate, nNotes, "OR_DS:DISCHARGE SUMMARY~TIUDCS;ORDV04;57;");
    }

    private ProgressNote[] toNotesFromDischargeSummaries(String response/*, String patientIen*/) {
        ProgressNote[] rtc;
        if ("".equals(response)) {
            rtc = new ProgressNote[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            List<ProgressNote> lst = new ArrayList<ProgressNote>();
            ProgressNote note = null;
            StringBuilder noteText = new StringBuilder();
            for (int i = 0; i < lines.length; i++) {
                if ("".equals(lines[i])) {
                    continue;
                }

                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);

                if (flds.length < 2) {
                    continue;
                }

                if ("[+]".equals(flds[1])) {
                    if (note != null) {
                        lst.add(note);
                    }
                } else if ("1".equals(flds[0])) {
                    note = new ProgressNote();
                    //note.StandardTitle = "DISCHARGE SUMMARY";
                    note.setNoteType("Discharge Summary");
                    //note.Text = VistaDaoUtils.removeCtlChars(note.Text);
                    //                String[] subflds = StringUtils.split(flds[1], StringUtils.SEMICOLON);
                    //
                    //                KeyValuePair site;
                    //
                    //                if (subflds.length == 2)
                    //                {
                    //                    site = new KeyValuePair<String, String>(subflds[1], subflds[0]);
                    //                }
                    //                else if (!flds[1].equals(""))
                    //                {
                    //                    site = new KeyValuePair<String, String>((String) cxn.getSiteCode().getKey(), flds[1]);
                    //                }
                    //                else
                    //                {
                    //                    site = cxn.getSiteCode();
                    //                }

                    fillCommonDataBeanValues(note);
                } else if ("2".equals(flds[0])) {
                    //note.AdmitTimestamp = VistaTimestamp.toUtcFromRdv(flds[1]);
                    if (note != null) {
                        note.setVisitDate(Utils.formatDisplayDateTime(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])).getTime()));
                    }
                } else if ("3".equals(flds[0])) {
                    //note.DischargeTimestamp = VistaTimestamp.toUtcFromRdv(flds[1]);
                    if (note != null) {
                        note.setNoteDate(Utils.getCal(Timestamp.toUtcFromRdv(flds[1])));
                    }
                } else if ("4".equals(flds[0])) {
                    //note.Author = new Author("", flds[1], "");
                    if (note != null) {
                        Provider provider = new Provider();
                        provider.setName(flds[1]);
                        note.setProvider(provider);
                    }
//                } else if ("5".equals(flds[0])) {
                    //note.ApprovedBy = new Author("", flds[1], "");
                } else if ("6".equals(flds[0])) {
                    // note.Status = flds[1];
                    if (note != null) {
                        note.setStatus(flds[1]);
                    }
                } else if ("7".equals(flds[0]) && note != null) {
                    noteText.setLength(0);
                    noteText.append(note.getNoteText());
                    noteText.append(flds[1]);
                    noteText.append("\r\n");
                    note.setNoteText(noteText.toString());
                }
            }
            rtc = lst.toArray(new ProgressNote[lst.size()]);
        }
        return rtc;
    }

    public ProgressNote[] getInpatientNotes(Calendar startDate, Calendar endDate) throws RPCException {
        return getInpatientNotes(cxn.getPatientIen(), startDate, endDate);
    }

    //SB: Status does not exactly map
    public Consult[] getOutpatientConsults(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetConsultsForPatientRequest(patientIen);
        String response = (String) cxn.query(request);
        Consult[] consults = toConsults(response/*, patientIen*/);
        ArrayList<Consult> consultFiltered = new ArrayList<Consult>();

        //filter by date range
        for (Consult consult : consults) {
            Calendar apptDate = consult.getRequestDate();
            if (apptDate.compareTo(startDate) >= 0 && apptDate.compareTo(endDate) <= 0) {
                consultFiltered.add(consult);
            }
        }

        // make the secondary call for each consult to get the Provider
        for (int i = 0; i < consultFiltered.size(); i++) {
            request = buildGetConsultRequest(consultFiltered.get(i).getId());
            response = (String) cxn.query(request);
            consultFiltered.get(i).setProvider(extractConsultProviderName(response));
        }

        return consultFiltered.toArray(new Consult[0]);
    }

    public Consult[] getOutpatientConsults(Calendar startDate, Calendar endDate) throws RPCException {
        return getOutpatientConsults(cxn.getPatientIen(), startDate, endDate);
    }

    private String buildGetConsultsForPatientRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQCN LIST");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    private String buildGetConsultRequest(String consultIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQCN GET CONSULT");
        vquery.addParameter(VistaQuery.LITERAL, consultIen);
        vquery.addParameter(VistaQuery.LITERAL, "0");
        return vquery.buildMessage();
    }

    private String extractConsultProviderName(String consult) {
        String rtc;
        if ("".equals(consult)
                || consult.startsWith("-1")) {
            rtc = "";
        } else {
            String[] lines = StringUtils.split(consult, StringUtils.CRLF);
            lines = StringUtils.trimArray(lines);

            String[] flds = StringUtils.split(lines[0], StringUtils.CARET);
            String[] subFlds = StringUtils.split(flds[13], StringUtils.SEMICOLON);

            rtc = subFlds[1];
        }
        return rtc;
    }

    public FreeTextReport getOutpatientConsultReport(String consultId) throws RPCException {
        String request = buildGetConsultReportRequest(consultId);
        String response = (String) cxn.query(request);

        FreeTextReport report = new FreeTextReport();
        report.setReportText(response);
        fillCommonDataBeanValues(report);

        return report;
    }

    private String buildGetConsultReportRequest(String consultId) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQCN DETAIL");
        vquery.addParameter(VistaQuery.LITERAL, consultId);
        return vquery.buildMessage();
    }

    private Consult[] toConsults(String response/*, String patientIen*/) {
        Consult[] rtc;
        if ("".equals(response)
                || response.startsWith("< PATIENT DOES NOT HAVE ANY CONSULTS/REQUESTS")) {
            rtc = new Consult[0];
        } else {
            String[] rex = StringUtils.split(response, StringUtils.CRLF);
            rex = StringUtils.trimArray(rex);
            Consult[] result = new Consult[rex.length];
            for (int i = 0; i < rex.length; i++) {
                String[] flds = StringUtils.split(rex[i], StringUtils.CARET);
                Consult cons = new Consult();
                cons.setId(flds[0]);
                cons.setRequestDate(Timestamp.toDateTime(flds[1]));

                //map VA consult status abbreviation to status name
                String status = flds[2];

                String mappedStatus = VA_STATUSMAP.get(status);

                if (!StringUtils.isEmpty(mappedStatus)) {
                    status = mappedStatus;
                }

                cons.setStatus(status);
                cons.setService(flds[3]);
                cons.setProcedureConsult(flds[4]);
                //c.Title = flds[6];
                //c.Service = new KeyValuePair<String, String>("", flds[2]);
                fillCommonDataBeanValues(cons);
                result[i] = cons;
            }
            rtc = result;
        }
        return rtc;
    }

    public FreeTextReport getVisitNotes(QueryBean queryBean) throws RPCException {

        // Find the TIU Document file IEN from the Visit IEN
        String request = buildVisitTIUIENRequest(queryBean.getItemId());
        String response = (String) cxn.query(request);
        String[] tiuDocFileIENs = toTIUDocFileIENs(response);

        // Get the Record Text
        StringBuilder recordTextSB = new StringBuilder("");
        for (String tiuIEN : tiuDocFileIENs) {
            request = buildTIUGetRecordTextRequest(tiuIEN);
            response = (String) cxn.query(request);

            recordTextSB.append(response);
            recordTextSB.append("\n");
        }

        FreeTextReport record = new FreeTextReport();
        record.setReportText(recordTextSB.toString());
        fillCommonDataBeanValues(record);

        return record;
    }

    private String buildVisitTIUIENRequest(String visitIEN) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("TIU NOTES BY VISIT");
        vquery.addParameter(VistaQuery.LITERAL, visitIEN);
        return vquery.buildMessage();
    }

    private String[] toTIUDocFileIENs(String response) {
        String[] rtc;
        if ("".equals(response)) {
            rtc = new String[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<String> lst = new ArrayList<String>();
//            String ien = null;

            for (String line : lines) {
                if ("".equals(line)) {
                    continue;
                }

                String[] flds = StringUtils.split(line, StringUtils.CARET);

                // the first field is the IEN
                if (!("".equals(flds[0]))) {
                    lst.add(flds[0]);
                }
            }

            rtc = (String[]) lst.toArray(new String[0]);
        }
        return rtc;
    }

    private String buildTIUGetRecordTextRequest(String tiuFileDocIEN) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("TIU GET RECORD TEXT");
        vquery.addParameter(VistaQuery.LITERAL, tiuFileDocIEN);
        return vquery.buildMessage();
    }

    /**
     * This method gets the detailed text of a TIU document from the TIU Document file (#8925).
     * It makes a call to the "TIU GET RECORD TEXT" RPC.<p/>
     * <p/>
     * This method can be used in conjunction with the getProgressNotesVPR() method. It can be
     * the secondary call to get the details of a progress note when the noteId is passed as the
     * parameter.
     *
     * @param docId TIU document ID
     * @return the text report for the document
     * @throws RPCException
     */
    public FreeTextReport getNotesById(String docId) throws RPCException {

        // Find the TIU Document file IEN from the Visit IEN
        String request = buildTIUGetRecordTextRequest(docId);
        String response = (String) cxn.query(request);

        FreeTextReport record = new FreeTextReport();
        record.setReportText(response);
        fillCommonDataBeanValues(record);

        return record;
    }

    public String getProgressNote(QueryBean queryBean) throws RPCException {
        String rtc = "";
        String noteID = queryBean.getItemId();

        // The text of the note is parsed when the list is retrieved
        // so we get the list again
        ProgressNote[] progressNotes = getOutpatientNotes(queryBean.getStartDate(), queryBean.getEndDate());

        // find the progress note we want and return the text of that note
        for (ProgressNote note : progressNotes) {
            if (note.getNoteId().equalsIgnoreCase(noteID)) {
                rtc = note.getNoteText();
                break;
            }
        }

        // if it didn't return from the for loop that means we couldn't match the note with the id
        return rtc;
    }

    /**
     * Uses the VPR RPC with "documents" as the type to get a list of discharge summaries. Will
     * return only documents that don't have have <code>DS</code> (Discharge Summaries) or <code>CR</code>
     * (Consult Results) as the <code>type</code>in the VPR response. The types in the VPR response for progress
     * notes are <code>A, D, C, W, PN, CP, SR, LR</code> for CWAD note/Allergy, CWAD note/Advance Directive,
     * CWAD note/Crisis Note, CWAD note/Clinical Warning, progress note, clinical procedure, surgery, and laboratory
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of documents
     * @param endDate    The end of the date range for the list of documents
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public ProgressNote[] getProgressNotesVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        // Get PN, CR,, H&P(LOINC 34117), DS, CWAD (see SETUP in VPRDTIU CLASS = "3^244")
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, "", false);
//        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, "PN", false);
        String response = (String) cxn.query(request);

        ProgressNote[] progressNoteArray = parseDocumentsFromVPR(response);
        ArrayList<ProgressNote> progressNotesArrayList = new ArrayList<ProgressNote>(Arrays.asList(progressNoteArray));

        // Add Surgeries
        request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, SURGERIES_CATEGORY_FILTER, false);
        response = (String) cxn.query(request);

        ProgressNote[] surgeriesArray = parseDocumentsFromVPR(response);
        progressNotesArrayList.addAll(Arrays.asList(surgeriesArray));

        // Add Clinical Procedures
        request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, CLINICAL_PROCEDURES_CATEGORY_FILTER, false);
        response = (String) cxn.query(request);

        ProgressNote[] clinProcsArray = parseDocumentsFromVPR(response);
        progressNotesArrayList.addAll(Arrays.asList(clinProcsArray));

        Set<String> vprNoteIdSet = new HashSet<String>((int) (progressNotesArrayList.size() * 1.5d));
        for (ProgressNote progressNote : progressNotesArrayList) {
            vprNoteIdSet.add(progressNote.getNoteId());
        }

        // Get uncosignedNotes
        ArrayList<ProgressNote> nonVPRNotes;
        ArrayList<ProgressNote> nonVPRDiscahrgeSummaries;
        if (APP_CONFIG.bseTokenEnabled() && cxn.getUserHostSite() != null && !"VBA".equalsIgnoreCase(cxn.getUserHostSite().getMoniker())) {
            nonVPRNotes = getNonVPRNotesBse(patientIen, startDate, endDate, vprNoteIdSet, TIU_CLASS_PROGRESS_NOTES);
            progressNotesArrayList.addAll(nonVPRNotes);
            for (ProgressNote progressNote : nonVPRNotes) {
                // add any new note Ids to the comparison set
                vprNoteIdSet.add(progressNote.getNoteId());
            }

            // Get uncosigned discharge summaries
            nonVPRDiscahrgeSummaries = getNonVPRNotesBse(patientIen, startDate, endDate, vprNoteIdSet, TIU_CLASS_DISCHARGE_SUMMARIES);
            progressNotesArrayList.addAll(nonVPRDiscahrgeSummaries);
        } else {
            nonVPRNotes = getNonVPRNotes(patientIen, startDate, endDate, vprNoteIdSet, TIU_CLASS_PROGRESS_NOTES);
            progressNotesArrayList.addAll(nonVPRNotes);
            for (ProgressNote progressNote : nonVPRNotes) {
                // add any new note Ids to the comparison set
                vprNoteIdSet.add(progressNote.getNoteId());
            }

            // Get uncosigned discharge summaries
            nonVPRDiscahrgeSummaries = getNonVPRNotes(patientIen, startDate, endDate, vprNoteIdSet, TIU_CLASS_DISCHARGE_SUMMARIES);
            progressNotesArrayList.addAll(nonVPRDiscahrgeSummaries);

        }
        return progressNotesArrayList.toArray(new ProgressNote[0]);
    }

    /**
     * Gets the notes not in the VPR call, needs to switch context since the RPC call is under the CPRS context and 
     * not be unde the same context as VPR
     *
     * @param patientIen
     * @param startDate
     * @param endDate
     * @param vprNoteIdSet
     * @param tiuClass
     * @return
     * @throws RPCException
     */
    private ArrayList<ProgressNote> getNonVPRNotes(String patientIen, Calendar startDate, Calendar endDate, Set<String> vprNoteIdSet, String tiuClass) throws RPCException {
        String response = null;
        ArrayList<ProgressNote> rtc = null;
        
        String previousContext = cxn.getCurrentContext();
        if (!VistaConstants.CPRS_CONTEXT.equalsIgnoreCase(previousContext)) {

            VistaQuery vquery = vistaQueryFactory.getVistaQuery("XWB CREATE CONTEXT");
            vquery.addEncryptedParameter(VistaQuery.LITERAL, VistaConstants.CPRS_CONTEXT);
            response = (String) cxn.query(vquery.buildMessage());
            if ("1".equals(response)) {
                response = callUncosignedNotesRpc(patientIen, startDate, endDate, tiuClass);
                rtc = toUncosignedNotes(response, startDate, endDate, vprNoteIdSet);
                
                //restore context to previous state
                vquery = vistaQueryFactory.getVistaQuery("XWB CREATE CONTEXT");
                vquery.addEncryptedParameter(VistaQuery.LITERAL, previousContext);
                cxn.query(vquery.buildMessage());
            }
        } else {
            // context switch is unnecessary, same as for the Bse call, so we'll use that one
            rtc = getNonVPRNotesBse(patientIen, startDate, endDate, vprNoteIdSet, tiuClass);
        }
        
        return rtc;
    }

    /**
     * Gets the notes not in the VPR call, doesn't need to switch context since for BSE all RPC calls will be under
     * the same context
     * 
     * @param patientIen
     * @param startDate
     * @param endDate
     * @param vprNoteIdSet
     * @param tiuClass
     * @return
     * @throws RPCException
     */
    private ArrayList<ProgressNote> getNonVPRNotesBse(String patientIen, Calendar startDate, Calendar endDate, Set<String> vprNoteIdSet, String tiuClass) throws RPCException {
        String response = callUncosignedNotesRpc(patientIen, startDate, endDate, tiuClass);

        ArrayList<ProgressNote> rtc = toUncosignedNotes(response, startDate, endDate, vprNoteIdSet);

        return rtc;
    }
    
    public ProgressNote[] getProgressNotesVPRImagesFieldOnly(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        // Get PN, CR,, H&P(LOINC 34117), DS, CWAD (see SETUP in VPRDTIU CLASS = "3^244")
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, "", false);
//        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, "PN", false);
        String response = (String) cxn.query(request);

        ProgressNote[] progressNoteArray = parseDocumentsFromVPRImagesFieldOnly(response);
        ArrayList<ProgressNote> progressNotesArrayList = new ArrayList<ProgressNote>(Arrays.asList(progressNoteArray));

        // Add Surgeries
        request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, SURGERIES_CATEGORY_FILTER, false);
        response = (String) cxn.query(request);

        ProgressNote[] surgeriesArray = parseDocumentsFromVPRImagesFieldOnly(response);
        progressNotesArrayList.addAll(Arrays.asList(surgeriesArray));

        // Add Clinical Procedures
        request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, CLINICAL_PROCEDURES_CATEGORY_FILTER, false);
        response = (String) cxn.query(request);

        ProgressNote[] clinProcsArray = parseDocumentsFromVPRImagesFieldOnly(response);
        progressNotesArrayList.addAll(Arrays.asList(clinProcsArray));
        
        return progressNotesArrayList.toArray(new ProgressNote[0]);
    }


    private static final String TIU_CLASS_PROGRESS_NOTES = "3";
    private static final String TIU_CLASS_DISCHARGE_SUMMARIES = "244";
    /*
     * note type is 3 for PN, 244 for DC
     */
    private String callUncosignedNotesRpc(String patientIen, Calendar startDate, Calendar endDate, String tiuClass) throws RPCException {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("TIU DOCUMENTS BY CONTEXT");
        vquery.addParameter(VistaQuery.LITERAL,tiuClass); // PROGRESS NOTES CLASS = 3, DISCHARGE SUMMARIES = 244
        vquery.addParameter(VistaQuery.LITERAL,"1"); // get all notes. Need to filter
        vquery.addParameter(VistaQuery.LITERAL,patientIen);
        vquery.addParameter(VistaQuery.LITERAL,Timestamp.fromDateTime(startDate));
        vquery.addParameter(VistaQuery.LITERAL,Timestamp.fromDateTime(endDate));
        vquery.addParameter(VistaQuery.LITERAL,"0");
        vquery.addParameter(VistaQuery.LITERAL,"9999999");
        vquery.addParameter(VistaQuery.LITERAL,"D"); // note dates from latest to earliest
        vquery.addParameter(VistaQuery.LITERAL,"0");

        return (String) cxn.query(vquery.buildMessage());
    }

    /**
     * Parse the response from TIU DOCUMENTS BY CONTEXT. Add the note text.
     * 
     * @param response
     * @param startDate
     * @param endDate
     * @param vprNoteIdSet
     * @return
     * @throws RPCException
     */
    private ArrayList<ProgressNote> toUncosignedNotes(String response, Calendar startDate, Calendar endDate, Set<String> vprNoteIdSet) throws RPCException {
        ArrayList<ProgressNote> rtc = new ArrayList<ProgressNote>();
        if (!"".equals(response)) {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            lines = StringUtils.trimArray(lines);

            for (int i = 0; i < lines.length; i++) {
                String[] fields = StringUtils.split(lines[i], StringUtils.CARET);

                ProgressNote note = new ProgressNote();
                note.setNoteDate(Timestamp.toDateTime(fields[2]));

                // the date filter may not be working so only consider notes in the date range
                // the note dates should be descending from latest to earliest
                if (note.getNoteDate().compareTo(endDate) > 0) {
                    // the note is after the end date so continue until we get to the first relevant note
                    continue;
                }
                if (note.getNoteDate().compareTo(startDate) < 0) {
                    // the note is before our start date so we can break the loop
                    break;
                }

                fillCommonDataBeanValues(note);
                note.setNoteId(fields[0]);
                note.setNoteTitle(fields[1]);
                note.setLocation(fields[5]);
                note.setStatus(fields[6]);

                // get Provider
                if (!StringUtils.isEmpty(fields[4])) {
                    String[] providerFields = StringUtils.split(fields[4], StringUtils.SEMICOLON);
                    Provider provider = new Provider();
                    if (providerFields.length > 2) {
                        provider.setName(providerFields[2]);
                        note.setProvider(provider);
                        note.setUserIen(providerFields[0]);
                    }
                }

                // only add the note if the noteId is not in the VPR list
                if (!vprNoteIdSet.contains(note.getNoteId())) {
                    // retrieve the note text for eligible notes
                    note.setNoteText(this.getNotesById(note.getNoteId()).getReportText());
                    rtc.add(note);
                }
            }
        }

        return rtc;
    }
    
    /**
     * Uses the VPR RPC with "documents" as the type to get a list of discharge summaries. Will
     * return only documents that don't have have <code>DS</code> (Discharge Summaries) or <code>CR</code>
     * (Consult Results) as the <code>type</code>in the VPR response. The types in the VPR response for progress
     * notes are <code>A, D, C, W, PN, CP, SR, LR</code> for CWAD note/Allergy, CWAD note/Advance Directive,
     * CWAD note/Crisis Note, CWAD note/Clinical Warning, progress note, clinical procedure, surgery, and laboratory
     *
     * @param startDate The start of the date range for the list of documents
     * @param endDate   The end of the date range for the list of documents
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public ProgressNote[] getProgressNotesVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getProgressNotesVPR(cxn.getPatientIen(), startDate, endDate);
    }

    public ProgressNote[] getProgressNotesVPRImagesFieldOnly(Calendar startDate, Calendar endDate) throws RPCException {
        return getProgressNotesVPRImagesFieldOnly(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the VPR RPC with "documents" as the type to get a list of discharge summaries. Will
     * return only documents that have <code>DS</code> as the <code>type</code>in the VPR response.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of documents
     * @param endDate    The end of the date range for the list of documents
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public ProgressNote[] getDischargeSummariesVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, DISCHARGE_SUMMARIES_CATEGORY_FILTER, true);
        String response = (String) cxn.query(request);

        ProgressNote[] summaries = parseDocumentsFromVPR(response);

        ArrayList<ProgressNote> progressNotesArrayList = new ArrayList<ProgressNote>(Arrays.asList(summaries));
        Set<String> vprNoteIdSet = new HashSet<String>((int) (progressNotesArrayList.size() * 1.5d));
        for (ProgressNote progressNote: progressNotesArrayList) {
            vprNoteIdSet.add(progressNote.getNoteId());
        }

        // Get uncosigned discharge summaries
        ArrayList<ProgressNote> nonVPRNotes;
        if (APP_CONFIG.bseTokenEnabled() && cxn.getUserHostSite() != null && !"VBA".equalsIgnoreCase(cxn.getUserHostSite().getMoniker())) {
            nonVPRNotes = getNonVPRNotesBse(patientIen, startDate, endDate, vprNoteIdSet, TIU_CLASS_DISCHARGE_SUMMARIES);
        } else {
            nonVPRNotes = getNonVPRNotes(patientIen, startDate, endDate, vprNoteIdSet, TIU_CLASS_DISCHARGE_SUMMARIES);
        }
        progressNotesArrayList.addAll(nonVPRNotes);

        EncountersRPC encountersRPC = new EncountersRPC(cxn);
        for (ProgressNote summary : summaries) {
            Encounter admission = encountersRPC.getPatientAdmission(patientIen, summary.getEncounter());

            if (admission != null && admission.getArrivalDateTime() != null) {
                String admissionDateString = Utils.formatDisplayDateTime(admission.getArrivalDateTime().getTime());
                summary.setVisitDate(admissionDateString);

            }
        }

        return progressNotesArrayList.toArray(new ProgressNote[0]);
    }

    /**
     * Uses the VPR RPC with "documents" as the type to get a list of discharge summaries. Will
     * return only documents that have <code>DS</code> as the <code>type</code>in the VPR response.
     *
     * @param startDate The start of the date range for the list of documents
     * @param endDate   The end of the date range for the list of documents
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public ProgressNote[] getDischargeSummariesVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getDischargeSummariesVPR(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the VPR RPC with "documents" as the type to get a list of consult results. Will
     * return only documents that have <code>CR</code> as the <code>type</code>in the VPR response. This is
     * not to be confused with the list of all the consults. It is only a list of consult result documents. If
     * there is a consult that does not have a resulted document, that consult will not be represented in this
     * list. The documents in this list must be cross-referenced to the consult by the document id. This list
     * does not contain the contents of the consult result.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of documents
     * @param endDate    The end of the date range for the list of documents
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public ProgressNote[] getConsultResultsVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DOCUMENTS, CONSULT_RESULTS_CATEGORY_FILTER, false);
        String response = (String) cxn.query(request);

        return parseDocumentsFromVPR(response);
    }

    /**
     * Uses the VPR RPC with "documents" as the type to get a list of consult results. Will
     * return only documents that have <code>CR</code> as the <code>type</code>in the VPR response. This is
     * not to be confused with the list of all the consults. It is only a list of consult result documents. If
     * there is a consult that does not have a resulted document, that consult will not be represented in this
     * list. The documents in this list must be cross-referenced to the consult by the document id. This list
     * does not contain the contents of the consult result.
     *
     * @param startDate The start of the date range for the list of documents
     * @param endDate   The end of the date range for the list of documents
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public ProgressNote[] getConsultResultsVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getConsultResultsVPR(cxn.getPatientIen(), startDate, endDate);
    }


    /**
     * Uses the VPR RPC with "consults" as the type to get a list of consult requests.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of consults
     * @param endDate    The end of the date range for the list of consults
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public Consult[] getConsultRequestsVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.CONSULTS);
        String response = (String) cxn.query(request);
        Consult[] consultRequests =  parseConsultsFromVPR(response);

        return consultRequests;
    }

    /**
     * Uses the VPR RPC with "consults" as the type to get a list of consult requests.
     *
     * @param startDate  The start of the date range for the list of consults
     * @param endDate    The end of the date range for the list of consults
     * @return The list of progress notes for the specified date range
     * @throws RPCException
     */
    public Consult[] getConsultRequestsVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getConsultRequestsVPR(cxn.getPatientIen(), startDate, endDate);
    }


    public Consult getOutpatientConsultRequestByIdVPR(String patientIen, String consultId) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, VPRUtil.VPRType.CONSULTS, consultId);
        String response = (String) cxn.query(request);

        Consult[] consultRequests =  parseConsultsFromVPR(response);

        Consult consult = null;
        if (consultRequests.length > 0) {
            consult = consultRequests[0];
        }
//        else {
//            consult = new Consult();
//            fillCommonDataBeanValues(consult);
//        }
        return consult;
    }

    public Consult getOutpatientConsultRequestByIdVPR(String consultId) throws RPCException {
        return getOutpatientConsultRequestByIdVPR(cxn.getPatientIen(), consultId);
    }

    public Consult[] getActiveConsultRequestsVPR(String patientIen) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, VPRUtil.VPRType.CONSULTS);
        String response = (String) cxn.query(request);
        Consult[] consultRequests =  parseConsultsFromVPR(response);

        ArrayList<Consult> activeConsults = new ArrayList<Consult>();
        for (Consult consult : consultRequests) {
            if (ACTIVE_STATUS.equalsIgnoreCase(consult.getStatus())) {
                activeConsults.add(consult);
            }
        }

        return activeConsults.toArray(new Consult[0]);
    }

    public Consult[] getActiveConsultRequestsVPR() throws  RPCException {
        return getActiveConsultRequestsVPR(cxn.getPatientIen());
    }

    private ProgressNote[] parseDocumentsFromVPR(String response) throws RPCException {
        ProgressNote[] rtc;
        if ("".equals(response)) {
            rtc = new ProgressNote[0];
        } else {
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

            ArrayList<ProgressNote> documentList = new ArrayList<ProgressNote>();

            Document doc = null;
            try {
                doc = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }

            NodeList documentsNodeList = null;
            try {
                documentsNodeList = (NodeList) exprDocumentNode.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);
            int documentNodeListLength = documentsNodeList.getLength();
            for (int documentNum = 0; documentNum < documentNodeListLength; documentNum++) {
                parserBatchManager.addNode(documentsNodeList.item(documentNum));
            }
            
            List<VPRDocumentParser> documentParserList = new ArrayList<VPRDocumentParser>();
            for (List<Node> documentParserBatch : parserBatchManager.getParserBatchList()) {
                documentParserList.add(new VPRDocumentParser(documentParserBatch));
            }
            
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<ProgressNote>>> futures = null;
            try {
                futures = executorService.invokeAll(documentParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
                
            } catch (InterruptedException e) {
                XPathUtil.logErrorMessage(LOGGER, e);
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }

            Iterator<Future<List<ProgressNote>>> futuresItr = futures.iterator();

            while (futuresItr.hasNext()) {
                try {
                    Future<List<ProgressNote>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR ProgressNote Data");
                    } else {
                        documentList.addAll(future.get());
                    }
                } catch (InterruptedException ie) {
                    XPathUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    XPathUtil.logErrorMessage(LOGGER, ee);
                }
            }

            rtc = documentList.toArray(new ProgressNote[0]);
        }
        return rtc;
    }

    private class VPRDocumentParser implements Callable<List<ProgressNote>> {
        private List<Node> documentNodeList;

        public VPRDocumentParser(List<Node> documentNodeList) {
            this.documentNodeList = documentNodeList;
        }

        public List<ProgressNote> call() throws RPCException {

            List<ProgressNote> documentList = new ArrayList<ProgressNote>();

            for (Node documentNode : documentNodeList) {
                String noteType = XPathUtil.evaluateNodeToString(documentNode, exprType, LOGGER);
                String documentName = XPathUtil.evaluateNodeToString(documentNode, exprLocalTitle, LOGGER);
                String documentIen = XPathUtil.evaluateNodeToString(documentNode, exprId, LOGGER);
                String encounter = XPathUtil.evaluateNodeToString(documentNode, exprEncounter, LOGGER);
                String referenced = XPathUtil.evaluateNodeToString(documentNode, exprReferenceDate, LOGGER);
                String facilityName = XPathUtil.evaluateNodeToString(documentNode, exprFacilityName, LOGGER);
                String images = XPathUtil.evaluateNodeToString(documentNode, exprImages, LOGGER);
                String status = XPathUtil.evaluateNodeToString(documentNode, exprStatus, LOGGER);
                String content = XPathUtil.evaluateNodeToString(documentNode, exprContent, LOGGER);
                
                ProgressNote document = new ProgressNote();
                fillCommonDataBeanValues(document);
                
                document.setNoteTitle(documentName);
                document.setNoteId(documentIen);
                if (noteType != null) {
                    String noteTypeDescription = NOTETYPEMAP.get(noteType);
                    if (noteTypeDescription != null) {
                        document.setNoteType(noteTypeDescription);
                    } else {
                        document.setNoteType(noteType);
                    }
                }
                document.setEncounter(encounter);
                document.setLocation(facilityName);
                document.setImages(images);
                document.setStatus(status);
                document.setNoteText(content);

                if (!StringUtils.isEmpty(referenced)) {
                    Calendar referenceDate;
                    referenceDate = Timestamp.toDateTime(referenced);
                    document.setNoteDate(referenceDate);
                }

                // Get the national codes
                String nationalTitleCode = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleCode, LOGGER);
                String nationalTitleName = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleName, LOGGER);
                String nationalTitleRoleCode = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleRoleCode, LOGGER);
                String nationalTitleRoleName = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleRoleName, LOGGER);
                String nationalTitleServiceCode = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleServiceCode, LOGGER);
                String nationalTitleServiceName = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleServiceName, LOGGER);
                String nationalTitleSettingCode = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleSettingCode, LOGGER);
                String nationalTitleSettingName = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleSettingName, LOGGER);
                String nationalTitleSubjectCode = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleSubjectCode, LOGGER);
                String nationalTitleSubjectName = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleSubjectName, LOGGER);
                String nationalTitleTypeCode = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleTypeCode, LOGGER);
                String nationalTitleTypeName = XPathUtil.evaluateNodeToString(documentNode, exprNationalTitleTypeName, LOGGER);

                List<Code> nationalCodeList = new ArrayList<Code>();
                if (!StringUtils.isEmpty(nationalTitleCode)) {
                    Code natlTitle = new Code();
                    natlTitle.setCode(nationalTitleCode);
                    natlTitle.setSystem("nationalTitle");
                    natlTitle.setDisplay(nationalTitleName);
                    nationalCodeList.add(natlTitle);

                    Code vuid = new Code();
                    vuid.setCode(nationalTitleCode);
                    vuid.setSystem("VUID");
                    vuid.setDisplay(nationalTitleName);
                    nationalCodeList.add(vuid);
                }
                if (!StringUtils.isEmpty(nationalTitleRoleCode)) {
                    Code natlTitleRole = new Code();
                    natlTitleRole.setCode(nationalTitleRoleCode);
                    natlTitleRole.setSystem("nationalTitleRole");
                    natlTitleRole.setDisplay(nationalTitleRoleName);
                    nationalCodeList.add(natlTitleRole);
                }
                if (!StringUtils.isEmpty(nationalTitleServiceCode)) {
                    Code natlTitleService = new Code();
                    natlTitleService.setCode(nationalTitleServiceCode);
                    natlTitleService.setSystem("nationalTitleService");
                    natlTitleService.setDisplay(nationalTitleServiceName);
                    nationalCodeList.add(natlTitleService);
                }
                if (!StringUtils.isEmpty(nationalTitleSettingCode)) {
                    Code natlTitleSetting = new Code();
                    natlTitleSetting.setCode(nationalTitleSettingCode);
                    natlTitleSetting.setSystem("nationalTitleSetting");
                    natlTitleSetting.setDisplay(nationalTitleSettingName);
                    nationalCodeList.add(natlTitleSetting);
                }
                if (!StringUtils.isEmpty(nationalTitleSubjectCode)) {
                    Code natlTitleSubject = new Code();
                    natlTitleSubject.setCode(nationalTitleSubjectCode);
                    natlTitleSubject.setSystem("nationalTitleSubject");
                    natlTitleSubject.setDisplay(nationalTitleSubjectName);
                    nationalCodeList.add(natlTitleSubject);
                }
                if (!StringUtils.isEmpty(nationalTitleTypeCode)) {
                    Code natlTitleType = new Code();
                    natlTitleType.setCode(nationalTitleTypeCode);
                    natlTitleType.setSystem("nationalTitleType");
                    natlTitleType.setDisplay(nationalTitleTypeName);
                    nationalCodeList.add(natlTitleType);
                }
                document.setCodes(nationalCodeList.toArray(new Code[0]));

                // Get provider information (clinicians) Look for the "S" role.
                Node cliniciansNode = XPathUtil.evaluateNodeToNode(documentNode, exprClinicians, LOGGER);
                if (cliniciansNode != null) {
                    NodeList clinicianNodeList = XPathUtil.evaluateNodeToNodeList(cliniciansNode, exprClinician, LOGGER);
                    if (clinicianNodeList != null) {
                        int listLength = clinicianNodeList.getLength();
                        for (int clinicianNum = 0; clinicianNum < listLength; clinicianNum++) {
                            Node clinicianNode = clinicianNodeList.item(clinicianNum);
                            clinicianNode.getParentNode().removeChild(clinicianNode);

                            String role = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianRole, LOGGER);
                            if (!StringUtils.isEmpty(role) && "S".equalsIgnoreCase(role)) {
                                Provider provider = new Provider();
                                provider.setRole(role);
                                String providerName = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianName, LOGGER);
                                String providerIen = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianCode, LOGGER);
                                String providerDateTimeString = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianDateTime, LOGGER);
                                String providerSignature = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianSignature, LOGGER);
                                String providerTaxonomyCode = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianTaxonomyCode, LOGGER);
                                String providerType = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianProviderType, LOGGER);
                                String providerClassification = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianClassification, LOGGER);
                                String providerSpecialization = XPathUtil.evaluateNodeToString(clinicianNode, exprClinicianSpecialization, LOGGER);

                                if (!StringUtils.isEmpty(providerName)) {
                                    provider.setName(providerName);
                                }
                                if (!StringUtils.isEmpty(providerIen)) {
                                    document.setUserIen(providerIen);
                                    provider.setCode(providerIen);
                                }

                                Calendar providerDate = null;
                                if (!StringUtils.isEmpty(providerDateTimeString)) {
                                    providerDate = Timestamp.toDateTime(providerDateTimeString);
                                    provider.setDateTime(providerDate);
                                }
                                if (!StringUtils.isEmpty(providerSignature)) {
                                    provider.setSignatureBlock(providerSignature);
                                }
                                if (!StringUtils.isEmpty(providerTaxonomyCode)) {
                                    provider.setTaxonomyCode(providerTaxonomyCode);
                                }
                                if (!StringUtils.isEmpty(providerType)) {
                                    provider.setProviderType(providerType);
                                }
                                if (!StringUtils.isEmpty(providerClassification)) {
                                    provider.setClassification(providerClassification);
                                }
                                if (!StringUtils.isEmpty(providerSpecialization)) {
                                    provider.setSpecialization(providerSpecialization);
                                }

                                document.setProvider(provider);
                            }
                        }
                    }
                }

                documentList.add(document);
            }
            return documentList;
        }
    }

    private ProgressNote[] parseDocumentsFromVPRImagesFieldOnly(String response) throws RPCException {
        ProgressNote[] rtc;
        if ("".equals(response)) {
            rtc = new ProgressNote[0];
        } else {
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

            ArrayList<ProgressNote> documentList = new ArrayList<ProgressNote>();

            Document doc = null;
            try {
                doc = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }

            NodeList documentsNodeList = null;
            try {
                documentsNodeList = (NodeList) exprDocumentNode.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);
            int documentNodeListLength = documentsNodeList.getLength();
            for (int documentNum = 0; documentNum < documentNodeListLength; documentNum++) {
                parserBatchManager.addNode(documentsNodeList.item(documentNum));
            }

            List<VPRDocumentImagesFieldOnlyParser> documentParserList = new ArrayList<VPRDocumentImagesFieldOnlyParser>();
            for (List<Node> documentParserBatch : parserBatchManager.getParserBatchList()) {
                documentParserList.add(new VPRDocumentImagesFieldOnlyParser(documentParserBatch));
            }

            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<ProgressNote>>> futures = null;
            try {
                futures = executorService.invokeAll(documentParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);

            } catch (InterruptedException e) {
                XPathUtil.logErrorMessage(LOGGER, e);
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }

            Iterator<Future<List<ProgressNote>>> futuresItr = futures.iterator();

            while (futuresItr.hasNext()) {
                try {
                    Future<List<ProgressNote>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR ProgressNote Data");
                    } else {
                        documentList.addAll(future.get());
                    }
                } catch (InterruptedException ie) {
                    XPathUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    XPathUtil.logErrorMessage(LOGGER, ee);
                }
            }

            rtc = documentList.toArray(new ProgressNote[0]);
        }
        return rtc;
    }

    private class VPRDocumentImagesFieldOnlyParser implements Callable<List<ProgressNote>> {
        private List<Node> documentNodeList;

        public VPRDocumentImagesFieldOnlyParser(List<Node> documentNodeList) {
            this.documentNodeList = documentNodeList;
        }

        public List<ProgressNote> call() throws RPCException {

            List<ProgressNote> documentList = new ArrayList<ProgressNote>();

            for (Node documentNode : documentNodeList) {
                String documentIen = XPathUtil.evaluateNodeToString(documentNode, exprId, LOGGER);
                String images = XPathUtil.evaluateNodeToString(documentNode, exprImages, LOGGER);

                ProgressNote document = new ProgressNote();
                fillCommonDataBeanValues(document);

                document.setNoteId(documentIen);
                document.setImages(images);

                documentList.add(document);
            }
            return documentList;
        }
    }

    private Consult[] parseConsultsFromVPR(String response) throws RPCException {
        Consult[] rtc;
        if ("".equals(response)) {
            rtc = new Consult[0];
        } else {
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

            ArrayList<Consult> consultList = new ArrayList<Consult>();
            
            Document document = null;
            try {
                document = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }

            NodeList consultNodeList = null;
            try {
                consultNodeList = (NodeList) exprConsult.evaluate(document, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);
            int consultNodeListLength = consultNodeList.getLength();
            for (int consultNum = 0; consultNum < consultNodeListLength; consultNum++) {
                parserBatchManager.addNode(consultNodeList.item(consultNum));
            }


            List<VPRConsultParser> consultParserList = new ArrayList<VPRConsultParser>();
            for (List<Node> consultBatch : parserBatchManager.getParserBatchList()) {
                consultParserList.add(new VPRConsultParser(consultBatch));
            }

            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<Consult>>> futures = null;
            try {
                futures = executorService.invokeAll(consultParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);

            } catch (InterruptedException e) {
                XPathUtil.logErrorMessage(LOGGER, e);
                throw new RPCException(e);
            }

            Iterator<Future<List<Consult>>> futureIterator = futures.iterator();

            while (futureIterator.hasNext()) {
                try {
                    Future<List<Consult>> future = futureIterator.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Consult Data");
                    } else {
                        consultList.addAll(future.get());
                    }
                } catch (InterruptedException ie) {
                    XPathUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    XPathUtil.logErrorMessage(LOGGER, ee);
                }
            }

            rtc = consultList.toArray(new Consult[0]);
        }
        return rtc;
    }

    private class VPRConsultParser implements Callable<List<Consult>> {
        private List<Node> consultNodeList;

        public VPRConsultParser(List<Node> consultNodeList) {
            this.consultNodeList = consultNodeList;
        }

        public List<Consult> call() throws RPCException {

            List<Consult> consultList = new ArrayList<Consult>();
            
            for (Node consultNode: consultNodeList) {
                String id = XPathUtil.evaluateNodeToString(consultNode, exprId, LOGGER);
                String orderId = XPathUtil.evaluateNodeToString(consultNode, exprOrderId, LOGGER);
                String procedure = XPathUtil.evaluateNodeToString(consultNode, exprProcedure, LOGGER);
                String requestedString = XPathUtil.evaluateNodeToString(consultNode, exprRequested, LOGGER);
                String service = XPathUtil.evaluateNodeToString(consultNode, exprService, LOGGER);
                String status = XPathUtil.evaluateNodeToString(consultNode, exprStatus, LOGGER);
                String type = XPathUtil.evaluateNodeToString(consultNode, exprType, LOGGER);
                String providerCode = XPathUtil.evaluateNodeToString(consultNode, exprProviderCode, LOGGER);
                String providerName = XPathUtil.evaluateNodeToString(consultNode, exprProviderName, LOGGER);
                String providerRole = XPathUtil.evaluateNodeToString(consultNode, exprProviderRole, LOGGER);
                String providerTaxonomyCode = XPathUtil.evaluateNodeToString(consultNode, exprProviderTaxonomyCode, LOGGER);
                String providerProviderType = XPathUtil.evaluateNodeToString(consultNode, exprProviderProviderType, LOGGER);
                String providerClassification = XPathUtil.evaluateNodeToString(consultNode, exprProviderClassification, LOGGER);
                String providerSpecialization = XPathUtil.evaluateNodeToString(consultNode, exprProviderSpecialization, LOGGER);
                String providerService = XPathUtil.evaluateNodeToString(consultNode, exprProviderService, LOGGER);
                String providerOfficePhone = XPathUtil.evaluateNodeToString(consultNode, exprProviderOfficePhone, LOGGER);
                String providerAnalogPager = XPathUtil.evaluateNodeToString(consultNode, exprProviderAnalogPager, LOGGER);
                String providerFax = XPathUtil.evaluateNodeToString(consultNode, exprProviderFax, LOGGER);
                String providerEmail = XPathUtil.evaluateNodeToString(consultNode, exprProviderEmail, LOGGER);

                Consult consult = new Consult();
                fillCommonDataBeanValues(consult);
                
                Node documentsNode = XPathUtil.evaluateNodeToNode(consultNode, exprDocuments, LOGGER);
                if (documentsNode != null) {
                    NodeList documentNodeList = XPathUtil.evaluateNodeToNodeList(documentsNode, exprDocument, LOGGER);

                    if (documentNodeList != null) {
                        int documentNodeListLength = documentNodeList.getLength();
                        List<EncounterDocument> documentList = new ArrayList<EncounterDocument>();
                        for (int documentNum = 0; documentNum < documentNodeListLength; documentNum++) {
                            Node documentNode = documentNodeList.item(documentNum);
                            documentNode.getParentNode().removeChild(documentNode);

                            EncounterDocument document = new EncounterDocument();
                            fillCommonDataBeanValues(document);

                            String documentId = XPathUtil.evaluateNodeToString(documentNode, exprDocumentId, LOGGER);
                            String documentLocalTitle = XPathUtil.evaluateNodeToString(documentNode, exprDocumentLocalTitle, LOGGER);
                            String documentNationalTitle = XPathUtil.evaluateNodeToString(documentNode, exprDocumentNationalTitle, LOGGER);

                            document.setId(documentId);
                            document.setLocalTitle(documentLocalTitle);
                            document.setNationalTitle(documentNationalTitle);

                            documentList.add(document);
                        }

                        if (!documentList.isEmpty()) {
                            consult.setDocuments(documentList.toArray(new EncounterDocument[0]));
                        }
                    }
                }

                Calendar requested = null;
                if (requestedString != null) {
                    requested = Timestamp.toDateTime(requestedString);
                }
                consult.setId(id);
                consult.setOrderId(orderId);
                consult.setProcedureConsult(procedure);
                consult.setRequestDate(requested);
                consult.setService(service);
                consult.setStatus(status);
                consult.setConsultType(type);

                Provider provider = new Provider();
                provider.setName(providerName);
                provider.setRole(providerRole);
                provider.setCode(providerCode);
                provider.setTaxonomyCode(providerTaxonomyCode);
                provider.setProviderType(providerProviderType);
                provider.setClassification(providerClassification);
                provider.setSpecialization(providerSpecialization);
                provider.setService(providerService);
                provider.setPhone(providerOfficePhone);
                provider.setAnalogPager(providerAnalogPager);
                provider.setFax(providerFax);
                provider.setEmail(providerEmail);

                consult.setProvider(provider);

                consultList.add(consult);

            }
            return consultList;
        }
    }

    public PatientPostings getPatientPostings() throws RPCException {
        return getPatientPostings(cxn.getPatientIen());
    }

    public PatientPostings getPatientPostings(String patientIen) throws RPCException {
        String request = buildGetPostingsListRequest(patientIen);
        String response = (String) cxn.query(request);

        PatientPostings postings = new PatientPostings();

        postings.setNoteList(toNotePostings(response));

        String cwad = getCwadFlag(patientIen);
        postings.setCwad(cwad.length() > 1 ? cwad : null);

        if (postings.getCwad() != null && postings.getCwad().contains("A")) {
            postings.setAllergyList(getAllergyPostings(patientIen));
        }

        fillCommonDataBeanValues(postings);

        return postings;
    }

    private List<PostingNote> toNotePostings(String response) {

        if ("".equals(response)) {
            return new ArrayList<PostingNote>();
        }

        String[] lines = StringUtils.split(response, StringUtils.CRLF);
        List<PostingNote> postingNoteList = new ArrayList<PostingNote>();

        for (String line: lines) {
            String[] pieces = StringUtils.split(line, StringUtils.CARET);
            if (pieces.length < 1) {
                continue;
            }

            PostingNote postingNote = new PostingNote();

            for (int i=0; i < pieces.length; i++) {
                String item = pieces[i];

                if (StringUtils.isEmpty(item)) {
                    continue;
                }
                switch (i) {
                    case 0:
                        postingNote.setNoteId(item);
                        break;
                    case 1:
                        postingNote.setNoteTitle(item);
                        break;
                    case 2:
                        Calendar date = Timestamp.toDateTime(item);
                        postingNote.setNoteDate(date);
                    default: // do nothing

                }
            }
            postingNoteList.add(postingNote);
        }

        return postingNoteList;
    }

    private String buildGetPostingsListRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQPP LIST");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);

        return vquery.buildMessage();
    }

    public FreeTextReport getPostingsAllergiesSummary() throws RPCException{
        return getPostingsAllergiesSummary(cxn.getPatientIen());
    }

    public FreeTextReport getPostingsAllergiesSummary(String patientIen) throws RPCException {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQAL LIST REPORT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        String request = vquery.buildMessage();
        String response = (String) cxn.query(request);

        FreeTextReport record = new FreeTextReport();
        record.setReportText(response);
        fillCommonDataBeanValues(record);

        return record;
    }

    public String getCwadFlag(String patientIen) throws RPCException {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT CWAD");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        String request = vquery.buildMessage();
        return (String) cxn.query(request);
    }

    private List<PostingAllergy> getAllergyPostings(String patientIen) throws RPCException {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQAL LIST");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        String request = vquery.buildMessage();
        String resp = (String) cxn.query(request);

        if ("".equals(resp)) {
            return new ArrayList<PostingAllergy>();
        }

        final int IDX_ALLERGY_ID = 0;
        final int IDX_ALLERGY_NAME = 1;
        final int IDX_ALLERGY_SEVERITY = 2;
        final int IDX_ALLERGY_SIGNS_AND_SYMPTOMS = 3;

        List<PostingAllergy> allergyList = new ArrayList<PostingAllergy>();

        String[] lines = StringUtils.split(resp, StringUtils.CRLF);
        for(String line : lines) {

            String[] pieces = StringUtils.split(line, StringUtils.CARET);

            if (pieces.length < 1)
                continue;

            PostingAllergy allergy = new PostingAllergy();
            allergy.setAllergyId(pieces[IDX_ALLERGY_ID]);
            allergy.setAllergyName(pieces[IDX_ALLERGY_NAME]);
            allergy.setSeverity(pieces[IDX_ALLERGY_SEVERITY]);
            allergy.setSignsAndSymptoms(pieces[IDX_ALLERGY_SIGNS_AND_SYMPTOMS]);

            allergyList.add(allergy);
        }

        return allergyList;
    }

    final String PCE_TYPE_EDUCATION = "PED";
    final String PCE_TYPE_HEALTH_FACTOR = "HF";
    final String PCE_TYPE_EXAM = "XAM";
    final String PCE_COMMENT = "COM";

    /**
     * Gets the first set of PCEs attached to one or more notes (by IEN). The list of IENs separated by semicolons  
     * 
     * @param noteIens one or more note Iens separated by semicolons
     * @return the first list of PCEs
     * @throws RPCException
     */
    public List<PCE> getPcesForNote(String noteIens) throws RPCException {
        if ("".equals(noteIens)) {
            return new ArrayList<PCE>();
        }
        String[] iens = StringUtils.split(noteIens, StringUtils.SEMICOLON);
        
        for (String ien : iens) {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPCE PCE4NOTE");
            vquery.addParameter(VistaQuery.LITERAL, ien.trim());
            String request = vquery.buildMessage();
            String resp = (String) cxn.query(request);

            if ("".equals(resp)) {
                continue;
            }

            List<PCE> pceList = new ArrayList<PCE>();

            String[] lines = StringUtils.split(resp, StringUtils.CRLF);
            for(int i = 0; i < lines.length; i++) {
                String line = lines[i];

                String[] pieces = StringUtils.split(line, StringUtils.CARET);

                if (pieces.length < 1) {
                    // empty line, continue to next
                    continue;
                }

                String type = pieces[0];
                if (PCE_TYPE_EDUCATION.equalsIgnoreCase(type)
                        || PCE_TYPE_EXAM.equalsIgnoreCase(type)
                        || PCE_TYPE_HEALTH_FACTOR.equalsIgnoreCase(type)) {
                    PCE pce = new PCE();
                    fillCommonDataBeanValues(pce);
                    pce.setType(pieces[0]);
                    pce.setName(pieces[3]);
                    pceList.add(pce);
                    if (pieces.length > 9) {
                        // there could be a comment, check the next line
                        line = lines[i + 1];
                        pieces = StringUtils.split(line, StringUtils.CARET);
                        type = pieces[0];
                        if (PCE_COMMENT.equalsIgnoreCase(type)) {
                            pce.setComment(pieces[2]);
                            // skip this line on the next time through for loop
                            i++;
                        }
                    }
                } 
            }

            if (pceList.size() > 0) {
                return pceList;
            }
        }

        // no lists found for all iens
        return new ArrayList<PCE>();
    }
}
