/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.Provider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.*;

/**
 *
 *
 */
public class OrderRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(AppointmentsRPC.class);

    private static final String XPATH_ORDER = "//order";
    private static final String XPATH_CONTENT = "./content";
    private static final String XPATH_ENTERED = "./entered/@value";
    private static final String XPATH_ID = "./id/@value";
    private static final String XPATH_PROVIDER_CODE = "./provider/@code";
    private static final String XPATH_PROVIDER_NAME = "./provider/@name";
    private static final String XPATH_PROVIDER_ROLE = "./provider/@role";
    private static final String XPATH_PROVIDER_TAXOMOMY_CODE = "./provider/@taxonomyCode";
    private static final String XPATH_PROVIDER_PROVIDER_TYPE = "./provider/@providerType";
    private static final String XPATH_PROVIDER_CLASSIFICATION = "./provider/@classification";
    private static final String XPATH_PROVIDER_SPECIALIZATION = "./provider/@specialization";
    private static final String XPATH_PROVIDER_SERVICE = "./provider/@service";
    private static final String XPATH_PROVIDER_OFFICE_PHONE = "./provider/@officePhone";
    private static final String XPATH_PROVIDER_ANALOG_PAGER = "./provider/@analogPager";
    private static final String XPATH_PROVIDER_FAX = "./provider/@fax";
    private static final String XPATH_PROVIDER_EMAIL = "./provider/@email";
    private static final String XPATH_START = "./start/@value";
    private static final String XPATH_STATUS_NAME = "./status/@name";
    private static final String XPATH_STOP = "./stop/@value";
    private static final String XPATH_TYPE = "./type/@value";
    private static final String XPATH_CODING_SYSTEM_CODE = "./codingSystem/@code";
    private static final String XPATH_CODING_SYSTEM_NAME = "./codingSystem/@name";
    private static final String XPATH_VUID = "./vuid/@value";
    
    private static final XPathExpression exprOrder = XPathUtil.xpathCompile(XPATH_ORDER);
    private static final XPathExpression exprContent = XPathUtil.xpathCompile(XPATH_CONTENT);
    private static final XPathExpression exprEntered = XPathUtil.xpathCompile(XPATH_ENTERED);
    private static final XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ID);
    private static final XPathExpression exprProviderCode = XPathUtil.xpathCompile(XPATH_PROVIDER_CODE);
    private static final XPathExpression exprProviderName = XPathUtil.xpathCompile(XPATH_PROVIDER_NAME);
    private static final XPathExpression exprProviderRole = XPathUtil.xpathCompile(XPATH_PROVIDER_ROLE);
    private static final XPathExpression exprProviderTaxonomyCode = XPathUtil.xpathCompile(XPATH_PROVIDER_TAXOMOMY_CODE);
    private static final XPathExpression exprProviderProviderType = XPathUtil.xpathCompile(XPATH_PROVIDER_PROVIDER_TYPE);
    private static final XPathExpression exprProviderClassification = XPathUtil.xpathCompile(XPATH_PROVIDER_CLASSIFICATION);
    private static final XPathExpression exprProviderSpecialization = XPathUtil.xpathCompile(XPATH_PROVIDER_SPECIALIZATION);
    private static final XPathExpression exprProviderService = XPathUtil.xpathCompile(XPATH_PROVIDER_SERVICE);
    private static final XPathExpression exprProviderOfficePhone = XPathUtil.xpathCompile(XPATH_PROVIDER_OFFICE_PHONE);
    private static final XPathExpression exprProviderAnalogPager = XPathUtil.xpathCompile(XPATH_PROVIDER_ANALOG_PAGER);
    private static final XPathExpression exprProviderFax = XPathUtil.xpathCompile(XPATH_PROVIDER_FAX);
    private static final XPathExpression exprProviderEmail = XPathUtil.xpathCompile(XPATH_PROVIDER_EMAIL);
    private static final XPathExpression exprStatusName = XPathUtil.xpathCompile(XPATH_STATUS_NAME);
    private static final XPathExpression exprStop = XPathUtil.xpathCompile(XPATH_STOP);
    private static final XPathExpression exprType = XPathUtil.xpathCompile(XPATH_TYPE);
    private static final XPathExpression exprCodingSystemCode = XPathUtil.xpathCompile(XPATH_CODING_SYSTEM_CODE);
    private static final XPathExpression exprCodingSystemName = XPathUtil.xpathCompile(XPATH_CODING_SYSTEM_NAME);
    private static final XPathExpression exprVuid = XPathUtil.xpathCompile(XPATH_VUID);

    private static final int PARSER_BATCH_SIZE = APP_CONFIG.getParserBatchSizeOrders();

    public OrderRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Order[] getPatientOrders(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientOrders(cxn.getPatientIen(), startDate, endDate);
    }

    public Order[] getPatientOrders(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        Order[] orders = getPatientOrdersVPR(patientIen, startDate, endDate);

        return orders;
    }

    public FreeTextReport getOrderDetail(String orderId) throws RPCException {
        return getOrderDetail(cxn.getPatientIen(), orderId);
    }

    public FreeTextReport getOrderDetail(String patientIen, String orderId) throws RPCException {
        String request = buildGetPatientOrderDetailRequest(patientIen, orderId);
        // The result of this RPC is a Formatted Text Report
        String reportText = (String) cxn.query(request);

        FreeTextReport report = new FreeTextReport();
        report.setReportText(reportText);
        fillCommonDataBeanValues(report);

        return report;
    }

    private String buildGetPatientOrderDetailRequest(String patientIen, String orderId) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQOR DETAIL");
        vquery.addParameter(VistaQuery.LITERAL, orderId);
        vquery.addParameter(VistaQuery.LITERAL, patientIen);

        return vquery.buildMessage();
    }


    public Order[] getPatientOrdersVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientOrdersVPR(cxn.getPatientIen(), startDate, endDate);
    }

    public Order[] getPatientOrdersVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        Order[] rtc;
        String request = buildGetPatientOrdersVPRRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        Order[] orders =  parseOrders(response);
        // filter date range by order date
        if (orders == null) {
            rtc = new Order[0];
        } else {
            rtc = filterOrdersByOrderDate(orders, startDate, endDate);
        }
        return rtc;
    }

    private Order[] filterOrdersByOrderDate(Order[] orders, Calendar startDate, Calendar endDate) {
        ArrayList<Order> ordersFilteredList = new ArrayList<Order>();
        Order[] rtc;

        for (Order order: orders) {
            Calendar orderDate = order.getOrderDate();
            if (orderDate.compareTo(startDate) >= 0 && orderDate.compareTo(endDate) <= 0) {
                ordersFilteredList.add(order);
            }
        }
        if (ordersFilteredList.isEmpty()) {
            rtc = new Order[0];
        } else {
            rtc = ordersFilteredList.toArray(new Order[0]);
        }
        return rtc;
    }

    private String buildGetPatientOrdersVPRRequest(String patientIen, Calendar startDate, Calendar endDate) {
        return VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.ORDERS);
    }

    private Order[] parseOrders(String response) throws RPCException {
        Order[] rtc;
        if ("".equals(response)) {
            rtc = new Order[0];
        } else {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            ArrayList<Order> orderList = new ArrayList<Order>();

            Document document = null;
            try {
                document = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList orderNodeList = null;
            try {
                orderNodeList = (NodeList) exprOrder.evaluate(document, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE);

            int orderNodeListLength = orderNodeList.getLength();
            for (int orderNum = 0; orderNum < orderNodeListLength; orderNum++) {
                parserBatchManager.addNode(orderNodeList.item(orderNum));
            }

            List<VPROrderParser> orderParserList = new ArrayList<VPROrderParser>();

            for(List<Node> parserBatch : parserBatchManager.getParserBatchList()) {
                orderParserList.add(new VPROrderParser(parserBatch));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<Order>>> futures = null;
            try {
                futures = executorService.invokeAll(orderParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                XPathUtil.logErrorMessage(LOGGER, e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<Order>>> futureIterator = futures.iterator();
    
            while (futureIterator.hasNext()) {
                try {
                    Future<List<Order>> future = futureIterator.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Order Data");
                    } else {
                        orderList.addAll(future.get());
                    }
                } catch (InterruptedException ie) {
                    XPathUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    XPathUtil.logErrorMessage(LOGGER, ee);
                }
            }
    
            rtc = orderList.toArray(new Order[0]);
        }
        return rtc;
    }

    private class VPROrderParser implements Callable<List<Order>> {
        private List<Node> orderNodeList;

        public VPROrderParser(List<Node> orderNodeList) {
            this.orderNodeList = orderNodeList;
        }

        public List<Order> call() throws Exception {

            List<Order> orders = new ArrayList<Order>();

            for(Node orderNode : orderNodeList) {
                String enteredString = XPathUtil.evaluateNodeToString(orderNode, exprEntered, LOGGER);
                String stopString = XPathUtil.evaluateNodeToString(orderNode, exprStop, LOGGER);
                Calendar entered = null;
                Calendar stop = null;
                if (!StringUtils.isEmpty(enteredString)) {
                    entered = Timestamp.toDateTime(enteredString);
                }
                if (!StringUtils.isEmpty(stopString)) {
                    stop = Timestamp.toDateTime(stopString);
                }

                String content = XPathUtil.evaluateNodeToString(orderNode, exprContent, LOGGER);
                String id = XPathUtil.evaluateNodeToString(orderNode, exprId, LOGGER);
                String providerCode = XPathUtil.evaluateNodeToString(orderNode, exprProviderCode, LOGGER);
                String providerName = XPathUtil.evaluateNodeToString(orderNode, exprProviderName, LOGGER);
                String providerRole = XPathUtil.evaluateNodeToString(orderNode, exprProviderRole, LOGGER);
                String providerTaxonomyCode = XPathUtil.evaluateNodeToString(orderNode, exprProviderTaxonomyCode, LOGGER);
                String providerProviderType = XPathUtil.evaluateNodeToString(orderNode, exprProviderProviderType, LOGGER);
                String providerClassification = XPathUtil.evaluateNodeToString(orderNode, exprProviderClassification, LOGGER);
                String providerSpecialization = XPathUtil.evaluateNodeToString(orderNode, exprProviderSpecialization, LOGGER);
                String providerService = XPathUtil.evaluateNodeToString(orderNode, exprProviderService, LOGGER);
                String providerOfficePhone = XPathUtil.evaluateNodeToString(orderNode, exprProviderOfficePhone, LOGGER);
                String providerAnalogPager = XPathUtil.evaluateNodeToString(orderNode, exprProviderAnalogPager, LOGGER);
                String providerFax = XPathUtil.evaluateNodeToString(orderNode, exprProviderFax, LOGGER);
                String providerEmail = XPathUtil.evaluateNodeToString(orderNode, exprProviderEmail, LOGGER);
                String statusName = XPathUtil.evaluateNodeToString(orderNode, exprStatusName, LOGGER);
                String type = XPathUtil.evaluateNodeToString(orderNode, exprType, LOGGER);

                String codingSystemCode = XPathUtil.evaluateNodeToString(orderNode, exprCodingSystemCode, LOGGER);
                String codingSystemName = XPathUtil.evaluateNodeToString(orderNode, exprCodingSystemName, LOGGER);
                String vuid = XPathUtil.evaluateNodeToString(orderNode, exprVuid, LOGGER);

                Order order = new Order();
                fillCommonDataBeanValues(order);

                if (content != null) {
                    order.setDescription(content.trim());
                }
                order.setEnteredDate(entered);
                order.setCompletedDate(stop);
                order.setOrderDate(entered);
                order.setOrderid(id);
                order.setType(type);
                order.setStatus(statusName);

                Provider provider = new Provider();
                provider.setName(providerName);
                provider.setRole(providerRole);
                provider.setCode(providerCode);
                provider.setTaxonomyCode(providerTaxonomyCode);
                provider.setProviderType(providerProviderType);
                provider.setClassification(providerClassification);
                provider.setSpecialization(providerSpecialization);
                provider.setPhone(providerOfficePhone);
                provider.setService(providerService);
                provider.setEmail(providerEmail);
                provider.setFax(providerFax);
                provider.setAnalogPager(providerAnalogPager);

                order.setOrderingProvider(provider);

                List<Code> codeList = new ArrayList<Code>();
                Code orderCode = new Code();
                orderCode.setSystem("TEXT");
                orderCode.setCode(order.getDescription());
                orderCode.setDisplay(order.getDescription());
                codeList.add(orderCode);
                if (vuid != null) {
                    Code vuidCode = new Code();
                    vuidCode.setCode(vuid);
                    vuidCode.setSystem("VUID");
                    codeList.add(vuidCode);
                }
                if (codingSystemCode != null) {
                    Code codingSystem = new Code();
                    codingSystem.setCode(codingSystemCode);
                    codingSystem.setSystem(codingSystemName);
                    codeList.add(codingSystem);
                }
                order.setCodes(codeList.toArray(new Code[0]));

                orders.add(order);
            }

            return orders;
        }
    }

}
