/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import java.util.*;
import java.util.concurrent.*;


/**
 *
 *
 */
public class PatientRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientRPC.class);
    
    // patient flags xpath
    private static final String XPATH_FLAG = "//flag";
    private static final String XPATH_FLAG_CATEGORY = "./category/@value";
    private static final String XPATH_FLAG_CONTENT = "./content";
    private static final String XPATH_FLAG_ID = "./id/@value";
    private static final String XPATH_FLAG_NAME = "./name/@value";
    private static final String XPATH_FLAG_ORIG_SITE = "./origSite/@name";
    private static final String XPATH_FLAG_OWN_SITE = "./ownSite/@name";
    private static final String XPATH_FLAG_REVIEW_DUE = "./reviewDue/@value";
    private static final String XPATH_FLAG_TYPE = "./type/@value";
    private static final String XPATH_FLAG_APPROVED_BY = "./approvedBy/@name";
    private static final String XPATH_FLAG_ASSIGNED = "./assigned/@value";
    private static final String XPATH_FLAG_DOCUMENT_CODE = "./document/@code";
    private static final String XPATH_FLAG_DOCUMENT_NAME = "./document/@name";

    private static final XPathExpression exprFlag = XPathUtil.xpathCompile(XPATH_FLAG);
    private static final XPathExpression exprFlagCategory = XPathUtil.xpathCompile(XPATH_FLAG_CATEGORY);
    private static final XPathExpression exprFlagContent = XPathUtil.xpathCompile(XPATH_FLAG_CONTENT);
    private static final XPathExpression exprFlagId = XPathUtil.xpathCompile(XPATH_FLAG_ID);
    private static final XPathExpression exprFlagName = XPathUtil.xpathCompile(XPATH_FLAG_NAME);
    private static final XPathExpression exprFlagOrigSite = XPathUtil.xpathCompile(XPATH_FLAG_ORIG_SITE);
    private static final XPathExpression exprFlagOwnSite = XPathUtil.xpathCompile(XPATH_FLAG_OWN_SITE);
    private static final XPathExpression exprFlagReviewDate = XPathUtil.xpathCompile(XPATH_FLAG_REVIEW_DUE);
    private static final XPathExpression exprFlagType = XPathUtil.xpathCompile(XPATH_FLAG_TYPE);
    private static final XPathExpression exprFlagApprovedBy = XPathUtil.xpathCompile(XPATH_FLAG_APPROVED_BY);
    private static final XPathExpression exprFlagAssigned = XPathUtil.xpathCompile(XPATH_FLAG_ASSIGNED);
    private static final XPathExpression exprFlagDocumentCode = XPathUtil.xpathCompile(XPATH_FLAG_DOCUMENT_CODE);
    private static final XPathExpression exprFlagDocumentName = XPathUtil.xpathCompile(XPATH_FLAG_DOCUMENT_NAME);

    private static final int PARSER_BATCH_SIZE_PATIENT_FLAGS = APP_CONFIG.getParserBatchSizePatientFlags();

    // patient demographics xpath
    private static final String XPATH_DEMO_FULLNAME = "./fullName/@value";
    private static final String XPATH_DEMO_GENDER = "./gender/@value";
    private static final String XPATH_DEMO_DOB = "./dob/@value";
    private static final String XPATH_DEMO_SSN = "./ssn/@value";
    private static final String XPATH_DEMO_ID = "./id/@value";

    private static final String XPATH_DEMO_SENSITIVE = "./sensitive/@value";
    private static final String XPATH_DEMO_SC_PERCENT = "./scPercent/@value";
    private static final String XPATH_DEMO_ADDRESS = "./address";
    private static final String XPATH_DEMO_STREET1 = "./@streetLine1";
    private static final String XPATH_DEMO_STREET2 = "./@streetLine2";
    private static final String XPATH_DEMO_STREET3 = "./@streetLine3";
    private static final String XPATH_DEMO_CITY = "./@city";
    private static final String XPATH_DEMO_STATE_PROVINCE = "./@stateProvince";
    private static final String XPATH_DEMO_POSTAL_CODE = "./@postalCode";
    private static final String XPATH_DEMO_TELECOM_LIST = "./telecomList";
    private static final String XPATH_DEMO_TELECOM = "./telecom";

    private static final String XPATH_DEMO_PATIENT = "//patient";
    private static final String XPATH_DEMO_ETHNICITIES = "./ethnicities";
    private static final String XPATH_DEMO_ETHNICITY = "./ethnicity";
    private static final String XPATH_DEMO_VALUE = "./@value";
    private static final String XPATH_DEMO_RACES = "./races";
    private static final String XPATH_DEMO_RACE = "./race";

    private static final String XPATH_DEMO_FLAGS = "./flags";
    private static final String XPATH_DEMO_FLAG = "./flag";
    private static final String XPATH_DEMO_NAME = "./@name";
    private static final String XPATH_DEMO_TEXT = "./@text";

    private static final String XPATH_DEMO_ELIGIBILITIES = "./eligibilities";
    private static final String XPATH_DEMO_ELIGIBILITY = "./eligibility";
    private static final String XPATH_DEMO_PRIMARY = "./@primary";
    private static final String XPATH_DEMO_ELIGIBILITY_STATUS = "./eligibilityStatus/@value";
    private static final String XPATH_DEMO_PC_ASSIGNED = "./pcAssigned/@value";
    private static final String XPATH_DEMO_PC_TEAM_NAME = "./pcTeam/@name";

    private static final String XPATH_DEMO_PC_PROVIDER = "./pcProvider";
    private static final String XPATH_DEMO_CODE = "./@code";
    private static final String XPATH_DEMO_OFFICE_PHONE = "./@officePhone";
    private static final String XPATH_DEMO_ANALOG_PAGER = "./@analogPager";
    private static final String XPATH_DEMO_FAX = "./@fax";
    private static final String XPATH_DEMO_EMAIL = "./@email";
    private static final String XPATH_DEMO_TAXOMOMY_CODE = "./@taxonomyCode";
    private static final String XPATH_DEMO_PROVIDER_TYPE = "./@providerType";
    private static final String XPATH_DEMO_CLASSIFICATION = "./@classification";
    private static final String XPATH_DEMO_SPECIALIZATION = "./@specialization";
    private static final String XPATH_DEMO_SERVICE = "./@service";

    private static final XPathExpression exprDemoPatient = XPathUtil.xpathCompile(XPATH_DEMO_PATIENT);
    private static final XPathExpression exprDemoEthnicities = XPathUtil.xpathCompile(XPATH_DEMO_ETHNICITIES);
    private static final XPathExpression exprDemoEthnicity = XPathUtil.xpathCompile(XPATH_DEMO_ETHNICITY);
    private static final XPathExpression exprDemoRaces = XPathUtil.xpathCompile(XPATH_DEMO_RACES);
    private static final XPathExpression exprDemoRace = XPathUtil.xpathCompile(XPATH_DEMO_RACE);
    private static final XPathExpression exprDemoValue = XPathUtil.xpathCompile(XPATH_DEMO_VALUE);

    private static final XPathExpression exprDemoFullName = XPathUtil.xpathCompile(XPATH_DEMO_FULLNAME);
    private static final XPathExpression exprDemoGender = XPathUtil.xpathCompile(XPATH_DEMO_GENDER);
    private static final XPathExpression exprDemoDob = XPathUtil.xpathCompile(XPATH_DEMO_DOB);
    private static final XPathExpression exprDemoSsn = XPathUtil.xpathCompile(XPATH_DEMO_SSN);
    private static final XPathExpression exprDemoId = XPathUtil.xpathCompile(XPATH_DEMO_ID);
    private static final XPathExpression exprDemoSensitive = XPathUtil.xpathCompile(XPATH_DEMO_SENSITIVE);
    private static final XPathExpression exprDemoScPercent = XPathUtil.xpathCompile(XPATH_DEMO_SC_PERCENT);
    private static final XPathExpression exprDemoAddress = XPathUtil.xpathCompile(XPATH_DEMO_ADDRESS);
    private static final XPathExpression exprDemoStreetLine1 = XPathUtil.xpathCompile(XPATH_DEMO_STREET1);
    private static final XPathExpression exprDemoStreetLine2 = XPathUtil.xpathCompile(XPATH_DEMO_STREET2);
    private static final XPathExpression exprDemoStreetLine3 = XPathUtil.xpathCompile(XPATH_DEMO_STREET3);
    private static final XPathExpression exprDemoCity = XPathUtil.xpathCompile(XPATH_DEMO_CITY);
    private static final XPathExpression exprDemoStateProvince = XPathUtil.xpathCompile(XPATH_DEMO_STATE_PROVINCE);
    private static final XPathExpression exprDemoPostalCode = XPathUtil.xpathCompile(XPATH_DEMO_POSTAL_CODE);
    private static final XPathExpression exprDemoTelecomList = XPathUtil.xpathCompile(XPATH_DEMO_TELECOM_LIST);
    private static final XPathExpression exprDemoTelecom = XPathUtil.xpathCompile(XPATH_DEMO_TELECOM);

    private static final XPathExpression exprDemoFlags = XPathUtil.xpathCompile(XPATH_DEMO_FLAGS);
    private static final XPathExpression exprDemoFlag = XPathUtil.xpathCompile(XPATH_DEMO_FLAG);
    private static final XPathExpression exprDemoName = XPathUtil.xpathCompile(XPATH_DEMO_NAME);
    private static final XPathExpression exprDemoText = XPathUtil.xpathCompile(XPATH_DEMO_TEXT);

    private static final XPathExpression exprDemoEligibilities = XPathUtil.xpathCompile(XPATH_DEMO_ELIGIBILITIES);
    private static final XPathExpression exprDemoEligibility = XPathUtil.xpathCompile(XPATH_DEMO_ELIGIBILITY);
    private static final XPathExpression exprDemoPrimary = XPathUtil.xpathCompile(XPATH_DEMO_PRIMARY);
    private static final XPathExpression exprDemoEligibilityStatus = XPathUtil.xpathCompile(XPATH_DEMO_ELIGIBILITY_STATUS);
    private static final XPathExpression exprDemoPcAssigned = XPathUtil.xpathCompile(XPATH_DEMO_PC_ASSIGNED);
    private static final XPathExpression exprDemoPcTeamName = XPathUtil.xpathCompile(XPATH_DEMO_PC_TEAM_NAME);
    private static final XPathExpression exprDemoPcProvider = XPathUtil.xpathCompile(XPATH_DEMO_PC_PROVIDER);
    private static final XPathExpression exprDemoCode = XPathUtil.xpathCompile(XPATH_DEMO_CODE);
    private static final XPathExpression exprDemoTaxonomyCode = XPathUtil.xpathCompile(XPATH_DEMO_TAXOMOMY_CODE);
    private static final XPathExpression exprDemoProviderType = XPathUtil.xpathCompile(XPATH_DEMO_PROVIDER_TYPE);
    private static final XPathExpression exprDemoClassification = XPathUtil.xpathCompile(XPATH_DEMO_CLASSIFICATION);
    private static final XPathExpression exprDemoSpecialization = XPathUtil.xpathCompile(XPATH_DEMO_SPECIALIZATION);
    private static final XPathExpression exprDemoService = XPathUtil.xpathCompile(XPATH_DEMO_SERVICE);
    private static final XPathExpression exprDemoOfficePhone = XPathUtil.xpathCompile(XPATH_DEMO_OFFICE_PHONE);
    private static final XPathExpression exprDemoAnalogPager = XPathUtil.xpathCompile(XPATH_DEMO_ANALOG_PAGER);
    private static final XPathExpression exprDemoFax = XPathUtil.xpathCompile(XPATH_DEMO_FAX);
    private static final XPathExpression exprDemoEmail = XPathUtil.xpathCompile(XPATH_DEMO_EMAIL);

    public PatientRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Patient[] match(String target) throws RPCException {
        String request = buildMatchRequest(target);
        String response = (String) cxn.query(request);
        return toMatches(response);
    }

    private String buildMatchRequest(String intarget) {
        String target = intarget;
        String rtc;
        int targetLength = target.length();
        if (targetLength < 2) {
            throw new IllegalArgumentException("Invalid search criteria");
        }
        VistaQuery vquery = vistaQueryFactory.getVistaQuery();

        // If first char is numeric, must be SSN
        if (StringUtils.isNumericChar(target.charAt(0))) {
            /*if (!SocSecNum.isWellFormed(target))
            {
                throw new RPCException("Invalid SSN");
            }*/
            StringBuilder sbld = new StringBuilder();
            sbld.append(target.charAt(0));
            for (int i = 1; i < targetLength; i++) {
                if (StringUtils.isNumericChar(target.charAt(i))) {
                    sbld.append(target.charAt(i));
                }
            }
            if (sbld.length() != 9) {
                throw new IllegalArgumentException("Invalid SSN");
            }
            vquery.setRpcName("ORWPT FULLSSN");
            vquery.addParameter(VistaQuery.LITERAL, sbld.toString());
            rtc = vquery.buildMessage();
        } else {
            // First char was not numeric.  Make sure it's alpha.
            target = target.toUpperCase(Locale.getDefault());
            if (!StringUtils.isAlphaChar(target.charAt(0))) {
                throw new IllegalArgumentException("Invalid search criteria");
            }
    
            // First char was alpha.  If second char is numeric, must be last5.
            if (StringUtils.isNumericChar(target.charAt(1))) {
                if (target.length() != 5) {
                    throw new IllegalArgumentException("Invalid last 5 identifier");
                }
                for (int i = 2; i < 5; i++) {
                    if (!StringUtils.isNumericChar(target.charAt(i))) {
                        throw new IllegalArgumentException("Invalid last 5 identifier");
                    }
                }
                vquery.setRpcName("ORWPT LAST5");
                vquery.addParameter(VistaQuery.LITERAL, target);
                rtc = vquery.buildMessage();
            } else {
        
                // Second char was not numeric.  Assume it's a preferredTerm.
                /*if (!PersonName.isValid(target))
                {
                    throw new RPCException("Invalid person name");
                }*/
                vquery.setRpcName("ORWPT LIST ALL");
                vquery.addParameter(VistaQuery.LITERAL, RPCUtils.adjustForNameSearch(target));
                vquery.addParameter(VistaQuery.LITERAL, "1");
                rtc = vquery.buildMessage();
            }
        }
        return rtc;
    }

    private Patient[] toMatches(String response) {
        String[] lines = StringUtils.split(response, StringUtils.CRLF);
        ArrayList<Patient> lst = new ArrayList<Patient>(lines.length);
        for (int i = 0; i < lines.length; i++) {
            if ("".equals(lines[i])) {
                continue;
            }
            Patient patient = toMatch(lines[i]);
            if (patient != null) {
                lst.add(patient);
            }
        }

//        for(Patient patient : lst)
//        {
//            patient.setPatientIdSiteCode((String) cxn.getSiteCode().getKey());
//        }

        return lst.toArray(new Patient[0]);
    }

    private Patient toMatch(String line) {
        Patient patient = new Patient();
        String[] fields = StringUtils.split(line, StringUtils.CARET);
        if (fields.length > 1) {
            patient.setName(fields[1]);
//            if (fields.length == 2) {
//                return patient;
//            } 
            if (fields.length != 2) {
                if (!"".equals(fields[2])) {
                    patient.setDob(Timestamp.toUtcString(fields[2]));
                }
                if (!"".equals(fields[3])) {
                    patient.setSSN(fields[3]);
                }
    
                //patient ien
                if (!StringUtils.isEmpty(fields[0])) {
                    Site site = cxn.getSite();
                    List<Site> vistaSites = new ArrayList<Site>();
                    vistaSites.add(site);
    
                    List<KeyValuePair<String, String>> patientIens = new ArrayList<KeyValuePair<String, String>>();
                    KeyValuePair<String, String> patIdKVP = new KeyValuePair<String, String>();
                    patIdKVP.setKey(cxn.getSite().getSiteCode());
                    patIdKVP.setValue(fields[0]);
                    patientIens.add(patIdKVP);
    
                    patient.setVistaSites(vistaSites);
                    patient.setPatientIens(patientIens);
                }
            }
        }
        return patient;
    }

    public Patient select(String patientIen) throws RPCException {
        return selectPlus(patientIen);
    }

    public PatientDemographics patientDemoSelect(String patientIen, User user) throws RPCException {
        if (StringUtils.isEmpty(patientIen) || !StringUtils.isNumeric(patientIen)) {
            throw new RPCException("Empty or non-numeric patientIen");
        }
        String request = buildSelectByRpcRequest(patientIen);
        String response = (String) cxn.query(request);
        PatientDemographics demographics = toPatientDemoSelectedByRPC(response/*, patientIen*/);

        demographics = getExtraDemoDetailsFromVPR(patientIen, demographics, user);
        return demographics;
    }


    public PatientDemographics patientDemoSelectVPR(String patientIen, User user) throws RPCException {

        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, Calendar.getInstance(), Calendar.getInstance(), VPRUtil.VPRType.DEMOGRAPHICS);
        String response = (String) cxn.query(request);

        PatientDemographics demographics = parseVPRDemographics(response);

//        Calendar startDate = Calendar.getInstance();
//        Calendar endDate = Calendar.getInstance();
        request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, VPRUtil.VPRType.FLAGS);
        response = (String) cxn.query(request);

        demographics = parseVPRFlags(response, demographics);

        return demographics;
    }


    public Boolean issueSensitivePatientBulletin(String patientIen) throws RPCException {
        Boolean rtc;
        String request = buildSensitivePatientBulletinRequest(patientIen);
        String response = (String) cxn.query(request);
        if ("1".equals(response)) {
            rtc = Boolean.TRUE;
        } else {
            rtc = Boolean.FALSE;
        }
        return rtc;
    }

    private String buildSensitivePatientBulletinRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery();
        vquery.setRpcName("DG SENSITIVE RECORD BULLETIN");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "1");

        return vquery.buildMessage();
    }

    //This call uses no DDR LISTER, just regular RPCs and GET VARIABLE VALUE.

    private Patient selectPlus(String patientIen) throws RPCException {
        Patient result = selectByRpc(patientIen);
        //SB 2/6/13: Janus will utilize VA MVI to retrieve and set patient locations
//        result.setLocations(getSiteIDs(result.getPatientId()));
        //SB: 2/7/13: Setting connection site as default location (so patient select does not break)
        result.setVistaSites(Arrays.asList(cxn.getSite()));

        if (result.isSensitive()) {
            boolean isIssueSecurityBulletin = false;

            try {
                isIssueSecurityBulletin = issueSensitivePatientBulletin(patientIen);
            } catch (RPCException e) {
                isIssueSecurityBulletin = false;
                LOGGER.error("An error occurred while issuing sensitive patient bulletin: " + e.getMessage());
            }

            if (!isIssueSecurityBulletin) {
                LOGGER.info("Failed to issue sensitive patient security bulletin at site: " + cxn.getSiteCode() + ".");
            }

        }

        return result;
    }
    //SB 2/6/13: Janus will utilize VA MVI to retrieve and set patient locations
//    private List<Site> getSiteIDs(String patientIen) throws RPCException
//    {
//        String request = buildGetRemoteSitesRequest(patientIen);
//        String response = (String) cxn.query(request);
//        if (response.startsWith("-1^"))
//        {
//            ArrayList<String> sites = new ArrayList<String>();
//            sites.add((String) cxn.getSiteCode().getKey());
//            return sites;
//        }
//        String[] lines = StringUtils.split(response, StringUtils.CRLF);
//        lines = StringUtils.trimArray(lines);
//        ArrayList<String> result = new ArrayList<String>();
//        result.add((String)cxn.getSiteCode().getKey());
//        for (int i = 0, j = 1; i < lines.length; i++, j++)
//        {
//            String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
//            //SB: do not include station 200 in patient site id list
//            if (flds[0].equals("200"))
//            {
//                continue;
//            }
//            result.add(flds[0]);
//        }
//        return result;
//    }

//    private String buildGetRemoteSitesRequest(String patientIen)
//    {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORWCIRN FACLIST");
//        vq.addParameter(vq.LITERAL, patientIen);
//        return vq.buildMessage();
//    }

    private Patient selectByRpc(String patientIen) throws RPCException {
        if (!StringUtils.isNumeric(patientIen)) {
            throw new RPCException("Non-numeric patientIen");
        }
        String request = buildSelectByRpcRequest(patientIen);
        String response = (String) cxn.query(request);
        return toPatientSelectedByRpc(response/*, patientIen*/);
    }

    private PatientDemographics toPatientDemoSelectedByRPC(String response/*, String patientIen*/) throws RPCException {
        if ("".equals(response)) {
            throw new RPCException("No such patient");
        }
        String[] fields = StringUtils.split(response, StringUtils.CARET);
        if (fields.length > 0 && "-1".equals(fields[0])) {
            throw new RPCException(fields[5]);
        }
        PatientDemographics patientDemo = new PatientDemographics();
        setPatSelectVals(fields, /*patientIen, */patientDemo);

        //set service connected
        if (fields.length > 12 && "1".equals(fields[11]) && !StringUtils.isEmpty(fields[12])) {
            patientDemo.setPercentServiceConnected(fields[12]);
        }

        addHomeData(patientDemo);
        addPCPData(patientDemo);
//        addPatientFlags(patientDemo);

        fillCommonDataBeanValues(patientDemo);

        return patientDemo;
    }

    private Patient toPatientSelectedByRpc(String response/*, String patientIen*/) throws RPCException {
        if ("".equals(response)) {
            throw new RPCException("No such patient");
        }
        String[] fields = StringUtils.split(response, StringUtils.CARET);
        Patient patient = new Patient();
        setPatSelectVals(fields, /*patientIen, */patient);

        return patient;
    }

    private void setPatSelectVals(String[] fields/*, String patientIen*/, Patient toPatient) throws RPCException {
        if (fields.length > 0 && "-1".equals(fields[0])) {
            throw new RPCException(fields[5]);
        }
        if (fields.length > 0) {
            toPatient.setName(fields[0]);
        }
        if (fields.length > 1) {
            toPatient.setGender(fields[1]);
        }
        if (fields.length > 2) {
            toPatient.setDob(Utils.formatDate(Utils.getDate(DateUtils.trimTime(Timestamp.toUtcString(fields[2])))));
        }
        if (fields.length > 3) {
            toPatient.setSSN(fields[3]);
        }
        if (fields.length > 7) {
            toPatient.setCWAD(fields[7]);
        }
//        toPatient.setPatientId(patientIen);
//        toPatient.setPatientIdSiteCode((String) cxn.getSiteCode().getKey());
        if (fields.length > 13) {
            toPatient.setICN(fields[13]);
        }
        if (fields.length > 14 && !StringUtils.isEmpty(fields[14])) {
            toPatient.setAge(fields[14]);
        }

        if (fields.length > 8 && "1".equals(fields[8])) {
            toPatient.setSensitive(true);
        }

        /*if (!fields[4].equals("") && !fields[5].equals(""))
        {
            HospitalLocation hl = new HospitalLocation(fields[4], fields[5]);
            hl.Room = fields[6];
            if (fields[15] != "")
            {
                hl.Specialty = new KeyValuePair<String, String>(fields[15], "");
            }
            patient.Location = hl;
            patient.IsInpatient = true;
        }
        patient.Cwad = fields[7];
        patient.IsRestricted = (fields[8] == "1");
        if (fields[9] != "")
        {
            patient.AdmitTimestamp = VistaTimestamp.toUtcString(fields[9]);
        }
        patient.IsServiceConnected = (fields[11] == "1");
        if (fields[12] != "")
        {
            patient.ScPercent = Convert.ToInt16(fields[12]);
        }*/
    }

    private String buildSelectByRpcRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT SELECT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

//    public Patient selectBySSN(String ssn) throws RPCException {
//        if (ssn.equals(""))
//        {
//            throw new IllegalArgumentException("Invalid SSN");
//        }
//        Patient[] matches = match(ssn);
//        if (matches == null || matches.length == 0)
//        {
//            return null;
//        }
//        if (matches.length > 1)
//        {
//            throw new RPCException("Multiple SSNs!");
//        }
//        return select(matches[0].getPatientId());
//    }

    public String getLocalPid(String mpiPID) throws RPCException {
        String arg = "$O(^DPT(\"AICN\"," + mpiPID + ",0))";
        String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        return response;
    }

    public PatientDemographics getPatientDemographics(String patientIen, User user) throws RPCException {
        return patientDemoSelect(patientIen, user);
    }

    public PatientDemographics getPatientDemographicsVPR(String patientIen, User user) throws RPCException {
        return patientDemoSelectVPR(patientIen, user);
    }


    private void addHomeData(PatientDemographics patientDemo) {
        try {
            String arg = "$G(^DPT(" + cxn.getPatientIen() + ",.11))";
            String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            if ("".equals(response)) {
                return;
            }
            String[] flds = StringUtils.split(response, StringUtils.CARET);

            Address homeAddress = new Address();

            homeAddress.setType("HOME");
            if (flds.length > 0) {
                homeAddress.setStreet1(flds[0]);
            }
            if (flds.length > 1) {
                homeAddress.setStreet2(flds[1]);
            }
            if (flds.length > 2) {
                homeAddress.setCity(flds[3]);
            }
            if (flds.length > 3 && !"".equals(flds[4])) {
                String stateName = getStateName(flds[4]);

                if (!StringUtils.isEmpty(stateName)) {
                    homeAddress.setStateProvince(stateName);
                }
            }

            if (flds.length > 4) {
                homeAddress.setPostalCode(flds[5]);
            }

            List<Address> addressList = new ArrayList<Address>();
            addressList.add(homeAddress);
            patientDemo.setAddress(addressList);

            String[] phones = getPersonalPhones(cxn.getPatientIen());

            if (phones != null) {
                if (phones.length > 0 && !"".equals(phones[0])) {
                    patientDemo.setPhone1(phones[0]);
                }

                if (phones.length > 1 && !"".equals(phones[1])) {
                    patientDemo.setPhone2(phones[1]);
                }
            }
        } catch (RPCException e) {
            //SB: error pulling patient home data, data my not be empty, log as a warning
            LOGGER.warn(e.getMessage(), e);
        }
    }

    private String getStateName(String stateIen) throws RPCException {
        String rtc;
        if (!StringUtils.isNumeric(stateIen)) {
            rtc = "";
        } else {
            String arg = "$P($G(^DIC(5," + stateIen + ",0)),U,1)";
    
            rtc = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        }
        return rtc;
    }

    private String[] getPersonalPhones(String patientIen) throws RPCException {
        String[] rtc;
        if (!StringUtils.isNumeric(patientIen)) {
            rtc = null;
        } else {
            String arg = "$G(^DPT(" + patientIen + ",.13))";
            String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            if ("".equals(response)) {
                rtc = null;
            } else {
                String[] flds = StringUtils.split(response, StringUtils.CARET);
                String[] phoneNumbers = new String[3];
        
                //home phone
                if (flds.length > 0) {
                    phoneNumbers[0] = flds[0];
                }
                //work phone
                if (flds.length > 1) {
                    phoneNumbers[1] = flds[1];
                }
                //cell phone
                if (flds.length > 3) {
                    phoneNumbers[2] = flds[3];
                }
        
        
                rtc = phoneNumbers;
            }
        }
        return rtc;
    }

    private void addPCPData(PatientDemographics patientDemo) {
        try {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT1 PRCARE");
            vquery.addParameter(VistaQuery.LITERAL, cxn.getPatientIen());
            String response = (String) cxn.query(vquery.buildMessage());
            if (StringUtils.isEmpty(response) || "^^".equals(response) || "^^^".equals(response)) {
                return;
            }

            String[] flds = StringUtils.split(response, StringUtils.CARET);

            if (flds.length > 0) {
                patientDemo.setClinic(flds[0]);
            }
            if (flds.length > 1) {
                patientDemo.setPrimaryProvider(flds[1]);
            }
        } catch (RPCException e) {
            //SB: error pulling patient pcp data, data my not be empty, log as a warning
            LOGGER.warn(e.getMessage(), e);
        }
    }

//    private void addPatientFlags(PatientDemographics patientDemo) throws RPCException {
//        Map<String, String> patFlags = getPatientFlags(cxn.getPatientIen());
//
//        addPatientFlags(patientDemo, patFlags);
//    }

    private void addPatientFlags(PatientDemographics patientDemo, Map<String, String> patFlags) {
        if (patFlags != null) {
    
            Set<Map.Entry<String, String>> entries = patFlags.entrySet();
    
            int count = 0;
    
            Iterator entryIterator = entries.iterator();
    
            while (entryIterator.hasNext()) {
                Map.Entry next = (Map.Entry) entryIterator.next();
    
                if (patientDemo.getCodeGreen() == null) {
                    patientDemo.setCodeGreen("");
                }
    
                if (count > 0) {
                    patientDemo.setCodeGreen(patientDemo.getCodeGreen() + "\r\n\r\n");
                }
    
                patientDemo.setCodeGreen(patientDemo.getCodeGreen() + (String) next.getValue());
                count++;
            }
        }
    }


//    private Map<String, String> getPatientFlags(String patientIen) throws RPCException {
//        String request = buildGetHasPatientFlagsRequest(patientIen);
//        String response = (String) cxn.query(request);
//        Map<String, String> patFlags = toPatientFlags(response);
//
//        if (patFlags == null) return null;
//
//        Set<Map.Entry<String, String>> entries = patFlags.entrySet();
//        Map<String, String> flags = new LinkedHashMap<String, String>();
//        for (Map.Entry<String, String> entry : entries) {
//            String flagQuery = buildGetPatientFlagRequest(patientIen, entry.getKey());
//            String flagResp = (String) cxn.query(flagQuery);
//            flags.put(entry.getKey(), flagResp);
//        }
//
//        return flags;
//    }

//    private String buildGetHasPatientFlagsRequest(String patientIen) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORPRF HASFLG");
//        vq.addParameter(vq.LITERAL, patientIen);
//        return vq.buildMessage();
//    }

//    private String buildGetPatientFlagRequest(String patientIen, String flagRecordId) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORPRF GETFLG");
//        vq.addParameter(vq.LITERAL, patientIen);
//        vq.addParameter(vq.LITERAL, flagRecordId);
//        return vq.buildMessage();
//    }

//    private Map<String, String> toPatientFlags(String response) {
//        if (response.equals("")) {
//            return null;    //Error?
//        }
//        String[] lines = StringUtils.split(response, StringUtils.CRLF);
//        Map<String, String> result = new LinkedHashMap<String, String>();
//        for (int i = 0; i < lines.length; i++) {
//            if (lines[i].equals("")) {
//                continue;
//            }
//            String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
//            result.put(flds[0], flds[1]);
//        }
//        return result;
//    }

    public PatientDemographics getPatientDemographics(User user) throws RPCException {
        return getPatientDemographics(cxn.getPatientIen(), user);
    }

    public PatientDemographics getPatientDemographicsVPR(User user) throws RPCException {
        return getPatientDemographicsVPR(cxn.getPatientIen(), user);
    }


    public PatientDemographicsDetail getPatientDemographicsDetail(String patientIen) throws RPCException {
        String query = buildDemographicDetailsQuery(patientIen);
        String response = (String) cxn.query(query);
        PatientDemographicsDetail demoDetails = new PatientDemographicsDetail();
        demoDetails.setReport(response);
        fillCommonDataBeanValues(demoDetails);
        return demoDetails;
    }

    public PatientDemographicsDetail getPatientDemographicsDetail() throws RPCException {
        return getPatientDemographicsDetail(cxn.getPatientIen());
    }

    private String buildDemographicDetailsQuery(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT PTINQ");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    public Alert getSensitivePatientAlert() throws RPCException {
        String query = buildSensitivePatientQuery(cxn.getPatientIen());
        String response = (String) cxn.query(query);
        String[] lines = StringUtils.split(response, StringUtils.CRLF);
        Alert sensitiveAlert = new Alert();
        fillCommonDataBeanValues(sensitiveAlert);

        sensitiveAlert.setAlertId(lines[0].trim());
        if (lines.length > 1) {
            StringBuilder alertMessageSB = new StringBuilder();
            for (int i = 1; i < lines.length; i ++) {
                alertMessageSB.append(lines[i]);
                alertMessageSB.append("\n");
            }
            sensitiveAlert.setMessageText(alertMessageSB.toString().trim());
        }

//        -1-RPC/API failed
//        Required variable not defined
//        0-No display/action required
//        Not an employee, not sensitive or not accessing own Patient
//        record
//        1-Display warning message
//        Sensitive - inpatient or a DG SENSITIVITY key holder
//        or Employee and DG SECURITY OFFICER key holder
//        2-Display warning message, require OK to continue and call
//        DG SENSITIVE RECORD BULLETIN RPC to update DG Security Log
//        file and generate Sensitive Record Access mail message.
//        Sensitive - not an inpatient and not a key holder
//        or Employee/not a DG SECURITY OFFICER key holder
//        3-Access to record denied
//        Accessing own Patient file record
//        4-Access to Patient file (#2) records denied
//        SSN not defined

        //return true if response is greater than 0
        return sensitiveAlert;
    }

    private String buildSensitivePatientQuery(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("DG SENSITIVE RECORD ACCESS");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    private PatientDemographics getExtraDemoDetailsFromVPR(String patientIen, PatientDemographics demographics, User user) throws RPCException {
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, VistaConstants.VPR_CONTEXT, true);

        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DEMOGRAPHICS);
        String response = (String) cxn.query(request);

//        PatientDemographics returnDemographics = parseVPRDemographics(response, demographics);
//        return returnDemographics;
        return  parseVPRDemographics(response, demographics);
    }

    private PatientDemographics parseVPRFlags(String response, PatientDemographics demographics) throws RPCException {
        PatientDemographics rtc;
        if ("".equals(response)) {
            rtc = demographics;
        } else {
    
            Document doc = null;
    
            try {
                doc = parseXMLDocument(response);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList flagsNodeList = null;
            try {
                flagsNodeList = (NodeList) exprFlag.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }

            ParserBatchManager parserBatchManager = new ParserBatchManager(PARSER_BATCH_SIZE_PATIENT_FLAGS);
            int listLength = flagsNodeList.getLength();
            for (int flagNum = 0; flagNum < listLength; flagNum++) {
                parserBatchManager.addNode(flagsNodeList.item(flagNum));
            }

            List<VPRFlagParser> flagParserList = new ArrayList<VPRFlagParser>();
            for(List<Node> parserBatch : parserBatchManager.getParserBatchList()) {
                flagParserList.add(new VPRFlagParser(parserBatch));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<List<Flag>>> futures = null;
            try {
                futures = executorService.invokeAll(flagParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
    
            } catch (InterruptedException e) {
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }
    
            Iterator<Future<List<Flag>>> futuresItr = futures.iterator();
            List<Flag> flagsList = new ArrayList<Flag>();
            while (futuresItr.hasNext()) {
                try {
                    Future<List<Flag>> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Flag Data");
                    } else {
                        List<Flag> flgs = future.get();
                        for(Flag flg : flgs) {
                            if (flg != null && !StringUtils.isEmpty(flg.getName())) {
                                if(demographics.getSite() != null){
                                    flg.setSite(demographics.getSite());
                                }
                                flagsList.add(flg);
                            }
                        }
                    }
    
                } catch (InterruptedException e) {
                    LOGGER.error(e.getMessage(), e);
                } catch (ExecutionException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
    
            if (!flagsList.isEmpty()) {
                demographics.setFlags(flagsList.toArray(new Flag[0]));
            }
    
            rtc = demographics;
        }
        return rtc;
    }

    private static class VPRFlagParser implements Callable<List<Flag>> {
        private List<Node> flagNodeList;

        public VPRFlagParser(List<Node> flagNodeList) {
            this.flagNodeList = flagNodeList;
        }


        public List<Flag> call() throws RPCException {

            List<Flag> flags = new ArrayList<Flag>();

            for(Node flagNode : flagNodeList) {
                Flag flag = new Flag();

                String category = XPathUtil.evaluateNodeToString(flagNode, exprFlagCategory, LOGGER);
                String content = XPathUtil.evaluateNodeToString(flagNode, exprFlagContent, LOGGER);
                String id = XPathUtil.evaluateNodeToString(flagNode, exprFlagId, LOGGER);
                String name = XPathUtil.evaluateNodeToString(flagNode, exprFlagName, LOGGER);
                String origSite = XPathUtil.evaluateNodeToString(flagNode, exprFlagOrigSite, LOGGER);
                String ownSite = XPathUtil.evaluateNodeToString(flagNode, exprFlagOwnSite, LOGGER);
                String reviewDate = XPathUtil.evaluateNodeToString(flagNode, exprFlagReviewDate, LOGGER);
                String type = XPathUtil.evaluateNodeToString(flagNode, exprFlagType, LOGGER);
                String approvedBy = XPathUtil.evaluateNodeToString(flagNode, exprFlagApprovedBy, LOGGER);
                String assigned = XPathUtil.evaluateNodeToString(flagNode, exprFlagAssigned, LOGGER);
                String documentCode = XPathUtil.evaluateNodeToString(flagNode, exprFlagDocumentCode, LOGGER);
                String documentName = XPathUtil.evaluateNodeToString(flagNode, exprFlagDocumentName, LOGGER);

                flag.setCategory(category);
                flag.setContent(content);
                flag.setId(id);
                flag.setName(name);
                flag.setOriginalSite(origSite);
                flag.setOwnerSite(ownSite);
                flag.setType(type);
                flag.setApprovedBy(approvedBy);
                flag.setDocumentName(documentName);
                flag.setDocumentId(documentCode);

                if (reviewDate != null) {
                    Calendar reviewDateCal = Timestamp.toDateTime(reviewDate);
                    flag.setReviewDue(reviewDateCal);
                }
                if (assigned != null) {
                    Calendar assignedCal = Timestamp.toDateTime(assigned);
                    flag.setAssigned(assignedCal);
                }

                flags.add(flag);
            }

            return flags;
        }
    }

    private PatientDemographics parseVPRDemographics(String response) throws RPCException {
        PatientDemographics demographics = new PatientDemographics();
        fillCommonDataBeanValues(demographics);
        return parseVPRDemographics(response, demographics);
    }

    private PatientDemographics parseVPRDemographics(String response, PatientDemographics demographics) throws RPCException {
        PatientDemographics rtc;
        if (!"".equals(response)) {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

            Document document = null;
            try {
                document = parseXMLDocument(sbld.toString());
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            Node patientNode = XPathUtil.evaluateNodeToNode(document, exprDemoPatient, LOGGER);
            if (patientNode != null) {
                Node racesNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoRaces, LOGGER);
                if (racesNode != null) {
                    NodeList raceNodeList = XPathUtil.evaluateNodeToNodeList(racesNode, exprDemoRace, LOGGER);
                    if (raceNodeList != null) {
                        int raceNodeListLength = raceNodeList.getLength();
                        List<Code> raceList = new ArrayList<Code>();
                        for (int raceNum = 0; raceNum < raceNodeListLength; raceNum++) {
                            Node raceNode = raceNodeList.item(raceNum);
                            raceNode.getParentNode().removeChild(raceNode);

                            Code race = new Code();
                            race.setSystem("CDC");

                            String raceCode = XPathUtil.evaluateNodeToString(raceNode, exprDemoValue, LOGGER);
                            race.setCode(raceCode);
                            raceList.add(race);
                        }

                        if (!raceList.isEmpty()) {
                            demographics.setRace(raceList.toArray(new Code[0]));
                        }
                    }
                }

                Node ethnicitiesNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoEthnicities, LOGGER);
                if (ethnicitiesNode != null) {
                    NodeList ethnicityNodeList = XPathUtil.evaluateNodeToNodeList(ethnicitiesNode, exprDemoEthnicity, LOGGER);
                    if (ethnicityNodeList != null) {
                        int ethnicityNodeListLength = ethnicityNodeList.getLength();
                        List<Code> ethnicityList = new ArrayList<Code>();
                        for (int ethnicityNum = 0; ethnicityNum < ethnicityNodeListLength; ethnicityNum++) {
                            Node ethnicityNode = ethnicityNodeList.item(ethnicityNum);
                            ethnicityNode.getParentNode().removeChild(ethnicityNode);

                            Code ethnicity = new Code();
                            ethnicity.setSystem("CDC");

                            String ethnicityCode = XPathUtil.evaluateNodeToString(ethnicityNode, exprDemoValue, LOGGER);
                            ethnicity.setCode(ethnicityCode);
                            ethnicityList.add(ethnicity);
                        }

                        if (!ethnicityList.isEmpty()) {
                            demographics.setEthnicity(ethnicityList.toArray(new Code[0]));
                        }
                    }
                }

                String fullName = XPathUtil.evaluateNodeToString(patientNode, exprDemoFullName, LOGGER);
                demographics.setName(fullName);
                String gender = XPathUtil.evaluateNodeToString(patientNode, exprDemoGender, LOGGER);
                demographics.setGender(gender);
                String dobString = XPathUtil.evaluateNodeToString(patientNode, exprDemoDob, LOGGER);
                dobString = Utils.formatDate(Utils.getDate(DateUtils.trimTime(Timestamp.toUtcString(dobString))));
                demographics.setDob(dobString);
                String ssnString = XPathUtil.evaluateNodeToString(patientNode, exprDemoSsn, LOGGER);
                demographics.setSSN(ssnString);
                String id = XPathUtil.evaluateNodeToString(patientNode, exprDemoId, LOGGER);
                demographics.setICN(id);
                String sensitive = XPathUtil.evaluateNodeToString(patientNode, exprDemoSensitive, LOGGER);
                if (sensitive != null && "1".equals(sensitive)) {
                    demographics.setSensitive(true);
                }
                String scPercent = XPathUtil.evaluateNodeToString(patientNode, exprDemoScPercent, LOGGER);
                demographics.setPercentServiceConnected(scPercent);

                Node addressNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoAddress, LOGGER);
                if (addressNode != null) {
                    String streetLine1 = XPathUtil.evaluateNodeToString(addressNode, exprDemoStreetLine1, LOGGER);
                    String streetLine2 = XPathUtil.evaluateNodeToString(addressNode, exprDemoStreetLine2, LOGGER);
                    String city = XPathUtil.evaluateNodeToString(addressNode, exprDemoCity, LOGGER);
                    String stateProvince = XPathUtil.evaluateNodeToString(addressNode, exprDemoStateProvince, LOGGER);
                    String postalCode = XPathUtil.evaluateNodeToString(addressNode, exprDemoPostalCode, LOGGER);

                    Address address = new Address();
                    address.setStreet1(streetLine1);
                    address.setStreet2(streetLine2);
                    address.setCity(city);
                    address.setStateProvince(stateProvince);
                    address.setPostalCode(postalCode);

                    List<Address> addressList = new ArrayList<Address>();
                    addressList.add(address);
                    demographics.setAddress(addressList);
                }

                Node telecomListNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoTelecomList, LOGGER);
                if (telecomListNode != null) {
                    NodeList telecomNodeList = XPathUtil.evaluateNodeToNodeList(telecomListNode, exprDemoTelecom, LOGGER);
                    if (telecomNodeList != null) {
                        int telecomNodeListLength = telecomNodeList.getLength();
                        List<String> phoneStringList = new ArrayList<String>();
                        for (int telecomNum = 0; telecomNum < telecomNodeListLength; telecomNum++) {
                            Node telecomNode = telecomNodeList.item(telecomNum);
                            telecomNode.getParentNode().removeChild(telecomNode);

                            String telecomString = XPathUtil.evaluateNodeToString(telecomNode, exprDemoValue, LOGGER);

                            phoneStringList.add(telecomString);
                        }

                        if (!phoneStringList.isEmpty()) {
                            demographics.setPhone1(phoneStringList.get(0));
                            if (phoneStringList.size() > 1) {
                                demographics.setPhone2(phoneStringList.get(1));
                            }
                        }
                    }
                }

                Node flagsNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoFlags, LOGGER);
                if (flagsNode != null) {
                    NodeList flagNodeList = XPathUtil.evaluateNodeToNodeList(flagsNode, exprDemoFlag, LOGGER);
                    if (flagNodeList != null) {
                        int flagNodeListLength = flagNodeList.getLength();
                        Map<String, String> flags = new LinkedHashMap<String, String>();
                        for (int flagNum = 0; flagNum < flagNodeListLength; flagNum++) {
                            Node flagNode = flagNodeList.item(flagNum);
                            flagNode.getParentNode().removeChild(flagNode);

                            String flagName = XPathUtil.evaluateNodeToString(flagNode, exprDemoName, LOGGER);
                            String flagText = XPathUtil.evaluateNodeToString(flagNode, exprDemoText, LOGGER);

                            StringBuilder flagSB = new StringBuilder("Alert/Flag: ");
                            flagSB.append(flagName);
                            flagSB.append("\r\n");
                            flagSB.append(flagText);
                            flags.put(flagName, flagSB.toString());
                        }

                        if (!flags.isEmpty()) {
                            addPatientFlags(demographics, flags);
                        }
                    }
                }

                Node eligibilitiesNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoEligibilities, LOGGER);
                if (eligibilitiesNode != null) {
                    NodeList eligibilitiesNodeList = XPathUtil.evaluateNodeToNodeList(eligibilitiesNode, exprDemoEligibility, LOGGER);
                    if (eligibilitiesNodeList != null) {
                        int eligibilitiesNodeListLength = eligibilitiesNodeList.getLength();
                        for (int i = 0; i < eligibilitiesNodeListLength; i++) {
                            Node eligibilityNode = eligibilitiesNodeList.item(i);
                            eligibilityNode.getParentNode().removeChild(eligibilityNode);

                            String eligibilityPrimary = XPathUtil.evaluateNodeToString(eligibilityNode, exprDemoPrimary, LOGGER);

                            if (eligibilityPrimary != null && "1".equals(eligibilityPrimary)) {
                                String eligibilityName = XPathUtil.evaluateNodeToString(eligibilityNode, exprDemoName, LOGGER);
                                String eligibilityStatus = XPathUtil.evaluateNodeToString(patientNode, exprDemoEligibilityStatus, LOGGER);
                                demographics.setEligibility(eligibilityName);
                                demographics.setEligibilityStatus(eligibilityStatus);
                                break;
                            }
                        }
                    }
                }
                String pcTeamName = XPathUtil.evaluateNodeToString(patientNode, exprDemoPcTeamName, LOGGER);
                demographics.setPcTeam(pcTeamName);
                String pcAssignedString = XPathUtil.evaluateNodeToString(patientNode, exprDemoPcAssigned, LOGGER);
                Calendar pcAssigned = null;
                if (!StringUtils.isEmpty(pcAssignedString)) {
                    pcAssigned = Timestamp.toDateTime(pcAssignedString);
                    demographics.setPcAssigned(pcAssigned);
                    demographics.setPcmDates(pcAssigned.getTime());
                }

                Node pcProviderNode = XPathUtil.evaluateNodeToNode(patientNode, exprDemoPcProvider, LOGGER);
                if (pcProviderNode != null) {
                    Provider pcProvider = new Provider();

                    String pcProviderCode = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoCode, LOGGER);
                    String pcProviderName = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoName, LOGGER);
                    String pcProviderTaxonomyCode = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoTaxonomyCode, LOGGER);
                    String pcProviderProviderType = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoProviderType, LOGGER);
                    String pcProviderClassification = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoClassification, LOGGER);
                    String pcProviderSpecialization = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoSpecialization, LOGGER);
                    String pcProviderService = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoService, LOGGER);
                    String pcProviderOfficePhone = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoOfficePhone, LOGGER);
                    String pcProviderAnalogPager = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoAnalogPager, LOGGER);
                    String pcProviderFax = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoFax, LOGGER);
                    String pcProviderEmail = XPathUtil.evaluateNodeToString(pcProviderNode, exprDemoEmail, LOGGER);

                    Node pcProviderAddressNode = XPathUtil.evaluateNodeToNode(pcProviderNode, exprDemoAddress, LOGGER);
                    if (pcProviderAddressNode != null) {
                        String pcProviderStreetLine1 = XPathUtil.evaluateNodeToString(pcProviderAddressNode, exprDemoStreetLine1, LOGGER);
                        String pcProviderStreetLine2 = XPathUtil.evaluateNodeToString(pcProviderAddressNode, exprDemoStreetLine2, LOGGER);
                        String pcProviderStreetLine3 = XPathUtil.evaluateNodeToString(pcProviderAddressNode, exprDemoStreetLine3, LOGGER);
                        String pcProviderCity = XPathUtil.evaluateNodeToString(pcProviderAddressNode, exprDemoCity, LOGGER);
                        String pcProviderStateProvince = XPathUtil.evaluateNodeToString(pcProviderAddressNode, exprDemoStateProvince, LOGGER);
                        String pcProviderPostalCode = XPathUtil.evaluateNodeToString(pcProviderAddressNode, exprDemoPostalCode, LOGGER);

                        pcProvider.setStreet1(pcProviderStreetLine1);
                        pcProvider.setStreet2(pcProviderStreetLine2);
                        pcProvider.setStreet3(pcProviderStreetLine3);
                        pcProvider.setCity(pcProviderCity);
                        pcProvider.setStateProvince(pcProviderStateProvince);
                        pcProvider.setPostalCode(pcProviderPostalCode);
                    }
                    pcProvider.setCode(pcProviderCode);
                    pcProvider.setName(pcProviderName);
                    demographics.setPrimaryProvider(pcProviderName);
                    pcProvider.setTaxonomyCode(pcProviderTaxonomyCode);
                    pcProvider.setProviderType(pcProviderProviderType);
                    pcProvider.setClassification(pcProviderClassification);
                    pcProvider.setSpecialization(pcProviderSpecialization);
                    pcProvider.setService(pcProviderService);
                    pcProvider.setPhone(pcProviderOfficePhone);
                    demographics.setPcmPhone(pcProviderOfficePhone);
                    pcProvider.setAnalogPager(pcProviderAnalogPager);
                    pcProvider.setFax(pcProviderFax);
                    pcProvider.setEmail(pcProviderEmail);

                    demographics.setPcProvider(pcProvider);
                }
            }
        }
        rtc = demographics;
        
        return rtc;
    }

}
