/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.*;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.XPathUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.Problem;
import gov.va.med.vds.vistabeans.ProblemDetail;
import gov.va.med.vds.vistabeans.ProblemNote;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;


/**
 *
 *
 */
public class ProblemsRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProblemsRPC.class);

    private static final String XPATH_PROBLEM = "//problem";
    private static final String XPATH_ACUITY_NAME = "./acuity/@name";
    private static final String XPATH_ENTERED = "./entered/@value";
    private static final String XPATH_ONSET = "./onset/@value";
    private static final String XPATH_LOCATION = "./location/@value";
    private static final String XPATH_ICD = "./icd/@value";
    private static final String XPATH_ID = "./id/@value";
    private static final String XPATH_NAME = "./name/@value";
    private static final String XPATH_PROVIDER_CODE = "./provider/@code";
    private static final String XPATH_SC = "./sc/@value";
    private static final String XPATH_STATUS_NAME = "./status/@name";
    private static final String XPATH_UPDATED = "./updated/@value";
    private static final String XPATH_COMMENTS = "./comments";
    private static final String XPATH_COMMENT_NODE = "./comment";
    private static final String XPATH_COMMENT_ENTERED = "@entered";
    private static final String XPATH_COMMENT_ENTERED_BY = "@enteredBy";
    private static final String XPATH_COMMENT_TEXT = "@commentText";
    private static final String XPATH_CODING_SYSTEM = "./codingSystem/@value";
    private static final String XPATH_SCTC = "./sctc/@value";
    private static final String XPATH_SCTT = "./sctt/@value";
    private static final String XPATH_ICDD = "./icdd/@value";
    
    private static XPathExpression exprProblem = XPathUtil.xpathCompile(XPATH_PROBLEM);
    private static XPathExpression exprAcuityName = XPathUtil.xpathCompile(XPATH_ACUITY_NAME);
    private static XPathExpression exprEntered = XPathUtil.xpathCompile(XPATH_ENTERED);
    private static XPathExpression exprOnset = XPathUtil.xpathCompile(XPATH_ONSET);
    private static XPathExpression exprLocation = XPathUtil.xpathCompile(XPATH_LOCATION);
    private static XPathExpression exprIcd = XPathUtil.xpathCompile(XPATH_ICD);
    private static XPathExpression exprId = XPathUtil.xpathCompile(XPATH_ID);
    private static XPathExpression exprName = XPathUtil.xpathCompile(XPATH_NAME);
    private static XPathExpression exprProviderCode = XPathUtil.xpathCompile(XPATH_PROVIDER_CODE);
    private static XPathExpression exprSc = XPathUtil.xpathCompile(XPATH_SC);
    private static XPathExpression exprStatusName = XPathUtil.xpathCompile(XPATH_STATUS_NAME);
    private static XPathExpression exprUpdated = XPathUtil.xpathCompile(XPATH_UPDATED);
    private static XPathExpression exprComments = XPathUtil.xpathCompile(XPATH_COMMENTS);
    private static XPathExpression exprCommentNode = XPathUtil.xpathCompile(XPATH_COMMENT_NODE);
    private static XPathExpression exprCommentEntered = XPathUtil.xpathCompile(XPATH_COMMENT_ENTERED);
    private static XPathExpression exprCommentEnteredBy = XPathUtil.xpathCompile(XPATH_COMMENT_ENTERED_BY);
    private static XPathExpression exprCommentText = XPathUtil.xpathCompile(XPATH_COMMENT_TEXT);
    private static XPathExpression exprCodingSystem = XPathUtil.xpathCompile(XPATH_CODING_SYSTEM);
    private static XPathExpression exprSctc = XPathUtil.xpathCompile(XPATH_SCTC);
    private static XPathExpression exprSctt = XPathUtil.xpathCompile(XPATH_SCTT);
    private static XPathExpression exprIcdd = XPathUtil.xpathCompile(XPATH_ICDD);

    public ProblemsRPC(RPCConnection cxn) {
        super(cxn);
    }

    //SB: Missing acuity, icdCode, consetDate, also status does not map
    public Problem[] getProblemList(String patientIen, String type) throws RPCException {

        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQPL PROBLEM LIST");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        String request = vquery.buildMessage();

        String response = (String) cxn.query(request);

        return toProblemList(response/*, patientIen*/);
    }

    public Problem[] getProblemList(String type) throws RPCException {
        return getProblemList(cxn.getPatientIen(), type);
    }

//    private String buildGetProblemListRequest(String patientIen, String type) throws RPCException
//    {
//        String arg = "";
//        if (type.equals("ACTIVE"))
//        {
//            arg = "OR_PLA:ACTIVE PROBLEMS~PLAILA;ORDV04;59;";
//        }
//        else if (type.equals("INACTIVE"))
//        {
//            arg = "OR_PLI:INACTIVE PROBLEMS~PLAILI;ORDV04;60;";
//        }
//        else if (type.equals("ALL"))
//        {
//            arg = "OR_DODPLL:ALL PROBLEM LIST~PLAILALL;ORDV04;61;";
//        }
//        else
//        {
//            throw new RPCException("Invalid type request: " + type + ".  Must be 'ACTIVE', 'INACTIVE', or 'ALL'.");
//        }
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
//        vq.addParameter(vq.LITERAL, patientIen);
//        vq.addParameter(vq.LITERAL, arg + '0');
//        vq.addParameter(vq.LITERAL, "");
//        vq.addParameter(vq.LITERAL, "");
//        vq.addParameter(vq.LITERAL, "");
//
//        return vq.buildMessage();
//    }

    private Problem[] toProblemList(String response/*, String patientIen*/) {
        final int PROBLEM_IEN = 0;
        final int PROBLEM_STATUS = 1;
        final int PROBLEM_DESCRIPTION = 2;
        final int PROBLEM_ICD_CODE = 3;
        final int PROBLEM_ONSET_DATE = 4;
        final int PROBLEM_LAST_MODIFIED_DATE = 5;
        final int PROBLEM_SERVICE_CONNECTION_STATUS = 6;
        final int PROBLEM_SPECIAL_EXPOSURE = 7;
        final int PROBLEM_CONDITION = 8;
        final int PROBLEM_LOCATION = 9;
        final int PROBLEM_LOCATION_TYPE = 10;
        final int PROBLEM_PROVIDER = 11;
        final int PROBLEM_SERVICE_IEN = 12;
        final int PROBLEM_PRIORITY = 13;
        final int PROBLEM_HAS_COMMENT = 14;
        final int PROBLEM_DATE_RECORDED = 15;
        final int PROBLEM_SC_CONDITIONS = 16;
        final int PROBLEM_INACTIVE_ICD_CODE = 17;

        Problem[] rtc;
        if ("".equals(response)) {
            rtc = new Problem[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<Problem> problemsList = new ArrayList<Problem>();

            // first line is the number of problems in the list
            int numProblems = Integer.parseInt(lines[0]);

            // No data available
            if (numProblems == 0) {
                rtc = new Problem[0];
                // TODO (SHI): Should this be a 1 item array with problem Description as "     No data available.                               |"
            } else {

                // roll through each problem in the response and add them to the Problems array
                for (int i = 1; i <= numProblems; i++) {
                    // separate out the fields in each problem line
                    String[] fields = StringUtils.split(lines[i], StringUtils.CARET);

                    Problem problem = new Problem();
                    fillCommonDataBeanValues(problem);

                    for (int j = 0; j < fields.length; j++) {
                        String field = fields[j];

                        // skip if field is null or empty
                        if (StringUtils.isEmpty(field)) {
                            continue;
                        }

                        // Put the data from the field into problem bean
                        switch (j) {
                            case PROBLEM_IEN:
                                problem.setId(field);
                                break;
                            case PROBLEM_STATUS:
                                problem.setStatus(field);
                                break;
                            case PROBLEM_DESCRIPTION:
                                problem.setDescription(field);
                                break;
                            case PROBLEM_ICD_CODE:
                                problem.setIcdCode(field);
                                Code icdCode = new Code();
                                icdCode.setCode(field);
                                icdCode.setSystem("ICD9");
                                Code[] codeArray = new Code[]{icdCode};
                                problem.setCodes(codeArray);
                                break;
                            case PROBLEM_ONSET_DATE:
                                problem.setOnsetDate(field);
                                break;
                            case PROBLEM_LAST_MODIFIED_DATE:
                                problem.setLastModifiedDate(Timestamp.toDateTime(field));
                                break;
                            case PROBLEM_SERVICE_CONNECTION_STATUS:
                                String connected = "";
                                if ("NSC".equals(field)) {
                                    connected = "NO";
                                } else if ("SC".equals(field)) {
                                    connected = "YES";
                                }

                                problem.setServiceConnected(connected);
                                break;
                            case PROBLEM_SPECIAL_EXPOSURE:
                                problem.setSpecialExposure(field);
                                break;
                            case PROBLEM_CONDITION:
                                problem.setCondition(field);
                                break;
                            case PROBLEM_LOCATION:
                                String[] locSubFields = StringUtils.split(field, StringUtils.SEMICOLON);
                                for (int k = 0; k < locSubFields.length; k++) {
                                    String locSubField = locSubFields[k];

                                    if (StringUtils.isEmpty(locSubField)) {
                                        continue;
                                    }

                                    if (k == 0) {
                                        // Location IEN
                                        problem.setLocationIEN(locSubField);
                                    } else if (k == 1) {
                                        // Location
                                        problem.setHospitalLocation(locSubField);
                                    }
                                }
                                break;
                            case PROBLEM_LOCATION_TYPE:
                                problem.setLocationType(field);
                                break;
                            case PROBLEM_PROVIDER:
                                String[] proSubFields = StringUtils.split(field, StringUtils.SEMICOLON);
                                for (int k = 0; k < proSubFields.length; k++) {
                                    String proSubField = proSubFields[k];

                                    if (StringUtils.isEmpty(proSubField)) {
                                        continue;
                                    }

                                    if (k == 0) {
                                        // Provider IEN
                                        problem.setProviderIEN(proSubField);
                                    } else if (k == 1) {
                                        // Provider
                                        problem.setResponsibleProvider(proSubField);
                                    }
                                }
                                break;
                            case PROBLEM_SERVICE_IEN:
                                String[] srvSubFields = StringUtils.split(field, StringUtils.SEMICOLON);
                                for (int k = 0; k < srvSubFields.length; k++) {
                                    String srvSubField = srvSubFields[k];

                                    if (StringUtils.isEmpty(srvSubField)) {
                                        continue;
                                    }

                                    if (k == 0) {
                                        // Service IEN
                                        problem.setServiceIEN(srvSubField);
                                    } else if (k == 1) {
                                        // Service Name
                                        problem.setServiceName(srvSubField);
                                    }
                                }
                                break;
                            case PROBLEM_PRIORITY:
                                problem.setPriority(field);
                                break;
                            case PROBLEM_HAS_COMMENT:
                                problem.setHasComment(field);
                                break;
                            case PROBLEM_DATE_RECORDED:
                                problem.setRecordedDate(Timestamp.toDateTime(field));
                                break;
                            case PROBLEM_SC_CONDITIONS:
                                problem.setScConditions(field);
                                break;
                            case PROBLEM_INACTIVE_ICD_CODE:
                                problem.setInactiveICDCode(field);
                                break;
                            default:
                                LOGGER.error("Invalid length");
                                break;
                        }

                    }

                    problemsList.add(problem);

                }

                rtc = problemsList.toArray(new Problem[0]);
            }
        }
        return rtc;
    }

//    private void addNoteDetailText(ProblemDetail problem, String detailText) {
//        if (problem == null || detailText == null) return;
//
//        ProblemNote[] notes = problem.getNotes();
//        if (notes == null) {
//            notes = new ProblemNote[1];
//            notes[0] = new ProblemNote();
//            fillCommonDataBeanValues(notes[0]);
//            notes[0].setNoteText("");
//        }
//
//        notes[0].setNoteText((notes[0].getNoteText()) + detailText + "\n");
//        problem.setNotes(notes);
//    }

    private ProblemNote[] getNotes(String detailText) {
        ProblemNote[] rtc;
        if (detailText == null) {
            rtc = null;
        } else {
            ProblemNote note = new ProblemNote();
            fillCommonDataBeanValues(note);
            note.setNoteText(detailText.concat("\n"));
            ProblemNote[] notes = new ProblemNote[1];
            notes[0] = note;
            rtc = notes;
        }
        return rtc;
    }

    public ProblemDetail getProblemDetail(String problemIEN) throws RPCException {
        String patientIen = cxn.getPatientIen();
        ProblemDetail problem = new ProblemDetail();
        fillCommonDataBeanValues(problem);

        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQPL DETAIL");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, problemIEN);
        String detailRequest = vquery.buildMessage();
        String detailResponse = (String) cxn.query(detailRequest);
        problem.setDetailText(detailResponse);

        vquery = vistaQueryFactory.getVistaQuery("ORQQPL PROB COMMENTS");
        vquery.addParameter(VistaQuery.LITERAL, problemIEN);
        String commentRequest = vquery.buildMessage();
        String commentResponse = (String) cxn.query(commentRequest);
        //addNoteDetailText(problem, commentResponse);
        problem.setNotes(getNotes(commentResponse));

        return problem;
    }

    /**
     * Uses the VPR RPC with "problems" as the type.
     *
     * @param startDate The start of the date range for the list of problems
     * @param endDate   The end of the date range for the list of problems
     * @return The list of problems for the specified date range
     * @throws RPCException
     */
    public Problem[] getProblemsVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getProblemsVPR(cxn.getPatientIen(), startDate, endDate);
    }

    /**
     * Uses the VPR RPC with "problems" as the type.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of problems
     * @param endDate    The end of the date range for the list of problems
     * @return The list of problems for the specified date range
     * @throws RPCException
     */
    public Problem[] getProblemsVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetProblemsVPRRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        Problem[] problems = parseProblemsFromVPR(/*patientIen, */response);

        return problems;
    }

    /**
     * Builds the VPR RPC request string using "problems" as the type. The problems contain the list of problems and
     * comments if entered. The RPC will fail if the "VPR APPLICATION PROXY" context is not set. That is set when
     * setting up the connection.
     *
     * @param patientIen Patient IEN
     * @param startDate  Start date of range
     * @param endDate    End date of range
     * @return The RPC response message string
     */
    private String buildGetProblemsVPRRequest(String patientIen, Calendar startDate, Calendar endDate) {
        return VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.PROBLEMS);
    }

    private ProblemDetail[] parseProblemsFromVPR(/*String patientIen, */String response) {
        ProblemDetail[] rtc;
        if ("".equals(response)) {
            rtc = new ProblemDetail[0];
        } else {

            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

            ArrayList<ProblemDetail> problemsList = new ArrayList<ProblemDetail>();

            try {
                Document doc = parseXMLDocument(sbld.toString());

                NodeList problemNodeList = (NodeList) exprProblem.evaluate(doc, XPathConstants.NODESET);
                StringBuilder commentSB = new StringBuilder();
                int problemNodeListLength = problemNodeList.getLength();
                for (int i = 0; i < problemNodeListLength; i++) {
                    Node problemNode = problemNodeList.item(i);
                    problemNode.getParentNode().removeChild(problemNode);

                    ProblemDetail problem = new ProblemDetail();
                    fillCommonDataBeanValues(problem);

                    String acuityName = (String) exprAcuityName.evaluate(problemNode, XPathConstants.STRING);
//                    String acuityCode = (String) exprAcuityCode.evaluate(problemNode, XPathConstants.STRING);
                    String entered = (String) exprEntered.evaluate(problemNode, XPathConstants.STRING);
                    String onset = (String) exprOnset.evaluate(problemNode, XPathConstants.STRING);
//                    String facilityName = (String) exprFacilityName.evaluate(problemNode, XPathConstants.STRING);
//                    String failityCode = (String) exprFacilityCode.evaluate(problemNode, XPathConstants.STRING);
                    String icd = (String) exprIcd.evaluate(problemNode, XPathConstants.STRING);
                    String location = (String) exprLocation.evaluate(problemNode, XPathConstants.STRING);
                    String id = (String) exprId.evaluate(problemNode, XPathConstants.STRING);
                    String name = (String) exprName.evaluate(problemNode, XPathConstants.STRING);
//                    String providerName = (String) exprProviderName.evaluate(problemNode, XPathConstants.STRING);
                    String providerCode = (String) exprProviderCode.evaluate(problemNode, XPathConstants.STRING);
//                    String removed = (String) exprRemoved.evaluate(problemNode, XPathConstants.STRING);
                    String sc = (String) exprSc.evaluate(problemNode, XPathConstants.STRING);
//                    String statusCode = (String) exprStatusCode.evaluate(problemNode, XPathConstants.STRING);
                    String statusName = (String) exprStatusName.evaluate(problemNode, XPathConstants.STRING);
//                    String unverified = (String) exprUnverified.evaluate(problemNode, XPathConstants.STRING);
                    String updated = (String) exprUpdated.evaluate(problemNode, XPathConstants.STRING);
                    String codingSystem = (String) exprCodingSystem.evaluate(problemNode, XPathConstants.STRING);
                    String sctc = (String) exprSctc.evaluate(problemNode, XPathConstants.STRING);
                    String sctt = (String) exprSctt.evaluate(problemNode, XPathConstants.STRING);
                    String icdd = (String) exprIcdd.evaluate(problemNode, XPathConstants.STRING);

                    Calendar enteredDate = Timestamp.toDateTime(entered);
                    if (!StringUtils.isEmpty(onset)) {
                        String formattedOnset = Timestamp.formatDisplayDateNoBadMonthOrDay(onset);
                        problem.setOnsetDate(formattedOnset);
                    }
                    Calendar updatedDate = Timestamp.toDateTime(updated);

                    Node commentsNode = (Node) exprComments.evaluate(problemNode, XPathConstants.NODE);
                    if (commentsNode != null) {
                        NodeList commentNodeList = (NodeList) exprCommentNode.evaluate(commentsNode, XPathConstants.NODESET);

                        commentSB.setLength(0);

                        String commentEntered = "";
                        String commentEnteredBy = "";
                        String commentText;

                        ArrayList<ProblemNote> problemNoteArrayList = new ArrayList<ProblemNote>();

                        ProblemNote problemNote = new ProblemNote();
                        fillCommonDataBeanValues(problemNote);
                        int commentNodeListLength = commentNodeList.getLength();
                        for (int j = 0; j < commentNodeListLength; j++) {
                            Node commentNode = commentNodeList.item(j);
                            commentNode.getParentNode().removeChild(commentNode);

                            String newCommentEntered = (String) exprCommentEntered.evaluate(commentNode, XPathConstants.STRING);
                            String newCommentEnteredBy = (String) exprCommentEnteredBy.evaluate(commentNode, XPathConstants.STRING);
                            commentText = (String) exprCommentText.evaluate(commentNode, XPathConstants.STRING);

                            if (!newCommentEntered.equals(commentEntered) || !newCommentEnteredBy.equals(commentEnteredBy)) {
                                if (j != 0) {
                                    problemNote.setNoteEnteredBy(commentEnteredBy);
                                    problemNote.setNoteDate(Timestamp.toDateTime(commentEntered));
                                    problemNote.setNoteText(commentSB.toString());
                                    problemNoteArrayList.add(problemNote);
                                }

                                problemNote = new ProblemNote();
                                fillCommonDataBeanValues(problemNote);
                            }

                            commentEntered = newCommentEntered;
                            commentEnteredBy = newCommentEnteredBy;

                            commentSB.append("\r\n");
                            commentSB.append(commentText);
                        }

                        if (commentNodeList.getLength() > 0) {
                            // There was at least one comment
                            problemNote.setNoteEnteredBy(commentEnteredBy);
                            problemNote.setNoteDate(Timestamp.toDateTime(commentEntered));
                            problemNote.setNoteText(commentSB.toString());
                            problemNoteArrayList.add(problemNote);

                            problem.setNotes(problemNoteArrayList.toArray(new ProblemNote[0]));
                            problem.setHasComment("1");
                        }
                    }

                    problem.setAcuity(acuityName);
                    problem.setHospitalLocation(location);

                    ArrayList<Code> codeArrayList = new ArrayList<Code>();

                    if ("10D".equals(codingSystem)) {
                        codingSystem = "ICD10";
                        name = icdd;
                        if (!StringUtils.isEmpty(sctc)) {
                            // only add the SNOMED codes for ICD10
                            Code snomedCode = new Code();
                            snomedCode.setCode(sctc);
                            snomedCode.setDisplay(sctt);
                            snomedCode.setSystem("SNOMED");
                            codeArrayList.add(snomedCode);
                        }
                    } else {
                        codingSystem ="ICD9";
                    }
                    problem.setDescription(name);

                    if (!StringUtils.isEmpty(icd)) {
                        problem.setIcdCode(icd);
                        Code icdCode = new Code();
                        icdCode.setCode(icd);
                        icdCode.setSystem(codingSystem);
                        codeArrayList.add(icdCode);
                    }

                    if (codeArrayList.size() > 0) {
                        problem.setCodes(codeArrayList.toArray(new Code[0]));
                    }

                    problem.setId(id);
                    problem.setProviderIEN(providerCode);
                    if ("YES".equalsIgnoreCase(sc) || "1".equalsIgnoreCase(sc)) {
                        problem.setServiceConnected("YES");
                    } else {
                        problem.setServiceConnected("NO");
                    }
                    problem.setStatus(statusName);
                    problem.setEnteredDate(enteredDate);
                    problem.setLastModifiedDate(updatedDate);

                /*
                TODO: Do we want to make an extra RPC call for each problem to get Detail Text?
                // Check if the problem has DetailText
                problem.setDetailText(getProblemDetail(id).getDetailText());
                */

                    problemsList.add(problem);
                }

            } catch (ParserConfigurationException pce) {
                logErrorMessage(pce);
            } catch (SAXException saxe) {
                logErrorMessage(saxe);
            } catch (IOException ioe) {
                logErrorMessage(ioe);
            } catch (XPathExpressionException xpe) {
                logErrorMessage(xpe);
            }


            rtc = problemsList.toArray(new ProblemDetail[0]);
        }
        return rtc;
    }

    private void logErrorMessage(Exception except) {
        LOGGER.error(except.getMessage());
    }


}
