/*
 * Janus 4.0 (c)
 * Copyright (c) 2015 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.patient.dao.rpc.utils;

import org.slf4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.xpath.*;

/**
 * Created by sishii on 4/7/15.
 */
public class XPathUtil {
    public static void logErrorMessage(Logger logger, Exception except) {
        logger.error(except.getMessage());
    }

    // XPath evaluation optimization, see https://blogs.sap.com/2009/12/04/performance-improvements-in-nw-java-applications-with-xml-processing/
    private static final String DTM_MANAGER_PROP_NAME =
            "com.sun.org.apache.xml.internal.dtm.DTMManager";
    private static final String DTM_MANAGER_CLASS_NAME =
            "com.sun.org.apache.xml.internal.dtm.ref.DTMManagerDefault";

    static {
        System.setProperty(DTM_MANAGER_PROP_NAME, DTM_MANAGER_CLASS_NAME);
    }

    public static String evaluateNodeToString(Node node, XPathExpression expression, Logger logger) {
        String returnString = null;
        try {
            returnString = (String) expression.evaluate(node, XPathConstants.STRING);
        } catch (XPathExpressionException e) {
            logErrorMessage(logger, e);
        }
        return returnString;
    }

    public static Node evaluateNodeToNode(Node node, XPathExpression expression, Logger logger) {
        Node returnNode = null;
        try {
            returnNode = (Node) expression.evaluate(node, XPathConstants.NODE);
        } catch (XPathExpressionException e) {
            logErrorMessage(logger, e);
        }
        return returnNode;
    }

    public static NodeList evaluateNodeToNodeList(Node node, XPathExpression expression, Logger logger) {
        NodeList returnNodeList = null;
        try {
            returnNodeList = (NodeList) expression.evaluate(node, XPathConstants.NODESET);
        } catch (XPathExpressionException e) {
            logErrorMessage(logger, e);
        }
        return returnNodeList;
    }

    private static final XPathFactory xpathFactory = XPathFactory.newInstance();

    public static XPathExpression xpathCompile(String expression) {
        final XPath xpath = xpathFactory.newXPath();

        try {
            return xpath.compile(expression);
        } catch (XPathExpressionException e) {
            throw new RuntimeException(e);
        }
    }

}
