/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.AppointmentsDao;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.lang.reflect.InvocationTargetException;
import java.util.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class AppointmentsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(AppointmentsDataService.class.getName());

    public List<PatientAppointments> getPatientAppointments(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);
        VistaRPCConnectionManager msq = null;
        VistaRPCConnectionManager msqVPR = null;

        try {
            List<PatientAppointments> retVal = new ArrayList<PatientAppointments>();

            msqVPR = setupVPRContextMultiSourceQuery(queryBean);
            //msq2 = setupMultiSourceQuery(queryBean);

            LinkedHashMap<Site, Object> apptsMap = VistaRPCConnectionManager.execute(msqVPR.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entries = apptsMap.entrySet();

            for (Map.Entry<Site, Object> entry : entries) {
                Object appointments = entry.getValue();
                if (appointments instanceof Exception) {
                    appointments = generateConnectionUnavailableBean(entry.getKey(), (Exception) appointments);
                }
                retVal.addAll(Arrays.asList((PatientAppointments[]) appointments));
            }

            msq = setupMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> apptsMapRPC = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS_RPC, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entriesRPC = apptsMapRPC.entrySet();

            for (Map.Entry<Site, Object> entry : entriesRPC) {
                Object appointments = entry.getValue();
                if (appointments instanceof Exception) {
                    appointments = generateConnectionUnavailableBean(entry.getKey(), (Exception) appointments);
                    retVal.addAll(Arrays.asList((PatientAppointments[]) appointments));
                } else {
                    PatientAppointments[] patAppoints = (PatientAppointments[])appointments;
                    if(apptsMap.get(entry.getKey()) == null){
                        retVal.addAll(Arrays.asList(patAppoints));
                    } else {
                        if (!(apptsMap.get(entry.getKey()) instanceof Exception)){
                            for(PatientAppointments patApptRPC : patAppoints) {
                                boolean apptExists = false;
                                for (PatientAppointments patAppt : (PatientAppointments[]) apptsMap.get(entry.getKey())) {
                                    Calendar vprDate = patAppt.getApptDate();
                                    vprDate.set(Calendar.MILLISECOND, 0);
                                    vprDate.set(Calendar.SECOND, 0);
                                    if ((vprDate.compareTo(patApptRPC.getApptDate()) == 0)
                                            && (patAppt.getClinic().equalsIgnoreCase(patApptRPC.getClinic()))) {
                                        apptExists = true;
                                    }
                                }
                                if (!apptExists) {
                                    retVal.add(patApptRPC);
                                }
                            }
                        }
                    }
                }
            }

            return retVal;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
            if (msqVPR != null) {
                try {
                    msqVPR.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

        }
    }
    
    public List<PatientAppointments> getPatientAppointmentsBse(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);
        VistaRPCConnectionManager msq = null;

        try {
            List<PatientAppointments> retVal = new ArrayList<PatientAppointments>();

            msq = setupVPRContextMultiSourceQuery(queryBean);

            LinkedHashMap<Site, Object> apptsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entries = apptsMap.entrySet();

            for (Map.Entry<Site, Object> entry : entries) {
                Object appointments = entry.getValue();
                if (appointments instanceof Exception) {
                    appointments = generateConnectionUnavailableBean(entry.getKey(), (Exception) appointments);
                }
                retVal.addAll(Arrays.asList((PatientAppointments[]) appointments));
            }

            LinkedHashMap<Site, Object> apptsMapRPC = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS_RPC, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entriesRPC = apptsMapRPC.entrySet();

            for (Map.Entry<Site, Object> entry : entriesRPC) {
                Object appointments = entry.getValue();
                if (appointments instanceof Exception) {
                    appointments = generateConnectionUnavailableBean(entry.getKey(), (Exception) appointments);
                    retVal.addAll(Arrays.asList((PatientAppointments[]) appointments));
                } else {
                    PatientAppointments[] patAppoints = (PatientAppointments[])appointments;
                    if(apptsMap.get(entry.getKey()) == null){
                        retVal.addAll(Arrays.asList(patAppoints));
                    } else {
                        if (!(apptsMap.get(entry.getKey()) instanceof Exception)){
                            for(PatientAppointments patApptRPC : patAppoints) {
                                boolean apptExists = false;
                                for (PatientAppointments patAppt : (PatientAppointments[]) apptsMap.get(entry.getKey())) {
                                    Calendar vprDate = patAppt.getApptDate();
                                    vprDate.set(Calendar.MILLISECOND, 0);
                                    vprDate.set(Calendar.SECOND, 0);
                                    if ((vprDate.compareTo(patApptRPC.getApptDate()) == 0)
                                            && (patAppt.getClinic().equalsIgnoreCase(patApptRPC.getClinic()))) {
                                        apptExists = true;
                                    }
                                }
                                if (!apptExists) {
                                    retVal.add(patApptRPC);
                                }
                            }
                        }
                    }
                }
            }

            return retVal;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<PatientAppointments> getPatientAppointmentsOneContext(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);
        VistaRPCConnectionManager msq = null;

        try {
            List<PatientAppointments> retVal = new ArrayList<PatientAppointments>();

            msq = setupVPRContextMultiSourceQuery(queryBean);

            LinkedHashMap<Site, Object> apptsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entries = apptsMap.entrySet();

            for (Map.Entry<Site, Object> entry : entries) {
                Object appointments = entry.getValue();
                if (appointments instanceof Exception) {
                    appointments = generateConnectionUnavailableBean(entry.getKey(), (Exception) appointments);
                }
                retVal.addAll(Arrays.asList((PatientAppointments[]) appointments));
            }

            LinkedHashMap<Site, Object> apptsMapRPC = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_APPOINTMENTS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_APPOINTMENTS_RPC_SET_CONTEXT, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entriesRPC = apptsMapRPC.entrySet();

            for (Map.Entry<Site, Object> entry : entriesRPC) {
                Object appointments = entry.getValue();
                if (appointments instanceof Exception) {
                    appointments = generateConnectionUnavailableBean(entry.getKey(), (Exception) appointments);
                    retVal.addAll(Arrays.asList((PatientAppointments[]) appointments));
                } else {
                    PatientAppointments[] patAppoints = (PatientAppointments[])appointments;
                    if(apptsMap.get(entry.getKey()) == null){
                        retVal.addAll(Arrays.asList(patAppoints));
                    } else {
                        if (!(apptsMap.get(entry.getKey()) instanceof Exception)){
                            for(PatientAppointments patApptRPC : patAppoints) {
                                boolean apptExists = false;
                                for (PatientAppointments patAppt : (PatientAppointments[]) apptsMap.get(entry.getKey())) {
                                    Calendar vprDate = patAppt.getApptDate();
                                    vprDate.set(Calendar.MILLISECOND, 0);
                                    vprDate.set(Calendar.SECOND, 0);
                                    if ((vprDate.compareTo(patApptRPC.getApptDate()) == 0)
                                            && (patAppt.getClinic().equalsIgnoreCase(patApptRPC.getClinic()))) {
                                        apptExists = true;
                                    }
                                }
                                if (!apptExists) {
                                    retVal.add(patApptRPC);
                                }
                            }
                        }
                    }
                }
            }

            return retVal;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getEncountersReport(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        RPCConnection cxn = null;

        try {
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            AppointmentsDao apptsDao = new AppointmentsDao(cxn);
            return apptsDao.getEncountersReport(queryBean);
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }


    private PatientAppointments[] generateConnectionUnavailableBean(Site site, Exception except) {

        PatientAppointments apt = new PatientAppointments();
        apt.setClinic(CONNECTION_UNAVAILABLE);
        apt.setApptDate(Utils.backDate().toGregorianCalendar());
        apt.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            apt.setClinic(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }

        return new PatientAppointments[]{apt};
    }
}
