/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.ClinicalRemindersDao;
import gov.va.med.vds.vistabeans.ClinicalReminder;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class ClinicalRemindersDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(ClinicalRemindersDataService.class.getName());

    public List<ClinicalReminder> getPatientClinicalReminders(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupMultiSourceQuery(queryBean);
//            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> remindersMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_CLINICAL_REMINDERS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_CLINICAL_REMINDERS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = remindersMap.entrySet();
            List<ClinicalReminder> remindersList = new ArrayList<ClinicalReminder>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object reminders = entry.getValue();
                if (reminders instanceof Exception) {
                    reminders = generateConnectionUnavailableBean(entry.getKey(), (Exception) reminders);
                }
                remindersList.addAll(Arrays.asList((ClinicalReminder[]) reminders));
            }

            return remindersList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getClinicalReminderDetail(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
//            Patient patient = queryBean.getPatient();
            String siteCode = queryBean.getRecordSite().getSiteCode();
            String reminderId = queryBean.getItemId();
            
            if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("siteCode is null or empty");
            } else if (StringUtils.isEmpty(reminderId)) {
                throw new IllegalArgumentException("remindersId is null or empty");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            ClinicalRemindersDao remindersDao = new ClinicalRemindersDao(cxn);
            return remindersDao.getClinicalReminderDetail(queryBean);
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private ClinicalReminder[] generateConnectionUnavailableBean(Site site, Exception except) {
       
        ClinicalReminder reminder = new ClinicalReminder();
        reminder.setDescription(CONNECTION_UNAVAILABLE);
        reminder.setLastOccurrenceDate(Utils.backDate().toGregorianCalendar());
        reminder.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            reminder.setDescription(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }
        
        return new ClinicalReminder[]{reminder};
    }
}
