/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.NotesDao;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vhahon.common.Utils;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class NotesDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(NotesDataService.class.getName());

    public List<ProgressNote> getPatientProgressNotes(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean, APP_CONFIG.getParserMaxThreadPoolSizeNotes());
            LinkedHashMap<Site, Object> notesOutptMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_NOTES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_PROGRESS_NOTES, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = notesOutptMap.entrySet();
            List<ProgressNote> notesOutptList = new ArrayList<ProgressNote>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object notesOutpt = entry.getValue();
                if (notesOutpt instanceof Exception) {
                    notesOutpt = generateConnectionUnavailableBeanProgressNote(entry.getKey(), (Exception) notesOutpt);
                }
                notesOutptList.addAll(Arrays.asList((ProgressNote[]) notesOutpt));
            }

            return notesOutptList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<ProgressNote> getPatientDischargeSummaries(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> notesInptMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_NOTES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_DISCHARGE_SUMMARIES, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = notesInptMap.entrySet();
            List<ProgressNote> notesInptList = new ArrayList<ProgressNote>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object notesInpt = entry.getValue();
                if (notesInpt instanceof Exception) {
                    notesInpt = generateConnectionUnavailableBeanProgressNote(entry.getKey(), (Exception) notesInpt);
                }
                notesInptList.addAll(Arrays.asList((ProgressNote[]) notesInpt));
            }

            return notesInptList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<Consult> getPatientConsultRequests(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> outputConsultsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_NOTES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_CONSULT_REQUESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = outputConsultsMap.entrySet();
            List<Consult> outptConsultsList = new ArrayList<Consult>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object outptConsults = entry.getValue();
                if (outptConsults instanceof Exception) {
                    outptConsults = generateConnectionUnavailableBeanConsult(entry.getKey(), (Exception) outptConsults);
                }
                outptConsultsList.addAll(Arrays.asList((Consult[]) outptConsults));
            }

            return outptConsultsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<Consult> getPatientActiveConsultRequests(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);
        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> outputConsultsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_NOTES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ACTIVE_CONSULT_REQUESTS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = outputConsultsMap.entrySet();
            List<Consult> outptConsultsList = new ArrayList<Consult>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object outptConsults = entry.getValue();
                if (outptConsults instanceof Exception) {
                    outptConsults = generateConnectionUnavailableBeanConsult(entry.getKey(), (Exception) outptConsults);
                }
                outptConsultsList.addAll(Arrays.asList((Consult[]) outptConsults));
            }

            return outptConsultsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<FreeTextReport> getConsultReport(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            String siteCode = queryBean.getRecordSite().getSiteCode();
            String consultId = queryBean.getItemId();
            if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("siteCode is null or empty");
            } else if (StringUtils.isEmpty(consultId)) {
                throw new IllegalArgumentException("consultId is null or empty");
            }
            
            cxn = setupSingleRemoteSourceQuery(queryBean);
            NotesDao notesDao = new NotesDao(cxn);
            FreeTextReport[] reports = notesDao.getConsultReport(queryBean);

            return Arrays.asList(reports);

        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public Consult getPatientConsultRequest(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);
        RPCConnection cxn = null;
        try {
            String siteCode = queryBean.getRecordSite().getSiteCode();
            String consultId = queryBean.getItemId();
            if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("siteCode is null or empty");
            } else if (StringUtils.isEmpty(consultId)) {
                throw new IllegalArgumentException("consultId is null or empty");
            }

            cxn = setupVPRContextSingleRemoteSourceQuery(queryBean);
            NotesDao notesDao = new NotesDao(cxn);
            Consult consult = notesDao.getPatientConsultRequestById(queryBean);

            return consult;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                if (cxn.getParserThreadPool() != null && cxn.getParserThreadPool().getThreadPool() != null && !cxn.getParserThreadPool().getThreadPool().isShutdown()) {
                    try {
                        cxn.getParserThreadPool().getThreadPool().shutdown();
                    } catch (SecurityException ex) {
                        LOGGER.error("Failed to shutdown parserExecutorService (thread pool)");
                    }
                }
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getProgressNote(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            String itemId = queryBean.getItemId();
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (itemId == null || itemId.length() <= 0) {
                throw new IllegalArgumentException("Item Id cannot be null or empty");
            } else if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            NotesDao notesDao = new NotesDao(cxn);
            FreeTextReport note = notesDao.getProgressNote(queryBean);

            return note;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getVisitNotes(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            String itemId = queryBean.getItemId();
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (itemId == null || itemId.length() <= 0) {
                throw new IllegalArgumentException("Item Id cannot be null or empty");
            } else if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            NotesDao notesDao = new NotesDao(cxn);
            FreeTextReport note = notesDao.getVisitNotes(queryBean);

            return note;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }



    public FreeTextReport getPatientPostingsAllergiesSummary(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            cxn = setupSingleRemoteSourceQuery(queryBean);
            NotesDao notesDao = new NotesDao(cxn);
            FreeTextReport note = notesDao.getPatientPostingsAllergiesSummary();

            return note;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<PatientPostings> getPatientPostingsList(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> notesMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_NOTES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_POSTINGS_LIST, null);

            Set<Map.Entry<Site, Object>> entries = notesMap.entrySet();
            List<PatientPostings> postingsList = new ArrayList<PatientPostings>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object postings = entry.getValue();
                if (postings instanceof Exception) {
                    postings = generateConnectionUnavailableBeanPostings(entry.getKey(), (Exception) postings);
                }
                postingsList.add((PatientPostings) postings);
            }

            return postingsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<PCE> getPcesForNote(QueryBean queryBean) throws VDSException {
        validateQueryBeanItemId(queryBean);

        RPCConnection cxn = null;

        try {
            cxn = setupSingleRemoteSourceQuery(queryBean);
            NotesDao notesDao = new NotesDao(cxn);
            List<PCE> pces = notesDao.getPcesForNote(queryBean);

            return pces;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }


    private ProgressNote[] generateConnectionUnavailableBeanProgressNote(Site site, Exception except) {
        ProgressNote note = new ProgressNote();
        note.setNoteType(CONNECTION_UNAVAILABLE);
        note.setNoteTitle(CONNECTION_UNAVAILABLE);
        note.setNoteDate(Utils.backDate().toGregorianCalendar());

        note.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            note.setNoteType(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }
        return new ProgressNote[] {note};
    }

    private Consult[] generateConnectionUnavailableBeanConsult(Site site, Exception except) {
        Consult consult = new Consult();
        consult.setService(CONNECTION_UNAVAILABLE);
        consult.setRequestDate(Utils.backDate().toGregorianCalendar());

        consult.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            consult.setService(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }
        return new Consult[] {consult};
    }

    private PatientPostings generateConnectionUnavailableBeanPostings(Site site, Exception except) {
        PatientPostings patientPostings = new PatientPostings();
        patientPostings.setCwad(CONNECTION_UNAVAILABLE);

        patientPostings.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            patientPostings.setCwad(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }
        return patientPostings;
    }
}
