/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

/**
 * This class contains data regarding a patient's allergy.
 */
public class Allergy extends DataBean {
    /**
     * name of allergy.
     */
    private String allergyName;
    /**
     * comment regarding allergy.
     */
    private String comment;
    /**
     * Internal Id of allergy.  Used to retrieve detailed allery information.
     */
    private Comment[] comments;

    private String allergyId;
    /**
     * Severity of the allergy reaction.
     */
    private String severity;

    private Code[] reactions;
    private Code[] drugClasses;
    private Code[] drugIngredients;
    /**
     * Provider (originator/enteredBy) of the allergy
     */
    private Provider provider;

    /**
     * date originated/entered
     */
    private Calendar enteredDate;

    /**
     * verified status
     */
    private String verified;

    /**
     * Historical or Observed status
     */
    private String historical;

    private Calendar onsetDate;

    /**
     * Name of allergy.
     */
    public String getAllergyName() {
        return allergyName;
    }

    public void setAllergyName(String newAllergyName) {
        allergyName = newAllergyName;
    }


    /**
     * Internal Id of Allergy.
     */
    public String getAllergyId() {
        return allergyId;
    }

    public void setAllergyId(String newAllergyId) {
        allergyId = newAllergyId;
    }

    /**
     * Comment about allergy
     */
    public String getComment() {
        return comment;
    }

    public void setComment(String newComment) {
        comment = newComment;
    }

    /**
     * Severity of the allery.
     */
    public String getSeverity() {
        return severity;
    }

    public void setSeverity(String newSeverity) {
        severity = newSeverity;
    }

    public Code[] getReactions() {
        Code[] rtc = null;
        if (reactions != null) {
            rtc = reactions.clone();
        }
        return rtc;
    }

    public void setReactions(Code[] reactionCodes) {
        if (reactionCodes == null) {
            this.reactions = null;
        } else {
            this.reactions = reactionCodes.clone();
        }
    }

    public Code[] getDrugClasses() {
        Code[] rtc = null;
        if (drugClasses != null) {
            rtc = drugClasses.clone();
        }
        return rtc;
    }

    public void setDrugClasses(Code[] drugClassCodes) {
        if (drugClassCodes == null) {
            this.drugClasses = null;
        } else {
            this.drugClasses = drugClassCodes.clone();
        }

    }

    public Code[] getDrugIngredients() {
        Code[] rtc = null;
        if (drugIngredients != null) {
            rtc = drugIngredients.clone();
        }
        return rtc;
    }

    public void setDrugIngredients(Code[] drugIngredientCodes) {
        if (drugIngredientCodes == null) {
            this.drugIngredients = null;
        } else {
            this.drugIngredients = drugIngredientCodes.clone();            
        }

    }

    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    public Calendar getEnteredDate() {
        return enteredDate;
    }

    public void setEnteredDate(Calendar enteredDate) {
        this.enteredDate = enteredDate;
    }

    public String getVerified() {
        return verified;
    }

    public void setVerified(String verified) {
        this.verified = verified;
    }

    public String getHistorical() {
        return historical;
    }

    public void setHistorical(String historical) {
        this.historical = historical;
    }
    
    public Comment[] getComments() {
        Comment[] rtc = null;
        if (comments != null) {
            rtc = comments.clone();
        }
        return rtc;
    }

    public void setComments(Comment[] comments) {
        if (comments == null) {
            this.comments = null;            
        } else {
            this.comments = comments.clone();
        }
    }

    public Calendar getOnsetDate() {
        return onsetDate;
    }

    public void setOnsetDate(Calendar onsetDate) {
        this.onsetDate = onsetDate;
    }
}