/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;


/**
 * Consults represents ConsultRequest for a patient within Vista.
 */
public class Consult extends DataBean {
    /**
     * Internal Id of consult, used to retrieve the textual report of the consult request.
     */
    private String id;
    /**
     * Indicator whether this is a procedure or a consult.
     */
    private String procedureConsult;
    /**
     * Date the  consult was requested.
     */
    private Calendar requestDate;
    /**
     * the service associated with the consult.
     */
    private String service;
    /**
     * Status of consult.
     */
    private String status;
    /**
     * textual report of the consult.  This attribute is only populated when requesting
     * the consult details via the getConsultReport method.
     */
    private String report;

    private Calendar patientNextAppointment;

    private String consultType;

    private String complexDataUrl;

    private String contentType;

    private EncounterDocument[] documents;

    private String orderId;

    /**
     * Provider
     */
    private Provider provider;

    public Consult() {
        super();
    }

    /**
     * Internal ID of the consult.  This is used to retrieve the consult report.
     */
    public String getId() {
        return id;
    }

    public void setId(String newId) {
        id = newId;
    }

    /**
     * Indicator as to whether this is a consult or procedure request.
     */
    public String getProcedureConsult() {
        return procedureConsult;
    }

    public void setProcedureConsult(String newProcedureConsult) {
        procedureConsult = newProcedureConsult;
    }

    /**
     * date of the consult request.
     */
    public Calendar getRequestDate() {
        return requestDate;
    }

    public void setRequestDate(Calendar newRequestDate) {
        requestDate = newRequestDate;
    }

    /**
     * Service associated with the consult.
     */
    public String getService() {
        return service;
    }

    public void setService(String newService) {
        service = newService;
    }

    /**
     * Status of consult.
     */
    public String getStatus() {
        return status;
    }

    public void setStatus(String newStatus) {
        status = newStatus;
    }

    /**
     * Report text of the consult.  This will only be populated when requesting a
     * consult report.
     */
    public String getReport() {
        return report;
    }

    public void setReport(String newReport) {
        report = newReport;
    }

    public Calendar getPatientNextAppointment() {
        return patientNextAppointment;
    }

    public void setPatientNextAppointment(Calendar PatientNextAppointment) {
        this.patientNextAppointment = PatientNextAppointment;
    }

    public String getConsultType() {
        return consultType;
    }

    public void setConsultType(String consultType) {
        this.consultType = consultType;
    }

    public String getComplexDataUrl() {
        return complexDataUrl;
    }

    public void setComplexDataUrl(String complexDataUrl) {
        this.complexDataUrl = complexDataUrl;
    }

    public String getContentType() {
        return contentType;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    /**
     * Sets the provider name. If the Provider has not yet been set, creates a new Provider.
     * Only the name will be set
     *
     * @param provider The Provider's name
     */
    public void setProvider(String provider) {
        this.provider = new Provider();
        this.provider.setName(provider);
    }

    public EncounterDocument[] getDocuments() {
        EncounterDocument[] rtc = null;
        if (documents != null) {
            rtc = documents.clone();
        }
        return rtc;
    }

    public void setDocuments(EncounterDocument[] documents) {
        if (documents == null) {
            this.documents = null;
        } else {
            this.documents = documents.clone();
        }
    }

    public String getOrderId() {
        return orderId;
    }

    public void setOrderId(String orderId) {
        this.orderId = orderId;
    }
}