/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.vistabeans;

import java.util.Calendar;

public class Encounter extends PatientAppointments {
    private Provider provider;
    private EncounterDocument[] documents;

    private Procedure[] procedures;
    private Diagnosis[] diagnosis;

    private String reasonNarrative;
    private String reasonCode;
    private String complexDataUrl;
    private String contentType;
    private String dischargeDisposition;
    private String encounterId;
    private String encounterNumber;
    private String enteredBy;
    private Calendar enteredByDate;
    private String evalManagementCode;
    private String status;
    private String sensitive;

    // Admission data
    private Calendar arrivalDateTime;
    private Calendar departureDateTime;
    private String roomBed;
    private String specialty;
    private Provider admittingProvider;
    private String admissionRegistrationNum;

    public Diagnosis[] getDiagnosis() {
        Diagnosis[] rtc = null;
        if (diagnosis != null) {
            rtc = diagnosis.clone();
        }
        return rtc;
    }

    public void setDiagnosis(Diagnosis[] diagnosis) {
        if (diagnosis == null) {
            this.diagnosis = null;
        } else {
            this.diagnosis = diagnosis.clone();
        }
    }

    public Procedure[] getProcedures() {
        Procedure[] rtc = null;
        if (procedures != null) {
            rtc = procedures.clone();
        }
        return rtc;
    }

    public void setProcedures(Procedure[] procedures) {
        if (procedures == null) {
            this.procedures = null;
        } else {
            this.procedures = procedures.clone();
        }
    }

    public Provider getProvider() {
        return provider;
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    public void setProvider(String provider) {
        this.provider = new Provider();
        this.provider.setName(provider);
    }


    public EncounterDocument[] getDocuments() {
        EncounterDocument[] rtc = null;
        if (documents != null) {
            rtc = documents.clone();
        }
        return rtc;
    }

    public void setDocuments(EncounterDocument[] documents) {
        if (documents == null) {
            this.documents = null;
        } else {
            this.documents = documents.clone();
        }

    }

    public String getReasonNarrative() {
        return reasonNarrative;
    }

    public void setReasonNarrative(String reasonNarrative) {
        this.reasonNarrative = reasonNarrative;
    }

    public String getReasonCode() {
        return reasonCode;
    }

    public void setReasonCode(String reasonCode) {
        this.reasonCode = reasonCode;
    }

    public Calendar getArrivalDateTime() {
        return arrivalDateTime;
    }

    public void setArrivalDateTime(Calendar arrivalDateTime) {
        this.arrivalDateTime = arrivalDateTime;
    }

    public Calendar getDepartureDateTime() {
        return departureDateTime;
    }

    public void setDepartureDateTime(Calendar departureDateTime) {
        this.departureDateTime = departureDateTime;
    }

    public String getRoomBed() {
        return roomBed;
    }

    public void setRoomBed(String roomBed) {
        this.roomBed = roomBed;
    }

    public String getSpecialty() {
        return specialty;
    }

    public void setSpecialty(String specialty) {
        this.specialty = specialty;
    }

    public String getComplexDataUrl() {
        return complexDataUrl;
    }

    public void setComplexDataUrl(String complexDataUrl) {
        this.complexDataUrl = complexDataUrl;
    }

    public String getContentType() {
        return contentType;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    public String getDischargeDisposition() {
        return dischargeDisposition;
    }

    public void setDischargeDisposition(String dischargeDisposition) {
        this.dischargeDisposition = dischargeDisposition;
    }

    public String getEncounterId() {
        return encounterId;
    }

    public void setEncounterId(String encounterId) {
        this.encounterId = encounterId;
    }

    public String getEncounterNumber() {
        return encounterNumber;
    }

    public void setEncounterNumber(String encounterNumber) {
        this.encounterNumber = encounterNumber;
    }

    public String getEnteredBy() {
        return enteredBy;
    }

    public void setEnteredBy(String enteredBy) {
        this.enteredBy = enteredBy;
    }

    public Calendar getEnteredByDate() {
        return enteredByDate;
    }

    public void setEnteredByDate(Calendar enteredByDate) {
        this.enteredByDate = enteredByDate;
    }

    public String getEvalManagementCode() {
        return evalManagementCode;
    }

    public void setEvalManagementCode(String evalManagementCode) {
        this.evalManagementCode = evalManagementCode;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public Provider getAdmittingProvider() {
        return admittingProvider;
    }

    public void setAdmittingProvider(Provider admittingProvider) {
        this.admittingProvider = admittingProvider;
    }

    /**
     * Sets the admitting prov name. If the Provider has not yet been set, creates a new Provider.
     * Only the name will be set and
     *
     * @param provider The Provider's name
     */
    public void setAdmittingProvider(String provider) {
        this.admittingProvider = new Provider();
        this.admittingProvider.setName(provider);
    }

    /**
     * @return the sensitive
     */
    public String getSensitive() {
        return sensitive;
    }

    /**
     * @param sensitive the sensitive to set
     */
    public void setSensitive(String sensitive) {
        this.sensitive = sensitive;
    }

    public String getAdmissionRegistrationNum() {
        return admissionRegistrationNum;
    }

    public void setAdmissionRegistrationNum(String admissionRegistrationNum) {
        this.admissionRegistrationNum = admissionRegistrationNum;
    }
}
