/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

public class ProblemDetail extends Problem {
    /**
     * Hospital location where the problem was first identified.
     */
    private String hospitalLocation;
    /**
     * User associated with the patient's problem.
     */
    private String responsibleProvider;
    /**
     * Date the problem was entered into the ehr.  this property is only populated
     * when requesting problem detail.
     */
    private Calendar enteredDate;
    /**
     * Person who entered the problem into the ehr.  this property is only populated
     * when requesting problem detail.
     */
    private String enteredBy;
    /**
     * Date the problem was first recorded.  This property is only populated when
     * requesting problem details.
     */
    private Calendar recordedDate;
    /**
     * Person who recorded the problem.  This property is only populated when requesting
     * problem detail.
     */
    private String recordedBy;
    /**
     * Array of notes associated with a Problem.  This is only populated when requesting
     * problem detail.
     */
    private ProblemNote[] notes;

    /**
     * Problem Detail Text
     */
    private String detailText;



    public ProblemDetail() {
        super();
        notes = null;
    }

    /**
     * Name of hospital location where this problem was identified.
     */
    public String getHospitalLocation() {
        return hospitalLocation;
    }

    public void setHospitalLocation(String newHospitalLocation) {
        hospitalLocation = newHospitalLocation;
    }

    /**
     * Responsible user assigned to problem.
     */
    public String getResponsibleProvider() {
        return responsibleProvider;
    }

    public void setResponsibleProvider(String newResponsibleProvider) {
        responsibleProvider = newResponsibleProvider;
    }

    /**
     * Calendar problem was entered into the ehr.  This attribute will only be populated
     * when requesting problem detail.
     */
    public Calendar getEnteredDate() {
        return enteredDate;
    }

    public void setEnteredDate(Calendar newEnteredDate) {
        enteredDate = newEnteredDate;
    }

    /**
     * Person who entered the problem. This attribute is populated only when requesting
     * problem detail.
     */
    public String getEnteredBy() {
        return enteredBy;
    }

    public void setEnteredBy(String newEnteredBy) {
        enteredBy = newEnteredBy;
    }

    /**
     * Calendar problem was recorded.  This will only be populated when requesting problem
     * details.
     */
    public Calendar getRecordedDate() {
        return recordedDate;
    }

    public void setRecordedDate(Calendar newRecordedDate) {
        recordedDate = newRecordedDate;
    }

    /**
     * Person who recorded the problem. This attribute is only populated when requesting
     * problem detail.
     */
    public String getRecordedBy() {
        return recordedBy;
    }

    public void setRecordedBy(String newRecordedBy) {
        recordedBy = newRecordedBy;
    }

    /**
     * Array of Problem notes associated with this problem. This will only be populated
     * when requesting problem details.
     */
    public ProblemNote[] getNotes() {
        ProblemNote[] rtc;
        if (notes == null) {
            rtc = null;
        } else {
            rtc = notes.clone();
        }
        return rtc;
    }

    public void setNotes(ProblemNote[] newNotes) {
        if (newNotes == null) {
            this.notes = null;
        } else {
            this.notes = newNotes.clone();    
        }
    }

    /**
     * Problem detail text.
     */
    public String getDetailText() {
        return detailText;
    }

    public void setDetailText(String detailText) {
        this.detailText = detailText;
    }



}