/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

/**
 * This class represents the notes associated with a problem.
 */
public class ProblemNote extends DataBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(ProblemNote.class);

    /**
     * Date the note was entered.
     */
    private Calendar noteDate;
    /**
     * Person who entered the note.
     */
    private String noteEnteredBy;
    /**
     * text of the note.
     */
    private String noteText;

    /**
     * Calendar note was entered.
     */
    public Calendar getNoteDate() {
        return noteDate;
    }

    public void setNoteDate(String newNoteDate) {
        Calendar date = null;

        SimpleDateFormat sdf = new SimpleDateFormat("MMM dd, yyyy@HH:mm", Locale.getDefault());
        date = new java.util.GregorianCalendar();
        try {
            date.setTime(sdf.parse(newNoteDate));
        } catch (ParseException e) {
            LOGGER.error(e.getMessage());
        }

        noteDate = date;
    }

    public void setNoteDate(Calendar newNoteDate) {
        noteDate = newNoteDate;
    }

    /**
     * Person who entered the note.
     */
    public String getNoteEnteredBy() {
        return noteEnteredBy;
    }

    public void setNoteEnteredBy(String newNoteEnteredBy) {
        noteEnteredBy = newNoteEnteredBy;
    }

    /**
     * Text of the note.
     */
    public String getNoteText() {
        return noteText;
    }

    public void setNoteText(String newNoteText) {
        noteText = newNoteText;
    }
}