/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Arrays;
import java.util.Calendar;

/**
 * This class is used to represent both ICD/CPT procedures and ICD/CPT Diagnoses.
 */
public class Procedure extends DataBean {
    /**
     * ICD/CPT code of the procedure/diagnosis.
     */
    private String code;
    /**
     * Code system: "ICD" or "CPT"
     */
    private String codeSystem;
    /**
     * Decription of the procedure/diagnosis.
     */
    private String description;
    /**
     * Status of the procedure/diagnosis.
     */
    private String status;
    /**
     * Date of  the Procedure/Diagnosis.
     */
    private Calendar date;

    /**
     * Encounter name
     */
    private String encounterName;

    /**
     * Location
     */
    private String location;

    private String encounterId;

    /**
     * Comment
     */
    private String comment;

    /**
     * Inpatient or outpatient
     */
    private String inOutpatient;


    private Provider provider;

    private String codingStatus;

    private String qualifiers;

    private String notes;

    private String bodySite;

    private EncounterDocument[] documents;

    private String complexDataUrl;

    private String sensitive;

    /**
     * Problem meta Type
     */
    private String type;

    /**
     * ICD code of procedure/diagnosis.
     */
    public String getCode() {
        return code;
    }

    public void setCode(String newCode) {
        code = newCode;
    }

    /**
     * Descriptive text of procedure/diagnosis.
     */
    public String getDescription() {
        return description;
    }

    public void setDescription(String newDescription) {
        description = newDescription;
    }

    /**
     * Status of procedure/diagnosis.
     */
    public String getStatus() {
        return status;
    }

    public void setStatus(String newStatus) {
        status = newStatus;
    }


    /**
     * Date of  procedure/diagnosis.
     */
    public Calendar getDate() {
        return date;
    }


    public void setDate(Calendar newDate) {
        date = newDate;
    }

    public String getCodeSystem() {
        return codeSystem;
    }

    public void setCodeSystem(String codeSystem) {
        this.codeSystem = codeSystem;
    }

    public String getEncounterName() {
        return encounterName;
    }

    public void setEncounterName(String encounterName) {
        this.encounterName = encounterName;
    }

    public String getLocation() {
        return location;
    }

    public void setLocation(String location) {
        this.location = location;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    /**
     * Name of User for the visit.
     */
    public Provider getProvider() {
        return provider;
    }

    /**
     * Sets the provider name. Creates a new Provider.
     * Only the name will be set
     *
     * @param provider The Provider's name
     */
    public void setProvider(String provider) {
        this.provider = new Provider();
        this.provider.setName(provider);
    }

    public void setProvider(Provider provider) {
        this.provider = provider;
    }

    public String getEncounterId() {
        return encounterId;
    }

    public void setEncounterId(String encounterId) {
        this.encounterId = encounterId;
    }

    /**
     * Get whether this is for an inpatient or outpatient procedure
     *
     * @return
     */
    public String getInOutpatient() {
        return inOutpatient;
    }

    /**
     * Set the procedure to inpatient or outpatient
     *
     * @param inOutpatient
     */
    public void setInOutpatient(String inOutpatient) {
        this.inOutpatient = inOutpatient;
    }

    public String getCodingStatus() {
        return codingStatus;
    }

    public void setCodingStatus(String codingStatus) {
        this.codingStatus = codingStatus;
    }

    public String getQualifiers() {
        return qualifiers;
    }

    public void setQualifiers(String qualifiers) {
        this.qualifiers = qualifiers;
    }

    public String getNotes() {
        return notes;
    }

    public void setNotes(String notes) {
        this.notes = notes;
    }

    public String getBodySite() {
        return bodySite;
    }

    public void setBodySite(String bodySite) {
        this.bodySite = bodySite;
    }

    public EncounterDocument[] getDocuments() {
        if(documents == null){
            return null;
        } else {
            return Arrays.copyOf(documents, documents.length);
        }
    }

    public void setDocuments(EncounterDocument[] documents) {
        if(documents == null) {
            this.documents = null;
        } else {
            this.documents = Arrays.copyOf(documents, documents.length);
        }
    }

    public String getComplexDataUrl() {
        return complexDataUrl;
    }

    public void setComplexDataUrl(String complexDataUrl) {
        this.complexDataUrl = complexDataUrl;
    }

    public String getType() {
        return type;
    }

    public void setType(String metaType) {
        this.type = metaType;
    }

    public String getSensitive(){
        return sensitive;
    }

    public void setSensitive(String sensitive){
        this.sensitive = sensitive;
    }
}