/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.List;

/**
 * Class represents a user/user.
 */
public class User {

    /** User profile identifier (used by jMeadows) **/
    private String userId;

    /**
     * Name of user.
     */
    private String name;

    /*
    * User's user id
    */
    private String userIen;

    /**
     * User's providerIen (as utilized by CHCS systems)
     */
    private String providerIen;

    /**
     * National User Id
     */
    private String NPI;

    /**
     * VistaSite associated with User's userIen
     */
    private Site hostSite;

    /**
     * User class of user.
     */
    private String providerClass;

    /**
     * Gender of user.
     */
    private String gender;
    /**
     * Office phone number for user.
     */
    private String dutyPhone;
    /**
     * User's pager number.
     */
    private String pager;
    /**
     * Flag to indicate the user is a user. (1=user)
     */
    private String providerFlag;

    /**
     * User's email address
     */
    private String email;

    /**
     * Encrypted Federated User Identifier
     */
    private String encryptedFederatedUid;

    /**
     * User's date of birth
     */
    private String dob;

    /**
     * User's signature class
     */
    private String sigClass;

    /** Stores the access and verify code in encrypted form. */
    private String encryptedAvCode;

    /**
     * User's Janus GUI configuration string
     */
    private String janusGUIConfig;

    /* Stores Janus GUI level user flags (e.g. can this user write back allergy?) */
    private List<JanusPermission> janusPermissions;

    /** user's agency (VA, DOD, etc.) **/
    private String agency;

    /** DMIS ID associated with the user **/
    private String dmisId;

    /** RPC Broker security enhancement token. **/
    private String bseToken;

    /** SAML token */
    private String samlToken;
    
    /** SmartCard id **/
    private String cardId;

    /** True if the user is restricted to a restricted patient list */
    private boolean restrictedUser;

    /** A list of restricted patients if the user is a restricted user */
    private List restrictedPatients;

    /** A list of restricted sites (as the SiteCode strings) that the user see patient data */
    private List restrictedSiteCodesList;

    /**
     * Gets user profile identifier
     * @return User profile identifier
     */
    public String getUserId() {
        return userId;
    }

    /**
     * Sets user profile identifier
     * @param userId User profile identifier
     */
    public void setUserId(String userId) {
        this.userId = userId;
    }

    /**
     * Name of user.
     */
    public String getName() {
        return name;
    }

    public void setName(String newName) {
        name = newName;
    }

    /**
     * User class of user.
     */
    public String getProviderClass() {
        return providerClass;
    }

    public void setProviderClass(String newProviderClass) {
        providerClass = newProviderClass;
    }

    /**
     * gender of user.
     */
    public String getGender() {
        return gender;
    }

    public void setGender(String newGender) {
        gender = newGender;
    }

    /**
     * Office phone of user.
     */
    public String getDutyPhone() {
        return dutyPhone;
    }

    public void setDutyPhone(String newDutyPhone) {
        dutyPhone = newDutyPhone;
    }

    /**
     * User's pager number.
     */
    public String getPager() {
        return pager;
    }

    public void setPager(String newPager) {
        pager = newPager;
    }

    /**
     * Flag indicating user has user status.
     */
    public String getProviderFlag() {
        return providerFlag;
    }

    public void setProviderFlag(String newProviderFlag) {
        providerFlag = newProviderFlag;
    }

    /**
     * Gets encrypted federated user identifier
     */
    public String getEncryptedFederatedUid() {
        return encryptedFederatedUid;
    }

    /**
     * Sets encrypted federated user identifier
     * @param encryptedFederatedUid
     */
    public void setEncryptedFederatedUid(String encryptedFederatedUid) {
        this.encryptedFederatedUid = encryptedFederatedUid;
    }

    /**
     * dob.
     */
    public String getDob() {
        return dob;
    }

    public void setDob(String dob) {
        this.dob = dob;
    }

    /**
     * email.
     */
    public String getEmail() {
        return email;
    }

    public void setEmail(String mail) {
        email = mail;
    }

    public String getUserIen() {
        return userIen;
    }

    public void setUserIen(String userIen) {
        this.userIen = userIen;
    }

    public String getSigClass() {
        return sigClass;
    }

    public void setSigClass(String sigClass) {
        this.sigClass = sigClass;
    }

    public Site getHostSite() {
        return hostSite;
    }

    public void setHostSite(Site hostSite) {
        this.hostSite = hostSite;
    }

    public String getEncryptedAvCode() {
        return encryptedAvCode;
    }

    public void setEncryptedAvCode(String encryptedAvCode) {
        this.encryptedAvCode = encryptedAvCode;
    }

    public String getJanusGUIConfig() {
        return janusGUIConfig;
    }

    public void setJanusGUIConfig(String janusGUIConfig) {
        this.janusGUIConfig = janusGUIConfig;
    }

    public String getNPI() {
        return NPI;
    }

    public void setNPI(String NPI) {
        this.NPI = NPI;
    }

    public List<JanusPermission> getJanusPermissions() {
        return janusPermissions;
    }

    public void setJanusPermissions(List<JanusPermission> janusPermissions) {
        this.janusPermissions = janusPermissions;
    }

    public String getAgency() {
        return agency;
    }

    public void setAgency(String agency) {
        this.agency = agency;
    }

    public String getProviderIen() {
        return providerIen;
    }

    public void setProviderIen(String providerIen) {
        this.providerIen = providerIen;
    }

    public String getBseToken() {
        return bseToken;
    }

    public String getSamlToken() {
        return samlToken;
    }

    public void setSamlToken(String samlToken) {
        this.samlToken = samlToken;
    }

    public void setBseToken(String bseToken) {
        this.bseToken = bseToken;
    }

    public String getDmisId() {
        return dmisId;
    }

    public void setDmisId(String dmisId) {
        this.dmisId = dmisId;
    }

    public String getCardId() {
        return cardId;
    }

    public void setCardId(String cardId) {
        this.cardId = cardId;
    }

    public boolean isRestrictedUser() {
        return restrictedUser;
    }

    public void setRestrictedUser(boolean restrictedUser) {
        this.restrictedUser = restrictedUser;
    }

    public List getRestrictedPatients() {
        return restrictedPatients;
    }

    public void setRestrictedPatients(List restrictedPatients) {
        this.restrictedPatients = restrictedPatients;
    }

    public List getRestrictedSiteCodesList() {
        return restrictedSiteCodesList;
    }

    public void setRestrictedSiteCodesList(List restrictedSiteCodesList) {
        this.restrictedSiteCodesList = restrictedSiteCodesList;
    }
}