/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;
import java.util.List;

/**
 * This class contains data regarding a VLER document.
 */
public class VLERDocument extends DataBean {
    /**
     * c32 document.
     */
    private byte[] document;
    /**
     * home community identifier.
     */
    private String homeCommunityId;
    /**
     * repository Unique identifier.
     */
    private String repositoryUniqueId;
    /**
     * document unique identifier.
     */
    private String documentUniqueId;
    /**
     * document mime type.
     */
    private String mimeType;
    /**
     * document name
     */
    private String name;
    /**
     * document author list.
     */
    private List<Author> authorList;
    /**
     * document creation date.
     */
    private Calendar creationTime;

    /**
     * document serviceStartTime date.
     */
    private Calendar serviceStartTime;

    /**
     * document source Patient ID.
     */
    private String sourcePatientId;




    public VLERDocument() {
        super();
    }

    public byte[] getDocument() {
        byte[] rtc = null;
        if (document != null) {
            rtc = document.clone();
        }
        return rtc;
    }

    public void setDocument(byte[] document) {
        if (document == null) {
            this.document = null;
        } else {
            this.document = document.clone();
        }
    }

    public String getHomeCommunityId() {
        return homeCommunityId;
    }

    public void setHomeCommunityId(String homeCommunityId) {
        this.homeCommunityId = homeCommunityId;
    }

    public String getRepositoryUniqueId() {
        return repositoryUniqueId;
    }

    public void setRepositoryUniqueId(String repositoryUniqueId) {
        this.repositoryUniqueId = repositoryUniqueId;
    }

    public String getDocumentUniqueId() {
        return documentUniqueId;
    }

    public void setDocumentUniqueId(String documentUniqueId) {
        this.documentUniqueId = documentUniqueId;
    }

    public String getMimeType() {
        return mimeType;
    }

    public void setMimeType(String mimeType) {
        this.mimeType = mimeType;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<Author> getAuthorList() {
        return authorList;
    }

    public void setAuthorList(List<Author> authorList) {
        this.authorList = authorList;
    }

    public Calendar getCreationTime() {
        return creationTime;
    }

    public Calendar getServiceStartTime() {
        return serviceStartTime;
    }

    public void setServiceStartTime(Calendar serviceStartTime) {
        this.serviceStartTime = serviceStartTime;
    }

    public void setCreationTime(Calendar creationTime) {
        this.creationTime = creationTime;
    }

    public String getSourcePatientId() {
        return sourcePatientId;
    }

    public void setSourcePatientId(String sourcePatientId) {
        this.sourcePatientId = sourcePatientId;
    }
}