package gov.va.med.vds.dataservice;

import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.patient.dao.VixDao;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import gov.va.med.vds.vistabeans.vix.StudyDetail;
import gov.va.med.vds.vistabeans.vix.StudyQuery;
import org.junit.Test;

import java.awt.image.BufferedImage;

import static org.junit.Assert.assertTrue;

public class VixDataServiceTest {
    private QueryBean initQueryBean()
    {
        // Required query request inputs
        Site site = new Site();
        site.setName("");
        site.setSiteCode("");

        User user = new User();
        user.setName("");
        user.setUserIen("");
        user.setEncryptedFederatedUid(EncryptUtil.encrypt(""));
        user.setBseToken("");
        user.setHostSite(site);

        Patient patient = new Patient();
        patient.setICN("");

        QueryBean queryBean = new QueryBean();
        queryBean.setUser(user);
        queryBean.setPatient(patient);

        // Study query context ID(s) delimited by ';'. Empty string or null returns all.
        queryBean.setItemId("");

        return queryBean;
    }

    @Test
    public void testGetStudyQueryFromSite()
    {
        QueryBean queryBean = initQueryBean();

        try {
            VixDao vixDao = new VixDao();
            StudyQuery studyQuery = vixDao.getStudyQueryFromSite(queryBean);
            assertTrue(studyQuery != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

    @Test
    public void testGetStudyQueryWithDetails()
    {
        QueryBean queryBean = initQueryBean();

        try {
            VixDao vixDao = new VixDao();
            StudyQuery studyQuery = vixDao.getStudyQueryWithDetails(queryBean);
            assertTrue(studyQuery != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

    @Test
    public void testStudyQueryWithoutDetails()
    {
        QueryBean queryBean = initQueryBean();

        try {
            VixDao vixDao = new VixDao();
            StudyQuery studyQuery = vixDao.getStudyQueryWithoutDetails(queryBean);
            assertTrue(studyQuery != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

    @Test
    public void testStudyDetails()
    {
        QueryBean queryBean = new QueryBean();
        queryBean.setItemId(""); // Study details URL

        try {
            VixDao vixDao = new VixDao();
            StudyDetail studyDetail = vixDao.getStudyDetail(queryBean);
            assertTrue(studyDetail != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

    @Test
    public void testThumbnail()
    {
        QueryBean queryBean = new QueryBean();
        queryBean.setItemId(""); // Thumbnail URL

        try {
            VixDao vixDao = new VixDao();
            BufferedImage bufferedImage = vixDao.getThumbnail(queryBean);
            assertTrue(bufferedImage != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

    @Test
    public void testStudyViewerUrl()
    {
        QueryBean queryBean = initQueryBean();

        try {
            VixDao vixDao = new VixDao();
            String studyViewerUrl = vixDao.getStudyViewerUrl(queryBean);
            assertTrue(studyViewerUrl != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

}
