/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.das;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.das.beans.Resource;
import gov.va.med.vds.das.beans.Subscription;
import gov.va.med.vds.dataservicehandler.DASServiceHandler;
import gov.va.med.vds.patient.dao.DASDao;
import gov.va.med.vds.patient.dataservice.AllergyDataService;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vds.vistabeans.das.DASResultCollection;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.LogManager;
import org.junit.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import static org.junit.Assert.*;
/**
 *
 *
 */
public class DASDataServiceTest //extends DataServiceBaseTest
{

    protected User user;
    protected Patient patient;
    protected String testPatientName;

    protected Site site;

    public DASDataServiceTest(){
        site = TestSiteUtils.getInstance().getSite(TestConfig.get(TestConfig.PROVIDER_SITE_CODE));
    }

    //**********
    //
    // To test the DAS VLER interface, modify the doGet() method in the
    // Notifications servlet (gov.va.med.vds.servlet.Notifications)
    // Ensure that the configuration in the .properties file is pointing to the proper endpoint:
    // appconfig.vler.das.callback = http://IP        :[port]/VistaDataService/notify/
    // After deploying to glassfish in debug mode, you can then kick off your test via:
    // http://IP        :[port]/VistaDataService/notify/blahblah
    //
    //**********
    @Test
    public void testGetVLERAllergies()
    {
////        QueryBean queryBean = initQueryBean();
        QueryBean queryBean = setupQuery(user, patient, null, null);

        try {
            DASDao dasDao = new DASDao();
            DASResultCollection resultCollection = dasDao.getVLERAllergies(queryBean);
            assertTrue(resultCollection != null);
        }
        catch (VDSException e) {
            assertTrue(false);
        }
    }

    @Test
    public void testGetDASData()
    {
////        QueryBean queryBean = initQueryBean();
        QueryBean queryBean = setupQuery(user, patient, null, null);

        try {
            DASServiceHandler dh = new DASServiceHandler();
            Subscription subscription = new Subscription();
            subscription.setPatientICN("106");
            subscription.setDomain("AllergyIntolerance");

            DASResultCollection resultCollection = dh.getDASData(subscription);
            assertTrue(resultCollection != null);
        }
        catch (Exception e) {
            assertTrue(false);
        }
    }

    @Test
    public void testParseDocStatus() throws Exception
    {
        try
        {
            String docStatus = "<wrap><documents><document><PartnerName>280004</PartnerName><PartnerDocumentID>2.16.840.1.113883.1.13.99999.999362</PartnerDocumentID><status>Available</status><timestamp>1483709141908</timestamp></document></documents><documents><document><PartnerName>KP</PartnerName><PartnerDocumentID>KP1</PartnerDocumentID><status>AvailableOld</status><timestamp></timestamp></document></documents></wrap>";

            final String XPATH_DOCUMENTS = "//document";
            final String XPATH_PARTNERNAME = "./PartnerName";
            final String XPATH_DOCUMENTID = "./PartnerDocumentID";
            final String XPATH_STATUS = "./status";
            final String XPATH_TIMESTAMP = "./timestamp";

            InputSource is = new InputSource(new StringReader(docStatus));
            Document xmlDoc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(is);

            XPath xpath = XPathFactory.newInstance().newXPath();
            XPathExpression exprDocs = xpath.compile(XPATH_DOCUMENTS);
            XPathExpression exprStatus = xpath.compile(XPATH_STATUS);
            XPathExpression exprPartnerName = xpath.compile(XPATH_PARTNERNAME);
            XPathExpression exprDocID = xpath.compile(XPATH_DOCUMENTID);
            XPathExpression exprTimestamp = xpath.compile(XPATH_TIMESTAMP);

            NodeList documents = (NodeList) exprDocs.evaluate(xmlDoc, XPathConstants.NODESET);

            List<gov.va.med.vds.das.beans.Document> documentList = new ArrayList<gov.va.med.vds.das.beans.Document>();

            for(int i = 0; i < documents.getLength(); i++) {
                Node doc = documents.item(i);
                gov.va.med.vds.das.beans.Document document = new gov.va.med.vds.das.beans.Document();
                String status = (String) exprStatus.evaluate(doc, XPathConstants.STRING);
                document.setPartnerDocumentID((String) exprDocID.evaluate(doc, XPathConstants.STRING));
                document.setPartnerName((String) exprPartnerName.evaluate(doc, XPathConstants.STRING));
                if(StringUtils.isNumeric((String) exprTimestamp.evaluate(doc, XPathConstants.STRING))) {
                    document.setTimestamp(Long.parseLong((String) exprTimestamp.evaluate(doc, XPathConstants.STRING)));
                }
                document.setStatus(status);
                documentList.add(document);
            }

            assertTrue(documentList.size() > 0);
//            for(Allergy allergy : allergies)
//            {
//                assertFalse(StringUtils.isEmpty(allergy.getSourceProtocol()));
//                assertFalse(allergy.getSite() == null);
//                assertFalse(allergy.getAllergyName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
//            }
        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERAllergies() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/NestedAllergyBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            objectMapper.configure(DeserializationFeature.FAIL_ON_INVALID_SUBTYPE, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERAllergies(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERProblems() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/ConditionBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERProblems(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERVitals() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/ObservationBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERVitals(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERImmunizations() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/ImmunizationBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERImmunizations(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERProcedures() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/ProcedureBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERProcedures(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERDemographics() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/PatientBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERDemographics(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testParseVLERDocuments() throws Exception
    {
        try
        {
            String input = IOUtils.toString(getClass().getResourceAsStream("/DocumentReferenceBundle.json"), "UTF-8");

            ObjectMapper objectMapper;
            objectMapper = new ObjectMapper();
            //allow unknown data fields
            objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            Resource resource = (Resource) objectMapper.readValue(input, Resource.class);

            DASServiceHandler dh  = new DASServiceHandler();
            DASResultCollection rc = dh.mapVLERDocuments(resource, new Subscription()) ;
            assertTrue(resource.getTotal() > 0);

        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testToken()
    {
        try
        {
            DASServiceHandler dh = new DASServiceHandler();
            QueryBean qb = new QueryBean();
            Patient pat = new Patient();
            pat.setICN("106");

            User user = new User();
            user.setName("TestUser");
            user.setUserId("987");

            qb.setPatient(pat);
            qb.setUser(user);

            String subString = dh.generateSubscription(qb, "AllergyIntolerance");
            //String subString = dh.generateSubscriptionString("Allergy", dh.generateSubscriptionString(qb, "AllergyIntolerance"));

            assertTrue(subString.length() > 0);


        }
        catch(Exception ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetAllergyDetail()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            AllergyDataService allergyService = new AllergyDataService();
            List<Allergy> allergies = allergyService.getPatientAllergies(queryBean);
            assertTrue(allergies.size() > 0);

            QueryBean qb2 = new QueryBean();
            qb2.setUser(user);
            qb2.setPatient(patient);
            qb2.setItemId(allergies.get(0).getAllergyId());
            qb2.setRecordSite(allergies.get(0).getSite());

            AllergyDetail allergyDetail = allergyService.getAllergyDetail(qb2);
            assertFalse(StringUtils.isEmpty(allergyDetail.getReportText()));
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    protected QueryBean setupQuery(User user, Patient patient, Calendar startDate, Calendar endDate)
    {
        QueryBean qb = new QueryBean();
        qb.setUser(user);
        qb.setPatient(patient);
        qb.setStartDate(startDate);
        qb.setEndDate(endDate);
        qb.setMax(new Integer(999));
        qb.setActive(new Integer(0));
        return qb;
    }


    @Before
    public void setUp()
    {
        LogManager.getRootLogger().setLevel(org.apache.log4j.Level.DEBUG);

        try
        {
            testPatientName = TestConfig.get(TestConfig.PATIENT_NAME);

            //select test patient
            PatientDataService patientService = new PatientDataService();
            QueryBean queryBean = new QueryBean();


            List<Endpoint> endpoints = new ArrayList<Endpoint>();
            Endpoint endpoint = new Endpoint();
            endpoint.setModality("HIS");
            endpoint.setProtocol("RPC");
            endpoint.setStatus("active");
            endpoint.setHost("IP          ");
            endpoint.setPort(PORT);
            endpoint.setSiteCode("994");
            endpoints.add(endpoint);

            site = new Site();
            site.setAgency("VA");
            site.setSiteCode("994");
            site.setEndpoints(endpoints);

            //user = loginTestProvider();
            user = new User();
            user.setAgency("VA");
            user.setName("TESTER,USER");
            user.setUserId("111");
            user.setHostSite(site);

            queryBean.setUser(user);
            queryBean.setMax(100);
            queryBean.setRecordSite(user.getHostSite());
            queryBean.setItemId(testPatientName);

            List<Site> sites = new ArrayList<Site>();
            sites.add(site);

            patient = new Patient();
            patient.setICN("106");
            patient.setName("TESTING,PATIENT");
            patient.setVistaSites(sites);
            patient.setSSN("444331111");

            //Patient[] patients = patientService.lookupPatient(queryBean);
            queryBean = new QueryBean();

            queryBean.setPatient(patient);
            queryBean.setUser(user);

//            if(!select(queryBean))
//                fail("Failed to select patient");

//            this.patient = patients[0];

        }
        catch (Exception ex)
        {
            fail(ex.getMessage());
//        } catch (VistaKernelHashCountLimitExceededException e) {
//            fail(e.getMessage());
        }
    }
    @BeforeClass
    public static void setUpClass()
    {

    }

    @AfterClass
    public static void tearDownClass()
    {
    }
    @After
    public void tearDown()
    {

    }
    
}
