/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.AdmissionsDataService;
import gov.va.med.vds.vistabeans.AdmissionDetail;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class AdmissionsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientAdmissions()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2003, 3, 1);
            endDate.set(2004, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            AdmissionsDataService admService = new AdmissionsDataService();
            List<PatientAdmission> adms = admService.getPatientAdmissions(queryBean);
            assertTrue(adms.size() > 0);
            for(PatientAdmission adm : adms)
            {
                assertFalse(StringUtils.isEmpty(adm.getSourceProtocol()));
                assertFalse(adm.getSite() == null);
                if (adm.getDrg() != null)
                    assertFalse(adm.getDrg().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetAdmissionDetails()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2003, 3, 1);
            endDate.set(2004, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            AdmissionsDataService admService = new AdmissionsDataService();
            List<PatientAdmission> adms = admService.getPatientAdmissions(queryBean);
            assertTrue(adms.size() > 0);

            queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setRecordSite(adms.get(0).getSite());
            queryBean.setItemId(adms.get(0).getAdmissionId());
            AdmissionDetail detail = admService.getAdmissionDetails(queryBean);
            assertTrue(detail.getAdmissionId().equals(adms.get(0).getAdmissionId()));
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
