package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.HealthSummariesDataService;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.HealthSummaryTypeList;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.HealthSummaryType;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;

/**
 *
 * Created by vacoishiis on 6/15/2017.
 */
public class HealthSummariesDataServiceTest extends DataServiceBaseTest {

    @Test
    public void testGetPatientHealthSummaryTypes() {
        try {
            QueryBean queryBean = setupQuery(user, patient);

            HealthSummariesDataService healthSummariesDataService = new HealthSummariesDataService();
            List<HealthSummaryTypeList> healthSummaryTypeListList = healthSummariesDataService.getPatientHealthSummaryTypes(queryBean);
            assertTrue(healthSummaryTypeListList.size() > 0);
            for (HealthSummaryTypeList healthSummaryTypeList : healthSummaryTypeListList) {
                assertFalse(StringUtils.isEmpty(healthSummaryTypeList.getSourceProtocol()));
                assertFalse(healthSummaryTypeList.getSite() == null);
                assertFalse(healthSummaryTypeList.getHealthSummaryTypes() == null);
                if (healthSummaryTypeList.getHealthSummaryTypes().length > 0) {
                    assertFalse(healthSummaryTypeList.getHealthSummaryTypes()[0].getName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
                }
            }

        } catch (VDSException ve) {
            fail(ve.getMessage());
        }
    }

    @Test
    public void testGetPatientHealthSummary() {
        try {
            QueryBean queryBean = setupQuery(user, patient);

            HealthSummariesDataService healthSummariesDataService = new HealthSummariesDataService();
            List<HealthSummaryTypeList> healthSummaryTypeListList = healthSummariesDataService.getPatientHealthSummaryTypes(queryBean);
            assertTrue(healthSummaryTypeListList.size() > 0);
            for (HealthSummaryTypeList healthSummaryTypeList : healthSummaryTypeListList) {
                assertFalse(StringUtils.isEmpty(healthSummaryTypeList.getSourceProtocol()));
                assertFalse(healthSummaryTypeList.getSite() == null);
                assertFalse(healthSummaryTypeList.getHealthSummaryTypes() == null);
                if (healthSummaryTypeList.getHealthSummaryTypes().length > 0) {
                    assertFalse(healthSummaryTypeList.getHealthSummaryTypes()[0].getName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
                }
            }
            HealthSummaryTypeList healthSummaryTypeList = healthSummaryTypeListList.get(0);

            for (HealthSummaryType healthSummaryType : healthSummaryTypeList.getHealthSummaryTypes()) {
                String summaryIen = healthSummaryType.getId();
                queryBean.setItemId(summaryIen);
                queryBean.setRecordSite(healthSummaryTypeList.getSite());
                FreeTextReport healthSummary = healthSummariesDataService.getPatientHealthSummary(queryBean);

                assertTrue(healthSummary.getReportText() != null);
                assertTrue(healthSummary.getReportText().length() > 0);
                assertFalse(StringUtils.isEmpty(healthSummary.getSourceProtocol()));
                assertFalse(healthSummary.getSite() == null);
                assertFalse(healthSummary.getReportText().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }

        } catch (VDSException ve) {
            fail(ve.getMessage());
        }
    }


}
