/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.PCEDataService;
import gov.va.med.vds.vistabeans.PCE;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class PCEDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientSkinTests()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2003, 0, 1);
            endDate.set(2016, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            PCEDataService pceService = new PCEDataService();
            List<PCE> skinTests = pceService.getPatientSkinTests(queryBean);
            assertTrue(skinTests.size() > 0);
            for(PCE skinTest : skinTests)
            {
                assertFalse(StringUtils.isEmpty(skinTest.getSourceProtocol()));
                assertFalse(skinTest.getSite() == null);
                assertFalse(skinTest.getName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientExams()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2007, 0, 1);
            endDate.set(2011, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            PCEDataService pceService = new PCEDataService();
            List<PCE> exams = pceService.getPatientExams(queryBean);
            assertTrue(exams.size() > 0);
            for(PCE exam : exams)
            {
                assertFalse(StringUtils.isEmpty(exam.getSourceProtocol()));
                assertFalse(exam.getSite() == null);
                assertFalse(exam.getName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientEducationTopics()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2007, 0, 1);
            endDate.set(2011, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            PCEDataService pceService = new PCEDataService();
            List<PCE> educationTopics = pceService.getPatientEducationTopics(queryBean);
            assertTrue(educationTopics.size() > 0);
            for(PCE educationTopic : educationTopics)
            {
                assertFalse(StringUtils.isEmpty(educationTopic.getSourceProtocol()));
                assertFalse(educationTopic.getSite() == null);
                assertFalse(educationTopic.getName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
    
}
