/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.provider.dataservice.UserDataService;
import gov.va.med.vds.vistabeans.Endpoint;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 *
 */
public class UserDataServiceTest
{
    
    private static final Logger logger = LoggerFactory.getLogger(UserDataServiceTest.class);

    private Site site;

    @Before
    public void setUp()
    {
        site = TestSiteUtils.getInstance().getSite(TestConfig.get(TestConfig.PROVIDER_SITE_CODE));
    }


    @Test
    public void testLogin()
    {
        try
        {
            String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
            String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);
            String siteCode = TestConfig.get(TestConfig.PROVIDER_SITE_CODE);

            //login as test user
            UserDataService providerService = new UserDataService();
            User user = providerService.login(site, VistaKernelHash.encrypt(accessCode, false), VistaKernelHash.encrypt(verifyCode, false));
            if (user.getUserIen().contains("^"))
                throw new RuntimeException("Invalid user");
            logger.debug("User Name: " + user.getName());
            assertTrue(user.getName().equals("HUI,TEST PROVIDER"));
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }
}
