/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.ehr.common;

import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.provider.dataservice.UserDataService;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;

import java.util.Calendar;
import java.util.logging.Level;
import java.util.logging.Logger;

public class DataServiceBaseTest
{
    private static final Logger logger = Logger.getLogger(DataServiceBaseTest.class.getName());

    protected User user;
    protected Patient patient;

    protected String accessCode;
    protected String verifyCode;
    protected String siteCode;

    protected Site site;

    @BeforeClass
    public static void setUpClass()
    {
    }

    @AfterClass
    public static void tearDownClass()
    {
    }

    @Before
    public void setUp()
    {
        accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
        verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);
        siteCode = TestConfig.get(TestConfig.PROVIDER_SITE_CODE);

        site = TestSiteUtils.getInstance().getSite(siteCode);

        try {
            user = providerLogin();
        } catch (VistaKernelHashCountLimitExceededException e) {
            logger.severe(e.getMessage());
        } catch (VDSException e) {
           logger.severe(e.getMessage());
        }
        patient = selectPatientByName(user, TestConfig.get(TestConfig.PATIENT_NAME));
    }

    protected User providerLogin() throws VistaKernelHashCountLimitExceededException, VDSException {
        String encryptedAccessCode = VistaKernelHash.encrypt(accessCode, false);
        String encryptedVerifyCode = VistaKernelHash.encrypt(verifyCode, false);
        UserDataService providerService = new UserDataService();
        return providerService.login(site, encryptedAccessCode, encryptedVerifyCode, true);
    }

    @After
    public void tearDown()
    {
    }

    protected QueryBean setupQuery(User user, Patient patient, Calendar startDate, Calendar endDate)
    {
        QueryBean qb = new QueryBean();
        qb.setUser(user);
        qb.setPatient(patient);
        qb.setStartDate(startDate);
        qb.setEndDate(endDate);
        qb.setMax(new Integer(999));
        qb.setActive(new Integer(0));
        return qb;
    }

    protected Patient selectPatientByName(User user, String patientName)
    {
        try
        {
            PatientDataService pds = new PatientDataService();
            QueryBean qb = new QueryBean();
            qb.setItemId(patientName);
            qb.setUser(user);
            qb.setRecordSite(user.getHostSite());
            Patient[] patients = pds.lookupPatient(qb);
            if(patients != null && patients.length > 0)
            {
                qb = new QueryBean();
                qb.setPatient(patients[0]);
                qb.setUser(user);
                qb.setRecordSite(user.getHostSite());
                if(pds.selectPatient(qb));
                    return patients[0];
            }
        }
        catch (VDSException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        }
        return null;
    }

    protected User providerLoginAs(String accessCode, String verifyCode,
            Site site) throws VDSException
    {
        try
        {
            String encryptedAccessCode = VistaKernelHash.encrypt(accessCode, false);
            String encryptedVerifyCode = VistaKernelHash.encrypt(verifyCode, false);
            UserDataService pds = new UserDataService();
            User user = pds.login(site, encryptedAccessCode, encryptedVerifyCode, true);

            if (user.getUserIen().contains("^"))
                throw new RuntimeException("Invalid user");

            return user;
        }
        catch (Exception ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
            throw new VDSException(ex.getMessage());
        }
    }
}
