/*
 * Janus 4.0 (c)
 * Copyright (c) 2015 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.EncountersRPC;
import gov.va.med.vds.vistabeans.Encounter;
import org.junit.Test;

import java.util.Calendar;
import java.util.logging.Logger;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;

/**
 *
 *
 */
public class EncountersRPCTest
{
    private static final Logger logger = Logger.getLogger(EncountersRPCTest.class.getName());

    @Test
    public void testGetPatientNonAdmissionsFromEncounters() throws RPCException
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.query(matches(".*visits.*"))).thenReturn(VPR_VISITS);

        EncountersRPC encountersRPC = new EncountersRPC(mockRPCConnection);
        Calendar date = Calendar.getInstance();
        String patientIen = "0";

        Encounter[] encounters = encountersRPC.getPatientNonAdmissionsFromEncounters(patientIen, date, date);
        assertTrue(encounters.length == 4);
        for(Encounter encounter : encounters) {
            assertFalse(encounter.getArrivalDateTime() != null);
        }
    }

    @Test
    public void testGetPatientAdmissionsFromEncounters() throws RPCException
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.query(matches(".*visits.*"))).thenReturn(VPR_VISITS);

        EncountersRPC encountersRPC = new EncountersRPC(mockRPCConnection);
        Calendar date = Calendar.getInstance();
        String patientIen = "0";

        Encounter[] encounters = encountersRPC.getPatientAdmissionsFromEncounters(patientIen, date, date);
        assertTrue(encounters.length == 2);
        for(Encounter encounter : encounters) {
            assertTrue(encounter.getArrivalDateTime() != null);
        }
    }


    private static final String VPR_VISITS = "<results version='1.1' timeZone='-0500'>\n" +
            "    <visits total='6'>" +
            "        <visit>\n" +
            "            <dateTime value='3050523.1'/>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <id value='4096'/>\n" +
            "            <location value='GENERAL MEDICINE'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='301' name='GENERAL INTERNAL MEDICINE'/>\n" +
            "            <type name='GENERAL MEDICINE VISIT'/>\n" +
            "            <visitString value='23;3050523.1;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <cpts>\n" +
            "                <cpt code='99201' name='OFFICE/OUTPATIENT VISIT NEW'/>\n" +
            "            </cpts>\n" +
            "            <dateTime value='3040402.030001'/>\n" +
            "            <documents>\n" +
            "                <document id='3220' localTitle='AUDIOLOGY - HEARING LOSS CONSULT'/>\n" +
            "            </documents>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <icds>\n" +
            "                <icd code='389.9' name='UNSPECIFIED HEARING LOSS' narrative='Hearing loss' ranking='PRIMARY'/>\n" +
            "            </icds>\n" +
            "            <id value='3789'/>\n" +
            "            <location value='GENERAL MEDICINE'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <providers>\n" +
            "                <provider code='11748' name='PATHOLOGY,ONE' role='P' primary='1' taxonomyCode='203B00000N' providerType='Physicians (M.D. and D.O.)' classification='Physician/Osteopath'/>\n" +
            "            </providers>\n" +
            "            <reason code='389.9' name='UNSPECIFIED HEARING LOSS' narrative='Hearing loss'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='301' name='GENERAL INTERNAL MEDICINE'/>\n" +
            "            <type code='99201' name='OFFICE/OUTPATIENT VISIT NEW'/>\n" +
            "            <visitString value='23;3040402.030001;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <dateTime value='3040331.161625'/>\n" +
            "            <documents>\n" +
            "                <document id='3006' localTitle='GENERAL MEDICINE NOTE'/>\n" +
            "            </documents>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <id value='3574'/>\n" +
            "            <location value='GENERAL MEDICINE'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='301' name='GENERAL INTERNAL MEDICINE'/>\n" +
            "            <type name='GENERAL MEDICINE VISIT'/>\n" +
            "            <visitString value='23;3040331.161625;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <dateTime value='3040331.124716'/>\n" +
            "            <documents>\n" +
            "                <document id='2875' localTitle='DIABETES'/>\n" +
            "            </documents>\n" +
            "            <facility code='888' name='FT. LOGAN'/>\n" +
            "            <id value='3443'/>\n" +
            "            <location value='DIABETIC'/>\n" +
            "            <patientClass value='AMB'/>\n" +
            "            <serviceCategory code='A' name='AMBULATORY'/>\n" +
            "            <stopCode code='306' name='DIABETES'/>\n" +
            "            <type name='DIABETIC VISIT'/>\n" +
            "            <visitString value='285;3040331.124716;A'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <cpts>\n" +
            "                <cpt code='99201' name='OFFICE/OUTPATIENT VISIT NEW'/>\n" +
            "                <cpt code='99202' name='OFFICE/OUTPATIENT VISIT NEW2'/>" +
            "            </cpts>\n" +
            "            <arrivalDateTime value='3040325.094947'/>\n" +
            "            <dateTime value='3040325.094947'/>\n" +
            "            <departureDateTime value='3040325.193123'/>\n" +
            "            <documents>\n" +
            "                <document id='3098' localTitle='Discharge Summary' nationalTitle='DISCHARGE SUMMARY'/>\n" +
            "            </documents>\n" +
            "            <facility code='998' name='ABILENE (CAA)'/>\n" +
            "            <id value='3667'/>\n" +
            "            <location value='7A GEN MED'/>\n" +
            "            <patientClass value='IMP'/>\n" +
            "            <providers>\n" +
            "                <provider code='20349' name='VEHU,ONEHUNDREDSIXTEEN' role='A'/>\n" +
            "            </providers>\n" +
            "            <reason narrative='htn'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='H' name='HOSPITALIZATION'/>\n" +
            "            <specialty value='GENERAL MEDICINE'/>\n" +
            "            <type name='HOSPITALIZATION'/>\n" +
            "            <visitString value='158;3040325.094947;H'/>\n" +
            "        </visit>\n" +
            "        <visit>\n" +
            "            <arrivalDateTime value='3030415.171245'/>\n" +
            "            <dateTime value='3030415.171245'/>\n" +
            "            <departureDateTime value='3040126.110114'/>\n" +
            "            <documents>\n" +
            "                <document id='2254' localTitle='ADMISSION REVIEW - NURSING'/>\n" +
            "            </documents>\n" +
            "            <facility code='998' name='ABILENE (CAA)'/>\n" +
            "            <id value='3158'/>\n" +
            "            <location value='3 NORTH SURG'/>\n" +
            "            <patientClass value='IMP'/>\n" +
            "            <providers>\n" +
            "                <provider code='20024' name='VEHU,TWENTYFOUR' role='P' primary='1'/>\n" +
            "                <provider code='20024' name='VEHU,TWENTYFOUR' role='A'/>\n" +
            "            </providers>\n" +
            "            <reason narrative='RIH'/>\n" +
            "            <roomBed value='AS-8'/>\n" +
            "            <service value='MEDICINE'/>\n" +
            "            <serviceCategory code='H' name='HOSPITALIZATION'/>\n" +
            "            <specialty value='GENERAL MEDICINE'/>\n" +
            "            <type name='HOSPITALIZATION'/>\n" +
            "            <visitString value='5;3030415.171245;H'/>\n" +
            "        </visit>" +
            "    </visits>\n" +
            "</results>";

}
