/*
 * Janus 4.0 (c)
 * Copyright (c) 2015 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.vds.patient.dao.rpc.LabsRPC;
import gov.va.med.vds.vistabeans.LabResult;
import org.junit.Test;

import java.lang.reflect.Method;
import java.util.ArrayList;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 *
 *
 */
public class LabsRPCTest2 {


    @Test
    public void testParseLabsFromVPR() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseLabsFromVPR";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = LABS_VPR_RESPONSE;

        Object labOrders = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labOrders);

//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReport1() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        ArrayList<LabResult> resultArrayList = (ArrayList<LabResult>) labResults;
        assertTrue(resultArrayList.size() == 4);
        assertTrue("Blood Type Testing".equals(resultArrayList.get(0).getTestName()));
        assertTrue("Blood Products Given".equals(resultArrayList.get(1).getTestName()));
        assertTrue("Blood Type Testing".equals(resultArrayList.get(2).getTestName()));
        assertTrue("Blood Products Given".equals(resultArrayList.get(3).getTestName()));

//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReport2() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_EMPTY;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 0);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReportProd() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_PRODUCTION;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 3);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReportProd2() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_PRODUCTION2;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 2);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReportNoReport() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = NO_BLOOD_BANK_REPORT;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 0);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseAutopsyReportToLabResult() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseAutopsyReportToLabResult";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = AUTOPSY_REPORT;

        Object labResult = method.invoke(rpc, parameters);

        assertNotNull(labResult);
        assertTrue("AUTOPSY".equals(((LabResult) labResult).getTestName()));

    }

    @Test
    public void testParseBloodbankReportNew022017() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_PRODUCTION_NEW_022017;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 9);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReportBroken1() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_PRODUCTION_BAD_1;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 4);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReportBroken2() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_PRODUCTION_BAD2;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 1);
//        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testParseBloodbankReportBroken3() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        LabsRPC rpc = new LabsRPC(mockRPCConnection);
        String methodName = "parseBloodbankReport";
//        String methodName = "privateTestMethod";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = BLOOD_BANK_REPORT_PRODUCTION_NEW_022017_BAD_TRANS;

        Object labResults = method.invoke(rpc, parameters);
//        method.invoke(rpc, parameters);

        assertNotNull(labResults);
        assertTrue(((ArrayList<LabResult>) labResults).size() == 9);
//        assertTrue(labOrders.length > 0);
    }

    private static final String LABS_VPR_RESPONSE = "<results version='1.05' timeZone='-0400'>\n" +
            "   <panels total='9'>\n" +
            "      <panel>\n" +
            "         <collected value='3140526.175519'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Padma Bodduluri Report Released Date/Time: May 26, 2014@17:57\n" +
            " Performing Lab: IPOTEST 1\n" +
            "                  \n" +
            " </comment>\n" +
            "         <facility code='451' name='IPOTEST 1'/>\n" +
            "         <groupName value='CHEM 0526 1'/>\n" +
            "         <id value='709;3140526;1;2'/>\n" +
            "         <order code='5587606' name='CHEM 7-8'/>\n" +
            "         <provider code='520736421' name='BODDULURI,PADMA' officePhone='7771235432' fax='3241235432' email='AAA@YAHOO.COM' taxonomyCode='146N00000X' providerType='Emergency Medical Service Providers' classification='Emergency Medical Technician, Basic' service='INFORMATION OFFICE'/>\n" +
            "         <resulted value='3140526.175742'/>\n" +
            "         <sample value='SERUM'/>\n" +
            "         <specimen code='0X500' name='SERUM'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6859472.824481;2' test='GLUCOSE' result='90' units='mg/dL' low='65' high='99' loinc='2345-7' vuid='4665460' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;3' test='UREA NITROGEN' result='10' units='mg/dL' low='7' high='18' loinc='3094-0' vuid='4673484' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;4' test='CREATININE' result='10' interpretation='H*' units='mg/dL' low='0.60' high='1.30' loinc='2160-0' vuid='4663483' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;5' test='SODIUM' result='10' interpretation='L*' units='meq/L' low='136' high='145' loinc='2951-2' vuid='4671912' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;6' test='POTASSIUM' result='10.0' interpretation='H*' units='meq/L' low='3.5' high='5.1' loinc='2823-3' vuid='4670505' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;7' test='CHLORIDE' result='10' interpretation='L' units='meq/L' low='98' high='107' loinc='2075-0' vuid='4662584' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;8' test='CO2' result='10.0' interpretation='L' units='meq/L' low='21.0' high='32.0' loinc='2028-9' vuid='4662085' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;9' test='CALCIUM' result='10.0' units='mg/dL' low='8.5' high='10.1' loinc='17861-6' vuid='4659597' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;790' test='ANION GAP' result='10.0' units='meq/L' low='3' high='14' loinc='33037-3' vuid='4675799' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;442660' test='eGFR' result='5.3' interpretation='L' units='mL/min' loinc='33914-3' vuid='4676763' performingLab='IPOTEST 1'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3140526.175519'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Padma Bodduluri Report Released Date/Time: May 26, 2014@17:57\n" +
            " Performing Lab: IPOTEST 1\n" +
            "                  \n" +
            " </comment>\n" +
            "         <facility code='451' name='IPOTEST 1'/>\n" +
            "         <groupName value='CHEM 0526 1'/>\n" +
            "         <id value='709;3140526;1;1'/>\n" +
            "         <order code='5587607' name='LIPID PROFILE'/>\n" +
            "         <provider code='520736421' name='BODDULURI,PADMA' officePhone='7771235432' fax='3241235432' email='AAA@YAHOO.COM' taxonomyCode='146N00000X' providerType='Emergency Medical Service Providers' classification='Emergency Medical Technician, Basic' service='INFORMATION OFFICE'/>\n" +
            "         <resulted value='3140526.175742'/>\n" +
            "         <sample value='SERUM'/>\n" +
            "         <specimen code='0X500' name='SERUM'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6859472.824481;12' test='CHOLESTEROL' result='70' units='MG/DL' low='0' high='200' loinc='2093-3' vuid='4662777' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;47' test='TRIGLYCERIDE' result='10' units='mg/dL' low='0' high='150' loinc='2571-8' vuid='4667823' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;80' test='HDL' result='35' interpretation='L' units='MG/DL' loinc='2085-9' vuid='4662693' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;291' test='LDL CHOLESTEROL (CALC)' result='25' units='mg/dL' low='0' high='130' loinc='13457-7' vuid='4654922' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;442072' test='VERY LOW DENSITY LIPOPROTEIN' result='15' units='mg/dl' loinc='13458-5' vuid='4654923' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;442385' test='LDL TOTAL DIRECT' result='16' units='MG/DL' low='0' high='130' loinc='18262-6' vuid='4660038' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859472.824481;442473' test='CARDIAC RISK FACTOR' result='15' interpretation='H' low='0.0' high='5.5' loinc='9830-1' vuid='4688295' performingLab='IPOTEST 1'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3140524.232908'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Padma Bodduluri Report Released Date/Time: May 24, 2014@23:29\n" +
            " Performing Lab: IPOTEST 1\n" +
            "                  \n" +
            " </comment>\n" +
            "         <facility code='451' name='IPOTEST 1'/>\n" +
            "         <groupName value='URINE 0524 1'/>\n" +
            "         <id value='519;3140524;1;1'/>\n" +
            "         <order code='5587603' name='URINALYSIS'/>\n" +
            "         <provider code='520736421' name='BODDULURI,PADMA' officePhone='7771235432' fax='3241235432' email='AAA@YAHOO.COM' taxonomyCode='146N00000X' providerType='Emergency Medical Service Providers' classification='Emergency Medical Technician, Basic' service='INFORMATION OFFICE'/>\n" +
            "         <resulted value='3140524.232956'/>\n" +
            "         <sample value='URINE'/>\n" +
            "         <specimen code='7X100' name='URINE'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6859474.767092;162' test='APPEARANCE' result='CLEAR' loinc='5767-9' vuid='4684011' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;683' test='URINE COLOR' result='WHITE' loinc='5778-6' vuid='4684023' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;685' test='SPECIFIC GRAVITY' result='1.010' low='1.005' high='1.025' loinc='5811-5' vuid='4684061' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;686' test='UROBILINOGEN' result='0.2' units='EU/DL' low='.1' high='1' loinc='19161-9' vuid='4660914' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;687' test='URINE BLOOD' result='NEG' loinc='33051-4' vuid='4675815' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;688' test='URINE BILIRUBIN' result='NEG' loinc='5770-3' vuid='4684015' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;689' test='URINE KETONES' result='NEGATIVE' units='MG/DL' loinc='5797-6' vuid='4684044' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;690' test='URINE GLUCOSE' result='NEG' units='MG/DL' loinc='2350-7' vuid='4665514' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;691' test='URINE PROTEIN' result='NEG' units='MG/DL' loinc='5804-0' vuid='4684053' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;795' test='NITRITE, URINE' result='NEG' loinc='5802-4' vuid='4684051' performingLab='IPOTEST 1'/>\n" +
            "            <value id='CH;6859474.767092;796' test='LEUKOCYTE ESTERASE, URINE' result='NEG' low='NEG' high='POS' loinc='5799-2' vuid='4684046' performingLab='IPOTEST 1'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3100806.095501'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Gonzalo D Scarfi MD Report Released Date/Time: Aug 06, 2010@17:11\n" +
            " Performing Lab: CHEYENNE VAMC [CLIA# 53D0987838]\n" +
            "                2360 E PERSHING BLVD 2360 East Pershing Boulevard CHEYENNE, WY 82001-5356\n" +
            " </comment>\n" +
            "         <facility code='442' name='CHEYENNE VAMC'/>\n" +
            "         <groupName value='CHEM 0806 102'/>\n" +
            "         <id value='CH;6899192.904499'/>\n" +
            "         <order name='ACCESSION'/>\n" +
            "         <provider code='2746' name='SCARFI,GONZALO D' taxonomyCode='207R00000X' providerType='Allopathic and Osteopathic Physicians' classification='Internal Medicine' service='MEDICAL SERVICE'/>\n" +
            "         <resulted value='3100806.171117'/>\n" +
            "         <sample value='SERUM'/>\n" +
            "         <specimen code='0X500' name='SERUM'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6899192.904499;2' test='GLUCOSE' result='130' interpretation='H' units='mg/dL' low='65' high='99' loinc='2345-7' vuid='4665460' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;3' test='UREA NITROGEN' result='12.6' units='mg/dL' low='7' high='18' loinc='3094-0' vuid='4673484' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;4' test='CREATININE' result='0.75' units='mg/dL' low='0.60' high='1.30' loinc='2160-0' vuid='4663483' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;5' test='SODIUM' result='137' units='meq/L' low='136' high='145' loinc='2951-2' vuid='4671912' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;6' test='POTASSIUM' result='4.5' units='meq/L' low='3.5' high='5.1' loinc='2823-3' vuid='4670505' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;7' test='CHLORIDE' result='102' units='meq/L' low='98' high='107' loinc='2075-0' vuid='4662584' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;8' test='CO2' result='28.9' units='meq/L' low='21.0' high='32.0' loinc='2028-9' vuid='4662085' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;9' test='CALCIUM' result='8.8' units='mg/dL' low='8.5' high='10.1' loinc='17861-6' vuid='4659597' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;12' test='CHOLESTEROL' result='100' units='MG/DL' low='0' high='200' loinc='2093-3' vuid='4662777' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;20' test='SGPT (ALT)' result='42' units='U/L' low='30' high='65' loinc='1742-6' vuid='4659145' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;47' test='TRIGLYCERIDE' result='74' units='mg/dL' low='0' high='150' loinc='2571-8' vuid='4667823' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;80' test='HDL' result='33.6' interpretation='L' units='MG/DL' loinc='2085-9' vuid='4662693' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;291' test='LDL CHOLESTEROL (CALC)' result='51.6' units='mg/dL' low='0' high='130' loinc='13457-7' vuid='4654922' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;790' test='ANION GAP' result='6.1' units='meq/L' low='3' high='14' loinc='33037-3' vuid='4675799' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;442072' test='VERY LOW DENSITY LIPOPROTEIN' result='14.8' units='mg/dl' loinc='13458-5' vuid='4654923' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;442385' test='LDL TOTAL DIRECT' result='52.9' units='MG/DL' low='0' high='130' loinc='18262-6' vuid='4660038' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;442473' test='CARDIAC RISK FACTOR' result='3.0' low='0.0' high='5.5' loinc='9830-1' vuid='4688295' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899192.904499;442660' test='eGFR' result='104.5' units='mL/min' loinc='33914-3' vuid='4676763' performingLab='CHEYENNE VAMC'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3100806.0955'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Gonzalo D Scarfi MD Report Released Date/Time: Aug 06, 2010@14:32\n" +
            " Performing Lab: FORT COLLINS CBOC\n" +
            "                2509 RESEARCH BLVD FORT COLLINS, CO 80526-8108\n" +
            " </comment>\n" +
            "         <facility code='442GC' name='FORT COLLINS'/>\n" +
            "         <groupName value='FC 0806 5'/>\n" +
            "         <id value='CH;6899192.9045'/>\n" +
            "         <order name='ACCESSION'/>\n" +
            "         <provider code='2746' name='SCARFI,GONZALO D' taxonomyCode='207R00000X' providerType='Allopathic and Osteopathic Physicians' classification='Internal Medicine' service='MEDICAL SERVICE'/>\n" +
            "         <resulted value='3100806.143207'/>\n" +
            "         <sample value='BLOOD'/>\n" +
            "         <specimen code='0X000' name='BLOOD'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6899192.9045;442708' test='POC A1C' result='7.1' interpretation='H' units='%' low='4' high='6' loinc='17856-6' vuid='4659591' performingLab='FORT COLLINS CBOC'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3100303.093'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Gonzalo D Scarfi MD Report Released Date/Time: Mar 03, 2010@15:23\n" +
            " Performing Lab: FORT COLLINS CBOC\n" +
            "                2509 RESEARCH BLVD FORT COLLINS, CO 80526-8108\n" +
            " </comment>\n" +
            "         <facility code='442GC' name='FORT COLLINS'/>\n" +
            "         <groupName value='FC 0303 2'/>\n" +
            "         <id value='CH;6899695.907'/>\n" +
            "         <order name='ACCESSION'/>\n" +
            "         <provider code='2746' name='SCARFI,GONZALO D' taxonomyCode='207R00000X' providerType='Allopathic and Osteopathic Physicians' classification='Internal Medicine' service='MEDICAL SERVICE'/>\n" +
            "         <resulted value='3100303.15235'/>\n" +
            "         <sample value='BLOOD'/>\n" +
            "         <specimen code='0X000' name='BLOOD'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6899695.907;442708' test='POC A1C' result='7.3' interpretation='H' units='%' low='4' high='6' loinc='17856-6' vuid='4659591' performingLab='FORT COLLINS CBOC'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3100303.092502'/>\n" +
            "         <comment xml:space='preserve'>For Test: MICROALBUMIN ** den 2783 LIMITATIONS: Heterophilic Abs can react with the assay components causing interference for this test. Evaluate levels carefully when the result is inappropriate for the apparent clinical state. Reference Ranges:  Males   0-16\n" +
            "                   Females 0-24\n" +
            " Although both the microalbumin and creatinine results are reported in mg/dL the ratio is multiplied by 1000 to convert the microalbumin result to mcg/dL which is the unit the reference range for the ratio is based on. interference with this test. Evaluate levels carefully when the result is inappropriate for the apparent clinical state. Performed at Eastern Colorado Health Care System, Denver, CO Ordering Provider: Gonzalo D Scarfi MD Report Released Date/Time: Mar 06, 2010@10:53\n" +
            " Performing Lab: EASTERN COLORADO HEALTH CARE SYSTEM - DENVER DIVSN  [CLIA# 06D0987873]\n" +
            "                1055 CLERMONT STREET DENVER, CO 80220\n" +
            " </comment>\n" +
            "         <facility code='442' name='CHEYENNE VAMC'/>\n" +
            "         <groupName value='DEN 10 2783'/>\n" +
            "         <id value='CH;6899695.907498'/>\n" +
            "         <order name='ACCESSION'/>\n" +
            "         <provider code='2746' name='SCARFI,GONZALO D' taxonomyCode='207R00000X' providerType='Allopathic and Osteopathic Physicians' classification='Internal Medicine' service='MEDICAL SERVICE'/>\n" +
            "         <resulted value='3100306.105332'/>\n" +
            "         <sample value='URINE'/>\n" +
            "         <specimen code='7X100' name='URINE'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6899695.907498;442235' test='Microalbumin' result='1.6' units='mg/dL' loinc='14957-5' vuid='4656545' performingLab='EASTERN COLORADO HEALTH CARE SYSTEM - DENVER DIVSN '/>\n" +
            "            <value id='CH;6899695.907498;442381' test='Albumin/Creat Ratio' result='9' units='RATIO' low='0' high='16' loinc='14959-1' vuid='4656547' performingLab='EASTERN COLORADO HEALTH CARE SYSTEM - DENVER DIVSN '/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3100303.092501'/>\n" +
            "         <comment xml:space='preserve'>TEST PERFORMED AT FT COLLINS CLINIC Ordering Provider: Gonzalo D Scarfi MD Report Released Date/Time: Mar 03, 2010@13:41\n" +
            " Performing Lab: CHEYENNE VAMC [CLIA# 53D0987838]\n" +
            "                2360 E PERSHING BLVD 2360 East Pershing Boulevard CHEYENNE, WY 82001-5356\n" +
            " </comment>\n" +
            "         <facility code='442' name='CHEYENNE VAMC'/>\n" +
            "         <groupName value='URINE 0303 49'/>\n" +
            "         <id value='CH;6899695.907499'/>\n" +
            "         <order name='ACCESSION'/>\n" +
            "         <provider code='2746' name='SCARFI,GONZALO D' taxonomyCode='207R00000X' providerType='Allopathic and Osteopathic Physicians' classification='Internal Medicine' service='MEDICAL SERVICE'/>\n" +
            "         <resulted value='3100303.134122'/>\n" +
            "         <sample value='URINE'/>\n" +
            "         <specimen code='7X100' name='URINE'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6899695.907499;162' test='APPEARANCE' result='CLEAR' loinc='5767-9' vuid='4684011' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;683' test='URINE COLOR' result='YELLOW' loinc='5778-6' vuid='4684023' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;685' test='SPECIFIC GRAVITY' result='1.005' low='1.005' high='1.025' loinc='5811-5' vuid='4684061' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;686' test='UROBILINOGEN' result='0.2' units='EU/DL' low='.1' high='1' loinc='19161-9' vuid='4660914' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;687' test='URINE BLOOD' result='NEG' loinc='33051-4' vuid='4675815' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;688' test='URINE BILIRUBIN' result='NEG' loinc='5770-3' vuid='4684015' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;689' test='URINE KETONES' result='NEGATIVE' units='MG/DL' loinc='5797-6' vuid='4684044' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;690' test='URINE GLUCOSE' result='NEG' units='MG/DL' loinc='2350-7' vuid='4665514' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;691' test='URINE PROTEIN' result='TRACE' units='MG/DL' loinc='5804-0' vuid='4684053' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;692' test='URINE PH' result='7.00' low='5' high='9' loinc='5803-2' vuid='4684052' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;795' test='NITRITE, URINE' result='NEG' loinc='5802-4' vuid='4684051' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.907499;796' test='LEUKOCYTE ESTERASE, URINE' result='SMALL' low='NEG' high='POS' loinc='5799-2' vuid='4684046' performingLab='CHEYENNE VAMC'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "      <panel>\n" +
            "         <collected value='3100303.0925'/>\n" +
            "         <comment xml:space='preserve'>Ordering Provider: Gonzalo D Scarfi MD Report Released Date/Time: Mar 03, 2010@15:46\n" +
            " Performing Lab: CHEYENNE VAMC [CLIA# 53D0987838]\n" +
            "                2360 E PERSHING BLVD 2360 East Pershing Boulevard CHEYENNE, WY 82001-5356\n" +
            " </comment>\n" +
            "         <facility code='442' name='CHEYENNE VAMC'/>\n" +
            "         <groupName value='CHEM 0303 187'/>\n" +
            "         <id value='CH;6899695.9075'/>\n" +
            "         <order name='ACCESSION'/>\n" +
            "         <provider code='2746' name='SCARFI,GONZALO D' taxonomyCode='207R00000X' providerType='Allopathic and Osteopathic Physicians' classification='Internal Medicine' service='MEDICAL SERVICE'/>\n" +
            "         <resulted value='3100303.154651'/>\n" +
            "         <sample value='SERUM'/>\n" +
            "         <specimen code='0X500' name='SERUM'/>\n" +
            "         <status value='completed'/>\n" +
            "         <type value='CH'/>\n" +
            "         <values>\n" +
            "            <value id='CH;6899695.9075;2' test='GLUCOSE' result='148' interpretation='H' units='mg/dL' low='65' high='99' loinc='2345-7' vuid='4665460' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;3' test='UREA NITROGEN' result='14.7' units='mg/dL' low='7' high='18' loinc='3094-0' vuid='4673484' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;4' test='CREATININE' result='0.82' units='mg/dL' low='0.60' high='1.30' loinc='2160-0' vuid='4663483' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;5' test='SODIUM' result='135' interpretation='L' units='meq/L' low='136' high='145' loinc='2951-2' vuid='4671912' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;6' test='POTASSIUM' result='4.1' units='meq/L' low='3.5' high='5.1' loinc='2823-3' vuid='4670505' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;7' test='CHLORIDE' result='98' units='meq/L' low='98' high='107' loinc='2075-0' vuid='4662584' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;8' test='CO2' result='28.3' units='meq/L' low='21.0' high='32.0' loinc='2028-9' vuid='4662085' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;9' test='CALCIUM' result='8.8' units='mg/dL' low='8.5' high='10.1' loinc='17861-6' vuid='4659597' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;790' test='ANION GAP' result='8.7' units='meq/L' low='3' high='14' loinc='33037-3' vuid='4675799' performingLab='CHEYENNE VAMC'/>\n" +
            "            <value id='CH;6899695.9075;442660' test='eGFR' result='94.3' units='mL/min' loinc='33914-3' vuid='4676763' performingLab='CHEYENNE VAMC'/>\n" +
            "         </values>\n" +
            "      </panel>\n" +
            "   </panels>\n" +
            "</results>\n";

    private static final String BLOOD_BANK_REPORT =
            " ******************************************************************\n" +
                    " *                                                                *\n" +
                    " *          Patient data is not available at this time            *\n" +
                    " *            due to a VistALink connection failure.              *\n" +
                    " *                                                                *\n" +
                    " *       Contact IRM and/or the Blood Bank for assistance.        *\n" +
                    " *                                                                *\n" +
                    " ******************************************************************\n" +
                    "********************************************************************************\n" +
                    "*                                                                              *\n" +
                    "*                                Error Message                                 *\n" +
                    "*                                                                              *\n" +
                    "*Unable to retrieve patient information at this time, please contact the Blood *\n" +
                    "*                  Bank. [restart VBECS VistALink listener]                    *\n" +
                    "*                                                                              *\n" +
                    "********************************************************************************\n" +
                    "                                                                               \n" +
                    "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
                    "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
                    "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
                    "   in this report may have been duplicated in the VBECS report above (if available).\n" +
                    "                                                                               \n" +
                    "                             ---- BLOOD BANK ----\n" +
                    "ABO Rh:  B POS\n" +
                    "                                                                               \n" +
                    "No UNITS assigned/xmatched\n" +
                    "                                                                               \n" +
                    "Component requests       Units Request date     Date wanted     Requestor   By\n" +
                    "LR|RBC|AS-3                2   01/15/2010@12:00 01/15/2010@12:00ENLOW       AS\n" +
                    "                                                                               \n" +
                    "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
                    "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
                    "   ---------       --- --    ---- ---  ---  --------------   -----------------\n" +
                    "   01/02/2014@12:00  B POS                                   Neg\n" +
                    "   03/09/2015@10:48:21 BPOS                                  Neg\n" +
                    "   08/12/2015@08:35  B POS                                   Neg\n" +
                    "   11/12/2015@08:35  B POS                                   Neg\n" +
                    "                                                                               \n" +
                    "Transfused Units\n" +
                    " Mar 10, 2015(2) LR\n" +
                    " Mar 12, 2015(2) LR\n" +
                    " Nov 12, 2015(2) LR\n" +
                    " Nov 13, 2015(2) LR\n" +
                    "  Blood Product Key: LR = LR|RBC|AS-3";

    private static final String BLOOD_BANK_REPORT_EMPTY =
            "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
                    "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
                    "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
                    "   in this report may have been duplicated in the VBECS report above (if available).\n" +
                    "                                                                               \n" +
                    "                             ---- BLOOD BANK ----\n" +
                    "ABO Rh:    \n" +
                    "                                                                               \n" +
                    "No UNITS assigned/xmatched\n" +
                    "                                                                               \n" +
                    "No component requests\n" +
                    "                                                                               \n" +
                    "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
                    "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
                    "   ---------       --- --    ---- ---  ---  --------------   -----------------\n";

    private static final String AUTOPSY_REPORT = "\r\n" +
            "                           ---- AUTOPSY ----\r\n" +
            "Acc #   Date/time Died    Age   AUTOPSY DATA        Date/time of Autopsy\r\n" +
            "AU 16 3 Jun 27, 2016 16:00 84   IPOBARCH,ELI D      Jun 28, 2016 17:00\r\n" +
            "Resident: ESPINOZA,JAQUELINE                           Senior: ESPINOZA,JAQUELINE\r\n" +
            "                                                                               \r\n" +
            "Clinical Diagnoses:\r\n" +
            "Gas gangrene in the abdomen.\r\n" +
            "Pathological Diagnoses:\r\n" +
            "Bacterial proliferation and gas accumulation were found in the sinusoids.\r\n" +
            "\r\n" +
            "Alcoholism the underlying condition.\r\n" +
            "\r\n";

    private static final String BLOOD_BANK_REPORT_PRODUCTION =
            "                             ---- BLOOD BANK ----\n" +
                    "ABO Rh: O  Pos\n" +
                    "                                                                               \n" +
                    "                                                                               \n" +
                    "                                                                               \n" +
                    "DIAGNOSTIC TESTS:\n" +
                    "                                 |---------------- DAT ----------------|\n" +
                    "                                 |-- Poly --|  |-- IgG --|  |-- Comp --|\n" +
                    " Date/Time        ABO Rh  ABS  Test   Intrp  Test  Intrp  Test   Intrp  Division\n" +
                    " ---------------  --- --- ---  ----   -----  ----  -----  ----   -----  --------\n" +
                    " 07/27/2016@05:11  O  Pos Neg                                           TAMPA FL VAMC\n" +
                    " 07/23/2016@04:29  O  Pos Neg                                           TAMPA FL VAMC\n" +
                    "  Comment (ABO): 1ST SAMPLE\n" +
                    " 07/23/2016@03:45  O  Pos                                               TAMPA FL VAMC\n" +
                    "  Comment (ABO): 2ND SAMPLE\n" +
                    "                                                                               \n" +
                    "                                                                               \n" +
                    "COMPONENT REQUESTS:\n" +
                    "                                                                               \n" +
                    " Component Type      Units Request date        Date wanted         Requestor By\n" +
                    " --------------      ----- ------------        -----------         --------- --\n" +
                    " RED BLOOD CELLS       1   Jul 29, 2016 04:00  Jul 29, 2016 04:00  LBC       LBC\n" +
                    " RED BLOOD CELLS       1   Jul 27, 2016 03:51  Jul 27, 2016 03:49  LBC       LBC\n" +
                    " RED BLOOD CELLS       1   Jul 26, 2016 05:33  Jul 26, 2016 05:33  LBC       LBC\n" +
                    " PLATELETS             2   Jul 25, 2016 08:36  Jul 25, 2016        PN        GO\n" +
                    " FRESH FROZEN PLASMA   2   Jul 25, 2016 08:36  Jul 25, 2016        PN        GO\n" +
                    " PLATELETS             2   Jul 22, 2016 11:09  Jul 25, 2016        CML       CML\n" +
                    " RED BLOOD CELLS       4   Jul 22, 2016 11:09  Jul 25, 2016        CML       CML\n" +
                    "                                                                               \n" +
                    "                                                                               \n" +
                    "Transfused Units \n" +
                    " Jul 27, 2016(1) RBC\n" +
                    "  Blood Product Key: RBC = RED BLOOD CELLS\n" +
                    "                                                                               \n" +
                    "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
                    "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
                    "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
                    "   in this report may have been duplicated in the VBECS report above (if available).\n" +
                    "                                                                               \n" +
                    "                             ---- BLOOD BANK ----\n" +
                    "ABO Rh:    \n" +
                    "                                                                               \n" +
                    "No UNITS assigned/xmatched\n" +
                    "                                                                               \n" +
                    "No component requests\n" +
                    "                                                                               \n" +
                    "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
                    "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
                    "   ---------       --- --    ---- ---  ---  --------------   -----------------\n";

    private static final String NO_BLOOD_BANK_REPORT = "No Blood Bank report available...";

    private static final String BLOOD_BANK_REPORT_PRODUCTION2 = "                             ---- BLOOD BANK ----\n" +
            "ABO Rh: O  Neg\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "DIAGNOSTIC TESTS:\n" +
            "                                 |---------------- DAT ----------------|\n" +
            "                                 |-- Poly --|  |-- IgG --|  |-- Comp --|\n" +
            " Date/Time        ABO Rh  ABS  Test   Intrp  Test  Intrp  Test   Intrp  Division\n" +
            " ---------------  --- --- ---  ----   -----  ----  -----  ----   -----  --------\n" +
            " 07/27/2016@17:34  O  Neg Neg                                           VA NWIHS, OMAHA DIVISION\n" +
            "  Comment (ABO): REVERSE TYPING DONE IN TUBE\n" +
            " 07/27/2016@14:28  O  Neg                                               VA NWIHS, OMAHA DIVISION\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "COMPONENT REQUESTS:\n" +
            "                                                                               \n" +
            " Component Type      Units Request date        Date wanted         Requestor By\n" +
            " --------------      ----- ------------        -----------         --------- --\n" +
            " RED BLOOD CELLS       1   Jul 28, 2016 01:14  Jul 28, 2016 01:13  HTL       HTL\n" +
            " RED BLOOD CELLS       2   Jul 27, 2016 14:02  Jul 27, 2016 13:58  HRH       HRH\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "Transfused Units \n" +
            " Jul 28, 2016(1) RBC\n" +
            " Jul 27, 2016(2) RBC\n" +
            "  Blood Product Key: RBC = RED BLOOD CELLS\n" +
            "                                                                               \n" +
            "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
            "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
            "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
            "   in this report may have been duplicated in the VBECS report above (if available).\n" +
            "                                                                               \n" +
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh:    \n" +
            "                                                                               \n" +
            "No UNITS assigned/xmatched\n" +
            "                                                                               \n" +
            "No component requests\n" +
            "                                                                               \n" +
            "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
            "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
            "   ---------       --- --    ---- ---  ---  --------------   -----------------\n";



    private static final String BLOOD_BANK_REPORT_PRODUCTION_NEW_022017 =
            "---- BLOOD BANK ----\n" +
            "ABO Rh: A  Neg\n" +
            "                                                                               \n" +
            "ANTIBODIES IDENTIFIED: \n" +
            "Anti-Jk(a)\n" +
            " COMMENT:\n" +
            " Preparation of red cell components for transfusion may be delayed due to\n" +
            " serologic problems. Contact transfusion service for information on potential\n" +
            " clinical significance and availability of blood components.\n" +
            "                                                                               \n" +
            "Anti-K\n" +
            " COMMENT:\n" +
            " Preparation of red cell components for transfusion may be delayed due to\n" +
            " serologic problems. Contact transfusion service for information on potential\n" +
            " clinical significance and availability of blood components.\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "DIAGNOSTIC TESTS:\n" +
            "                                    |------------------- DAT -------------------|\n" +
            "                                    |----- Poly -----|  |-- IgG --|  |-- Comp --|\n" +
            "    Date/Time        ABO Rh  ABS  Test         Intrp  Test  Intrp  Test   Intrp  Division\n" +
            "    ---------------  --- --- ---  ----         -----  ----  -----  ----   -----  --------\n" +
            "    05/05/2015@09:21                                  0     Neg    0      Neg    HINES, IL VAMC\n" +
            "      Comment (DAT Comp): DAT patient control negative, results valid\n" +
            "      Comment (Comp INTRP): DAT patient control negative, results valid\n" +
            "      Comment (DAT IgG): DAT patient control negative, results valid\n" +
            "      Comment (IgG INTRP): DAT patient control negative, results valid\n" +
            "(1) 05/05/2015@09:20              Microscopic  Pos                               HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            "(1) 05/05/2015@09:15  A  Neg Pos                                                 HINES, IL VAMC\n" +
            "      Comment (ABS): tube/PeG\n" +
            "      Comment (ABS): tube/PeG\n" +
            "    04/23/2015@07:58              0            Neg                               HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            "(1) 04/23/2015@07:49  A  Neg Pos                                                 HINES, IL VAMC\n" +
            "      Comment (ABO): ABO/Rh done in tubes\n" +
            "    04/16/2015@09:37              0            Neg                               HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            "(1) 04/16/2015@09:06  A  Neg Pos                                                 HINES, IL VAMC\n" +
            "    02/27/2015@08:32  A  Neg Neg                                                 HINES, IL VAMC\n" +
            "    02/12/2015@10:28  A  Neg Neg                                                 HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            " \n" +
            "   ----- STANDARD COMMENTS FOR DIAGNOSTIC TESTS ABOVE -----\n" +
            "(1)\n" +
            "Preparation of red cell components for transfusion may be delayed due to\n" +
            "serologic problems. Contact transfusion service for information on potential\n" +
            "clinical significance and availability of blood components.\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "COMPONENT REQUESTS:\n" +
            "                                                                               \n" +
            " Component Type      Units Request date        Date wanted    a     Requestor By\n" +
            " --------------      ----- ------------        -----------         --------- --\n" +
            " RED BLOOD CELLS       1   May 05, 2015 08:08  May 05, 2015 08:08  MMP       JSM\n" +
            " RED BLOOD CELLS       1   Apr 16, 2015 07:03  Apr 16, 2015 07:03  MMP       JSM\n" +
            " RED BLOOD CELLS       1   Feb 12, 2015 10:44  Feb 12, 2015        FJ        FJ\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "Transfused Units \n" +
            " May 05, 2015(1) RBC\n" +
            " Apr 16, 2015(1) RBC\n" +
            " Feb 12, 2015(1) RBC\n" +
            "  Blood Product Key: RBC = RED BLOOD CELLS\n" +
            "                                                                               \n" +
            "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
            "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
            "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
            "   in this report may have been duplicated in the VBECS report above (if available).\n" +
            "                                                                               \n" +
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh:    \n" +
            "                                                                               \n" +
            "No UNITS assigned/xmatched\n" +
            "                                                                               \n" +
            "No component requests\n" +
            "                                                                               \n" +
            "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
            "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
            "   ---------       --- --    ---- ---  ---  --------------   -----------------";


    private static final String BLOOD_BANK_REPORT_PRODUCTION_BAD_1 =
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh: O  Neg\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "DIAGNOSTIC TESTS:\n" +
            "                                 |---------------- DAT ----------------|\n" +
            "                                 |-- Poly --|  |-- IgG --|  |-- Comp --|\n" +
            " Date/Time        ABO Rh  ABS  Test   Intrp  Test  Intrp  Test   Intrp  Division\n" +
            " ---------------  --- --- ---  ----   -----  ----  -----  ----   -----  --------\n" +
            "     07/27/2016@17:34  O  Neg Neg                                           VA NWIHS, OMAHA DIVISION\n" +
            "      Comment (ABO): REVERSE TYPING DONE IN TUBE\n" +
            "     07/27/2016@14:28  O  Neg                                               VA NWIHS, OMAHA DIVISION\n" +
            "   07/21/2016@17:34  O  Neg Neg                                           VA NWIHS, OMAHA DIVISION\n" +
            "      Comment (ABO): REVERSE TYPING DONE IN TUBE\n" +
            "   07/19/2016@14:28  O  Neg                                               VA NWIHS, OMAHA DIVISION\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "COMPONENT REQUESTS:\n" +
            "                                                                               \n" +
            " Component Type      Units Request date        Date wanted         Requestor By\n" +
            " --------------      ----- ------------        -----------         --------- --\n" +
            " RED BLOOD CELLS       1   Jul 28, 2016 01:14  Jul 28, 2016 01:13  HTL       HTL\n" +
            " RED BLOOD CELLS       2   Jul 27, 2016 14:02  Jul 27, 2016 13:58  HRH       HRH\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "Transfused Units \n" +
            " Jul 28, 2016(1) RBC\n" +
            " Jul 27, 2016(2) RBC\n" +
            "  Blood Product Key: RBC = RED BLOOD CELLS\n" +
            "                                                                               \n" +
            "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
            "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
            "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
            "   in this report may have been duplicated in the VBECS report above (if available).\n" +
            "                                                                               \n" +
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh:    \n" +
            "                                                                               \n" +
            "No UNITS assigned/xmatched\n" +
            "                                                                               \n" +
            "No component requests\n" +
            "                                                                               \n" +
            "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
            "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
            "   ---------       --- --    ---- ---  ---  --------------   -----------------\n";

    private static final String BLOOD_BANK_REPORT_PRODUCTION_BAD2 =
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh: O  Neg\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "DIAGNOSTIC TESTS:\n" +
            "                                 |---------------- DAT ----------------|\n" +
            "                                 |-- Poly --|  |-- IgG --|  |-- Comp --|\n" +
            " Date/Time        ABO Rh  ABS  Test   Intrp  Test  Intrp  Test   Intrp  Division\n" +
            " ---------------  --- --- ---  ----   -----  ----  -----  ----   -----  --------\n" +
            " *** 07/27/2016@17:34  O  Neg Neg                                           VA NWIHS, OMAHA DIVISION\n" +
            "  Comment (ABO): REVERSE TYPING DONE IN TUBE\n" +
            " 07/27/2016@14:28  O  Neg                                               VA NWIHS, OMAHA DIVISION\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "COMPONENT REQUESTS:\n" +
            "                                                                               \n" +
            " Component Type      Units Request date        Date wanted         Requestor By\n" +
            " --------------      ----- ------------        -----------         --------- --\n" +
            " RED BLOOD CELLS       1   Jul 28, 2016 01:14  Jul 28, 2016 01:13  HTL       HTL\n" +
            " RED BLOOD CELLS       2   Jul 27, 2016 14:02  Jul 27, 2016 13:58  HRH       HRH\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "Transfused Units \n" +
            " Jul 28, 2016(1) RBC\n" +
            " Jul 27, 2016(2) RBC\n" +
            "  Blood Product Key: RBC = RED BLOOD CELLS\n" +
            "                                                                               \n" +
            "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
            "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
            "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
            "   in this report may have been duplicated in the VBECS report above (if available).\n" +
            "                                                                               \n" +
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh:    \n" +
            "                                                                               \n" +
            "No UNITS assigned/xmatched\n" +
            "                                                                               \n" +
            "No component requests\n" +
            "                                                                               \n" +
            "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
            "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
            "   ---------       --- --    ---- ---  ---  --------------   -----------------\n";

    private static final String BLOOD_BANK_REPORT_PRODUCTION_NEW_022017_BAD_TRANS =
            "---- BLOOD BANK ----\n" +
            "ABO Rh: A  Neg\n" +
            "                                                                               \n" +
            "ANTIBODIES IDENTIFIED: \n" +
            "Anti-Jk(a)\n" +
            " COMMENT:\n" +
            " Preparation of red cell components for transfusion may be delayed due to\n" +
            " serologic problems. Contact transfusion service for information on potential\n" +
            " clinical significance and availability of blood components.\n" +
            "                                                                               \n" +
            "Anti-K\n" +
            " COMMENT:\n" +
            " Preparation of red cell components for transfusion may be delayed due to\n" +
            " serologic problems. Contact transfusion service for information on potential\n" +
            " clinical significance and availability of blood components.\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "DIAGNOSTIC TESTS:\n" +
            "                                    |------------------- DAT -------------------|\n" +
            "                                    |----- Poly -----|  |-- IgG --|  |-- Comp --|\n" +
            "    Date/Time        ABO Rh  ABS  Test         Intrp  Test  Intrp  Test   Intrp  Division\n" +
            "    ---------------  --- --- ---  ----         -----  ----  -----  ----   -----  --------\n" +
            "    05/05/2015@09:21                                  0     Neg    0      Neg    HINES, IL VAMC\n" +
            "      Comment (DAT Comp): DAT patient control negative, results valid\n" +
            "      Comment (Comp INTRP): DAT patient control negative, results valid\n" +
            "      Comment (DAT IgG): DAT patient control negative, results valid\n" +
            "      Comment (IgG INTRP): DAT patient control negative, results valid\n" +
            "(1) 05/05/2015@09:20              Microscopic  Pos                               HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            "(1) 05/05/2015@09:15  A  Neg Pos                                                 HINES, IL VAMC\n" +
            "      Comment (ABS): tube/PeG\n" +
            "      Comment (ABS): tube/PeG\n" +
            "    04/23/2015@07:58              0            Neg                               HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            "(1) 04/23/2015@07:49  A  Neg Pos                                                 HINES, IL VAMC\n" +
            "      Comment (ABO): ABO/Rh done in tubes\n" +
            "    04/16/2015@09:37              0            Neg                               HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            "(1) 04/16/2015@09:06  A  Neg Pos                                                 HINES, IL VAMC\n" +
            "    02/27/2015@08:32  A  Neg Neg                                                 HINES, IL VAMC\n" +
            "    02/12/2015@10:28  A  Neg Neg                                                 HINES, IL VAMC\n" +
            "      Comment (DAT Poly): DAT patient control negative, results valid\n" +
            "      Comment (Poly INTRP): DAT patient control negative, results valid\n" +
            " \n" +
            "   ----- STANDARD COMMENTS FOR DIAGNOSTIC TESTS ABOVE -----\n" +
            "(1)\n" +
            "Preparation of red cell components for transfusion may be delayed due to\n" +
            "serologic problems. Contact transfusion service for information on potential\n" +
            "clinical significance and availability of blood components.\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "COMPONENT REQUESTS:\n" +
            "                                                                               \n" +
            " Component Type      Units Request date        Date wanted    a     Requestor By\n" +
            " --------------      ----- ------------        -----------         --------- --\n" +
            " RED BLOOD CELLS       1   May 05, 2015 08:08  May 05, 2015 08:08  MMP       JSM\n" +
            " RED BLOOD CELLS       1   Apr 16, 2015 07:03  Apr 16, 2015 07:03  MMP       JSM\n" +
            " RED BLOOD CELLS       1   Feb 12, 2015 10:44  Feb 12, 2015        FJ        FJ\n" +
            "                                                                               \n" +
            "                                                                               \n" +
            "Transfused Units \n" +
            " May 05, 2015(1) RBC\n" +
            " (1)Apr 16, 2015(1) RBC\n" +
            " Feb 12, 2015(1) RBC\n" +
            "  Blood Product Key: RBC = RED BLOOD CELLS\n" +
            "                                                                               \n" +
            "                   *** [LEGACY VISTA BLOOD BANK REPORT] ***\n" +
            "   The following historical information comes from the Legacy VISTA Blood Bank System\n" +
            "   It represents data collected prior to the installation of VBECS. Some of the information\n" +
            "   in this report may have been duplicated in the VBECS report above (if available).\n" +
            "                                                                               \n" +
            "                             ---- BLOOD BANK ----\n" +
            "ABO Rh:    \n" +
            "                                                                               \n" +
            "No UNITS assigned/xmatched\n" +
            "                                                                               \n" +
            "No component requests\n" +
            "                                                                               \n" +
            "                             |---     AHG(direct)     ---|   |-AHG(indirect)-|\n" +
            "   Date/time       ABO Rh    POLY IgG  C3   Interpretation   (Antibody screen)\n" +
            "   ---------       --- --    ---- ---  ---  --------------   -----------------";

}
