/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.MedicationsRPC;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Medication;
import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

/**
 *
 *
 */
public class MedicationsRPCTest extends BaseRPCTest
{
    private Calendar startDate;
    private Calendar endDate;
    private MedicationsRPC medsRPC;

    @Before
    public void setUp()
    {
        super.setUp();
        medsRPC = new MedicationsRPC(vistaCxn);

        startDate = Calendar.getInstance();
        endDate = Calendar.getInstance();

        startDate.set(2009, 0, 1);
        endDate.set(2010, 0, 5);
    }

    @Test
    public void testOutpatientMedications() throws RPCException
    {
        Medication[] meds = medsRPC.getOutpatientMedications(patientId, startDate, endDate);
        assertTrue(meds.length > 0);
    }

    @Test
    public void testOutpatientMedicationDetail() throws RPCException
    {
        Medication[] meds = medsRPC.getOutpatientMedications(patientId, startDate, endDate);
        assertTrue(meds.length > 0);

        FreeTextReport medDetail = medsRPC.getMedicationDetail(patientId, meds[0].getMedId());
        assertFalse(medDetail.getReportText().isEmpty());
    }

    @Test
    public void testInpatientMedicationDetail() throws RPCException
    {
        Medication[] meds = medsRPC.getInpatientMedications(patientId, startDate, endDate);
        assertTrue(meds.length > 0);

        FreeTextReport medDetail = medsRPC.getMedicationDetail(patientId, meds[0].getMedId());
        assertFalse(medDetail.getReportText().isEmpty());
    }


    @Test
    public void testInpatientMedications() throws RPCException
    {
        Medication[] meds = medsRPC.getInpatientMedications(patientId, startDate, endDate);
        assertTrue(meds.length > 0);
    }

    @Test
    public void testGetMedicationsVPR() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        MedicationsRPC labsVPRRPC = new MedicationsRPC(cxn);

        Medication[] meds = labsVPRRPC.getMedicationsVPR(patientId, startDate, endDate, "A");

        assertTrue(meds.length > 0);
    }
}
