package gov.va.med.des.webservice;

import gov.va.med.des.common.AppConfig;
import gov.va.med.monitor.ServiceMonitor;
import gov.va.med.monitor.ServiceTask;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServlet;
import javax.xml.ws.WebServiceContext;
import java.util.Timer;

public class DESMonitor extends HttpServlet {

        private static final Logger logger = LoggerFactory.getLogger(DESMonitor.class);
        private boolean isFirstTime = true;
        private Timer serviceTimer = null;

        /**
         * Web context.
         */
        private WebServiceContext wsContext;

        public DESMonitor()
        {
            logger.debug("DESMonitor class created (" + this + ")");
        }

    public void init() {
        cancelTimers();

        initiateServiceTimer();

        logger.debug("Initialize completed");
    }

    private void initiateServiceTimer() {
        logger.debug("Initiating service timer start...");

        logger.debug("isFirstTime = " + isFirstTime);

        if (isFirstTime) {
            isFirstTime = false;

            AppConfig appConfig = AppConfig.getInstance();

            serviceTimer = new Timer();
            serviceTimer.schedule(new ServiceTask(),
                    appConfig.getServiceMonitorTaskTimerDelay(),
                    appConfig.getServiceMonitorTaskTimerPeriod());

            logger.debug("isFirstTime = " + isFirstTime);
        }

        logger.debug("Initiating service timer finished");
    }

    private void cancelServiceTimer() {
        if (serviceTimer != null) {
            serviceTimer.cancel();
            serviceTimer.purge();
        }

        logger.debug("Canceling service timers");
    }

    public String verify() {
        return ("DESMonitor WebService is running!");
    }


    private void cancelTimers() {
        ServiceMonitor.getInstance().cancelEmailTimer();
        cancelServiceTimer();
    }

    public void destroy() {
        cancelTimers();
    }


}