/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.common;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Cache all lab types into HashMap in memory
 * User: Melcolm Goto
 * Date: 4/9/14
 */

/* yida
Copy this cache system
 */
public class LabLabelCache {

    private static final Map<String, String> MAPPINGS = new HashMap<String, String>();
    private static boolean isLoaded = false;

    private static final Logger LOGGER = LoggerFactory.getLogger(LabLabelCache.class);
    private static final AppConfig APPCONFIG = AppConfig.getInstance();

    static {
        try {
            loadDataFromConfig();
        }    catch (JMeadowsException e) {
            LOGGER.error(e.getMessage());
        }
    }

    private LabLabelCache() {
    }

    /**
     * Load mapping data from config file
     * This should be loaded only once when application starts
     */
    public static void loadDataFromConfig() throws JMeadowsException {
        if (!isLoaded || getSize() == 0) {
            LOGGER.info("read lab labels from config file");


            try {
                // Load the SQLServerDriver class, build the
                // connection string, and get a connection

                List<String> labCHList = Arrays.asList(APPCONFIG.getLabTypeCh().split(","));
                for(int i=0; i < labCHList.size(); i++){
                    MAPPINGS.put(labCHList.get(i), labCHList.get(0));
                }

                List<String> labMIList = Arrays.asList(APPCONFIG.getLabTypeMi().split(","));
                for(int i=0; i < labMIList.size(); i++){
                    MAPPINGS.put(labMIList.get(i), labMIList.get(0));
                }

                List<String> labAPList = Arrays.asList(APPCONFIG.getLabTypeAp().split(","));
                for(int i=0; i < labAPList.size(); i++){
                    MAPPINGS.put(labAPList.get(i), labAPList.get(0));
                }

                List<String> labAUList = Arrays.asList(APPCONFIG.getLabTypeAu().split(","));
                for(int i=0; i < labAUList.size(); i++){
                    MAPPINGS.put(labAUList.get(i), labAUList.get(0));
                }

                List<String> labBBList = Arrays.asList(APPCONFIG.getLabTypeBb().split(","));
                for(int i=0; i < labBBList.size(); i++){
                    MAPPINGS.put(labBBList.get(i), labBBList.get(0));
                }

                List<String> labCYList = Arrays.asList(APPCONFIG.getLabTypeCy().split(","));
                for(int i=0; i < labCYList.size(); i++){
                    MAPPINGS.put(labCYList.get(i), labCYList.get(0));
                }

                List<String> labEMList = Arrays.asList(APPCONFIG.getLabTypeEm().split(","));
                for(int i=0; i < labEMList.size(); i++){
                    MAPPINGS.put(labEMList.get(i), labEMList.get(0));
                }

                List<String> labSPList = Arrays.asList(APPCONFIG.getLabTypeSp().split(","));
                for(int i=0; i < labSPList.size(); i++){
                    MAPPINGS.put(labSPList.get(i), labSPList.get(0));
                }

                List<String> labSTList = Arrays.asList(APPCONFIG.getLabTypeSt().split(","));
                for(int i=0; i < labSTList.size(); i++){
                    MAPPINGS.put(labSTList.get(i), labSTList.get(0));
                }

                List<String> labHIVList = Arrays.asList(APPCONFIG.getLabTypeHiv().split(","));
                for(int i=0; i < labHIVList.size(); i++){
                    MAPPINGS.put(labHIVList.get(i), labHIVList.get(0));
                }

                isLoaded = true;

            } catch (Exception e) {
                LOGGER.error("Lab type label cache exception: " + e.getMessage());
                throw new JMeadowsException(e);
            } finally {

            }
        }
    }

    /**
     * Get code by providing a source string
     * @param source
     * @return
     */
    public static String getLabel(String source) {
        String label = source;
        try {
            loadDataFromConfig();
            if(MAPPINGS.get(source) != null) {
                label = MAPPINGS.get(source);
            }
        } catch (JMeadowsException e) {
            LOGGER.error(e.getMessage());
        }
        return label;
    }

    /**
     * Get size of the HashMap
     * @return
     */
    public static int getSize() {
        return MAPPINGS.size();
    }
}
