package gov.va.med.jmeadows.common;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by yidali on 6/29/17.
 */
public class LanguageCache {
    private static final Map<String, String> MAPPINGS = new HashMap<String, String>();
    private static boolean isLoaded = false;

    private static final Logger LOGGER = LoggerFactory.getLogger(LabLabelCache.class);
    private static final AppConfig APPCONFIG = AppConfig.getInstance();

    static {
        try {
            loadDataFromConfig();
        }    catch (JMeadowsException e) {
            LOGGER.error(e.getMessage());
        }
    }

    private LanguageCache() {
    }

    /**
     * Load mapping data from config file
     * This should be loaded only once when application starts
     */
    public static void loadDataFromConfig() throws JMeadowsException {
        if (!isLoaded || getSize() == 0) {
            LOGGER.info("read lab labels from config file");


            try {
                // Load the SQLServerDriver class, build the
                // connection string, and get a connection

                List<String> languageCodes = Arrays.asList(APPCONFIG.getLanguageCode().split(","));
                List<String> languageDescription = Arrays.asList(APPCONFIG.getLanguageDescription().split(","));
                for(int i=0; i < languageCodes.size(); i++){
                    MAPPINGS.put(languageCodes.get(i), languageDescription.get(i));
                }

                isLoaded = true;

            } catch (Exception e) {
                LOGGER.error("Lab type label cache exception: " + e.getMessage());
                throw new JMeadowsException(e);
            } finally {

            }
        }
    }

    /**
     * Get code by providing a source string
     * @param source
     * @return
     */
    public static String getLabel(String source) {
        String label = source;
        try {
            loadDataFromConfig();
            if(MAPPINGS.get(source) != null) {
                label = MAPPINGS.get(source);
            }
        } catch (JMeadowsException e) {
            LOGGER.error(e.getMessage());
        }
        return label;
    }

    /**
     * Get size of the HashMap
     * @return
     */
    public static int getSize() {
        return MAPPINGS.size();
    }
}
