/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.dao.beans;

import gov.va.med.vds.webservice.Patient;
import gov.va.med.vds.webservice.User;

import java.util.Calendar;

public class JMeadowsQuery {
    private User user;
    private Patient patient;
    private String sortBy;

    private String recordSiteCode;

    private String siteMoniker;

    private String siteAgency;

    private String requestingApp;

    /**
     * Set to 1 if requesting only active data (appointments, prescriptions, clinical
     * reminders)
     */
    private Integer active;
    /**
     * End date for the query.
     */
    private Calendar endDate;
    /**
     * Start date for the query.
     */
    private Calendar startDate;
    /**
     * Maximum number of items to return.  This is usually a required attribute when
     * the return is an array of items.
     */
    private Integer max;

    /**
     * Status of data being requested.  This could be Inpatient vs outpatient (procedures,
     * prescriptions/medications)
     */
    private String status;

    /**
     * This is the internal is of the data item when detailed informations is being
     * requested.
     */
    private String itemId;


    /**
     * This flag will cause jMeadows to query for VLER data
     *
     */
    private boolean includeVLER;

    /**
     * This flag will cause jMeadows to query for DAS VLER data
     *
     */
    private boolean includeDASVLER;

    /**
     * This flag will cause a force refresh for VLER queries
     *
     */
    private boolean forceRefresh;

    private String complexTransaction;

    private String info;

    public JMeadowsQuery() {
        this.includeVLER = false;
        this.includeDASVLER = false;
        this.forceRefresh = false;
    }

    public User getUser() {
        return user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public Patient getPatient() {
        return patient;
    }

    public void setPatient(Patient patient) {
        this.patient = patient;
    }

    public String getSortBy() {
        return sortBy;
    }

    public void setSortBy(String sortBy) {
        this.sortBy = sortBy;
    }

    public String getRecordSiteCode() {
        return recordSiteCode;
    }

    public void setRecordSiteCode(String recordSiteCode) {
        this.recordSiteCode = recordSiteCode;
    }

    public String getSiteMoniker() { return siteMoniker; }

    public void setSiteMoniker(String siteMoniker) { this.siteMoniker = siteMoniker; }

    public String getSiteAgency() { return siteAgency; }

    public void setSiteAgency(String siteAgency) { this.siteAgency = siteAgency; }

    public String getRequestingApp() {
        return requestingApp;
    }

    public void setRequestingApp(String requestingApp) {
        this.requestingApp = requestingApp;
    }

    public Integer getActive() {
        return active;
    }

    public void setActive(Integer active) {
        this.active = active;
    }

    public Calendar getEndDate() {
        return endDate;
    }

    public void setEndDate(Calendar endDate) {
        this.endDate = endDate;
    }

    public Calendar getStartDate() {
        return startDate;
    }

    public void setStartDate(Calendar startDate) {
        this.startDate = startDate;
    }

    public Integer getMax() {
        return max;
    }

    public void setMax(Integer max) {
        this.max = max;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getItemId() {
        return itemId;
    }

    public void setItemId(String itemId) {
        this.itemId = itemId;
    }

    public boolean isIncludeVLER() {
        return includeVLER;
    }

    public void setIncludeVLER(boolean includeVLER) {
        this.includeVLER = includeVLER;
    }

    public boolean isIncludeDASVLER() {
        return includeDASVLER;
    }

    public void setIncludeDASVLER(boolean includeDASVLER) {
        this.includeDASVLER = includeDASVLER;
    }

    public boolean isForceRefresh() {
        return forceRefresh;
    }

    public void setForceRefresh(boolean forceRefresh) {
        this.forceRefresh = forceRefresh;
    }

    public String getComplexTransaction() { return complexTransaction; }

    public void setComplexTransaction(String complexTransaction) { this.complexTransaction = complexTransaction; }

    public String getInfo() { return info; }

    public void setInfo(String info) { this.info = info; }
}
