package gov.va.med.jmeadows.dao.hdd;


public enum CodingSystem {
    RXNORM("RXNORM", "2.16.840.1.113883.1.1.BOGUS"),
    SNOMED("SNOMED", "2.16.840.1.113883.2.1.BOGUS"),
    ICD10_SNOMED_CT("SNOMED_CT", "2.16.840.1.113883.6.96"),
    //LOINC("LOINC", "2.16.840.1.113883.3.1.BOGUS"),
    CDC_RACE("CDC", "2.16.840.1.113883.23.1.BOGUS"),
    CVX("CVX", "2.16.840.1.113883.4.1.BOGUS"),
    CPT("CPT", "2.16.840.1.113883.6.12"),
    ICD9("ICD9", "2.16.840.1.113883.6.103"),
    VUID("VUID", "2.16.840.1.113883.5.1.BOGUS"),
    DOD_NCID("DOD_NCID", "2.16.840.1.113883.3.42.126.100001.16"),
    DOD_ALLERGY_IEN("DOD_ALLERGY_IEN", "2.16.840.1.113883.3.42.127.100001.209"),
    DOD_MEDCIN("DOD_MEDCIN", "2.16.840.1.113883.6.26"),
    DOD_IEN("2.16.840.1.113883.3.42.126.100001.16", "2.16.840.1.113883.3.42.126.100001.16"),
    //UMLS("UMLS", "2.16.840.1.113883.9.1.BOGUS"),
    X12("X12", "2.16.840.1.113883.22.1.BOGUS"),
    TEXT("TEXT", "2.16.840.1.113883.13.1.BOGUS"),

    //for non-HDD mappings
    ALLERGIES("ALLERGIES", "2.16.840.1.113883.9.1.BOGUS"),
    ////since vitals, notes, and labs use LOINC, we need separate codes as target systems
    VITALS("VITALS", "2.16.840.1.113883.10.1.BOGUS"),
    NOTES("NOTES", "2.16.840.1.113883.11.1.BOGUS"),
    LABS("LABS", "2.16.840.1.113883.12.1.BOGUS"),
    RADIOLOGY("RADIOLOGY", "2.16.840.1.113883.16.1.BOGUS"),

    //customMaps
    VA_MEDICATIONS("VA_MEDICATIONS", "1"),
    VA_VITALS("VA_VITALS", "2"),
    VA_IMMUNIZATIONS("VA_IMMUNIZATIONS", "3"),
    VA_NOTES("VA_NOTES", "4"),
    VA_ALLERGIES("VA_ALLERGIES", "6"),
    VA_PROBLEMS("VA_PROBLEMS", "7"),
    VA_PROBLEMS_SNOMED("VA_PROBLEMS_SNOMED", "24"),
    VA_LABS("VA_LABS", "8"),
    VA_RADS("VA_RADS", "9"),
    VA_LABS_TEXT("VA_LABS_TEXT", "20"),
    VA_MEDICATION_TEXT("VA_MEDICATION_TEXT", "21"),
    VA_X12("VA_X12", "22"),
    VA_RACE("VA_RACE", "23"),
    DOD_MEDICATIONS("DOD_MEDICATIONS", "D1"),
    DOD_VITALS("DOD_VITALS", "D2"),
    DOD_IMMUNIZATIONS("DOD_IMMUNIZATIONS", "D3"),
    DOD_NOTES("DOD_NOTES", "D4"),
    DOD_ALLERGIES("DOD_ALLERGIES", "D6"),
    DOD_PROBLEMS("DOD_PROBLEMS", "D7"),
    DOD_LABS("DOD_LABS", "D8"),
    DOD_RADS("DOD_RADS", "D9"),
    DOD_HISTORY("DOD_HISTORY", "D21"),
    DOD_X12("DOD_X12", "D22"),
    DOD_RACE("DOD_RACE", "D23"),

    CONSULT_ORDERS("CONSULT_ORDERS", "2.16.840.1.113883.17.1.BOGUS"),
    LAB_ORDERS("LAB_ORDERS", "2.16.840.1.113883.18.1.BOGUS"),
    RADIOLOGY_ORDERS("RADIOLOGY_ORDERS", "2.16.840.1.113883.19.1.BOGUS"),
    MEDICATION_ORDERS("MEDICATION_ORDERS", "2.16.840.1.113883.20.1.BOGUS"),

    ICD9_CM_DISEASES("ICD-9-CM Diseases", "2.16.840.1.113883.6.103"),
    ICD9_CM_PROCEDURES("ICD-9-CM Procedures", "2.16.840.1.113883.6.104"),
    ICD10_CM("ICD-10-CM", "2.16.840.1.113883.6.90"),
    ICD10_PCS("ICD-10_PCS", "2.16.840.1.113883.6.4"),
    LOINC("LOINC", "2.16.840.1.113883.6.1"),
    MS_DRG("MS-DRG", "2.16.840.1.113883.3.232.99.8"),
    NDC("NDC", "2.16.840.1.113883.6.69"),
    TRICARE_CHAMPUS_MS_DRG("TRICARE/CHAMPUS MS-DRG", "2.16.840.1.113883.3.232.99.9"),
    CMS_MS_DRG_MDC("CMS MS-DRG MDC", "2.16.840.1.113883.3.232.99.10"),
    CHAMPUS_TRICARE_MS_DRG_MDC("CHAMPUS/TRICARE MS-DRG MDC", "2.16.840.1.113883.3.232.99.11"),
    APC("APC", "2.16.840.1.113883.3.232.99.7"),
    HCPCS_LEVEL_II_CODE_SET("HCPCS Level II Code Set", "2.16.840.1.113883.6.285"),
    HCPCS_MODIFIERS("HCPCS Modifiers", "2.16.840.1.113883.3.232.99.12"),
    HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET("HAHS Emergency Department Diagnosis Codeset", "2.16.840.1.113883.3.232.99.5"),
    HAHS_PROBLEM_LIST_CODESET("HAHS Problem List Codeset", "2.16.840.1.113883.3.232.99.6");

    private String displayName;
    private String OID;

    private CodingSystem(String displayName, String OID) {
        this.displayName = displayName;
        this.OID = OID;
    }

    public String getDisplayName() {
        return displayName;
    }

//    public void setDisplayName(String displayName) {
//        this.displayName = displayName;
//    }

    public String getOID() {
        return OID;
    }

//    public void setOID(String OID) {
//        this.OID = OID;
//    }

    public static CodingSystem getCodingSystemByDisplayName(String displayName)
    {
        CodingSystem rtc = null;
        for (CodingSystem codingSystem : CodingSystem.values()) {
            if (codingSystem.getDisplayName().equalsIgnoreCase(displayName)) {
                rtc = codingSystem;
                break;
            }
        }
        return rtc;
    }

    public static CodingSystem getCodingSystemByOID(String oid) {
        CodingSystem rtc = null;
        for (CodingSystem codingSystem : CodingSystem.values()) {
            if (codingSystem.getOID().equals(oid)) {
                rtc = codingSystem;
                break;
            }
        }
        return rtc;
    }
}
