/**
 * Copyright (c) 2017 Hokukahu LLC. All Rights Reserved.*
 * Created by sishii on 6/19/17.
 */

package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.util.*;
import gov.va.med.jmeadows.smartbeans.patient.SmartHealthSummaryTypeList;
import gov.va.med.vds.webservice.FreeTextReport;
import gov.va.med.vds.webservice.HealthSummaryTypeList;
import gov.va.med.vds.webservice.Site;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class HealthSummariesDao extends DaoBase {
    public List<HealthSummaryTypeList> getPatientHealthSummaryTypes(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT);

        auditQuery("PatHealthSummaryTypes", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
                configs.add(
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_vistaHandler)
                                .dataServiceMethod("getPatientHealthSummaryTypesList")
                                .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                                .location(loc)
                                .build()
                );
            }
        }

        Object[] healthSummaries = MultiLocationQuery.execute(configs, new SmartHealthSummaryTypeList(), queryBean);

        List<HealthSummaryTypeList> patHealthSummaries = null;

        if (healthSummaries != null) {
            patHealthSummaries = Arrays.asList((HealthSummaryTypeList[]) healthSummaries);
        }

        return patHealthSummaries;
    }

    public FreeTextReport getPatientHealthSummary(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.ITEM_ID, QueryBeanParams.RECORD_SITE_CODE);

        auditQuery("PatHealthSummary", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        FreeTextReport report = null;

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                    queryBean.getPatient().getVistaSites())) {

                report = sm_vistaHandler.getPatientHealthSummary(VistaBeanFactory.create(queryBean));
                break;
            }
        }

        return report;
    }
}
