/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBase;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.ProviderMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.UserLocation;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.provider.SmartAppointment;
import gov.va.med.vds.webservice.PatientAppointments;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author vhahonbrunos
 */
public class ProviderAppointmentsDao extends DaoBase {

     /**
     * Gets a list of past and/or future appointments which are assigned to the
     * user passed in the queryBean.
     * @param queryBean requires a user, startDate and endDate. Start and
     * end dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of past and/or future patient appointments
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<PatientAppointments> getProviderAppointments(JMeadowsQuery queryBean) throws JMeadowsException
    {
        List<PatientAppointments> rtc = null;
        
        validateQueryBean(queryBean, QueryBeanParams.USER);

        auditQuery("ProviderAppointments", queryBean);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null)
        {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -1, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(Utils.getDate(Utils.getRelativeDate(0, 1, 0)))));
        }

        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        List<UserLocation> locations;

        locations = LocationHelper.getUserLocations(queryBean.getUser());

        for (UserLocation location : locations) {
            String systemType = location.getSystemType();
            String method = "getProviderAppointments";

            if (SYSTEM_TYPE_CHCS.equalsIgnoreCase(systemType)) {
                rtc = Arrays.asList((PatientAppointments[])
                        new SmartAppointment().generateConnectionUnavailableBean(location, PROVIDER_QUERY_NOT_SUPPORTED_DOD));
                break;
            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(systemType)) {
                configs.add(
                    new ProviderMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod(method)
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(location)
                            .build()
                );
            }
        }

        if (rtc == null) {
            Object[] aptObjs = MultiLocationQuery.execute(
                    configs, new SmartAppointment(), queryBean);
    
            List<PatientAppointments> appointmentsList = null;
    
            if (aptObjs != null) {
                appointmentsList = Arrays.asList((PatientAppointments []) aptObjs);
            }
    
            rtc = appointmentsList;
        }
        return rtc;
    }
}
