/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBase;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.ProviderMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.UserLocation;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.patient.SmartProgressNote;
import gov.va.med.vds.webservice.ProgressNote;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author vhahonbrunos
 */
public class ProviderNotesDao extends DaoBase {

    /**
     * Returns the given providers unsigned notes from a VistA system ONLY.
     * @param queryBean requires user, optional date range. If either the
     * beginning or ending date range are null, the range T-2 thru T is used.
     * @return a list of unsigned progress notes
     * @throws JMeadowsException when the data cannot be fetched
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<ProgressNote> getProviderUnsignedNotes(JMeadowsQuery queryBean) throws JMeadowsException {
        validateQueryBean(queryBean, QueryBeanParams.USER);

        auditQuery("ProviderUnsignedNotes", queryBean);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null) {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -2, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        List<MultiLocationQueryConfig> configs =
                new ArrayList<MultiLocationQueryConfig>();

        List<UserLocation> locations;

        locations = LocationHelper.getUserLocations(queryBean.getUser());

        for (UserLocation location : locations) {
            String systemType = location.getSystemType();
            String method = "getProviderUnsignedNotes";
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(systemType)) {
                configs.add(
                    new ProviderMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod(method)
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(location)
                            .build()
                );
            }
        }

        Object[] unsignedNotesObject = MultiLocationQuery.execute(configs,
                new SmartProgressNote(), queryBean);

        List<ProgressNote> unsignedNotes = null;

        if (unsignedNotesObject != null) {
            unsignedNotes = Arrays.asList((ProgressNote []) unsignedNotesObject);
        }

        return unsignedNotes;
    }
}
