/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */

package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.common.LanguageCache;
import gov.va.med.jmeadows.dao.beans.PDWSQueryBean;
import gov.va.med.jmeadows.dao.beans.ResponsePatientQuery;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.pdws.*;
import gov.va.med.pdws.ObjectFactory;
import gov.va.med.vds.webservice.*;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;
import java.io.Serializable;
import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.xml.bind.JAXBElement;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PDWSServiceHandler extends DataServiceHandler {
    private static final Logger LOGGER = LoggerFactory.getLogger(PDWSServiceHandler.class);
    private final String AGENCY_DOD = APP_CONFIG.getAgencyDOD();
    private final String AGENCY_VA = APP_CONFIG.getAgencyVA();
    private final String PERSON_SEARCH_URL = APP_CONFIG.getPdwsPsUrl();
    private final String PERSON_RETRIEVE_URL = APP_CONFIG.getPdwsPrUrl();
    private final String FAMILY_SEARCH_URL = APP_CONFIG.getPdwsFsUrl();
    private final String FAMILY_RETRIEVE_URL = APP_CONFIG.getPdwsFrUrl();
    private final Boolean DEBUG_SOAP_OUTPUT;
    private final EntityPatientInquiry entityPatientInquiry = new EntityPatientInquiry();
    private static final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd");

    public PDWSServiceHandler() {
        super();
        serviceName = "PDWSService";
        serviceNamespace = "http://webservice.pdws.URL       /";
        qServiceName = new QName(serviceNamespace, serviceName);
        String debug = AppConfig.getInstance().getProperties().getProperty("DEBUG_SOAP_OUTPUT", "false");
        if ("true".equalsIgnoreCase(debug)) {
            DEBUG_SOAP_OUTPUT = true;
        } else {
            DEBUG_SOAP_OUTPUT = false;
        }
    }

    private EntityPatientSearchPortType getEntityPatientSearchPort() {
        EntityPatientSearchPortType port = entityPatientInquiry.getEntityPatientSearchPortSoap11();

        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, PERSON_SEARCH_URL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        if (DEBUG_SOAP_OUTPUT) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private EntityPatientRetrievePortType getEntityPatientRetrievePort() {
        EntityPatientRetrievePortType port = entityPatientInquiry.getEntityPatientRetrievePortSoap11();

        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, PERSON_RETRIEVE_URL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        if (DEBUG_SOAP_OUTPUT) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private EntityPatientFamilySearchPortType getEntityFamilySearchPort() {
        EntityPatientFamilySearchPortType port = entityPatientInquiry.getEntityPatientFamilySearchPortSoap11();

        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, FAMILY_SEARCH_URL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        if (DEBUG_SOAP_OUTPUT) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private EntityPatientFamilyRetrievePortType getEntityFamilyRetrievePort() {
        EntityPatientFamilyRetrievePortType port = entityPatientInquiry.getEntityPatientFamilyRetrievePortSoap11();

        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, FAMILY_RETRIEVE_URL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        if (DEBUG_SOAP_OUTPUT) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private PRPAIN201305UV02 generateRequest(PDWSQueryBean queryBean) {
        MCCIMT000100UV01Device device;
        EnGiven enGiven;
        II ii;
        CS cs;
        EN en;
        ST st;
        AD ad;

        User user = queryBean.getUser();
        Site host = user.getHostSite();
        String agency = user.getAgency();
        String dmisId = "";
        String siteId = "";

        if (host != null) {
            dmisId = host.getDmisId();
            siteId = host.getSiteCode();
        }

        ObjectFactory factory = new ObjectFactory();
        PRPAIN201305UV02 request = factory.createPRPAIN201305UV02();

        //id
        ii = factory.createII();
        ii.setRoot("2.16.840.1.113883.3.2017");
        ii.setExtension("DMDC");
        request.setId(ii);

        //creationTime
        TS ts = factory.createTS();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddhhmmss", Locale.getDefault());
        ts.setValue(simpleDateFormat.format(new Date()));
        request.setCreationTime(ts);

        //interactionId
        ii = factory.createII();
        ii.setRoot("2.16.840.1.113883.1.6");
        ii.setExtension("PRPA_IN201305UV02");
        request.setInteractionId(ii);

        //processingCode
        cs = factory.createCS();
        cs.setCode(APP_CONFIG.getPDWSProcCode());
        request.setProcessingCode(cs);

        //processingModeCode
        cs = factory.createCS();
        cs.setCode("I");
        request.setProcessingModeCode(cs);

        //acceptAckCode
        cs = factory.createCS();
        cs.setCode("AL");
        request.setAcceptAckCode(cs);

        //receiver
        MCCIMT000100UV01Receiver receiver = factory.createMCCIMT000100UV01Receiver();
        request.getReceiver().add(receiver);

        //receiver/typeCode
        receiver.setTypeCode(CommunicationFunctionType.RCV);

        //receiver/device
        device = factory.createMCCIMT000100UV01Device();
        receiver.setDevice(device);

        //receiver/device/classCode
        device.setClassCode(EntityClassDevice.DEV);

        //receiver/device/determinerCode
        device.setDeterminerCode("INSTANCE");

        //receiver/device/id
        ii = factory.createII();
        ii.setRoot("1.3.6.1.4.1.21367.2009.1.3.1.145");
        device.getId().add(ii);

        //sender
        MCCIMT000100UV01Sender sender = factory.createMCCIMT000100UV01Sender();
        request.setSender(sender);

        //sender/typeCode
        sender.setTypeCode(CommunicationFunctionType.SND);

        //sender/device
        device = factory.createMCCIMT000100UV01Device();
        sender.setDevice(device);

        //sender/device/classCode
        device.setClassCode(EntityClassDevice.DEV);

        //sender/device/determinerCode
        device.setDeterminerCode("INSTANCE");

        //sender/device/id
        ii = factory.createII();
        ii.setRoot("2.16.840.1.113883.3.2017.11.7");
        ii.setExtension(APP_CONFIG.getPDWSSndDevId());
        device.getId().add(ii);

        //sender/device/asAgent
        MCCIMT000100UV01Agent agent = factory.createMCCIMT000100UV01Agent();
        device.setAsAgent(factory.createMCCIMT000100UV01DeviceAsAgent(agent));

        //sender/device/asAgent/classCode
        agent.getClassCode().add("AGNT");

        //sender/device/asAgent/representedOrganization
        MCCIMT000100UV01Organization organization = factory.createMCCIMT000100UV01Organization();
        agent.setRepresentedOrganization(factory.createMCCIMT000100UV01AgentRepresentedOrganization(organization));

        //sender/device/asAgent/representedOrganization/classCode
        organization.setClassCode("ORG");

        //sender/device/asAgent/representedOrganization/determinerCode
        organization.setDeterminerCode("INSTANCE");

        //sender/device/asAgent/representedOrganization/id
        ii = factory.createII();
        if (AGENCY_DOD.equalsIgnoreCase(agency)) {
            ii.setRoot("2.16.840.1.113883.3.42.10002.100001.8");
            ii.setExtension(dmisId);
        } else if (AGENCY_VA.equalsIgnoreCase(agency)) {
            ii.setRoot("2.16.840.1.113883.4.349");
            ii.setExtension(siteId);
        }
        organization.getId().add(ii);

        //controlActProcess
        PRPAIN201305UV02QUQIMT021001UV01ControlActProcess controlActProcess = factory.createPRPAIN201305UV02QUQIMT021001UV01ControlActProcess();
        request.setControlActProcess(controlActProcess);

        //controlActProcess/classCode
        controlActProcess.setClassCode(ActClassControlAct.CACT);

        //controlActProcess/moodCode
        controlActProcess.setMoodCode(XActMoodIntentEvent.EVN);

        //controlActProcess/code
        CD cd = factory.createCD();
        cd.setCode("PRPA_TE201305UV02");
        controlActProcess.setCode(cd);

        //controlActProcess/codeSystem
        cd.setCodeSystem("2.16.840.1.113883.1.6");

        //controlActProcess/dataEnterer
        QUQIMT021001UV01DataEnterer dataEnterer = factory.createQUQIMT021001UV01DataEnterer();
        controlActProcess.getDataEnterer().add(dataEnterer);

        //controlActProcess/dataEnterer/typeCode
        dataEnterer.getTypeCode().add("ENT");

        //controlActProcess/dataEnterer/contextControlCode
        dataEnterer.setContextControlCode("AP");

        //controlActProcess/dataEnterer/assignedPerson
        COCTMT090100UV01AssignedPerson assignedPerson = factory.createCOCTMT090100UV01AssignedPerson();
        dataEnterer.setAssignedPerson(assignedPerson);

        //controlActProcess/dataEnterer/assignedPerson/classCode
        assignedPerson.setClassCode("ASSIGNED");

        //controlActProcess/dataEnterer/assignedPerson/id
        ii = factory.createII();
        if (AGENCY_DOD.equalsIgnoreCase(agency)) {
            ii.setRoot("2.16.840.1.113883.3.42.127.100001.79");
            ii.setExtension(user.getUserIen() + "^" + dmisId);
        } else if (AGENCY_VA.equalsIgnoreCase(agency)) {
            ii.setRoot("2.16.840.1.113883.4.349");
            ii.setExtension(user.getUserIen() + "^" + siteId);
        }
        assignedPerson.getId().add(ii);

        //controlActProcess/dataEnterer/assignedPerson/Person
        COCTMT090100UV01Person person1 = factory.createCOCTMT090100UV01Person();
        assignedPerson.setAssignedPerson(factory.createCOCTMT090100UV01AssignedPersonAssignedPerson(person1));

        //controlActProcess/dataEnterer/assignedPerson/Person/name
        en = factory.createEN();
        en.getContent().add(queryBean.getUser().getName());
        person1.getName().add(en);

        //controlActProcess/queryByParameter
        PRPAMT201306UV02QueryByParameter queryByParameter = factory.createPRPAMT201306UV02QueryByParameter();
        controlActProcess.setQueryByParameter(factory.createPRPAIN201305UV02QUQIMT021001UV01ControlActProcessQueryByParameter(queryByParameter));

        //controlActProcess/queryByParameter/queryId
        ii = factory.createII();
        ii.setRoot("2.16.840.1.113883.3.2017");
        ii.setExtension("333334433");
        queryByParameter.setQueryId(ii);

        //controlActProcess/queryByParameter/statusCode
        cs = factory.createCS();
        cs.setCode("new");
        queryByParameter.setStatusCode(cs);

        //controlActProcess/queryByParameter/initialQuantity
        INT i = factory.createINT();
        i.setValue(BigInteger.valueOf(10));
        queryByParameter.setInitialQuantity(i);

        //controlActProcess/queryByParameter/parameterList
        PRPAMT201306UV02ParameterList parameterList = factory.createPRPAMT201306UV02ParameterList();
        queryByParameter.setParameterList(parameterList);

        //controlActProcess/queryByParameter/parameterList/livingSubjectId
        if (!"".equals(queryBean.getPatientId()) && !"".equals(queryBean.getPatientIdType())) {
            ii = factory.createII();
            ii.setExtension(queryBean.getPatientId());

            String type = queryBean.getPatientIdType();
            if ("SSN".equalsIgnoreCase(type)) {
                ii.setRoot("2.16.840.1.113883.4.1");
            } else if ("TIN".equalsIgnoreCase(type)) {
                ii.setRoot("2.16.840.1.113883.3.42.10001.100001.225");
            } else if ("FIN".equalsIgnoreCase(type)) {
                ii.setRoot("2.16.840.1.113883.3.42.10001.100001.224");
            } else if ("ITIN".equalsIgnoreCase(type)) {
                ii.setRoot("2.16.840.1.113883.4.2");
            } else if ("PIN".equalsIgnoreCase(type)) {
                ii.setRoot("2.16.840.1.113883.3.42.10001.100001.226");
            } else if ("EDIPI".equalsIgnoreCase(type)) {
                ii.setRoot("2.16.840.1.113883.3.42.10001.100001.12");
            }

            PRPAMT201306UV02LivingSubjectId livingSubjectId = factory.createPRPAMT201306UV02LivingSubjectId();
            livingSubjectId.getValue().add(ii);
            parameterList.getLivingSubjectId().add(livingSubjectId);

            //controlActProcess/queryByParameter/parameterList/livingSubjectId/semanticsText
            st = factory.createST();
            st.getContent().add("LivingSubject.id");
            livingSubjectId.setSemanticsText(st);
        }

        //controlActProcess/queryByParameter/parameterList/livingSubjectAdministrativeGender
        if (queryBean.getGender() != null) {
            PRPAMT201306UV02LivingSubjectAdministrativeGender adminGender = factory.createPRPAMT201306UV02LivingSubjectAdministrativeGender();
            CE ce = factory.createCE();
            ce.setCode(queryBean.getGender());
            adminGender.getValue().add(ce);
            parameterList.getLivingSubjectAdministrativeGender().add(adminGender);

            //controlActProcess/queryByParameter/parameterList/livingSubjectAdministrativeGender/semanticsText
            st = factory.createST();
            st.getContent().add("LivingSubject.administrativeGender");
            adminGender.setSemanticsText(st);
        }

        //controlActProcess/queryByParameter/parameterList/livingSubjectBirthTime
        PRPAMT201306UV02LivingSubjectBirthTime birthTime = factory.createPRPAMT201306UV02LivingSubjectBirthTime();
        IVLTS ivlts = factory.createIVLTS();
        ivlts.setValue(queryBean.getDob());
        birthTime.getValue().add(ivlts);
        parameterList.getLivingSubjectBirthTime().add(birthTime);

        //controlActProcess/queryByParameter/parameterList/livingSubjectBirthTime/semanticsText
        st = factory.createST();
        st.getContent().add("LivingSubject.birthTime");
        birthTime.setSemanticsText(st);

        //controlActProcess/queryByParameter/parameterList/livingSubjectName
        PRPAMT201306UV02LivingSubjectName name = factory.createPRPAMT201306UV02LivingSubjectName();
        en = factory.createEN();
        en.getUse().add("L");

        if (StringUtils.isNotBlank(queryBean.getFirstName())) {
            enGiven = factory.createEnGiven();
            enGiven.getContent().add(queryBean.getFirstName());
            en.getContent().add(factory.createENGiven(enGiven));
        }

        if (StringUtils.isNotBlank(queryBean.getMiddleName())) {
            enGiven = factory.createEnGiven();
            enGiven.getContent().add(queryBean.getMiddleName());
            en.getContent().add(factory.createENGiven(enGiven));
        }

        if (StringUtils.isNotBlank(queryBean.getLastName())) {
            EnFamily enFamily = factory.createEnFamily();
            enFamily.getContent().add(queryBean.getLastName());
            en.getContent().add(factory.createENFamily(enFamily));
        }

        name.getValue().add(en);
        parameterList.getLivingSubjectName().add(name);

        //controlActProcess/queryByParameter/parameterList/livingSubjectName/semanticsText
        st = factory.createST();
        st.getContent().add("LivingSubject.name");
        name.setSemanticsText(st);

        //controlActProcess/queryByParameter/parameterList/patientAddress/value
        PRPAMT201306UV02PatientAddress address = factory.createPRPAMT201306UV02PatientAddress();
        ad = factory.createAD();
        ad.getUse().add("HP");

        //controlActProcess/queryByParameter/parameterList/patientAddress/value/[@use="HP"]/streetAddressLine
        if (StringUtils.isNotBlank(queryBean.getAddress())) {
            AdxpStreetAddressLine addressLine = factory.createAdxpStreetAddressLine();
            addressLine.getContent().add(queryBean.getAddress());
            ad.getContent().add(factory.createADStreetAddressLine(addressLine));
        }

        //controlActProcess/queryByParameter/parameterList/patientAddress/value/[@use="HP"]/city
        if (StringUtils.isNotBlank(queryBean.getCity())) {
            AdxpCity city = factory.createAdxpCity();
            city.getContent().add(queryBean.getCity());
            ad.getContent().add(factory.createADCity(city));
        }

        //controlActProcess/queryByParameter/parameterList/patientAddress/value/[@use="HP"]/state
        if (StringUtils.isNotBlank(queryBean.getState())) {
            AdxpState state = factory.createAdxpState();
            state.getContent().add(queryBean.getState());
            ad.getContent().add(factory.createADState(state));
        }

        //controlActProcess/queryByParameter/parameterList/patientAddress/value/[@use="HP"]/zip
        if (StringUtils.isNotBlank(queryBean.getZip())) {
            AdxpPostalCode zip = factory.createAdxpPostalCode();
            zip.getContent().add(queryBean.getZip());
            ad.getContent().add(factory.createADPostalCode(zip));
        }

        address.getValue().add(ad);
        parameterList.getPatientAddress().add(address);

        //controlActProcess/queryByParameter/parameterList/otherIDsScopingOrganization
        PRPAMT201306UV02OtherIDsScopingOrganization otherIDsScopingOrganization = factory.createPRPAMT201306UV02OtherIDsScopingOrganization();
        parameterList.getOtherIDsScopingOrganization().add(otherIDsScopingOrganization);

        //controlActProcess/queryByParameter/parameterList/otherIDsScopingOrganization/value/id
        ii = factory.createII();
        ii.setRoot("2.16.840.1.113883.4.1");
        otherIDsScopingOrganization.getValue().add(ii);

        //controlActProcess/queryByParameter/parameterList/otherIDsScopingOrganization/semanticsText
        st = factory.createST();
        st.getContent().add("OtherIDs.scopingOrganization.id");
        otherIDsScopingOrganization.setSemanticsText(st);

        //controlActProcess/queryByParameter/parameterList/patientTelcom
        if (StringUtils.isNotBlank(queryBean.getTel())) {
            PRPAMT201306UV02PatientTelecom telecom = factory.createPRPAMT201306UV02PatientTelecom();
            TEL tel = factory.createTEL();
            tel.setValue(queryBean.getTel());
            tel.getUse().add("H");
            telecom.getValue().add(tel);
            parameterList.getPatientTelecom().add(telecom);
        }

        return request;
    }

    private ResponsePatientQuery parseResponse(PRPAIN201306UV02 response, String inquiryType) {
        ResponsePatientQuery respPatQuery = new ResponsePatientQuery();
        boolean done = false;
        
        // Initially set to failure with generic error message. Will only be
        // set to success if there are no acknowledgement errors and no
        // errors retrieving patient list, even if the list is empty
        respPatQuery.setSuccess(false);
        respPatQuery.setErrorMsg("Error retrieving patients."
                + "<br><br>Please try again."
                + "<br><br>If the problem persists, notify your system administrator or help desk.");

        // Create list of result patients
        List<Patient> resultPatientList = new ArrayList<Patient>();

        if (response != null) {
            if (response.getAcknowledgement() != null) {
                AcknowledgementDetailType ackDetailTypeCode;
                String ackCode;
                String ackDetailCode;
                String displayMessage;
                CS cs;
                CE ce;

                for (MCCIMT000300UV01Acknowledgement acknowledgement : response.getAcknowledgement()) {
                    cs = acknowledgement.getTypeCode();
                    if (cs == null) { continue; }

                    ackCode = cs.getCode();

                    if (acknowledgement.getAcknowledgementDetail() == null) { continue; }
                    for (MCCIMT000300UV01AcknowledgementDetail ackDetail : acknowledgement.getAcknowledgementDetail()) {
                        ackDetailTypeCode = ackDetail.getTypeCode();

                        ce = ackDetail.getCode();
                        if (ce == null) { continue; }

                        ackDetailCode = ce.getCode();
                        displayMessage = ce.getDisplayName();

                        if ("AA".equalsIgnoreCase(ackCode)) {
                            if (ackDetailTypeCode == AcknowledgementDetailType.W 
                                    && ackDetailCode.contains("PN") || ackDetailCode.contains("FM")) {
                                respPatQuery.setErrorMsg(displayMessage);
                            }
                        } else if ("AE".equalsIgnoreCase(ackCode) 
                                && ackDetailTypeCode == AcknowledgementDetailType.E 
                                && !ackDetailCode.contains("PN") && !ackDetailCode.contains("FM")) {
                            respPatQuery.setErrorMsg(displayMessage);
                            respPatQuery.setSuccess(false);
                            done = true;
                            break;
                        }
                    }
                }
            }
            if (!done) {
                PRPAIN201306UV02MFMIMT700711UV01ControlActProcess controlActProcess = response.getControlActProcess();
                if ((controlActProcess != null) && (controlActProcess.getSubject() != null)) {
                    PRPAIN201306UV02MFMIMT700711UV01RegistrationEvent registrationEvent;
                    PRPAIN201306UV02MFMIMT700711UV01Subject2 subject1;
                    PRPAMT201310UV02Patient patient;
                    PRPAMT201310UV02Person patientPerson;
                    CE ce;
                    TS ts;
                    CS cs;
                    SC sc;
                    Code code;
                    List<Patient> personalRelationshipList;
                    List<Person> sponsorList;
                    List<PRPAMT201310UV02ContactParty> contactPartyList;
                    Patient resultPatient;
                    PatientDemographics resultDemographics;
                    PatientDemographics tempDemographics;

                    for (PRPAIN201306UV02MFMIMT700711UV01Subject1 subject : controlActProcess.getSubject()) {
                        registrationEvent = subject.getRegistrationEvent();
                        if (registrationEvent == null) { continue; }
    
                        subject1 = registrationEvent.getSubject1();
                        if (subject1 == null) { continue; }
    
                        patient = subject1.getPatient();
                        if (patient == null) { continue; }
    
                        resultPatient = new Patient();
                        resultDemographics = new PatientDemographics();
                        tempDemographics = new PatientDemographics();
    
                        // Patient EDIPI
                        if (patient.getId() != null) {
                            for (II ii : patient.getId()) {
                                resultPatient.setEDIPI(ii.getExtension());
                            }
                        }
    
                        if (patient.getPatientPerson() != null) {
                            patientPerson = patient.getPatientPerson().getValue();
                            if (patientPerson != null) {
                                // Patient gender
                                ce = patientPerson.getAdministrativeGenderCode();
                                if (ce != null) {
                                    resultPatient.setGender(ce.getCode());
                                }
    
                                // Patient date of birth
                                ts = patientPerson.getBirthTime();
                                if (ts != null) {
                                    resultPatient.setDob(ts.getValue());
                                }
    
                                // Patient name
                                if (patientPerson.getName() != null) {
                                    for (PN pn : patientPerson.getName()) {
                                        if (pn.getUse() == null) { continue; }
                                        for (String use : pn.getUse()) {
                                            if ("L".equalsIgnoreCase(use)) {
                                                resultPatient.setName(parseName(pn));
                                            }
                                        }
                                    }
                                }
                                // Patient SSN
                                if (patientPerson.getAsOtherIDs() != null) {
                                    for (PRPAMT201310UV02OtherIDs otherIDs : patientPerson.getAsOtherIDs()) {
                                        if (otherIDs.getId() == null) { continue; }
                                        for (II ii : otherIDs.getId()) {
                                            cs = otherIDs.getStatusCode();
                                            if (cs == null) { continue; }
                                            if ("2.16.840.1.113883.4.1".equalsIgnoreCase(ii.getRoot())
                                                    && "active".equalsIgnoreCase(cs.getCode())) {
                                                resultPatient.setSSN(ii.getExtension());
                                            }
                                        }
                                    }
                                }
                                if ("PR".equalsIgnoreCase(inquiryType)) {
                                    // Patient rank
                                    if (patientPerson.getAsEmployee() != null) {
                                        for (PRPAMT201310UV02Employee employee : patientPerson.getAsEmployee()) {
                                            sc = employee.getJobTitleName();
                                            if (sc == null) { continue; }
                                            resultDemographics.setRank(sc.getDisplayName());
                                        }
                                    }
    
                                    // Patient race code
                                    if (patientPerson.getRaceCode() != null) {
                                        code = new Code();
                                        for (CE ce1 : patientPerson.getRaceCode()) {
                                            code.setCode(ce1.getCode());
                                            code.setSystem(CodingSystem.CDC_RACE.getDisplayName());
                                            resultDemographics.getRace().add(code);
                                        }
                                    }
                                    
                                    // Patient Language (currently is a string but can be a code)
                                    if (patientPerson.getLanguageCommunication() != null) {
                                        code = new Code();
                                        for (PRPAMT201310UV02LanguageCommunication lc1 : patientPerson.getLanguageCommunication()) {
                                            String langDisplay = lc1.getLanguageCode().getCode();
                                            resultDemographics.getLanguages().add(LanguageCache.getLabel(langDisplay));
                                        }
                                    }

                                    // Patient home & work telephone numbers
                                    if (patientPerson.getTelecom() != null) {
                                        for (TEL tel : patientPerson.getTelecom()) {
                                            if ((tel.getUse() == null)
                                                    || (tel.getUse().size() > 1)
                                                    || (tel.getValue().contains("mailto"))
                                                    || (tel.getValue().contains("@"))) {
                                                continue;
                                            }
                                            for (String use : tel.getUse()) {
                                                if ("HP".equalsIgnoreCase(use)) {
                                                    resultDemographics.setPhone1(parsePhoneNumber(tel.getValue()));
                                                } else if ("WP".equalsIgnoreCase(use)) {
                                                    resultDemographics.setPhone2(parsePhoneNumber(tel.getValue()));
                                                }
                                            }
                                        }
                                    }
    
                                    // Patient's address
                                    parseAddress(patientPerson, tempDemographics);
    
                                    // Sponsors
                                    sponsorList = parseSponsors(patientPerson);
                                    if(sponsorList != null && sponsorList.size() > 0) {
                                        resultDemographics.getSponsors().addAll(sponsorList);
                                    }

                                    BL organDonorInd = patientPerson.getOrganDonorInd();
                                    if (organDonorInd != null && organDonorInd.isValue() != null) {
                                        resultDemographics.setOrganDonorInd(String.valueOf(organDonorInd.isValue()));
                                    }

                                    List<PRPAMT201310UV02Subject4> subjectOf2 = patient.getSubjectOf2();
                                    if (subjectOf2 != null) {
                                        for (int i = 0; i < subjectOf2.size(); i++) {
                                            PRPAMT201310UV02Subject4 prpamt201310UV02Subject4 = subjectOf2.get(i);
                                            PRPAMT201310UV02AdministrativeObservation administrativeObservation = prpamt201310UV02Subject4.getAdministrativeObservation();
                                            if (administrativeObservation != null) {
                                                CD code1 = administrativeObservation.getCode();
                                                if (code1 != null && code1.getCode() != null) {
                                                    String codeString = code1.getCode().trim();
                                                    //blood donor indication date
                                                    if ("2.16.840.1.113883.3.2017.11.6.2.2".equals(codeString)
                                                            && administrativeObservation.getEffectiveTime() != null
                                                            && administrativeObservation.getEffectiveTime().size() > 0
                                                            //note this assumes the date is the first entry
                                                            && administrativeObservation.getEffectiveTime().get(0) != null) {
                                                        IVLTS sxcmts = (IVLTS) administrativeObservation.getEffectiveTime().get(0);
                                                        if (sxcmts != null) {
                                                            List<JAXBElement<? extends QTY>> rest = sxcmts.getRest();
                                                            if (rest != null && rest.size() > 0) {
                                                                JAXBElement jaxbElement = rest.get(0);
                                                                IVXBTS ixIvxbts = (IVXBTS) jaxbElement.getValue();
                                                                if (ixIvxbts != null) {
                                                                    String value = ixIvxbts.getValue();
                                                                    if (value != null) {
                                                                        Date d = null;

                                                                        try {
                                                                            d = simpleDateFormat.parse(value);
                                                                        } catch (ParseException e) {
                                                                            LOGGER.error("Error parsing organ donor ind date: ", e);
                                                                        }
                                                                        GregorianCalendar orgIndDateGregorianCalendar = new GregorianCalendar();
                                                                        orgIndDateGregorianCalendar.setTime(d);
                                                                        DatatypeFactory datatypeFactory = null;
                                                                        try {
                                                                            datatypeFactory = DatatypeFactory.newInstance();
                                                                        } catch (DatatypeConfigurationException e) {
                                                                            LOGGER.error("Error creating DatatypeFactory: ", e);
                                                                        }
                                                                        XMLGregorianCalendar xd = datatypeFactory.newXMLGregorianCalendar(orgIndDateGregorianCalendar);

                                                                        resultDemographics.setOrganDonorIndDate(xd);
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                    //blood type
                                                    else if ("2.16.840.1.113883.3.2017.11.6.2.3".equals(codeString)
                                                            && administrativeObservation.getValue() != null) {
                                                        CD value1 = (CD) administrativeObservation.getValue();
                                                        if (value1 != null) {
                                                            resultDemographics.setBloodType(value1.getDisplayName());
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }


                                    // Emergency Contact/Next of Kin
                                    contactPartyList = patientPerson.getContactParty();
                                    if ((contactPartyList != null) && (!contactPartyList.isEmpty())) {
                                        for (PRPAMT201310UV02ContactParty contactParty : contactPartyList)
                                        {
                                            if ("ECON".equalsIgnoreCase(contactParty.getClassCode().name()))
                                            {
                                                Person person = parsePerson(contactParty);
                                                resultDemographics.getEmergencyContact().add(person);
                                            }
                                            else if ("NOK".equalsIgnoreCase(contactParty.getClassCode().name()))
                                            {
                                                Person person = parsePerson(contactParty);
                                                resultDemographics.getNextOfKin().add(person);
                                            }
                                        }
                                    }

                                } else if ("FR".equalsIgnoreCase(inquiryType)) {
                                    // Family members (dependents)
                                    personalRelationshipList = parsePersonalRelationship(patientPerson);
                                    if (personalRelationshipList != null) {
                                        for (Patient patient1 : personalRelationshipList) {
                                            // Set sponsor SSN for each family member
                                            patient1.setSponsorSSN(resultPatient.getSSN());
                                        }
                                        resultPatientList.addAll(personalRelationshipList);
                                    }
                                }
                            }
                        }
                        if (!"FR".equalsIgnoreCase(inquiryType)) {
                            // Set patient data
                            resultPatientList.add(resultPatient);
                        }
                        if ("PR".equalsIgnoreCase(inquiryType)) {
                            // Patient's primary care manager (PCM)
                            parsePrimaryCareManager(patient, tempDemographics);
    
                            // Set demographics data
                            Address address = new Address();
                            resultDemographics.setEDIPI(resultPatient.getEDIPI());
                            resultDemographics.setSSN(resultPatient.getSSN());
                            resultDemographics.setName(resultPatient.getName());
                            resultDemographics.setGender(resultPatient.getGender());
                            resultDemographics.setDob(resultPatient.getDob());
                            if(tempDemographics.getAddress() != null && tempDemographics.getAddress().size() > 0) {
//                                address.setStreet1(tempDemographics.getAddress().get(0).getStreet1());
//                                address.setStreet2(tempDemographics.getAddress().get(0).getStreet2());
//                                address.setCity(tempDemographics.getAddress().get(0).getCity());
//                                address.setStateProvince(tempDemographics.getAddress().get(0).getStateProvince());
//                                address.setPostalCode(tempDemographics.getAddress().get(0).getPostalCode());
                                resultDemographics.getAddress().addAll(tempDemographics.getAddress());
                            }
                            resultDemographics.setPrimaryProvider(tempDemographics.getPrimaryProvider());
                            resultDemographics.setPcmDates(tempDemographics.getPcmDates());
                            resultDemographics.setPcmClinic(tempDemographics.getPcmClinic());
                            respPatQuery.setDemographics(resultDemographics);
                        }
                    }
                    respPatQuery.setSuccess(true);
                    respPatQuery.setErrorMsg(null);
                    respPatQuery.setPatients(resultPatientList);
                }
            }
        }
        return respPatQuery;
    }

    private void parseAddress(PRPAMT201310UV02Person person, PatientDemographics patientDemographics) {
        List<String> streetAddressList = new ArrayList<String>();
        List<Serializable> serializableList;
        Object object;
        Boolean isBadAddress;
        Boolean isConfidential;

        List<Address> homeList = new ArrayList<Address>();
        List<Address> tempList = new ArrayList<Address>();
        List<Address> workList = new ArrayList<Address>();
        List<Address> postalList = new ArrayList<Address>();
        List<Address> pharmList = new ArrayList<Address>();

        // Check for home primary address and bad address
        for (AD ad : person.getAddr()) {

            Address address = new Address();

            isBadAddress = false;
            isConfidential = false;

            if (ad.getUse() == null) {
                continue;
            }

            for (String use : ad.getUse()) {
                if (use.contains("BAD")) {
                    isBadAddress = true;
                    address.setType("BAD");
                } else if (use.contains("DIR")) {
                    isConfidential = true;
                    address.setType("CONFIDENTIAL");
                } else if (use.contains("HP")) {
                    address.setType("Home");
                } else if (use.contains("TMP")) {
                    address.setType("Temporary");
                } else if (use.contains("PST")) {
                    address.setType("Postal");
                } else if (use.contains("WP")) {
                    address.setType("Work");
                } else if (use.contains("PHYS")) {
                    address.setType("Pharmacy");
                }
            }

            // If not home primary address or address is bad or address content is null, then skip to next
            if (isConfidential || isBadAddress || (ad.getContent() == null)) {
                continue;
            }

            streetAddressList.clear();

            for (Serializable serializable : ad.getContent()) {
                if (serializable instanceof JAXBElement) {
                    object = ((JAXBElement) serializable).getValue();
                    if (object == null) {
                        continue;
                    }

                    // Street address
                    if (object instanceof AdxpStreetAddressLine) {
                        serializableList = ((AdxpStreetAddressLine) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            streetAddressList.add(serializableList.get(0).toString());
                        }
                    } else if (object instanceof AdxpCity) {                    // City
                        serializableList = ((AdxpCity) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            address.setCity(serializableList.get(0).toString());
                        }
                    } else if (object instanceof AdxpState) {                    // State
                        serializableList = ((AdxpState) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            address.setStateProvince(serializableList.get(0).toString());
                        }
                    }
                    // Zip code
                    if (object instanceof AdxpPostalCode) {
                        serializableList = ((AdxpPostalCode) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            address.setPostalCode(serializableList.get(0).toString());
                        }
                    }
                }
            }

            // Set street address from list
            if (!streetAddressList.isEmpty()) {
                address.setStreet1(streetAddressList.get(0));
                if (streetAddressList.size() > 1) {
                    address.setStreet2(streetAddressList.get(1));
                }
            }

            //patientDemographics.getAddress().add(address);
            if (address.getType().equalsIgnoreCase("Home")) {
                homeList.add(address);
            } else if (address.getType().equalsIgnoreCase("Temporary")) {
                tempList.add(address);
            } else if (address.getType().equalsIgnoreCase("Postal")) {
                postalList.add(address);
            } else if (address.getType().equalsIgnoreCase("Work")) {
                workList.add(address);
            } else if (address.getType().equalsIgnoreCase("Pharmacy")) {
                pharmList.add(address);
            }
        }

        if(homeList.size()>0) patientDemographics.getAddress().addAll(homeList);
        if(tempList.size()>0) patientDemographics.getAddress().addAll(tempList);
        if(postalList.size()>0) patientDemographics.getAddress().addAll(postalList);
        if(workList.size()>0) patientDemographics.getAddress().addAll(workList);
        if(pharmList.size()>0) patientDemographics.getAddress().addAll(pharmList);
    }

    private Person parsePerson(PRPAMT201310UV02ContactParty contactParty) {
        Person person = new Person();
        if (contactParty.getTelecom() != null) {
            for (TEL tel : contactParty.getTelecom()) {
                if ((tel.getUse() == null)
//                        || (tel.getUse().size() > 1) //?
                        || (tel.getValue().contains("mailto"))
                        || (tel.getValue().contains("@"))) {
                    continue;
                }
                String telStr="";
                for (int i = 0; i < tel.getUse().size(); i++) {
                    String use = tel.getUse().get(i);
                    if (i>0)
                        telStr = telStr.concat(", ");
                    telStr = telStr.concat(parsePhoneNumber(tel.getValue()));
                }

                Telecom telecom = new Telecom();
                telecom.setValue(telStr);
                person.getTelcoms().add(telecom);
            }
        }

        // Name
        if (contactParty.getContactPerson().getValue().getName() != null) {
            for (EN en : contactParty.getContactPerson().getValue().getName() ) {
                if (en.getUse() == null) { continue; }
                for (String use : en.getUse()) {
                    if ("L".equalsIgnoreCase(use)) {  //? PDWS_Ext_Serv_Desc_R17.docx p86
                        person.setName(parseName(en));
                    }
                }
            }
        }

        PatientDemographics tempDemographics = new PatientDemographics();
        parseAddressContactParty(contactParty, tempDemographics);

        if(tempDemographics.getAddress() != null && tempDemographics.getAddress().size() > 0) {
            Address address = new Address();
            address.setStreet1(tempDemographics.getAddress().get(0).getStreet1());
            address.setStreet2(tempDemographics.getAddress().get(0).getStreet2());
            address.setCity(tempDemographics.getAddress().get(0).getCity());
            address.setStateProvince(tempDemographics.getAddress().get(0).getStateProvince());
            address.setPostalCode(tempDemographics.getAddress().get(0).getPostalCode());

            person.getAddress().add(address);
        }

        if (contactParty.getCode()!=null && contactParty.getCode().getDisplayName()!=null)
            person.setRelationship(contactParty.getCode().getDisplayName());

        return person;
    }


    private void parseAddressContactParty(PRPAMT201310UV02ContactParty person, PatientDemographics patientDemographics) {
        List<String> streetAddressList = new ArrayList<String>();
        List<Serializable> serializableList;
        Object object;
        Boolean isBadAddress;
        Boolean isHomePrimary;
        Boolean isTempAddress;

        // Check for home primary address and bad address
        for (AD ad : person.getAddr()) {
            isBadAddress = false;
            isHomePrimary = false;
            isTempAddress = false;

            if (ad.getUse() == null) { continue; }

            for (String use : ad.getUse()) {
                if (use.contains("H")) {  //? PDWS_Ext_Serv_Desc_R17.docx p84
                    isHomePrimary = true;
                } else if (use.contains("BAD")) {
                    isBadAddress = true;
                } else if (use.contains("TMP")) {
                    isTempAddress = true;
                }
            }

            // If not home primary address or address is bad or address content is null, then skip to next
            if ((!isHomePrimary && !isTempAddress) || isBadAddress || (ad.getContent() == null)) { continue; }

            streetAddressList.clear();

            Address address = new Address();

            for (Serializable serializable : ad.getContent()) {
                if (serializable instanceof JAXBElement) {
                    object = ((JAXBElement) serializable).getValue();
                    if (object == null) { continue; }

                    // Street address
                    if (object instanceof AdxpStreetAddressLine) {
                        serializableList = ((AdxpStreetAddressLine) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            streetAddressList.add(serializableList.get(0).toString());
                        }
                    } else if (object instanceof AdxpCity) {                    // City
                        serializableList = ((AdxpCity) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            address.setCity(serializableList.get(0).toString());
                        }
                    } else if (object instanceof AdxpState) {                    // State
                        serializableList = ((AdxpState) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            address.setStateProvince(serializableList.get(0).toString());
                        }
                    }
                    // Zip code
                    if (object instanceof AdxpPostalCode) {
                        serializableList = ((AdxpPostalCode) object).getContent();
                        if ((serializableList != null) && !serializableList.isEmpty()) {
                            address.setPostalCode(serializableList.get(0).toString());
                        }
                    }
                }
            }

            // Set street address from list
            if (!streetAddressList.isEmpty()) {
                address.setStreet1(streetAddressList.get(0));

                if (streetAddressList.size() > 1) {
                    address.setStreet2(streetAddressList.get(1));
                }
            }
            patientDemographics.getAddress().add(address);
        }
    }

    private void parsePrimaryCareManager(PRPAMT201310UV02Patient patient, PatientDemographics patientDemographics) {
        if (patient.getCoveredPartyOf() == null) { return; }

        COCTMT510000UV06CoverageRecord coverageRecord;
        COCTMT510000UV06PolicyOrProgram policyOrProgram;
        COCTMT510000UV06Benefit benefit;
        COCTMT530000UVObservation observation;
        COCTMT090000UV01AssignedEntity assignedEntity;
        COCTMT090000UV01Person person;
        IVLTS ivlts;
        IVXBTS ivxbts;

        for (PRPAMT201310UV02CoveredParty coveredParty : patient.getCoveredPartyOf()) {
            coverageRecord = coveredParty.getCoverageRecord();
            if (coverageRecord == null) { continue; }

            if (coverageRecord.getComponent() == null) { continue; }

            for (COCTMT510000UV06Component component : coverageRecord.getComponent()) {
                if (component.getPolicyOrProgram() == null) { continue; }

                policyOrProgram = component.getPolicyOrProgram();
                for (COCTMT510000UV06Coverage2 coverage2 : policyOrProgram.getCoverageOf()) {
                    if (coverage2.getBenefit() == null) { continue; }

                    benefit = coverage2.getBenefit();
                    for (COCTMT510000UV06Definition definition : benefit.getDefinition()) {
                        if (definition.getObservation() == null) { continue; }

                        observation = definition.getObservation().getValue();
                        if (observation == null) { continue; }

                        if (observation.getPerformer() == null) { continue; }
                        for (COCTMT530000UVPerformer performer : observation.getPerformer()) {
                            if (performer.getAssignedEntity() == null) { continue; }

                            assignedEntity = performer.getAssignedEntity().getValue();
                            if (assignedEntity == null) { continue; }

                            if(patientDemographics.getPrimaryProvider() == null){
                                patientDemographics.setPrimaryProvider("");
                            }
                            //patientDemographics.setPcmDates(null);
                            if(patientDemographics.getPcmClinic() == null){
                                patientDemographics.setPcmClinic("");
                            }

                            // PCM name
                            if (assignedEntity.getAssignedPerson() != null) {
                                person = assignedEntity.getAssignedPerson().getValue();
                                if ((person != null) && (person.getName() != null)) {
                                    for (EN en : person.getName()) {
                                        if (en.getContent() == null) { continue; }
                                        for (Serializable content : en.getContent()) {
                                            patientDemographics.setPrimaryProvider(content.toString());
                                        }
                                    }
                                }
                            }

                            // PCM date
                            if (assignedEntity.getEffectiveTime() != null) {
                                ivlts = assignedEntity.getEffectiveTime();
                                if (ivlts.getRest() != null) {
                                    for (JAXBElement element : ivlts.getRest()) {
                                        if (!"low".equalsIgnoreCase(element.getName().getLocalPart())) { continue; }

                                        ivxbts = (IVXBTS) element.getValue();
                                        if (ivxbts == null) { continue; }

                                        // YYYYMMDD format (8 characters)
                                        if (ivxbts.getValue().length() != 8) { continue; }
                                        patientDemographics.setPcmDates(Utils.toXMLGregorianCalendar(ivxbts.getValue()));
                                    }
                                }
                            }

                            // PCM clinic
                            // Note: Currently PDWS does not return the PCM clinic. Set to empty string until
                            // PDWS returns the clinic name.
                        }
                    }
                }
            }
        }
    }

    private String parseName(EN en) {
        StringBuilder familyName = new StringBuilder();
        StringBuilder givenName = new StringBuilder();
        StringBuilder prefixName = new StringBuilder();
        StringBuilder suffixName = new StringBuilder();
        List<Serializable> serializableList1;
        List<Serializable> serializableList2;
        String rtc;
        Object object;

        serializableList1 = en.getContent();
        if (serializableList1 == null) {
            rtc = null;
        } else {
    
            for (Serializable serializable1 : serializableList1) {
                if (!(serializable1 instanceof JAXBElement)) { continue; }
    
                object = ((JAXBElement) serializable1).getValue();
                if (object == null) { continue; }
    
                if (object instanceof EnFamily) {
                    serializableList2 = ((EnFamily) object).getContent();
                    if (serializableList2 == null) { continue; }
    
                    for (Serializable serializable2 : serializableList2) {
                        familyName.append(serializable2.toString());
                        familyName.append(" ");
                    }
                } else if (object instanceof EnGiven) {
                    serializableList2 = ((EnGiven) object).getContent();
                    if (serializableList2 == null) { continue; }
    
                    for (Serializable serializable2 : serializableList2) {
                        givenName.append(serializable2.toString());
                        givenName.append(" ");
                    }
                } else if (object instanceof EnPrefix) {
                    serializableList2 = ((EnPrefix) object).getContent();
                    if (serializableList2 == null) { continue; }
    
                    for (Serializable serializable2 : serializableList2) {
                        prefixName.append(serializable2.toString());
                        prefixName.append(" ");
                    }
                } else if (object instanceof EnSuffix) {
                    serializableList2 = ((EnSuffix) object).getContent();
                    if (serializableList2 == null) { continue; }
    
                    for (Serializable serializable2 : serializableList2) {
                        suffixName.append(serializable2.toString());
                        suffixName.append(" ");
                    }
                }
            }
    
            // add prefix before given name
            if (prefixName.length() > 0) {
                givenName = prefixName.append(givenName);
            }
    
//            // add suffix after family name
//            if (suffixName.length() > 0) {
//                familyName.append(suffixName);
//            }

            // add suffix after given name
            if (suffixName.length() > 0) {
                givenName = givenName.append(suffixName);
            }
    
            rtc = familyName.toString().trim() + "," + givenName.toString().trim();
        }
        return rtc;
    }

    private String parsePhoneNumber(String telecom) {
        StringBuilder phoneNumber = new StringBuilder();
        StringTokenizer tokenizer = new StringTokenizer(telecom, " ");
        String token;
        char code;

        while (tokenizer.hasMoreTokens()) {
            token = tokenizer.nextToken();
            code = token.toLowerCase(Locale.getDefault()).charAt(0);
            // Do not accept tokens that begin with (p)riority, (y)es text, (n)o text
            if ((code != 'p') && (code != 'y') && (code != 'n')) {
                phoneNumber.append(token);
                phoneNumber.append(" ");
            }
        }

        return phoneNumber.toString().trim();
    }

    private List<Person> parseSponsors(PRPAMT201310UV02Person person) {
        List<Person> sponsorList = new ArrayList<Person>();

        if (person.getPersonalRelationship() != null) {
            COCTMT030007UVPerson person1;

            for (PRPAMT201310UV02PersonalRelationship personalRelationship : person.getPersonalRelationship()) {
                Person sponsor = new Person();
                if (personalRelationship.getRelationshipHolder1() == null) { continue; }

                person1 = personalRelationship.getRelationshipHolder1().getValue();
                if (person1 == null) { continue; }

                // Name
                if (person1.getName() != null) {
                    for (EN en : person1.getName()) {
                        if (en.getUse() == null) { continue; }
                        for (String use : en.getUse()) {
                            if ("L".equalsIgnoreCase(use)) {
                                sponsor.setName(parseName(en));
                            }
                        }
                    }
                }

                // Relation
                if (personalRelationship.getCode() != null) {
                    sponsor.setRelationship(personalRelationship.getCode().getDisplayName());
                }

                // SSN
                if (person1.getAsOtherIDs() != null) {
                    for (COCTMT030007UVOtherIDs otherIDs : person1.getAsOtherIDs()) {
                        if (otherIDs.getId() == null) { continue; }
                        for (II ii : otherIDs.getId()) {
                            if ("2.16.840.1.113883.4.1".equalsIgnoreCase(ii.getRoot())) {
                                sponsor.setIdentifier(ii.getExtension());
                            }
                        }
                    }
                }

                sponsorList.add(sponsor);
            }
        }

        return sponsorList;
    }

    private List<Patient> parsePersonalRelationship(PRPAMT201310UV02Person person) {
        List<Patient> patientList = new ArrayList<Patient>();

        if (person.getPersonalRelationship() != null) {
            COCTMT030007UVPerson person1;
            Patient patient;

            for (PRPAMT201310UV02PersonalRelationship personalRelationship : person.getPersonalRelationship()) {
                if (personalRelationship.getRelationshipHolder1() == null) { continue; }

                person1 = personalRelationship.getRelationshipHolder1().getValue();
                if (person1 == null) { continue; }

                patient = new Patient();

                // EDIPI
                if (person1.getId() != null) {
                    for (II ii : person1.getId()) {
                        if ("2.16.840.1.113883.3.42.10001.100001.12".equalsIgnoreCase(ii.getRoot())) {
                            patient.setEDIPI(ii.getExtension());
                        }
                    }
                }

                // Name
                if (person1.getName() != null) {
                    for (EN en : person1.getName()) {
                        if (en.getUse() == null) { continue; }
                        for (String use : en.getUse()) {
                            if ("L".equalsIgnoreCase(use)) {
                                patient.setName(parseName(en));
                            }
                        }
                    }
                }

                // Gender
                if (person1.getAdministrativeGenderCode() != null) {
                    patient.setGender(person1.getAdministrativeGenderCode().getCode());
                }

                // Date of birth
                if (person1.getBirthTime() != null) {
                    patient.setDob(person1.getBirthTime().getValue());
                }

                // SSN
                if (person1.getAsOtherIDs() != null) {
                    for (COCTMT030007UVOtherIDs otherIDs : person1.getAsOtherIDs()) {
                        if (otherIDs.getId() == null) { continue; }
                        for (II ii : otherIDs.getId()) {
                            if ("2.16.840.1.113883.4.1".equalsIgnoreCase(ii.getRoot())) {
                                patient.setSSN(ii.getExtension());
                            }
                        }
                    }
                }

                patientList.add(patient);
            }
        }

        return patientList;
    }

    public ResponsePatientQuery lookupPatient(PDWSQueryBean queryBean) throws JMeadowsException {
        PRPAIN201305UV02 request = generateRequest(queryBean);
        PRPAIN201306UV02 response;
        ResponsePatientQuery rtc = null;

        String inquiryType = queryBean.getInquiryType();
        if ("PS".equalsIgnoreCase(inquiryType)) {
            // Person Search
            response = getEntityPatientSearchPort().entityPatientSearch(request);
        } else if ("PR".equalsIgnoreCase(inquiryType)) {
            // Patient Retrieve
            response = getEntityPatientRetrievePort().entityPatientRetrieve(request);
        } else if ("FS".equalsIgnoreCase(inquiryType)) {
            // Family Search
            response = getEntityFamilySearchPort().entityPatientFamilySearch(request);
        } else if ("FR".equalsIgnoreCase(inquiryType)) {
            // Family Retrieve
            response = getEntityFamilyRetrievePort().entityPatientFamilyRetrieve(request);
        } else {
            // Invalid InquiryType
            throw new JMeadowsException("Invalid InquiryType:" + inquiryType);
        }

        if (response != null) {
            rtc = parseResponse(response, inquiryType);
            if (rtc != null && !rtc.getPatients().isEmpty()) {
                boolean found = true;
                while (found) {
                    found = false;
                    List<Patient> patients = rtc.getPatients();
                    Patient blacklisted = null;
                    for (Patient patient : patients) {
                        if (PDWSBlackList.getInstance().isBlackListed(patient.getEDIPI())) {
                            found = true;
                            blacklisted = patient;
                            break;
                        }
                    }
                    if (blacklisted != null) {
                        LOGGER.error("removing blacklisted patient");
                        patients.remove(blacklisted);
                        rtc.setPatients(patients);
                    }
                }
            }
        }
        return rtc;
    }
}
