/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vds.webservice.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;


/**
 *
 * @author vhahonbrunos
 */
public class VistaDataServiceHandler extends DataServiceHandler {
    private VistaDataService m_VistaDataService;
    private static final Logger LOGGER = LoggerFactory.getLogger(VistaDataServiceHandler.class);

    public VistaDataServiceHandler(String url) {
        super();

        serviceURL = url;
        serviceName = "VistaDataService";
        serviceNamespace = "http://webservice.vds.URL       /";
        qServiceName = new QName(serviceNamespace, serviceName);

    }

    private VistaData getVistaDataPort() {

        if (m_VistaDataService == null) {
            createService();
        }

        VistaData port = m_VistaDataService.getVistaDataPort();

        //set port endpoint address
        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        // SOAP message logging - Enable to show logging in output window
        if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()))
        {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private synchronized void createService() {
        if (m_VistaDataService == null) {
            try {
                m_VistaDataService = new VistaDataService();
            } catch (Exception e) {
                logError(e);
                throw new RuntimeException(e);
            }
        }
    }

    public User login(Site site, String accessCode, String verifyCode) throws JMeadowsException
    {       
        User user = null;

        try {
            VistaData port = getVistaDataPort();
            user = port.login(site, accessCode, verifyCode);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return user;
    }

    public User loginSaml(Site site, String samlToken) throws JMeadowsException
    {
        User user = null;

        try {
            VistaData port = getVistaDataPort();
            user = port.loginSaml(site, samlToken);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return user;
    }
    
    public AdmissionDetail getAdmissionDetails(QueryBean queryBean) throws JMeadowsException {
        AdmissionDetail returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getAdmissionDetails(queryBean);

        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }


        return returnVal;
    }

    public Diagnosis[] getAdmissionDiagnosis(QueryBean queryBean) throws JMeadowsException {
        Diagnosis[] diagnosis = null;

        try {

            VistaData port = getVistaDataPort();
            List<Diagnosis> diag = port.getAdmissionDiagnosis(queryBean);
            diagnosis = diag.toArray(new Diagnosis[diag.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
        return diagnosis;

    }

    public Procedure[] getAdmissionProcedures(QueryBean queryBean) throws JMeadowsException {
        Procedure[] admissionProcedures = null;

        try {
            VistaData port = getVistaDataPort();
            List<Procedure> proc = port.getAdmissionProcedures(queryBean);
            admissionProcedures = proc.toArray(new Procedure[proc.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return admissionProcedures;
    }

    public FreeTextReport getClinicalReminderDetail(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getClinicalReminderDetail(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }


        return returnVal;

    }

    public List<FreeTextReport> getConsultReport(QueryBean queryBean) throws JMeadowsException {
        try {
            VistaData port = getVistaDataPort();
            return port.getConsultReport(queryBean);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }

    public FreeTextReport getMedicationDetail(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getMedicationDetail(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public LabResult[] getLabOrderResult(QueryBean queryBean) throws JMeadowsException {
        LabResult[] orderResult = null;

        try {
            VistaData port = getVistaDataPort();
            List<LabResult> ores = port.getLabOrderResult(queryBean);
            orderResult = ores.toArray(new LabResult[ores.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return orderResult;
    }

    public FreeTextReport getPatientLabReport(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport report = null;

        try {
            VistaData port = getVistaDataPort();
            report = port.getPatientLabReport(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return report;
    }

    public LabResult[] getPatientLabResults(QueryBean queryBean) throws JMeadowsException {
        List<LabResult> labResults = null;

        try {
            VistaData port = getVistaDataPort();
            labResults = port.getPatientLabResults(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return labResults.toArray(new LabResult[labResults.size()]);
    }

    public FreeTextReport getOrderDetail(QueryBean queryBean) throws JMeadowsException
    {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getOrderDetail(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public PatientAdmission[] getPatientAdmissions(QueryBean queryBean) throws JMeadowsException {
        PatientAdmission[] patAdmissions = null;

        try {
            VistaData port = getVistaDataPort();
            List<PatientAdmission> adm = port.getPatientAdmissions(queryBean);
            patAdmissions = adm.toArray(new PatientAdmission[adm.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return patAdmissions;
    }

    /**
     * Retrieves list of patient allergies.
     * @param queryBean Query parameters.
     *                  Required params: User, Patient
     * @return A list of patient allergies.
     * @throws JMeadowsException if there was an error..
     */
    public Allergy[] getPatientAllergies(QueryBean queryBean) throws JMeadowsException {
        Allergy[] allergy = null;

        try {
            VistaData port = getVistaDataPort();
            List<Allergy> algy = port.getPatientAllergies(queryBean);
            allergy = algy.toArray(new Allergy[algy.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return allergy;
    }

    /**
     * Retrieves details of an allergy.
     * @param queryBean Query parameters.
     *                  Required params: User, Patient, itemId, recordSite
     * @return AllergyDetail containing allergy report text.
     * @throws JMeadowsException if there was an error..
     */
    public AllergyDetail getAllergyDetail(QueryBean queryBean) throws JMeadowsException {
        AllergyDetail allergyDetail = null;

        try {
            VistaData port = getVistaDataPort();
            allergyDetail = port.getAllergyDetail(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return allergyDetail;
    }

    public PatientAppointments[] getPatientAppointments(QueryBean queryBean) throws JMeadowsException {
        PatientAppointments[] appointments = null;

        try {
            VistaData port = getVistaDataPort();
            List<PatientAppointments> apts = port.getPatientAppointments(queryBean);
            appointments = apts.toArray(new PatientAppointments[apts.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return appointments;
    }

    public Encounter[] getPatientEncounters(QueryBean queryBean) throws JMeadowsException {
        Encounter[] encounters = null;

        try {
            VistaData port = getVistaDataPort();
            List<Encounter> enc = port.getPatientEncounters(queryBean);
            encounters = enc.toArray(new Encounter[enc.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return encounters;
    }

    public FreeTextReport getEncountersReport(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getEncountersReport(queryBean);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public ClinicalReminder[] getPatientClinicalReminders(QueryBean queryBean) throws JMeadowsException {
        ClinicalReminder[] reminders = null;

        try {
            VistaData port = getVistaDataPort();
            List<ClinicalReminder> remind = port.getPatientClinicalReminders(queryBean);
            reminders = remind.toArray(new ClinicalReminder[remind.size()]);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }

        return reminders;
    }

//    public String getPatientCodeGreen(QueryBean queryBean) throws JMeadowsException {
//        String returnVal = null;
//
//        try {
//            VistaData port = getVistaDataPort();
//            returnVal = port.getPatientCodeGreen(queryBean);
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return returnVal;
//    }

    public Consult[] getPatientConsultRequests(QueryBean queryBean) throws JMeadowsException {
        Consult[] consults = null;

        try {
            VistaData port = getVistaDataPort();
            List<Consult> cons = port.getPatientConsultRequests(queryBean);
            consults = cons.toArray(new Consult[cons.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return consults;
    }

    public Vitals[] getPatientCurrentVitals(QueryBean queryBean) throws JMeadowsException {
        Vitals[] vitals = null;

        try {
            VistaData port = getVistaDataPort();
            List<Vitals> vit = port.getPatientCurrentVitals(queryBean);
            vitals = vit.toArray(new Vitals[vit.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return vitals;

    }

    public PatientDemographics[] getPatientDemographics(QueryBean queryBean) throws JMeadowsException {
        PatientDemographics[] returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            List<PatientDemographics> demo = port.getPatientDemographics(queryBean);
            returnVal = demo.toArray(new PatientDemographics[demo.size()]);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public ProgressNote[] getPatientDischargeSummaries(QueryBean queryBean) throws JMeadowsException {
        ProgressNote[] notes = null;

        try {
            VistaData port = getVistaDataPort();
            List<ProgressNote> sum = port.getPatientDischargeSummaries(queryBean);
            notes = sum.toArray(new ProgressNote[sum.size()]);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }

        return notes;
    }

    public boolean isSensitivePatient(QueryBean queryBean) throws JMeadowsException {
        try {
            VistaData port = getVistaDataPort();
            List<Alert> alerts = port.getSensitivePatientAlerts(queryBean);
            int oldAlertLevel = -2;
            int newAlertLevel;

            for (Alert alert: alerts) {
                newAlertLevel = Integer.parseInt(alert.getAlertId());
                if (newAlertLevel > oldAlertLevel) {
                    oldAlertLevel = newAlertLevel;
                }
            }

            if (oldAlertLevel == -2) {
                throw new JMeadowsException("Unable to determine if patient has sensitive records.");
            }

            if (oldAlertLevel > 0) {
                return true;
            } else {
                return false;
            }

        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public List<Alert> getSensitivePatientAlerts(QueryBean queryBean) throws JMeadowsException {
        try {
            VistaData port = getVistaDataPort();
            return port.getSensitivePatientAlerts(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public PatientDemographicsDetail[] getPatientDemographicsDetail(QueryBean queryBean) throws JMeadowsException {
        PatientDemographicsDetail[] returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            List<PatientDemographicsDetail> demo = port.getPatientDemographicsDetail(queryBean);
            returnVal = demo.toArray(new PatientDemographicsDetail[demo.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public Immunization[] getPatientImmunizations(QueryBean queryBean) throws JMeadowsException {
        Immunization[] immunization = null;
        try {
            VistaData port = getVistaDataPort();
            List<Immunization> imm = port.getPatientImmunizations(queryBean);
            immunization = imm.toArray(new Immunization[imm.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return immunization;
    }

    public LabOrder[] getPatientLabs(QueryBean queryBean) throws JMeadowsException {
        LabOrder[] orders = null;
        try {
            VistaData port = getVistaDataPort();
            List<LabOrder> ord = port.getPatientLabs(queryBean);
            orders = ord.toArray(new LabOrder[ord.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return orders;
    }

    public LabResult[] getPatientLabTestResults(QueryBean queryBean) throws JMeadowsException {
        LabResult[] results = null;
        try {
            VistaData port = getVistaDataPort();
            List<LabResult> res = port.getPatientLabTestResults(queryBean);
            results = res.toArray(new LabResult[res.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return results;
    }

    public Medication[] getPatientMedications(QueryBean queryBean) throws JMeadowsException {
        Medication[] medication = null;
        try {
            VistaData port = getVistaDataPort();
            List<Medication> meds = port.getPatientMedications(queryBean);
            medication = meds.toArray(new Medication[meds.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return medication;
    }

//    public Medication[] getPatientMedicationsNVA(QueryBean queryBean) throws JMeadowsException {
//        Medication[] medication = null;
//        try {
//            VistaData port = getVistaDataPort();
//            medication = port.getPatientMedicationsNVA(queryBean).toArray(new Medication[0]);
//
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return medication;
//
//    }

    public Order[] getPatientOrders(QueryBean queryBean) throws JMeadowsException {
        Order[] orders = null;

        try {
            VistaData port = getVistaDataPort();
            List<Order> ord = port.getPatientOrders(queryBean);
            orders = ord.toArray(new Order[ord.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return orders;
    }

    public Problem[] getPatientProblemList(QueryBean queryBean) throws JMeadowsException {
        Problem[] problems = null;

        try {
            VistaData port = getVistaDataPort();
            List<Problem> prob = port.getPatientProblemList(queryBean);
            problems = prob.toArray(new Problem[prob.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return problems;
    }

//    public Procedure[] getPatientProcedures(QueryBean queryBean) throws JMeadowsException {
//        Procedure[] procedure = null;
//
//        try {
//            VistaData port = getVistaDataPort();
//            procedure = port.getPatientProcedures(queryBean).toArray(new Procedure[0]);
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return procedure;
//
//    }

    public ProgressNote[] getPatientProgressNotes(QueryBean queryBean) throws JMeadowsException {
        ProgressNote[] notes = null;

        try {
            VistaData port = getVistaDataPort();
            List<ProgressNote> note = port.getPatientProgressNotes(queryBean);
            notes = note.toArray(new ProgressNote[note.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return notes;
    }

    public RadiologyReport[] getPatientRads(QueryBean queryBean) throws JMeadowsException {
        RadiologyReport[] exams = null;

        try {
            VistaData port = getVistaDataPort();
            List<RadiologyReport> rpt = port.getPatientRads(queryBean);
            exams = rpt.toArray(new RadiologyReport[rpt.size()]);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }

        return exams;
    }

    public Vitals[] getPatientVitals(QueryBean queryBean) throws JMeadowsException {
        Vitals[] vitals = null;

        try {
            VistaData port = getVistaDataPort();
            List<Vitals> vit = port.getPatientVitals(queryBean);
            vitals = vit.toArray(new Vitals[vit.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return vitals;
    }

//    public Prescription getPrescriptionFills(QueryBean queryBean) throws JMeadowsException {
//        Prescription returnVal = null;
//
//        try {
//            VistaData port = getVistaDataPort();
//            returnVal = port.getPrescriptionFills(queryBean);
//        } catch (Exception e) {
//            throw new JMeadowsException(e);
//        }
//
//        return returnVal;
//
//    }

    public ProblemDetail getProblemDetail(QueryBean queryBean) throws JMeadowsException {
        ProblemDetail returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getProblemDetail(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public FreeTextReport getProgressNote(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getProgressNote(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public User getProviderProfile(Site site, String userIen, String providerNPI) throws JMeadowsException {
        User returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getProviderProfile(site, userIen, providerNPI);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public RadiologyReport getRadiologyReport(QueryBean queryBean) throws JMeadowsException {
        RadiologyReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getRadiologyReport(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;

    }

    public List<Alert> getUserAlerts(QueryBean queryBean) throws JMeadowsException {

        try {
            VistaData port = getVistaDataPort();
            return port.getUserAlerts(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public Diagnosis[] getVisitDiagnosis(QueryBean queryBean) throws JMeadowsException {
        Diagnosis[] returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            List<Diagnosis> diag = port.getVisitDiagnosis(queryBean);
            returnVal = diag.toArray(new Diagnosis[diag.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public FreeTextReport getVisitNotes(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVisitNotes(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public Procedure[] getVisitProcedures(QueryBean queryBean) throws JMeadowsException {
        Procedure[] returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            List<Procedure> proc = port.getVisitProcedures(queryBean);
            returnVal = proc.toArray(new Procedure[proc.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }


        return returnVal;
    }


    public List<Patient> lookupPatient(QueryBean queryBean) throws JMeadowsException {

        try {
            VistaData port = getVistaDataPort();
            //return a max of 100 patients
            queryBean.setMax(100);
            return port.lookupPatient(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public boolean selectPatient(QueryBean queryBean) throws JMeadowsException {
        boolean returnVal = false;

        try {
            VistaData port = getVistaDataPort();
            //return a max of 100 patients
            queryBean.setMax(100);
            returnVal = port.selectPatient(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;

    }

    public ReferralBean[] getDRReferrals(QueryBean queryBean) throws JMeadowsException {
        ReferralBean[] returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            List<ReferralBean> ref = port.getDRReferrals(queryBean);
            returnVal = ref.toArray(new ReferralBean[ref.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return returnVal;
    }

    public PatientAdmission[] getProviderAdmissions(QueryBean queryBean) throws JMeadowsException {
        try {
            List<PatientAdmission> admissions = null;
            VistaData port = getVistaDataPort();
            admissions = port.getProviderAdmissions(queryBean);
            return admissions.toArray(new PatientAdmission[admissions.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public PatientAdmission[] getWardAdmissions(QueryBean queryBean) throws JMeadowsException {
        try {
            List<PatientAdmission> admissions = null;
            VistaData port = getVistaDataPort();
            admissions = port.getWardAdmissions(queryBean);
            return admissions.toArray(new PatientAdmission[admissions.size()]);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }

    public PatientAppointments[] getProviderAppointments(QueryBean queryBean) throws JMeadowsException {
        try {
            List<PatientAppointments> appointments = null;
            VistaData port = getVistaDataPort();
            appointments = port.getProviderAppointments(queryBean);
            return appointments.toArray(new PatientAppointments[appointments.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public ProgressNote[] getProviderUnsignedNotes(QueryBean queryBean) throws JMeadowsException {
        try {
            List<ProgressNote> unsignedNotes = null;
            VistaData port = getVistaDataPort();
            unsignedNotes = port.getProviderUnsignedNotes(queryBean);
            return unsignedNotes.toArray(new ProgressNote[unsignedNotes.size()]);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }

    public Consult[] getProviderConsultsRequested(QueryBean queryBean) throws JMeadowsException {
        try {
            List<Consult> consultsRequested = null;
            VistaData port = getVistaDataPort();
            consultsRequested = port.getProviderConsultsRequested(queryBean);
            return consultsRequested.toArray(new Consult[consultsRequested.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public Consult[] getProviderConsultsReceived(QueryBean queryBean) throws JMeadowsException {
        try {
            List<Consult> consultsReceived = null;
            VistaData port = getVistaDataPort();
            consultsReceived = port.getProviderConsultsReceived(queryBean);
            return consultsReceived.toArray(new Consult[consultsReceived.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public Order[] getProviderOrdersPending(QueryBean queryBean) throws JMeadowsException {
        try {
            List<Order> orders = null;
            VistaData port = getVistaDataPort();
            orders = port.getProviderOrdersPending(queryBean);
            return orders.toArray(new Order[orders.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public Order[] getProviderOrdersResulted(QueryBean queryBean) throws JMeadowsException {
        try {
            List<Order> orders = null;
            VistaData port = getVistaDataPort();
            orders = port.getProviderOrdersResulted(queryBean);
            return orders.toArray(new Order[orders.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public LabResult[] getProviderLabAbnormalResults(QueryBean queryBean) throws JMeadowsException {
        try {
            List<LabResult> labResults = null;
            VistaData port = getVistaDataPort();
            labResults = port.getProviderLabAbnormalResults(queryBean);
            return labResults.toArray(new LabResult[labResults.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

    public InsuranceBean[] getPatientInsurances(QueryBean queryBean) throws JMeadowsException {
        InsuranceBean[] insurances = null;

        try {
            VistaData port = getVistaDataPort();
            List<InsuranceBean> ins = port.getPatientInsurance(queryBean);
            insurances = ins.toArray(new InsuranceBean[ins.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return insurances;
    }

    public Procedure[] getPatientProcedures(QueryBean queryBean) throws JMeadowsException {
        Procedure[] procedures = null;

        try {
            VistaData port = getVistaDataPort();
            List<Procedure> proc = port.getPatientProcedures(queryBean);
            procedures = proc.toArray(new Procedure[proc.size()]);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return procedures;
    }

    public StudyQuery[] getVixStudyQueryFromSite(QueryBean queryBean) throws JMeadowsException {
        StudyQuery[] studyQueries = null;

        try {
            VistaData port = getVistaDataPort();
            StudyQuery studyQuery = port.getVixStudyQueryFromSite(queryBean);

            List<StudyQuery> list = new ArrayList<StudyQuery>();
            list.add(studyQuery);

            studyQueries = list.toArray(new StudyQuery[list.size()]);
        }
        catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return studyQueries;
    }

    public StudyQuery getVixStudyQueryWithDetails(QueryBean queryBean) throws JMeadowsException {
        StudyQuery studyQuery;

        try {
            VistaData port = getVistaDataPort();
            studyQuery = port.getVixStudyQueryWithDetails(queryBean);
        }
        catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return studyQuery;
    }

    public StudyQuery getVixStudyQueryWithoutDetails(QueryBean queryBean) throws JMeadowsException
    {
        StudyQuery studyQuery;

        try {
            VistaData port = getVistaDataPort();
            studyQuery = port.getVixStudyQueryWithoutDetails(queryBean);
        }
        catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return studyQuery;
    }

    public StudyDetail getVixStudyDetail(QueryBean queryBean) throws JMeadowsException
    {
        StudyDetail studyDetail;

        try {
            VistaData port = getVistaDataPort();
            studyDetail = port.getVixStudyDetail(queryBean);
        }
        catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return studyDetail;
    }

    public String getVixStudyViewerUrl(QueryBean queryBean) throws JMeadowsException {
        String viewerUrl;

        try {
            VistaData port = getVistaDataPort();
            viewerUrl = port.getVixStudyViewerUrl(queryBean);
        }
        catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return viewerUrl;
    }

    public HealthSummaryTypeList[] getPatientHealthSummaryTypesList(QueryBean queryBean) throws JMeadowsException {
        List<HealthSummaryTypeList> listList = null;

        try {
            VistaData port = getVistaDataPort();
            listList = port.getPatientHealthSummaryTypes(queryBean);

        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }

        return listList.toArray(new HealthSummaryTypeList[listList.size()]);
    }

    public FreeTextReport getPatientHealthSummary(QueryBean queryBean) throws JMeadowsException {
        FreeTextReport returnVal = null;

        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getPatientHealthSummary(queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }


        return returnVal;

    }
    
    public List<Pce> getPcesForNote(QueryBean queryBean) throws JMeadowsException {
        List<Pce> pces = null;
        
        try {
            VistaData port = getVistaDataPort();
            pces = port.getPcesForNote(queryBean);
        }
        catch (Exception e) {
                logError(e);
                throw new JMeadowsException(e);
        }
        
        return pces;
    }
    
    private void logError(Exception e){
        LOGGER.error("VistaDataService ERROR: " + e.getMessage(), e);
    }
}
