/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.hdd.VLERSiteCache;
import gov.va.med.vds.webservice.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;


/**
 *
 * @author vhahongotom
 */
public class VistaDataVLERServiceHandler extends DataServiceHandler {
    private VistaDataService m_VistaDataVLERService;
    private static final Logger LOGGER = LoggerFactory.getLogger(VistaDataVLERServiceHandler.class);

    public VistaDataVLERServiceHandler(String url) {
        super();

        serviceURL = url;
        serviceName = "VistaDataVLERService";
        serviceNamespace = "http://webservice.vds.URL       /";
        qServiceName = new QName(serviceNamespace, serviceName);

        serviceConnectionTimeoutMS = APP_CONFIG.getVLERConnectionTimeoutMS();
        serviceRequestTimeoutMS = APP_CONFIG.getVLERRequestTimeoutMS();

    }

    private synchronized void createService() {
        if (m_VistaDataVLERService == null) {
            try {
                m_VistaDataVLERService = new VistaDataService();
            } catch (Exception e) {
                logError(e);
                throw new RuntimeException(e);
            }
        }
    }
    private VistaData getVistaDataPort() {

        if (m_VistaDataVLERService == null) {
            createService();
        }

        VistaData port = m_VistaDataVLERService.getVistaDataPort();

        //set port endpoint address
        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        // SOAP message logging - Enable to show logging in output window
        if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment())) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    public  List<VlerDocument> getVLERDocumentList(QueryBean queryBean) throws JMeadowsException {

        try {
            VistaData port = getVistaDataPort();
            List<VlerDocument> vdList = port.getPatientDocumentList(queryBean);
            for (VlerDocument vd : vdList) {
                if (vd.getHomeCommunityId() != null) {
                    Author author = new Author();
                    author.setInstitution(VLERSiteCache.getVLERSiteName(vd.getHomeCommunityId()));
                    author.setName(vd.getHomeCommunityId());
                    vd.getAuthorList().add(0, author);
                }
            }
            return vdList;
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
    }

//    public String getVLERSiteName(String hcid) {
//        String output = "";
//        String hci = "";
//        if(hcid.startsWith("urn")) {
//            hci = hcid.substring(8, hcid.length());
//        } else {
//            hci = hcid;
//        }
//        if (VLERSiteCache.getCode("", hci, "") != null) {
//            output = VLERSiteCache.getCode("", hci, "").getDisplay();
//        }
//        return output;
//    }

    public VlerDocument getVLERDocument(VlerDocument vDoc, QueryBean queryBean) throws JMeadowsException {
        VlerDocument returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal =  port.getPatientDocument(vDoc, queryBean);
        } catch (Exception e) {
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    public DasResultCollection getDASVLERAllergies(QueryBean queryBean) throws JMeadowsException {
        DasResultCollection returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVLERPatientAllergies(queryBean);
        } catch (Exception e){
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    public DasResultCollection getDASVLERProblems(QueryBean queryBean) throws JMeadowsException {
        DasResultCollection returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVLERPatientProblems(queryBean);
        } catch (Exception e){
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    public DasResultCollection getDASVLERVitals(QueryBean queryBean) throws JMeadowsException {
        DasResultCollection returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVLERPatientVitals(queryBean);
        } catch (Exception e){
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    public DasResultCollection getDASVLERProcedures(QueryBean queryBean) throws JMeadowsException {
        DasResultCollection returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVLERPatientProcedures(queryBean);
        } catch (Exception e){
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    public DasResultCollection getDASVLERImmunizations(QueryBean queryBean) throws JMeadowsException {
        DasResultCollection returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVLERPatientImmunizations(queryBean);
        } catch (Exception e){
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    public DasResultCollection getDASVLERDemographics(QueryBean queryBean) throws JMeadowsException {
        DasResultCollection returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal = port.getVLERPatientDemographics(queryBean);
        } catch (Exception e){
            logError(e);
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

    private void logError(Exception e){
        LOGGER.error("VistaDataService ERROR: " + e.getMessage(), e);
    }

}
