/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Diagnosis;
import gov.va.med.vhahon.common.Utils;
//import org.hui.chcs.DiagnosisBean;
//import org.hui.chcs.IcdBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

//import gov.va.med.vhahon.beans.Diagnosis;
//import gov.va.med.vhahon.beans.Procedure;
//import gov.va.med.chcs.beans.DiagnosisBean;
//import gov.va.med.vhahon.beans.*;
//import gov.va.med.chcs.beans.*;
//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartDiagnosis extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartDiagnosis.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;

    public Diagnosis m_diagnosis;

    public SmartDiagnosis() {
        super();
        m_diagnosis = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartDiagnosis(Diagnosis proc, String sort) {
        super();
        m_diagnosis = proc;
        getSortValue(sort);

    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartDiagnosis) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_diagnosis);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartDiagnosis[] temp = new SmartDiagnosis[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDiagnosis newAll = new SmartDiagnosis((Diagnosis) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartDiagnosis newAll = new SmartDiagnosis((Diagnosis) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Diagnosis[] ret = new Diagnosis[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_diagnosis;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartDiagnosis[] temp = new SmartDiagnosis[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDiagnosis newAll = new SmartDiagnosis((Diagnosis) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Diagnosis[] ret = new Diagnosis[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_diagnosis;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartDiagnosis tmp = (SmartDiagnosis) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_diagnosis.getDate(), tmp.m_diagnosis.getDate());
        }
        if (this.m_sortBy == SITE && this.m_diagnosis.getSite() != null && tmp.m_diagnosis.getSite() != null) {
            retVal = compareString(this.m_diagnosis.getSite().getSiteCode(), tmp.m_diagnosis.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_diagnosis.getDescription(), tmp.m_diagnosis.getDescription());
        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("date".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
        }

    }

//    public Diagnosis[] getDiagnosisFromCHCS_ICDBean(Object[] dodBean) {
//        IcdBean[] chcsBean = (IcdBean[]) dodBean;
//        int size = chcsBean.length;
//        Diagnosis[] vistaBean = new Diagnosis[size];
//        Diagnosis bean = null;
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            bean = new Diagnosis();
//            bean.setCode(chcsBean[i].getCode());
//            bean.setDescription(chcsBean[i].getDescription());
//            bean.setStatus(chcsBean[i].getStatus());
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//        }
//
//        return vistaBean;
//
//    }
//
//    public Diagnosis[] getDiagnosisFromCHCS(Object[] dodBean) {
//        DiagnosisBean[] chcsBean = (DiagnosisBean[]) dodBean;
//        int size = chcsBean.length;
//        Diagnosis[] vistaBean = new Diagnosis[size];
//        Diagnosis bean = null;
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            bean = new Diagnosis();
//            bean.setCode(chcsBean[i].getCode());
//            bean.setDescription(chcsBean[i].getDiagnosis());
//            bean.setStatus(chcsBean[i].getStatus());
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//        }
//
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Diagnosis diag = new Diagnosis();
        diag.setDescription(errorMsg);
        diag.setDate(Utils.backDate());
        diag.setSite(getConnectionUnavailableSite(location));

        return new Diagnosis[]{diag};
    }
}