/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.HealthSummaryType;
import gov.va.med.vds.webservice.HealthSummaryTypeList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class SmartHealthSummaryTypeList extends SmartBean implements Comparable {

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartHealthSummaryTypeList.class);

    public static final int NAME = 0;
    public static final int SITE = 1;

    public HealthSummaryTypeList m_healthSummaryTypeList;

    public SmartHealthSummaryTypeList() {
        super();
        m_healthSummaryTypeList = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartHealthSummaryTypeList(HealthSummaryTypeList healthSummary, String sort) {
        super();
        m_healthSummaryTypeList = healthSummary;
        getSortValue(sort);
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc = null;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartHealthSummaryTypeList[] temp = new SmartHealthSummaryTypeList[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartHealthSummaryTypeList newAll = new SmartHealthSummaryTypeList((HealthSummaryTypeList) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartHealthSummaryTypeList newAll = new SmartHealthSummaryTypeList((HealthSummaryTypeList) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            HealthSummaryTypeList[] ret = new HealthSummaryTypeList[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_healthSummaryTypeList;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        int size = array1.length;
        SmartHealthSummaryTypeList[] temp = new SmartHealthSummaryTypeList[size];
        int cnt = 0;
        for (int i = 0; i < array1.length; i++) {
            SmartHealthSummaryTypeList newAll = new SmartHealthSummaryTypeList((HealthSummaryTypeList) array1[i], sort);
            temp[cnt] = newAll;
            cnt++;
        }
        Arrays.sort(temp);
        HealthSummaryTypeList[] ret = new HealthSummaryTypeList[temp.length];
        for (int i = 0; i < temp.length; i++) {
            ret[i] = temp[i].m_healthSummaryTypeList;
        }
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartHealthSummaryTypeList) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_healthSummaryTypeList);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartHealthSummaryTypeList tmp = (SmartHealthSummaryTypeList) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_healthSummaryTypeList.getSite() != null && tmp.m_healthSummaryTypeList.getSite() != null) {
            retVal = compareString(this.m_healthSummaryTypeList.getSite().getSiteCode(), tmp.m_healthSummaryTypeList.getSite().getSiteCode());
        }
//        else {
//            retVal = compareString(this.m_healthSummaryTypeList.get(), tmp.m_healthSummaryTypeList.getName());
//        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }
        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = NAME;
        m_bAscending = true;
        if (sort != null) {
            sort = setAscending(sort);
            if ("healthSummaryTypeName".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }


    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        HealthSummaryType healthSummaryType = new HealthSummaryType();
        healthSummaryType.setName(errorMsg);
        healthSummaryType.setSite(getConnectionUnavailableSite(location));

        return new HealthSummaryType[]{healthSummaryType};
    }
}
