package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.HistoryV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.beans.Finding;
import gov.va.med.jmeadows.dao.beans.PatientHistory;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: yidali
 * Date: 4/7/14
 * Time: 2:01 PM
 * To change this template use File | Settings | File Templates.
 */
public class SmartHistory extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartHistory.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int RELATIONSHIP = 3;

    public PatientHistory m_Patient_history;

    public SmartHistory() {
        super();
        m_Patient_history = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartHistory(PatientHistory history, String sort) {
        super();
        m_Patient_history = history;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartHistory) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_Patient_history);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {

            int size = array1.length;
            SmartHistory[] temp = new SmartHistory[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartHistory newAll = new SmartHistory((PatientHistory) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientHistory[] ret = new PatientHistory[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_history;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartHistory tmp = (SmartHistory) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_Patient_history.getCustodianId() != null && tmp.m_Patient_history.getCustodianId() != null) {
            retVal = compareString(this.m_Patient_history.getCustodianId(), tmp.m_Patient_history.getCustodianId());
        }
        if (this.m_sortBy == RELATIONSHIP) {
            retVal = compareString(this.m_Patient_history.getRelationship(), tmp.m_Patient_history.getRelationship());
        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("date".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = RELATIONSHIP;
            }
        }

    }


    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartHistory[] temp = new SmartHistory[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartHistory newAll = new SmartHistory((PatientHistory) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartHistory newAll = new SmartHistory((PatientHistory) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientHistory[] ret = new PatientHistory[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_history;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientHistory hist = new PatientHistory();
        hist.setType(errorMsg);
        hist.setDateOnset(Utils.backDate().toGregorianCalendar());
        hist.setSite(getConnectionUnavailableSite(location));
        return new PatientHistory[]{hist};
    }

    public PatientHistory[] mapHistoryFromBHIE(Object resultCollection) {
        V4ResultCollection historyResultCollection = (V4ResultCollection) resultCollection;
        List<PatientHistory> vistaBeans = new ArrayList<PatientHistory>();

        if (historyResultCollection.getDataRecordList() != null) {

            List<HistoryV4> history = null;

            if (historyResultCollection.getDataRecordList().getFamilyHistory() != null &&
                    !historyResultCollection.getDataRecordList().getFamilyHistory().isEmpty())
                history = historyResultCollection.getDataRecordList().getFamilyHistory();
            else if (historyResultCollection.getDataRecordList().getSocialHistory() != null &&
                    !historyResultCollection.getDataRecordList().getSocialHistory().isEmpty())
                history = historyResultCollection.getDataRecordList().getSocialHistory();
            else if (historyResultCollection.getDataRecordList().getOpmHistory() != null &&
                    !historyResultCollection.getDataRecordList().getOpmHistory().isEmpty())
                history = historyResultCollection.getDataRecordList().getOpmHistory();

            if (history != null) {
                for (HistoryV4 bhieHistory : history) {
                    if (bhieHistory == null) {
                        continue;
                    }

                    PatientHistory bean = new PatientHistory();

//                bean.setChronicity(bhieHistory.getChronicity());

//                bean.setDateOnset(Utils.getCal(bhieHistory.getOnsetDate().getStart()));
//                if (bhieHistory.getType() != null
//                        && !bhieHistory.getType().isEmpty()
//                        && bhieHistory.getType().get(0) != null) {
//                    bean.setCouplerType(bhieHistory.getType().get(0).getDisplay());
//                }

//                if (bhieHistory.getRepository().getIds() != null
//                        && !bhieHistory.getRepository().getIds().isEmpty()
//                        && bhieHistory.getRepository().getIds().get(0) != null) {
//                    bean.setCustodianId(bhieHistory.getRepository().getIds().get(0).getAssigningAuthority());
//                }
//                bean.setCustodianName(bhieHistory.getRepository().getName());
                    XMLGregorianCalendar cal = bhieToXMLCalV4(bhieHistory.getNotedDate());
                    if (cal != null) {
                        bean.setDateReported(cal.toGregorianCalendar());
                    }
//                bean.setEncComment(bhieHistory.getEncComment());
//                bean.setEnteredBy(bhieHistory.getEnteredBy().getName());
//                bean.setEnteredByDate(Utils.getCal(bhieHistory.getNotedDate().getStart()));
//                bean.setFacility(bhieHistory.getFacility().getName());
//                bean.setFormattedFindings(bhieHistory.getFormattedFindings());
//                if (bhieHistory.getFinding() != null
//                        && !bhieHistory.getFinding().isEmpty()
//                        && bhieHistory.getFinding().get(0) != null) {
//                    bean.setHistoryMedcinId(bhieHistory.getFinding().get(0).getCode());
//                }
//
//                if (bhieHistory.getDataDomain() != null
//                        && !bhieHistory.getDataDomain().isEmpty()
//                        && bhieHistory.getDataDomain().get(0) != null) {
//                    bean.setLoincCode(bhieHistory.getDataDomain().get(0).getCode());
//                }
//                bean.setOtherIdentifier(bhieHistory.getOtherIdentifier()); //empty?
//                bean.setPeriodEndDate(Utils.getCal(bhieHistory.getPeriodEndDate())); empty??
//                bean.setPeriodStartDate(Utils.getCal(bhieHistory.getOnsetDate().getStart()));
//                if (bhieHistory.getRelationship() != null
//                        && !bhieHistory.getRelationship().isEmpty()
//                        && bhieHistory.getRelationship().get(0) != null) {
//                    bean.setRelationship(bhieHistory.getRelationship().get(0).getDisplay());
//                    bean.setRelationshipId(bhieHistory.getRelationship().get(0).getCode()); //code
//                }
//                bean.setReportingClinician(bhieHistory.getReportingClinician()); //enteredby or provider?
//                if (bhieHistory.getRepository() != null
//                        && bhieHistory.getRepository().getIds() !=null
//                        && !bhieHistory.getRepository().getIds().isEmpty()
//                        && bhieHistory.getRepository().getIds().get(0) != null) {
//                    bean.setRepositoryId(bhieHistory.getRepository().getIds().get(0).getId());
//                }
//                bean.setSensitivity(bhieHistory.getSensitivity());
//                if (bhieHistory.getSource() != null
//                        && !bhieHistory.getSource().isEmpty()
//                        && bhieHistory.getSource().get(0) != null) {
//                    bean.setSource(bhieHistory.getSource().get(0).);
//                }
                    if (bhieHistory.getStatus() != null
                            && !bhieHistory.getStatus().isEmpty()
                            && bhieHistory.getStatus().get(0) != null) {
                        bean.setStatus(bhieHistory.getStatus().get(0).getDisplay());
                    }
                    if (bhieHistory.getType() != null
                            && !bhieHistory.getType().isEmpty()
                            && bhieHistory.getType().get(0) != null) {
                        bean.setType(bhieHistory.getType().get(0).getDisplay());
                    }
//                bean.setUnverifiedFlag(bhieHistory.getUnverifiedFlag());

                    List<Finding> findings = new ArrayList<Finding>();
                    for (int i = 0; i < bhieHistory.getFinding().size(); i++) {
                        SpdcBeanV4 finding =   bhieHistory.getFinding().get(i);
                        Finding tempFinding = new Finding();
//                    tempFinding.setAnnotation(finding.getAnnotation());  empty
//                    tempFinding.setCategory(finding.getCategory()); empty
//                    tempFinding.setFlag(finding.getFlag()); empty
                        StringBuffer prefix = new StringBuffer("");
                        String display = finding.getDisplay() != null ? finding.getDisplay() : "";
                        //check whether the display text is all uppercase by looking at first few characters...
                        boolean uc = (4 <= getUpperCaseCount(display, 6));

                        if (i == 0) {
                            if (bhieHistory.isNegationInd()) {
                                if (uc)
                                    prefix.append("NO ");
                                else
                                    prefix.append("No ");
                            }
                            if (bhieHistory.getRelationship() != null) {
                                for (SpdcBeanV4 rel : bhieHistory.getRelationship()) {
                                    if (rel.getDisplay()!=null &&
					!StringUtils.equalsIgnoreCase("Other, see comments", rel.getDisplay().trim())
					) {
                                        if (uc)
                                            prefix.append(rel.getDisplay().toUpperCase() + " ");
                                        else
                                            prefix.append(rel.getDisplay() + " ");
                                    }
                                }
                            }
                        }
                        tempFinding.setItem(prefix + display);

                        findings.add(tempFinding);

                        if (finding.getSystem() != null) {

                            Code code = new Code();
                            code.setDisplay(finding.getDisplay());
                            code.setSystem(finding.getSystem());
                            code.setCode(finding.getCode());
                            bean.getCodes().add(code);
                        }
                    }
                    bean.setFindings(findings);

//                all empty?
                    List<String> comments = new ArrayList<String>();
                    for (String comment : bhieHistory.getComments()) {
                        comments.add(comment);
                    }
                    bean.setComments(comments);

//                if (StringUtils.isNotBlank(bhieHistory.getLoincCode())) {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.LOINC.getDisplayName());
//                    code.setCode(bhieHistory.getLoincCode());
//                    bean.getCodes().add(code);
//                }
//
//                if (StringUtils.isNotBlank(bhieHistory.getHistoryMedcinId())) {
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.DOD_MEDCIN.getDisplayName());
//                    code.setCode(bhieHistory.getHistoryMedcinId());
//                    bean.getCodes().add(code);
//                }

//                addBHIESiteDataToVistaBean(bhieHistory.getCustodianId(),
//                        bhieHistory.getCustodianName(), bhieHistory.getFacility(), bhieHistory.getEventId(), bean);

                    String facilityName = "";
                    String repositoryName = "";
                    String repositoryId = "";
                    String recordId = "";

                    if (bhieHistory.getFacility() != null && bhieHistory.getFacility().getName() != null) {
                        facilityName = bhieHistory.getFacility().getName();
                    }

                    if (bhieHistory.getRepository() != null && bhieHistory.getRepository().getName() != null) {
                        repositoryName = bhieHistory.getRepository().getName();
                        if (bhieHistory.getRepository().getIds() != null && !bhieHistory.getRepository().getIds().isEmpty() && bhieHistory.getRepository().getIds().get(0) != null) {
                            repositoryId = bhieHistory.getRepository().getIds().get(0).getId();
                        }
                    }

                    if (bhieHistory.getRecordId() != null) {
                        recordId = bhieHistory.getRecordId().getId();
                    }

                    addBHIESiteDataToVistaBean(repositoryId,
                            repositoryName, facilityName, recordId, bean);

                    vistaBeans.add(bean);
                }

            }
        }

        //include status report
        String siteStatusReport = generateBhieStatusReport(historyResultCollection.getStatusList(), historyResultCollection.getErrorList());

        vistaBeans.addAll(Arrays.asList(
                (PatientHistory[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new PatientHistory[vistaBeans.size()]);
    }

    /**
     * Returns count of uppercase characters in first N alphabetic characters.
     * @param str
     * @return
     */
    int getUpperCaseCount(String str, int n) {
        if (str == null) return 0;

        int upperCount = 0, alpha = 0, pos = 0;

        while (pos < str.length() && alpha < n) {
            char c = str.toCharArray()[pos];
            if (Character.isAlphabetic(c))
                alpha++;
            if (Character.isUpperCase(c))
                upperCount++;

            pos++;
        }
        return upperCount;
    }

}
