/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.ImmunizationV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.vds.webservice.*;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
//import org.hui.chcs.ImmunizationBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

import static gov.va.med.jmeadows.dao.hdd.VLERSiteCache.getVLERSiteName;

//import gov.va.med.vhahon.beans.Immunization;
//import mil.osd.ha.citpo.jvi.webservice.beans.ImmunizationBean;
//import gov.va.med.vhahon.beans.*;
//import gov.va.med.chcs.beans.*;
//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartImmunization extends SmartBean implements Comparable {

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartImmunization.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;

    public Immunization m_immunization;

    public SmartImmunization() {
        super();
        m_immunization = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartImmunization(Immunization imm, String sort) {
        super();
        m_immunization = imm;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartImmunization) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_immunization);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartImmunization[] temp = new SmartImmunization[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartImmunization newAll = new SmartImmunization((Immunization) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartImmunization newAll = new SmartImmunization((Immunization) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Immunization[] ret = new Immunization[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_immunization;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartImmunization[] temp = new SmartImmunization[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartImmunization newAll = new SmartImmunization((Immunization) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Immunization[] ret = new Immunization[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_immunization;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartImmunization tmp = (SmartImmunization) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_immunization.getDateTime(), tmp.m_immunization.getDateTime());
        }
        if (this.m_sortBy == SITE && this.m_immunization.getSite() != null && tmp.m_immunization.getSite() != null) {
            retVal = compareString(this.m_immunization.getSite().getSiteCode(), tmp.m_immunization.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_immunization.getName(), tmp.m_immunization.getName());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("datetime".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("name".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
        }

    }

    public Immunization[] mapImmunizationsFromBHIE(Object resultCollection) {
        V4ResultCollection immunizationResultCollection = (V4ResultCollection) resultCollection;
        List<Immunization> vistaBeans = new ArrayList<Immunization>();

        if (immunizationResultCollection.getDataRecordList() != null && immunizationResultCollection.getDataRecordList().getImmunizations() != null) {
            for (ImmunizationV4 bhieImmunization : immunizationResultCollection.getDataRecordList().getImmunizations()) {
                if (bhieImmunization == null) {
                    continue;
                }

                Immunization bean = new Immunization();
                if (getPrimarySpdcV4(bhieImmunization.getVaccine()) != null) {
                    bean.setName(getPrimarySpdcV4(bhieImmunization.getVaccine()).getDisplay());
                }

                if (bhieImmunization.getDate() != null && bhieToXMLCalV4(bhieImmunization.getDate()) != null) {
                    Calendar dateTimeCal = bhieToXMLCalV4(bhieImmunization.getDate()).toGregorianCalendar();
                    bean.setDateTime(Utils.formatDisplayDate(dateTimeCal));
                }
                bean.setSeries(bhieImmunization.getSeriesInfo());
                bean.setNarrativeId(bhieImmunization.getNarrativeId());
                bean.setContraindicated(bhieImmunization.isNegationInd());
                bean.setExemptionRefusalReason(bhieImmunization.getNegationReason());
                if (bhieImmunization.isNegationInd()) {
                    bean.setExemptionRefusalDate(bhieToXMLCalV4(bhieImmunization.getDate()));
                }
                bean.setLot(bhieImmunization.getLotNumber());
                if (bhieImmunization.getManufacturer() != null
                        && getPrimarySpdcV4(bhieImmunization.getManufacturer()) != null) {
                    bean.setManufacturerName(getPrimarySpdcV4(bhieImmunization.getManufacturer()).getDisplay());
                }
                if (bhieImmunization.getProvider() != null) {
                    bean.setOrderingProvider(bhieImmunization.getProvider().getName());
                }
                if (bhieImmunization.getDosage() != null) {
                    bean.setDose(bhieImmunization.getDosage().getRaw());
                    if (bhieImmunization.getDosage().getUnits() != null
                            && !bhieImmunization.getDosage().getUnits().isEmpty()
                            && getPrimarySpdcV4(bhieImmunization.getDosage().getUnits()) != null) {
                        bean.setUnits(getPrimarySpdcV4(bhieImmunization.getDosage().getUnits()).getDisplay());
                    }
                }
                if (bhieImmunization.getRouteOfAdministration() != null
                        && getPrimarySpdcV4(bhieImmunization.getRouteOfAdministration()) != null) {
                    bean.setRoute(getPrimarySpdcV4(bhieImmunization.getRouteOfAdministration()).getDisplay());
                }
                if (bhieImmunization.getSiteOfAdministration() != null
                        && getPrimarySpdcV4(bhieImmunization.getSiteOfAdministration()) != null) {
                    bean.setBodySite(getPrimarySpdcV4(bhieImmunization.getSiteOfAdministration()).getDisplay());
                }
                //SB: Manufactured material maps to CVX for DoD Adaptor v1.5 (v1.6 will use CVX)
                if (bhieImmunization.getVaccine() != null
                        && getPrimarySpdcV4(bhieImmunization.getVaccine()) != null
                        && getPrimarySpdcV4(bhieImmunization.getVaccine()).getCode() != null) {
                    Code code = new Code();
                    code.setPrimary(String.valueOf(getPrimarySpdcV4(bhieImmunization.getVaccine()).isPrimary()));
                    code.setDisplay(getPrimarySpdcV4(bhieImmunization.getVaccine()).getDisplay());
                    code.setSystem(getPrimarySpdcV4(bhieImmunization.getVaccine()).getSystem());
                    code.setCode(getPrimarySpdcV4(bhieImmunization.getVaccine()).getCode());
                    bean.getCodes().add(code);
                }
                if (bhieImmunization.getReaction() != null
                        && bhieImmunization.getReaction().getReaction() != null
                        && !bhieImmunization.getReaction().getReaction().isEmpty()
                        && getPrimarySpdcV4(bhieImmunization.getReaction().getReaction()) != null) {
                    bean.setReaction(getPrimarySpdcV4(bhieImmunization.getReaction().getReaction()).getDisplay());
                }

                if (bhieImmunization.getResults() != null
                        && bhieImmunization.getResults().getResult() != null
                        && !bhieImmunization.getResults().getResult().isEmpty()
                        && getPrimarySpdcV4(bhieImmunization.getResults().getResult()) != null
                        && getPrimarySpdcV4(bhieImmunization.getResults().getResult()).getCode() != null) {
                    bean.setResult(getPrimarySpdcV4(bhieImmunization.getResults().getResult()).getCode());
                } else {
                    bean.setResult("");
                }
 
                if (bhieImmunization.getResults() != null
                        && bhieImmunization.getResults().getSize() != null
                        && !bhieImmunization.getResults().getSize().isEmpty()) {
                    bean.setSize(bhieImmunization.getResults().getSize());
                }
                
                String repositoryId = "";
                String repositoryName = "";
                String recordId = "";
                String facilityName = "";

                if (bhieImmunization.getRepository() != null
                        && bhieImmunization.getRepository().getIds() != null
                        && !bhieImmunization.getRepository().getIds().isEmpty()) {
//                    repositoryId = bhieImmunization.getRepository().getIds().get(0).getId();
                    if(!StringUtils.isEmpty(bhieImmunization.getRepository().getIds().get(0).getId())){
                        repositoryId = bhieImmunization.getRepository().getIds().get(0).getId();
                    } else {
                        repositoryId = bhieImmunization.getRepository().getIds().get(0).getAssigningAuthority();
                    }
                    repositoryName = bhieImmunization.getRepository().getName();
                }
                if (bhieImmunization.getFacility() != null) {
                    facilityName = bhieImmunization.getFacility().getName();
                }
                if (bhieImmunization.getRecordId() != null) {
                    recordId = bhieImmunization.getRecordId().getId();
                }

                if (immunizationResultCollection.isVler()) {
                    addBHIESiteDataToVistaBean(repositoryId,
                            "VLER", facilityName, recordId, bean);
                } else {
                    addBHIESiteDataToVistaBean(repositoryId,
                            repositoryName, facilityName, recordId, bean);
                }
                vistaBeans.add(bean);
            }
        }


        //include status report
        String siteStatusReport = generateBhieStatusReportV4(immunizationResultCollection.getStatusListv4(), immunizationResultCollection.getErrorListv4(), immunizationResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Immunization[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Immunization[vistaBeans.size()]);
    }

    public Immunization[] mapDASVLERImmunizations(Object resultCollection) throws JMeadowsException {
        DasResultCollection vlerResultCollection = (DasResultCollection) resultCollection;
        List<Immunization> vistaBeans = new ArrayList<Immunization>();

        try {

            if (vlerResultCollection.getImmunizations() != null) {
                //SmartAllergy smartAllergy = new SmartAllergy();
                vistaBeans = vlerResultCollection.getImmunizations();

                for(Immunization vistaBean : vistaBeans){
                    if(StringUtils.isNotEmpty(vistaBean.getSite().getSiteCode())){
                        String vlerSiteName = getVLERSiteName(vistaBean.getSite().getSiteCode());
                        if(StringUtils.isNotEmpty(vlerSiteName)) {
                            vistaBean.getSite().setName(vlerSiteName);
                        }
                    }
                }
            }

            String siteStatusReport = generateBhieStatusReportV4(mapDASStatusList(vlerResultCollection.getStatusList()),
                    vlerResultCollection.getErrorList(), true);

            vistaBeans.addAll(Arrays.asList(
                    (Immunization[]) generateConnectionUnavailableBean(
                            BHIE_STATUS_REPORT_LOCATION,
                            siteStatusReport)
            ));

        } catch (Exception ex) {

            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }

        return vistaBeans.toArray(new Immunization[vistaBeans.size()]);
    }

//    public Immunization[] mapImmunizationsFromCHCS(Object[] dodBean) {
//        int size = dodBean.length;
//        Immunization[] vistaBean = new Immunization[size];
//
//        StringBuilder reaction = new StringBuilder();
//        for (int i = 0; i < dodBean.length; i++) {
//            Immunization bean = new Immunization();
//            bean.setName(((ImmunizationBean) dodBean[i]).getVaccine());
//            bean.setDateTime(Utils.getXMLCal(((ImmunizationBean) dodBean[i]).getAdministerDT()));
//            bean.setSeries(((ImmunizationBean) dodBean[i]).getDoseNumber());
//            reaction.setLength(0);
//            if (((ImmunizationBean) dodBean[i]).getResult() != null && ((ImmunizationBean) dodBean[i]).getResult().length() > 0) {
//                reaction.append("Result: ");
//                reaction.append(((ImmunizationBean) dodBean[i]).getResult());
//            }
//            if (((ImmunizationBean) dodBean[i]).getResultComment() != null && ((ImmunizationBean) dodBean[i]).getResultComment().length() > 0) {
//                reaction.append("\nResult Comment: ");
//                reaction.append(((ImmunizationBean) dodBean[i]).getResultComment());
//            }
////        bean.setSiteCode(((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getLocationCode());
////       bean.setSiteName(((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getLocationName());
////        bean.setName(((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getVaccine());
////        bean.setDateTime(((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getAdministerDT());
////        bean.setSeries(((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getDoseNumber());
////        String temp="";
////        if (((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getResult() != null && ((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getResult().length()>0)
////          temp = "Result: " + ((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getResult();
////        if (((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getResultComment() != null && ((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getResultComment().length()>0)
////          temp = temp + "\nResult Comment: " + ((Mil_osd_ha_citpo_jvi_webservice_beans_ImmunizationBean)dodBean[i]).getResultComment();
//            bean.setReaction(reaction.toString());
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Immunization imm = new Immunization();

        imm.setName(errorMsg);
        imm.setDateTime(Utils.formatDisplayDate(Utils.backDate()));
        imm.setSite(getConnectionUnavailableSite(location));

        return new Immunization[]{imm};
    }
}