/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Medication;
import gov.va.med.vds.webservice.PrescriptionFill;
import gov.va.med.vds.webservice.Provider;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartMedication extends SmartBean implements Comparable {
//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartMedication.class);

//    public Prescription m_prescription;

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;
    public static final int ACTIVE = 4;
    public static final String VA_ORIGIN = "15177975";
    public static final String VA_REPOSITORY = "2.16.840.1.113883.3.42.10002.100001.8.22200";

    public Medication m_medication;

    public SmartMedication() {
        super();
        m_medication = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartMedication(Medication med, String sort) {
        super();
        m_medication = med;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartMedication) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_medication);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = array1;
        } else if (array1 == null) {
            rtc = array2;
        } else {
            int size = array1.length + array2.length;
            SmartMedication[] temp = new SmartMedication[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartMedication newAll = new SmartMedication((Medication) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartMedication newAll = new SmartMedication((Medication) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Medication[] ret = new Medication[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_medication;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartMedication[] temp = new SmartMedication[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartMedication newAll = new SmartMedication((Medication) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Medication[] ret = new Medication[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_medication;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartMedication tmp = (SmartMedication) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_medication.getLastFilledDate(), tmp.m_medication.getLastFilledDate());
        }
        if (this.m_sortBy == SITE && this.m_medication.getSite() != null && tmp.m_medication.getSite() != null) {
            retVal = compareString(this.m_medication.getSite().getSiteCode(), tmp.m_medication.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_medication.getDrugName(), tmp.m_medication.getDrugName());
        }
        if (this.m_sortBy == ACTIVE) {
            retVal = compareString(this.m_medication.getActive(), tmp.m_medication.getActive());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }


        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("fillOrderDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("drugName".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
            if ("active".equalsIgnoreCase(sort)) {
                m_sortBy = ACTIVE;
            }
        }
    }

    /**
     * Map outpatient medications from BHIE.
     * <p/>
     * This method implements the following business rules, as outlined in the BHIE 5 ICD:
     * <p/>
     * 1. If the bhie medication comment field contains the string "^I^" then the medication type is "inpatient"
     * otherwise it is considered "outpatient".
     * 2. Any string beginning and ending with a caret ("^") should be removed from the comment text before
     * displaying to an end user (this mapping function will remove the caret from the comment field when
     * mapping)
     *
     * @param resultCollection
     * @return
     */
    public Medication[] getOutpatientMedicationsFromBHIE(Object resultCollection) {
        V4ResultCollection medicationResultCollection = (V4ResultCollection) resultCollection;
        List<Medication> vistaBeans = new ArrayList<Medication>();

        if (medicationResultCollection.getDataRecordList() != null && medicationResultCollection.getDataRecordList().getMedications() != null) {
//            StringBuilder sb = new StringBuilder();
            for (MedicationV4 bhieMed : medicationResultCollection.getDataRecordList().getMedications()) {
                if (bhieMed == null) {
                    continue;
                }
                if (bhieMed.getCareSetting() != null && bhieMed.getCareSetting().toLowerCase().indexOf("inpatient") >= 0) {
                    continue;
                }
                //only map outpatient medications, check for "^I^".
//                String comments = bhieMed.getComment();

                if(bhieMed.getRepository() != null &&
                        bhieMed.getRepository().getIds() != null &&
                        !bhieMed.getRepository().getIds().isEmpty() &&
                        bhieMed.getRepository().getIds().get(0) != null &&
                        VA_REPOSITORY.equalsIgnoreCase(bhieMed.getRepository().getIds().get(0).getAssigningAuthority())){
                    continue;
                }
                if(bhieMed.getSystemOfOrigin() != null
                        && bhieMed.getSystemOfOrigin().getCode() != null
                        && VA_ORIGIN.equalsIgnoreCase(bhieMed.getSystemOfOrigin().getCode())){
                    //ignore VA meds, they will show up as duplicates
                    continue;
                }

                vistaBeans.addAll(mapMedicationFromBHIE(bhieMed, "O"));
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(medicationResultCollection.getStatusListv4(),
                medicationResultCollection.getErrorListv4(), medicationResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Medication[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Medication[vistaBeans.size()]);
    }

    /**
     * Map inpatient medications from BHIE.
     * <p/>
     * This method implements the following business rules, as outlined in the BHIE 5 ICD:
     * <p/>
     * 1. If the bhie medication comment field contains the string "^I^" then the medication type is "inpatient"
     * otherwise it is considered "outpatient".
     * 2. Any string beginning and ending with a caret ("^") should be removed from the comment text before
     * displaying to an end user (this mapping function will remove the caret from the comment field when
     * mapping)
     *
     * @param resultCollection
     * @return
     */
    public Medication[] getInpatientMedicationsFromBHIE(Object resultCollection) {
        V4ResultCollection medicationResultCollection = (V4ResultCollection) resultCollection;
        List<Medication> vistaBeans = new ArrayList<Medication>();

        if (medicationResultCollection.getDataRecordList() != null && medicationResultCollection.getDataRecordList().getMedications() != null) {

            for (MedicationV4 bhieMed : medicationResultCollection.getDataRecordList().getMedications()) {
                if (bhieMed == null) {
                    continue;
                }
                if (bhieMed.getCareSetting() == null || bhieMed.getCareSetting().toLowerCase().indexOf("inpatient") < 0) {
                    continue;
                }
                if(bhieMed.getRepository() != null &&
                        bhieMed.getRepository().getIds() != null &&
                        !bhieMed.getRepository().getIds().isEmpty() &&
                        bhieMed.getRepository().getIds().get(0) != null &&
                        VA_REPOSITORY.equalsIgnoreCase(bhieMed.getRepository().getIds().get(0).getAssigningAuthority())){
                    continue;
                }
                if(bhieMed.getSystemOfOrigin() != null
                        && bhieMed.getSystemOfOrigin().getCode() != null
                        && VA_ORIGIN.equalsIgnoreCase(bhieMed.getSystemOfOrigin().getCode())){
                    //ignore VA meds, they will show up as duplicates
                    continue;
                }
                vistaBeans.addAll(mapMedicationFromBHIE(bhieMed, "I"));
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(medicationResultCollection.getStatusListv4(),
                medicationResultCollection.getErrorListv4(), medicationResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Medication[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Medication[vistaBeans.size()]);
    }

    // Version 4
    private List<Medication> mapMedicationFromBHIE(gov.va.med.bhie.MedicationV4 bhieMed, String medType) {

        List<Medication> vistaBeans = new ArrayList<Medication>();

        if (bhieMed.getDrugs() != null && !bhieMed.getDrugs().isEmpty()) {
            for (int i = 0; i < bhieMed.getDrugs().size(); i++) {
                Medication bean = new Medication();

                if (bhieMed.getDrugs().get(i) != null) {
                    DrugV4 drug = bhieMed.getDrugs().get(i);
                    if (drug.getDrug() != null && !drug.getDrug().isEmpty() && getPrimarySpdcV4(drug.getDrug()) != null) {
                        bean.setDrugName(getPrimarySpdcV4(drug.getDrug()).getDisplay());
                    }

                    if(drug.getDosage() != null && drug.getDosage().getUnits() != null
                            && !drug.getDosage().getUnits().isEmpty()
                            && getPrimarySpdcV4(drug.getDosage().getUnits()) != null){
                        bean.setUnits(getPrimarySpdcV4(drug.getDosage().getUnits()).getDisplay());
                    }

                    for (SpdcBeanV4 spdcBean : drug.getDrug()) {
                        if (spdcBean != null) {
                            Code code = new Code();
                            code.setSystem(spdcBean.getSystem());
                            code.setDisplay(spdcBean.getDisplay());
                            if(spdcBean.getCode() != null && !spdcBean.getCode().isEmpty()){
                                bean.setCoded(true);
                            }
                            code.setCode(spdcBean.getCode());
                            bean.getCodes().add(code);
                        }
                    }
                }

                fillMedBean(bhieMed, medType, bean);

                vistaBeans.add(bean);
            }
        } else {
            //for records where the drugs array is null
            Medication bean = new Medication();

            bean.setDrugName("No medication name supplied");

            fillMedBean(bhieMed, medType, bean);

            vistaBeans.add(bean);
        }

        return vistaBeans;
    }

    private void fillMedBean(MedicationV4 bhieMed, String medType, Medication bean){
        if(bhieMed.getSystemOfOrigin() == null && bhieMed.isOtc()){
            bean.setManuallyEntered(true);
        }

        if (bhieMed.getComment() != null) {
            bean.setComment(bhieMed.getComment());
        }

        if (bhieMed.getOrderDate() != null) {
            bean.setOrderDate(bhieToXMLCalV4(bhieMed.getOrderDate()));
        }
        if (bhieMed.getLastFillDate() != null) {
            bean.setLastFilledDate(bhieToXMLCalV4(bhieMed.getLastFillDate()));
        }

        if (bhieMed.getStartDate() != null) {
            bean.setStartDate(bhieToXMLCalV4(bhieMed.getStartDate()));
        }
        if (bhieMed.getEndDate() != null) {
            bean.setStopDate(bhieToXMLCalV4(bhieMed.getEndDate()));
        }
        if(bhieMed.getCurrentSupplyEndDate() != null){
            bean.setCurrentSupplyEndDate(bhieToXMLCalV4(bhieMed.getCurrentSupplyEndDate()));
        }
        if (bhieMed.getOrderNumber() != null) {
            bean.setOrderId(bhieMed.getOrderNumber().getId());
        }

        Provider orderingProvider = new Provider();

        if (bhieMed.getProvider() != null) {
            orderingProvider = mapProvider(bhieMed.getProvider());

        }

        bean.setOrderingProvider(orderingProvider);
        bean.setCurrentProvider(orderingProvider);
        bean.setDaysSupply(bhieMed.getDaysSupply());

        if (bhieMed.getDispensingPharmacy() != null) {
            bean.setLastDispensingPharmacy(bhieMed.getDispensingPharmacy().getName());
        }
        if(bhieMed.getQuantity() != null && bhieMed.getQuantity().getRaw() != null){
            bean.setQuantity(bhieMed.getQuantity().getRaw());
        }

        bean.setRefills(bhieMed.getNumberOfRefills());
        bean.setSigCode(bhieMed.getSig());
        if (bhieMed.getRecordId() != null) {
            bean.setMedId(bhieMed.getRecordId().getId());
        }
        if (bhieMed.getStatus() != null
                && !bhieMed.getStatus().isEmpty()
                && bhieMed.getStatus().get(0) != null) {
            bean.setStatus(bhieMed.getStatus().get(0).getDisplay());
        }
        bean.setMedType(medType);
        if (bhieMed.getRxNumber() != null) {
            bean.setRxNumber(bhieMed.getRxNumber().getId());
        }

        bean.setChildResistant(bhieMed.isChildResistant());
        bean.setFillExpirationDate(bhieToXMLCalV4(bhieMed.getEndDate()));
        bean.setRefillsRemaining(bhieMed.getRefillsRemaining());

        String repositoryId = "";
        String repositoryName = "";
        String facilityName = "";
        String eventId = "";
        if (bhieMed.getRepository() != null && bhieMed.getRepository().getIds() != null && !bhieMed.getRepository().getIds().isEmpty()) {
            repositoryId = bhieMed.getRepository().getIds().get(0).getId();
        }
        if (bhieMed.getRepository() != null) {
            repositoryName = bhieMed.getRepository().getName();
        }
        if (bhieMed.getFacility() != null) {
            facilityName = bhieMed.getFacility().getName();
        }
        if (bhieMed.getRecordId() != null) {
            eventId = bhieMed.getRecordId().getId();
        }
        if (repositoryId == null || repositoryId.length() == 0) {
            repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
        }
        if (repositoryName == null || repositoryName.length() == 0) {
            repositoryName = "ahlta";
        }
        addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                facilityName, eventId, bean);

        //add fill dates
        List<MedicationFillInfoV4> medFillInfoList = bhieMed.getFillInfo();
        for (MedicationFillInfoV4 medFillInfo : medFillInfoList) {
            PrescriptionFill prescriptionFill = new PrescriptionFill();
            prescriptionFill.setDispenseDate(bhieToXMLCalV4(medFillInfo.getFillDate()));
            if (medFillInfo.getDispensingPharmacy() != null) {
                prescriptionFill.setDispensingPharmacy(medFillInfo.getDispensingPharmacy().getName());
            }
//                    for (SpdcBeanV4 action : medFillInfo.getAction()) {
//                        if (action != null && "Entered".equalsIgnoreCase(action.getDisplay())) {
//                            bean.setManuallyEntered(true);
//                        }
//                    }

            if(medFillInfo.getQuantity() != null && medFillInfo.getQuantity().getRaw() != null){
                prescriptionFill.setDispensingQuantity(medFillInfo.getQuantity().getRaw());
            }

            repositoryId = "";
            repositoryName = "";
            facilityName = "";
            eventId = "";
            if (bhieMed.getRepository() != null && bhieMed.getRepository().getIds() != null && !bhieMed.getRepository().getIds().isEmpty()) {
//                repositoryId = bhieMed.getRepository().getIds().get(0).getId();
                if(!StringUtils.isEmpty(bhieMed.getRepository().getIds().get(0).getId())){
                    repositoryId = bhieMed.getRepository().getIds().get(0).getId();
                } else {
                    repositoryId = bhieMed.getRepository().getIds().get(0).getAssigningAuthority();
                }
            }
            if (bhieMed.getRepository() != null) {
                repositoryName = bhieMed.getRepository().getName();
            }
            if (bhieMed.getFacility() != null) {
                facilityName = bhieMed.getFacility().getName();
            }
            if (bhieMed.getRecordId() != null) {
                eventId = bhieMed.getRecordId().getId();
            }
            addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                    facilityName, eventId, prescriptionFill);

            bean.getPrescriptionFills().add(prescriptionFill);
        }


    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Medication med = new Medication();

        med.setDrugName(errorMsg);
        med.setLastFilledDate(Utils.backDate());
        med.setOrderDate(Utils.backDate());
        med.setSite(getConnectionUnavailableSite(location));

        return new Medication[]{med};
    }
}