package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.NoteV4;
import gov.va.med.bhie.RepositoryV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.MhsGenesis;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartMhsGenesis extends SmartBean implements Comparable {
    public static final int DATE = 1;
    public static final int SITE = 2;

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartMhsGenesis.class);

    public MhsGenesis m_mhsGenesis;

    public SmartMhsGenesis()
    {
        super();
        m_mhsGenesis = null;
        m_sortBy = DATE;
        m_bAscending = false;
    }

    public SmartMhsGenesis(MhsGenesis mhsGenesis, String sort)
    {
        super();
        m_mhsGenesis = mhsGenesis;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj)
    {
        boolean rtc;

        if (this == obj) {
            rtc = true;
        }
        else if (obj instanceof SmartMhsGenesis) {
            rtc = compareTo(obj) == 0;
        }
        else {
            rtc = false;
        }

        return rtc;
    }

    @Override
    public int hashCode()
    {
        int result = HashCodeUtil.SEED;

        result = HashCodeUtil.hash(result, m_mhsGenesis);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort)
    {
        Object[] rtc;

        if (array2 == null) {
            rtc = sort(array1, sort);
        }
        else if (array1 == null) {
            rtc = sort(array2, sort);
        }
        else {
            int size = array1.length + array2.length;
            SmartMhsGenesis[] temp = new SmartMhsGenesis[size];
            int cnt = 0;

            for (int i = 0; i < array1.length; i++) {
                SmartMhsGenesis newAll = new SmartMhsGenesis((MhsGenesis) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }

            for (int i = 0; i < array2.length; i++) {
                SmartMhsGenesis newAll = new SmartMhsGenesis((MhsGenesis) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }

            Arrays.sort(temp);
            MhsGenesis[] ret = new MhsGenesis[temp.length];

            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_mhsGenesis;
            }

            rtc = ret;
        }

        return rtc;
    }

    public Object[] sort(Object[] array1, String sort)
    {
        Object[] rtc;

        if (array1 == null) {
            rtc = null;
        }
        else {
            int size = array1.length;
            SmartMhsGenesis[] temp = new SmartMhsGenesis[size];
            int cnt = 0;

            for (int i = 0; i < array1.length; i++) {
                SmartMhsGenesis newAll = new SmartMhsGenesis((MhsGenesis) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }

            Arrays.sort(temp);
            MhsGenesis[] ret = new MhsGenesis[temp.length];

            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_mhsGenesis;
            }

            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj)
    {
        SmartMhsGenesis tmp = (SmartMhsGenesis) obj;
        int retVal = 0;

        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_mhsGenesis.getEnteredDate(), tmp.m_mhsGenesis.getEnteredDate());
        }

        if (this.m_sortBy == SITE && this.m_mhsGenesis.getSite() != null && tmp.m_mhsGenesis.getSite() != null) {
            retVal = compareString(this.m_mhsGenesis.getSite().getSiteCode(), tmp.m_mhsGenesis.getSite().getSiteCode());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort)
    {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);

            if ("appointmentDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }

            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg)
    {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        MhsGenesis mhsGenesis = new MhsGenesis();
        mhsGenesis.setTitle(errorMsg);
        mhsGenesis.setEnteredDate(Utils.backDate().toGregorianCalendar());
        mhsGenesis.setSite(getConnectionUnavailableSite(location));
        return new MhsGenesis[]{mhsGenesis};
    }

    public MhsGenesis[] mapMhsGenesisFromBhie(Object resultCollection) throws JMeadowsException
    {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<MhsGenesis> genesisList = new ArrayList<MhsGenesis>();

        try {
            if (noteResultCollection.getDataRecordList() != null) {
                List<NoteV4> noteList = new ArrayList<NoteV4>();

                if (noteResultCollection.getDataRecordList().getDocuments() != null
                        && noteResultCollection.getDataRecordList().getDocuments().size() > 0) {
                    noteList.addAll(noteResultCollection.getDataRecordList().getDocuments());
                }

                MhsGenesis genesis;
                SpdcBeanV4 spdc;
                Code code;
                RepositoryV4 repository;
                String repositoryId;
                String repositoryName;
                String facilityName;
                String eventId;

                for (NoteV4 note : noteList) {
                    if (note == null) {
                        continue;
                    }

                    genesis = new MhsGenesis();

                    genesis.setComplexDataUrl(note.getComplexDataUrl());
                    genesis.setSensitive(note.getSensitive());
                    genesis.setTitle(note.getTitle());

                    // Content type
                    if (note.getContentType() != null && !note.getContentType().isEmpty()) {
                        spdc = getPrimarySpdcV4(note.getContentType());
                        if (spdc != null) {
                            genesis.setContentType(spdc.getCode());
                        }
                    }

                    // Data domain
                    if (note.getDataDomain() != null && !note.getDataDomain().isEmpty()) {
                        code = new Code();
                        code.setDisplay(note.getDataDomain().get(0).getDisplay());
                        code.setSystem(note.getDataDomain().get(0).getSystem());
                        code.setCode(note.getDataDomain().get(0).getCode());
                        genesis.getCodes().add(code);
                    }

                    // Entered date
                    if (note.getEnteredDate() != null && note.getEnteredDate().getStart() != null) {
                        genesis.setEnteredDate(bhieToXMLCalV4(note.getEnteredDate()).toGregorianCalendar());
                    }

                    // Facility
                    if (note.getFacility() != null) {
                        facilityName = note.getFacility().getName();
                        if (StringUtils.isNotBlank(facilityName) && StringUtils.contains(facilityName, "^")) {
                            facilityName = facilityName.substring(StringUtils.indexOf(facilityName, '^') + 1);
                            genesis.setFacility(facilityName);
                        }
                    }

                    // Type
                    if (note.getType() != null && !note.getType().isEmpty()) {
                        spdc = note.getType().get(0);
                        if (spdc != null) {
                            genesis.setType(spdc.getDisplay());
                        }
                    }

                    repositoryId = "";
                    repositoryName = "";
                    facilityName = "";
                    eventId = "";

                    repository = note.getRepository();
                    if (repository != null) {
                        if (repository.getIds() != null && !repository.getIds().isEmpty()) {
                            repositoryId = repository.getIds().get(0).getId();
                        }
                        repositoryName = repository.getName();
                    }

                    if (note.getFacility() != null) {
                        facilityName = note.getFacility().getName();
                    }

                    if (note.getRecordId() != null) {
                        eventId = note.getRecordId().getId();
                    }

                    if (StringUtils.isBlank(repositoryId)) {
                        repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                    }

                    if (StringUtils.isBlank(repositoryName)) {
                        repositoryName = "ahlta";
                    }
		    /*
		    JLV-1346:
2) date field should be no date for Medical CCD, Dental CCD, Aggregate CCD; blank dates should be sorted at the top.
3) Name the CCD documents the following: a) Medical CCD should be ""Medical Summary""; b) Dental CCD should be ""Dental Summary""; c) Aggregate CCD should be ""Path, Micro, Rad, & Dental Notes""
            */
                    if (StringUtils.equals("Medical CCD", genesis.getTitle())) {
                        genesis.setTitle("Medical Summary");
                        genesis.setEnteredDate(null);
                    } else if (StringUtils.equals("Dental CCD", genesis.getTitle())) {
                        genesis.setTitle("Dental Summary");
                        genesis.setEnteredDate(null);
                    } else if (StringUtils.equals("Aggregate CCD", genesis.getTitle())) {
                        genesis.setTitle("Path, Micro, Rad, & Dental Notes");
                        genesis.setEnteredDate(null);
                    }

                    addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, genesis);

                    genesisList.add(genesis);
                }
            }
        }
        catch (Exception ex) {
            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }

        String siteStatusReport = generateBhieStatusReportV4(noteResultCollection.getStatusListv4(),
                noteResultCollection.getErrorListv4(), noteResultCollection.isVler());

        genesisList.addAll(Arrays.asList((MhsGenesis[]) generateConnectionUnavailableBean(BHIE_STATUS_REPORT_LOCATION, siteStatusReport)));

        return genesisList.toArray(new MhsGenesis[genesisList.size()]);
    }
}
