/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Procedure;
import gov.va.med.vhahon.common.Utils;
//import org.hui.chcs.ProcedureBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

//import gov.va.med.vhahon.beans.LabOrder;
//import gov.va.med.vhahon.beans.Procedure;
//import gov.va.med.chcs.beans.ProcedureBean;
//import gov.va.med.vhahon.beans.*;
//import gov.va.med.chcs.beans.*;
//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartProcedure extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartProcedure.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;

    public Procedure m_procedure;

    public SmartProcedure() {
        super();
        m_procedure = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartProcedure(Procedure proc, String sort) {
        super();
        m_procedure = proc;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartProcedure) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_procedure);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartProcedure[] temp = new SmartProcedure[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProcedure newAll = new SmartProcedure((Procedure) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartProcedure newAll = new SmartProcedure((Procedure) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Procedure[] ret = new Procedure[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_procedure;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartProcedure[] temp = new SmartProcedure[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProcedure newAll = new SmartProcedure((Procedure) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Procedure[] ret = new Procedure[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_procedure;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartProcedure tmp = (SmartProcedure) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_procedure.getDate(), tmp.m_procedure.getDate());
        }
        if (this.m_sortBy == SITE && this.m_procedure.getSite() != null && tmp.m_procedure.getSite() != null) {
            retVal = compareString(this.m_procedure.getSite().getSiteCode(), tmp.m_procedure.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_procedure.getDescription(), tmp.m_procedure.getDescription());
        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("date".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
        }

    }

//    public Procedure[] getProceduresFromCHCS(Object[] dodBean) {
//        ProcedureBean[] chcsBean = (ProcedureBean[]) dodBean;
//        int size = chcsBean.length;
//        Procedure[] vistaBean = new Procedure[size];
//        Procedure bean = null;
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            bean = new Procedure();
//            bean.setCode(chcsBean[i].getCode());
//            bean.setDate(Utils.getXMLCal(chcsBean[i].getProcedureDate()));
//            bean.setDescription(chcsBean[i].getProcedure());
//            bean.setStatus(chcsBean[i].getStatus());
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//        }
//
//        return vistaBean;
//
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Procedure proc = new Procedure();
        proc.setDescription(errorMsg);
        proc.setDate(Utils.backDate());
        proc.setSite(getConnectionUnavailableSite(location));
        return new Procedure[]{proc};
    }

}