/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.PatientDocument;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.*;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static gov.va.med.jmeadows.dao.hdd.VLERSiteCache.getVLERSiteName;

//import gov.va.med.vhahon.beans.Procedure;
//import gov.va.med.vhahon.beans.Vitals;
//import gov.va.med.vhahon.beans.*;
//import mil.osd.ha.citpo.jvi.webservice.*;
//import gov.va.medora.HuiSvc.TaggedVitalSignArray;
//import gov.va.medora.HuiSvc.VitalSignSetTO;
//import gov.va.medora.HuiSvc.VitalSignTO;
//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartVitals extends SmartBean implements Comparable {

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;

    //vital type snomeds
//    private static final int SNOMED_BLOOD_PRESSURE = 75367002;
    private static final int SNOMED_DIASTOLIC_BP = 271650006;
    private static final int SNOMED_SYSTOLIC_BP = 271649006;
    private static final int SNOMED_HEART_RATE = 364075005;
    private static final int SNOMED_RESPITORY_RATE = 86290005;
    private static final int SNOMED_HEIGHT = 50373000;
    private static final int SNOMED_02_SATURATION = 103228002;
    private static final int SNOMED_PAIN_SEVERITY = 405161002;
    private static final int SNOMED_TEMPERATURE = 386725007;
    private static final int SNOMED_WEIGHT = 27113001;
    private static final int SNOMED_CG = 248365001;

    //vital test unit snomeds
    private static final int SNOMED_PERCENT = 118582008;
    private static final int SNOMED_PAIN_SCALE = 419410004;
    private static final int SNOMED_PER_MIN = 258983007;
    private static final int SNOMED_CELSIUS = 258710007;
    private static final int SNOMED_FAHRENHEIT = 258712004;
    private static final int SNOMED_INCHES = 258677007;
    private static final int SNOMED_CENTIMETERS = 258672001;
    private static final int SNOMED_MMHG = 259018001;
    private static final int SNOMED_OUNCES = 258692008;
    private static final int SNOMED_POUNDS = 258693003;
    private static final int SNOMED_KILOGRAMS = 258683005;
    private static final int SNOMED_GRAM = 258682000;


    //    private static String VISTA_LABEL_VITAL_SIGN = "VITAL SIGN";
    private static String VISTA_LABEL_TEMPERATURE = "TEMPERATURE";
    private static String VISTA_LABEL_PULSE = "PULSE";
    private static String VISTA_LABEL_RESPIRATION = "RESPIRATION";
    private static String VISTA_LABEL_BLOOD_PRESSURE = "BLOOD PRESSURE";
    private static String VISTA_LABEL_HEIGHT = "HEIGHT";
    private static String VISTA_LABEL_WEIGHT = "WEIGHT";
    private static String VISTA_LABEL_PAIN = "PAIN";
    private static String VISTA_LABEL_PULSE_OXIMETRY = "PULSE OXIMETRY";
    private static String VISTA_LABEL_CIRCUMFERENCE_GIRTH = "CIRCUMFERENCE / GIRTH";
//    private static String VISTA_LABEL_CENTRAL_VENOUS_PRESSURE = "CENTRAL VENOUS PRESSURE";
//    private static String VISTA_LABEL_BODY_MASS_INDEX = "BODY MASS INDEX";

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartVitals.class);

    public gov.va.med.vds.webservice.Vitals m_vitals;

    public SmartVitals() {
        super();
        m_vitals = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartVitals(gov.va.med.vds.webservice.Vitals vital, String sort) {
        super();
        m_vitals = vital;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartVitals) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_vitals);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartVitals[] temp = new SmartVitals[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartVitals newAll = new SmartVitals((gov.va.med.vds.webservice.Vitals) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartVitals newAll = new SmartVitals((gov.va.med.vds.webservice.Vitals) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            gov.va.med.vds.webservice.Vitals[] ret = new gov.va.med.vds.webservice.Vitals[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_vitals;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartVitals[] temp = new SmartVitals[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartVitals newAll = new SmartVitals((gov.va.med.vds.webservice.Vitals) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            gov.va.med.vds.webservice.Vitals[] ret = new gov.va.med.vds.webservice.Vitals[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_vitals;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartVitals tmp = (SmartVitals) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_vitals.getDateTimeTaken(), tmp.m_vitals.getDateTimeTaken());
        }
        if (this.m_sortBy == SITE && this.m_vitals.getSite() != null && tmp.m_vitals.getSite() != null) {
            retVal = compareString(this.m_vitals.getSite().getSiteCode(), tmp.m_vitals.getSite().getSiteCode());
        }
        if (this.m_sortBy == TYPE) {
            retVal = compareString(this.m_vitals.getVitalType(), tmp.m_vitals.getVitalType());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("datetimetaken".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("vitaltype".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
        }

    }

//  public static Vitals[] getCurrentVitalsFromMDWS(TaggedVitalSignArray[] vitalsArr)
//  {
//      int size = 0;
//      for(TaggedVitalSignArray vta : vitalsArr)
//      {
//          if (vta.getVitals() != null)
//          {
//              size += vta.getVitals().getVitalSignTO().length;
//          }
//      }
//
//      Vitals[] vistaVitals = new Vitals[size];
//
//      if (size > 0)
//      {
//          int i = 0;
//          for(TaggedVitalSignArray vta : vitalsArr)
//          {
//              if (vta.getVitals() != null)
//              {
//                for(VitalSignTO vital : vta.getVitals().getVitalSignTO())
//                {
//                    Vitals vistaVital = new Vitals();
//
//                    vistaVital.setSiteCode(vta.getTag());
//                    vistaVital.setDateTimeTaken(Utils.getXMLCal(vital.getTimestamp()));
//                    vistaVital.setRate(vital.getValue1());
//                    if (vital.getType() != null)
//                    {
//                        vistaVital.setVitalType(vital.getType().getName());
//                    }
//
//                    vistaVitals[i] = vistaVital;
//                    i++;
//                }
//              }
//          }
//      }
//
//      return vistaVitals;
//  }
//
//  public static Vitals[] getVitalsFromMDWS(TaggedVitalSignSetArray[] vitalsSetArr)
//  {
//      int size = 0;
//      for(TaggedVitalSignSetArray tv : vitalsSetArr)
//      {
//          if (tv.getSets() != null)
//          {
//              for(VitalSignSetTO vto : tv.getSets().getVitalSignSetTO())
//              {
//                size += vto.getVitalSigns().getVitalSignTO().length;
//              }
//          }
//      }
//
//      Vitals[] vistaVitals = new Vitals[size];
//
//      if (size > 0)
//      {
//          int i = 0;
//          for(TaggedVitalSignSetArray tv : vitalsSetArr)
//          {
//            if (tv.getSets() != null)
//            {
//                for(VitalSignSetTO vto : tv.getSets().getVitalSignSetTO())
//                {
//                    for(VitalSignTO vital : vto.getVitalSigns().getVitalSignTO())
//                    {
//                        Vitals vistaVital = new Vitals();
//
//                        vistaVital.setSiteCode(tv.getTag());
//                        vistaVital.setDateTimeTaken(Utils.getXMLCal(vto.getTimestamp()));
//                        vistaVital.setRate(vital.getValue1());
//                        if (vital.getType() != null)
//                        {
//                            vistaVital.setVitalType(vital.getType().getName());
//                        }
//
//                        vistaVitals[i] = vistaVital;
//                        i++;
//                    }
//                }
//              }
//          }
//      }
//      return vistaVitals;
//  }

//    public Vitals[] mapVitalsFromBHIE(Object resultCollection) {
//        VitalsResultCollection vitalsResultCollection = (VitalsResultCollection) resultCollection;
//        List<Vitals> vistaBeans = new ArrayList<Vitals>();
//
//        if (vitalsResultCollection.getDataRecordList() != null) {
//
//            for (VitalsDataRecord dataRecord : vitalsResultCollection.getDataRecordList()) {
//                if (dataRecord == null || dataRecord.getVitals() == null) {
//                    continue;
//                }
//
//                gov.va.med.bhie.Vitals bhieVitals = dataRecord.getVitals();
//                List<gov.va.med.bhie.VitalResult> vitalResults = bhieVitals.getVitalResults();
//
//
//                if (vitalResults != null) {
//                    Map<String, VitalResult> bpResults = new LinkedHashMap<String, VitalResult>();
//                    for (gov.va.med.bhie.VitalResult vitalResult : vitalResults) {
//                        String testCodeStr = vitalResult.getTestCode();
//
//                        Vitals bean = new Vitals();
//                        bean.setDateTimeTaken(bhieToXMLCal(vitalResult.getPerformedDate()));
//                        bean.setRate(vitalResult.getTestValue());
//                        bean.setVitalType(vitalResult.getTestName());
//                        bean.setUnits(vitalResult.getTestUnits());
//                        bean.setUnitsCode(mapUnitsCodeToUnitsString(vitalResult.getTestUnitsCode()));
//
//                        //attempt to map vital by snomed code
//                        try {
//                            int vitalSnomed = Integer.parseInt(testCodeStr);
//
//                            switch (vitalSnomed) {
//                                case SNOMED_DIASTOLIC_BP:
//                                    bean = null;
//
//                                    bpResults.put("diastolic", vitalResult);
//
//                                    if (bpResults.containsKey("systolic")) {
//                                        bean = mapBloodPressureSystolicDiastolic(bpResults);
//
//                                    }
//
//
//                                    break;
//                                case SNOMED_SYSTOLIC_BP:
//                                    bean = null;
//
//                                    bpResults.put("systolic", vitalResult);
//
//                                    if (bpResults.containsKey("diastolic")) {
//                                        bean = mapBloodPressureSystolicDiastolic(bpResults);
//                                    }
//
//                                    break;
//                                case SNOMED_WEIGHT:
//                                    bean.setVitalType(VISTA_LABEL_WEIGHT);
//                                    break;
//                                case SNOMED_HEIGHT:
//                                    bean.setVitalType(VISTA_LABEL_HEIGHT);
//                                    break;
//                                case SNOMED_RESPITORY_RATE:
//                                    bean.setVitalType(VISTA_LABEL_RESPIRATION);
//                                    break;
//                                case SNOMED_TEMPERATURE:
//                                    bean.setVitalType(VISTA_LABEL_TEMPERATURE);
//                                    break;
//                                case SNOMED_HEART_RATE:
//                                    bean.setVitalType(VISTA_LABEL_PULSE);
//                                    break;
//                                case SNOMED_PAIN_SEVERITY:
//                                    bean.setVitalType(VISTA_LABEL_PAIN);
//                                    break;
//                                case SNOMED_02_SATURATION:
//                                    bean.setVitalType(VISTA_LABEL_PULSE_OXIMETRY);
//                                    break;
//                                case SNOMED_CG:
//                                    bean.setVitalType(VISTA_LABEL_CIRCUMFERENCE_GIRTH);
//                                    break;
//                                default:
//                                    LOGGER.debug("Could not map vital type by snomed.");
//                                    break;
//                            }
//                        } catch (NumberFormatException e) {
//                            LOGGER.debug("Failed to map vital by snomed code: " + e.getMessage());
//                        }
//
//                        if (bean != null) {
//                            if (VISTA_LABEL_BLOOD_PRESSURE.equals(bean.getVitalType())) {
//                                //assign blood pressure LOINC to combined systolic / diastolic vital
//                                Code code = new Code();
//                                code.setSystem(CodingSystem.LOINC.getDisplayName());
//                                code.setCode("55284-4");
//                                bean.getCodes().add(code);
//                            } else {
//                                if (StringUtils.isNotBlank(vitalResult.getNcid())) {
//                                    Code code = new Code();
//                                    code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
//                                    code.setCode(vitalResult.getNcid());
//                                    bean.getCodes().add(code);
//                                }
//                            }
//
//                            for (Modifier modifier : vitalResult.getModifiers()) {
//                                if (modifier == null || "NormalizedObsId".equalsIgnoreCase(modifier.getModifierType())) {
//                                    continue;
//                                }
//
//                                VitalModifier vModifier = new VitalModifier();
//                                vModifier.setType(modifier.getModifierType());
//                                vModifier.setValue(modifier.getValue());
//                                vModifier.setUnits(modifier.getUnits());
//                                bean.getVitalModifiers().add(vModifier);
//                            }
//
//                            bean.setComments(bhieVitals.getComments());
//                            bean.setResultComments(vitalResult.getComment());
//                            bean.setVitalsIEN(bhieVitals.getVitalsId());
//
//                            addBHIESiteDataToVistaBean(bhieVitals.getCustodianId(),
//                                    bhieVitals.getCustodianName(), bhieVitals.getFacility(), vitalResult.getUniqueId(), bean);
//
//                            vistaBeans.add(bean);
//                        }
//
//                    }
//                }
//            }
//        }
//
//        //include site status report
//        String siteStatusReport = generateBhieStatusReport(vitalsResultCollection.getStatusList(),
//                vitalsResultCollection.getErrorList());
//
//        vistaBeans.addAll(Arrays.asList(
//                (Vitals[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return vistaBeans.toArray(new Vitals[vistaBeans.size()]);
//    }

//    public gov.va.med.vds.webservice.Vitals[] mapVitalsFromCHCS(Object[] dodBean) {
//        int size = dodBean.length;
//        int cnt = 0;
//        gov.va.med.vds.webservice.Vitals[] temp = new gov.va.med.vds.webservice.Vitals[size];
//        gov.va.med.vds.webservice.Vitals[] vistaBean = null;
//        gov.va.med.vds.webservice.Vitals bean = null;
//
//        for (int i = 0; i < dodBean.length; i++) {
//            if (dodBean[i] != null) {
//                bean = new gov.va.med.vds.webservice.Vitals();
////            bean.setDateTimeTaken(Utils.getXMLCal(dodBean[i].getDateTimeTaken()));
////            bean.setRate(dodBean[i].getRate());
////            bean.setVitalsIEN(dodBean[i].getVitalsIEN());
////            bean.setVitalType(dodBean[i].getVitalType());
//                bean.setDateTimeTaken(Utils.getXMLCal(((org.hui.chcs.Vitals) dodBean[i]).getDateTimeTaken()));
//                bean.setRate(((org.hui.chcs.Vitals) dodBean[i]).getRate());
//                bean.setVitalsIEN(((org.hui.chcs.Vitals) dodBean[i]).getVitalsIEN());
//                bean.setVitalType(((org.hui.chcs.Vitals) dodBean[i]).getVitalType());
//                addCHCSSiteDataToVistaBean(bean);
//                temp[cnt] = bean;
//                cnt++;
//            }
//        }
//
//        vistaBean = new gov.va.med.vds.webservice.Vitals[cnt];
//        for (int i = 0; i < cnt; i++) {
//            vistaBean[i] = temp[i];
//        }
//
//        return vistaBean;
//
//    }

//    private Vitals mapBloodPressureSystolicDiastolic(Map<String, VitalResult> bpResults) {
//        VitalResult systolicResult = bpResults.get("systolic");
//        VitalResult diastolicResult = bpResults.get("diastolic");
//        Vitals rtc;
//
//        if (systolicResult == null || diastolicResult == null) {
//            rtc = null;
//        } else if (!systolicResult.getTestUnitsCode().equals(diastolicResult.getTestUnitsCode())) {
//            rtc = null;
//        } else {
//            Vitals bean = new Vitals();
//            bean.setDateTimeTaken(bhieToXMLCal(systolicResult.getStartTime()));
//            bean.setRate(systolicResult.getTestValue() + "/" + diastolicResult.getTestValue());
//            bean.setVitalType(VISTA_LABEL_BLOOD_PRESSURE);
//            bean.setUnits(systolicResult.getTestUnits());
//            bean.setUnitsCode(mapUnitsCodeToUnitsString(systolicResult.getTestUnitsCode()));
//
//            rtc = bean;
//        }
//        return rtc;
//    }

    private Vitals mapBloodPressureSystolicDiastolicV4(Map<String, VitalsResultV4> bpResults) {
        Vitals bean = new Vitals();

        VitalsResultV4 systolicResult = bpResults.get("systolic");
        VitalsResultV4 diastolicResult = bpResults.get("diastolic");

        if (systolicResult == null || diastolicResult == null) {
            bean = null;
        } else {
            if (systolicResult.getTest() != null && systolicResult.getTest().getUnits() != null
                    && diastolicResult.getTest() != null && diastolicResult.getTest().getUnits() != null) {

                String systolicUnitsCode = "";
                String diastolicUnitsCode = "";
                if (getPrimarySpdcV4(systolicResult.getTest().getUnits()) != null) {
                    systolicUnitsCode = getPrimarySpdcV4(systolicResult.getTest().getUnits()).getCode();
                }
                if (getPrimarySpdcV4(diastolicResult.getTest().getUnits()) != null) {
                    diastolicUnitsCode = getPrimarySpdcV4(diastolicResult.getTest().getUnits()).getCode();
                }

                if (systolicUnitsCode.equals(diastolicUnitsCode)) {
                    if (getPrimarySpdcV4(diastolicResult.getTest().getUnits()) != null) {
                        bean.setUnits(getPrimarySpdcV4(diastolicResult.getTest().getUnits()).getDisplay());
                    }
                    if (getPrimarySpdcV4(systolicResult.getTest().getUnits()) != null) {
                        bean.setUnitsCode(getPrimarySpdcV4(systolicResult.getTest().getUnits()).getCode());
                    }
                } else {
                    bean = null;
                }
            }
            if (bean != null) {
                if (systolicResult.getTest() != null && systolicResult.getTest().getValue() != null
                        && diastolicResult.getTest() != null && diastolicResult.getTest().getValue() != null) {
                    SpdcBeanV4 systolicValue = getPrimarySpdcV4(systolicResult.getTest().getValue());
                    SpdcBeanV4 diastolicValue = getPrimarySpdcV4(diastolicResult.getTest().getValue());

                    if (systolicValue != null && diastolicValue != null) {
                        bean.setRate(systolicValue.getDisplay() + "/" + diastolicValue.getDisplay());
                    }
                }
                bean.setDateTimeTaken(bhieToXMLCalV4(systolicResult.getStartTime()));
                bean.setVitalType(VISTA_LABEL_BLOOD_PRESSURE);
            }
        }

        return bean;
    }

    private Vitals mapBloodPressureSystolicV4(Map<String, VitalsResultV4> bpResults) {
        Vitals bean = new Vitals();

        VitalsResultV4 systolicResult = bpResults.get("systolic");

        if (systolicResult == null) {
            bean = null;
        } else {
            if (systolicResult.getTest() != null && systolicResult.getTest().getUnits() != null) {
                if (getPrimarySpdcV4(systolicResult.getTest().getUnits()) != null) {
                    bean.setUnitsCode(getPrimarySpdcV4(systolicResult.getTest().getUnits()).getCode());
                    bean.setUnits(getPrimarySpdcV4(systolicResult.getTest().getUnits()).getDisplay());
                }

            }
            if (bean != null) {
                if (systolicResult.getTest() != null && systolicResult.getTest().getValue() != null) {
                    SpdcBeanV4 systolicValue = getPrimarySpdcV4(systolicResult.getTest().getValue());

                    if (systolicValue != null) {
                        bean.setRate(systolicValue.getDisplay() + "/");
                    }
                }
                bean.setDateTimeTaken(bhieToXMLCalV4(systolicResult.getStartTime()));
                bean.setVitalType(VISTA_LABEL_BLOOD_PRESSURE);
            }
        }

        return bean;
    }

    private String mapUnitsCodeToUnitsString(String unitsCode) {
        String rtc;
        if (unitsCode == null) {
            rtc = null;
        } else {
            if (unitsCode.equals(Integer.toString(SNOMED_PERCENT))) {
                rtc = "%";
            } else if (unitsCode.equals(Integer.toString(SNOMED_MMHG))) {
                rtc = "mmHg";
            } else if (unitsCode.equals(Integer.toString(SNOMED_PAIN_SCALE))) {
                //return no units for pain scale
                rtc = null;
            } else if (unitsCode.equals(Integer.toString(SNOMED_PER_MIN))) {
                rtc = "/min";
            } else if (unitsCode.equals(Integer.toString(SNOMED_CELSIUS))) {
                rtc = "C";
            } else if (unitsCode.equals(Integer.toString(SNOMED_FAHRENHEIT))) {
                rtc = "F";
            } else if (unitsCode.equals(Integer.toString(SNOMED_GRAM))) {
                rtc = "gm";
            } else if (unitsCode.equals(Integer.toString(SNOMED_INCHES))) {
                rtc = "in";
            } else if (unitsCode.equals(Integer.toString(SNOMED_CENTIMETERS))) {
                rtc = "cm";
            } else if (unitsCode.equals(Integer.toString(SNOMED_OUNCES))) {
                rtc = "oz";
            } else if (unitsCode.equals(Integer.toString(SNOMED_POUNDS))) {
                rtc = "lb";
            } else if (unitsCode.equals(Integer.toString(SNOMED_KILOGRAMS))) {
                rtc = "Kg";
            } else {
                rtc = null;
            }
        }
        return rtc;
    }

    public Vitals[] mapVitalsFromBHIEV4(Object resultCollection) {
        V4ResultCollection vitalsResultCollection = (V4ResultCollection) resultCollection;
        List<Vitals> vistaBeans = new ArrayList<Vitals>();

        if (vitalsResultCollection.getDataRecordList() != null && vitalsResultCollection.getDataRecordList().getVitals() != null) {

            for (VitalV4 bhieVital : vitalsResultCollection.getDataRecordList().getVitals()) {
                if (bhieVital == null) {
                    continue;
                }

                List<VitalsResultV4> vitalResults = bhieVital.getResults();
                if (vitalResults != null) {
                    Map<String, VitalsResultV4> bpResults = new LinkedHashMap<String, VitalsResultV4>();
                    List<Vitals> systolicBeans = new ArrayList<Vitals>();

                    for (int i = 0; i < vitalResults.size(); i++) {
                        Vitals bean = new Vitals();

                        VitalsResultV4 vitalResult = vitalResults.get(i);

                        boolean isOnlySystolic = false;

                        if (vitalResult.getTest() != null && vitalResult.getTest().getType() != null) {
                            SpdcBeanV4 testType = getPrimarySpdcV4(vitalResult.getTest().getType());
                            if (testType != null) {
                                bean.setVitalType(testType.getDisplay());
                            }
                        }
                        if (vitalResult.getTest() != null && vitalResult.getTest().getValue() != null) {
                            SpdcBeanV4 testValue = getPrimarySpdcV4(vitalResult.getTest().getValue());
                            if (testValue != null) {
                                bean.setRate(testValue.getDisplay());
                            }
                        }
                        if (vitalResult.getTest() != null && vitalResult.getTest().getUnits() != null) {
                            SpdcBeanV4 testUnits = getPrimarySpdcV4(vitalResult.getTest().getUnits());
                            if (testUnits != null) {
                                bean.setUnits(testUnits.getDisplay());
                                bean.setUnitsCode(testUnits.getCode());
                            }
                        }

                        List<SpdcBeanV4> types = vitalResult.getTest().getType();
                        for (SpdcBeanV4 type : types) {
                            int vitalSnomed = -1;
                            try {
                                vitalSnomed = Integer.parseInt(type.getCode());
                            } catch (NumberFormatException e) {
                                LOGGER.debug("Cannot parser Vitals snomed code");
                            }

                            //attempt to map vital by snomed code
                            switch (vitalSnomed) {
                                case SNOMED_DIASTOLIC_BP:
                                    bean = null;

                                    bpResults.put("diastolic", vitalResult);

                                    if (bpResults.containsKey("systolic")) {
                                        bean = mapBloodPressureSystolicDiastolicV4(bpResults);
                                        systolicBeans.clear();
                                        isOnlySystolic = false;
                                    }

                                    break;
                                case SNOMED_SYSTOLIC_BP:
                                    bean = null;
                                    bpResults.put("systolic", vitalResult);

                                    if (bpResults.containsKey("diastolic")) {
                                        bean = mapBloodPressureSystolicDiastolicV4(bpResults);
                                        systolicBeans.clear();
                                        isOnlySystolic = false;
                                    } else {
                                        isOnlySystolic = true;
                                        bean = mapBloodPressureSystolicV4(bpResults);
                                        //systolicBeans.add(bean);
                                    }

                                    break;
                                case SNOMED_WEIGHT:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_WEIGHT);
                                    }
                                    break;
                                case SNOMED_HEIGHT:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_HEIGHT);
                                    }
                                    break;
                                case SNOMED_RESPITORY_RATE:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_RESPIRATION);
                                    }
                                    break;
                                case SNOMED_TEMPERATURE:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_TEMPERATURE);
                                    }
                                    break;
                                case SNOMED_HEART_RATE:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_PULSE);
                                    }
                                    break;
                                case SNOMED_PAIN_SEVERITY:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_PAIN);
                                    }
                                    break;
                                case SNOMED_02_SATURATION:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_PULSE_OXIMETRY);
                                    }
                                    break;
                                case SNOMED_CG:
                                    if (bean != null) {
                                        bean.setVitalType(VISTA_LABEL_CIRCUMFERENCE_GIRTH);
                                    }
                                    break;
                                default:
                                    LOGGER.debug("Could not map vital type by snomed.");
                                    break;
                            }
                        }

                        if (bean != null) {
                            if (VISTA_LABEL_BLOOD_PRESSURE.equals(bean.getVitalType())) {
                                //assign blood pressure LOINC to combined systolic / diastolic vital
                                Code code = new Code();
                                code.setSystem("2.16.840.1.113883.6.1");
                                code.setCode("55284-4");
                                code.setDisplay("Blood Pressure");
                                bean.getCodes().add(code);
                            } else {
                                if (vitalResult.getTest() != null && vitalResult.getTest().getType() != null
                                        && getPrimarySpdcV4(vitalResult.getTest().getType()) != null
                                        && getPrimarySpdcV4(vitalResult.getTest().getType()).getCode() != null) {
                                    for (SpdcBeanV4 type : vitalResult.getTest().getType()) {
                                        if (type == null) {
                                            continue;
                                        }
                                        if (type.getSystem() != null && type.getCode() != null) {
                                            Code code = new Code();
                                            code.setSystem(type.getSystem());
                                            code.setPrimary(String.valueOf(type.isPrimary()));
                                            code.setCode(type.getCode());
                                            code.setDisplay(type.getDisplay());
                                            bean.getCodes().add(code);
                                        }
                                    }
                                }
                            }

                            bean.setDateTimeTaken(bhieToXMLCalV4(vitalResult.getPerformedDate()));
                            bean.setNarrativeId(bhieVital.getNarrativeId());
                            bean.setComments(bhieVital.getComment());
                            bean.setResultComments(vitalResult.getComment());

                            List<VitalModifier> modifierList = new ArrayList<VitalModifier>();
                            for (VitalsResultModifierV4 vitalModifier : vitalResult.getModifiers()) {
                                VitalModifier vitalMod = new VitalModifier();
                                if (vitalModifier.getType() != null && !vitalModifier.getType().isEmpty()
                                        && getPrimarySpdcV4(vitalModifier.getType()) != null) {
                                    vitalMod.setType(getPrimarySpdcV4(vitalModifier.getType()).getDisplay());
                                }
                                if (vitalModifier.getUnits() != null && !vitalModifier.getUnits().isEmpty()
                                        && getPrimarySpdcV4(vitalModifier.getUnits()) != null) {
                                    vitalMod.setUnits(getPrimarySpdcV4(vitalModifier.getUnits()).getDisplay());
                                }
                                if (vitalModifier.getValue() != null && !vitalModifier.getValue().isEmpty()
                                        && getPrimarySpdcV4(vitalModifier.getValue()) != null) {
                                    vitalMod.setValue(getPrimarySpdcV4(vitalModifier.getValue()).getDisplay());
                                }
                                modifierList.add(vitalMod);
                            }
                            bean.getVitalModifiers().addAll(modifierList);

                            String repositoryId = "";
                            String repositoryName = "";
                            String recordId = "";
                            String facilityName = "";

                            if (bhieVital.getRepository() != null && bhieVital.getRepository().getIds() != null && !bhieVital.getRepository().getIds().isEmpty()) {
//                                repositoryId = bhieVital.getRepository().getIds().get(0).getId();
                                if (!StringUtils.isEmpty(bhieVital.getRepository().getIds().get(0).getId())) {
                                    repositoryId = bhieVital.getRepository().getIds().get(0).getId();
                                } else {
                                    repositoryId = bhieVital.getRepository().getIds().get(0).getAssigningAuthority();
                                }
                                repositoryName = bhieVital.getRepository().getName();
                            }
                            if (bhieVital.getFacility() != null) {
                                facilityName = bhieVital.getFacility().getName();
                            }
                            if (bhieVital.getRecordId() != null) {
                                recordId = bhieVital.getRecordId().getId();
                                bean.setVitalsIEN(bhieVital.getRecordId().getId());
                            }

                            if (vitalsResultCollection.isVler()) {
                                addBHIESiteDataToVistaBean(repositoryId,
                                        "VLER", facilityName, recordId, bean);
                            } else {
                                addBHIESiteDataToVistaBean(repositoryId,
                                        repositoryName, facilityName, recordId, bean);
                            }

                            //do not add bean if bloodPressure
                            if (!VISTA_LABEL_BLOOD_PRESSURE.equals(bean.getVitalType())) {
                                vistaBeans.add(bean);
                            } else if (VISTA_LABEL_BLOOD_PRESSURE.equals(bean.getVitalType()) && !isOnlySystolic) {
                                vistaBeans.add(bean);
                            } else if (isOnlySystolic) {
                                //store systolic bean temporarily. we'll add it at the end of this loop
                                // if no diastolic detected
                                systolicBeans.add(bean);
                            }
                        }

                        //end of loops, add systolic if it exists
                        if (i == (vitalResults.size() - 1)) {
                            vistaBeans.addAll(systolicBeans);
                        }
                    }
                }
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(vitalsResultCollection.getStatusListv4(),
                vitalsResultCollection.getErrorListv4(), vitalsResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (Vitals[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Vitals[vistaBeans.size()]);
    }

    public Vitals[] mapDASVLERVitals(Object resultCollection) throws JMeadowsException {
        DasResultCollection vlerResultCollection = (DasResultCollection) resultCollection;
        List<Vitals> vistaBeans = new ArrayList<Vitals>();

        try {

            if (vlerResultCollection.getVitals() != null) {
                vistaBeans = vlerResultCollection.getVitals();

                for (Vitals vistaBean : vistaBeans) {
                    if (StringUtils.isNotEmpty(vistaBean.getSite().getSiteCode())) {
                        String vlerSiteName = getVLERSiteName(vistaBean.getSite().getSiteCode());
                        if (StringUtils.isNotEmpty(vlerSiteName)) {
                            vistaBean.getSite().setName(vlerSiteName);
                        }
                    }
                }
            }

            String siteStatusReport = generateBhieStatusReportV4(mapDASStatusList(vlerResultCollection.getStatusList()),
                    vlerResultCollection.getErrorList(), true);

            vistaBeans.addAll(Arrays.asList(
                    (Vitals[]) generateConnectionUnavailableBean(
                            BHIE_STATUS_REPORT_LOCATION,
                            siteStatusReport)
            ));

        } catch (Exception ex) {

            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }

        return vistaBeans.toArray(new Vitals[vistaBeans.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        gov.va.med.vds.webservice.Vitals vital = new gov.va.med.vds.webservice.Vitals();

        vital.setVitalType(errorMsg);
        vital.setDateTimeTaken(Utils.backDate());
        vital.setSite(getConnectionUnavailableSite(location));
        return new gov.va.med.vds.webservice.Vitals[]{vital};
    }
}
