/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.provider;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.smartbeans.patient.SmartBean;
import gov.va.med.vds.webservice.ProgressNote;
import gov.va.med.vhahon.common.Utils;
//import org.hui.chcs.NoteBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;
import web.webservice.jvi.citpo.ha.osd.mil.Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean;

import java.util.Arrays;

public class SmartProgressNote extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartProgressNote.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;
    public static final int HCP = 4;
    public static final int LOC = 5;

    public ProgressNote m_note;

    public SmartProgressNote() {
        super();
        m_note = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartProgressNote(ProgressNote note, String sort) {
        super();
        m_note = note;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartProgressNote) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_note);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartProgressNote[] temp = new SmartProgressNote[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProgressNote newAll = new SmartProgressNote((ProgressNote) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartProgressNote newAll = new SmartProgressNote((ProgressNote) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            ProgressNote[] ret = new ProgressNote[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_note;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartProgressNote[] temp = new SmartProgressNote[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProgressNote newAll = new SmartProgressNote((ProgressNote) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            ProgressNote[] ret = new ProgressNote[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_note;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartProgressNote tmp = (SmartProgressNote) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_note.getNoteDate(), tmp.m_note.getNoteDate());
        }
        if (this.m_sortBy == SITE && this.m_note.getSite() != null && tmp.m_note.getSite() != null) {
            retVal = compareString(this.m_note.getSite().getSiteCode(), tmp.m_note.getSite().getSiteCode());
        }
        if (this.m_sortBy == TYPE) {
            retVal = compareString(this.m_note.getNoteType(), tmp.m_note.getNoteType());
        }
//        if (this.m_sortBy == HCP)
//            retVal = compareString(this.m_note.getProvider(), tmp.m_note.getProvider());
        if (this.m_sortBy == LOC) {
            retVal = compareString(this.m_note.getLocation(), tmp.m_note.getLocation());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("notedate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("notetype".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
            if ("user".equalsIgnoreCase(sort)) {
                m_sortBy = HCP;
            }
            if ("location".equalsIgnoreCase(sort)) {
                m_sortBy = LOC;
            }
        }

    }

    public static ProgressNote[] getDischargeNotesFromSHARE(Object[] dodBean) {
        Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[] shareBean =
                (Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[]) dodBean;
        int size = shareBean.length;
        int cnt = 0;
        ProgressNote[] temp = new ProgressNote[size];
        ProgressNote[] retBean = null;
        ProgressNote bean = null;
        for (int i = 0; i < size; i++) {
            bean = new ProgressNote();
            bean.setNoteDate(Utils.getXMLCal(shareBean[i].getVerifiedDate()));
            bean.setNoteId(shareBean[i].getItemId());
            bean.setLocation(shareBean[i].getNoteTitle());
            bean.setNoteType(shareBean[i].getNoteTypeName());
//            bean.setProvider(shareBean[i].getVerifiedBy());
            bean.setImages(shareBean[i].getCompleteNote());
            bean.setStatus(shareBean[i].getNoteFormat());
            bean.setVisitDate("");
            addSHARESiteDataToVistaBean(bean);
            temp[cnt] = bean;
            cnt = cnt + 1;
        }

        if (cnt > 0) {
            retBean = new ProgressNote[cnt];
            for (int i = 0; i < cnt; i++) {
                retBean[i] = temp[i];
            }
        }


        return retBean;

    }

    public static ProgressNote[] getProgressNotesFromSHARE(Object[] dodBean) {
        Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[] shareBean =
                (Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean[]) dodBean;
        int size = shareBean.length;
        int cnt = 0;
        ProgressNote[] temp = new ProgressNote[size];
        ProgressNote[] retBean = null;
        ProgressNote bean = null;
        String itemid = null;

        for (int i = 0; i < size; i++) {
            itemid = shareBean[i].getItemId();
            if (itemid == null) {
                itemid = "";
            }


            if (!shareBean[i].getNoteTitle().contains("CONSULT")
                    && !shareBean[i].getNoteTitle().contains("Consult")
                    && !shareBean[i].getNoteTitle().contains("consult")
                    && !shareBean[i].getNoteTitle().contains("DISCHARGE")
                    && !shareBean[i].getNoteTitle().contains("Discharge")
                    && !shareBean[i].getNoteTitle().contains("discharge")
                    && !shareBean[i].getNoteTypeName().contains("CONSULT")
                    && !itemid.contains("CIS")) {
                bean = new ProgressNote();
                bean.setNoteDate(Utils.getXMLCal(shareBean[i].getVerifiedDate()));
                bean.setNoteId(shareBean[i].getItemId());
                bean.setLocation(shareBean[i].getNoteTitle());
                bean.setNoteType(shareBean[i].getNoteTypeName());
//                bean.setProvider(shareBean[i].getVerifiedBy());
                bean.setImages(shareBean[i].getCompleteNote());
                bean.setStatus(shareBean[i].getNoteFormat());
                bean.setVisitDate("");
                addSHARESiteDataToVistaBean(bean);
                temp[cnt] = bean;
                cnt = cnt + 1;
            }

        }

        if (cnt > 0) {
            retBean = new ProgressNote[cnt];
            for (int i = 0; i < cnt; i++) {
                retBean[i] = temp[i];
            }
        }


        return retBean;

    }

//    public static ProgressNote[] getProgressNotesFromCHCS(Object[] dodBean) {
//        NoteBean[] shareBean = (NoteBean[]) dodBean;
//        int size = shareBean.length;
//        int cnt = 0;
//        ProgressNote[] temp = new ProgressNote[size];
//        ProgressNote[] retBean = null;
//        ProgressNote bean = null;
//        String itemid = null;
//        String noteTitle = "";
//        String noteTypeName = "";
//        for (int i = 0; i < size; i++) {
//            itemid = shareBean[i].getItemId();
//            if (itemid == null) {
//                itemid = "";
//            }
//            noteTitle = "";
//            if (shareBean[i].getNoteTitle() != null) {
//                noteTitle = shareBean[i].getNoteTitle();
//            }
//            noteTypeName = "";
//            if (shareBean[i].getNoteTypeName() != null) {
//                noteTypeName = shareBean[i].getNoteTypeName();
//            }
//
//            if (!noteTitle.contains("CONSULT")
//                    && !noteTitle.contains("Consult")
//                    && !noteTitle.contains("consult")
//                    && !noteTitle.contains("DISCHARGE")
//                    && !noteTitle.contains("Discharge")
//                    && !noteTitle.contains("discharge")
//                    && !noteTypeName.contains("CONSULT")
//                    && !noteTypeName.contains("ENCOUNTER")
//                    && !itemid.contains("CIS")) {
//                bean = new ProgressNote();
//                bean.setNoteDate(Utils.getXMLCal(shareBean[i].getVerifiedDate()));
//                bean.setNoteId(shareBean[i].getItemId());
//                bean.setLocation(shareBean[i].getNoteTitle());
//                bean.setNoteType(shareBean[i].getNoteTypeName());
////                bean.setProvider(shareBean[i].getVerifiedBy());
//                bean.setImages(shareBean[i].getCompleteNote());
//                bean.setStatus(shareBean[i].getNoteFormat());
//                addCHCSSiteDataToVistaBean(bean);
//                if (bean.getStatus() == null) {
//                    bean.setStatus("Text");
//                }
//                bean.setVisitDate("");
//                temp[cnt] = bean;
//                cnt = cnt + 1;
//            }
//
//        }
//
//        if (cnt > 0) {
//            retBean = new ProgressNote[cnt];
//            for (int i = 0; i < cnt; i++) {
//                retBean[i] = temp[i];
//            }
//        }
//
//
//        return retBean;
//
//    }


    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        ProgressNote note = new ProgressNote();
        note.setNoteId("");
        note.setNoteType(errorMsg);
        note.setNoteDate(Utils.backDate());
        note.setSite(getConnectionUnavailableSite(location));

        return new ProgressNote[]{note};
    }
}