/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.dao.TestConfig;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.*;
import gov.va.med.vds.webservice.DataBean;
import gov.va.med.vds.webservice.Patient;
import gov.va.med.vds.webservice.User;
import gov.va.med.vhahon.common.Utils;
import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author vhahonbrunos
 */
public class DaoBaseTest
{
    protected static final Logger logger = LoggerFactory.getLogger(DaoBaseTest.class);

    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();

    protected User user;
    protected Patient patient;
    protected Calendar startDate;
    protected Calendar endDate;

    protected String ERROR_MSG_CONNECTION_UNAVAILABLE = "Connection Unavailable";

    @BeforeClass
    public static void setUpClass()
    {

    }

    @AfterClass
    public static void tearDownClass()
    {
    }

    @Before
    public void setUp()
    {
        LogManager.getRootLogger().setLevel(Level.DEBUG);

        try
        {

            UserDao userDao = new UserDao();
            List<AuditEntry>  loginInfo = userDao.getLoginInfo("999999", "");

            //select test patient
            PatientDao patientService = new PatientDao();
            JMeadowsQuery queryBean = new JMeadowsQuery();
            queryBean.setRequestingApp("jmeadows_unittest");

            startDate = Calendar.getInstance();
            endDate = Calendar.getInstance();
            startDate.set(1990, 0, 1);

            user = loginTestProvider();
            user.setUserId("123");

            PDWSQueryBean pdwsQueryBean = new PDWSQueryBean();
            pdwsQueryBean.setUser( user );
            pdwsQueryBean.setPatientId( TestConfig.get(TestConfig.PATIENT_SSN) );
            pdwsQueryBean.setPatientIdType( "SSN");
            pdwsQueryBean.setFirstName( TestConfig.get(TestConfig.PATIENT_NAME_FIRST) );
            pdwsQueryBean.setLastName( TestConfig.get(TestConfig.PATIENT_NAME_LAST));
            pdwsQueryBean.setRequestingApp("jMeaodws_unit_test" );
            pdwsQueryBean.setInquiryType("PS");
            ResponsePatientQuery responsePatientQuery = patientService.lookupPatientPDWS(pdwsQueryBean);

            for(Patient p : responsePatientQuery.getPatients())
            {
//                String nm =  TestConfig.get(TestConfig.PATIENT_NAME_LAST) + "," + TestConfig.get(TestConfig.PATIENT_NAME_FIRST);
//                if (p.getName().equalsIgnoreCase(nm))
//                {
                    this.patient = p;
                    break;
//                }
            }

            JMeadowsQuery qb = new JMeadowsQuery();

            qb.setUser(user);
            qb.setItemId(this.patient.getEDIPI());
            qb.setStatus("EDIPI");
            ResponsePatientSelect responsePatientSelect = patientService.selectPatientMVI(qb);
            this.patient = responsePatientSelect.getPatient();

            //setting AHLTA UnitNumber to blank, because our mock service doesn't support it
            //this.patient.setAhltaUnitNumber("");
        } 
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }

    protected User loginTestProvider() throws JMeadowsException {
        try
        {
            String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
            String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);
            String siteCode = TestConfig.get(TestConfig.PROVIDER_SITE_CODE);
            UserDao userDao = new UserDao();

            User user;

            user = userDao.login(siteCode, VistaKernelHash.encrypt(accessCode, false), VistaKernelHash.encrypt(verifyCode, false), "jmeadows_unittest");

            if (user == null || user.getUserIen().contains("^"))
                throw new RuntimeException("Invalid User");

            return user;
        }
        catch (Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    @After
    public void tearDown()
    {

    }

    protected JMeadowsQuery setupQuery(User user, Patient patient, Calendar startDate, Calendar endDate)
    {
        JMeadowsQuery qb = new JMeadowsQuery();
        qb.setUser(user);
        qb.setPatient(patient);
        qb.setStartDate(Utils.getCalendar(startDate.getTime()));
        qb.setEndDate(Utils.getCalendar(endDate.getTime()));
        qb.setMax(999);
        qb.setActive(0);
        return qb;
    }

    protected void testForBeanMeta(List beans)
    {
        testForBeanMeta(beans, true);
    }

    protected void testForBeanMeta(List beans, boolean patientBasedQuery)
    {
        for(Object bean : beans)
        {
            DataBean dataBean = (DataBean) bean;

            assertTrue(dataBean.getSite() != null);
            /*if(patientBasedQuery)
            {
                assertTrue(dataBean.getPatientId() != null && dataBean.getPatientId().length() > 0);
            }*/
        }
    }

    protected void testForProviderBeanMeta(List beans)
    {
        testForBeanMeta(beans, false);
    }

    /**
     * Helper function which permits the testing of non matching providers.
     * @param p the user object which stores location id mappings
     * @param vistaIen the new ien of the vista user or null if none
     * @param chcsIen the ien of the chcs user or null if none
     */
    protected void mockProviderBasedQuery(User p, String vistaIen, String chcsIen)
    {
//        List<KeyValuePair> locationIdMappings = p.getLocationIdMappings();
//        locationIdMappings.clear();
//        if(vistaIen != null)
//        {
//            KeyValuePair kvp = new KeyValuePair();
//            kvp.setKey("459");
//            kvp.setValue(vistaIen);
//            locationIdMappings.add(kvp);
//        }
//        if(chcsIen != null)
//        {
//            KeyValuePair kvp = new KeyValuePair();
//            kvp.setKey("A0101");
//            kvp.setValue(chcsIen);
//            locationIdMappings.add(kvp);
//        }
    }

//    protected void mockPatientBasedQuery(Patient p, String newPatientIen)
//    {
//        if(p == null)
//        {
//            throw new IllegalArgumentException("Patient cannot be null");
//        }
//        if(newPatientIen == null)
//        {
//            throw new IllegalArgumentException("newPatientIen cannot be null");
//        }
//        p.setPatientId(newPatientIen);
//    }
}
