package gov.va.med.jmeadows.dao.patient;

import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.beans.PatientDocument;
import gov.va.med.vds.webservice.KeyValuePair;
import gov.va.med.vds.webservice.RadiologyReport;
import gov.va.med.vds.webservice.Site;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class VixDaoTest extends DaoBaseTest {
    private DocumentDao documentDao;
    private RadiologyDao radiologyDao;
    private VixDao vixDao;

    public VixDaoTest() throws JMeadowsException
    {
        documentDao = new DocumentDao();
        radiologyDao = new RadiologyDao();
        vixDao = new VixDao();
    }

    private JMeadowsQuery initQueryBean(boolean modifyUserAndPatient)
    {
        JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

        if (modifyUserAndPatient) {
            Site site = new Site();
            site.setName("");
            site.setSiteCode("");
            site.setAgency("");

            user.setName("");
            user.setBseToken("");
            user.setUserIen("");
            user.setEncryptedFederatedUid(EncryptUtil.encrypt(""));
            user.setHostSite(site);

            patient.setICN("");

            KeyValuePair ienSitePair = new KeyValuePair();
            ienSitePair.setKey("");
            ienSitePair.setValue("");
            patient.getPatientIens().clear();
            patient.getPatientIens().add(ienSitePair);
        }

        return qb;
    }

    @Test
    public void testDocumentStudyQuery()
    {
        try {
            JMeadowsQuery qb = initQueryBean(true);

            List<PatientDocument> notes = documentDao.getPatientDocuments(qb);

            assertTrue(notes.get(0).getVixStudy().getViewerUrl() != null);
        }
        catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testRadiologyStudyQuery()
    {
        boolean useGenericRad = true;

        try {
            JMeadowsQuery qb = initQueryBean(true);

            List<RadiologyReport> reports;

            if (useGenericRad) {
                Site site = new Site();
                site.setName("");
                site.setSiteCode("");
                site.setAgency("");

                RadiologyReport report = new RadiologyReport();
                report.setSite(site);
                report.setCaseNumber("");
                report.setExamId("");
                report.setFacilityName("");
                report.setHasImages(true);

                reports = new ArrayList<RadiologyReport>();
                reports.add(report);
            }
            else {
                reports = radiologyDao.getPatientRads(qb);
            }

            vixDao.mapDoDRadiologyReports(qb, reports);

            assertTrue(reports.get(0).getVixStudy().getViewerUrl() != null);
        }
        catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }
}
