/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBaseTest;
import gov.va.med.jmeadows.dao.provider.ProviderOrdersDao;
import gov.va.med.vds.webservice.Order;
import gov.va.med.vds.webservice.User;
import org.junit.Test;

import java.text.SimpleDateFormat;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author vhahonbrunos
 */
public class ProviderOrdersDaoTest extends DaoBaseTest
{
    private ProviderOrdersDao orderDao;
    
    public ProviderOrdersDaoTest() throws JMeadowsException
    {
        super();
        orderDao = new ProviderOrdersDao();
    }

    @Test
    public void testGetProviderPendingOrders()
    {
        try 
        {
            startDate.set(2011, 3, 1);
            endDate.set(2011, 3, 10);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            
            mockProviderBasedQuery(qb.getUser(), null, "258");

            List<Order> orders = orderDao.getProviderOrdersPending(qb);
            assertTrue(orders.size() > 0);
            testForProviderBeanMeta(orders);
            logger.debug("--- PROVIDER PENDING ORDERS ---");
            for(Order o : orders)
            {
                logger.debug("OrderId: " + o.getOrderid());
                logger.debug("PatientId: " + o.getPatientId());
                logger.debug("PatientName: " + o.getPatientName());
                SimpleDateFormat sdf = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
                if(o.getOrderDate() != null)
                {
                    logger.debug("StartDate: " + sdf.format(o.getOrderDate().toGregorianCalendar().getTime()));
                }
                if(o.getEnteredDate() != null)
                {
                    logger.debug("OrderDate: " + sdf.format(o.getEnteredDate().toGregorianCalendar().getTime()));
                }
                logger.debug("OrderDescription: " + o.getDescription());
                logger.debug("OrderType: " + o.getType());
                logger.debug("OrderStatus: " + o.getStatus());
                logger.debug("OrderResult: " + o.getOrderResult());
                if(ERROR_MSG_CONNECTION_UNAVAILABLE.equalsIgnoreCase(o.getDescription()))
                {
                    fail("Data fetch failed...");
                }
                logger.debug("---");
            }
        } 
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetProviderResultedOrders()
    {
        try
        {
            startDate.set(2011, 2, 1);
            endDate.set(2011, 2, 20);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            User p = qb.getUser();

            mockProviderBasedQuery(qb.getUser(), null, "258");

            List<Order> orders = orderDao.getProviderOrdersResulted(qb);
            assertTrue(orders.size() > 0);
            logger.debug("# of Resulted Orders: " + orders.size());
            testForProviderBeanMeta(orders);
            for(Order o : orders)
            {
                logger.debug("OrderId: " + o.getOrderid());
                logger.debug("PatientId: " + o.getPatientId());
                logger.debug("PatientName: " + o.getPatientName());
                SimpleDateFormat sdf = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
                logger.debug("StartDate: " + sdf.format(o.getOrderDate().toGregorianCalendar().getTime()));
                logger.debug("OrderDate: " + sdf.format(o.getEnteredDate().toGregorianCalendar().getTime()));
                logger.debug("CompletedDate: " + sdf.format(o.getCompletedDate().toGregorianCalendar().getTime()));
                logger.debug("OrderDescription: " + o.getDescription());
                logger.debug("OrderType: " + o.getType());
                logger.debug("OrderStatus: " + o.getStatus());
                logger.debug("OrderResult: " + o.getOrderResult());
                
                if(ERROR_MSG_CONNECTION_UNAVAILABLE.equalsIgnoreCase(o.getDescription()))
                {
                    fail("Data fetch failed...");
                }
                logger.debug("---");
            }
        }
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }
}
